@@ -6,12 +6,11 @@
 #ifndef ABYSS_H
 #define ABYSS_H
 
-#include "externs.h"
-
 // When shifting areas in the abyss, shift the square containing player LOS
 // plus a little extra so that the player won't be disoriented by taking a
 // step backward after an abyss shift.
 const int ABYSS_AREA_SHIFT_RADIUS = LOS_RADIUS + 2;
+const int ABYSSAL_RUNE_MIN_LEVEL = 3;
 extern const coord_def ABYSS_CENTRE;
 
 struct abyss_state
@@ -27,6 +26,8 @@ struct abyss_state
 extern abyss_state abyssal_state;
 
 void abyss_morph();
+
+void banished(const string &who = "");
 void push_features_to_abyss();
 
 void generate_abyss();
