@@ -7,7 +7,7 @@
 #define ACQUIRE_H
 
 bool acquirement(object_class_type force_class, int agent,
-                 bool quiet = false, int *item_index = NULL,
+                 bool quiet = false, int *item_index = nullptr,
                  bool debug = false);
 
 int acquirement_create_item(object_class_type class_wanted,
