@@ -1,16 +1,14 @@
 #include "AppHdr.h"
 
 #include "actor.h"
+
 #include "coord.h"
 #include "losglobal.h"
-#include "player.h"
-#include "monster.h"
 #include "state.h"
-#include "viewgeom.h"
 
 bool actor::observable() const
 {
-    return crawl_state.game_is_arena() || is_player() || you.can_see(this);
+    return crawl_state.game_is_arena() || is_player() || you.can_see(*this);
 }
 
 bool actor::see_cell(const coord_def &p) const
@@ -30,13 +28,13 @@ bool player::see_cell(const coord_def &p) const
     if (!in_bounds(pos()))
         return false; // A non-arena player at (0,0) can't see anything.
     if (xray_vision)
-        return (pos() - p).abs() <= dist_range(current_vision);
+        return (pos() - p).rdist() <= current_vision;
     return actor::see_cell(p);
 }
 
-bool actor::can_see(const actor *target) const
+bool actor::can_see(const actor &target) const
 {
-    return target->visible_to(this) && see_cell(target->pos());
+    return target.visible_to(this) && see_cell(target.pos());
 }
 
 bool actor::see_cell_no_trans(const coord_def &p) const
@@ -49,12 +47,12 @@ bool player::trans_wall_blocking(const coord_def &p) const
     return see_cell(p) && !see_cell_no_trans(p);
 }
 
-bool player::can_see(const actor* a) const
+bool player::can_see(const actor& a) const
 {
     if (crawl_state.game_is_arena() || crawl_state.arena_suspended)
-        return see_cell(a->pos());
+        return see_cell(a.pos());
     else if (xray_vision)
-        return see_cell(a->pos());
+        return see_cell(a.pos());
     else
         return actor::can_see(a);
 }
