@@ -1,29 +1,26 @@
 #include "AppHdr.h"
+
+#include "actor.h"
+
 #include <sstream>
 
 #include "act-iter.h"
-#include "actor.h"
 #include "areas.h"
-#include "artefact.h"
 #include "art-enum.h"
 #include "attack.h"
-#include "coord.h"
+#include "directn.h"
 #include "env.h"
 #include "fprop.h"
 #include "itemprop.h"
-#include "libutil.h"
 #include "los.h"
 #include "misc.h"
-#include "mon-abil.h"
+#include "mon-behv.h"
 #include "mon-death.h"
-#include "ouch.h"
-#include "player.h"
 #include "religion.h"
-#include "random.h"
-#include "state.h"
 #include "stepdown.h"
 #include "stringutil.h"
 #include "terrain.h"
+#include "transform.h"
 #include "traps.h"
 
 actor::~actor()
@@ -34,7 +31,7 @@ actor::~actor()
 
 bool actor::will_trigger_shaft() const
 {
-    return ground_level() && body_weight() > 0 && is_valid_shaft_level()
+    return is_valid_shaft_level()
            // let's pretend that they always make their saving roll
            && !(is_monster()
                 && mons_is_elven_twin(static_cast<const monster* >(this)));
@@ -45,12 +42,6 @@ level_id actor::shaft_dest(bool known = false) const
     return generic_shaft_dest(pos(), known);
 }
 
-bool actor::airborne() const
-{
-    flight_type fly = flight_mode();
-    return fly == FL_LEVITATE || fly == FL_WINGED && !(cannot_move() || caught());
-}
-
 /**
  * Check if the actor is on the ground (or in water).
  */
@@ -77,13 +68,13 @@ hands_reqd_type actor::hands_reqd(const item_def &item) const
 
 /**
  * Wrapper around the virtual actor::can_wield(const item_def&,bool,bool,bool,bool) const overload.
- * @param item May be NULL, in which case a dummy item will be passed in.
+ * @param item May be nullptr, in which case a dummy item will be passed in.
  */
 bool actor::can_wield(const item_def* item, bool ignore_curse,
                       bool ignore_brand, bool ignore_shield,
                       bool ignore_transform) const
 {
-    if (item == NULL)
+    if (item == nullptr)
     {
         // Unarmed combat.
         item_def fake;
@@ -117,7 +108,7 @@ bool actor::handle_trap()
     trap_def* trap = find_trap(pos());
     if (trap)
         trap->trigger(*this);
-    return trap != NULL;
+    return trap != nullptr;
 }
 
 int actor::skill_rdiv(skill_type sk, int mult, int div) const
@@ -137,15 +128,16 @@ int actor::check_res_magic(int power)
     // value because mrs = hd * 2 * 3 for most monsters, and the weak, low
     // level monsters have been adjusted so that the "3" is typically a 1.
     // There are some notable one hd monsters that shouldn't fall under this,
-    // so we do < 6, instead of <= 6...  or checking mons->hit_dice.  The
+    // so we do < 6, instead of <= 6...  or checking mons->hit_dice. The
     // goal here is to make the first level easier for these classes and give
     // them a better shot at getting to level two or three and spells that can
     // help them out (or building a level or two of their base skill so they
     // aren't resisted as often). - bwr
+    // If you change this, also change desc_success_chance() to match.
     if (is_monster() && mrs < 6 && coinflip())
         return -1;
 
-    power = stepdown_value(power, 30, 40, 100, 120);
+    power = ench_power_stepdown(power);
 
     const int mrchance = (100 + mrs) - power;
     const int mrch2 = random2(100) + random2(101);
@@ -175,7 +167,7 @@ bool actor::can_hibernate(bool holi_only, bool intrinsic_only) const
     {
         // The monster is cold-resistant and can't be hibernated.
         if (intrinsic_only && is_monster()
-                ? get_mons_resist(this->as_monster(), MR_RES_COLD) > 0
+                ? get_mons_resist(as_monster(), MR_RES_COLD) > 0
                 : res_cold() > 0)
         {
             return false;
@@ -221,30 +213,7 @@ void actor::shield_block_succeeded(actor *foe)
     }
 }
 
-int actor::body_weight(bool base) const
-{
-    switch (body_size(PSIZE_BODY, base))
-    {
-    case SIZE_TINY:
-        return 150;
-    case SIZE_LITTLE:
-        return 300;
-    case SIZE_SMALL:
-        return 425;
-    case SIZE_MEDIUM:
-        return 550;
-    case SIZE_LARGE:
-        return 1300;
-    case SIZE_BIG:
-        return 1500;
-    case SIZE_GIANT:
-        return 1800;
-    default:
-        die("invalid body weight");
-    }
-}
-
-bool actor::inaccuracy() const
+int actor::inaccuracy() const
 {
     return wearing(EQ_AMULET, AMU_INACCURACY);
 }
@@ -256,16 +225,17 @@ bool actor::gourmand(bool calc_unid, bool items) const
 
 bool actor::res_corr(bool calc_unid, bool items) const
 {
-    return items && wearing(EQ_AMULET, AMU_RESIST_CORROSION, calc_unid);
+    return items && (wearing(EQ_AMULET, AMU_RESIST_CORROSION, calc_unid)
+                     || scan_artefacts(ARTP_RCORR, calc_unid));
 }
 
 // This is a bit confusing. This is not the function that determines whether or
 // not an actor is capable of teleporting, only whether they are specifically
-// under the influence of the "notele" effect. See item_blocks_teleport() in
-// item_use.cc for a superset of this function.
-bool actor::has_notele_item(bool calc_unid) const
+// under the influence of the "notele" effect. See actor::no_tele() for a
+// superset of this function.
+bool actor::has_notele_item(bool calc_unid, vector<item_def> *matches) const
 {
-    return scan_artefacts(ARTP_PREVENT_TELEPORTATION, calc_unid);
+    return scan_artefacts(ARTP_PREVENT_TELEPORTATION, calc_unid, matches);
 }
 
 bool actor::stasis(bool calc_unid, bool items) const
@@ -290,17 +260,9 @@ bool actor::clarity(bool calc_unid, bool items) const
                      || scan_artefacts(ARTP_CLARITY, calc_unid));
 }
 
-int actor::faith(bool calc_unid, bool items) const
+bool actor::faith(bool calc_unid, bool items) const
 {
-    int net_faith = 0;
-
-    if (items && wearing(EQ_AMULET, AMU_FAITH, calc_unid))
-        net_faith++;
-
-    if (is_player() && player_mutation_level(MUT_FORLORN))
-        net_faith--;
-
-    return net_faith;
+    return items && wearing(EQ_AMULET, AMU_FAITH, calc_unid);
 }
 
 bool actor::warding(bool calc_unid, bool items) const
@@ -327,7 +289,7 @@ bool actor::no_cast(bool calc_unid, bool items) const
 bool actor::rmut_from_item(bool calc_unid) const
 {
     return wearing(EQ_AMULET, AMU_RESIST_MUTATION, calc_unid)
-           || is_player() && player_equip_unrand(UNRAND_ORDER);
+           || scan_artefacts(ARTP_RMUT, calc_unid);
 }
 
 bool actor::evokable_berserk(bool calc_unid) const
@@ -339,14 +301,14 @@ bool actor::evokable_berserk(bool calc_unid) const
 bool actor::evokable_invis(bool calc_unid) const
 {
     return wearing(EQ_RINGS, RING_INVISIBILITY, calc_unid)
-           || wearing_ego(EQ_CLOAK, SPARM_DARKNESS, calc_unid)
+           || wearing_ego(EQ_CLOAK, SPARM_INVISIBILITY, calc_unid)
            || scan_artefacts(ARTP_INVISIBLE, calc_unid);
 }
 
 // Return an int so we know whether an item is the sole source.
 int actor::evokable_flight(bool calc_unid) const
 {
-    if (is_player() && you.form == TRAN_TREE)
+    if (is_player() && get_form()->forbids_flight())
         return 0;
 
     return wearing(EQ_RINGS, RING_FLIGHT, calc_unid)
@@ -354,11 +316,6 @@ int actor::evokable_flight(bool calc_unid) const
            + scan_artefacts(ARTP_FLY, calc_unid);
 }
 
-int actor::evokable_jump(bool calc_unid) const
-{
-    return wearing_ego(EQ_ALL_ARMOUR, SPARM_JUMPING, calc_unid);
-}
-
 int actor::spirit_shield(bool calc_unid, bool items) const
 {
     int ss = 0;
@@ -428,7 +385,7 @@ bool actor_slime_wall_immune(const actor *act)
  */
 bool actor::is_wall_clinging() const
 {
-    return props.exists("clinging") && props["clinging"].get_bool();
+    return props.exists(CLING_KEY) && props[CLING_KEY].get_bool();
 }
 
 /**
@@ -462,16 +419,17 @@ bool actor::check_clinging(bool stepped, bool door)
                     && !airborne();
 
     if (can_cling_to_walls())
-        props["clinging"] = clinging;
-    else if (props.exists("clinging"))
-        props.erase("clinging");
+        props[CLING_KEY] = clinging;
+    else if (props.exists(CLING_KEY))
+        props.erase(CLING_KEY);
 
     if (!stepped && was_clinging && !clinging)
     {
-        if (you.can_see(this))
+        if (you.can_see(*this))
         {
-            mprf("%s fall%s off the %s.", name(DESC_THE).c_str(),
-                 is_player() ? "" : "s", door ? "door" : "wall");
+            mprf("%s %s off the %s.", name(DESC_THE).c_str(),
+                 conj_verb("fall").c_str(),
+                 door ? "door" : "wall");
         }
         apply_location_effects(pos());
     }
@@ -480,8 +438,8 @@ bool actor::check_clinging(bool stepped, bool door)
 
 void actor::clear_clinging()
 {
-    if (props.exists("clinging"))
-        props["clinging"] = false;
+    if (props.exists(CLING_KEY))
+        props[CLING_KEY] = false;
 }
 
 void actor::clear_constricted()
@@ -493,10 +451,9 @@ void actor::clear_constricted()
 
 // End my constriction of i->first, but don't yet update my constricting map,
 // so as not to invalidate i.
-void actor::end_constriction(actor::constricting_t::iterator i,
-                             bool intentional, bool quiet)
+void actor::end_constriction(mid_t whom, bool intentional, bool quiet)
 {
-    actor *const constrictee = actor_by_mid(i->first);
+    actor *const constrictee = actor_by_mid(whom);
 
     if (!constrictee)
         return;
@@ -512,6 +469,9 @@ void actor::end_constriction(actor::constricting_t::iterator i,
                 pronoun(PRONOUN_POSSESSIVE).c_str(),
                 constrictee->name(DESC_THE).c_str());
     }
+
+    if (constrictee->is_player())
+        you.redraw_evasion = true;
 }
 
 void actor::stop_constricting(mid_t whom, bool intentional, bool quiet)
@@ -519,11 +479,11 @@ void actor::stop_constricting(mid_t whom, bool intentional, bool quiet)
     if (!constricting)
         return;
 
-    constricting_t::iterator i = constricting->find(whom);
+    auto i = constricting->find(whom);
 
     if (i != constricting->end())
     {
-        end_constriction(i, intentional, quiet);
+        end_constriction(whom, intentional, quiet);
         constricting->erase(i);
 
         if (constricting->empty())
@@ -539,10 +499,8 @@ void actor::stop_constricting_all(bool intentional, bool quiet)
     if (!constricting)
         return;
 
-    constricting_t::iterator i;
-
-    for (i = constricting->begin(); i != constricting->end(); ++i)
-        end_constriction(i, intentional, quiet);
+    for (const auto &entry : *constricting)
+        end_constriction(entry.first, intentional, quiet);
 
     delete constricting;
     constricting = 0;
@@ -572,17 +530,15 @@ void actor::clear_far_constrictions()
         return;
 
     vector<mid_t> need_cleared;
-    constricting_t::iterator i;
-    for (i = constricting->begin(); i != constricting->end(); ++i)
+    for (const auto &entry : *constricting)
     {
-        actor* const constrictee = actor_by_mid(i->first);
+        actor* const constrictee = actor_by_mid(entry.first);
         if (!constrictee || !adjacent(pos(), constrictee->pos()))
-            need_cleared.push_back(i->first);
+            need_cleared.push_back(entry.first);
     }
 
-    vector<mid_t>::iterator j;
-    for (j = need_cleared.begin(); j != need_cleared.end(); ++j)
-        stop_constricting(*j, false, false);
+    for (mid_t whom : need_cleared)
+        stop_constricting(whom, false, false);
 }
 
 void actor::start_constricting(actor &whom, int dur)
@@ -595,6 +551,9 @@ void actor::start_constricting(actor &whom, int dur)
     (*constricting)[whom.mid] = dur;
     whom.constricted_by = mid;
     whom.held = constriction_damage() ? HELD_CONSTRICTED : HELD_MONSTER;
+
+    if (whom.is_player())
+        you.redraw_evasion = true;
 }
 
 int actor::num_constricting() const
@@ -617,16 +576,16 @@ void actor::accum_has_constricted()
     if (!constricting)
         return;
 
-    constricting_t::iterator i;
-    for (i = constricting->begin(); i != constricting->end(); ++i)
-        i->second += you.time_taken;
+    for (auto &entry : *constricting)
+        entry.second += you.time_taken;
 }
 
 bool actor::can_constrict(actor* defender)
 {
+    ASSERT(defender); // XXX: change to actor &defender
     return (!is_constricting() || has_usable_tentacle())
            && !defender->is_constricted()
-           && can_see(defender)
+           && can_see(*defender)
            && !confused()
            && body_size(PSIZE_BODY) >= defender->body_size(PSIZE_BODY)
            && defender->res_constrict() < 3
@@ -653,7 +612,7 @@ void actor::handle_constriction()
     if (!constricting || !constriction_damage())
         return;
 
-    actor::constricting_t::iterator i = constricting->begin();
+    auto i = constricting->begin();
     // monster_die() can cause constricting() to go away.
     while (constricting && i != constricting->end())
     {
@@ -678,12 +637,13 @@ void actor::handle_constriction()
         damage = timescale_damage(this, damage);
         DIAG_ONLY(const int timescale_dam = damage);
 
-        damage = defender->hurt(this, damage, BEAM_MISSILE, false);
+        damage = defender->hurt(this, damage, BEAM_MISSILE,
+                                KILLED_BY_MONSTER, "", "", false);
         DIAG_ONLY(const int infdam = damage);
 
         string exclamations;
         if (damage <= 0 && is_player()
-            && you.can_see(defender))
+            && you.can_see(*defender))
         {
             exclamations = ", but do no damage.";
         }
@@ -704,7 +664,7 @@ void actor::handle_constriction()
             }
         }
 
-        if (is_player() || you.can_see(this))
+        if (is_player() || you.can_see(*this))
         {
             mprf("%s %s %s%s%s",
                  (is_player() ? "You"
@@ -718,7 +678,7 @@ void actor::handle_constriction()
 #endif
                  exclamations.c_str());
         }
-        else if (you.can_see(defender) || defender->is_player())
+        else if (you.can_see(*defender) || defender->is_player())
         {
             mprf("%s %s constricted%s%s",
                  defender->name(DESC_THE).c_str(),
@@ -751,7 +711,7 @@ string actor::describe_props() const
     if (props.size() == 0)
         return "";
 
-    for (CrawlHashTable::const_iterator i = props.begin(); i != props.end(); ++i)
+    for (auto i = props.begin(); i != props.end(); ++i)
     {
         if (i != props.begin())
             oss <<  ", ";
@@ -810,7 +770,7 @@ bool actor::torpor_slowed() const
 {
     if (!props.exists(TORPOR_SLOWED_KEY) || is_sanctuary(pos())
         || is_stationary()
-        || (is_monster() && this->as_monster()->check_stasis(true))
+        || (is_monster() && as_monster()->check_stasis(true))
         || (!is_monster() && stasis()))
     {
         return false;
@@ -821,9 +781,7 @@ bool actor::torpor_slowed() const
         const monster *mons = *ri;
         if (mons && mons->type == MONS_TORPOR_SNAIL
             && !is_sanctuary(mons->pos())
-            && !mons_aligned(mons, this)
-            && !mons->has_ench(ENCH_CHARM) && mons->attitude == ATT_HOSTILE)
-            // friendly torpor snails are way too abusable otherwise :(
+            && !mons_aligned(mons, this))
         {
             return true;
         }
@@ -831,3 +789,85 @@ bool actor::torpor_slowed() const
 
     return false;
 }
+
+string actor::resist_margin_phrase(int margin) const
+{
+    if (res_magic() == MAG_IMMUNE)
+        return " " + conj_verb("are") + " unaffected.";
+
+    static const string resist_messages[][2] =
+    {
+      { " barely %s.",                  "resist" },
+      { " %s to resist.",               "struggle" },
+      { " %s with significant effort.", "resist" },
+      { " %s with some effort.",        "resist" },
+      { " easily %s.",                  "resist" },
+      { " %s with almost no effort.",   "resist" },
+    };
+
+    const int index = max(0, min((int)ARRAYSZ(resist_messages) - 1,
+                                 ((margin + 45) / 15)));
+
+    return make_stringf(resist_messages[index][0].c_str(),
+                        conj_verb(resist_messages[index][1]).c_str());
+}
+
+void actor::collide(coord_def newpos, const actor *agent, int pow)
+{
+    actor *other = actor_at(newpos);
+    ASSERT(this != other);
+    ASSERT(alive());
+
+    if (is_insubstantial())
+        return;
+
+    if (is_monster())
+        behaviour_event(as_monster(), ME_WHACK, agent);
+
+    dice_def damage(2, 1 + pow / 10);
+
+    if (other && other->alive())
+    {
+        if (other->is_monster())
+            behaviour_event(other->as_monster(), ME_WHACK, agent);
+        if (you.can_see(*this) || you.can_see(*other))
+        {
+            mprf("%s %s with %s!",
+                 name(DESC_THE).c_str(),
+                 conj_verb("collide").c_str(),
+                 other->name(DESC_THE).c_str());
+        }
+        const string thisname = name(DESC_A, true);
+        const string othername = other->name(DESC_A, true);
+        other->hurt(agent, other->apply_ac(damage.roll()),
+                    BEAM_MISSILE, KILLED_BY_COLLISION,
+                    othername, thisname);
+        if (alive())
+        {
+            hurt(agent, apply_ac(damage.roll()), BEAM_MISSILE,
+                 KILLED_BY_COLLISION, thisname, othername);
+        }
+        return;
+    }
+
+    if (you.can_see(*this))
+    {
+        if (!can_pass_through_feat(grd(newpos)))
+        {
+            mprf("%s %s into %s!",
+                 name(DESC_THE).c_str(), conj_verb("slam").c_str(),
+                 env.map_knowledge(newpos).known()
+                 ? feature_description_at(newpos, false, DESC_THE, false)
+                       .c_str()
+                 : "something");
+        }
+        else
+        {
+            mprf("%s violently %s moving!",
+                 name(DESC_THE).c_str(), conj_verb("stop").c_str());
+        }
+    }
+    hurt(agent, apply_ac(damage.roll()), BEAM_MISSILE,
+         KILLED_BY_COLLISION, "",
+         feature_description_at(newpos, false, DESC_A, false));
+}
