@@ -9,45 +9,41 @@
 #include "areas.h"
 
 #include "act-iter.h"
-#include "art-enum.h"
 #include "artefact.h"
-#include "beam.h"
+#include "art-enum.h"
 #include "cloud.h"
-#include "coord.h"
 #include "coordit.h"
 #include "env.h"
 #include "fprop.h"
 #include "godconduct.h"
 #include "libutil.h"
 #include "losglobal.h"
+#include "message.h"
 #include "mon-behv.h"
-#include "mon-util.h"
-#include "monster.h"
-#include "player.h"
 #include "religion.h"
 #include "stepdown.h"
 #include "terrain.h"
 #include "traps.h"
 #include "travel.h"
 
-enum areaprop_flag
+enum class areaprop
 {
-    APROP_SANCTUARY_1   = (1 << 0),
-    APROP_SANCTUARY_2   = (1 << 1),
-    APROP_SILENCE       = (1 << 2),
-    APROP_HALO          = (1 << 3),
-    APROP_LIQUID        = (1 << 4),
-    APROP_ACTUAL_LIQUID = (1 << 5),
-    APROP_ORB           = (1 << 6),
-    APROP_UMBRA         = (1 << 7),
-    APROP_QUAD          = (1 << 8),
-    APROP_DISJUNCTION   = (1 << 9),
-    APROP_SOUL_AURA     = (1 << 10),
+    SANCTUARY_1   = (1 << 0),
+    SANCTUARY_2   = (1 << 1),
+    SILENCE       = (1 << 2),
+    HALO          = (1 << 3),
+    LIQUID        = (1 << 4),
+    ACTUAL_LIQUID = (1 << 5),
+    ORB           = (1 << 6),
+    UMBRA         = (1 << 7),
+    QUAD          = (1 << 8),
+    DISJUNCTION   = (1 << 9),
+    SOUL_AURA     = (1 << 10),
 #if TAG_MAJOR_VERSION == 34
-    APROP_HOT           = (1 << 11),
+    HOT           = (1 << 11),
 #endif
-    APROP_GOLD          = (1 << 12),
 };
+DEF_BITFIELD(areaprops, areaprop);
 
 struct area_centre
 {
@@ -58,7 +54,7 @@ struct area_centre
     explicit area_centre (area_centre_type t, coord_def c, int r) : type(t), centre(c), radius(r) { }
 };
 
-typedef FixedArray<uint32_t, GXM, GYM> propgrid_t;
+typedef FixedArray<areaprops, GXM, GYM> propgrid_t;
 
 static vector<area_centre> _agrid_centres;
 
@@ -66,14 +62,14 @@ static propgrid_t _agrid;
 static bool _agrid_valid = false;
 static bool no_areas = false;
 
-static void _set_agrid_flag(const coord_def& p, areaprop_flag f)
+static void _set_agrid_flag(const coord_def& p, areaprop f)
 {
     _agrid(p) |= f;
 }
 
-static bool _check_agrid_flag(const coord_def& p, areaprop_flag f)
+static bool _check_agrid_flag(const coord_def& p, areaprop f)
 {
-    return _agrid(p) & f;
+    return bool(_agrid(p) & f);
 }
 
 void invalidate_agrid(bool recheck_new)
@@ -87,10 +83,10 @@ void areas_actor_moved(const actor* act, const coord_def& oldpos)
 {
     if (act->alive() &&
         (you.entering_level
-         || act->halo_radius2() > -1 || act->silence_radius2() > -1
-         || act->liquefying_radius2() > -1 || act->umbra_radius2() > -1
+         || act->halo_radius() > -1 || act->silence_radius() > -1
+         || act->liquefying_radius() > -1 || act->umbra_radius() > -1
 #if TAG_MAJOR_VERSION == 34
-         || act->heat_radius2() > -1
+         || act->heat_radius() > -1
 #endif
          ))
     {
@@ -103,56 +99,56 @@ static void _actor_areas(actor *a)
 {
     int r;
 
-    if ((r = a->silence_radius2()) >= 0)
+    if ((r = a->silence_radius()) >= 0)
     {
-        _agrid_centres.push_back(area_centre(AREA_SILENCE, a->pos(), r));
+        _agrid_centres.emplace_back(AREA_SILENCE, a->pos(), r);
 
-        for (radius_iterator ri(a->pos(), r, C_CIRCLE); ri; ++ri)
-            _set_agrid_flag(*ri, APROP_SILENCE);
+        for (radius_iterator ri(a->pos(), r, C_SQUARE); ri; ++ri)
+            _set_agrid_flag(*ri, areaprop::SILENCE);
         no_areas = false;
     }
 
-    if ((r = a->halo_radius2()) >= 0)
+    if ((r = a->halo_radius()) >= 0)
     {
-        _agrid_centres.push_back(area_centre(AREA_HALO, a->pos(), r));
+        _agrid_centres.emplace_back(AREA_HALO, a->pos(), r);
 
-        for (radius_iterator ri(a->pos(), r, C_CIRCLE, LOS_DEFAULT); ri; ++ri)
-            _set_agrid_flag(*ri, APROP_HALO);
+        for (radius_iterator ri(a->pos(), r, C_SQUARE, LOS_DEFAULT); ri; ++ri)
+            _set_agrid_flag(*ri, areaprop::HALO);
         no_areas = false;
     }
 
-    if ((r = a->liquefying_radius2()) >= 0)
+    if ((r = a->liquefying_radius()) >= 0)
     {
-        _agrid_centres.push_back(area_centre(AREA_LIQUID, a->pos(), r));
+        _agrid_centres.emplace_back(AREA_LIQUID, a->pos(), r);
 
-        for (radius_iterator ri(a->pos(), r, C_CIRCLE, LOS_SOLID); ri; ++ri)
+        for (radius_iterator ri(a->pos(), r, C_SQUARE, LOS_SOLID); ri; ++ri)
         {
             dungeon_feature_type f = grd(*ri);
 
-            _set_agrid_flag(*ri, APROP_LIQUID);
+            _set_agrid_flag(*ri, areaprop::LIQUID);
 
             if (feat_has_solid_floor(f) && !feat_is_water(f))
-                _set_agrid_flag(*ri, APROP_ACTUAL_LIQUID);
+                _set_agrid_flag(*ri, areaprop::ACTUAL_LIQUID);
         }
         no_areas = false;
     }
 
-    if ((r = a->umbra_radius2()) >= 0)
+    if ((r = a->umbra_radius()) >= 0)
     {
-        _agrid_centres.push_back(area_centre(AREA_UMBRA, a->pos(), r));
+        _agrid_centres.emplace_back(AREA_UMBRA, a->pos(), r);
 
-        for (radius_iterator ri(a->pos(), r, C_CIRCLE, LOS_DEFAULT); ri; ++ri)
-            _set_agrid_flag(*ri, APROP_UMBRA);
+        for (radius_iterator ri(a->pos(), r, C_SQUARE, LOS_DEFAULT); ri; ++ri)
+            _set_agrid_flag(*ri, areaprop::UMBRA);
         no_areas = false;
     }
 
 #if TAG_MAJOR_VERSION == 34
-    if ((r = a->heat_radius2()) >= 0)
+    if ((r = a->heat_radius()) >= 0)
     {
-        _agrid_centres.push_back(area_centre(AREA_HOT, a->pos(), r));
+        _agrid_centres.emplace_back(AREA_HOT, a->pos(), r);
 
-        for (radius_iterator ri(a->pos(), r, C_CIRCLE, LOS_NO_TRANS); ri; ++ri)
-            _set_agrid_flag(*ri, APROP_HOT);
+        for (radius_iterator ri(a->pos(), r, C_SQUARE, LOS_NO_TRANS); ri; ++ri)
+            _set_agrid_flag(*ri, areaprop::HOT);
         no_areas = false;
     }
 #endif
@@ -162,7 +158,7 @@ static void _actor_areas(actor *a)
  * Update the area grid cache.
  *
  * Updates the _agrid FixedArray of grid information flags using the
- * areaprop_flag types.
+ * areaprop types.
  */
 static void _update_agrid()
 {
@@ -172,7 +168,7 @@ static void _update_agrid()
         return;
     }
 
-    _agrid.init(0);
+    _agrid.init(areaprops());
     _agrid_centres.clear();
 
     no_areas = true;
@@ -181,65 +177,45 @@ static void _update_agrid()
     for (monster_iterator mi; mi; ++mi)
         _actor_areas(*mi);
 
-    if (you_worship(GOD_GOZAG))
-    {
-        const int r = 2;
-        for (rectangle_iterator ri(0); ri; ++ri)
-        {
-            // ASSUMPTION: gold will always be on the top of the pile.
-            if (igrd(*ri) != NON_ITEM && mitm[igrd(*ri)].base_type == OBJ_GOLD
-                && mitm[igrd(*ri)].special > 0)
-            {
-                no_areas = false;
-                _agrid_centres.push_back(area_centre(AREA_GOLD, *ri, r));
-                for (radius_iterator rdi(*ri, r, C_CIRCLE, LOS_NO_TRANS);
-                     rdi; ++rdi)
-                {
-                    _set_agrid_flag(*rdi, APROP_GOLD);
-                }
-            }
-        }
-    }
-
     if (player_has_orb() && !you.pos().origin())
     {
-        const int r = 5;
-        _agrid_centres.push_back(area_centre(AREA_ORB, you.pos(), r));
-        for (radius_iterator ri(you.pos(), r, C_CIRCLE, LOS_DEFAULT); ri; ++ri)
-            _set_agrid_flag(*ri, APROP_ORB);
+        const int r = 2;
+        _agrid_centres.emplace_back(AREA_ORB, you.pos(), r);
+        for (radius_iterator ri(you.pos(), r, C_SQUARE, LOS_DEFAULT); ri; ++ri)
+            _set_agrid_flag(*ri, areaprop::ORB);
         no_areas = false;
     }
 
     if (you.duration[DUR_QUAD_DAMAGE])
     {
-        const int r = 5;
-        _agrid_centres.push_back(area_centre(AREA_QUAD, you.pos(), r));
-        for (radius_iterator ri(you.pos(), r, C_CIRCLE);
+        const int r = 2;
+        _agrid_centres.emplace_back(AREA_QUAD, you.pos(), r);
+        for (radius_iterator ri(you.pos(), r, C_SQUARE);
              ri; ++ri)
         {
             if (cell_see_cell(you.pos(), *ri, LOS_DEFAULT))
-                _set_agrid_flag(*ri, APROP_QUAD);
+                _set_agrid_flag(*ri, areaprop::QUAD);
         }
         no_areas = false;
     }
 
     if (you.duration[DUR_DISJUNCTION])
     {
-        const int r = 27;
-        _agrid_centres.push_back(area_centre(AREA_DISJUNCTION, you.pos(), r));
-        for (radius_iterator ri(you.pos(), r, C_CIRCLE);
+        const int r = 4;
+        _agrid_centres.emplace_back(AREA_DISJUNCTION, you.pos(), r);
+        for (radius_iterator ri(you.pos(), r, C_SQUARE);
              ri; ++ri)
         {
             if (cell_see_cell(you.pos(), *ri, LOS_DEFAULT))
-                _set_agrid_flag(*ri, APROP_DISJUNCTION);
+                _set_agrid_flag(*ri, areaprop::DISJUNCTION);
         }
         no_areas = false;
     }
 
     if (!env.sunlight.empty())
     {
-        for (size_t i = 0; i < env.sunlight.size(); ++i)
-            _set_agrid_flag(env.sunlight[i].first, APROP_HALO);
+        for (const auto &entry : env.sunlight)
+            _set_agrid_flag(entry.first, areaprop::HALO);
         no_areas = false;
     }
 
@@ -250,25 +226,25 @@ static void _update_agrid()
 
 static area_centre_type _get_first_area(const coord_def& f)
 {
-    uint32_t a = _agrid(f);
-    if (a & APROP_SANCTUARY_1)
+    areaprops a = _agrid(f);
+    if (a & areaprop::SANCTUARY_1)
         return AREA_SANCTUARY;
-    if (a & APROP_SANCTUARY_2)
+    if (a & areaprop::SANCTUARY_2)
         return AREA_SANCTUARY;
-    if (a & APROP_SILENCE)
+    if (a & areaprop::SILENCE)
         return AREA_SILENCE;
 #if TAG_MAJOR_VERSION == 34
-    if (a & APROP_HOT)
+    if (a & areaprop::HOT)
         return AREA_HOT;
 #endif
-    if (a & APROP_HALO)
+    if (a & areaprop::HALO)
         return AREA_HALO;
-    if (a & APROP_UMBRA)
+    if (a & areaprop::UMBRA)
         return AREA_UMBRA;
     // liquid is always applied; actual_liquid is on top
     // of this. If we find the first, we don't care about
     // the second.
-    if (a & APROP_LIQUID)
+    if (a & areaprop::LIQUID)
         return AREA_LIQUID;
 
     return AREA_NONE;
@@ -282,7 +258,7 @@ coord_def find_centre_for(const coord_def& f, area_centre_type at)
     if (!_agrid_valid)
         _update_agrid();
 
-    if (_agrid(f) == 0)
+    if (!_agrid(f))
         return coord_def(-1, -1);
 
     if (_agrid_centres.empty())
@@ -299,16 +275,15 @@ coord_def find_centre_for(const coord_def& f, area_centre_type at)
     // on the off chance that there is an error, assert here
     ASSERT(at != AREA_NONE);
 
-    for (unsigned int i = 0; i < _agrid_centres.size(); i++)
+    for (const area_centre &a : _agrid_centres)
     {
-        area_centre a = _agrid_centres[i];
         if (a.type != at)
             continue;
 
         if (a.centre == f)
             return f;
 
-        int d = distance2(a.centre, f);
+        int d = grid_distance(a.centre, f);
         if (d <= a.radius && (d <= dist || dist == 0))
         {
             possible = a.centre;
@@ -335,7 +310,7 @@ bool remove_sanctuary(bool did_attack)
     if (!in_bounds(env.sanctuary_pos))
         return false;
 
-    const int radius = 5;
+    const int radius = 4;
     bool seen_change = false;
     for (rectangle_iterator ri(env.sanctuary_pos, radius, true); ri; ++ri)
         if (is_sanctuary(*ri))
@@ -368,7 +343,7 @@ bool remove_sanctuary(bool did_attack)
 // For the last (radius) counter turns the sanctuary will slowly shrink.
 void decrease_sanctuary_radius()
 {
-    const int radius = 5;
+    const int radius = 4;
 
     // For the last (radius-1) turns 33% chance of not decreasing.
     if (env.sanctuary_time < radius && one_chance_in(3))
@@ -386,10 +361,10 @@ void decrease_sanctuary_radius()
 
     for (rectangle_iterator ri(env.sanctuary_pos, size+1, true); ri; ++ri)
     {
-        int dist = distance2(*ri, env.sanctuary_pos);
+        int dist = grid_distance(*ri, env.sanctuary_pos);
 
         // If necessary overwrite sanctuary property.
-        if (dist > size*size)
+        if (dist > size)
             _remove_sanctuary_property(*ri);
     }
 
@@ -409,22 +384,18 @@ void create_sanctuary(const coord_def& center, int time)
 
     // radius could also be influenced by Inv
     // and would then have to be stored globally.
-    const int radius      = 5;
+    const int radius      = 4;
     int       blood_count = 0;
     int       trap_count  = 0;
     int       scare_count = 0;
     int       cloud_count = 0;
-    monster* seen_mon    = NULL;
-
-    // Since revealing mimics can move monsters, we do it first.
-    for (radius_iterator ri(center, radius, C_POINTY); ri; ++ri)
-        discover_mimic(*ri);
+    monster* seen_mon    = nullptr;
 
     int shape = random2(4);
-    for (radius_iterator ri(center, radius, C_POINTY); ri; ++ri)
+    for (radius_iterator ri(center, radius, C_SQUARE); ri; ++ri)
     {
         const coord_def pos = *ri;
-        const int dist = distance2(center, pos);
+        const int dist = grid_distance(center, pos);
 
         if (testbits(env.pgrid(pos), FPROP_BLOODY) && you.see_cell(pos))
             blood_count++;
@@ -447,10 +418,8 @@ void create_sanctuary(const coord_def& center, int time)
             in_yellow = (x == 0 || y == 0 || x == y || x == -y);
             break;
         case 1:                 // circles
-            in_yellow = (dist >= (radius-1)*(radius-1)
-                         && dist <= radius*radius
-                         || dist >= (radius/2-1)*(radius/2-1)
-                            && dist <= radius*radius/4);
+            in_yellow = (dist == radius
+                         || dist == radius/2);
             break;
         case 2:                 // latticed
             in_yellow = (x%2 == 0 || y%2 == 0);
@@ -484,7 +453,7 @@ void create_sanctuary(const coord_def& center, int time)
                 mons_start_fleeing_from_sanctuary(mon);
 
                 // Check to see that monster is actually fleeing.
-                if (mons_is_fleeing(mon) && you.can_see(mon))
+                if (mons_is_fleeing(mon) && you.can_see(*mon))
                 {
                     scare_count++;
                     seen_mon = mon;
@@ -518,7 +487,7 @@ void create_sanctuary(const coord_def& center, int time)
     if (blood_count > 0)
         mprf(MSGCH_GOD, "By Zin's power, all blood is cleared from the sanctuary.");
 
-    if (scare_count == 1 && seen_mon != NULL)
+    if (scare_count == 1 && seen_mon != nullptr)
         simple_monster_message(seen_mon, " turns to flee the light!");
     else if (scare_count > 0)
         mpr("The monsters scatter in all directions!");
@@ -527,27 +496,27 @@ void create_sanctuary(const coord_def& center, int time)
 /////////////
 // Silence
 
-// pre-squared radius, calculated from remaining duration
+// radius, calculated from remaining duration
 // dur starts at 10 (low power) and is capped at 100
-// maximal range: 6*6 + 1 = 37
+// maximal range: 5
 // last 6 turns: range 0, hence only the player silenced
 static int _silence_range(int dur)
 {
     if (dur <= 0)
         return -1;
     dur /= BASELINE_DELAY; // now roughly number of turns
-    return max(0, min(dur - 6, 37));
+    return isqrt(max(0, min(3*(dur - 5)/4, 25)));
 }
 
-int player::silence_radius2() const
+int player::silence_radius() const
 {
     return _silence_range(duration[DUR_SILENCE]);
 }
 
-int monster::silence_radius2() const
+int monster::silence_radius() const
 {
     if (type == MONS_SILENT_SPECTRE)
-        return 150;
+        return 10;
 
     if (!has_ench(ENCH_SILENCE))
         return -1;
@@ -565,7 +534,7 @@ bool silenced(const coord_def& p)
         return false;
     if (!_agrid_valid)
         _update_agrid();
-    return _check_agrid_flag(p, APROP_SILENCE);
+    return _check_agrid_flag(p, areaprop::SILENCE);
 }
 
 /////////////
@@ -577,7 +546,7 @@ bool haloed(const coord_def& p)
         return false;
     if (!_agrid_valid)
         _update_agrid();
-    return _check_agrid_flag(p, APROP_HALO);
+    return _check_agrid_flag(p, areaprop::HALO);
 }
 
 bool actor::haloed() const
@@ -585,7 +554,7 @@ bool actor::haloed() const
     return ::haloed(pos());
 }
 
-int player::halo_radius2() const
+int player::halo_radius() const
 {
     int size = -1;
 
@@ -593,44 +562,44 @@ int player::halo_radius2() const
         && !penance[GOD_SHINING_ONE])
     {
         // The cap is reached at piety 160 = ******.
-        size = min(LOS_RADIUS*LOS_RADIUS + 1, piety * piety / 393);
+        size = min((int)piety, piety_breakpoint(5)) * LOS_RADIUS / piety_breakpoint(5);
     }
 
     if (player_equip_unrand(UNRAND_BRILLIANCE))
-        size = max(size, 10);
+        size = max(size, 3);
 
     return size;
 }
 
-int monster::halo_radius2() const
+int monster::halo_radius() const
 {
     item_def* weap = mslot_item(MSLOT_WEAPON);
     int size = -1;
 
-    if (weap && is_unrandom_artefact(*weap) && weap->special == UNRAND_BRILLIANCE)
-        size = 10;
+    if (weap && is_unrandom_artefact(*weap, UNRAND_BRILLIANCE))
+        size = 3;
 
     if (holiness() != MH_HOLY)
         return size;
-    // The values here depend on 1. power, 2. sentience.  Thus, high-ranked
+    // The values here depend on 1. power, 2. sentience. Thus, high-ranked
     // sentient celestials have really big haloes, while holy animals get
-    // small ones.
+    // little or none.
     switch (type)
     {
     case MONS_ANGEL:
-        return 26;
+        return 4;
     case MONS_CHERUB:
-        return 29;
+        return 4;
     case MONS_DAEVA:
-        return 32;
-    case MONS_SERAPH:
-        return 50;
+        return 4;
     case MONS_OPHAN:
-        return 65; // highest rank among sentient ones
+        return 6;
+    case MONS_SERAPH:
+        return 7; // highest rank among sentient ones
     case MONS_HOLY_SWINE:
-        return 2;  // only notionally holy
+        return 1;  // only notionally holy
     case MONS_MENNAS:
-        return 5;  // ???  Low on grace or what?
+        return 2;  // ???  Low on grace or what?
     default:
         return -1;
     }
@@ -640,12 +609,12 @@ int monster::halo_radius2() const
 // Leda's Liquefaction
 //
 
-int player::liquefying_radius2() const
+int player::liquefying_radius() const
 {
     return _silence_range(duration[DUR_LIQUEFYING]);
 }
 
-int monster::liquefying_radius2() const
+int monster::liquefying_radius() const
 {
     if (!has_ench(ENCH_LIQUEFYING))
         return -1;
@@ -669,10 +638,10 @@ bool liquefied(const coord_def& p, bool check_actual)
 
     // "actually" liquefied (ie, check for movement)
     if (check_actual)
-        return _check_agrid_flag(p, APROP_ACTUAL_LIQUID);
+        return _check_agrid_flag(p, areaprop::ACTUAL_LIQUID);
     // just recoloured for consistency
     else
-        return _check_agrid_flag(p, APROP_LIQUID);
+        return _check_agrid_flag(p, areaprop::LIQUID);
 }
 
 /////////////
@@ -686,7 +655,7 @@ bool orb_haloed(const coord_def& p)
     if (!_agrid_valid)
         _update_agrid();
 
-    return _check_agrid_flag(p, APROP_ORB);
+    return _check_agrid_flag(p, areaprop::ORB);
 }
 
 /////////////
@@ -700,7 +669,7 @@ bool quad_haloed(const coord_def& p)
     if (!_agrid_valid)
         _update_agrid();
 
-    return _check_agrid_flag(p, APROP_QUAD);
+    return _check_agrid_flag(p, areaprop::QUAD);
 }
 
 /////////////
@@ -714,7 +683,7 @@ bool disjunction_haloed(const coord_def& p)
     if (!_agrid_valid)
         _update_agrid();
 
-    return _check_agrid_flag(p, APROP_DISJUNCTION);
+    return _check_agrid_flag(p, areaprop::DISJUNCTION);
 }
 
 /////////////
@@ -728,7 +697,7 @@ bool umbraed(const coord_def& p)
     if (!_agrid_valid)
         _update_agrid();
 
-    return _check_agrid_flag(p, APROP_UMBRA);
+    return _check_agrid_flag(p, areaprop::UMBRA);
 }
 
 // Whether actor is in an umbra.
@@ -737,7 +706,7 @@ bool actor::umbraed() const
     return ::umbraed(pos());
 }
 
-int player::umbra_radius2() const
+int player::umbra_radius() const
 {
     int size = -1;
 
@@ -745,20 +714,20 @@ int player::umbra_radius2() const
         && !penance[GOD_DITHMENOS])
     {
         // The cap is reached at piety 160 = ******.
-        size = min(LOS_RADIUS*LOS_RADIUS + 1, piety * piety / 393);
+        size = min((int)piety, piety_breakpoint(5)) * LOS_RADIUS / piety_breakpoint(5);
     }
 
     if (player_equip_unrand(UNRAND_SHADOWS))
-        size = max(size, 10);
+        size = max(size, 3);
 
     return size;
 }
 
-int monster::umbra_radius2() const
+int monster::umbra_radius() const
 {
     item_def* ring = mslot_item(MSLOT_JEWELLERY);
-    if (ring && is_unrandom_artefact(*ring) && ring->special == UNRAND_SHADOWS)
-        return 10;
+    if (ring && is_unrandom_artefact(*ring, UNRAND_SHADOWS))
+        return 3;
 
     if (holiness() != MH_UNDEAD)
         return -1;
@@ -766,7 +735,7 @@ int monster::umbra_radius2() const
     switch (type)
     {
     case MONS_PROFANE_SERVITOR:
-        return 40; // Very unholy!
+        return 5; // Very unholy!
     default:
         return -1;
     }
@@ -777,7 +746,7 @@ int monster::umbra_radius2() const
 // Heat aura (lava orcs).
 
 // Player radius
-int player::heat_radius2() const
+int player::heat_radius() const
 {
     if (species != SP_LAVA_ORC)
         return -1;
@@ -785,11 +754,11 @@ int player::heat_radius2() const
     if (!temperature_effect(LORC_HEAT_AURA))
         return -1;
 
-    return 2; // Surrounds you to radius of 1.
+    return 1; // Surrounds you to radius of 1.
 }
 
 // Stub for monster radius
-int monster::heat_radius2() const
+int monster::heat_radius() const
 {
     return -1;
 }
@@ -802,7 +771,7 @@ bool heated(const coord_def& p)
     if (!_agrid_valid)
         _update_agrid();
 
-    return _check_agrid_flag(p, APROP_HOT);
+    return _check_agrid_flag(p, areaprop::HOT);
 }
 
 bool actor::heated() const
@@ -810,17 +779,3 @@ bool actor::heated() const
     return ::heated(pos());
 }
 #endif
-
-/////////////
-// Gold aura (Gozag).
-
-bool golden(const coord_def& p)
-{
-    if (!map_bounds(p))
-        return false;
-
-    if (!_agrid_valid)
-        _update_agrid();
-
-    return _check_agrid_flag(p, APROP_GOLD);
-}
