@@ -8,31 +8,25 @@
 #include "arena.h"
 
 #include "act-iter.h"
-#include "cio.h"
 #include "colour.h"
 #include "command.h"
 #include "dungeon.h"
 #include "end.h"
-#include "env.h"
-#include "externs.h"
 #include "food.h"
+#include "itemname.h"
 #include "items.h"
-#include "itemname.h" // for make_name()
 #include "libutil.h"
 #include "los.h"
 #include "macro.h"
 #include "maps.h"
 #include "message.h"
-#include "mon-behv.h"
+#include "misc.h"
+#include "mgen_data.h"
 #include "mon-death.h"
 #include "mon-pick.h"
-#include "mon-util.h"
-#include "mon-place.h"
-#include "mgen_data.h"
+#include "mon-tentacle.h"
 #include "ng-init.h"
-#include "options.h"
 #include "spl-miscast.h"
-#include "spl-util.h"
 #include "state.h"
 #include "stringutil.h"
 #include "teleport.h"
@@ -43,7 +37,6 @@
 #include "unicode.h"
 #include "version.h"
 #include "view.h"
-#include "viewgeom.h"
 
 #define ARENA_VERBOSE
 
@@ -135,20 +128,16 @@ namespace arena
     static bool cycle_random     = false;
     static uint32_t cycle_random_pos = 0;
 
-    static FILE *file = NULL;
+    static FILE *file = nullptr;
     static level_id place(BRANCH_DEPTHS, 1);
 
     static void adjust_spells(monster* mons, bool no_summons, bool no_animate)
     {
         monster_spells &spells(mons->spells);
-        for (int i = 0; i < NUM_MONSTER_SPELL_SLOTS; ++i)
-        {
-            spell_type sp = spells[i];
-            if (no_summons && spell_typematch(sp, SPTYP_SUMMONING))
-                spells[i] = SPELL_NO_SPELL;
-            else if (no_animate && sp == SPELL_ANIMATE_DEAD)
-                spells[i] = SPELL_NO_SPELL;
-        }
+        erase_if(spells, [&](const mon_spell_slot &t) {
+            return (no_summons && spell_typematch(t.spell, SPTYP_SUMMONING))
+                || (no_animate && t.spell == SPELL_ANIMATE_DEAD);
+        });
     }
 
     static void adjust_monsters()
@@ -163,23 +152,22 @@ namespace arena
 
     static void list_eq(const monster *mon)
     {
-        if (!Options.arena_list_eq || file == NULL)
+        if (!Options.arena_list_eq || file == nullptr)
             return;
 
         vector<int> items;
-
-        for (int i = 0; i < NUM_MONSTER_SLOTS; i++)
-            if (mon->inv[i] != NON_ITEM)
-                items.push_back(mon->inv[i]);
+        for (short it : mon->inv)
+            if (it != NON_ITEM)
+                items.push_back(it);
 
         if (items.empty())
             return;
 
         fprintf(file, "%s:\n", mon->name(DESC_PLAIN, true).c_str());
 
-        for (unsigned int i = 0; i < items.size(); i++)
+        for (int iidx : items)
         {
-            item_def &item = mitm[items[i]];
+            item_def &item = mitm[iidx];
             fprintf(file, "        %s\n",
                     item.name(DESC_PLAIN, false, true).c_str());
         }
@@ -302,10 +290,9 @@ namespace arena
         fact.clear();
         fact.desc = spec;
 
-        vector<string> monsters = split_string(",", spec);
-        for (int i = 0, size = monsters.size(); i < size; ++i)
+        for (const string &monster : split_string(",", spec))
         {
-            const string err = fact.members.add_mons(monsters[i], false);
+            const string err = fact.members.add_mons(monster, false);
             if (!err.empty())
                 throw err;
         }
@@ -371,10 +358,9 @@ namespace arena
             }
         }
 
-        const string glyphs = strip_tag_prefix(spec, "ban_glyphs:");
-        for (unsigned int i = 0; i < glyphs.size(); i++)
-            if (!(glyphs[i] & !127))
-                banned_glyphs[static_cast<int>(glyphs[i])] = true;
+        for (unsigned char gly : strip_tag_prefix(spec, "ban_glyphs:"))
+            if (gly < ARRAYSZ(banned_glyphs))
+                banned_glyphs[gly] = true;
 
         vector<string> factions = split_string(" v ", spec);
 
@@ -443,26 +429,26 @@ namespace arena
         int line = 1;
 
         cgotoxy(1, line++, GOTO_STAT);
-        textcolor(WHITE);
+        textcolour(WHITE);
         center_print(crawl_view.hudsz.x, string("Crawl ") + Version::Long);
         line++;
 
         cgotoxy(1, line++, GOTO_STAT);
-        textcolor(YELLOW);
+        textcolour(YELLOW);
         center_print(crawl_view.hudsz.x, faction_a.desc,
                      total_trials ? team_a_wins : -1);
         cgotoxy(1, line++, GOTO_STAT);
-        textcolor(LIGHTGREY);
+        textcolour(LIGHTGREY);
         center_print(crawl_view.hudsz.x, "vs");
         cgotoxy(1, line++, GOTO_STAT);
-        textcolor(YELLOW);
+        textcolour(YELLOW);
         center_print(crawl_view.hudsz.x, faction_b.desc,
                      total_trials ? trials_done - team_a_wins - ties : -1);
 
         if (total_trials > 1 && trials_done < total_trials)
         {
             cgotoxy(1, line++, GOTO_STAT);
-            textcolor(BROWN);
+            textcolour(BROWN);
             center_print(crawl_view.hudsz.x,
                          make_stringf("Round %d of %d",
                                       after_fight ? trials_done
@@ -472,7 +458,7 @@ namespace arena
         else
         {
             cgotoxy(1, line++, GOTO_STAT);
-            textcolor(BROWN);
+            textcolour(BROWN);
             clear_to_end_of_line();
         }
     }
@@ -588,7 +574,7 @@ namespace arena
 
     static void dump_messages()
     {
-        if (!Options.arena_dump_msgs || file == NULL)
+        if (!Options.arena_dump_msgs || file == nullptr)
             return;
 
         vector<string> messages;
@@ -654,16 +640,14 @@ namespace arena
             return;
         }
 
-        for (unsigned int i = 0; i < a_spawners.size(); i++)
+        for (int idx : a_spawners)
         {
-            int idx = a_spawners[i];
             menv[idx].speed_increment *= faction_b.active_members;
             menv[idx].speed_increment /= faction_a.active_members;
         }
 
-        for (unsigned int i = 0; i < b_spawners.size(); i++)
+        for (int idx : b_spawners)
         {
-            int idx = b_spawners[i];
             menv[idx].speed_increment *= faction_a.active_members;
             menv[idx].speed_increment /= faction_b.active_members;
         }
@@ -679,7 +663,7 @@ namespace arena
             if (mon->type == MONS_TEST_SPAWNER)
                 continue;
 
-            MiscastEffect(*mon, mon->mindex(), SPTYP_RANDOM,
+            MiscastEffect(*mon, *mon, WIZARD_MISCAST, SPTYP_RANDOM,
                           random_range(1, 3), "arena miscast", NH_NEVER);
         }
     }
@@ -707,7 +691,7 @@ namespace arena
             return;
         }
 
-        if (file != NULL)
+        if (file != nullptr)
             fflush(file);
 
         cursor_control coff(true);
@@ -896,7 +880,7 @@ namespace arena
             msg = "---------- " + msg + " ----------";
 
         if (was_tied)
-            mpr(msg.c_str());
+            mpr(msg);
         else
             mprf(msg.c_str(),
                  faction_a.won ? faction_a.desc.c_str()
@@ -931,11 +915,11 @@ namespace arena
             game_ended_with_error(error);
         }
 
-        if (file != NULL)
+        if (file != nullptr)
             end(0, false, "Results file already open");
         file = fopen("arena.result", "w");
 
-        if (file != NULL)
+        if (file != nullptr)
         {
             string spec = find_monster_spec();
             fprintf(file, "%s\n", spec.c_str());
@@ -958,15 +942,15 @@ namespace arena
 
     static void global_shutdown()
     {
-        if (file != NULL)
+        if (file != nullptr)
             fclose(file);
 
-        file = NULL;
+        file = nullptr;
     }
 
     static void write_results()
     {
-        if (file != NULL)
+        if (file != nullptr)
         {
             if (Options.arena_dump_msgs || Options.arena_list_eq)
                 fprintf(file, "========================================\n");
@@ -980,12 +964,12 @@ namespace arena
 
     static void write_error(const string &error)
     {
-        if (file != NULL)
+        if (file != nullptr)
         {
             fprintf(file, "err: %s\n", error.c_str());
             fclose(file);
         }
-        file = NULL;
+        file = nullptr;
     }
 
     static void simulate()
@@ -1063,7 +1047,7 @@ bool arena_veto_random_monster(monster_type type)
         return true;
     if (!arena::allow_immobile && mons_class_is_stationary(type))
         return true;
-    if (!arena::allow_zero_xp && mons_class_flag(type, M_NO_EXP_GAIN))
+    if (!arena::allow_zero_xp && !mons_class_gives_xp(type))
         return true;
     if (!(mons_char(type) & !127) && arena::banned_glyphs[mons_char(type)])
         return true;
@@ -1145,12 +1129,6 @@ void arena_placed_monster(monster* mons)
             item_def &item(mitm[it]);
             item.flags |= ISFLAG_IDENT_MASK;
 
-            // Don't leak info on wands or potions.
-            if (item.base_type == OBJ_WANDS
-                || item.base_type == OBJ_POTIONS)
-            {
-                item.colour = random_colour();
-            }
             // Set the "drop" time here in case the monster drops the
             // item without dying, like being polymorphed.
             arena::item_drop_times[it] = arena::turns;
@@ -1158,7 +1136,7 @@ void arena_placed_monster(monster* mons)
     }
 
     if (arena::name_monsters && !mons->is_named())
-        mons->mname = make_name(random_int(), false);
+        mons->mname = make_name();
 
     if (summoned)
     {
@@ -1198,7 +1176,7 @@ void arena_split_monster(monster* split_from, monster* split_to)
 }
 
 void arena_monster_died(monster* mons, killer_type killer,
-                        int killer_index, bool silent, int corpse)
+                        int killer_index, bool silent, const item_def* corpse)
 {
     if (mons_is_tentacle_or_tentacle_segment(mons->type))
         ; // part of a monster, or a spell
@@ -1217,7 +1195,7 @@ void arena_monster_died(monster* mons, killer_type killer,
     {
         arena::faction_b.won = true;
     }
-    // Everyone is dead.  Is it a tie, or something else?
+    // Everyone is dead. Is it a tie, or something else?
     else if (arena::faction_a.active_members <= 0
              && arena::faction_b.active_members <= 0)
     {
@@ -1225,7 +1203,7 @@ void arena_monster_died(monster* mons, killer_type killer,
             mpr("Last arena monster was dismissed.");
         // If all monsters are dead, and the last one to die is a giant
         // spore or ball lightning, then that monster's faction is the
-        // winner, since self-destruction is their purpose.  But if a
+        // winner, since self-destruction is their purpose. But if a
         // trap causes the spore to explode, and that kills everything,
         // it's a tie, since it counts as the trap killing everyone.
         else if (mons_self_destructs(mons) && MON_KILL(killer))
@@ -1247,7 +1225,7 @@ void arena_monster_died(monster* mons, killer_type killer,
              && killer == KILL_MISC
              && killer_index == NON_MONSTER))
     {
-        arena::faction *fac = NULL;
+        arena::faction *fac = nullptr;
         if (mons->attitude == ATT_FRIENDLY)
             fac = &arena::faction_a;
         else if (mons->attitude == ATT_HOSTILE)
@@ -1266,7 +1244,7 @@ void arena_monster_died(monster* mons, killer_type killer,
                 && (fac->members.get_monster(member_idx).type
                     == MONS_SLIME_CREATURE))
             {
-                for (unsigned int i = 1; i < mons->number; i++)
+                for (int i = 1; i < mons->blob_size; i++)
                 {
                     fac->respawn_list.push_back(member_idx);
                     fac->respawn_pos.push_back(mons->pos());
@@ -1277,8 +1255,8 @@ void arena_monster_died(monster* mons, killer_type killer,
         }
     }
 
-    if (corpse != -1 && corpse != NON_ITEM)
-        arena::item_drop_times[corpse] = arena::turns;
+    if (corpse)
+        arena::item_drop_times[corpse->index()] = arena::turns;
 
     // Won't be dropping any items.
     if (mons->flags & MF_HARD_RESET)
@@ -1312,7 +1290,7 @@ static bool _sort_by_age(int a, int b)
 }
 
 // Culls the items which have been on the floor the longest, culling the
-// newest items last.  Items which a monster dropped voluntarily or
+// newest items last. Items which a monster dropped voluntarily or
 // because of being polymorphed, rather than because of dying, are
 // culled earlier than they should be, but it's not like we have to be
 // fair to the arena monsters.
@@ -1342,9 +1320,8 @@ int arena_cull_items()
 
     vector<int> ammo;
 
-    for (unsigned int i = 0, end = items.size(); i < end; i++)
+    for (int idx : items)
     {
-        const int      idx = items[i];
         const item_def &item(mitm[idx]);
 
         // If the drop time is 0 then this is probably thrown ammo.
@@ -1381,9 +1358,9 @@ int arena_cull_items()
 #ifdef DEBUG_DIAGNOSTICS
     const int count1 = cull_count;
 #endif
-    for (unsigned int i = 0; i < ammo.size(); i++)
+    for (int idx : ammo)
     {
-        DESTROY_ITEM(ammo[i]);
+        DESTROY_ITEM(idx);
         if (cull_count >= cull_target)
             break;
     }
