@@ -43,8 +43,6 @@ public:
     bool    stab_attempt;
     int     stab_bonus;
 
-    bool    apply_bleeding;
-
     // Fetched/Calculated from the attacker, stored to save execution time
     int             ev_margin;
 
@@ -133,11 +131,11 @@ protected:
     int calc_stat_to_hit_base();
     int calc_stat_to_dam_base();
     virtual int calc_mon_to_hit_base() = 0;
-    virtual int apply_damage_modifiers(int damage, int damage_max,
-                                       bool &half_ac) = 0;
+    virtual int apply_damage_modifiers(int damage, int damage_max) = 0;
     virtual int calc_damage();
     int test_hit(int to_hit, int ev, bool randomise_ev);
-    int apply_defender_ac(int damage, int damage_max = 0, bool half_ac = false);
+    int apply_defender_ac(int damage, int damage_max = 0) const;
+    bool attack_warded_off();
     // Determine if we're blocking (partially or entirely)
     virtual bool attack_shield_blocked(bool verbose);
     virtual bool attack_ignores_shield(bool verbose) = 0;
