@@ -16,38 +16,34 @@
 #include "env.h"
 #include "godabil.h"
 #include "godcompanions.h"
-#include "goditem.h"
 #include "libutil.h"
 #include "message.h"
 #include "mon-behv.h"
 #include "mon-death.h"
-#include "mon-util.h"
-#include "monster.h"
-#include "player.h"
-#include "random.h"
+#include "mon-tentacle.h"
 #include "religion.h"
 #include "state.h"
 #include "travel.h"
-#include "transform.h"
 
 // Called whenever an already existing monster changes its attitude, possibly
 // temporarily.
 void mons_att_changed(monster* mon)
 {
     const mon_attitude_type att = mon->temp_attitude();
+    const monster_type mc = mons_base_type(mon);
 
-    if (mons_is_tentacle_head(mons_base_type(mon))
-        || mon->type == MONS_ELDRITCH_TENTACLE)
+    if (mons_is_tentacle_head(mc)
+        || mons_is_solo_tentacle(mc))
     {
         for (monster_iterator mi; mi; ++mi)
             if (mi->is_child_tentacle_of(mon))
             {
                 mi->attitude = att;
-                if (mon->type != MONS_ELDRITCH_TENTACLE)
+                if (!mons_is_solo_tentacle(mc))
                 {
                     for (monster_iterator connect; connect; ++connect)
                     {
-                        if (connect->is_child_tentacle_of(mi->as_monster()))
+                        if (connect->is_child_tentacle_of(*mi))
                             connect->attitude = att;
                     }
                 }
@@ -72,7 +68,7 @@ static void _fedhas_neutralise_plant(monster* plant);
 
 void beogh_follower_convert(monster* mons, bool orc_hit)
 {
-    if (!player_genus(GENPC_ORCISH) || crawl_state.game_is_arena())
+    if (!species_is_orcish(you.species) || crawl_state.game_is_arena())
         return;
 
     // For followers of Beogh, decide whether orcs will join you.
@@ -90,7 +86,7 @@ void beogh_follower_convert(monster* mons, bool orc_hit)
 
         const int hd = mons->get_experience_level();
 
-        if (you.piety >= piety_breakpoint(2) && !player_under_penance()
+        if (in_good_standing(GOD_BEOGH, 2)
             && random2(you.piety / 15) + random2(4 + you.experience_level / 3)
                  > random2(hd) + hd + random2(5))
         {
@@ -119,11 +115,10 @@ void slime_convert(monster* mons)
 
 void fedhas_neutralise(monster* mons)
 {
-    if (you_worship(GOD_FEDHAS)
+    if (in_good_standing(GOD_FEDHAS)
         && mons->attitude == ATT_HOSTILE
         && fedhas_neutralises(mons)
-        && !testbits(mons->flags, MF_ATT_CHANGE_ATTEMPT)
-        && !player_under_penance())
+        && !testbits(mons->flags, MF_ATT_CHANGE_ATTEMPT))
     {
         _fedhas_neutralise_plant(mons);
         mons->flags |= MF_ATT_CHANGE_ATTEMPT;
@@ -166,7 +161,7 @@ bool yred_slaves_abandon_you()
     for (radius_iterator ri(you.pos(), LOS_DEFAULT); ri; ++ri)
     {
         monster* mons = monster_at(*ri);
-        if (mons == NULL)
+        if (mons == nullptr)
             continue;
 
         if (is_yred_undead_slave(mons))
@@ -217,7 +212,7 @@ bool beogh_followers_abandon_you()
     for (radius_iterator ri(you.pos(), LOS_DEFAULT); ri; ++ri)
     {
         monster* mons = monster_at(*ri);
-        if (mons == NULL)
+        if (mons == nullptr)
             continue;
 
         // Note that orc high priests' summons are gifts of Beogh,
@@ -246,7 +241,7 @@ bool beogh_followers_abandon_you()
                 // For now CREATED_FRIENDLY stays.
                 mons_att_changed(mons);
 
-                if (you.can_see(mons))
+                if (you.can_see(*mons))
                     num_reconvert++; // Only visible ones.
 
                 reconvert = true;
@@ -298,9 +293,10 @@ static void _print_converted_orc_speech(const string key,
 void beogh_convert_orc(monster* orc, bool emergency,
                        bool converted_by_follower)
 {
+    ASSERT(orc); // XXX: change to monster &orc
     ASSERT(mons_genus(orc->type) == MONS_ORC);
 
-    if (you.can_see(orc)) // show reaction
+    if (you.can_see(*orc)) // show reaction
     {
         if (emergency || !orc->alive())
         {
@@ -370,11 +366,12 @@ static void _fedhas_neutralise_plant(monster* plant)
 
 static void _jiyva_convert_slime(monster* slime)
 {
+    ASSERT(slime); // XXX: change to monster &slime
     ASSERT(mons_is_slime(slime));
 
     behaviour_event(slime, ME_ALERT);
 
-    if (you.can_see(slime))
+    if (you.can_see(*slime))
     {
         if (mons_genus(slime->type) == MONS_GIANT_EYEBALL)
         {
@@ -410,75 +407,74 @@ void gozag_set_bribe(monster* traitor)
 {
     // Try to bribe the monster.
     const int bribability = gozag_type_bribable(traitor->type);
+
     if (bribability > 0)
     {
-        const branch_type br = gozag_bribable_branch(traitor->type);
-        if (!player_in_branch(br))
-            return;
-
         const monster* leader =
             traitor->props.exists("band_leader")
             ? monster_by_mid(traitor->props["band_leader"].get_int())
-            : NULL;
-
-        int cost = max(1, exper_value(traitor) / 20);
+            : nullptr;
 
         if (leader)
         {
-            if (leader->has_ench(ENCH_PERMA_BRIBED)
-                || leader ->props.exists(GOZAG_PERMABRIBE_KEY))
+            if (leader->has_ench(ENCH_FRIENDLY_BRIBED)
+                || leader->props.exists(FRIENDLY_BRIBE_KEY))
             {
-                gozag_deduct_bribe(br, 2*cost);
-                traitor->props[GOZAG_PERMABRIBE_KEY].get_bool() = true;
+                traitor->props[FRIENDLY_BRIBE_KEY].get_bool() = true;
             }
-            else if (leader->has_ench(ENCH_BRIBED)
-                     || leader->props.exists(GOZAG_BRIBE_KEY))
+            else if (leader->has_ench(ENCH_NEUTRAL_BRIBED)
+                     || leader->props.exists(NEUTRAL_BRIBE_KEY))
             {
-                gozag_deduct_bribe(br, cost);
-                // Don't continue if we exhausted our funds.
-                if (branch_bribe[br] > 0)
-                    traitor->props[GOZAG_BRIBE_KEY].get_bool() = true;
+                traitor->props[NEUTRAL_BRIBE_KEY].get_bool() = true;
             }
         }
         else if (x_chance_in_y(bribability, GOZAG_MAX_BRIBABILITY))
         {
-            // Sometimes get permanent followers at twice the cost.
-            if (branch_bribe[br] > 2*cost && one_chance_in(3))
-            {
-                gozag_deduct_bribe(br, 2*cost);
-                traitor->props[GOZAG_PERMABRIBE_KEY].get_bool() = true;
-            }
+            // Sometimes get permanent followers
+            if (one_chance_in(3))
+                traitor->props[FRIENDLY_BRIBE_KEY].get_bool() = true;
             else
-            {
-                gozag_deduct_bribe(br, cost);
-                // Don't continue if we exhausted our funds.
-                if (branch_bribe[br] > 0)
-                    traitor->props[GOZAG_BRIBE_KEY].get_bool() = true;
-            }
+                traitor->props[NEUTRAL_BRIBE_KEY].get_bool() = true;
         }
     }
 }
 
 void gozag_check_bribe(monster* traitor)
 {
+    branch_type branch = gozag_fixup_branch(you.where_are_you);
+
+    if (branch_bribe[branch] == 0)
+        return; // Do nothing if branch isn't currently bribed.
+
+    const int base_cost = max(1, exper_value(traitor) / 20);
+
+    int cost = 0;
+
     string msg;
-    if (traitor->props.exists(GOZAG_PERMABRIBE_KEY))
+
+    if (traitor->props.exists(FRIENDLY_BRIBE_KEY))
     {
-        traitor->props.erase(GOZAG_PERMABRIBE_KEY);
-        traitor->add_ench(ENCH_PERMA_BRIBED);
+        traitor->props.erase(FRIENDLY_BRIBE_KEY);
+        traitor->add_ench(mon_enchant(ENCH_FRIENDLY_BRIBED, 0, 0,
+                                      INFINITE_DURATION));
         msg = getSpeakString(traitor->name(DESC_DBNAME, true)
                              + " Gozag permabribe");
         if (msg.empty())
             msg = getSpeakString("Gozag permabribe");
+
+        // Actual allies deduct more gold.
+        cost = 2 * base_cost;
     }
-    else if (traitor->props.exists(GOZAG_BRIBE_KEY))
+    else if (traitor->props.exists(NEUTRAL_BRIBE_KEY))
     {
-        traitor->props.erase(GOZAG_BRIBE_KEY);
-        traitor->add_ench(ENCH_BRIBED);
+        traitor->props.erase(NEUTRAL_BRIBE_KEY);
+        traitor->add_ench(ENCH_NEUTRAL_BRIBED);
         msg = getSpeakString(traitor->name(DESC_DBNAME, true)
                              + " Gozag bribe");
         if (msg.empty())
             msg = getSpeakString("Gozag bribe");
+
+        cost = base_cost;
     }
 
     if (!msg.empty())
@@ -487,36 +483,31 @@ void gozag_check_bribe(monster* traitor)
         msg = do_mon_str_replacements(msg, traitor);
         strip_channel_prefix(msg, channel);
         mprf(channel, "%s", msg.c_str());
+        // !msg.empty means a monster was bribed.
+        gozag_deduct_bribe(branch, cost);
     }
 }
 
 void gozag_break_bribe(monster* victim)
 {
-    if (!victim->has_ench(ENCH_BRIBED)
-        && !victim->has_ench(ENCH_PERMA_BRIBED)
-        && !victim->props.exists(GOZAG_BRIBE_KEY)
-        && !victim->props.exists(GOZAG_PERMABRIBE_KEY))
+    ASSERT(victim); // XXX: change to monster &victim
+
+    if (!victim->has_ench(ENCH_NEUTRAL_BRIBED)
+        && !victim->has_ench(ENCH_FRIENDLY_BRIBED)
+        && !victim->props.exists(NEUTRAL_BRIBE_KEY)
+        && !victim->props.exists(FRIENDLY_BRIBE_KEY))
     {
         return;
     }
 
-    const branch_type br = gozag_bribable_branch(victim->type);
-    ASSERT(br != NUM_BRANCHES);
-
-    // Deduct a perma-bribe increment.
-    gozag_deduct_bribe(gozag_bribable_branch(victim->type),
-                       max(1, exper_value(victim) / 10));
-
     // Un-bribe the victim.
-    victim->props[GOZAG_BRIBE_BROKEN_KEY].get_bool() = true;
-    victim->del_ench(ENCH_BRIBED);
-    victim->del_ench(ENCH_PERMA_BRIBED);
-    victim->props.erase(GOZAG_BRIBE_BROKEN_KEY);
-    victim->props.erase(GOZAG_BRIBE_KEY);
-    victim->props.erase(GOZAG_PERMABRIBE_KEY);
+    victim->del_ench(ENCH_NEUTRAL_BRIBED);
+    victim->del_ench(ENCH_FRIENDLY_BRIBED);
+    victim->props.erase(NEUTRAL_BRIBE_KEY);
+    victim->props.erase(FRIENDLY_BRIBE_KEY);
 
     // Make other nearby bribed monsters un-bribed, too.
     for (monster_iterator mi; mi; ++mi)
-        if (mi->can_see(victim))
+        if (mi->can_see(*victim))
             gozag_break_bribe(*mi);
 }
