@@ -1,11 +1,13 @@
 #include "AppHdr.h"
 
 #include "branch.h"
-#include "externs.h"
-#include "player.h"
-#include "travel.h"
 #include "branch-data.h"
 
+#include "itemname.h"
+#include "player.h"
+#include "stringutil.h"
+#include "travel.h"
+
 FixedVector<level_id, NUM_BRANCHES> brentry;
 FixedVector<int, NUM_BRANCHES> brdepth;
 FixedVector<int, NUM_BRANCHES> branch_bribe;
@@ -26,20 +28,21 @@ const Branch* branch_iterator::operator*() const
     static const branch_type branch_order[] = {
         BRANCH_DUNGEON,
         BRANCH_TEMPLE,
-        BRANCH_ORC,
-        BRANCH_ELF,
-#if TAG_MAJOR_VERSION == 34
-        BRANCH_DWARF,
-#endif
         BRANCH_LAIR,
         BRANCH_SWAMP,
         BRANCH_SHOALS,
         BRANCH_SNAKE,
         BRANCH_SPIDER,
         BRANCH_SLIME,
+        BRANCH_ORC,
+        BRANCH_ELF,
+#if TAG_MAJOR_VERSION == 34
+        BRANCH_DWARF,
+#endif
         BRANCH_VAULTS,
 #if TAG_MAJOR_VERSION == 34
         BRANCH_BLADE,
+        BRANCH_FOREST,
 #endif
         BRANCH_CRYPT,
         BRANCH_TOMB,
@@ -50,9 +53,6 @@ const Branch* branch_iterator::operator*() const
         BRANCH_COCYTUS,
         BRANCH_TARTARUS,
         BRANCH_ZOT,
-#if TAG_MAJOR_VERSION == 34
-        BRANCH_FOREST,
-#endif
         BRANCH_ABYSS,
         BRANCH_PANDEMONIUM,
         BRANCH_ZIGGURAT,
@@ -140,7 +140,7 @@ bool is_connected_branch(level_id place)
     return is_connected_branch(place.branch);
 }
 
-branch_type str_to_branch(const string &branch, branch_type err)
+branch_type branch_by_abbrevname(const string &branch, branch_type err)
 {
     for (branch_iterator it; it; ++it)
         if (it->abbrevname && it->abbrevname == branch)
@@ -149,6 +149,15 @@ branch_type str_to_branch(const string &branch, branch_type err)
     return err;
 }
 
+branch_type branch_by_shortname(const string &branch)
+{
+    for (branch_iterator it; it; ++it)
+        if (it->shortname && it->shortname == branch)
+            return it->id;
+
+    return NUM_BRANCHES;
+}
+
 int current_level_ambient_noise()
 {
     return branches[you.where_are_you].ambient_noise;
@@ -179,3 +188,51 @@ branch_type parent_branch(branch_type branch)
     // If it's not in the game, use the default parent.
     return branches[branch].parent_branch;
 }
+
+int runes_for_branch(branch_type branch)
+{
+    switch (branch)
+    {
+    case BRANCH_VAULTS:   return VAULTS_ENTRY_RUNES;
+    case BRANCH_ZIGGURAT: return ZIG_ENTRY_RUNES;
+    case BRANCH_ZOT:      return ZOT_ENTRY_RUNES;
+    default:              return 0;
+    }
+}
+
+/**
+ * Write a description of the rune(s), if any, this branch contains.
+ *
+ * @param br             the branch in question
+ * @param remaining_only whether to only mention a rune if the player
+ *                       hasn't picked it up yet.
+ * @returns a string mentioning all applicable runes.
+ */
+string branch_rune_desc(branch_type br, bool remaining_only)
+{
+    string desc;
+    vector<string> rune_names;
+
+    for (rune_type rune : branches[br].runes)
+        if (!(remaining_only && you.runes[rune]))
+            rune_names.push_back(rune_type_name(rune));
+
+    if (!rune_names.empty())
+    {
+        desc = make_stringf("This branch contains the %s rune%s of Zot.",
+                            comma_separated_line(begin(rune_names),
+                                                 end(rune_names)).c_str(),
+                            rune_names.size() > 1 ? "s" : "");
+    }
+
+    return desc;
+}
+
+branch_type rune_location(rune_type rune)
+{
+    for (const auto& br : branches)
+        if (find(br.runes.begin(), br.runes.end(), rune) != br.runes.end())
+            return br.id;
+
+    return NUM_BRANCHES;
+}
