@@ -7,14 +7,19 @@
 #define BRANCH_H
 
 #include "enum.h"
+#include "itemprop-enum.h"
 
 enum branch_flag_type
 {
     BFLAG_NONE = 0,
 
+    BFLAG_NO_MAP          = (1 << 0), // Can't be magic mapped or remembered.
     BFLAG_ISLANDED        = (1 << 1), // May have isolated zones with no stairs.
     BFLAG_NO_XLEV_TRAVEL  = (1 << 2), // Can't cross-level travel to or from it.
     BFLAG_NO_ITEMS        = (1 << 3), // Branch gets no random items.
+    BFLAG_DANGEROUS_END   = (1 << 4), // bottom level is more dangerous than normal
+    BFLAG_SPOTTY          = (1 << 5), // Connect vaults with more open paths, not hallways.
+    BFLAG_NO_SHAFTS       = (1 << 6), // Don't generate random shafts.
 };
 
 struct Branch
@@ -29,22 +34,22 @@ struct Branch
     int absdepth;               // base item generation/etc depth
 
     uint32_t branch_flags;
-    uint32_t default_level_flags;
+
     dungeon_feature_type entry_stairs;
     dungeon_feature_type exit_stairs;
     const char* shortname;      // "Slime Pits"
     const char* longname;       // "The Pits of Slime"
     const char* abbrevname;     // "Slime"
     const char* entry_message;
-    bool has_uniques;
     colour_t floor_colour;          // Zot needs special handling.
     colour_t rock_colour;
     int travel_shortcut;         // Which key to press for travel.
-    bool dangerous_bottom_level; // bottom level is more dangerous than normal
+    vector<rune_type> runes;      // Contained rune(s) (if any).
     int ambient_noise;           // affects noise loudness and player stealth
 };
 
-class branch_iterator {
+class branch_iterator
+{
 public:
     branch_iterator();
 
@@ -74,13 +79,17 @@ bool is_connected_branch(branch_type branch);
 bool is_connected_branch(level_id place);
 level_id current_level_parent();
 
-branch_type str_to_branch(const string &branch, branch_type err = NUM_BRANCHES);
+branch_type branch_by_abbrevname(const string &branch, branch_type err = NUM_BRANCHES);
+branch_type branch_by_shortname(const string &branch);
 
-bool branch_has_monsters(branch_type branch);
 int current_level_ambient_noise();
 
 branch_type get_branch_at(const coord_def& pos);
 bool branch_is_unfinished(branch_type branch);
 
 branch_type parent_branch(branch_type branch);
+int runes_for_branch(branch_type branch);
+
+string branch_rune_desc(branch_type br, bool remaining_only);
+branch_type rune_location(rune_type rune);
 #endif
