@@ -6,8 +6,8 @@
 #ifndef CHARDUMP_H
 #define CHARDUMP_H
 
-#include <string>
 #include <cstdio>
+#include <string>
 
 enum item_origin_dump_selector
 {
@@ -26,11 +26,11 @@ enum item_origin_dump_selector
 class scorefile_entry;
 string morgue_directory();
 bool dump_char(const string &fname, bool quiet = false, bool full_id = false,
-               const scorefile_entry *se = NULL);
+               const scorefile_entry *se = nullptr);
 void dump_map(const char* fname, bool debug = false, bool dist = false);
 void dump_map(FILE *fp, bool debug = false, bool dist = false);
 void display_notes();
-string munge_description(string inStr);
+string chardump_desc(const item_def& item);
 const char *hunger_level();
 
 #ifdef DGL_WHEREIS
