@@ -6,17 +6,21 @@
 #include "AppHdr.h"
 
 #include "cio.h"
-#include "externs.h"
+
+#include <queue>
+
 #include "libutil.h"
 #include "macro.h"
 #include "message.h"
 #include "options.h"
+#include "output.h"
 #include "state.h"
 #include "stringutil.h"
 #include "unicode.h"
 #include "viewgeom.h"
-
-#include <queue>
+#if defined(USE_TILE_LOCAL) && defined(TOUCH_UI)
+#include "windowmanager.h"
+#endif
 
 static keycode_type _numpad2vi(keycode_type key)
 {
@@ -137,6 +141,42 @@ void nowrap_eol_cprintf(const char *s, ...)
     cprintf("%s", chop_string(buf, max(wrapcol + 1 - wherex(), 0), false).c_str());
 }
 
+static void wrapcprint_skipping(int skiplines, int wrapcol, const string &buf)
+{
+    ASSERT(skiplines >= 0);
+
+    const GotoRegion region = get_cursor_region();
+    const int max_y = cgetsize(region).y;
+
+    size_t linestart = 0;
+    size_t len = buf.length();
+
+    while (linestart < len)
+    {
+        const coord_def pos = cgetpos(region);
+
+        const int avail = wrapcol - pos.x + 1;
+        if (avail > 0)
+        {
+            const string line = chop_string(buf.c_str() + linestart, avail, false);
+            linestart += line.length();
+            if (skiplines == 0)
+                cprintf("%s", line.c_str());
+        }
+
+        // No room for more lines, quit now.
+        if (pos.y >= max_y)
+            break;
+        if (linestart < len)
+        {
+            // Only advance the cursor line if we printed something.
+            cgotoxy(1, pos.y + (skiplines ? 0 : 1), region);
+        }
+        if (skiplines)
+            --skiplines;
+    }
+}
+
 // cprintf that knows how to wrap down lines
 static void wrapcprintf(int wrapcol, const char *s, ...)
 {
@@ -144,23 +184,7 @@ static void wrapcprintf(int wrapcol, const char *s, ...)
     va_start(args, s);
     string buf = vmake_stringf(s, args);
     va_end(args);
-
-    const GotoRegion region = get_cursor_region();
-    const int max_y = cgetsize(region).y;
-    while (!buf.empty())
-    {
-        const coord_def pos = cgetpos(region);
-
-        const int avail = wrapcol - pos.x + 1;
-        if (avail > 0)
-            cprintf("%s", wordwrap_line(buf, avail).c_str());
-
-        // No room for more lines, quit now.
-        if (pos.y >= max_y)
-            break;
-        if (!buf.empty())
-            cgotoxy(1, pos.y + 1, region);
-    }
+    wrapcprint_skipping(0, wrapcol, buf);
 }
 
 int cancellable_get_line(char *buf, int len, input_history *mh,
@@ -209,7 +233,7 @@ void input_history::new_input(const string &s)
 const string *input_history::prev()
 {
     if (history.empty())
-        return NULL;
+        return nullptr;
 
     if (pos == history.begin())
         pos = history.end();
@@ -220,7 +244,7 @@ const string *input_history::prev()
 const string *input_history::next()
 {
     if (history.empty())
-        return NULL;
+        return nullptr;
 
     if (pos == history.end() || ++pos == history.end())
         pos = history.begin();
@@ -243,9 +267,9 @@ void input_history::clear()
 // line_reader
 
 line_reader::line_reader(char *buf, size_t sz, int wrap)
-    : buffer(buf), bufsz(sz), history(NULL), region(GOTO_CRT),
-      start(coord_def(0,0)), keyfn(NULL), wrapcol(wrap),
-      cur(NULL), length(0), pos(-1)
+    : buffer(buf), bufsz(sz), history(nullptr), region(GOTO_CRT),
+      start(coord_def(0,0)), keyfn(nullptr), wrapcol(wrap),
+      cur(nullptr), length(0), pos(-1)
 {
 }
 
@@ -279,6 +303,27 @@ void line_reader::cursorto(int ncx)
 {
     int x = (start.x + ncx - 1) % wrapcol + 1;
     int y = start.y + (start.x + ncx - 1) / wrapcol;
+
+    if (y < 1)
+    {
+        // Cursor would go above the visible area. "Scroll" backwards so that
+        // it goes on the top line, and redraw.
+        start.y += 1 - y;
+        const int skip = max(0, 1 - start.y);
+        // If the beginning of the buffer becomes visible, paint over
+        // the place where the prompt used to be. FIXME: It would be nice
+        // to remember and display the visible part of the prompt.
+        if (skip == 0)
+        {
+            cgotoxy(1, start.y + skip, region);
+            cprintf("%*s", start.x - 1, "");
+        }
+        else
+            cgotoxy(start.x, start.y + skip, region);
+        wrapcprint_skipping(skip, wrapcol, buffer);
+        y = 1;
+    }
+
     int diff = y - cgetsize(region).y;
     if (diff > 0)
     {
@@ -287,8 +332,11 @@ void line_reader::cursorto(int ncx)
         cscroll(diff, region);
         start.y -= diff;
         y -= diff;
-        cgotoxy(start.x, start.y, region);
-        wrapcprintf(wrapcol, "%s", buffer);
+
+        const int skip = max(0, 1 - start.y);
+
+        cgotoxy(start.x, start.y + skip, region);
+        wrapcprint_skipping(skip, wrapcol, buffer);
     }
     cgotoxy(x, y, region);
 }
@@ -319,6 +367,11 @@ int line_reader::read_line(bool clear_previous)
     if (clear_previous)
         *buffer = 0;
 
+#if defined(USE_TILE_LOCAL) && defined(TOUCH_UI)
+    if (wm)
+        wm->show_keyboard();
+#endif
+
 #ifdef USE_TILE_WEB
     if (!tiles.is_in_crt_menu())
     {
@@ -328,7 +381,10 @@ int line_reader::read_line(bool clear_previous)
         if (!tag.empty())
             tiles.json_write_string("tag", tag);
         if (history)
-            tiles.json_write_string("historyId", make_stringf("%p", history));
+        {
+            tiles.json_write_string("historyId",
+                                    make_stringf("%p", (void *)history));
+        }
         tiles.json_write_string("prefill", buffer);
         tiles.json_write_int("maxlen", (int) bufsz - 1);
         tiles.json_write_int("size", (int) min(bufsz - 1, strlen(buffer) + 15));
@@ -432,7 +488,9 @@ void line_reader::backspace()
 
     cursorto(pos);
     buffer[length] = 0;
-    wrapcprintf(wrapcol, "%s ", cur);
+    // Two spaces in case we deleted a double-width character, or
+    // caused a double-width character to move back a line.
+    wrapcprintf(wrapcol, "%s  ", cur);
     cursorto(pos);
 }
 
@@ -514,13 +572,16 @@ int line_reader::process_key(int ch)
     CASE_ESCAPE
         return CK_ESCAPE;
     case CK_UP:
+    case CONTROL('P'):
     case CK_DOWN:
+    case CONTROL('N'):
     {
         if (!history)
             break;
 
-        const string *text = (ch == CK_UP) ? history->prev()
-                                           : history->next();
+        const string *text = (ch == CK_UP || ch == CONTROL('P'))
+                             ? history->prev()
+                             : history->next();
 
         if (text)
         {
@@ -561,18 +622,21 @@ int line_reader::process_key(int ch)
         break;
     }
     case CK_DELETE:
+    case CONTROL('D'):
         if (*cur)
         {
-            char *np = next_glyph(cur);
+            const char *np = next_glyph(cur);
             ASSERT(np);
-            char *c = cur;
-            while (*np)
-                *c++ = *np++;
-            length = np - buffer;
+            const size_t del_bytes = np - cur;
+            const size_t follow_bytes = (buffer + length) - np;
+            // Copy the NUL too.
+            memmove(cur, np, follow_bytes + 1);
+            length -= del_bytes;
 
             cursorto(pos);
-            buffer[length-1] = 0;
-            wrapcprintf(wrapcol, "%s ", cur);
+            // Two spaces in case we deleted a double-width character, or
+            // caused a double-width character to move back a line.
+            wrapcprintf(wrapcol, "%s  ", cur);
             cursorto(pos);
         }
         break;
@@ -590,6 +654,7 @@ int line_reader::process_key(int ch)
         break;
 
     case CK_LEFT:
+    case CONTROL('B'):
         if (char *np = prev_glyph(cur, buffer))
         {
             cur = np;
@@ -598,6 +663,7 @@ int line_reader::process_key(int ch)
         }
         break;
     case CK_RIGHT:
+    case CONTROL('F'):
         if (char *np = next_glyph(cur))
         {
             cur = np;
@@ -620,6 +686,9 @@ int line_reader::process_key(int ch)
     case CK_MOUSE_CLICK:
         // FIXME: ought to move cursor to click location, if it's within the input
         return -1;
+    case CK_REDRAW:
+        redraw_screen();
+        return -1;
     default:
         if (wcwidth(ch) >= 0 && length + wclen(ch) < static_cast<int>(bufsz))
         {
