@@ -7,11 +7,11 @@ extern "C" {
 #include <lualib.h>
 }
 
-#include <cstdio>
 #include <cstdarg>
-#include <string>
+#include <cstdio>
 #include <map>
 #include <set>
+#include <string>
 
 #ifndef CLUA_MAX_MEMORY_USE
 #define CLUA_MAX_MEMORY_USE (6 * 1024)
@@ -62,7 +62,7 @@ public:
 
     const lua_datum &operator = (const lua_datum &other);
 
-    void shutdown(CLua &lua);
+    void shutdown(CLua &lua) override;
 
     ~lua_datum();
 
@@ -102,6 +102,8 @@ public:
     }
 
     void save(writer &outf);
+    void save_persist();
+    void load_persist();
     void gc();
 
     void setglobal(const char *name);
@@ -126,6 +128,7 @@ public:
     void pushglobal(const string &name);
 
     maybe_bool callmbooleanfn(const char *fn, const char *params, ...);
+    maybe_bool callmaybefn(const char *fn, const char *params, ...);
     bool callbooleanfn(bool defval, const char *fn, const char *params, ...);
     bool callfn(const char *fn, int nargs, int nret = 1);
     bool callfn(const char *fn, const char *params, ...);
@@ -174,7 +177,7 @@ private:
 
 private:
     void init_lua();
-    void set_error(int err, lua_State *ls = NULL);
+    void set_error(int err, lua_State *ls = nullptr);
     void load_cmacro();
     void load_chooks();
     void init_throttle();
@@ -186,12 +189,14 @@ private:
     bool proc_returns(const char *par) const;
 
     bool calltopfn(lua_State *ls, const char *format, va_list args,
-                   int retc = -1, va_list *fnr = NULL);
+                   int retc = -1, va_list *fnr = nullptr);
     maybe_bool callmbooleanfn(const char *fn, const char *params,
                               va_list args);
+    maybe_bool callmaybefn(const char *fn, const char *params,
+                           va_list args);
 
     int push_args(lua_State *ls, const char *format, va_list args,
-                    va_list *cpto = NULL);
+                    va_list *cpto = nullptr);
     int return_count(lua_State *ls, const char *format);
 
     struct CLuaSave
@@ -211,9 +216,9 @@ public:
     lua_text_pattern(const string &pattern);
     ~lua_text_pattern();
 
-    bool valid() const;
-    bool matches(const string &s) const;
-    const string &tostring() const { return pattern; }
+    bool valid() const override;
+    bool matches(const string &s) const override;
+    const string &tostring() const override { return pattern; }
 
     static bool is_lua_pattern(const string &s);
 
@@ -238,8 +243,6 @@ private:
 #endif
 extern CLua clua;
 
-void lua_set_exclusive_item(const item_def *item = NULL);
-
 string quote_lua_string(const string &s);
 
 #endif
