@@ -8,8 +8,9 @@
 {'-', CMD_EDIT_PLAYER_TILE},
 #endif
 // movement keys
-{CK_CLEAR, CMD_MOVE_NOWHERE},
-{'.', CMD_MOVE_NOWHERE},
+{'s', CMD_WAIT},
+{CK_CLEAR, CMD_WAIT},
+{'.', CMD_WAIT},
 {CK_END, CMD_MOVE_DOWN_LEFT},
 {CK_LEFT, CMD_MOVE_LEFT},
 {CK_DOWN, CMD_MOVE_DOWN},
@@ -48,7 +49,6 @@
 {'p', CMD_PRAY},
 {'q', CMD_QUAFF},
 {'r', CMD_READ},
-{'s', CMD_WAIT},
 {'t', CMD_SHOUT},
 {'V', CMD_EVOKE},
 {'v', CMD_EVOKE_WIELDED},
@@ -111,11 +111,11 @@
 {'~', CMD_MACRO_ADD},
 #ifdef WIZARD
 {'&', CMD_WIZARD},
+{'+', CMD_EXPLORE_MODE},
 #endif
 {'"', CMD_LIST_JEWELLERY},
 {'{', CMD_INSCRIBE_ITEM},
 {'[', CMD_LIST_ARMOUR},
-{']', CMD_LIST_EQUIPMENT},
 {'(', CMD_CYCLE_QUIVER_BACKWARD},
 {')', CMD_CYCLE_QUIVER_FORWARD},
 {CONTROL('V'), CMD_TOGGLE_VIEWPORT_MONSTER_HP},
@@ -129,22 +129,22 @@
 {CK_SHIFT_CLEAR, CMD_REST},
 {CK_CTRL_CLEAR, CMD_REST},
 {'5', CMD_REST},
-{CONTROL('B'), CMD_OPEN_DOOR_DOWN_LEFT},
-{CONTROL('H'), CMD_OPEN_DOOR_LEFT},
-{CONTROL('J'), CMD_OPEN_DOOR_DOWN},
-{CONTROL('K'), CMD_OPEN_DOOR_UP},
-{CONTROL('L'), CMD_OPEN_DOOR_RIGHT},
-{CONTROL('N'), CMD_OPEN_DOOR_DOWN_RIGHT},
-{CONTROL('U'), CMD_OPEN_DOOR_UP_RIGHT},
-{CONTROL('Y'), CMD_OPEN_DOOR_UP_LEFT},
-{CK_CTRL_END, CMD_OPEN_DOOR_DOWN_LEFT},
-{CK_CTRL_LEFT, CMD_OPEN_DOOR_LEFT},
-{CK_CTRL_DOWN, CMD_OPEN_DOOR_DOWN},
-{CK_CTRL_UP, CMD_OPEN_DOOR_UP},
-{CK_CTRL_RIGHT, CMD_OPEN_DOOR_RIGHT},
-{CK_CTRL_PGDN, CMD_OPEN_DOOR_DOWN_RIGHT},
-{CK_CTRL_PGUP, CMD_OPEN_DOOR_UP_RIGHT},
-{CK_CTRL_HOME, CMD_OPEN_DOOR_UP_LEFT},
+{CONTROL('B'), CMD_ATTACK_DOWN_LEFT},
+{CONTROL('H'), CMD_ATTACK_LEFT},
+{CONTROL('J'), CMD_ATTACK_DOWN},
+{CONTROL('K'), CMD_ATTACK_UP},
+{CONTROL('L'), CMD_ATTACK_RIGHT},
+{CONTROL('N'), CMD_ATTACK_DOWN_RIGHT},
+{CONTROL('U'), CMD_ATTACK_UP_RIGHT},
+{CONTROL('Y'), CMD_ATTACK_UP_LEFT},
+{CK_CTRL_END, CMD_ATTACK_DOWN_LEFT},
+{CK_CTRL_LEFT, CMD_ATTACK_LEFT},
+{CK_CTRL_DOWN, CMD_ATTACK_DOWN},
+{CK_CTRL_UP, CMD_ATTACK_UP},
+{CK_CTRL_RIGHT, CMD_ATTACK_RIGHT},
+{CK_CTRL_PGDN, CMD_ATTACK_DOWN_RIGHT},
+{CK_CTRL_PGUP, CMD_ATTACK_UP_RIGHT},
+{CK_CTRL_HOME, CMD_ATTACK_UP_LEFT},
 {CONTROL('A'), CMD_TOGGLE_AUTOPICKUP},
 {CONTROL('C'), CMD_CLEAR_MAP},
 {CONTROL('E'), CMD_TOGGLE_TRAVEL_SPEED},
@@ -176,6 +176,7 @@
 {'"', CMD_TARGET_WIZARD_DEBUG_PORTAL},
 {CONTROL('B'), CMD_TARGET_WIZARD_BANISH_MONSTER},
 {CONTROL('K'), CMD_TARGET_WIZARD_KILL_MONSTER},
+{'(', CMD_TARGET_WIZARD_CREATE_MIMIC},
 #endif
 {'v', CMD_TARGET_DESCRIBE},
 {'?', CMD_TARGET_HELP},
@@ -332,6 +333,7 @@
 {';', CMD_MAP_GOTO_TARGET},
 {'!', CMD_MAP_ANNOTATE_LEVEL},
 {'o', CMD_MAP_EXPLORE},
+{'v', CMD_MAP_DESCRIBE},
 {ESCAPE, CMD_MAP_EXIT_MAP},
 #ifdef CLUA_BINDINGS
 {CONTROL('I'), CMD_AUTOFIGHT},
