@@ -2,24 +2,20 @@
 
 #include "colour.h"
 
+#include <cmath>
+#include <memory>
+#include <utility>
+
 #include "areas.h"
 #include "branch.h"
 #include "cloud.h"
 #include "dgn-height.h"
-#include "env.h"
-#include "libutil.h"
-#include "mon-info.h"
-#include "mon-info.h"
-#include "mon-util.h"
 #include "options.h"
-#include "player.h"
-#include "random.h"
 #include "stringutil.h"
+#include "libutil.h" // map_find
 
-#include <utility>
-#include <math.h>
-
-static element_colour_calc* element_colours[NUM_COLOURS] = {};
+static FixedVector<unique_ptr<element_colour_calc>, NUM_COLOURS> element_colours;
+// Values point into element_colours.
 static map<string, element_colour_calc*> element_colours_str;
 
 typedef vector< pair<int, int> > random_colour_map;
@@ -37,7 +33,7 @@ struct random_element_colour_calc : public element_colour_calc
         {};
 
     virtual int get(const coord_def& loc = coord_def(),
-                    bool non_random = false);
+                    bool non_random = false) override;
 
 protected:
     random_colour_map rand_vals;
@@ -66,9 +62,14 @@ int random_element_colour_calc::get(const coord_def& loc, bool non_random)
     return (*real_calc)(rand(non_random), loc, rand_vals);
 }
 
-colour_t random_colour()
+colour_t random_colour(bool ui_rand)
 {
-    return 1 + random2(15);
+    return 1 + (ui_rand ? ui_random : random2)(15);
+}
+
+static bool _ui_coinflip()
+{
+    return static_cast<bool>(ui_random(2));
 }
 
 colour_t random_uncommon_colour()
@@ -76,7 +77,9 @@ colour_t random_uncommon_colour()
     colour_t result;
 
     do
+    {
         result = random_colour();
+    }
     while (result == LIGHTCYAN || result == CYAN || result == BROWN);
 
     return result;
@@ -121,13 +124,9 @@ static int _randomized_element_colour(int rand, const coord_def&,
                                       random_colour_map rand_vals)
 {
     int accum = 0;
-    for (random_colour_map::const_iterator it = rand_vals.begin();
-         it != rand_vals.end();
-         ++it)
-    {
-        if ((accum += it->first) > rand)
-            return it->second;
-    }
+    for (const auto &entry : rand_vals)
+        if ((accum += entry.first) > rand)
+            return entry.second;
 
     return BLACK;
 }
@@ -180,13 +179,13 @@ static int _etc_elemental(int, const coord_def& loc)
         case 0:
             return element_colour(ETC_EARTH, false, loc);
         case 1:
-            return element_colour(coinflip() ? ETC_AIR : ETC_ELECTRICITY,
+            return element_colour(_ui_coinflip() ? ETC_AIR : ETC_ELECTRICITY,
                                   false, loc);
         case 2:
             // Not ETC_FIRE, which is Makhleb; instead do magma-y colours.
-            if (coinflip())
+            if (_ui_coinflip())
                 return RED;
-            return coinflip() ? BROWN : LIGHTRED;
+            return _ui_coinflip() ? BROWN : LIGHTRED;
         case 3:
             return element_colour(ETC_ICE, false, loc);
     }
@@ -263,14 +262,14 @@ static int _etc_tree(int, const coord_def& loc)
     h+=h<<10; h^=h>>6;
     h+=h<<3; h^=h>>11; h+=h<<15;
     return (h>>30) ? GREEN :
-        you.where_are_you == BRANCH_SWAMP ? BROWN : LIGHTGREEN; // Swamp trees are mangroves.
+        player_in_branch(BRANCH_SWAMP) ? BROWN : LIGHTGREEN; // Swamp trees are mangroves.
 }
 
 bool get_tornado_phase(const coord_def& loc)
 {
     coord_def center = get_cloud_originator(loc);
     if (center.origin())
-        return coinflip(); // source died/went away
+        return _ui_coinflip(); // source died/went away
     else
     {
         int x = loc.x - center.x;
@@ -353,7 +352,7 @@ colour_t rune_colour(int type)
 
 static int _etc_random(int, const coord_def&)
 {
-    return random_colour();
+    return random_colour(true);
 }
 
 static element_colour_calc *_create_random_element_colour_calc(element_type type,
@@ -372,7 +371,7 @@ static element_colour_calc *_create_random_element_colour_calc(element_type type
 
         int colour = va_arg(ap, int);
 
-        rand_vals.push_back(make_pair(prob, colour));
+        rand_vals.emplace_back(prob, colour);
     }
 
     va_end(ap);
@@ -387,15 +386,15 @@ void add_element_colour(element_colour_calc *colour)
     COMPILE_CHECK(NUM_COLOURS <= 128);
     if (colour->type >= ETC_FIRST_LUA)
     {
-        ASSERT(element_colours[colour->type] == element_colours_str[colour->name]);
-        delete element_colours[colour->type];
+        ASSERT(element_colours[colour->type].get()
+               == element_colours_str[colour->name]);
     }
     else
     {
         ASSERT(!element_colours[colour->type]);
         ASSERT(!element_colours_str.count(colour->name));
     }
-    element_colours[colour->type] = colour;
+    element_colours[colour->type].reset(colour);
     element_colours_str[colour->name] = colour;
 }
 
@@ -650,23 +649,17 @@ void init_element_colours()
     add_element_colour(new element_colour_calc(
                             ETC_ELEMENTAL, "elemental", _etc_elemental
                        ));
+    add_element_colour(_create_random_element_colour_calc(
+                            ETC_INCARNADINE, "incarnadine",
+                            60,  MAGENTA,
+                            60,  RED,
+                        0));
     // redefined by Lua later
     add_element_colour(new element_colour_calc(
                             ETC_DISCO, "disco", _etc_random
                        ));
 }
 
-void clear_colours_on_exit()
-{
-    for (int i = 0; i < NUM_COLOURS; i++)
-    {
-        delete element_colours[i];
-        element_colours[i] = 0;
-    }
-
-    element_colours_str.clear();
-}
-
 int element_colour(int element, bool no_random, const coord_def& loc)
 {
     // pass regular colours through for safety.
@@ -762,8 +755,9 @@ const string colour_to_str(colour_t colour)
         return cols[colour];
 }
 
-// Returns -1 if unmatched else returns 0-15.
-int str_to_colour(const string &str, int default_colour, bool accept_number)
+// Returns default_colour (default -1) if unmatched else returns 0-15.
+int str_to_colour(const string &str, int default_colour, bool accept_number,
+                  bool accept_elemental)
 {
     int ret;
 
@@ -782,15 +776,13 @@ int str_to_colour(const string &str, int default_colour, bool accept_number)
             ret = 8;
     }
 
-    if (ret == 16)
+    if (ret == 16 && accept_elemental)
     {
         // Maybe we have an element colour attribute.
-        map<string, element_colour_calc*>::const_iterator it
-            = element_colours_str.find(str);
-        if (it != element_colours_str.end())
+        if (element_colour_calc **calc = map_find(element_colours_str, str))
         {
-            ASSERT(it->second);
-            ret = it->second->type;
+            ASSERT(*calc);
+            ret = (*calc)->type;
         }
     }
 
@@ -798,7 +790,7 @@ int str_to_colour(const string &str, int default_colour, bool accept_number)
     {
         // Check if we have a direct colour index.
         const char *s = str.c_str();
-        char *es = NULL;
+        char *es = nullptr;
         const int ci = static_cast<int>(strtol(s, &es, 10));
         if (s != es && es && ci >= 0 && ci < 16)
             ret = ci;
@@ -828,7 +820,7 @@ static unsigned short _dos_reverse_brand(unsigned short colour)
         // evil, so we strip the background high-intensity bit. This, sadly,
         // limits us to 7 background colours.
 
-        // Strip off high-intensity bit.  Special case DARKGREY, since it's the
+        // Strip off high-intensity bit. Special case DARKGREY, since it's the
         // high-intensity counterpart of black, and we don't want black on
         // black.
         //
