@@ -17,6 +17,9 @@
 
 #include "ctest.h"
 
+#include <algorithm>
+#include <vector>
+
 #include "clua.h"
 #include "cluautil.h"
 #include "coordit.h"
@@ -24,6 +27,8 @@
 #include "end.h"
 #include "errors.h"
 #include "files.h"
+#include "itemname.h"
+#include "jobs.h"
 #include "libutil.h"
 #include "maps.h"
 #include "message.h"
@@ -32,10 +37,6 @@
 #include "ng-init.h"
 #include "state.h"
 #include "stringutil.h"
-#include "zotdef.h"
-
-#include <algorithm>
-#include <vector>
 
 static const string test_dir = "test";
 static const string script_dir = "scripts";
@@ -84,7 +85,7 @@ static const struct luaL_reg crawl_test_lib[] =
     { "begin_test", crawl_begin_test },
     { "test_success", crawl_test_success },
     { "script_args", crawl_script_args },
-    { NULL, NULL }
+    { nullptr, nullptr }
 };
 
 static void _init_test_bindings()
@@ -107,10 +108,8 @@ static bool _is_test_selected(const string &testname)
 
     if (crawl_state.tests_selected.empty() && !starts_with(testname, "big/"))
         return true;
-    for (int i = 0, size = crawl_state.tests_selected.size();
-         i < size; ++i)
+    for (const string& phrase : crawl_state.tests_selected)
     {
-        const string &phrase(crawl_state.tests_selected[i]);
         if (testname == phrase || testname == phrase + ".lua")
             return true;
     }
@@ -132,7 +131,7 @@ static void run_test(const string &file)
     if (dlua.error.empty())
         ++nsuccess;
     else
-        failures.push_back(file_error(file, dlua.error));
+        failures.emplace_back(file, dlua.error);
 }
 
 static bool _has_test(const string& test)
@@ -158,7 +157,7 @@ static void _run_test(const string &name, void (*func)())
     }
     catch (const ext_fail_exception &E)
     {
-        failures.push_back(file_error(name, E.msg));
+        failures.emplace_back(name, E.msg);
     }
 }
 
@@ -177,42 +176,40 @@ void run_tests()
     _init_test_bindings();
 
     _run_test("makeitem", makeitem_tests);
-    _run_test("zotdef_wave", debug_waves);
     _run_test("mon-pick", debug_monpick);
     _run_test("mon-data", debug_mondata);
+    _run_test("mon-spell", debug_monspells);
     _run_test("coordit", coordit_tests);
+    _run_test("makename", make_name_tests);
+    _run_test("job-data", debug_jobdata);
 
-    // Get a list of Lua files in test. Order of execution of
-    // tests should be irrelevant.
+    // Get a list of Lua files in test.
     {
-        const vector<string> tests(
-            get_dir_files_recursive(crawl_state.script? script_dir : test_dir,
-                              ".lua"));
+        const string &dir = crawl_state.script ? script_dir : test_dir;
+        vector<string> tests = get_dir_files_recursive(dir, ".lua");
+
+        // Make the order consistent from one run to the next, for
+        // reproducibility.
+        sort(begin(tests), end(tests));
 
         for_each(tests.begin(), tests.end(), run_test);
 
         if (failures.empty() && !ntests && crawl_state.script)
         {
-            failures.push_back(
-                file_error(
-                    "Script setup",
-                    "No scripts found matching " +
-                    comma_separated_line(crawl_state.tests_selected.begin(),
-                                         crawl_state.tests_selected.end(),
-                                         ", ",
-                                         ", ")));
+            failures.emplace_back("Script setup",
+                    "No scripts found matching "
+                    + comma_separated_line(crawl_state.tests_selected.begin(),
+                                           crawl_state.tests_selected.end(),
+                                           ", ", ", "));
         }
     }
 
     if (crawl_state.test_list)
         end(0);
     cio_cleanup();
-    for (int i = 0, size = failures.size(); i < size; ++i)
-    {
-        const file_error &fe(failures[i]);
-        fprintf(stderr, "%s error: %s\n",
-                activity, fe.second.c_str());
-    }
+    for (const file_error &fe : failures)
+        fprintf(stderr, "%s error: %s\n", activity, fe.second.c_str());
+
     const int code = failures.empty() ? 0 : 1;
     end(code, false, "%d %ss, %d succeeded, %d failed",
         ntests, activity, nsuccess, (int)failures.size());
