@@ -12,19 +12,15 @@
 #include "coordit.h"
 #include "decks.h"
 #include "dungeon.h"
-#include "env.h"
 #include "items.h"
 #include "libutil.h"
 #include "mapmark.h"
+#include "message.h"
 #include "mon-behv.h"
 #include "mon-death.h"
 #include "mon-transit.h"
-#include "mon-util.h"
-#include "player.h"
 #include "religion.h"
-#include "show.h"
 #include "state.h"
-#include "travel.h"
 #include "view.h"
 
 static void _daction_hog_to_human(monster *mon, bool in_transit);
@@ -69,6 +65,8 @@ static const char *daction_names[] =
     "bribe timeout",
     "remove Gozag shops",
     "apply Gozag bribes",
+    "Makhleb's servants go hostile",
+    "make all monsters hate you",
 };
 #endif
 
@@ -102,6 +100,8 @@ bool mons_matches_daction(const monster* mon, daction_type act)
     // Not a stored counter:
     case DACT_ALLY_TROG:
         return mon->friendly() && mons_is_god_gift(mon, GOD_TROG);
+    case DACT_ALLY_MAKHLEB:
+        return mon->friendly() && mons_is_god_gift(mon, GOD_MAKHLEB);
     case DACT_HOLY_PETS_GO_NEUTRAL:
         return mon->friendly()
                && !mon->has_ench(ENCH_CHARM)
@@ -130,10 +130,9 @@ bool mons_matches_daction(const monster* mon, daction_type act)
                    || mon->props.exists(ORIG_MONSTER_KEY));
 
     case DACT_BRIBE_TIMEOUT:
-        return mon->has_ench(ENCH_BRIBED)
-               || mon->has_ench(ENCH_PERMA_BRIBED)
-               || mon->props.exists(GOZAG_BRIBE_KEY)
-               || mon->props.exists(GOZAG_PERMABRIBE_KEY);
+        return mon->has_ench(ENCH_NEUTRAL_BRIBED)
+               || mon->props.exists(NEUTRAL_BRIBE_KEY)
+               || mon->props.exists(FRIENDLY_BRIBE_KEY);
 
     case DACT_SET_BRIBES:
         return !testbits(mon->flags, MF_WAS_IN_VIEW);
@@ -143,15 +142,15 @@ bool mons_matches_daction(const monster* mon, daction_type act)
     }
 }
 
-void update_da_counters(LevelInfo *lev)
+void update_daction_counters(LevelInfo *lev)
 {
-    for (int act = 0; act < NUM_DA_COUNTERS; act++)
-        lev->da_counters[act] = 0;
+    for (int act = 0; act < NUM_DACTION_COUNTERS; act++)
+        lev->daction_counters[act] = 0;
 
     for (monster_iterator mi; mi; ++mi)
-        for (int act = 0; act < NUM_DA_COUNTERS; act++)
+        for (int act = 0; act < NUM_DACTION_COUNTERS; act++)
             if (mons_matches_daction(*mi, static_cast<daction_type>(act)))
-                lev->da_counters[act]++;
+                lev->daction_counters[act]++;
 }
 
 void add_daction(daction_type act)
@@ -165,8 +164,8 @@ void add_daction(daction_type act)
 
     // If we're removing a counted monster type, zero the counter even though
     // it hasn't been actually removed from the levels yet.
-    if (act < NUM_DA_COUNTERS)
-        travel_cache.clear_da_counter(act);
+    if (act < NUM_DACTION_COUNTERS)
+        travel_cache.clear_daction_counter(act);
 
     // Immediately apply it to the current level.
     catchup_dactions();
@@ -193,6 +192,7 @@ void apply_daction_to_mons(monster* mon, daction_type act, bool local,
         case DACT_ALLY_SLIME:
         case DACT_ALLY_PLANT:
         case DACT_ALLY_TROG:
+        case DACT_ALLY_MAKHLEB:
             dprf("going hostile: %s", mon->name(DESC_PLAIN, true).c_str());
             mon->attitude = ATT_HOSTILE;
             mon->del_ench(ENCH_CHARM, true);
@@ -206,9 +206,12 @@ void apply_daction_to_mons(monster* mon, daction_type act, bool local,
             if (act == DACT_ALLY_PLANT || act == DACT_ALLY_SLIME)
                 mon->flags &= ~MF_ATT_CHANGE_ATTEMPT;
 
-            // No global message for Trog.
-            if (act == DACT_ALLY_TROG && local)
+            // No global message for Trog, Makhleb, or Ru.
+            if (local && (act == DACT_ALLY_TROG
+                          || act == DACT_ALLY_MAKHLEB))
+            {
                 simple_monster_message(mon, " turns against you!");
+            }
             break;
 
         case DACT_OLD_ENSLAVED_SOULS_POOF:
@@ -223,8 +226,10 @@ void apply_daction_to_mons(monster* mon, daction_type act, bool local,
 
         case DACT_HOLY_PETS_GO_NEUTRAL:
         case DACT_PIKEL_SLAVES:
+        {
             // monster changes attitude
-            mon->attitude = ATT_GOOD_NEUTRAL;
+            bool hostile = player_mutation_level(MUT_NO_LOVE);
+            mon->attitude = hostile ? ATT_HOSTILE : ATT_GOOD_NEUTRAL;
             mons_att_changed(mon);
 
             if (act == DACT_PIKEL_SLAVES)
@@ -234,21 +239,25 @@ void apply_daction_to_mons(monster* mon, daction_type act, bool local,
                 mon->mname = "freed slave";
             }
             else if (local)
-                simple_monster_message(mon, " becomes indifferent.");
-            mon->behaviour = BEH_WANDER;
+                simple_monster_message(mon, hostile ? " turns on you!" : " becomes indifferent.");
+            mon->behaviour = hostile ? BEH_SEEK : BEH_WANDER;
             break;
-
+        }
         case DACT_KIRKE_HOGS:
             _daction_hog_to_human(mon, in_transit);
             break;
 
         case DACT_BRIBE_TIMEOUT:
-            mon->del_ench(ENCH_BRIBED);
-            mon->del_ench(ENCH_PERMA_BRIBED);
-            if (mon->props.exists(GOZAG_BRIBE_KEY))
-                mon->props.erase(GOZAG_BRIBE_KEY);
-            if (mon->props.exists(GOZAG_PERMABRIBE_KEY))
-                mon->props.erase(GOZAG_PERMABRIBE_KEY);
+            if (mon->del_ench(ENCH_NEUTRAL_BRIBED))
+            {
+                mon->attitude = ATT_NEUTRAL;
+                mon->flags   |= MF_WAS_NEUTRAL;
+                mons_att_changed(mon);
+            }
+            if (mon->props.exists(NEUTRAL_BRIBE_KEY))
+                mon->props.erase(NEUTRAL_BRIBE_KEY);
+            if (mon->props.exists(FRIENDLY_BRIBE_KEY))
+                mon->props.erase(FRIENDLY_BRIBE_KEY);
             break;
 
         case DACT_SET_BRIBES:
@@ -276,6 +285,7 @@ static void _apply_daction(daction_type act)
     case DACT_ALLY_SLIME:
     case DACT_ALLY_PLANT:
     case DACT_ALLY_TROG:
+    case DACT_ALLY_MAKHLEB:
     case DACT_OLD_ENSLAVED_SOULS_POOF:
     case DACT_SLIME_NEW_ATTEMPT:
     case DACT_HOLY_PETS_GO_NEUTRAL:
@@ -304,13 +314,9 @@ static void _apply_daction(daction_type act)
         }
         break;
     case DACT_ROT_CORPSES:
-        for (int i = 0; i < MAX_ITEMS; i++)
-            if (mitm[i].base_type == OBJ_CORPSES && mitm[i].sub_type == CORPSE_BODY)
-                mitm[i].special = 1; // thoroughly rotten
-        break;
-    case DACT_TOMB_CTELE:
-        if (player_in_branch(BRANCH_TOMB))
-            unset_level_flags(LFLAG_NO_TELE_CONTROL, you.depth != 3);
+        for (auto &item : mitm)
+            if (item.is_type(OBJ_CORPSES, CORPSE_BODY))
+                item.special = 1; // thoroughly rotten
         break;
     case DACT_GOLD_ON_TOP:
     {
@@ -362,8 +368,10 @@ static void _apply_daction(daction_type act)
 #if TAG_MAJOR_VERSION == 34
     case DACT_END_SPIRIT_HOWL:
     case DACT_HOLY_NEW_ATTEMPT:
+    case DACT_ALLY_SACRIFICE_LOVE:
+    case DACT_TOMB_CTELE:
 #endif
-    case NUM_DA_COUNTERS:
+    case NUM_DACTION_COUNTERS:
     case NUM_DACTIONS:
         ;
     }
@@ -375,16 +383,17 @@ void catchup_dactions()
         _apply_daction(you.dactions[env.dactions_done++]);
 }
 
-unsigned int query_da_counter(daction_type c)
+unsigned int query_daction_counter(daction_type c)
 {
-    return travel_cache.query_da_counter(c) + count_daction_in_transit(c);
+    return travel_cache.query_daction_counter(c) + count_daction_in_transit(c);
 }
 
 static void _daction_hog_to_human(monster *mon, bool in_transit)
 {
+    ASSERT(mon); // XXX: change to monster &mon
     // Hogs to humans
     monster orig;
-    const bool could_see = you.can_see(mon);
+    const bool could_see = you.can_see(*mon);
 
     // Was it a converted monster or original band member?
     if (mon->props.exists(ORIG_MONSTER_KEY))
@@ -409,7 +418,7 @@ static void _daction_hog_to_human(monster *mon, bool in_transit)
     const float hp
         = (float) mon->hit_points / (float) mon->max_hit_points;
     // Preserve some flags.
-    const uint64_t preserve_flags =
+    const monster_flags_t preserve_flags =
         mon->flags & ~(MF_JUST_SUMMONED | MF_WAS_IN_VIEW);
     // Preserve enchantments.
     mon_enchant_list enchantments = mon->enchantments;
@@ -433,20 +442,16 @@ static void _daction_hog_to_human(monster *mon, bool in_transit)
     mon->hit_points   = max(1, (int) (mon->max_hit_points * hp));
     mon->flags        = mon->flags | preserve_flags;
 
-    const bool can_see = you.can_see(mon);
+    const bool can_see = you.can_see(*mon);
 
     // A monster changing factions while in the arena messes up
     // arena book-keeping.
     if (!crawl_state.game_is_arena())
     {
         // * A monster's attitude shouldn't downgrade from friendly
-        //   or good-neutral because you helped it.  It'd suck to
+        //   or good-neutral because you helped it. It'd suck to
         //   lose a permanent ally that way.
-        //
-        // * A monster has to be smart enough to realize that you
-        //   helped it.
-        if (mon->attitude == ATT_HOSTILE
-            && mons_intel(mon) >= I_NORMAL)
+        if (mon->attitude == ATT_HOSTILE)
         {
             mon->attitude = ATT_GOOD_NEUTRAL;
             mon->flags   |= MF_WAS_NEUTRAL;
