@@ -33,8 +33,7 @@
 #include "travel.h"
 #include "version.h"
 #include "view.h"
-#include "zotdef.h"
-
+#
 #if defined(TARGET_OS_WINDOWS) || defined(TARGET_COMPILER_MINGW)
 #define NOCOMM            /* Comm driver APIs and definitions */
 #define NOLOGERROR        /* LogError() and related definitions */
@@ -143,10 +142,9 @@ static void _dump_player(FILE *file)
     fprintf(file, "Player:\n");
     fprintf(file, "{{{{{{{{{{{\n");
 
-    fprintf(file, "Name:       [%s]\n", you.your_name.c_str());
-    fprintf(file, "Species:    %s\n", species_name(you.species).c_str());
-    fprintf(file, "Job:        %s\n\n", get_job_name(you.char_class));
-    fprintf(file, "class_name: %s\n\n", you.class_name.c_str());
+    fprintf(file, "Name:    [%s]\n", you.your_name.c_str());
+    fprintf(file, "Species: %s\n", species_name(you.species).c_str());
+    fprintf(file, "Job:     %s\n\n", get_job_name(you.char_class));
 
     fprintf(file, "HP: %d/%d; mods: %d/%d\n", you.hp, you.hp_max,
             you.hp_max_adj_temp, you.hp_max_adj_perm);
@@ -430,7 +428,7 @@ static void _debug_marker_scan()
 
         if (type < MAT_FEATURE || type >= NUM_MAP_MARKER_TYPES)
         {
-            mprf(MSGCH_ERROR, "Makrer #%d at (%d, %d) has invalid type %d",
+            mprf(MSGCH_ERROR, "Marker #%d at (%d, %d) has invalid type %d",
                  i, marker->pos.x, marker->pos.y, (int) type);
         }
 
@@ -571,11 +569,8 @@ static void _dump_ver_stuff(FILE* file)
 static void _dump_command_line(FILE *file)
 {
     fprintf(file, "Command line:");
-    for (int i = 0, size = crawl_state.command_line_arguments.size();
-         i < size; ++i)
-    {
-        fprintf(file, " %s", crawl_state.command_line_arguments[i].c_str());
-    }
+    for (const string& str : crawl_state.command_line_arguments)
+        fprintf(file, " %s", str.c_str());
     if (crawl_state.command_line_arguments.empty())
         fprintf(file, " (unknown)");
     fprintf(file, "\n\n");
@@ -590,7 +585,7 @@ static void _dump_options(FILE *file)
     fprintf(file, "\n\n");
 }
 
-// Defined in stuff.cc.  Not a part of crawl_state, since that's a
+// Defined in stuff.cc. Not a part of crawl_state, since that's a
 // global C++ instance which is free'd by exit() hooks when exit()
 // is called, and we don't want to reference free'd memory.
 extern bool CrawlIsExiting;
@@ -602,7 +597,7 @@ void do_crash_dump()
         // We crashed during exit() callbacks, so it's likely that
         // any global C++ instances we could reference would be
         // free'd and invalid, plus their content likely wouldn't help
-        // tracking it down anyway.  Thus, just do the bare bones
+        // tracking it down anyway. Thus, just do the bare bones
         // info to stderr and quit.
         fprintf(stderr, "Crashed while calling exit()!!!!\n");
 
@@ -706,10 +701,7 @@ void do_crash_dump()
     crawl_state.dump();
     _dump_player(file);
 
-    if (crawl_state.game_is_zotdef())
-        fprintf(file, "ZotDef wave data: %s\n", zotdef_debug_wave_desc().c_str());
-
-    // Next item and monster scans.  Any messages will be sent straight to
+    // Next item and monster scans. Any messages will be sent straight to
     // the file because of set_msg_dump_file()
 #ifdef DEBUG_ITEM_SCAN
     debug_item_scan();
