@@ -10,10 +10,8 @@
 #include "coord.h"
 #include "coordit.h"
 #include "dungeon.h"
-#include "libutil.h"
 #include "stringutil.h"
 #include "terrain.h"
-#include "traps.h"
 
 static bool _find_forbidden_in_area(dgn_region& area, unsigned int mask);
 static int _count_antifeature_in_box(int x0, int y0, int x1, int y1,
@@ -51,9 +49,6 @@ void dgn_build_basic_level()
 {
     int level_number = env.absdepth0;
 
-    env.level_build_method += " basic";
-    env.level_layout_types.insert("basic");
-
     int corrlength = 2 + random2(14);
     int no_corr = (one_chance_in(100) ? 500 + random2(500)
                                       : 30 + random2(200));
@@ -98,10 +93,10 @@ void dgn_build_basic_level()
         upstairs.push_back(begin);
     }
 
-    for (vector<coord_def>::iterator pathstart = upstairs.begin();
+    for (auto pathstart = upstairs.begin();
          pathstart != upstairs.end(); pathstart++)
     {
-        vector<coord_def>::iterator pathend = pathstart;
+        auto pathend = pathstart;
         pathend++;
         for (; pathend != upstairs.end(); pathend++)
             join_the_dots(*pathstart, *pathend, MMT_VAULT);
@@ -130,9 +125,6 @@ void dgn_build_basic_level()
 
 void dgn_build_bigger_room_level()
 {
-    env.level_build_method += " bigger_room";
-    env.level_layout_types.insert("open");
-
     for (rectangle_iterator ri(10); ri; ++ri)
         if (grd(*ri) == DNGN_ROCK_WALL
             && !map_masked(*ri, MMT_VAULT))
@@ -161,14 +153,13 @@ void dgn_build_bigger_room_level()
 // A more chaotic version of city level.
 void dgn_build_chaotic_city_level(dungeon_feature_type force_wall)
 {
-    env.level_build_method += make_stringf(" chaotic_city [%s]",
+    // TODO: Attach this information to the vault name string
+    //       instead of the build method string.
+    env.level_build_method += make_stringf(" [%s]",
         force_wall == NUM_FEATURES ? "any" : dungeon_feature_name(force_wall));
-    env.level_layout_types.insert("city");
 
     int number_boxes = 5000;
     dungeon_feature_type drawing = DNGN_ROCK_WALL;
-    uint8_t b1x, b1y, b2x, b2y;
-    int i;
 
     number_boxes = random_choose_weighted(32, 4000,
                                           24, 3000,
@@ -189,7 +180,8 @@ void dgn_build_chaotic_city_level(dungeon_feature_type force_wall)
                      DNGN_FLOOR, MMT_VAULT);
 
     // replace_area can also be used to fill in:
-    for (i = 0; i < number_boxes; i++)
+    uint8_t b1x, b1y, b2x, b2y;
+    for (int i = 0; i < number_boxes; i++)
     {
         int room_width = 3 + random2(7) + random2(5);
         int room_height = 3 + random2(7) + random2(5);
@@ -434,7 +426,9 @@ static void _builder_extras(int level_number)
 static bool _octa_room(dgn_region& region, int oblique_max,
                        dungeon_feature_type type_floor)
 {
-    env.level_build_method += make_stringf(" octa_room [%d %s]", oblique_max,
+    // TODO: Attach this information to the vault name string
+    //       instead of the build method string.
+    env.level_build_method += make_stringf(" octa_room [oblique %d, %s]", oblique_max,
                                            dungeon_feature_name(type_floor));
 
     int x,y;
@@ -851,13 +845,8 @@ static void _diamond_rooms(int level_number)
 
 static int _good_door_spot(int x, int y)
 {
-    if (!feat_is_solid(grd[x][y]) && grd[x][y] < DNGN_ENTER_PANDEMONIUM
-        || feat_is_closed_door(grd[x][y]))
-    {
-        return 1;
-    }
-
-    return 0;
+    return (!feat_is_solid(grd[x][y]) || feat_is_closed_door(grd[x][y]))
+            && !feat_is_critical(grd[x][y]);
 }
 
 // Returns TRUE if a room was made successfully.
@@ -896,7 +885,7 @@ static bool _make_room(int sx,int sy,int ex,int ey,int max_doors, int doorlevel)
     for (rx = sx; rx <= ex; rx++)
         for (ry = sy; ry <= ey; ry++)
         {
-            if (grd[rx][ry] <= DNGN_FLOOR)
+            if (!feat_has_dry_floor(grd[rx][ry]))
                 grd[rx][ry] = DNGN_FLOOR;
         }
 
@@ -1050,6 +1039,8 @@ static void _many_pools(dungeon_feature_type pool_type)
     const int num_pools = 20 + random2avg(9, 2);
     int pools = 0;
 
+        // TODO: Attach this information to the vault name string
+        //       instead of the build method string.
     env.level_build_method += make_stringf(" many_pools [%s %d]",
         dungeon_feature_name(pool_type), num_pools);
 
@@ -1079,9 +1070,9 @@ static bool _may_overwrite_pos(coord_def c)
     const dungeon_feature_type grid = grd(c);
 
     // Don't overwrite any stairs or branch entrances.
-    if (grid >= DNGN_ENTER_SHOP && grid <= DNGN_TELEPORTER
-        || grid >= DNGN_ENTER_FIRST_PORTAL && grid <= DNGN_EXIT_LAST_PORTAL
-        || grid == DNGN_EXIT_HELL)
+    if (feat_is_stair(grid)
+        || grid == DNGN_ENTER_SHOP
+        || grid == DNGN_TELEPORTER)
     {
         return false;
     }
@@ -1093,20 +1084,19 @@ static bool _may_overwrite_pos(coord_def c)
 
 static void _build_river(dungeon_feature_type river_type) //mv
 {
-    int i,j;
-    int y, width;
-
     if (player_in_branch(BRANCH_CRYPT) || player_in_branch(BRANCH_TOMB))
         return;
 
+    // TODO: Attach this information to the vault name string
+    //       instead of the build method string.
     env.level_build_method += make_stringf(" river [%s]",
                                            dungeon_feature_name(river_type));
 
     // Made rivers less wide... min width five rivers were too annoying. -- bwr
-    width = 3 + random2(4);
-    y = 10 - width + random2avg(GYM-10, 3);
+    int width = 3 + random2(4);
+    int y = 10 - width + random2avg(GYM-10, 3);
 
-    for (i = 5; i < (GXM - 5); i++)
+    for (int i = 5; i < (GXM - 5); i++)
     {
         if (one_chance_in(3))   y++;
         if (one_chance_in(3))   y--;
@@ -1116,7 +1106,7 @@ static void _build_river(dungeon_feature_type river_type) //mv
         if (width < 2) width = 2;
         if (width > 6) width = 6;
 
-        for (j = y; j < y+width ; j++)
+        for (int j = y; j < y+width ; j++)
             if (j >= 5 && j <= GYM - 5)
             {
                 // Note that vaults might have been created in this area!
@@ -1147,6 +1137,8 @@ static void _build_lake(dungeon_feature_type lake_type) //mv
     if (player_in_branch(BRANCH_CRYPT) || player_in_branch(BRANCH_TOMB))
         return;
 
+        // TODO: Attach this information to the vault name string
+        //       instead of the build method string.
     env.level_build_method += make_stringf(" lake [%s]",
                                            dungeon_feature_name(lake_type));
 
