@@ -4,18 +4,15 @@
  **/
 #include "AppHdr.h"
 
+#include "dgn-proclayouts.h"
+
 #include <cmath>
 
-#include "dgn-proclayouts.h"
 #include "coord.h"
 #include "coordit.h"
 #include "files.h"
-#include "hash.h"
 #include "perlin.h"
 #include "terrain.h"
-#include "worley.h"
-
-#include "mpr.h"
 
 static dungeon_feature_type _pick_pseudorandom_wall(uint64_t val)
 {
@@ -28,7 +25,7 @@ static dungeon_feature_type _pick_pseudorandom_wall(uint64_t val)
         DNGN_ROCK_WALL,
         DNGN_ROCK_WALL,
         DNGN_ROCK_WALL,
-        DNGN_GREEN_CRYSTAL_WALL,
+        DNGN_CRYSTAL_WALL,
         DNGN_METAL_WALL
     };
     return features[val%9];
@@ -208,7 +205,7 @@ dungeon_feature_type sanitize_feature(dungeon_feature_type feature, bool strict)
             feature = DNGN_CLEAR_ROCK_WALL;
             break;
         case DNGN_SLIMY_WALL:
-            feature = DNGN_GREEN_CRYSTAL_WALL;
+            feature = DNGN_CRYSTAL_WALL;
             break;
         case DNGN_UNSEEN:
             feature = DNGN_FLOOR;
@@ -340,23 +337,6 @@ ClampLayout::operator()(const coord_def &p, const uint32_t offset) const
     return ProceduralSample(p, sample.feat(), cp);
 }
 
-ProceduralSample
-CityLayout::operator()(const coord_def &p, const uint32_t offset) const
-{
-    const double scale = 9.0;
-    double x = p.x / scale;
-    double y = p.y / scale;
-    double z = 12.0;
-    worley::noise_datum n = worley::noise(x, y, z + 0xF00);
-    int size = 3 + (n.id[0] % 5) & (n.id[0] / 5) % 5;
-    int x_off = ceil(n.pos[0][0] * scale);
-    int y_off = ceil(n.pos[0][1] * scale);
-    int dist = coord_def(x_off, y_off).rdist();
-    if (dist == size)
-        return ProceduralSample(p, DNGN_ROCK_WALL, offset + 4096);
-    return ProceduralSample(p, DNGN_FLOOR, offset + 4096);
-}
-
 ProceduralSample
 UnderworldLayout::operator()(const coord_def &p, const uint32_t offset) const
 {
@@ -494,7 +474,7 @@ UnderworldLayout::operator()(const coord_def &p, const uint32_t offset) const
         dungeon_feature_type city_wall = DNGN_ROCK_WALL;
         if (rich > 0.5) city_wall = DNGN_STONE_WALL;
         else if (rich > 0.75) city_wall = DNGN_METAL_WALL;
-        else if (rich > 0.9) city_wall = DNGN_GREEN_CRYSTAL_WALL;
+        else if (rich > 0.9) city_wall = DNGN_CRYSTAL_WALL;
 
         // Doors and windows
         if (jitter>0.5 && jitter<0.6) city_wall = DNGN_CLOSED_DOOR;
@@ -591,7 +571,7 @@ double WorleyFunction::operator()(const coord_def &p, const uint32_t offset) con
 
 double WorleyFunction::operator()(double x, double y, double z) const
 {
-    worley::noise_datum d = this->datum(x,y,z);
+    worley::noise_datum d = datum(x,y,z);
     return d.distance[1]-d.distance[0];
 }
 
