@@ -18,9 +18,9 @@ bool is_vowel(const ucs_t chr);
 string pluralise(const string &name,
                  const char * const stock_plural_quals[]
                      = standard_plural_qualifiers,
-                 const char *no_of[] = nullptr);
+                 const char * const no_of[] = nullptr);
+string pluralise_monster(const string &name);
 string apostrophise(const string &name);
-string apostrophise_fixup(const string &msg);
 string conjugate_verb(const string &verb, bool plural);
 const char *decline_pronoun(gender_type gender, pronoun_type variant);
 
