@@ -1,10 +1,12 @@
 #ifndef ENV_H
 #define ENV_H
 
+#include <set>
+#include <memory> // unique_ptr
+
 #include "map_knowledge.h"
 #include "monster.h"
 #include "trap_def.h"
-#include <set>
 
 typedef FixedArray<short, GXM, GYM> grid_heightmap;
 typedef uint32_t terrain_property_t;
@@ -12,7 +14,7 @@ typedef uint32_t terrain_property_t;
 typedef set<string> string_set;
 
 struct vault_placement;
-typedef vector<vault_placement*> vault_placement_refv;
+typedef vector<unique_ptr<vault_placement>> vault_placement_refv;
 
 typedef FixedArray< map_cell, GXM, GYM > MapKnowledge;
 
@@ -86,7 +88,7 @@ struct crawl_environment
 
     // Rate at which random monsters spawn, with lower numbers making
     // them spawn more often (5 or less causes one to spawn about every
-    // 5 turns).  Set to 0 to stop random generation.
+    // 5 turns). Set to 0 to stop random generation.
     int spawn_random_rate;
 
     // Time when level was saved (hence we write out you.elapsed_time
@@ -99,10 +101,6 @@ struct crawl_environment
     // Number of turns the player has spent on this level.
     int turns_on_level;
 
-    // Flags for things like preventing teleport control; see
-    // level_flag_type in enum.h
-    uint32_t level_flags;
-
     // Index into the delayed actions array.
     unsigned int dactions_done;
 
