@@ -3,12 +3,14 @@
  * @brief Handling of error conditions that are not program bugs.
 **/
 
-#include <stdarg.h>
-#include <errno.h>
-#include <string.h>
-
 #include "AppHdr.h"
+
 #include "errors.h"
+
+#include <cerrno>
+#include <cstdarg>
+#include <cstring>
+
 #include "stringutil.h"
 
 NORETURN void fail(const char *msg, ...)
