@@ -14,16 +14,17 @@
 #include "coord.h"
 #include "coordit.h"
 #include "dgn-overview.h"
+#include "english.h"
+#include "env.h"
+#include "hints.h"
 #include "libutil.h"
 #include "map_knowledge.h"
 #include "mon-util.h"
 #include "options.h"
-#include "env.h"
 #include "stringutil.h"
 #include "tags.h"
 #include "terrain.h"
 #include "travel.h"
-#include "hints.h"
 #include "view.h"
 
 // defined in dgn-overview.cc
@@ -52,14 +53,16 @@ static bool _need_auto_exclude(const monster* mon, bool sleepy = false)
                                            && testbits(mon->flags, MF_SEEN));
     lowercase(name);
 
-    for (unsigned i = 0; i < Options.auto_exclude.size(); ++i)
-        if (Options.auto_exclude[i].matches(name)
+    for (const text_pattern &pat : Options.auto_exclude)
+    {
+        if (pat.matches(name)
             && _mon_needs_auto_exclude(mon, sleepy)
             && (mon->attitude == ATT_HOSTILE
                 || mon->type == MONS_HYPERACTIVE_BALLISTOMYCETE))
         {
             return true;
         }
+    }
 
     return false;
 }
@@ -90,8 +93,6 @@ void set_auto_exclude(const monster* mon)
         int rad = _get_full_exclusion_radius();
         if (mon->type == MONS_HYPERACTIVE_BALLISTOMYCETE)
             rad = 2;
-        else if (mons_is_mimic(mon->type))
-            rad = 1;
         set_exclude(mon->pos(), rad, true);
         // FIXME: If this happens for several monsters in the same turn
         //        (as is possible for some vaults), this could be really
@@ -133,7 +134,7 @@ class opacity_excl : public opacity_func
 public:
     CLONE(opacity_excl)
 
-    opacity_type operator()(const coord_def& p) const
+    opacity_type operator()(const coord_def& p) const override
     {
         map_cell& cell = env.map_knowledge(p);
         if (!cell.seen())
@@ -148,14 +149,10 @@ public:
 };
 static opacity_excl opc_excl;
 
-// Note: circle_def(r, C_ROUND) gives a circle with square radius r*r+1;
-// this doesn't work well for radius 0, but then we want to
-// skip LOS calculation in that case anyway since it doesn't
-// currently short-cut for small bounds. So radius 0, 1 are special-cased.
 travel_exclude::travel_exclude(const coord_def &p, int r,
                                bool autoexcl, string dsc, bool vaultexcl)
     : pos(p), radius(r),
-      los(los_def(p, opc_excl, circle_def(r, C_ROUND))),
+      los(los_def(p, opc_excl, circle_def(r, C_SQUARE))),
       uptodate(false), autoex(autoexcl), desc(dsc), vault(vaultexcl)
 {
     set_los();
@@ -164,7 +161,7 @@ travel_exclude::travel_exclude(const coord_def &p, int r,
 // For exclude_map[p] = foo;
 travel_exclude::travel_exclude()
     : pos(-1, -1), radius(-1),
-      los(coord_def(-1, -1), opc_excl, circle_def(-1, C_ROUND)),
+      los(coord_def(-1, -1), opc_excl, circle_def(-1, C_SQUARE)),
       uptodate(false), autoex(false), desc(""), vault(false)
 {
 }
@@ -177,7 +174,7 @@ void travel_exclude::set_los()
     if (radius > 1)
     {
         // Radius might have been changed, and this is cheap.
-        los.set_bounds(circle_def(radius, C_ROUND));
+        los.set_bounds(circle_def(radius, C_SQUARE));
         los.update();
     }
 }
@@ -218,7 +215,7 @@ void exclude_set::clear()
 
 void exclude_set::erase(const coord_def &p)
 {
-    exclude_set::iterator it = exclude_roots.find(p);
+    auto it = exclude_roots.find(p);
 
     if (it == exclude_roots.end())
         return;
@@ -256,7 +253,7 @@ void exclude_set::add_exclude_points(travel_exclude& ex)
     else
         ex.los.update();
 
-    for (radius_iterator ri(ex.pos, ex.radius, C_ROUND); ri; ++ri)
+    for (radius_iterator ri(ex.pos, ex.radius, C_SQUARE); ri; ++ri)
         if (ex.affects(*ri))
             exclude_points.insert(*ri);
 }
@@ -298,12 +295,7 @@ bool exclude_set::is_exclude_root(const coord_def &p) const
 
 travel_exclude* exclude_set::get_exclude_root(const coord_def &p)
 {
-    exclude_set::iterator it = exclude_roots.find(p);
-
-    if (it != exclude_roots.end())
-        return &it->second;
-
-    return NULL;
+    return map_find(exclude_roots, p);
 }
 
 size_t exclude_set::size() const
@@ -340,10 +332,9 @@ exclude_set::iterator exclude_set::end()
 
 static void _mark_excludes_non_updated(const coord_def &p)
 {
-    for (exclude_set::iterator it = curr_excludes.begin();
-         it != curr_excludes.end(); ++it)
+    for (auto &entry : curr_excludes)
     {
-        travel_exclude &ex = it->second;
+        travel_exclude &ex = entry.second;
         ex.uptodate = ex.uptodate && !ex.in_bounds(p);
     }
 }
@@ -364,8 +355,8 @@ void update_exclusion_los(vector<coord_def> changed)
     if (changed.empty())
         return;
 
-    for (unsigned int i = 0; i < changed.size(); ++i)
-        _mark_excludes_non_updated(changed[i]);
+    for (coord_def c : changed)
+        _mark_excludes_non_updated(c);
 
     curr_excludes.update_excluded_points(true);
 }
@@ -436,9 +427,8 @@ void deferred_exclude_update()
 {
     _exclude_update();
 #ifdef USE_TILE
-    exclude_set::iterator it;
-    for (it = curr_excludes.begin(); it != curr_excludes.end(); ++it)
-        _tile_exclude_gmap_update(it->second.pos);
+    for (const auto &entry : curr_excludes)
+        _tile_exclude_gmap_update(entry.second.pos);
 #endif
 }
 
@@ -446,7 +436,7 @@ void clear_excludes()
 {
 #ifdef USE_TILE
     // Tiles needs to update the minimap for each exclusion that is removed,
-    // but the exclusions need to be removed first.  Therefore, make a copy
+    // but the exclusions need to be removed first. Therefore, make a copy
     // of the current set of exclusions and iterate through the copy below.
     exclude_set excludes = curr_excludes;
 #endif
@@ -455,8 +445,8 @@ void clear_excludes()
     clear_level_exclusion_annotation();
 
 #ifdef USE_TILE
-    for (exclude_set::iterator it = excludes.begin(); it != excludes.end(); ++it)
-        _tile_exclude_gmap_update(it->second.pos);
+    for (const auto &entry : excludes)
+        _tile_exclude_gmap_update(entry.second.pos);
 #endif
 
     _exclude_update();
@@ -466,13 +456,12 @@ static void _exclude_gate(const coord_def &p, bool del = false)
 {
     set<coord_def> all_doors;
     find_connected_identical(p, all_doors);
-    for (set<coord_def>::const_iterator dc = all_doors.begin();
-         dc != all_doors.end(); ++dc)
+    for (const auto &dc : all_doors)
     {
         if (del)
-            del_exclude(*dc);
+            del_exclude(dc);
         else
-            set_exclude(*dc, 0);
+            set_exclude(dc, 0);
     }
 }
 
@@ -582,7 +571,7 @@ void maybe_remove_autoexclusion(const coord_def &p)
             return;
 
         const monster* m = monster_at(p);
-        if (!m || !you.can_see(m)
+        if (!m || !you.can_see(*m)
             || m->attitude != ATT_HOSTILE
                 && m->type != MONS_HYPERACTIVE_BALLISTOMYCETE
             || strcmp(mons_type_name(m->type, DESC_PLAIN).c_str(),
@@ -598,18 +587,17 @@ string exclude_set::get_exclusion_desc()
 {
     vector<string> desc;
     int count_other = 0;
-    for (exclmap::iterator it = exclude_roots.begin();
-         it != exclude_roots.end(); ++it)
+    for (auto &entry : exclude_roots)
     {
-        travel_exclude &ex = it->second;
+        travel_exclude &ex = entry.second;
 
         // Don't count cloud exclusions.
         if (strstr(ex.desc.c_str(), "cloud"))
             continue;
 
         // Don't duplicate if there's already an annotation from unique monsters.
-        set<pair<string, level_id> >::iterator ma
-            = auto_unique_annotations.find(pair<string, level_id>(ex.desc, level_id::current()));
+        auto ma = auto_unique_annotations.find(make_pair(ex.desc,
+                                                         level_id::current()));
         if (ma != auto_unique_annotations.end())
             continue;
 
@@ -642,9 +630,8 @@ string exclude_set::get_exclusion_desc()
                         desc.push_back(old_desc);
                     else
                     {
-                        snprintf(info, INFO_SIZE, "%d %s",
-                                 count, pluralise(old_desc).c_str());
-                        desc.push_back(info);
+                        desc.push_back(make_stringf("%d %s",
+                                       count, pluralise(old_desc).c_str()));
                         count = 1;
                     }
                     old_desc = tmp;
@@ -655,18 +642,16 @@ string exclude_set::get_exclusion_desc()
             desc.push_back(old_desc);
         else
         {
-            snprintf(info, INFO_SIZE, "%d %s",
-                     count, pluralise(old_desc).c_str());
-            desc.push_back(info);
+            desc.push_back(make_stringf("%d %s",
+                           count, pluralise(old_desc).c_str()));
         }
     }
 
     if (count_other > 0)
     {
-        snprintf(info, INFO_SIZE, "%d %sexclusion%s",
-                 count_other, desc.empty() ? "" : "more ",
-                 count_other > 1 ? "s" : "");
-        desc.push_back(info);
+        desc.push_back(make_stringf("%d %sexclusion%s",
+                                    count_other, desc.empty() ? "" : "more ",
+                                    count_other > 1 ? "s" : ""));
     }
     else if (desc.empty())
         return "";
@@ -674,9 +659,10 @@ string exclude_set::get_exclusion_desc()
     string desc_str = "";
     if (desc.size() > 1 || count_other == 0)
     {
-        snprintf(info, INFO_SIZE, "exclusion%s: ",
-                 desc.size() > 1 ? "s" : "");
-        desc_str += info;
+        desc_str += "exclusion";
+        if (desc.size() > 1)
+            desc_str += "s";
+        desc_str += ": ";
     }
     return desc_str + comma_separated_line(desc.begin(), desc.end(),
                                            " and ", ", ");
@@ -685,12 +671,11 @@ string exclude_set::get_exclusion_desc()
 void marshallExcludes(writer& outf, const exclude_set& excludes)
 {
     marshallShort(outf, excludes.size());
-    exclude_set::const_iterator it;
     if (excludes.size())
     {
-        for (it = excludes.begin(); it != excludes.end(); ++it)
+        for (const auto &entry : excludes)
         {
-            const travel_exclude &ex = it->second;
+            const travel_exclude &ex = entry.second;
             marshallCoord(outf, ex.pos);
             marshallShort(outf, ex.radius);
             marshallBoolean(outf, ex.autoex);
