@@ -6,19 +6,18 @@
 #ifndef EXTERNS_H
 #define EXTERNS_H
 
-#include <vector>
-#include <list>
-#include <string>
-#include <map>
-#include <set>
-#include <memory>
-#include <cstdlib>
-#include <deque>
-
-#include <time.h>
-#include <stdint.h>
 #define __STDC_FORMAT_MACROS
-#include <inttypes.h>
+#include <cinttypes>
+#include <cstdint>
+#include <cstdlib>
+#include <ctime>
+#include <deque>
+#include <list>
+#include <map>
+#include <memory>
+#include <set>
+#include <string>
+#include <vector>
 
 #include "bitary.h"
 #include "enum.h"
@@ -26,7 +25,6 @@
 #include "mpr.h"
 #include "pattern.h"
 #include "store.h"
-
 #include "tiledef_defines.h"
 
 struct tile_flavour
@@ -59,20 +57,10 @@ protected:
     tileidx_t m_tile;
 };
 
-#define INFO_SIZE       200          // size of message buffers
-#define ITEMNAME_SIZE   200          // size of item names/shop names/etc
-#define HIGHSCORE_SIZE  800          // <= 10 Lines for long format scores
-
-extern char info[INFO_SIZE];         // defined in main.cc {dlb}
-
-#define kNameLen        30
-const int kFileNameLen = 250;
-
-// Used only to bound the init file name length
-const int kPathLen = 256;
+#define MAX_NAME_LENGTH 30
 
 // This value is used to mark that the current berserk is free from
-// penalty (Xom's granted or from a deck of cards).
+// penalty (used for Xom's special berserk).
 #define NO_BERSERK_PENALTY    -1
 
 typedef FixedArray<dungeon_feature_type, GXM, GYM> feature_grid;
@@ -82,12 +70,10 @@ typedef FixedBitArray<GXM, GYM> map_bitmask;
 struct item_def;
 struct coord_def;
 class level_id;
-class player_quiver;
 class map_marker;
 class actor;
 class player;
 class monster;
-class KillMaster;
 class ghost_demon;
 
 typedef pair<coord_def, int> coord_weight;
@@ -97,14 +83,13 @@ template <typename Z> static inline Z sgn(Z x)
     return x < 0 ? -1 : (x > 0 ? 1 : 0);
 }
 
-static inline int dist_range(int x) { return x*x + 1; };
-
 struct coord_def
 {
     int         x;
     int         y;
 
-    explicit coord_def(int x_in = 0, int y_in = 0) : x(x_in), y(y_in) { }
+    coord_def(int x_in, int y_in) : x(x_in), y(y_in) { }
+    coord_def() : coord_def(0,0) { }
 
     void set(int xi, int yi)
     {
@@ -251,9 +236,6 @@ struct coord_def
     {
         return xi == x && yi == y;
     }
-
-    int range() const;
-    int range(const coord_def other) const;
 };
 
 extern const coord_def INVALID_COORD;
@@ -267,37 +249,49 @@ struct run_check_dir
     coord_def delta;
 };
 
+/**
+ * Persistent unique identifier for an actor.
+ *
+ * An mid_t is a persistent (across levels and across save/restore)
+ * and unique (within a given game) identifier for a monster, player,
+ * or fake actor. The value 0 indicates "no actor", and any value
+ * greater than or equal to MID_FIRST_NON_MONSTER indicates an actor
+ * other than a monster.
+ *
+ * mid_t should be used for anything that needs to remember monster
+ * identity from one turn to the next, as mindexes may be reused
+ * if a monster dies, and are not unique across levels.
+ */
 typedef uint32_t mid_t;
 #define PRImidt PRIu32
-#define MID_PLAYER      ((mid_t)0xffffffff)
+#define MID_NOBODY        ((mid_t)0x00000000)
+#define MID_PLAYER        ((mid_t)0xffffffff)
 // the numbers are meaningless, there's just plenty of space for gods, env,
 // and whatever else we want to have, while keeping all monster ids smaller.
-#define MID_ANON_FRIEND ((mid_t)0xffff0000)
+#define MID_ANON_FRIEND   ((mid_t)0xffff0000)
 #define MID_YOU_FAULTLESS ((mid_t)0xffff0001)
+/// Upper bound on the number of monsters that can ever exist in a game.
+#define MID_FIRST_NON_MONSTER MID_ANON_FRIEND
 
-static inline monster_type operator++(monster_type &x)
-{
-    x = static_cast<monster_type>(x + 1);
-    return x;
-}
+/**
+ * Define overloaded ++ and -- operators for the enum T.
+ *
+ * This macro produces several inline function definitions; use it only at
+ * file/namespace scope. It requires a trailing semicolon.
+ *
+ * @param T A type expression naming the enum type to augument. Evaluated
+ *          several times.
+ */
+#define DEF_ENUM_INC(T) \
+    static inline T &operator++(T &x) { return x = static_cast<T>(x + 1); } \
+    static inline T &operator--(T &x) { return x = static_cast<T>(x - 1); } \
+    static inline T operator++(T &x, int) { T y = x; ++x; return y; } \
+    static inline T operator--(T &x, int) { T y = x; --x; return y; } \
+    COMPILE_CHECK(is_enum<T>::value)
 
-static inline monster_type operator--(monster_type &x)
-{
-    x = static_cast<monster_type>(x - 1);
-    return x;
-}
-
-static inline spell_type operator++(spell_type &x)
-{
-    x = static_cast<spell_type>(x + 1);
-    return x;
-}
-
-static inline spell_type operator--(spell_type &x)
-{
-    x = static_cast<spell_type>(x - 1);
-    return x;
-}
+DEF_ENUM_INC(monster_type);
+DEF_ENUM_INC(spell_type);
+DEF_ENUM_INC(skill_type);
 
 struct cloud_struct
 {
@@ -347,6 +341,8 @@ struct shop_struct
 
     FixedVector<uint8_t, 3> keeper_name;
 
+    vector<item_def> stock;
+
     shop_struct () : pos(), greed(0), type(SHOP_UNASSIGNED), level(0),
                      shop_name(""), shop_type_name(""), shop_suffix_name("") { }
 
@@ -361,10 +357,6 @@ struct delay_queue_item
     int         parm2;
     int         parm3;
     bool        started;
-#if TAG_MAJOR_VERSION == 34
-    int         trits[6];
-#endif
-    size_t      len;
 };
 
 // Identifies a level. Should never include virtual methods or
@@ -384,6 +376,7 @@ public:
     // 'pos' on the current level leads to.
     static level_id get_next_level_id(const coord_def &pos);
 
+    // Important that if run after this, ::valid() is false.
     level_id()
         : branch(BRANCH_DUNGEON), depth(-1)
     {
@@ -394,9 +387,10 @@ public:
     }
 
     static level_id parse_level_id(const string &s) throw (string);
+#if TAG_MAJOR_VERSION == 34
     static level_id from_packed_place(const unsigned short place);
+#endif
 
-    unsigned short packed_place() const;
     string describe(bool long_name = false, bool with_number = true) const;
 
     void clear()
@@ -430,16 +424,6 @@ public:
         return branch < id.branch || (branch==id.branch && depth < id.depth);
     }
 
-    bool operator == (const branch_type _branch) const
-    {
-        return branch == _branch;
-    }
-
-    bool operator != (const branch_type _branch) const
-    {
-        return !operator == (_branch);
-    }
-
     void save(writer&) const;
     void load(reader&);
 };
@@ -507,7 +491,7 @@ struct level_pos
 class monster;
 
 // We are not 64 bits clean here yet since many places still pass (or store!)
-// it as 32 bits or, worse, longs.  I considered setting this as uint32_t,
+// it as 32 bits or, worse, longs. I considered setting this as uint32_t,
 // however, since free bits are exhausted, it's very likely we'll have to
 // extend this in the future, so this should be easier than undoing the change.
 typedef uint32_t iflags_t;
@@ -517,30 +501,63 @@ struct item_def
     object_class_type base_type:8; ///< basic class (eg OBJ_WEAPON)
     uint8_t        sub_type;       ///< type within that class (eg WPN_DAGGER)
 #pragma pack(push,2)
-    /// +to hit/dam, charges, corpse mon id
-    union { short plus; monster_type mon_type:16; };
+    union
+    {
+        // These must all be the same size!
+        short plus;                 ///< + to hit/dam (weapons, rods)
+        monster_type mon_type:16;   ///< corpse/chunk monster type
+        skill_type skill:16;        ///< the skill provided by a manual
+        short charges;              ///< # of charges held by a wand, etc
+                                    // for rods, is charge * ROD_CHARGE_MULT
+        short initial_cards;        ///< the # of cards a deck *started* with
+        short net_durability;       ///< damage dealt to a net
+        short book_param;           ///< level of spells in a monolevel book
+    };
+    union
+    {
+        // These must all be the same size!
+        short plus2;        ///< legacy/generic name for this union
+        short used_count;   ///< the # of known times it was used (decks, wands)
+                            // for wands, may hold negative ZAPCOUNT knowledge
+                            // info (e.g. "recharged", "empty", "unknown")
+        short net_placed;   ///< is this throwing net trapping something?
+        short skill_points; ///< # of skill points a manual gives
+        short charge_cap;   ///< max charges stored by a rod * ROD_CHARGE_MULT
+        short stash_freshness; ///< where stash.cc stores corpse freshness
+    };
 #pragma pack(pop)
-    short          plus2;          ///< sub-sub type for boots/helms, net wear
-    int            special;        ///< special stuff
-    colour_t       colour;         ///< item colour
-    uint8_t        rnd;            ///< random number, used for tile choice
+    union
+    {
+        // These must all be the same size!
+        int special;        ///< special stuff
+        deck_rarity_type deck_rarity;    ///< plain, ornate, legendary
+        int rod_plus;           ///< rate at which a rod recharges; +slay
+        uint32_t subtype_rnd;   ///< appearance of un-ID'd items, by subtype.
+                                /// jewellery, scroll, staff, wand, potions
+                                // see comment in item_colour()
+        int brand;          ///< weapon and armour brands; also marks artefacts
+        int freshness;      ///< remaining time until a corpse rots
+    };
+    uint8_t        rnd;            ///< random number, used for tile choice,
+                                   /// randart colours, and other per-item
+                                   /// random cosmetics. 0 = uninitialized
     short          quantity;       ///< number of items
     iflags_t       flags;          ///< item status flags
 
     /// The location of the item. Items in player inventory are indicated by
-    /// pos (-1, -1), items in monster inventory by (-2, -2), and and items
+    /// pos (-1, -1), items in monster inventory by (-2, -2), and items
     /// in shops by (0, y) for y >= 5.
     coord_def pos;
     /// Index in the mitm array of the next item in the stack. NON_ITEM for
     /// the last item in a stack. For items in player inventory, instead
     /// equal to slot. For items in monster inventory, equal to
-    /// NON_ITEM + 1 + mindex
+    /// NON_ITEM + 1 + mindex. For items in shops, equal to ITEM_IN_SHOP.
     short  link;
     // Inventory letter of the item.
     short  slot;
 
-    unsigned short orig_place;
-    short          orig_monnum;
+    level_id orig_place;
+    short    orig_monnum;
 
     string inscription;
 
@@ -548,8 +565,8 @@ struct item_def
 
 public:
     item_def() : base_type(OBJ_UNASSIGNED), sub_type(0), plus(0), plus2(0),
-                 special(0), colour(0), rnd(0), quantity(0), flags(0),
-                 pos(), link(NON_ITEM), slot(0), orig_place(0),
+                 special(0), rnd(0), quantity(0), flags(0),
+                 pos(), link(NON_ITEM), slot(0), orig_place(),
                  orig_monnum(0), inscription()
     {
     }
@@ -560,9 +577,14 @@ public:
                 iflags_t ignore_flags = 0x0) const;
     bool has_spells() const;
     bool cursed() const;
-    int book_number() const;
+    colour_t get_colour() const;
     zap_type zap() const; ///< what kind of beam it shoots (if wand).
 
+    bool is_type(int base, int sub) const
+    {
+        return base_type == base && sub_type == sub;
+    }
+
     /**
      * Find the index of an item in the mitm array. Results are undefined
      * if this item is not in the array!
@@ -585,9 +607,9 @@ public:
     /**
      * Sets this item as being held by a given monster.
      *
-     * @param midx The mindex of the monster.
+     * @param mon The monster. Must be in menv!
      */
-    void set_holding_monster(int midx);
+    void set_holding_monster(const monster& mon);
 
     /**
      * Get the monster holding this item.
@@ -600,6 +622,7 @@ public:
     bool held_by_monster() const;
 
     bool defined() const;
+    bool appearance_initialized() const;
     bool is_valid(bool info = false) const;
 
     /** Should this item be preserved as far as possible? */
@@ -614,6 +637,24 @@ public:
 private:
     string name_aux(description_level_type desc, bool terse, bool ident,
                     bool with_inscription, iflags_t ignore_flags) const;
+
+    colour_t randart_colour() const;
+
+    colour_t ring_colour() const;
+    colour_t amulet_colour() const;
+
+    colour_t rune_colour() const;
+
+    colour_t weapon_colour() const;
+    colour_t missile_colour() const;
+    colour_t armour_colour() const;
+    colour_t wand_colour() const;
+    colour_t food_colour() const;
+    colour_t jewellery_colour() const;
+    colour_t potion_colour() const;
+    colour_t book_colour() const;
+    colour_t miscellany_colour() const;
+    colour_t corpse_colour() const;
 };
 
 typedef item_def item_info;
@@ -625,6 +666,8 @@ public:
     int runmode;
     int mp;
     int hp;
+    bool notified_mp_full;
+    bool notified_hp_full;
     coord_def pos;
     int travel_speed;
 
@@ -668,14 +711,63 @@ private:
 
 typedef vector<delay_queue_item> delay_queue_type;
 
-class monster_spells : public FixedVector<spell_type, NUM_MONSTER_SPELL_SLOTS>
+enum mon_spell_slot_flag
 {
-public:
-    monster_spells()
-        : FixedVector<spell_type, NUM_MONSTER_SPELL_SLOTS>(SPELL_NO_SPELL)
-    { }
-    void clear() { init(SPELL_NO_SPELL); }
+    MON_SPELL_NO_FLAGS  = 0,
+    MON_SPELL_EMERGENCY = 1 << 0, // only use this spell slot in emergencies
+    MON_SPELL_NATURAL   = 1 << 1, // physiological, not really a spell
+    MON_SPELL_MAGICAL   = 1 << 2, // a generic magical ability
+    MON_SPELL_DEMONIC   = 1 << 3, // demonic
+    MON_SPELL_WIZARD    = 1 << 4, // a real spell, affected by AM and silence
+    MON_SPELL_PRIEST    = 1 << 5,
+
+    MON_SPELL_FIRST_CATEGORY = MON_SPELL_NATURAL,
+    MON_SPELL_LAST_CATEGORY  = MON_SPELL_PRIEST,
+
+    MON_SPELL_TYPE_MASK = MON_SPELL_NATURAL | MON_SPELL_MAGICAL
+                             | MON_SPELL_DEMONIC | MON_SPELL_WIZARD
+                             | MON_SPELL_PRIEST,
+    MON_SPELL_INNATE_MASK = MON_SPELL_NATURAL | MON_SPELL_MAGICAL
+                             | MON_SPELL_DEMONIC,
+    MON_SPELL_ANTIMAGIC_MASK = MON_SPELL_MAGICAL | MON_SPELL_DEMONIC
+                             | MON_SPELL_WIZARD,
+
+    MON_SPELL_BREATH    = 1 << 6, // sets a breath timer, requires it to be 0
+    MON_SPELL_NO_SILENT = 1 << 7, // can't be used while silenced/mute/etc.
+
+    MON_SPELL_SILENCE_MASK = MON_SPELL_WIZARD | MON_SPELL_PRIEST
+                             | MON_SPELL_NO_SILENT,
+
+    MON_SPELL_INSTANT   = 1 << 8, // allows another action on the same turn
+    MON_SPELL_NOISY     = 1 << 9, // makes noise despite being innate
+
+    MON_SPELL_LAST_FLAG = MON_SPELL_NOISY,
 };
+DEF_BITFIELD(mon_spell_slot_flags, mon_spell_slot_flag, 9);
+const int MON_SPELL_LAST_EXPONENT = mon_spell_slot_flags::last_exponent;
+COMPILE_CHECK(mon_spell_slot_flags::exponent(MON_SPELL_LAST_EXPONENT)
+              == MON_SPELL_LAST_FLAG);
+
+struct mon_spell_slot
+{
+    // Allow implicit conversion (and thus copy-initialization) from a
+    // three-element initializer list, but not from a smaller list or
+    // from a plain spell_type.
+    constexpr mon_spell_slot(spell_type spell_, uint8_t freq_,
+                             mon_spell_slot_flags flags_)
+        : spell(spell_), freq(freq_), flags(flags_)
+    { }
+    explicit constexpr mon_spell_slot(spell_type spell_ = SPELL_NO_SPELL,
+                                      uint8_t freq_ = 0)
+        : mon_spell_slot(spell_, freq_, MON_SPELL_NO_FLAGS)
+    { }
+
+    spell_type spell;
+    uint8_t freq;
+    mon_spell_slot_flags flags;
+};
+
+typedef vector<mon_spell_slot> monster_spells;
 
 class reader;
 class writer;
@@ -772,6 +864,6 @@ struct cglyph_t
     }
 };
 
-typedef FixedArray<item_type_id_state_type, NUM_OBJECT_CLASSES, MAX_SUBTYPES> id_arr;
+typedef FixedArray<bool, NUM_OBJECT_CLASSES, MAX_SUBTYPES> id_arr;
 
 #endif // EXTERNS_H
