@@ -26,7 +26,7 @@ public:
     bool did_leave_vault() const { return left_vault; }
 
 protected:
-    bool path_flood(const coord_def &c, const coord_def &dc);
+    bool path_flood(const coord_def &c, const coord_def &dc) override;
 protected:
     bool point_hunt, want_exit, no_vault, check_traversable;
     bool needed_features[NUM_FEATURES];
@@ -35,7 +35,7 @@ protected:
     const map_mask *vaults;
 
     const fgrd &fgrid;
-    const bound_check &bcheck;
+    bound_check bcheck;
 };
 
 template <typename fgrd, typename bound_check>
@@ -43,7 +43,7 @@ flood_find<fgrd, bound_check>::flood_find(const fgrd &f, const bound_check &bc,
                                      bool avoid_vaults, bool _check_traversable)
     : travel_pathfind(), point_hunt(false), want_exit(false),
       no_vault(avoid_vaults), check_traversable(_check_traversable),
-      needed_features(), needed_points(), left_vault(true), vaults(NULL),
+      needed_features(), needed_points(), left_vault(true), vaults(nullptr),
       fgrid(f), bcheck(bc)
 {
     memset(needed_features, false, sizeof needed_features);
