@@ -4,13 +4,12 @@
 #ifdef USE_TILE_LOCAL
 #ifdef USE_FT
 
-#include "tilefont.h"
-
 #include <map>
-
 #include <ft2build.h>
 #include FT_FREETYPE_H
 
+#include "tilefont.h"
+
 // TODO enne - Fonts could be made better by:
 //
 // * handling kerning
@@ -27,13 +26,13 @@ public:
 
     // font loading
     virtual bool load_font(const char *font_name, unsigned int font_size,
-                           bool outline);
+                           bool outline, int scale_num, int scale_den) override;
 
     // render just text
     virtual void render_textblock(unsigned int x, unsigned int y,
                                   ucs_t *chars, uint8_t *colours,
                                   unsigned int width, unsigned int height,
-                                  bool drop_shadow = false);
+                                  bool drop_shadow = false) override;
 
     // render text + background box
     virtual void render_string(unsigned int x, unsigned int y,
@@ -44,32 +43,34 @@ public:
                                unsigned char box_alpha = 0,
                                unsigned char box_colour = 0,
                                unsigned int outline = 0,
-                               bool tooltip = false);
+                               bool tooltip = false) override;
 
     // FontBuffer helper functions
     virtual void store(FontBuffer &buf, float &x, float &y,
-                       const string &s, const VColour &c);
+                       const string &s, const VColour &c) override;
     virtual void store(FontBuffer &buf, float &x, float &y,
-                       const formatted_string &fs);
+                       const formatted_string &fs) override;
     virtual void store(FontBuffer &buf, float &x, float &y, ucs_t c,
-                       const VColour &col);
+                       const VColour &col) override;
 
-    virtual unsigned int char_width() const;
-    virtual unsigned int char_height() const;
+    virtual unsigned int char_width() const override;
+    virtual unsigned int char_height() const override;
 
-    virtual unsigned int string_width(const char *text) ;
-    virtual unsigned int string_width(const formatted_string &str) ;
-    virtual unsigned int string_height(const char *text) const;
-    virtual unsigned int string_height(const formatted_string &str) const;
+    virtual unsigned int string_width(const char *text) override;
+    virtual unsigned int string_width(const formatted_string &str) override;
+    virtual unsigned int string_height(const char *text) const override;
+    virtual unsigned int string_height(const formatted_string &str) const override;
 
     // Try to split this string to fit in w x h pixel area.
     virtual formatted_string split(const formatted_string &str,
                                    unsigned int max_width,
-                                   unsigned int max_height);
+                                   unsigned int max_height) override;
 
-    virtual const GenericTexture *font_tex() const;
+    virtual const GenericTexture *font_tex() const override;
 
 protected:
+    // Not overrides! These two have an additional orig_x parameter compared
+    // to the virtuals.
     void store(FontBuffer &buf, float &x, float &y,
                const string &s, const VColour &c, float orig_x);
     void store(FontBuffer &buf, float &x, float &y, const formatted_string &fs,
@@ -141,6 +142,9 @@ protected:
     FT_Face face;
     bool    outl;
     unsigned char *pixels;
+
+    int scale_num;
+    int scale_den;
 };
 
 #endif // USE_FT
