@@ -4,8 +4,6 @@
 #include <string>
 #include <vector>
 
-#include "externs.h"
-
 // Definitions for formatted_string
 
 enum fs_op_type
@@ -28,7 +26,7 @@ public:
     void cprintf(PRINTF(1, ));
     void cprintf(const string &s);
     void add_glyph(cglyph_t g);
-    void textcolor(int color);
+    void textcolour(int colour);
     formatted_string chop(int length) const;
     void del_char();
     void all_caps();
@@ -51,11 +49,12 @@ public:
     static formatted_string parse_string(
             const string &s,
             bool  eot_ends_format = true,
-            bool (*process_tag)(const string &tag) = NULL,
+            bool (*process_tag)(const string &tag) = nullptr,
             int main_colour = LIGHTGREY);
 
     static void parse_string_to_multiple(const string &s,
-                                         vector<formatted_string> &out);
+                                         vector<formatted_string> &out,
+                                         int wrap_col = 0);
 
     static int get_colour(const string &tag);
 
@@ -74,8 +73,8 @@ public:
         bool relative;
         string text;
 
-        fs_op(int color)
-            : type(FSOP_COLOUR), x(color), y(-1), relative(false), text()
+        fs_op(int colour)
+            : type(FSOP_COLOUR), x(colour), y(-1), relative(false), text()
         {
         }
 
@@ -98,6 +97,8 @@ public:
 int count_linebreaks(const formatted_string& fs);
 
 int tagged_string_tag_length(const string& s);
+int printed_width(const string& s);
+
 void display_tagged_block(const string& s);
 
 #endif
