@@ -14,9 +14,8 @@ enum feature_property_type
     // NOTE: Bloody floor and sanctuary are exclusive.
     FPROP_UNUSED        = (1 << 6),  // used to be force_exclude
     FPROP_NO_CLOUD_GEN  = (1 << 7),
-    FPROP_NO_RTELE_INTO = (1 << 8),
-    FPROP_NO_CTELE_INTO = (1 << 9),
-    FPROP_NO_TELE_INTO  = FPROP_NO_RTELE_INTO | FPROP_NO_CTELE_INTO,
+    FPROP_NO_TELE_INTO  = (1 << 8),
+    FPROP_UNUSED_3      = (1 << 9),  // used to be no_ctele_into
 
     // Squares that the tide should not affect.
     FPROP_NO_TIDE       = (1 << 10),
