@@ -1,3 +1,4 @@
+
 /**
  * @file
  * @brief Player ghost and random Pandemonium demon handling.
@@ -7,135 +8,116 @@
 
 #include "ghost.h"
 
+#include <vector>
+
 #include "act-iter.h"
-#include "artefact.h"
 #include "colour.h"
 #include "database.h"
 #include "env.h"
-#include "externs.h"
 #include "itemname.h"
 #include "itemprop.h"
-#include "libutil.h"
+#include "mon-book.h"
+#include "mon-cast.h"
+#include "mon-transit.h"
 #include "ng-input.h"
-#include "random.h"
-#include "skills2.h"
-#include "spl-cast.h"
+#include "skills.h"
 #include "spl-util.h"
 #include "stringutil.h"
-#include "mon-util.h"
-#include "mon-transit.h"
-#include "player.h"
-
-#include <vector>
 
 #define MAX_GHOST_DAMAGE     50
 #define MAX_GHOST_HP        400
 #define MAX_GHOST_EVASION    60
-#define MIN_GHOST_SPEED       6
-#define MAX_GHOST_SPEED      13
 
 vector<ghost_demon> ghosts;
 
-// Order for looking for conjurations for the 1st & 2nd spell slots,
-// when finding spells to be remembered by a player's ghost.
+// Pan lord conjuration spell list.
 static spell_type search_order_conj[] =
 {
-    SPELL_LEHUDIBS_CRYSTAL_SPEAR,
     SPELL_FIRE_STORM,
     SPELL_GLACIATE,
+    SPELL_LEHUDIBS_CRYSTAL_SPEAR,
     SPELL_CHAIN_LIGHTNING,
-    SPELL_BOLT_OF_DRAINING,
-    SPELL_AGONY,
-    SPELL_DISINTEGRATE,
-    SPELL_LIGHTNING_BOLT,
-    SPELL_AIRSTRIKE,
-    SPELL_STICKY_FLAME,
-    SPELL_ISKENDERUNS_MYSTIC_BLAST,
     SPELL_IOOD,
-    SPELL_BOLT_OF_MAGMA,
-    SPELL_THROW_ICICLE,
+    SPELL_CORROSIVE_BOLT,
+    SPELL_DISINTEGRATE,
     SPELL_BOLT_OF_FIRE,
     SPELL_BOLT_OF_COLD,
-    SPELL_FIREBALL,
-    SPELL_DELAYED_FIREBALL,
-    SPELL_VENOM_BOLT,
     SPELL_IRON_SHOT,
-    SPELL_LRD,
-    SPELL_STONE_ARROW,
+    SPELL_POISON_ARROW,
+    SPELL_BOLT_OF_DRAINING,
+    SPELL_QUICKSILVER_BOLT,
     SPELL_FORCE_LANCE,
-    SPELL_DISCHARGE,
+    SPELL_FIREBALL,
+    SPELL_BOLT_OF_MAGMA,
+    SPELL_LRD,
+    SPELL_LIGHTNING_BOLT,
+    SPELL_BLINKBOLT,
+    SPELL_VENOM_BOLT,
+    SPELL_AGONY,
+    SPELL_DRAIN_MAGIC,
+    SPELL_SLEEP,
+    SPELL_ISKENDERUNS_MYSTIC_BLAST,
+    SPELL_STICKY_FLAME_RANGE,
+    SPELL_STEAM_BALL,
+    SPELL_THROW_ICICLE,
+    SPELL_AIRSTRIKE,
+    SPELL_SMITING,
     SPELL_DAZZLING_SPRAY,
+    SPELL_STONE_ARROW,
+    SPELL_DISCHARGE,
+    SPELL_VAMPIRIC_DRAINING,
     SPELL_THROW_FLAME,
     SPELL_THROW_FROST,
-    SPELL_FREEZE,
-    SPELL_PAIN,
-    SPELL_STING,
-    SPELL_SHOCK,
-    SPELL_SANDBLAST,
-    SPELL_MAGIC_DART,
-    SPELL_HIBERNATION,
-    SPELL_FLAME_TONGUE,
-    SPELL_CORONA,
     SPELL_NO_SPELL,                        // end search
 };
 
-// Order for looking for summonings and self-enchants for the 3rd spell
-// slot.
-static spell_type search_order_third[] =
+// Pan lord self-enchantment / summoning spell list.
+static spell_type search_order_selfench[] =
 {
-    SPELL_SYMBOL_OF_TORMENT,
-    SPELL_SUMMON_GREATER_DEMON,
-    SPELL_DRAGON_CALL,
+    SPELL_SUMMON_DRAGON,
     SPELL_SUMMON_HORRIBLE_THINGS,
+    SPELL_SUMMON_GREATER_DEMON,
     SPELL_HAUNT,
     SPELL_SUMMON_HYDRA,
-    SPELL_SUMMON_DEMON,
-    SPELL_HASTE,
-    SPELL_SILENCE,
-    SPELL_BATTLESPHERE,
-    SPELL_SUMMON_BUTTERFLIES,
-    SPELL_SUMMON_SWARM,
-    SPELL_MONSTROUS_MENAGERIE,
-    SPELL_SWIFTNESS,
-    SPELL_SUMMON_ICE_BEAST,
-    SPELL_ANIMATE_DEAD,
-    SPELL_TWISTED_RESURRECTION,
-    SPELL_INVISIBILITY,
-    SPELL_CALL_IMP,
-    SPELL_SUMMON_SMALL_MAMMAL,
     SPELL_MALIGN_GATEWAY,
-    SPELL_CONTROLLED_BLINK,
+    SPELL_HASTE,
+    SPELL_INVISIBILITY,
+    SPELL_SYMBOL_OF_TORMENT,
+    SPELL_MONSTROUS_MENAGERIE,
+    SPELL_SILENCE,
+    SPELL_SHADOW_CREATURES,
+    SPELL_SUMMON_DEMON,
+    SPELL_SUMMON_VERMIN,
+    SPELL_SUMMON_SWARM,
+    SPELL_BATTLESPHERE,
+    SPELL_FULMINANT_PRISM,
+    SPELL_SUMMON_ICE_BEAST,
+    SPELL_SWIFTNESS,
     SPELL_BLINK,
+    SPELL_SUMMON_BUTTERFLIES,
     SPELL_NO_SPELL,                        // end search
-    // No Simulacrum: iffy for pghosts (picking up material components),
-    // largely useless on Pan lords.
 };
 
-// Order for looking for enchants for the 4th & 5th spell slots.  If
-// this fails, go through conjurations.  Note: Dig must be in misc2
-// (5th) position to work.
+// Pan lord misc spell list.
 static spell_type search_order_misc[] =
 {
-    SPELL_TORNADO,
     SPELL_SHATTER,
-    SPELL_AGONY,
+    SPELL_SYMBOL_OF_TORMENT,
     SPELL_BANISHMENT,
     SPELL_FREEZING_CLOUD,
-    SPELL_OZOCUBUS_REFRIGERATION,
-    SPELL_OLGREBS_TOXIC_RADIANCE,
+    SPELL_POISONOUS_CLOUD,
     SPELL_MASS_CONFUSION,
     SPELL_ENGLACIATION,
     SPELL_DISPEL_UNDEAD,
-    SPELL_CONJURE_BALL_LIGHTNING,
-    SPELL_PARALYSE,
-    SPELL_CONFUSE,
-    SPELL_MEPHITIC_CLOUD,
-    SPELL_SLOW,
-    SPELL_PETRIFY,
-    SPELL_POLYMORPH,
-    SPELL_TELEPORT_OTHER,
     SPELL_DIG,
-    SPELL_CORONA,
+    SPELL_PETRIFY,
+    SPELL_OLGREBS_TOXIC_RADIANCE,
+    SPELL_PARALYSE,
+    SPELL_POLYMORPH,
+    SPELL_MEPHITIC_CLOUD,
+    SPELL_CONFUSE,
+    SPELL_TELEPORT_OTHER,
+    SPELL_SLOW,
     SPELL_NO_SPELL,                        // end search
 };
 
@@ -160,16 +142,14 @@ void ghost_demon::reset()
     ac               = 0;
     damage           = 0;
     speed            = 10;
+    move_energy      = 10;
     see_invis        = false;
     brand            = SPWPN_NORMAL;
     att_type         = AT_HIT;
     att_flav         = AF_PLAIN;
     resists          = 0;
-    spellcaster      = false;
-    cycle_colours    = false;
-    colour           = BLACK;
-    fly              = FL_NONE;
-    acting_part      = MONS_0;
+    colour           = COLOUR_UNDEF;
+    flies            = false;
 }
 
 /**
@@ -190,14 +170,32 @@ static brand_type _random_special_pan_lord_brand()
                          SPWPN_PAIN,
                          SPWPN_ANTIMAGIC,
                          SPWPN_DISTORTION,
-                         SPWPN_CHAOS,
-                         -1);
+                         SPWPN_CHAOS);
 }
 
-void ghost_demon::init_random_demon()
+#define ADD_SPELL(which_spell) \
+    do { \
+        const auto spell = (which_spell); \
+        if (spell != SPELL_NO_SPELL) \
+            spells.emplace_back(spell, 0, MON_SPELL_DEMONIC); \
+    } while (0)
+
+static int _panlord_random_resist_level()
 {
-    do name = make_name(random_int(), false);
-        while (!getLongDescription(name).empty());
+    return random_choose_weighted(1, -1,
+                                  3,  0,
+                                  3,  1,
+                                  3,  2,
+                                  0);
+}
+
+void ghost_demon::init_pandemonium_lord()
+{
+    do
+    {
+        name = make_name();
+    }
+    while (!getLongDescription(name).empty());
 
     // hp - could be defined below (as could ev, AC, etc.). Oh well, too late:
     max_hp = 100 + roll_dice(3, 50);
@@ -208,17 +206,8 @@ void ghost_demon::init_random_demon()
     see_invis = true;
 
     resists = 0;
-
-    if (!one_chance_in(3))
-        resists |= MR_RES_FIRE * random_range(1, 2);
-    else if (one_chance_in(10))
-        resists |= MR_VUL_FIRE;
-
-    if (!one_chance_in(3))
-        resists |= MR_RES_COLD * random_range(1, 2);
-    else
-        resists |= MR_VUL_COLD;
-
+    resists |= mrd(MR_RES_FIRE, _panlord_random_resist_level());
+    resists |= mrd(MR_RES_COLD, _panlord_random_resist_level());
     // Demons, like ghosts, automatically get poison res. and life prot.
 
     // resist electricity:
@@ -229,16 +218,14 @@ void ghost_demon::init_random_demon()
     damage = 20 + roll_dice(2, 20);
 
     // Does demon fly?
-    fly = (one_chance_in(3) ? FL_NONE :
-           one_chance_in(5) ? FL_LEVITATE
-                            : FL_WINGED);
+    flies = x_chance_in_y(2, 3);
 
     // hit dice:
     xl = 10 + roll_dice(2, 10);
 
     // Is demon a spellcaster?
     // Non-spellcasters always have branded melee and are faster instead.
-    spellcaster = x_chance_in_y(3,4);
+    const bool spellcaster = x_chance_in_y(3,4);
 
     if (one_chance_in(3) || !spellcaster)
         brand = _random_special_pan_lord_brand();
@@ -253,148 +240,107 @@ void ghost_demon::init_random_demon()
     else
         speed = 8 + roll_dice(2,5);
 
-    spells.init(SPELL_NO_SPELL);
+    spells.clear();
 
     if (spellcaster)
     {
         // This bit uses the list of player spells to find appropriate
         // spells for the demon, then converts those spells to the monster
-        // spell indices.  Some special monster-only spells are at the end.
+        // spell indices. Some special monster-only spells are at the end.
 
         if (coinflip())
-            spells[0] = RANDOM_ELEMENT(search_order_conj);
+            ADD_SPELL(RANDOM_ELEMENT(search_order_conj));
 
         // Might duplicate the first spell, but that isn't a problem.
         if (coinflip())
-            spells[1] = RANDOM_ELEMENT(search_order_conj);
+            ADD_SPELL(RANDOM_ELEMENT(search_order_conj));
 
-        if (!one_chance_in(4))
-            spells[2] = RANDOM_ELEMENT(search_order_third);
+        ADD_SPELL(one_chance_in(4) ? SPELL_SUMMON_DEMON
+                                   : RANDOM_ELEMENT(search_order_selfench));
 
         if (coinflip())
-        {
-            spells[3] = RANDOM_ELEMENT(search_order_misc);
-            if (spells[3] == SPELL_DIG)
-                spells[3] = SPELL_NO_SPELL;
-        }
+            ADD_SPELL(RANDOM_ELEMENT(search_order_misc));
 
         if (coinflip())
-            spells[4] = RANDOM_ELEMENT(search_order_misc);
+            ADD_SPELL(RANDOM_ELEMENT(search_order_misc));
 
-        spells[5] = random_choose_weighted(2, SPELL_TELEPORT_SELF,
-                                           1, SPELL_BLINK,
-                                           1, SPELL_NO_SPELL,
-                                           0);
-
-        // Convert the player spell indices to monster spell ones.
-        // Pan lords also get their Agony upgraded to Torment.
-        for (int i = 0; i < NUM_MONSTER_SPELL_SLOTS; ++i)
-        {
-            spells[i] = translate_spell(spells[i]);
-            if (spells[i] == SPELL_AGONY)
-                spells[i] = SPELL_SYMBOL_OF_TORMENT;
-            if (spells[i] == SPELL_CONJURE_BALL_LIGHTNING
-                || spells[i] == SPELL_OZOCUBUS_REFRIGERATION
-                || spells[i] == SPELL_TORNADO)
-            {
-                spells[i] = SPELL_NO_SPELL;
-            }
-        }
-
-        // Give demon a chance for some monster-only spells.
+        // Give demon a chance for some nasty spells.
         // Demon-summoning should be fairly common.
         if (one_chance_in(4))
         {
-            spells[0] = random_choose(SPELL_HELLFIRE_BURST,
-                                      SPELL_FIRE_STORM,
-                                      SPELL_GLACIATE,
-                                      SPELL_METAL_SPLINTERS,
-             /* eye of devastation */ SPELL_ENERGY_BOLT,
-                                      SPELL_ORB_OF_ELECTRICITY,
-                                      -1);
+            ADD_SPELL(random_choose(SPELL_HELLFIRE_BURST,
+                                    SPELL_FIRE_STORM,
+                                    SPELL_GLACIATE,
+                                    SPELL_METAL_SPLINTERS,
+           /* eye of devastation */ SPELL_ENERGY_BOLT,
+                                    SPELL_ORB_OF_ELECTRICITY));
         }
 
         if (one_chance_in(25))
-            spells[1] = SPELL_STEAM_BALL;
+            ADD_SPELL(SPELL_HELLFIRE);
         if (one_chance_in(25))
-            spells[1] = SPELL_QUICKSILVER_BOLT;
+            ADD_SPELL(SPELL_HELLFIRE_BURST);
         if (one_chance_in(25))
-            spells[1] = SPELL_HELLFIRE;
-        if (one_chance_in(25))
-            spells[1] = SPELL_IOOD;
+            ADD_SPELL(SPELL_IOOD);
 
-        if (one_chance_in(25))
-            spells[2] = SPELL_SMITING;
-        if (one_chance_in(25))
-            spells[2] = SPELL_HELLFIRE_BURST;
         if (one_chance_in(22))
-            spells[2] = SPELL_SUMMON_HYDRA;
+            ADD_SPELL(SPELL_SUMMON_HYDRA);
         if (one_chance_in(20))
-            spells[2] = SPELL_SUMMON_DRAGON;
+            ADD_SPELL(SPELL_SUMMON_DRAGON);
         if (one_chance_in(12))
-            spells[2] = SPELL_SUMMON_GREATER_DEMON;
+            ADD_SPELL(SPELL_SUMMON_GREATER_DEMON);
         if (one_chance_in(12))
-            spells[2] = SPELL_SUMMON_DEMON;
+            ADD_SPELL(SPELL_SUMMON_DEMON);
         if (one_chance_in(10))
-            spells[2] = SPELL_SUMMON_EYEBALLS;
+            ADD_SPELL(SPELL_SUMMON_EYEBALLS);
 
         if (one_chance_in(20))
-            spells[3] = SPELL_SUMMON_GREATER_DEMON;
+            ADD_SPELL(SPELL_SUMMON_GREATER_DEMON);
         if (one_chance_in(20))
-            spells[3] = SPELL_SUMMON_DEMON;
+            ADD_SPELL(SPELL_SUMMON_DEMON);
         if (one_chance_in(20))
-            spells[3] = SPELL_MALIGN_GATEWAY;
-
-        // At least they can summon demons.
-        if (spells[3] == SPELL_NO_SPELL)
-            spells[3] = SPELL_SUMMON_DEMON;
+            ADD_SPELL(SPELL_MALIGN_GATEWAY);
 
         if (one_chance_in(15))
-            spells[4] = SPELL_DIG;
+            ADD_SPELL(SPELL_DIG);
+
+        normalize_spell_freq(spells, xl);
     }
 
-    // Does demon cycle colours?
-    cycle_colours = one_chance_in(10);
-
-    colour = random_colour();
-
+    colour = one_chance_in(10) ? ETC_RANDOM : random_monster_colour();
 }
 
-// Returns the movement speed for a player ghost.  Note that this is a
-// real speed, not a movement cost, so higher is better.
-static int _player_ghost_base_movement_speed()
+// Returns the movement speed for a player ghost. Note that this is a
+// a movement cost, so lower is better.
+//FIXME: deduplicate with player_movement_speed()
+static int _player_ghost_movement_energy()
 {
-    int speed = 10;
+    int energy = 10;
 
     if (int fast = player_mutation_level(MUT_FAST, false))
-        speed += fast + 1;
+        energy -= fast + 1;
     if (int slow = player_mutation_level(MUT_SLOW, false))
-        speed -= slow + 1;
+        energy += slow + 2;
 
     if (you.wearing_ego(EQ_BOOTS, SPARM_RUNNING))
-        speed += 1;
+        energy -= 1;
 
-    // Cap speeds.
-    if (speed < MIN_GHOST_SPEED)
-        speed = MIN_GHOST_SPEED;
-    else if (speed > MAX_GHOST_SPEED)
-        speed = MAX_GHOST_SPEED;
+    if (you.wearing_ego(EQ_ALL_ARMOUR, SPARM_PONDEROUSNESS))
+        energy += 1;
 
-    return speed;
+    if (energy < FASTEST_PLAYER_MOVE_SPEED)
+        energy = FASTEST_PLAYER_MOVE_SPEED;
+
+    return energy;
 }
 
-void ghost_demon::init_player_ghost()
+void ghost_demon::init_player_ghost(bool actual_ghost)
 {
     name   = you.your_name;
-    max_hp = ((get_real_hp(false) >= MAX_GHOST_HP)
-             ? MAX_GHOST_HP
-             : get_real_hp(false));
-    ev     = player_evasion();
+    max_hp = min(get_real_hp(false), MAX_GHOST_HP);
+    ev     = min(you.evasion(EV_IGNORE_HELPLESS), MAX_GHOST_EVASION);
     ac     = you.armour_class();
 
-    if (ev > MAX_GHOST_EVASION)
-        ev = MAX_GHOST_EVASION;
-
     see_invis      = you.can_see_invisible();
     resists        = 0;
     set_resist(resists, MR_RES_FIRE, player_res_fire());
@@ -407,9 +353,11 @@ void ghost_demon::init_player_ghost()
     // multi-level for players, boolean as an innate monster resistance
     set_resist(resists, MR_RES_STEAM, player_res_steam() ? 1 : 0);
     set_resist(resists, MR_RES_STICKY_FLAME, player_res_sticky_flame());
-    set_resist(resists, MR_RES_ASPHYX, you.res_asphyx());
     set_resist(resists, MR_RES_ROTTING, you.res_rotting());
-    speed          = _player_ghost_base_movement_speed();
+    set_resist(resists, MR_RES_PETRIFY, you.res_petrify());
+
+    move_energy = _player_ghost_movement_energy();
+    speed       = 10;
 
     damage = 4;
     brand = SPWPN_NORMAL;
@@ -423,7 +371,10 @@ void ghost_demon::init_player_ghost()
         {
             damage = property(weapon, PWPN_DAMAGE);
 
-            damage *= 25 + you.skills[melee_skill(weapon)];
+            // Bows skill doesn't make bow-bashing better.
+            skill_type sk = is_range_weapon(weapon) ? SK_FIGHTING
+                                                    : item_attack_skill(weapon);
+            damage *= 25 + you.skills[sk];
             damage /= 25;
 
             if (weapon.base_type == OBJ_WEAPONS)
@@ -481,11 +432,9 @@ void ghost_demon::init_player_ghost()
     best_skill_level = you.skills[best_skill];
     xl = you.experience_level;
 
-    // These are the same as in mon-data.h.
-    colour = WHITE;
-    fly = FL_LEVITATE;
+    flies = true;
 
-    add_spells();
+    add_spells(actual_ghost);
 }
 
 static colour_t _ugly_thing_assign_colour(colour_t force_colour,
@@ -493,13 +442,15 @@ static colour_t _ugly_thing_assign_colour(colour_t force_colour,
 {
     colour_t colour;
 
-    if (force_colour != BLACK)
+    if (force_colour != COLOUR_UNDEF)
         colour = force_colour;
     else
     {
         do
+        {
             colour = ugly_thing_random_colour();
-        while (force_not_colour != BLACK && colour == force_not_colour);
+        }
+        while (force_not_colour != COLOUR_UNDEF && colour == force_not_colour);
     }
 
     return colour;
@@ -566,7 +517,7 @@ static attack_flavour _ugly_thing_colour_to_flavour(colour_t u_colour)
  * @param very_ugly     Whether the ugly thing is a very ugly thing.
  * @param only_mutate   Whether to mutate the ugly thing's colour away from its
  *                      old colour (the force_colour).
- * @param force_colour  The ugly thing's colour. (Default BLACK = random)
+ * @param force_colour  The ugly thing's colour. (Default COLOUR_UNDEF = random)
  */
 void ghost_demon::init_ugly_thing(bool very_ugly, bool only_mutate,
                                   colour_t force_colour)
@@ -579,6 +530,7 @@ void ghost_demon::init_ugly_thing(bool very_ugly, bool only_mutate,
     ev = stats->ev;
     ac = stats->AC;
     damage = stats->attack[0].damage;
+    move_energy = stats->energy_usage.move;
 
     // If we're mutating an ugly thing, leave its experience level, hit
     // dice and maximum hit points as they are.
@@ -596,12 +548,12 @@ void ghost_demon::init_ugly_thing(bool very_ugly, bool only_mutate,
 
     att_type = RANDOM_ELEMENT(att_types);
 
-    // An ugly thing always gets a low-intensity colour.  If we're
+    // An ugly thing always gets a low-intensity colour. If we're
     // mutating it, it always gets a different colour from what it had
     // before.
     colour = _ugly_thing_assign_colour(make_low_colour(force_colour),
                                        only_mutate ? make_low_colour(colour)
-                                                   : BLACK);
+                                                   : COLOUR_UNDEF);
 
     // Pick a compatible attack flavour for this colour.
     att_flav = _ugly_thing_colour_to_flavour(colour);
@@ -668,8 +620,8 @@ void ghost_demon::init_dancing_weapon(const item_def& weapon, int power)
     if (power > 100)
         power = 100;
 
-    colour = weapon.colour;
-    fly = FL_LEVITATE;
+    colour = weapon.get_colour();
+    flies = true;
 
     // We want Tukima to reward characters who invest heavily in
     // Hexes skill. Therefore, weapons benefit from very high skill.
@@ -681,7 +633,7 @@ void ghost_demon::init_dancing_weapon(const item_def& weapon, int power)
     // Bardiche:          speed 10, 40+20 damage, 18 AC, 40 HP, 15 EV
     // Dagger:            speed 20,  8+ 4 damage,  4 AC, 20 HP, 20 EV
     // Quick blade:       speed 23, 10+ 5 damage,  5 AC, 14 HP, 22 EV
-    // Cutlass:           speed 18, 14+ 7 damage,  7 AC, 24 HP, 19 EV
+    // Rapier:            speed 18, 14+ 7 damage,  7 AC, 24 HP, 19 EV
 
     xl = 15;
 
@@ -712,8 +664,8 @@ void ghost_demon::init_spectral_weapon(const item_def& weapon,
     if (wpn_skill > 270)
         wpn_skill = 270;
 
-    colour = weapon.colour;
-    fly = FL_LEVITATE;
+    colour = weapon.get_colour();
+    flies = true;
 
     // Hit dice (to hit) scales with weapon skill alone.
     // Damage scales with weapon skill, but how well depends on spell power.
@@ -755,84 +707,47 @@ void ghost_demon::init_spectral_weapon(const item_def& weapon,
     max_hp  = 10 + div_rand_round(power,3);
 }
 
-static bool _know_spell(spell_type spell)
-{
-    return you.has_spell(spell) && spell_fail(spell) < 50;
-}
-
-/**
- * Searches a list of ghost spells for the first one that
- * the player can cast.
- * @param spells The list of spells; it must be terminated by SPELL_NO_SPELL.
- * @param ignore_up_to_spell Ignore entries in the list up to and
- *                           including this one.
- * @return  The first spell the player knows.
- */
-static spell_type search_spell_list(spell_type* spells, spell_type ignore_up_to_spell)
-{
-    unsigned i = 0;
-    while (ignore_up_to_spell != SPELL_NO_SPELL
-            && spells[i] != SPELL_NO_SPELL)
-    {
-        if (spells[i++] == ignore_up_to_spell)
-            break;
-    }
-
-    while (spells[i] != SPELL_NO_SPELL)
-    {
-        if (_know_spell(spells[i]))
-            return spells[i];
-        ++i;
-    }
-
-    return SPELL_NO_SPELL;
-}
-
 // Used when creating ghosts: goes through and finds spells for the
-// ghost to cast.  Death is a traumatic experience, so ghosts only
+// ghost to cast. Death is a traumatic experience, so ghosts only
 // remember a few spells.
-void ghost_demon::add_spells()
+void ghost_demon::add_spells(bool actual_ghost)
 {
-    spells.init(SPELL_NO_SPELL);
+    spells.clear();
 
-    spells[0] = search_spell_list(search_order_conj, SPELL_NO_SPELL);
-    spells[1] = search_spell_list(search_order_conj, spells[0]);
-    spells[2] = search_spell_list(search_order_third, SPELL_NO_SPELL);
-    spells[3] = search_spell_list(search_order_misc, SPELL_NO_SPELL);
-    spells[4] = search_spell_list(search_order_misc, spells[3]);
-
-    if (spells[3] == SPELL_NO_SPELL)
+    for (int i = 0; i < you.spell_no; i++)
     {
-        spells[3] = search_spell_list(search_order_conj, spells[1]);
-        if (spells[4] == SPELL_NO_SPELL)
-            spells[4] = search_spell_list(search_order_conj, spells[3]);
-    }
-    else if (spells[4] == SPELL_NO_SPELL)
-         spells[4] = search_spell_list(search_order_conj, spells[1]);
-
-    // Look for Blink or Teleport Self for the emergency slot.
-    if (_know_spell(SPELL_CONTROLLED_BLINK)
-        || _know_spell(SPELL_BLINK))
-    {
-        spells[5] = SPELL_CONTROLLED_BLINK;
+        const int chance = max(0, 50 - failure_rate_to_int(raw_spell_fail(you.spells[i])));
+        const spell_type spell = translate_spell(you.spells[i]);
+        if (spell != SPELL_NO_SPELL
+            && !(get_spell_flags(spell) & SPFLAG_NO_GHOST)
+            && is_valid_mon_spell(spell)
+            && x_chance_in_y(chance*chance, 50*50))
+        {
+            spells.emplace_back(spell, 0, MON_SPELL_WIZARD);
+        }
     }
 
-    for (int i = 0; i < NUM_MONSTER_SPELL_SLOTS; ++i)
-        spells[i] = translate_spell(spells[i]);
+    normalize_spell_freq(spells, xl);
 
-    spellcaster = has_spells();
+    // After normalizing the frequencies!
+    if (species_is_draconian(species)
+        && species != SP_BASE_DRACONIAN
+        && species != SP_GREY_DRACONIAN
+        // Don't give pillusions extra breath
+        && actual_ghost)
+    {
+        spells.emplace_back(SPELL_BOLT_OF_DRAINING, 33, // Not too common
+                            MON_SPELL_NATURAL | MON_SPELL_BREATH);
+    }
 }
 
 bool ghost_demon::has_spells() const
 {
-    for (int i = 0; i < NUM_MONSTER_SPELL_SLOTS; ++i)
-        if (spells[i] != SPELL_NO_SPELL)
-            return true;
-    return false;
+    return spells.size() > 0;
 }
 
 // When passed the number for a player spell, returns the equivalent
-// monster spell.  Returns SPELL_NO_SPELL on failure (no equivalent).
+// monster spell. Returns SPELL_NO_SPELL on failure (no equivalent).
 spell_type ghost_demon::translate_spell(spell_type spell) const
 {
     switch (spell)
@@ -854,7 +769,7 @@ vector<ghost_demon> ghost_demon::find_ghosts()
 {
     vector<ghost_demon> gs;
 
-    if (!you.is_undead)
+    if (you.undead_state(false) == US_ALIVE)
     {
         ghost_demon player;
         player.init_player_ghost();
@@ -863,7 +778,7 @@ vector<ghost_demon> ghost_demon::find_ghosts()
     }
 
     // Pick up any other ghosts that happen to be on the level if we
-    // have space.  If the player is undead, add one to the ghost quota
+    // have space. If the player is undead, add one to the ghost quota
     // for the level.
     find_extra_ghosts(gs, n_extra_ghosts() + 1 - gs.size());
 
@@ -879,8 +794,7 @@ void ghost_demon::find_transiting_ghosts(
     const m_transit_list *mt = get_transit_list(level_id::current());
     if (mt)
     {
-        for (m_transit_list::const_iterator i = mt->begin();
-             i != mt->end() && n > 0; ++i)
+        for (auto i = mt->begin(); i != mt->end() && n > 0; ++i)
         {
             if (i->mons.type == MONS_PLAYER_GHOST)
             {
@@ -932,9 +846,8 @@ int ghost_demon::n_extra_ghosts()
 // Sanity checks for some ghost values.
 bool debug_check_ghosts()
 {
-    for (unsigned int k = 0; k < ghosts.size(); ++k)
+    for (const ghost_demon &ghost : ghosts)
     {
-        ghost_demon ghost = ghosts[k];
         // Values greater than the allowed maximum or less then the
         // allowed minimum signalise bugginess.
         if (ghost.damage < 0 || ghost.damage > MAX_GHOST_DAMAGE)
@@ -945,8 +858,6 @@ bool debug_check_ghosts()
             return false;
         if (ghost.ev > MAX_GHOST_EVASION)
             return false;
-        if (ghost.speed < MIN_GHOST_SPEED || ghost.speed > MAX_GHOST_SPEED)
-            return false;
         if (get_resist(ghost.resists, MR_RES_ELEC) < 0)
             return false;
         if (ghost.brand < SPWPN_NORMAL || ghost.brand > MAX_GHOST_BRAND)
@@ -970,23 +881,19 @@ bool debug_check_ghosts()
         if (ghost.att_type != AT_HIT || ghost.att_flav != AF_PLAIN)
             return false;
 
-        // Only Pandemonium lords cycle colours.
-        if (ghost.cycle_colours)
-            return false;
-
         // Name validation.
         if (!validate_player_name(ghost.name, false))
             return false;
         // Many combining characters can come per every letter, but if there's
         // that much, it's probably a maliciously forged ghost of some kind.
-        if (ghost.name.length() > kNameLen * 10 || ghost.name.empty())
+        if (ghost.name.length() > MAX_NAME_LENGTH * 10 || ghost.name.empty())
             return false;
         if (ghost.name != trimmed_string(ghost.name))
             return false;
 
         // Check for non-existing spells.
-        for (int sp = 0; sp < NUM_MONSTER_SPELL_SLOTS; ++sp)
-            if (ghost.spells[sp] < 0 || ghost.spells[sp] >= NUM_SPELLS)
+        for (const mon_spell_slot &slot : ghost.spells)
+            if (slot.spell < 0 || slot.spell >= NUM_SPELLS)
                 return false;
     }
     return true;
@@ -1004,137 +911,28 @@ int ghost_level_to_rank(const int xl)
     return 7;
 }
 
-static spell_type servitor_spells_primary[] =
+// Approximate inverse, in the middle of the range
+int ghost_rank_to_level(const int rank)
 {
-    SPELL_LEHUDIBS_CRYSTAL_SPEAR,
-    SPELL_IOOD,
-    SPELL_IRON_SHOT,
-    SPELL_BOLT_OF_FIRE,
-    SPELL_BOLT_OF_COLD,
-    SPELL_POISON_ARROW,
-    SPELL_LIGHTNING_BOLT,
-    SPELL_BOLT_OF_MAGMA,
-    SPELL_BOLT_OF_DRAINING,
-    SPELL_VENOM_BOLT,
-    SPELL_THROW_ICICLE,
-    SPELL_STONE_ARROW,
-    SPELL_ISKENDERUNS_MYSTIC_BLAST,
-    SPELL_NO_SPELL,                        // end search
-};
-
-static spell_type servitor_spells_secondary[] =
-{
-    SPELL_CONJURE_BALL_LIGHTNING,
-    SPELL_FIREBALL,
-    SPELL_AIRSTRIKE,
-    SPELL_LRD,
-    SPELL_FREEZING_CLOUD,
-    SPELL_POISONOUS_CLOUD,
-    SPELL_FORCE_LANCE,
-    SPELL_DAZZLING_SPRAY,
-    SPELL_MEPHITIC_CLOUD,
-    SPELL_NO_SPELL,                        // end search
-};
-
-static spell_type servitor_spells_fallback[] =
-{
-    SPELL_STICKY_FLAME,
-    SPELL_THROW_FLAME,
-    SPELL_THROW_FROST,
-    SPELL_FREEZE,
-    SPELL_FLAME_TONGUE,
-    SPELL_STING,
-    SPELL_SANDBLAST,
-    SPELL_MAGIC_DART,
-    SPELL_NO_SPELL,                        // end search
-};
-
-static spell_type _best_aligned_spell(vector<spell_type> spells, skill_type skill)
-{
-    for (unsigned int i = 0; i < spells.size(); ++i)
+    switch (rank)
     {
-        if (spell_typematch(spells[i], skill))
-            return spells[i];
+    case 0:
+        return 2;
+    case 1:
+        return 5;
+    case 2:
+        return 9;
+    case 3:
+        return 13;
+    case 4:
+        return 19;
+    case 5:
+        return 24;
+    case 6:
+        return 26;
+    case 7:
+        return 27;
+    default:
+        die("Bad ghost rank %d", rank);
     }
-
-    // If we couldn't find any that match, just pick the first one
-    return spells[0];
-}
-
-// Select servitor spells based on those known to the player
-// (Primary determines whether we are populating the first 3 or next 2 slots)
-bool ghost_demon::populate_servitor_spells(spell_type* spell_list, bool primary,
-                                           skill_type primary_skill)
-{
-    vector<spell_type> candidates;
-    const unsigned int num = (primary ? 3 : 2);
-    const unsigned int offset = (primary ? 0 : 3);
-
-    int i = 0;
-    spell_type spell = SPELL_NO_SPELL;
-    while ((spell = spell_list[i++]) != SPELL_NO_SPELL)
-    {
-        if (_know_spell(spell))
-            candidates.push_back(spell);
-    }
-
-    if (candidates.size() >= num)
-    {
-        for (unsigned int j = offset; j < offset + num; ++j)
-            spells[j] = candidates[j - offset];
-    }
-    else if (candidates.size() > 0)
-    {
-        // Choose the highest-level spell best aligned with our spell
-        // skills to duplicate
-        const spell_type copy_spell = _best_aligned_spell(candidates,
-                                                            primary_skill);
-
-        for (unsigned int j = offset; j < offset + num; ++j)
-        {
-            if (candidates.size() > j - offset)
-                spells[j] = candidates[j - offset];
-            else
-                spells[j] = copy_spell;
-        }
-    }
-
-    return candidates.size() > 0;
-}
-
-void ghost_demon::init_spellforged_servitor()
-{
-    // Determine highest magic skill (used for solving some tie-breakers)
-    skill_type best_magic_skill = NUM_SKILLS;
-    int skill_level = -1;
-    for (int i = SK_FIRE_MAGIC; i <= SK_POISON_MAGIC; ++i)
-    {
-        if (you.skill((skill_type)i) >= skill_level)
-        {
-            skill_level = you.skill((skill_type)i);
-            best_magic_skill = (skill_type)i;
-        }
-    }
-
-    int pow = calc_spell_power(SPELL_SPELLFORGED_SERVITOR, true);
-
-    colour = LIGHTMAGENTA; // cf. mon-data.h
-    speed = 10;
-    ev = 10;
-    ac = 10;
-    xl = 9 + div_rand_round(pow, 14);
-    max_hp = 80;
-    spellcaster = true;
-    damage = 0;
-    att_type = AT_NONE;
-
-    // Give the servitor its spells
-    bool primary   = populate_servitor_spells(servitor_spells_primary, true,
-                                              best_magic_skill);
-    bool secondary = populate_servitor_spells(servitor_spells_secondary, false,
-                                              best_magic_skill);
-
-    if (!primary && !secondary)
-        populate_servitor_spells(servitor_spells_fallback, true, best_magic_skill);
-
 }
