@@ -7,20 +7,18 @@
 
 #include "godblessing.h"
 
-#include "artefact.h"   // if_artefact()
-#include "env.h"        // mitm
-#include "itemprop.h"   // do_uncurse()
+#include "artefact.h"
+#include "env.h"
+#include "itemprop.h"
+#include "items.h"
 #include "item_use.h"
-#include "items.h"      // items_stack()
-#include "makeitem.h"   // item_set_appearance()
-#include "mgen_data.h"
-#include "monster.h"
-#include "mon-gear.h"   // give_specific_item()
-#include "mon-place.h"  // create_monster()
-#include "mon-util.h"   // give_monster_proper_name()
+#include "makeitem.h"
+#include "message.h"
+#include "mon-gear.h"
+#include "mon-place.h"
 #include "religion.h"
-#include "stringutil.h"    // make_stringf
-#include "view.h"       // flash_monster_colour
+#include "stringutil.h"
+#include "view.h"
 
 /**
  * Given the weapon type, find an upgrade.
@@ -46,20 +44,22 @@ static int _upgrade_weapon_type(int old_type, bool has_shield, bool highlevel)
 
         case WPN_DAGGER:      return WPN_FALCHION;
         case WPN_SHORT_SWORD: return WPN_LONG_SWORD;
-        case WPN_CUTLASS:     return WPN_SCIMITAR;
+        case WPN_RAPIER:      return WPN_SCIMITAR;
         case WPN_FALCHION:    return WPN_LONG_SWORD;
         case WPN_LONG_SWORD:  return WPN_SCIMITAR;
         case WPN_SCIMITAR:    return !has_shield ? WPN_GREAT_SWORD   :
-                                     highlevel   ? WPN_BASTARD_SWORD :
+                                     highlevel   ? WPN_DOUBLE_SWORD :
                                                    WPN_SCIMITAR;
-        case WPN_GREAT_SWORD: return highlevel ? WPN_CLAYMORE : WPN_GREAT_SWORD;
+        case WPN_GREAT_SWORD: return highlevel ? WPN_TRIPLE_SWORD :
+                                                 WPN_GREAT_SWORD;
 
         case WPN_HAND_AXE:    return WPN_WAR_AXE;
             // Low level orcs shouldn't get fairly rare items.
         case WPN_WAR_AXE:     return !highlevel  ? WPN_WAR_AXE   :
                                      has_shield  ? WPN_BROAD_AXE :
                                                    WPN_BATTLEAXE;
-        case WPN_BATTLEAXE:   return highlevel ? WPN_EXECUTIONERS_AXE : WPN_BATTLEAXE;
+        case WPN_BATTLEAXE:   return highlevel ? WPN_EXECUTIONERS_AXE :
+                                                 WPN_BATTLEAXE;
 
         case WPN_SPEAR:       return WPN_TRIDENT;
         case WPN_TRIDENT:     return has_shield ? WPN_TRIDENT : WPN_HALBERD;
@@ -178,7 +178,7 @@ void gift_ammo_to_orc(monster* orc, bool initial_gift)
 static string _beogh_bless_melee_weapon(monster* mon)
 {
     item_def* wpn_ptr = mon->melee_weapon();
-    ASSERT(wpn_ptr != NULL);
+    ASSERT(wpn_ptr != nullptr);
     item_def& wpn = *wpn_ptr;
 
     const int old_weapon_type = wpn.sub_type;
@@ -227,11 +227,11 @@ static string _beogh_bless_ranged_weapon(monster* mon)
 {
     // if they already have a launcher, restock it.
     // likewise if they have a shield but no launcher (give tomahawks)
-    const bool mon_has_launcher = mon->launcher() != NULL;
-    if (mon_has_launcher || mon->shield() != NULL)
+    const bool mon_has_launcher = mon->launcher() != nullptr;
+    if (mon_has_launcher || mon->shield() != nullptr)
     {
         gift_ammo_to_orc(mon);
-        if (mon->missiles() != NULL)
+        if (mon->missiles() != nullptr)
             return mon_has_launcher ? "ammunition" : "ranged armament";
 
         dprf("Couldn't give ammo to follower!");
@@ -240,14 +240,14 @@ static string _beogh_bless_ranged_weapon(monster* mon)
 
     // no launcher, no shield: give them a crossbow & some ammo.
     _gift_weapon_to_orc(mon, WPN_ARBALEST);
-    if (mon->launcher() == NULL)
+    if (mon->launcher() == nullptr)
     {
         dprf("Couldn't give crossbow to follower!");
         return ""; // ?
     }
 
     gift_ammo_to_orc(mon, true);
-    if (mon->missiles() == NULL)
+    if (mon->missiles() == nullptr)
         dprf("Couldn't give initial ammo to follower");
     return "ranged armament";
 }
@@ -261,10 +261,10 @@ static string _beogh_bless_ranged_weapon(monster* mon)
 static string _beogh_bless_weapon(monster* mon)
 {
     const item_def* wpn_ptr = mon->melee_weapon();
-    if (wpn_ptr == NULL)
+    if (wpn_ptr == nullptr)
     {
         _gift_weapon_to_orc(mon, _orc_weapon_gift_type(mon->type));
-        if (mon->weapon() != NULL)
+        if (mon->weapon() != nullptr)
             return "armament";
 
         dprf("Couldn't give a weapon to follower!");
@@ -273,10 +273,10 @@ static string _beogh_bless_weapon(monster* mon)
 
     const item_def* launch_ptr = mon->launcher();
     const item_def* ammo_ptr = mon->missiles();
-    if (launch_ptr != NULL && ammo_ptr == NULL)
+    if (launch_ptr != nullptr && ammo_ptr == nullptr)
     {
         gift_ammo_to_orc(mon);
-        if (mon->missiles() != NULL)
+        if (mon->missiles() != nullptr)
             return "ammunition";
 
         dprf("Couldn't give ammo to follower!");
@@ -355,9 +355,9 @@ static string _beogh_bless_armour(monster* mon)
     // Pick either a monster's armour or its shield.
     const item_def* melee_weap = mon->melee_weapon();
     const item_def* launcher = mon->launcher();
-    const bool can_use_shield = (melee_weap == NULL
+    const bool can_use_shield = (melee_weap == nullptr
                                  || mon->hands_reqd(*melee_weap) != HANDS_TWO)
-                                && (launcher == NULL
+                                && (launcher == nullptr
                                    || mon->hands_reqd(*launcher) != HANDS_TWO);
     const int slot = coinflip() && can_use_shield ? shield : armour;
 
@@ -403,8 +403,8 @@ static string _beogh_bless_armour(monster* mon)
 
 static bool _blessing_balms(monster* mon)
 {
-    // Remove poisoning, sickness, confusion, and rotting, like a potion
-    // of curing, but without the healing. Also, remove slowing and
+    // Remove poisoning, sickness and confusion, like a potion of
+    // curing, but without the healing. Also, remove slowing and
     // fatigue.
     bool success = false;
 
@@ -417,9 +417,6 @@ static bool _blessing_balms(monster* mon)
     if (mon->del_ench(ENCH_CONFUSION, true))
         success = true;
 
-    if (mon->del_ench(ENCH_ROT, true))
-        success = true;
-
     if (mon->del_ench(ENCH_SLOW, true))
         success = true;
 
@@ -465,75 +462,6 @@ static bool _increase_ench_duration(monster* mon,
     return true;
 }
 
-/**
- * Attempt to bless a follower's weapon.
- *
- * @param[in] mon      The follower whose weapon should be blessed.
- * @return             The type of blessing; may be empty.
- */
-static string _tso_bless_weapon(monster* mon)
-{
-    if (mon->type == MONS_DANCING_WEAPON)
-    {
-        dprf("Can't bless a dancing weapon's weapon!");
-        return "";
-    }
-
-    item_def* wpn_ptr = mon->weapon();
-    if (wpn_ptr == NULL)
-    {
-        dprf("Couldn't bless follower's weapon; they have none!");
-        return "";
-    }
-
-    item_def& wpn = *wpn_ptr;
-
-    // Enchant and uncurse it.
-    if (!enchant_weapon(wpn, true))
-    {
-        dprf("Couldn't bless follower's weapon!");
-        return "";
-    }
-
-    item_set_appearance(wpn);
-    set_ident_flags(wpn, ISFLAG_KNOW_PLUSES);
-    return "superior armament";
-}
-
-/**
- * Attempt to bless a follower's armour.
- *
- * @param[in] follower      The follower whose armour should be blessed.
- * @return                  The type of blessing; may be empty.
- */
-static string _tso_bless_armour(monster* follower)
-{
-    // Pick either a monster's armour or its shield.
-    const int armour = follower->inv[MSLOT_ARMOUR];
-    const int shield = follower->inv[MSLOT_SHIELD];
-    if (armour == NON_ITEM && shield == NON_ITEM)
-    {
-        dprf("Can't bless the armour of a naked follower!");
-        return "";
-    }
-
-    const int slot = shield == NON_ITEM || (coinflip() && armour != NON_ITEM) ?
-                     armour : shield;
-
-    item_def& arm(mitm[slot]);
-
-    int ac_change;
-    if (!enchant_armour(ac_change, true, arm))
-    {
-        dprf("Couldn't enchant follower's armour!");
-        return "";
-    }
-
-    item_set_appearance(arm);
-    set_ident_flags(arm, ISFLAG_KNOW_PLUSES);
-    return "improved armour";
-}
-
 static bool _tso_blessing_extend_stay(monster* mon)
 {
     if (!mon->has_ench(ENCH_ABJ))
@@ -679,9 +607,9 @@ static string _bless_with_healing(monster* follower)
 static void _display_god_blessing(monster* follower, god_type god,
                                   string blessing)
 {
-    ASSERT(follower);
+    ASSERT(follower); // XXX: change to monster &follower
 
-    string whom = you.can_see(follower) ? follower->name(DESC_THE)
+    string whom = you.can_see(*follower) ? follower->name(DESC_THE)
     : "a follower";
 
     simple_god_message(make_stringf(" blesses %s with %s.",
@@ -696,8 +624,7 @@ static void _display_god_blessing(monster* follower, god_type god,
 /**
  * Have Beogh attempt to bless the specified follower.
  *
- * He may choose to bless another nearby follower, if no follower is specified,
- * or the follower is invalid.
+ * If no follower is specified, there is a chance of sending reinforcements.
  *
  * @param[in] follower      The follower to try to bless.
  * @param[in] force         Whether to check follower validity.
@@ -705,30 +632,12 @@ static void _display_god_blessing(monster* follower, god_type god,
  */
 static bool _beogh_bless_follower(monster* follower, bool force)
 {
-    // If a follower was specified, and it's suitable, pick it.
-    // Otherwise, pick a random follower.
-    // XXX: factor out into another function?
-    if (!follower || (!force && !is_follower(follower)))
-    {
-        if (!one_chance_in(10))
-            return false;
-
-        // Choose a random follower in LOS, preferably a named or
-        // priestly one.
-        follower = choose_random_nearby_monster(0, is_follower, true);
-    }
-
     if (!follower)
     {
-        if (coinflip())
+        // 1/20 chance of spawning a palband
+        if (!one_chance_in(5))
             return false;
 
-        // Try *again*, on the entire level (2.5% chance).
-        follower = choose_random_monster_on_level(0, is_follower);
-    }
-
-    if (!follower)
-    {
         // If no follower was found, attempt to send
         // reinforcements.
         _beogh_blessing_reinforcements();
@@ -817,8 +726,7 @@ static string _tso_bless_duration(monster* follower)
 /**
  * Have The Shining One attempt to bless the specified follower.
  *
- * 10% chance of blessing equipment; otherwise, increase duration, or
- * barring that, just heal & cure.
+ * Blessings can increase duration, or heal & cure.
  *
  * @param[in] follower      The follower to try to bless.
  * @param[in] force         Whether to check follower validity.
@@ -831,15 +739,10 @@ static bool _tso_bless_follower(monster* follower, bool force)
         return false;
 
     string blessing = "";
-    if (one_chance_in(10) || force)
-    {
-        blessing = coinflip() ? _tso_bless_weapon(follower)
-                              : _tso_bless_armour(follower);
-    }
+    if (blessing.empty() && coinflip())
+        blessing = _bless_with_healing(follower);
     if (blessing.empty())
         blessing = _tso_bless_duration(follower);
-    if (blessing.empty())
-        blessing = _bless_with_healing(follower);
 
     if (blessing.empty())
         return false;
@@ -848,14 +751,36 @@ static bool _tso_bless_follower(monster* follower, bool force)
     return true;
 }
 
-// Bless the follower indicated in follower, if any.  If there isn't
-// one, bless a random follower within sight of the player, if any, or,
-// with decreasing chances, any follower on the level.
+// Bless the follower indicated in follower, if any. If there isn't
+// one, bless a random follower within sight of the player, if any, or
+// any follower on the level.
 // Blessing can be enforced with a wizard mode command.
 bool bless_follower(monster* follower,
                     god_type god,
                     bool force)
 {
+    // most blessings fail, tragically...
+    if (!force && !one_chance_in(4))
+        return false;
+
+    // If a follower was specified, and it's suitable, pick it.
+    // Otherwise, pick a random follower.
+    // XXX: factor out into another function?
+    if (!follower || (!force && !is_follower(follower)))
+    {
+        // Choose a random follower in LOS, preferably a named or
+        // priestly one.
+        follower = choose_random_nearby_monster(0, is_follower,
+                                                god == GOD_BEOGH);
+    }
+
+    // Try *again*, on the entire level
+    if (!follower)
+    {
+        follower = choose_random_monster_on_level(0, is_follower,
+                                                  god == GOD_BEOGH);
+    }
+
     switch (god)
     {
         case GOD_BEOGH: return _beogh_bless_follower(follower, force);
