@@ -5,17 +5,16 @@
 
 #include "AppHdr.h"
 
-#include <algorithm>
-
 #include "godcompanions.h"
 
-#include "actor.h"
-#include "mon-message.h"
+#include <algorithm>
+
+#include "branch.h"
+#include "dgn-overview.h"
+#include "message.h"
 #include "mon-util.h"
 #include "religion.h"
-#include "stringutil.h"
 #include "spl-other.h"
-#include "branch.h"
 
 map<mid_t, companion> companion_list;
 
@@ -34,22 +33,27 @@ void init_companions()
 void add_companion(monster* mons)
 {
     ASSERT(mons->alive());
+    // Right now this is a special case for Saint Roka, but
+    // future orcish uniques should behave in the same way.
+    mons->props["no_annotate"] = true;
+    remove_unique_annotation(mons);
     companion_list[mons->mid] = companion(*mons);
 }
 
 void remove_companion(monster* mons)
 {
+    mons->props["no_annotate"] = false;
+    set_unique_annotation(mons);
     companion_list.erase(mons->mid);
 }
 
 void remove_enslaved_soul_companion()
 {
-    for (map<mid_t, companion>::iterator i = companion_list.begin();
-         i != companion_list.end(); ++i)
+    for (auto &entry : companion_list)
     {
-        monster* mons = monster_by_mid(i->first);
+        monster* mons = monster_by_mid(entry.first);
         if (!mons)
-            mons = &i->second.mons.mons;
+            mons = &entry.second.mons.mons;
         if (mons_enslaved_soul(mons))
         {
             remove_companion(mons);
@@ -60,8 +64,7 @@ void remove_enslaved_soul_companion()
 
 void remove_all_companions(god_type god)
 {
-    for (map<mid_t, companion>::iterator i = companion_list.begin();
-         i != companion_list.end();)
+    for (auto i = companion_list.begin(); i != companion_list.end();)
     {
         monster* mons = monster_by_mid(i->first);
         if (!mons)
@@ -88,17 +91,16 @@ void move_companion_to(const monster* mons, const level_id lid)
 
 void update_companions()
 {
-    for (map<mid_t, companion>::iterator i = companion_list.begin();
-         i != companion_list.end(); ++i)
+    for (auto &entry : companion_list)
     {
-        monster* mons = monster_by_mid(i->first);
+        monster* mons = monster_by_mid(entry.first);
         if (mons)
         {
             if (mons->is_divine_companion())
             {
                 ASSERT(mons->alive());
-                i->second.mons = follower(*mons);
-                i->second.timestamp = you.elapsed_time;
+                entry.second.mons = follower(*mons);
+                entry.second.timestamp = you.elapsed_time;
             }
         }
     }
@@ -106,20 +108,17 @@ void update_companions()
 
 void populate_offlevel_recall_list(vector<pair<mid_t, int> > &recall_list)
 {
-    for (map<mid_t, companion>::iterator i = companion_list.begin();
-         i != companion_list.end(); ++i)
+    for (auto &entry : companion_list)
     {
-        int mid = i->first;
-        companion* comp = &i->second;
+        int mid = entry.first;
+        companion &comp = entry.second;
         if (companion_is_elsewhere(mid, true))
         {
             // Recall can't pull monsters out of the Abyss
-            if (comp->level.branch == BRANCH_ABYSS)
+            if (comp.level.branch == BRANCH_ABYSS)
                 continue;
 
-            pair<mid_t, int> p = make_pair(mid,
-                                           comp->mons.mons.get_experience_level());
-            recall_list.push_back(p);
+            recall_list.emplace_back(mid, comp.mons.mons.get_experience_level());
         }
     }
 }
@@ -203,13 +202,12 @@ void wizard_list_companions()
         return;
     }
 
-    for (map<mid_t, companion>::iterator i = companion_list.begin();
-        i != companion_list.end(); ++i)
+    for (auto &entry : companion_list)
     {
-        companion* comp = &i->second;
-        monster* mon = &comp->mons.mons;
-        mprf("%s (%d)(%s:%d)", mon->name(DESC_PLAIN, true).c_str(),
-                mon->mid, branches[comp->level.branch].abbrevname, comp->level.depth);
+        companion &comp = entry.second;
+        monster &mon = comp.mons.mons;
+        mprf("%s (%d)(%s:%d)", mon.name(DESC_PLAIN, true).c_str(), mon.mid,
+             branches[comp.level.branch].abbrevname, comp.level.depth);
     }
 }
 
@@ -219,8 +217,7 @@ void wizard_list_companions()
 // allow the creation of these invalid companions are fully mopped up
 void fixup_bad_companions()
 {
-    for (map<mid_t, companion>::iterator i = companion_list.begin();
-         i != companion_list.end();)
+    for (auto i = companion_list.begin(); i != companion_list.end();)
     {
         if (invalid_monster_type(i->second.mons.mons.type))
             companion_list.erase(i++);
