@@ -18,18 +18,19 @@ struct god_conduct_trigger
     god_conduct_trigger(conduct_type c = NUM_CONDUCTS,
                         int pg = 0,
                         bool kn = true,
-                        const monster* vict = NULL);
+                        const monster* vict = nullptr);
 
     void set(conduct_type c = NUM_CONDUCTS,
              int pg = 0,
              bool kn = true,
-             const monster* vict = NULL);
+             const monster* vict = nullptr);
 
     ~god_conduct_trigger();
 };
 
-bool did_god_conduct(conduct_type thing_done, int level, bool known = true,
-                     const monster* victim = NULL);
+void did_kill_conduct(conduct_type thing_done, const monster &victim);
+void did_god_conduct(conduct_type thing_done, int level, bool known = true,
+                     const monster* victim = nullptr);
 void set_attack_conducts(god_conduct_trigger conduct[3], const monster* mon,
                          bool known = true);
 void enable_attack_conducts(god_conduct_trigger conduct[3]);
@@ -37,4 +38,6 @@ void disable_attack_conducts(god_conduct_trigger conduct[3]);
 
 void god_conduct_turn_start();
 
+bool god_punishes_spell(spell_type spell, god_type god);
+
 #endif
