@@ -10,7 +10,7 @@ class GodMenuEntry : public MenuEntry
 public:
     GodMenuEntry(god_type god, bool long_name = false);
 
-    virtual string get_text(const bool unused = false) const;
+    virtual string get_text(const bool unused = false) const override;
 
 public:
     god_type god;
