@@ -7,9 +7,9 @@
 #define GODWRATH_H
 
 bool divine_retribution(god_type god, bool no_bonus = false, bool force = false);
-bool do_god_revenge(conduct_type thing_done);
-void ash_reduce_penance(int amount);
+void reduce_xp_penance(god_type god, int amount);
+
+void beogh_idol_revenge();
 
 void gozag_incite(monster *mon);
-bool gozag_goldify(const item_def &it, int quant_got, bool quiet = false);
 #endif
