@@ -6,12 +6,10 @@
 #ifndef HINTS_H
 #define HINTS_H
 
+#include <cstdio>
+#include <cstdlib>
 #include <string>
 #include <vector>
-#include <stdio.h>
-#include <stdlib.h>
-
-#include "externs.h"
 
 class formatted_string;
 class writer;
@@ -77,7 +75,9 @@ enum hints_event_type
     HINT_GAINED_RANGED_SKILL,
     HINT_CHOOSE_STAT,
     HINT_MAKE_CHUNKS,
+#if TAG_MAJOR_VERSION == 34
     HINT_OFFER_CORPSE,
+#endif
     HINT_NEW_ABILITY_GOD,
     HINT_NEW_ABILITY_MUT,
     HINT_NEW_ABILITY_ITEM,
@@ -94,8 +94,8 @@ enum hints_event_type
     HINT_YOU_ENCHANTED,
 #if TAG_MAJOR_VERSION == 34
     HINT_CONTAMINATED_CHUNK,
-#endif
     HINT_YOU_SICK,
+#endif
     HINT_YOU_POISON,
     HINT_YOU_ROTTING,
     HINT_YOU_CURSED,
@@ -151,17 +151,16 @@ enum hints_event_type
 struct newgame_def;
 void init_hints();
 void tutorial_init_hints();
-void pick_hints(newgame_def* choice);
+void pick_hints(newgame_def& choice);
 void hints_load_game();
 
-formatted_string hints_starting_info2();
 void hints_starting_screen();
 void hints_new_turn();
 void print_hint(string key, const string arg1 = "", const string arg2 = "");
 void hints_death_screen();
 void hints_finished();
 
-void hints_dissection_reminder(bool healthy);
+void hints_dissection_reminder();
 void hints_healing_check();
 
 void taken_new_item(object_class_type item_type);
@@ -182,7 +181,7 @@ void hints_inscription_info(string prompt);
 bool hints_pos_interesting(int x, int y);
 void hints_describe_pos(int x, int y);
 bool hints_monster_interesting(const monster* mons);
-void hints_describe_monster(const monster_info& mi, bool has_stat_desc);
+string hints_describe_monster(const monster_info& mi, bool has_stat_desc);
 
 void hints_observe_cell(const coord_def& gc);
 
