@@ -11,6 +11,9 @@ enum armour_type
     ARM_CHAIN_MAIL,
     ARM_PLATE_ARMOUR,
     ARM_LAST_MUNDANE_BODY = ARM_PLATE_ARMOUR,
+#if TAG_MAJOR_VERSION > 34
+    ARM_CRYSTAL_PLATE_ARMOUR,
+#endif
 
     ARM_CLOAK,
 
@@ -30,8 +33,9 @@ enum armour_type
     ARM_LARGE_SHIELD,
     ARM_LAST_SHIELD = ARM_LARGE_SHIELD,
 
+#if TAG_MAJOR_VERSION == 34
     ARM_CRYSTAL_PLATE_ARMOUR,
-    ARM_MIN_UNBRANDED = ARM_CRYSTAL_PLATE_ARMOUR,
+#endif
 
     ARM_ANIMAL_SKIN,
 
@@ -54,11 +58,23 @@ enum armour_type
     ARM_SWAMP_DRAGON_ARMOUR,
     ARM_PEARL_DRAGON_HIDE,
     ARM_PEARL_DRAGON_ARMOUR,
-    ARM_MAX_UNBRANDED = ARM_PEARL_DRAGON_ARMOUR,
+#if TAG_MAJOR_VERSION > 34
+    ARM_SHADOW_DRAGON_HIDE,
+    ARM_SHADOW_DRAGON_ARMOUR,
+    ARM_QUICKSILVER_DRAGON_HIDE,
+    ARM_QUICKSILVER_DRAGON_ARMOUR,
+#endif
 
     ARM_CENTAUR_BARDING,
     ARM_NAGA_BARDING,
 
+#if TAG_MAJOR_VERSION == 34
+    ARM_SHADOW_DRAGON_HIDE,
+    ARM_SHADOW_DRAGON_ARMOUR,
+    ARM_QUICKSILVER_DRAGON_HIDE,
+    ARM_QUICKSILVER_DRAGON_ARMOUR,
+#endif
+
     NUM_ARMOURS
 };
 
@@ -138,9 +154,11 @@ enum hands_reqd_type
 
 enum jewellery_type
 {
+#if TAG_MAJOR_VERSION == 34
     RING_REGENERATION,
-    RING_FIRST_RING = RING_REGENERATION,
+#endif
     RING_PROTECTION,
+    RING_FIRST_RING = RING_PROTECTION,
     RING_PROTECTION_FROM_FIRE,
     RING_POISON_RESISTANCE,
     RING_PROTECTION_FROM_COLD,
@@ -151,7 +169,7 @@ enum jewellery_type
     RING_LOUDNESS,
     RING_TELEPORTATION,
     RING_EVASION,
-    RING_SUSTAIN_ABILITIES,
+    RING_SUSTAIN_ATTRIBUTES,
     RING_STEALTH,
     RING_DEXTERITY,
     RING_INTELLIGENCE,
@@ -168,6 +186,7 @@ enum jewellery_type
     // RINGS after num_rings are for unique types for artefacts
     //   (no non-artefact version).
     // Currently none.
+    // XXX: trying to add one doesn't actually work
 
     AMU_RAGE = 35,
     AMU_FIRST_AMULET = AMU_RAGE,
@@ -184,6 +203,7 @@ enum jewellery_type
     AMU_GUARDIAN_SPIRIT,
     AMU_FAITH,
     AMU_STASIS,
+    AMU_REGENERATION,
 
     NUM_JEWELLERY
 };
@@ -220,6 +240,9 @@ enum misc_item_type
 #endif
     MISC_DECK_OF_SUMMONING,
     MISC_DECK_OF_WONDERS,
+#if TAG_MAJOR_VERSION > 34
+    MISC_DECK_OF_ODDITIES,
+#endif
     MISC_DECK_OF_PUNISHMENT,
 
     // mixed decks
@@ -227,16 +250,46 @@ enum misc_item_type
     MISC_DECK_OF_CHANGES,
     MISC_DECK_OF_DEFENCE,
 
+#if TAG_MAJOR_VERSION == 34
     MISC_RUNE_OF_ZOT,
+#endif
 
     MISC_QUAD_DAMAGE, // Sprint only
 
     MISC_PHIAL_OF_FLOODS,
     MISC_SACK_OF_SPIDERS,
 
-    NUM_MISCELLANY, // mv: used for random generation
-    MISC_FIRST_DECK = MISC_DECK_OF_ESCAPE,
-    MISC_LAST_DECK  = MISC_DECK_OF_DEFENCE,
+    MISC_PHANTOM_MIRROR,
+#if TAG_MAJOR_VERSION == 34
+    MISC_DECK_OF_ODDITIES,
+#endif
+
+    NUM_MISCELLANY,
+    MISC_DECK_UNKNOWN = NUM_MISCELLANY,
+};
+
+// in no particular order (but we need *a* fixed order for dbg-scan)
+const vector<misc_item_type> deck_types =
+{
+    MISC_DECK_OF_ESCAPE, MISC_DECK_OF_DESTRUCTION, MISC_DECK_OF_SUMMONING,
+    MISC_DECK_OF_WONDERS, MISC_DECK_OF_ODDITIES, MISC_DECK_OF_PUNISHMENT,
+    MISC_DECK_OF_WAR, MISC_DECK_OF_CHANGES, MISC_DECK_OF_DEFENCE,
+#if TAG_MAJOR_VERSION == 34
+    MISC_DECK_OF_DUNGEONS,
+#endif
+};
+
+// in no particular order (but we need *a* fixed order for dbg-scan)
+const vector<misc_item_type> misc_types =
+{
+    MISC_FAN_OF_GALES, MISC_LAMP_OF_FIRE, MISC_STONE_OF_TREMORS,
+    MISC_LANTERN_OF_SHADOWS, MISC_HORN_OF_GERYON, MISC_BOX_OF_BEASTS,
+    MISC_CRYSTAL_BALL_OF_ENERGY, MISC_DISC_OF_STORMS, MISC_PHIAL_OF_FLOODS,
+    MISC_QUAD_DAMAGE, MISC_SACK_OF_SPIDERS,
+    MISC_PHANTOM_MIRROR,
+#if TAG_MAJOR_VERSION == 34
+    MISC_BOTTLED_EFREET, MISC_BUGGY_EBONY_CASKET,
+#endif
 };
 
 enum missile_type
@@ -285,7 +338,7 @@ enum rune_type
     RUNE_GLOORX_VLOQ,
 
     RUNE_SPIDER,
-    RUNE_FOREST,
+    RUNE_FOREST, // unused
     NUM_RUNE_TYPES
 };
 
@@ -334,7 +387,7 @@ enum special_armour_type
     SPARM_COLD_RESISTANCE,
     SPARM_POISON_RESISTANCE,
     SPARM_SEE_INVISIBLE,
-    SPARM_DARKNESS,
+    SPARM_INVISIBILITY,
     SPARM_STRENGTH,
     SPARM_DEXTERITY,
     SPARM_INTELLIGENCE,
@@ -352,7 +405,9 @@ enum special_armour_type
     SPARM_REFLECTION,
     SPARM_SPIRIT_SHIELD,
     SPARM_ARCHERY,
+#if TAG_MAJOR_VERSION == 34
     SPARM_JUMPING,
+#endif
     NUM_REAL_SPECIAL_ARMOURS,
     NUM_SPECIAL_ARMOURS,
 };
@@ -426,7 +481,7 @@ enum rod_type
     ROD_WARDING,
 #endif
     ROD_SHADOWS,
-    ROD_STRIKING,
+    ROD_IRON,
 #if TAG_MAJOR_VERSION == 34
     ROD_VENOM,
 #endif
@@ -437,7 +492,9 @@ enum weapon_type
 {
     WPN_CLUB,
     WPN_WHIP,
+#if TAG_MAJOR_VERSION == 34
     WPN_HAMMER,
+#endif
     WPN_MACE,
     WPN_FLAIL,
     WPN_MORNINGSTAR,
@@ -449,7 +506,10 @@ enum weapon_type
     WPN_DAGGER,
     WPN_QUICK_BLADE,
     WPN_SHORT_SWORD,
+    WPN_RAPIER,
+#if TAG_MAJOR_VERSION > 34
     WPN_CUTLASS,
+#endif
 
     WPN_FALCHION,
     WPN_LONG_SWORD,
@@ -491,8 +551,8 @@ enum weapon_type
     WPN_GIANT_SPIKED_CLUB,
 
     WPN_DEMON_BLADE,
-    WPN_BASTARD_SWORD,
-    WPN_CLAYMORE,
+    WPN_DOUBLE_SWORD,
+    WPN_TRIPLE_SWORD,
 
     WPN_DEMON_TRIDENT,
     WPN_SCYTHE,
@@ -503,15 +563,17 @@ enum weapon_type
 
 #if TAG_MAJOR_VERSION == 34
     WPN_HUNTING_SLING,
-#endif
 
     WPN_BLESSED_FALCHION,
     WPN_BLESSED_LONG_SWORD,
     WPN_BLESSED_SCIMITAR,
     WPN_BLESSED_GREAT_SWORD,
+#endif
     WPN_EUDEMON_BLADE,
-    WPN_BLESSED_BASTARD_SWORD,
-    WPN_BLESSED_CLAYMORE,
+#if TAG_MAJOR_VERSION == 34
+    WPN_BLESSED_DOUBLE_SWORD,
+    WPN_BLESSED_TRIPLE_SWORD,
+#endif
     WPN_SACRED_SCOURGE,
     WPN_TRISHULA,
 
@@ -519,6 +581,8 @@ enum weapon_type
     WPN_GREATSLING,
     WPN_HAND_CROSSBOW,
     WPN_TRIPLE_CROSSBOW,
+
+    WPN_CUTLASS,
 #endif
 
     NUM_WEAPONS,
@@ -543,16 +607,16 @@ enum weapon_property_type
 
 enum vorpal_damage_type
 {
-    // These are the types of damage a weapon can do.  You can set more
+    // These are the types of damage a weapon can do. You can set more
     // than one of these.
     DAM_BASH            = 0x0000,       // non-melee weapon blugeoning
     DAM_BLUDGEON        = 0x0001,       // crushing
     DAM_SLICE           = 0x0002,       // slicing/chopping
     DAM_PIERCE          = 0x0004,       // stabbing/piercing
-    DAM_WHIP            = 0x0008,       // whip slashing (no butcher)
+    DAM_WHIP            = 0x0008,       // whip slashing
     DAM_MAX_TYPE        = DAM_WHIP,
 
-    // These are used for vorpal weapon descriptions.  You shouldn't set
+    // These are used for vorpal weapon descriptions. You shouldn't set
     // more than one of these.
     DVORP_NONE          = 0x0000,       // used for non-melee weapons
     DVORP_CRUSHING      = 0x1000,
@@ -560,10 +624,9 @@ enum vorpal_damage_type
     DVORP_PIERCING      = 0x3000,
     DVORP_CHOPPING      = 0x4000,       // used for axes
     DVORP_SLASHING      = 0x5000,       // used for whips
-    DVORP_STABBING      = 0x6000,       // used for knives/daggers
 
-    DVORP_CLAWING       = 0x7000,       // claw damage
-    DVORP_TENTACLE      = 0x8000,       // tentacle damage
+    DVORP_CLAWING       = 0x6000,       // claw damage
+    DVORP_TENTACLE      = 0x7000,       // tentacle damage
 
     // These are shortcuts to tie vorpal/damage types for easy setting...
     // as above, setting more than one vorpal type is trouble.
@@ -573,7 +636,6 @@ enum vorpal_damage_type
     DAMV_PIERCING       = DVORP_PIERCING | DAM_PIERCE,
     DAMV_CHOPPING       = DVORP_CHOPPING | DAM_SLICE,
     DAMV_SLASHING       = DVORP_SLASHING | DAM_WHIP,
-    DAMV_STABBING       = DVORP_STABBING | DAM_PIERCE,
 
     DAM_MASK            = 0x0fff,       // strips vorpal specification
     DAMV_MASK           = 0xf000,       // strips non-vorpal specification
