@@ -6,8 +6,6 @@
 #ifndef ITEMS_H
 #define ITEMS_H
 
-#include "externs.h"
-
 // Ways to get items, other than finding them on the ground or looting them
 // from slain monsters.
 enum item_source_type
@@ -28,6 +26,9 @@ enum item_source_type
     AQ_WIZMODE    = 200,
 };
 
+/// top-priority item override colour
+#define FORCED_ITEM_COLOUR_KEY "forced_item_colour"
+
 int get_max_subtype(object_class_type base_type);
 bool item_type_has_unidentified(object_class_type base_type);
 
@@ -41,9 +42,9 @@ bool move_item_to_grid(int *const obj, const coord_def& p,
                         bool silent = false);
 void move_item_stack_to_grid(const coord_def& from, const coord_def& to);
 void note_inscribe_item(item_def &item);
+bool move_item_to_inv(item_def& item);
 bool move_item_to_inv(int obj, int quant_got, bool quiet = false);
-bool merge_items_into_inv(const item_def &it, int quant_got, int &inv_slot,
-                          bool quiet = false);
+item_def* auto_assign_item_slot(item_def& item);
 void mark_items_non_pickup_at(const coord_def &pos);
 void mark_items_non_visit_at(const coord_def &pos);
 void clear_item_pickup_flags(item_def &item);
@@ -53,7 +54,7 @@ bool items_stack(const item_def &item1, const item_def &item2);
 void merge_item_stacks(const item_def &source, item_def &dest, int quant = -1);
 void get_gold(const item_def& item, int quant, bool quiet);
 
-item_def *find_floor_item(object_class_type cls, int sub_type);
+item_def *find_floor_item(object_class_type cls, int sub_type = -1);
 int item_on_floor(const item_def &item, const coord_def& where);
 
 void init_item(int item);
@@ -69,7 +70,7 @@ void destroy_item(item_def &item, bool never_created = false);
 void destroy_item(int dest, bool never_created = false);
 void lose_item_stack(const coord_def& where);
 
-void item_check(bool verbose);
+void item_check();
 void request_autopickup(bool do_pickup = true);
 
 bool player_on_single_stack();
@@ -79,7 +80,7 @@ void pickup(bool partial_quantity = false);
 bool item_is_branded(const item_def& item);
 void item_list_on_square(vector<const item_def*>& items, int obj);
 
-bool copy_item_to_grid(const item_def &item, const coord_def& p,
+bool copy_item_to_grid(item_def &item, const coord_def& p,
                        int quant_drop = -1,    // item.quantity by default
                        bool mark_dropped = false,
                        bool silent = false);
@@ -87,7 +88,7 @@ coord_def item_pos(const item_def &item);
 
 bool move_top_item(const coord_def &src, const coord_def &dest);
 
-// Get the top item in a given cell. If there are no items, return NULL.
+// Get the top item in a given cell. If there are no items, return nullptr.
 const item_def* top_item_at(const coord_def& where);
 
 // Returns whether there is more than one item in a given cell.
@@ -116,10 +117,9 @@ void origin_purchased(item_def &item);
 void origin_acquired(item_def &item, int agent);
 void origin_set_startequip(item_def &item);
 void origin_set_unknown(item_def &item);
-void origin_set_inventory(void (*oset)(item_def &item));
-bool origin_is_god_gift(const item_def& item, god_type *god = NULL);
+bool origin_is_god_gift(const item_def& item, god_type *god = nullptr);
 bool origin_is_acquirement(const item_def& item,
-                           item_source_type *type = NULL);
+                           item_source_type *type = nullptr);
 
 bool item_needs_autopickup(const item_def &);
 bool can_autopickup();
@@ -138,20 +138,16 @@ bool item_is_melded(const item_def& item);
 equipment_type item_equip_slot(const item_def &item);
 
 void item_was_lost(const item_def &item);
-void item_was_destroyed(const item_def &item, int cause = -1);
+void item_was_destroyed(const item_def &item);
 
-bool get_item_by_name(item_def *item, char* specs,
+bool get_item_by_name(item_def *item, const char* specs,
                       object_class_type class_wanted,
                       bool create_for_real = false);
 
 void move_items(const coord_def r, const coord_def p);
 object_class_type get_random_item_mimic_type();
-object_class_type get_item_mimic_type();
-bool is_valid_mimic_item(const item_def &item);
 
 bool maybe_identify_base_type(item_def &item);
-// Returns the Orb's position on the ground, or origin()
-coord_def orb_position();
 int count_movable_items(int obj);
 
 // stack_iterator guarantees validity so long as you don't manually
@@ -166,7 +162,7 @@ public:
     operator bool() const;
     item_def& operator *() const;
     item_def* operator->() const;
-    int link() const;
+    int index() const;
 
     const stack_iterator& operator ++ ();
     stack_iterator operator ++ (int);
