@@ -6,22 +6,24 @@
 #include "AppHdr.h"
 
 #include "kills.h"
-#include "l_libs.h"
 
 #include <algorithm>
 
+#include "clua.h"
 #include "describe.h"
-#include "mon-info.h"
+#include "english.h"
 #include "files.h"
 #include "ghost.h"
 #include "libutil.h"
-#include "monster.h"
+#include "l_libs.h"
 #include "mon-death.h"
-#include "place.h"
-#include "travel.h"
-#include "tags.h"
-#include "clua.h"
+#include "mon-info.h"
+#include "monster.h"
 #include "options.h"
+#include "place.h"
+#include "stringutil.h"
+#include "tags.h"
+#include "travel.h"
 #include "unwind.h"
 #include "viewchar.h"
 
@@ -43,24 +45,11 @@ static const char *kill_category_names[] =
     "others",
 };
 
-KillMaster::KillMaster()
-{
-}
-
-KillMaster::KillMaster(const KillMaster &other)
-{
-    *this = other;
-}
-
-KillMaster::~KillMaster()
-{
-}
-
 const char *KillMaster::category_name(kill_category kc) const
 {
     if (kc >= KC_YOU && kc < KC_NCATEGORIES)
         return kill_category_names[kc];
-    return NULL;
+    return nullptr;
 }
 
 bool KillMaster::empty() const
@@ -154,7 +143,7 @@ string KillMaster::kill_info() const
         add_kill_info(killtext,
                        kills,
                        count,
-                       i == KC_YOU ? NULL
+                       i == KC_YOU ? nullptr
                                    : category_name((kill_category) i),
                        needseparator);
         needseparator = true;
@@ -174,7 +163,7 @@ string KillMaster::kill_info() const
     bool custom = false;
     unwind_var<int> lthrottle(clua.throttle_unit_lines, 500000);
     // Call the kill dump Lua function with null a, to tell it we're done.
-    if (!clua.callfn("c_kill_list", "ss>b", NULL, grandt.c_str(), &custom)
+    if (!clua.callfn("c_kill_list", "ss>b", nullptr, grandt.c_str(), &custom)
         || !custom)
 #endif
     {
@@ -233,18 +222,11 @@ void KillMaster::add_kill_info(string &killtext,
 
         killtext += "\n";
 
-        for (int i = 0, sz = kills.size(); i < sz; ++i)
-        {
-            killtext += "  " + kills[i].desc;
-            killtext += "\n";
-        }
-        {
-            char numbuf[100];
-            snprintf(numbuf, sizeof numbuf,
-                    "%d creature%s vanquished." "\n", count,
-                    count > 1? "s" : "");
-            killtext += numbuf;
-        }
+        for (const kill_exp &kill : kills)
+            killtext += "  " + kill.desc + "\n";
+
+        killtext += make_stringf("%d creature%s vanquished.\n",
+                                 count, count == 1 ? "" : "s");
     }
 #ifdef CLUA_BINDINGS
     lua_pop(clua, 1);
@@ -291,12 +273,11 @@ void Kills::merge(const Kills &k)
     ghosts.insert(ghosts.end(), k.ghosts.begin(), k.ghosts.end());
 
     // Regular kills are messier to merge.
-    for (kill_map::const_iterator i = k.kills.begin();
-            i != k.kills.end(); ++i)
+    for (const auto &entry : k.kills)
     {
-        const kill_monster_desc &kmd = i->first;
+        const kill_monster_desc &kmd = entry.first;
         kill_def &ki = kills[kmd];
-        const kill_def &ko = i->second;
+        const kill_def &ko = entry.second;
         bool uniq = mons_is_unique(kmd.monnum);
         ki.merge(ko, uniq);
     }
@@ -318,7 +299,7 @@ void Kills::record_kill(const monster* mon)
 
     kill_def &k = kills[descriptor];
     if (k.kills)
-        k.add_kill(mon, get_packed_place());
+        k.add_kill(mon, level_id::current());
     else
         k = kill_def(mon);
 }
@@ -326,18 +307,16 @@ void Kills::record_kill(const monster* mon)
 int Kills::get_kills(vector<kill_exp> &all_kills) const
 {
     int count = 0;
-    kill_map::const_iterator iter = kills.begin();
-    for (; iter != kills.end(); ++iter)
+    for (const auto &entry : kills)
     {
-        const kill_monster_desc &md = iter->first;
-        const kill_def &k = iter->second;
-        all_kills.push_back(kill_exp(k, md));
+        const kill_monster_desc &md = entry.first;
+        const kill_def &k = entry.second;
+        all_kills.emplace_back(k, md);
         count += k.kills;
     }
 
-    ghost_vec::const_iterator gi = ghosts.begin();
-    for (; gi != ghosts.end(); ++gi)
-        all_kills.push_back(kill_exp(*gi));
+    for (const kill_ghost &gh : ghosts)
+        all_kills.emplace_back(gh);
     count += ghosts.size();
 
     sort(all_kills.begin(), all_kills.end());
@@ -349,20 +328,16 @@ void Kills::save(writer& outf) const
     // How many kill records do we have?
     marshallInt(outf, kills.size());
 
-    for (kill_map::const_iterator iter = kills.begin();
-          iter != kills.end(); ++iter)
+    for (const auto &entry : kills)
     {
-        iter->first.save(outf);
-        iter->second.save(outf);
+        entry.first.save(outf);
+        entry.second.save(outf);
     }
 
     // How many ghosts do we have?
     marshallShort(outf, ghosts.size());
-    for (ghost_vec::const_iterator iter = ghosts.begin();
-         iter != ghosts.end(); ++iter)
-    {
-        iter->save(outf);
-    }
+    for (const auto &ghost : ghosts)
+        ghost.save(outf);
 }
 
 void Kills::load(reader& inf)
@@ -392,8 +367,7 @@ void Kills::record_ghost_kill(const monster* mon)
     // We should never get to this point, but just in case... {due}
     if (mon->is_summoned())
         return;
-    kill_ghost ghostk(mon);
-    ghosts.push_back(ghostk);
+    ghosts.emplace_back(mon);
 }
 
 int Kills::num_kills(const monster* mon) const
@@ -410,7 +384,7 @@ int Kills::num_kills(const monster_info& mon) const
 
 int Kills::num_kills(kill_monster_desc desc) const
 {
-    kill_map::const_iterator iter = kills.find(desc);
+    auto iter = kills.find(desc);
     int total = (iter == kills.end() ? 0 : iter->second.kills);
 
     if (desc.modifier == kill_monster_desc::M_SHAPESHIFTER)
@@ -426,23 +400,9 @@ int Kills::num_kills(kill_monster_desc desc) const
 kill_def::kill_def(const monster* mon) : kills(0), exp(0)
 {
     exp = exper_value(mon);
-    add_kill(mon, get_packed_place());
+    add_kill(mon, level_id::current());
 }
 
-// For monster names ending with these suffixes, we pluralise directly without
-// attempting to use the "of" rule. For instance:
-//
-//      moth of wrath           => moths of wrath but
-//      moth of wrath zombie    => moth of wrath zombies.
-//
-// This is not necessary right now, since there are currently no monsters that
-// require this special treatment (no monster with 'of' in its name is eligible
-// for zombies or skeletons).
-static const char *modifier_suffixes[] =
-{
-    "zombie", "skeleton", "simulacrum", NULL,
-};
-
 // For a non-unique monster, prefixes a suitable article if we have only one
 // kill, else prefixes a kill count and pluralises the monster name.
 static string n_names(const string &name, int n)
@@ -451,8 +411,7 @@ static string n_names(const string &name, int n)
     {
         char buf[20];
         snprintf(buf, sizeof buf, "%d ", n);
-        return buf + pluralise(name, standard_plural_qualifiers,
-                               modifier_suffixes);
+        return buf + pluralise_monster(name);
     }
     else
         return article_a(name, false);
@@ -489,22 +448,22 @@ void kill_def::merge(const kill_def &k, bool uniq)
     else
     {
         kills += k.kills;
-        for (int i = 0, size = k.places.size(); i < size; ++i)
-            add_place(k.places[i], uniq);
+        for (level_id lvl : k.places)
+            add_place(lvl, uniq);
     }
 }
 
-void kill_def::add_kill(const monster* mon, unsigned short place)
+void kill_def::add_kill(const monster* mon, level_id place)
 {
     kills++;
     // They're only unique if they aren't summoned.
     add_place(place, mons_is_unique(mon->type) && !mon->is_summoned());
 }
 
-void kill_def::add_place(unsigned short place, bool force)
+void kill_def::add_place(level_id place, bool force)
 {
-    for (unsigned i = 0; i < places.size(); ++i)
-        if (places[i] == place) return;
+    if (find(begin(places), end(places), place) != end(places))
+        return; // Already there
 
     if (force || places.size() < PLACE_LIMIT)
         places.push_back(place);
@@ -575,18 +534,17 @@ string kill_def::append_places(const kill_monster_desc &md,
 {
     if (Options.dump_kill_places == KDO_NO_PLACES) return name;
 
-    int nplaces = places.size();
+    size_t nplaces = places.size();
     if (nplaces == 1 || mons_is_unique(md.monnum)
             || Options.dump_kill_places == KDO_ALL_PLACES)
     {
         string augmented = name;
         augmented += " (";
-        for (vector<unsigned short>::const_iterator iter = places.begin();
-             iter != places.end(); ++iter)
+        for (auto iter = places.begin(); iter != places.end(); ++iter)
         {
             if (iter != places.begin())
                 augmented += " ";
-            augmented += short_place_name(*iter);
+            augmented += iter->describe();
         }
         augmented += ")";
         return augmented;
@@ -600,11 +558,8 @@ void kill_def::save(writer& outf) const
     marshallShort(outf, exp);
 
     marshallShort(outf, places.size());
-    for (vector<unsigned short>::const_iterator iter = places.begin();
-         iter != places.end(); ++iter)
-    {
-        marshallShort(outf, *iter);
-    }
+    for (auto lvl : places)
+        lvl.save(outf);
 }
 
 void kill_def::load(reader& inf)
@@ -615,13 +570,29 @@ void kill_def::load(reader& inf)
     places.clear();
     short place_count = unmarshallShort(inf);
     for (short i = 0; i < place_count; ++i)
-        places.push_back((unsigned short) unmarshallShort(inf));
+    {
+#if TAG_MAJOR_VERSION == 34
+        if (inf.getMinorVersion() < TAG_MINOR_PLACE_UNPACK)
+        {
+            places.push_back(level_id::from_packed_place(
+                                (unsigned short) unmarshallShort(inf)));
+        }
+        else
+        {
+#endif
+        level_id tmp;
+        tmp.load(inf);
+        places.push_back(tmp);
+#if TAG_MAJOR_VERSION == 34
+        }
+#endif
+    }
 }
 
 kill_ghost::kill_ghost(const monster* mon)
 {
     exp = exper_value(mon);
-    place = get_packed_place();
+    place = level_id::current();
     ghost_name = mon->ghost->name;
 
     // Check whether this is really a ghost, since we also have to handle
@@ -637,21 +608,27 @@ string kill_ghost::info() const
 {
     return ghost_name
            + (Options.dump_kill_places != KDO_NO_PLACES ?
-                " (" + short_place_name(place) + ")" : string(""));
+                " (" + place.describe() + ")" :
+                string(""));
 }
 
 void kill_ghost::save(writer& outf) const
 {
     marshallString4(outf, ghost_name);
     marshallShort(outf, (unsigned short) exp);
-    marshallShort(outf, place);
+    place.save(outf);
 }
 
 void kill_ghost::load(reader& inf)
 {
     unmarshallString4(inf, ghost_name);
     exp = unmarshallShort(inf);
-    place = (unsigned short) unmarshallShort(inf);
+#if TAG_MAJOR_VERSION == 34
+    if (inf.getMinorVersion() < TAG_MINOR_PLACE_UNPACK)
+        place = level_id::from_packed_place((unsigned short) unmarshallShort(inf));
+    else
+#endif
+    place.load(inf);
 }
 
 kill_monster_desc::kill_monster_desc(const monster* mon)
@@ -684,7 +661,7 @@ kill_monster_desc::kill_monster_desc(const monster* mon)
         default: break;
     }
     if (modifier != M_NORMAL)
-        monnum = mon->base_monster;
+        monnum = mons_species(mon->base_monster);
 
     if (mon->is_shapeshifter())
         modifier = M_SHAPESHIFTER;
@@ -806,36 +783,6 @@ static int kill_lualc_modifier(lua_State *ls)
     return 0;
 }
 
-static int kill_lualc_places(lua_State *ls)
-{
-    if (!lua_islightuserdata(ls, 1))
-    {
-        luaL_argerror(ls, 1, "Unexpected argument type");
-        return 0;
-    }
-
-    kill_exp *ke = static_cast<kill_exp*>(lua_touserdata(ls, 1));
-    if (ke)
-    {
-        lua_newtable(ls);
-        for (int i = 0, count = ke->places.size(); i < count; ++i)
-        {
-            lua_pushnumber(ls, ke->places[i]);
-            lua_rawseti(ls, -2, i + 1);
-        }
-        return 1;
-    }
-    return 0;
-}
-
-static int kill_lualc_place_name(lua_State *ls)
-{
-    int num = luaL_checkint(ls, 1);
-    string plname = short_place_name(num);
-    lua_pushstring(ls, plname.c_str());
-    return 1;
-}
-
 static int kill_lualc_isunique(lua_State *ls)
 {
     if (!lua_islightuserdata(ls, 1))
@@ -1025,8 +972,6 @@ static const struct luaL_reg kill_lib[] =
     { "desc",       kill_lualc_desc },
     { "monnum",     kill_lualc_monnum },
     { "modifier",   kill_lualc_modifier },
-    { "places",     kill_lualc_places },
-    { "place_name", kill_lualc_place_name },
     { "holiness",   kill_lualc_holiness },
     { "symbol",     kill_lualc_symbol },
     { "isghost",    kill_lualc_isghost },
@@ -1035,7 +980,7 @@ static const struct luaL_reg kill_lib[] =
     { "rawwrite",   kill_lualc_rawwrite },
     { "write",      kill_lualc_write },
     { "summary",    kill_lualc_summary },
-    { NULL, NULL }
+    { nullptr, nullptr }
 };
 
 void cluaopen_kills(lua_State *ls)
