@@ -1,32 +1,29 @@
 #include "AppHdr.h"
 
 #include "l_libs.h"
-#include "cluautil.h"
 
 #include <cmath>
 
 #include "branch.h"
 #include "cloud.h"
+#include "cluautil.h"
 #include "colour.h"
 #include "coord.h"
 #include "coordit.h"
+#include "dgn-shoals.h"
 #include "directn.h"
 #include "dungeon.h"
-#include "dgn-shoals.h"
-#include "env.h"
 #include "flood_find.h"
 #include "l_defs.h"
 #include "libutil.h"
 #include "mapmark.h"
 #include "maps.h"
-#include "random.h"
 #include "shout.h"
 #include "spl-util.h"
 #include "state.h"
 #include "stringutil.h"
 #include "tiledef-dngn.h"
 #include "tileview.h"
-#include "unwind.h"
 #include "view.h"
 
 static const char *VAULT_PLACEMENT_METATABLE = "crawl.vault-placement";
@@ -177,52 +174,6 @@ static int dgn_tags_remove(lua_State *ls)
     PLUARET(string, map->tags.c_str());
 }
 
-static const string level_flag_names[] =
-{"no_tele_control", "not_mappable", "no_magic_map", ""};
-
-static int dgn_lflags(lua_State *ls)
-{
-    MAP(ls, 1, map);
-
-    try
-    {
-        map->level_flags = map_flags::parse(level_flag_names,
-                                            luaL_checkstring(ls, 2));
-    }
-    catch (const string &error)
-    {
-        luaL_argerror(ls, 2, error.c_str());
-    }
-
-    return 0;
-}
-
-static int dgn_change_level_flags(lua_State *ls)
-{
-    map_flags flags;
-
-    try
-    {
-        flags = map_flags::parse(level_flag_names,
-                                 luaL_checkstring(ls, 1));
-    }
-    catch (const string &error)
-    {
-        luaL_argerror(ls, 2, error.c_str());
-        lua_pushboolean(ls, false);
-        return 1;
-    }
-
-    bool silent = lua_toboolean(ls, 2);
-
-    bool changed1 = set_level_flags(flags.flags_set, silent);
-    bool changed2 = unset_level_flags(flags.flags_unset, silent);
-
-    lua_pushboolean(ls, changed1 || changed2);
-
-    return 1;
-}
-
 static void _chance_magnitude_check(lua_State *ls, int which_par, int chance)
 {
     if (chance < 0 || chance > CHANCE_ROLL)
@@ -698,8 +649,8 @@ static int dgn_gly_points(lua_State *ls)
     MAP(ls, 1, map);
     vector<coord_def> cs = map->find_glyph(*luaL_checkstring(ls, 2));
 
-    for (int i = 0, size = cs.size(); i < size; ++i)
-        dlua_push_coordinates(ls, cs[i]);
+    for (coord_def c : cs)
+        dlua_push_coordinates(ls, c);
     return cs.size() * 2;
 }
 
@@ -894,7 +845,7 @@ static int dgn_num_matching_markers(lua_State *ls)
     const char* val_ptr = lua_tostring(ls, 2);
     const char* val;
 
-    if (val_ptr == NULL)
+    if (val_ptr == nullptr)
         val = "";
     else
         val = val_ptr;
@@ -999,7 +950,7 @@ static int lua_dgn_set_branch_epilogue(lua_State *ls)
     if (!branch_name)
         return 0;
 
-    branch_type br = str_to_branch(branch_name);
+    branch_type br = branch_by_abbrevname(branch_name);
     if (br == NUM_BRANCHES)
     {
         luaL_error(ls, make_stringf("unknown branch: '%s'.", branch_name).c_str());
@@ -1035,7 +986,7 @@ static int lua_dgn_set_border_fill_type(lua_State *ls)
         return 0;
     }
 
-    if (is_valid_border_feat(fill_type))
+    if (feat_is_valid_border(fill_type))
         map->border_fill_type = fill_type;
     else
         luaL_error(ls, ("set_border_fill_type cannot be the feature '" +
@@ -1143,7 +1094,7 @@ static int dgn_random_walk(lua_State *ls)
     // Fourth param being true means that we can move past
     // statues.
     const dungeon_feature_type minmove =
-    lua_isnil(ls, 4) ? DNGN_MINMOVE : DNGN_ORCISH_IDOL;
+    lua_isnil(ls, 4) ? DNGN_LAVA : DNGN_ORCISH_IDOL;
 
     if (!in_bounds(x, y))
     {
@@ -1198,22 +1149,6 @@ static int dgn_random_walk(lua_State *ls)
     return 2;
 }
 
-static cloud_type dgn_cloud_name_to_type(string name)
-{
-    lowercase(name);
-
-    if (name == "random")
-        return CLOUD_RANDOM;
-    else if (name == "debugging")
-        return CLOUD_DEBUGGING;
-
-    for (int i = CLOUD_NONE; i < CLOUD_RANDOM; i++)
-        if (cloud_type_name(static_cast<cloud_type>(i)) == name)
-            return static_cast<cloud_type>(i);
-
-    return CLOUD_NONE;
-}
-
 static kill_category dgn_kill_name_to_category(string name)
 {
     if (name.empty())
@@ -1235,12 +1170,10 @@ static int lua_cloud_pow_min;
 static int lua_cloud_pow_max;
 static int lua_cloud_pow_rolls;
 
-static int make_a_lua_cloud(coord_def where, int garbage, int spread_rate,
+static int make_a_lua_cloud(coord_def where, int /*garbage*/, int spread_rate,
                             cloud_type ctype, const actor *agent, int colour,
                             string name, string tile, int excl_rad)
 {
-    UNUSED(garbage);
-
     const int pow = random_range(lua_cloud_pow_min,
                                  lua_cloud_pow_max,
                                  lua_cloud_pow_rolls);
@@ -1257,7 +1190,7 @@ static int dgn_apply_area_cloud(lua_State *ls)
     lua_cloud_pow_rolls = luaL_checkint(ls, 5);
     const int size      = luaL_checkint(ls, 6);
 
-    const cloud_type ctype = dgn_cloud_name_to_type(luaL_checkstring(ls, 7));
+    const cloud_type ctype = cloud_name_to_type(luaL_checkstring(ls, 7));
     const char*      kname = lua_isstring(ls, 8) ? luaL_checkstring(ls, 8)
     : "";
     const kill_category kc = dgn_kill_name_to_category(kname);
@@ -1353,7 +1286,7 @@ static int dgn_place_cloud(lua_State *ls)
 {
     const int x         = luaL_checkint(ls, 1);
     const int y         = luaL_checkint(ls, 2);
-    const cloud_type ctype = dgn_cloud_name_to_type(luaL_checkstring(ls, 3));
+    const cloud_type ctype = cloud_name_to_type(luaL_checkstring(ls, 3));
     const int cl_range      = luaL_checkint(ls, 4);
     const char*      kname = lua_isstring(ls, 5) ? luaL_checkstring(ls, 5)
     : "";
@@ -1505,7 +1438,7 @@ LUAFN(dgn_with_map_bounds_fn)
         // happen when lua_call does its longjmp.
         err = lua_pcall(ls, 0, 1, 0);
 
-        _dgn_map_safe_bounds_fn.reset(NULL);
+        _dgn_map_safe_bounds_fn.reset(nullptr);
     }
 
     if (err)
@@ -1526,13 +1459,12 @@ LUAFN(dgn_with_map_anchors)
 
         map_anchor_points.clear();
 
-        int i;
-        for (i = 1; i < top; i += 2)
+        for (int i = 1; i < top; i += 2)
         {
             if (lua_isnumber(ls, i) && lua_isnumber(ls, i + 1))
             {
-                map_anchor_points.push_back(coord_def(lua_tointeger(ls, i),
-                                                      lua_tointeger(ls, i + 1)));
+                map_anchor_points.emplace_back(lua_tointeger(ls, i),
+                                               lua_tointeger(ls, i + 1));
             }
         }
 
@@ -1605,8 +1537,7 @@ LUAFN(_dgn_place_map)
         if (dgn_place_map(map, check_collision, no_exits, where)
             && !env.level_vaults.empty())
         {
-            lua_pushlightuserdata(ls,
-                                  env.level_vaults[env.level_vaults.size() - 1]);
+            lua_pushlightuserdata(ls, env.level_vaults.back().get());
         }
         else
             lua_pushnil(ls);
@@ -1648,9 +1579,15 @@ static int _dgn_push_vault_placement(lua_State *ls, const vault_placement *vp)
     return dlua_push_object_type(ls, VAULT_PLACEMENT_METATABLE, *vp);
 }
 
+static int _dgn_push_vault_placement_uptr(lua_State *ls,
+                                          const unique_ptr<vault_placement> &vp)
+{
+    return _dgn_push_vault_placement(ls, vp.get());
+}
+
 LUAFN(_dgn_maps_used_here)
 {
-    return clua_gentable(ls, env.level_vaults, _dgn_push_vault_placement);
+    return clua_gentable(ls, env.level_vaults, _dgn_push_vault_placement_uptr);
 }
 
 LUAFN(_dgn_vault_at)
@@ -1716,7 +1653,7 @@ LUAFN(_dgn_marker_at_pos)
 
     map_marker* marker = env.markers.find(p);
 
-    if (marker == NULL)
+    if (marker == nullptr)
         lua_pushnil(ls);
     else
         _push_mapmarker(ls, marker);
@@ -1744,9 +1681,8 @@ LUAFN(_dgn_resolve_map)
 
     // Save the vault_placement into Temp_Vaults because the map_def
     // will need to be alive through to the end of dungeon gen.
-    Temp_Vaults.push_back(vault_placement());
-
-    vault_placement &place(Temp_Vaults[Temp_Vaults.size() - 1]);
+    Temp_Vaults.emplace_back();
+    vault_placement &place(Temp_Vaults.back());
 
     if (vault_main(place, map, check_collisions) != MAP_NONE)
     {
@@ -1868,7 +1804,6 @@ const struct luaL_reg dgn_dlib[] =
 { "tags",  dgn_tags },
 { "has_tag", dgn_has_tag },
 { "tags_remove", dgn_tags_remove },
-{ "lflags", dgn_lflags },
 { "chance", dgn_chance },
 { "depth_chance", dgn_depth_chance },
 { "weight", dgn_weight },
@@ -1913,7 +1848,6 @@ const struct luaL_reg dgn_dlib[] =
 { "remove_listener", dgn_remove_listener },
 { "remove_marker", dgn_remove_marker },
 { "num_matching_markers", dgn_num_matching_markers },
-{ "change_level_flags", dgn_change_level_flags },
 { "get_floor_colour", dgn_get_floor_colour },
 { "get_rock_colour",  dgn_get_rock_colour },
 { "change_floor_colour", dgn_change_floor_colour },
@@ -1966,7 +1900,7 @@ const struct luaL_reg dgn_dlib[] =
 
 { "apply_tide", dgn_apply_tide },
 
-{ NULL, NULL }
+{ nullptr, nullptr }
 };
 
 #define VP(name) \
@@ -2014,7 +1948,7 @@ static const luaL_reg dgn_vaultplacement_ops[] =
     { "orient", _vp_orient },
     { "map", _vp_map },
     { "exits", _vp_exits },
-    { NULL, NULL }
+    { nullptr, nullptr }
 };
 
 static void _dgn_register_metatables(lua_State *ls)
