@@ -1,12 +1,11 @@
 #include "AppHdr.h"
 
-#include "cluautil.h"
 #include "l_libs.h"
 
+#include "butcher.h"
+#include "cluautil.h"
 #include "food.h"
-#include "invent.h"
 #include "itemprop.h"
-#include "items.h"
 #include "player.h"
 
 /////////////////////////////////////////////////////////////////////
@@ -66,7 +65,9 @@ static int food_can_eat(lua_State *ls)
     if (lua_isboolean(ls, 2))
         hungercheck = lua_toboolean(ls, 2);
 
-    bool edible = item && can_ingest(*item, true, hungercheck);
+    const bool edible = item && (item->base_type == OBJ_FOOD
+                                 || item->base_type == OBJ_CORPSES)
+                             &&  can_eat(*item, true, hungercheck);
     lua_pushboolean(ls, edible);
     return 1;
 }
@@ -76,41 +77,20 @@ static int food_eat(lua_State *ls)
     LUA_ITEM(ls, item, 1);
 
     bool eaten = false;
-    if (!you.turn_is_over)
-    {
-        // Nasty special case: can_ingest() allows potions (why???), so we need
-        // to weed them away here; we wouldn't be able to return success status
-        // otherwise.
-        if (item && can_ingest(*item, false) && item->base_type != OBJ_POTIONS)
-            eaten = eat_item(*item);
-
-    }
+    if (!you.turn_is_over && item && can_eat(*item, false))
+        eaten = eat_item(*item);
     lua_pushboolean(ls, eaten);
     return 1;
 }
 
-static int food_rotting(lua_State *ls)
-{
-    LUA_ITEM(ls, item, 1);
-
-    bool rotting = false;
-    if (item && item->base_type == OBJ_FOOD && item->sub_type == FOOD_CHUNK)
-        rotting = food_is_rotten(*item);
-
-    lua_pushboolean(ls, rotting);
-    return 1;
-}
-
 static int food_dangerous(lua_State *ls)
 {
     LUA_ITEM(ls, item, 1);
 
     bool dangerous = false;
     if (item)
-    {
-        dangerous = (is_poisonous(*item) || is_mutagenic(*item)
-                     || causes_rot(*item) || is_forbidden_food(*item));
-    }
+        dangerous = is_bad_food(*item);
+
     lua_pushboolean(ls, dangerous);
     return 1;
 }
@@ -118,9 +98,7 @@ static int food_dangerous(lua_State *ls)
 static int food_ischunk(lua_State *ls)
 {
     LUA_ITEM(ls, item, 1);
-    lua_pushboolean(ls,
-            item && item->base_type == OBJ_FOOD
-                 && item->sub_type == FOOD_CHUNK);
+    lua_pushboolean(ls, item && item->is_type(OBJ_FOOD, FOOD_CHUNK));
     return 1;
 }
 
@@ -145,6 +123,22 @@ static int food_isveggie(lua_State *ls)
     return 1;
 }
 
+static int food_bottleable(lua_State *ls)
+{
+    LUA_ITEM(ls, item, 1);
+    lua_pushboolean(ls, item && item->is_type(OBJ_CORPSES, CORPSE_BODY)
+                             && can_bottle_blood_from_corpse(item->mon_type));
+    return 1;
+}
+
+// differs from food_can_eat in several respects
+static int food_edible(lua_State *ls)
+{
+    LUA_ITEM(ls, item, 1);
+    lua_pushboolean(ls, item && !is_inedible(*item));
+    return 1;
+}
+
 static const struct luaL_reg food_lib[] =
 {
     { "do_eat",            food_do_eat },
@@ -154,13 +148,14 @@ static const struct luaL_reg food_lib[] =
     { "prompt_inv_menu",   food_prompt_inventory_menu },
     { "can_eat",           food_can_eat },
     { "eat",               food_eat },
-    { "rotting",           food_rotting },
     { "dangerous",         food_dangerous },
     { "ischunk",           food_ischunk },
     { "isfruit",           food_isfruit },
     { "ismeaty",           food_ismeaty },
     { "isveggie",          food_isveggie },
-    { NULL, NULL },
+    { "bottleable",        food_bottleable },
+    { "edible",            food_edible },
+    { nullptr, nullptr },
 };
 
 void cluaopen_food(lua_State *ls)
