@@ -6,13 +6,10 @@
 #include "AppHdr.h"
 
 #include "l_libs.h"
-#include "l_defs.h"
 
 #include "branch.h"
 #include "cluautil.h"
 #include "coord.h"
-#include "exclude.h"
-#include "player.h"
 #include "terrain.h"
 #include "travel.h"
 
@@ -50,10 +47,17 @@ LUAFN(l_feature_is_traversable)
     PLUARET(boolean, feat_is_traversable_now(feat));
 }
 
+LUAFN(l_feature_is_solid)
+{
+    const string &name = luaL_checkstring(ls, 1);
+    const dungeon_feature_type feat = dungeon_feature_by_name(name);
+    PLUARET(boolean, feat_is_solid(feat));
+}
+
 LUAFN(l_find_deepest_explored)
 {
     const string &branch = luaL_checkstring(ls, 1);
-    const level_id lid(str_to_branch(branch), 1);
+    const level_id lid(branch_by_abbrevname(branch), 1);
     if (lid.branch == NUM_BRANCHES)
         luaL_error(ls, "Bad branch name: '%s'", branch.c_str());
     if (!is_known_branch_id(lid.branch))
@@ -66,9 +70,10 @@ static const struct luaL_reg travel_lib[] =
     { "set_exclude", l_set_exclude },
     { "del_exclude", l_del_exclude },
     { "feature_traversable", l_feature_is_traversable },
+    { "feature_solid", l_feature_is_solid },
     { "find_deepest_explored", l_find_deepest_explored },
 
-    { NULL, NULL }
+    { nullptr, nullptr }
 };
 
 void cluaopen_travel(lua_State *ls)
