@@ -1,39 +1,38 @@
 #include "AppHdr.h"
 
-#include "cluautil.h"
 #include "l_libs.h"
 
 #include "ability.h"
 #include "abyss.h"
 #include "areas.h"
+#include "artefact.h"
 #include "branch.h"
 #include "chardump.h"
-#include "coord.h"
+#include "cluautil.h"
 #include "delay.h"
+#include "english.h"
 #include "env.h"
 #include "food.h"
 #include "initfile.h"
 #include "itemname.h"
 #include "itemprop.h"
 #include "items.h"
-#include "libutil.h"
-#include "ng-setup.h"
+#include "jobs.h"
 #include "mapmark.h"
 #include "misc.h"
-#include "mon-util.h"
 #include "mutation.h"
 #include "newgame_def.h"
-#include "jobs.h"
+#include "ng-setup.h"
 #include "ouch.h"
 #include "place.h"
 #include "prompt.h"
 #include "religion.h"
 #include "shopping.h"
-#include "species.h"
 #include "skills.h"
-#include "skills2.h"
+#include "spl-book.h"
 #include "spl-transloc.h"
 #include "spl-util.h"
+#include "status.h"
 #include "stringutil.h"
 #include "transform.h"
 #include "traps.h"
@@ -87,19 +86,13 @@ function evil_god(god) */
 LUARET1(you_evil_god, boolean,
         lua_isstring(ls, 1) ? is_evil_god(str_to_god(lua_tostring(ls, 1)))
         : is_evil_god(you.religion))
-/*
---- Does this [player's] god like fresh corpses?
--- @param god defaults to you.god()
-function god_likes_fresh_corpses(god) */
-LUARET1(you_god_likes_fresh_corpses, boolean,
-        lua_isstring(ls, 1) ?
-        god_likes_fresh_corpses(str_to_god(lua_tostring(ls, 1))) :
-        god_likes_fresh_corpses(you.religion))
 LUARET2(you_hp, number, you.hp, you.hp_max)
 LUARET2(you_mp, number, you.magic_points, you.max_magic_points)
+LUARET1(you_base_mp, number, get_real_mp(false))
+LUARET1(you_rot, number, player_rotted())
 LUARET1(you_poison_survival, number, poison_survival())
 LUARET1(you_corrosion, number, you.props["corrosion_amount"].get_int())
-LUARET1(you_hunger, number, you.hunger_state)
+LUARET1(you_hunger, number, you.hunger_state - 1)
 LUARET1(you_hunger_name, string, hunger_level())
 LUARET2(you_strength, number, you.strength(false), you.max_strength())
 LUARET2(you_intelligence, number, you.intel(false), you.max_intel())
@@ -113,12 +106,14 @@ LUARET1(you_skill_progress, number,
 LUARET1(you_can_train_skill, boolean,
         lua_isstring(ls, 1) ? you.can_train[str_to_skill(lua_tostring(ls, 1))]
                             : false)
+LUARET1(you_best_skill, string,
+        skill_name(best_skill(SK_FIRST_SKILL, SK_LAST_SKILL)))
 LUARET1(you_res_poison, number, player_res_poison(false))
 LUARET1(you_res_fire, number, player_res_fire(false))
 LUARET1(you_res_cold, number, player_res_cold(false))
 LUARET1(you_res_draining, number, player_prot_life(false))
 LUARET1(you_res_shock, number, player_res_electricity(false))
-LUARET1(you_res_statdrain, boolean, player_sust_abil(false))
+LUARET1(you_res_statdrain, boolean, player_sust_attr(false))
 LUARET1(you_res_mutation, number, you.rmut_from_item(false) ? 1 : 0)
 LUARET1(you_see_invisible, boolean, you.can_see_invisible(false))
 // Returning a number so as not to break existing scripts.
@@ -126,8 +121,7 @@ LUARET1(you_spirit_shield, number, you.spirit_shield(false) ? 1 : 0)
 LUARET1(you_gourmand, boolean, you.gourmand(false))
 LUARET1(you_res_corr, boolean, you.res_corr(false))
 LUARET1(you_like_chunks, number, player_likes_chunks(true))
-LUARET1(you_saprovorous, number, player_mutation_level(MUT_SAPROVOROUS))
-LUARET1(you_flying, boolean, you.flight_mode())
+LUARET1(you_flying, boolean, you.airborne())
 LUARET1(you_transform, string, you.form ? transform_name() : "")
 LUARET1(you_berserk, boolean, you.berserk())
 LUARET1(you_confused, boolean, you.confused())
@@ -155,7 +149,6 @@ LUARET1(you_mighty, boolean, you.duration[DUR_MIGHT])
 LUARET1(you_agile, boolean, you.duration[DUR_AGILITY])
 LUARET1(you_brilliant, boolean, you.duration[DUR_BRILLIANCE])
 LUARET1(you_phase_shifted, boolean, you.duration[DUR_PHASE_SHIFT])
-LUARET1(you_rotting, boolean, you.rotting)
 LUARET1(you_silenced, boolean, silenced(you.pos()))
 LUARET1(you_sick, boolean, you.disease)
 LUARET1(you_contaminated, number, get_contamination_level())
@@ -181,6 +174,7 @@ LUARET1(you_taking_stairs, boolean,
         || current_delay_action() == DELAY_DESCENDING_STAIRS)
 LUARET1(you_turns, number, you.num_turns)
 LUARET1(you_time, number, you.elapsed_time)
+LUARET1(you_spell_levels, number, player_spell_levels())
 LUARET1(you_can_smell, boolean, you.can_smell())
 LUARET1(you_has_claws, number, you.has_claws(false))
 
@@ -188,14 +182,25 @@ LUARET1(you_see_cell_rel, boolean,
         you.see_cell(coord_def(luaL_checkint(ls, 1), luaL_checkint(ls, 2)) + you.pos()))
 LUARET1(you_see_cell_no_trans_rel, boolean,
         you.see_cell_no_trans(coord_def(luaL_checkint(ls, 1), luaL_checkint(ls, 2)) + you.pos()))
-LUARET1(you_piety_rank, number, piety_rank(you.piety) - 1)
+LUARET1(you_piety_rank, number, piety_rank())
 LUARET1(you_constricted, boolean, you.is_constricted())
 LUARET1(you_constricting, boolean, you.is_constricting())
 
+static int l_you_monster(lua_State *ls)
+{
+    const monster_type mons = player_species_to_mons_species(you.species);
+
+    string name = mons_type_name(mons, DESC_PLAIN);
+    lowercase(name);
+
+    lua_pushstring(ls, name.c_str());
+    return 1;
+}
+
 static int l_you_genus(lua_State *ls)
 {
     bool plural = lua_toboolean(ls, 1);
-    string genus = species_name(you.species, true);
+    string genus = species_name(you.species, SPNAME_GENUS);
     lowercase(genus);
     if (plural)
         genus = pluralise(genus);
@@ -267,6 +272,27 @@ static int l_you_spell_table(lua_State *ls)
     return 1;
 }
 
+static int l_you_mem_spells(lua_State *ls)
+{
+    lua_newtable(ls);
+
+    char buf[2];
+    buf[1] = 0;
+
+    vector<int> books;
+    vector<spell_type> mem_spells = get_mem_spell_list(books);
+
+    for (size_t i = 0; i < mem_spells.size(); ++i)
+    {
+        buf[0] = index_to_letter(i);
+
+        lua_pushstring(ls, buf);
+        lua_pushstring(ls, spell_title(mem_spells[i]));
+        lua_rawset(ls, -3);
+    }
+    return 1;
+}
+
 static int l_you_abils(lua_State *ls)
 {
     lua_newtable(ls);
@@ -304,12 +330,11 @@ static int l_you_abil_table(lua_State *ls)
     char buf[2];
     buf[1] = 0;
 
-    vector<talent> talents = your_talents(false);
-    for (int i = 0, size = talents.size(); i < size; ++i)
+    for (const talent &tal : your_talents(false))
     {
-        buf[0] = talents[i].hotkey;
+        buf[0] = tal.hotkey;
         lua_pushstring(ls, buf);
-        lua_pushstring(ls, ability_name(talents[i].which));
+        lua_pushstring(ls, ability_name(tal.which));
         lua_rawset(ls, -3);
     }
     return 1;
@@ -333,8 +358,8 @@ static int you_gold(lua_State *ls)
 static int you_can_consume_corpses(lua_State *ls)
 {
     lua_pushboolean(ls,
-                    can_ingest(OBJ_FOOD, FOOD_CHUNK, true, false)
-                    || can_ingest(OBJ_CORPSES, CORPSE_BODY, true, false)
+                     player_mutation_level(MUT_HERBIVOROUS) < 3
+                     && !you_foodless(true)
                   );
     return 1;
 }
@@ -388,6 +413,22 @@ LUAFN(you_mutation)
     return luaL_argerror(ls, 1, err.c_str());
 }
 
+LUAFN(you_temp_mutation)
+{
+    string mutname = luaL_checkstring(ls, 1);
+    for (int i = 0; i < NUM_MUTATIONS; ++i)
+    {
+        const char *wizname = mutation_name(static_cast<mutation_type>(i));
+        if (!wizname)
+            continue;
+        if (!strcmp(mutname.c_str(), wizname))
+            PLUARET(integer, you.temp_mutation[i]);
+    }
+
+    string err = make_stringf("No such mutation: '%s'.", mutname.c_str());
+    return luaL_argerror(ls, 1, err.c_str());
+}
+
 LUAFN(you_is_level_on_stack)
 {
     string levname = luaL_checkstring(ls, 1);
@@ -430,6 +471,36 @@ LUAFN(you_train_skill)
     PLUARET(number, you.train[sk]);
 }
 
+LUAFN(you_status)
+{
+    const char* which = nullptr;
+    if (lua_gettop(ls) >= 1)
+        which = luaL_checkstring(ls, 1);
+
+    string status_effects = "";
+    status_info inf;
+    for (unsigned i = 0; i <= STATUS_LAST_STATUS; ++i)
+    {
+        if (fill_status_info(i, &inf) && !inf.short_text.empty())
+        {
+            if (which)
+            {
+                if (inf.short_text == which)
+                    PLUARET(boolean, true);
+            }
+            else
+            {
+                if (!status_effects.empty())
+                    status_effects += ",";
+                status_effects += inf.short_text;
+            }
+        }
+    }
+    if (which)
+        PLUARET(boolean, false);
+    PLUARET(string, status_effects.c_str());
+}
+
 static const struct luaL_reg you_clib[] =
 {
     { "turn_is_over", you_turn_is_over },
@@ -438,6 +509,8 @@ static const struct luaL_reg you_clib[] =
     { "spells"      , l_you_spells },
     { "spell_letters", l_you_spell_letters },
     { "spell_table" , l_you_spell_table },
+    { "spell_levels", you_spell_levels },
+    { "mem_spells", l_you_mem_spells },
     { "abilities"   , l_you_abils },
     { "ability_letters", l_you_abil_letters },
     { "ability_table", l_you_abil_table },
@@ -445,6 +518,7 @@ static const struct luaL_reg you_clib[] =
     { "race"        , you_race },
     { "class"       , you_class },
     { "genus"       , l_you_genus },
+    { "monster"     , l_you_monster },
     { "wizard"      , you_wizard },
     { "god"         , you_god },
     { "gold"        , you_gold },
@@ -452,6 +526,8 @@ static const struct luaL_reg you_clib[] =
     { "evil_god"    , you_evil_god },
     { "hp"          , you_hp },
     { "mp"          , you_mp },
+    { "base_mp"     , you_base_mp },
+    { "rot"         , you_rot },
     { "hunger"      , you_hunger },
     { "hunger_name" , you_hunger_name },
     { "strength"    , you_strength },
@@ -461,7 +537,8 @@ static const struct luaL_reg you_clib[] =
     { "base_skill"  , you_base_skill },
     { "skill_progress", you_skill_progress },
     { "can_train_skill", you_can_train_skill },
-    { "train_skill", you_train_skill },
+    { "best_skill",   you_best_skill },
+    { "train_skill",  you_train_skill },
     { "xl"          , you_xl },
     { "xl_progress" , you_xl_progress },
     { "res_poison"  , you_res_poison },
@@ -473,7 +550,6 @@ static const struct luaL_reg you_clib[] =
     { "res_mutation", you_res_mutation },
     { "see_invisible", you_see_invisible },
     { "spirit_shield", you_spirit_shield },
-    { "saprovorous",  you_saprovorous },
     { "like_chunks",  you_like_chunks },
     { "gourmand",     you_gourmand },
     { "res_corr",     you_res_corr },
@@ -507,7 +583,6 @@ static const struct luaL_reg you_clib[] =
     { "mighty",       you_mighty },
     { "agile",        you_agile },
     { "brilliant",    you_brilliant },
-    { "rotting",      you_rotting },
     { "silenced",     you_silenced },
     { "sick",         you_sick },
     { "contaminated", you_contaminated },
@@ -518,8 +593,8 @@ static const struct luaL_reg you_clib[] =
     { "constricted",  you_constricted },
     { "constricting", you_constricting },
     { "antimagic",    you_antimagic },
+    { "status",       you_status },
 
-    { "god_likes_fresh_corpses",  you_god_likes_fresh_corpses },
     { "can_consume_corpses",      you_can_consume_corpses },
 
     { "stop_activity", you_stop_activity },
@@ -541,12 +616,13 @@ static const struct luaL_reg you_clib[] =
     { "see_cell_no_trans", you_see_cell_no_trans_rel },
 
     { "mutation",          you_mutation },
+    { "temp_mutation",     you_temp_mutation },
 
     { "num_runes",          you_num_runes },
     { "have_rune",          _you_have_rune },
     { "have_orb",           you_have_orb},
 
-    { NULL, NULL },
+    { nullptr, nullptr },
 };
 
 void cluaopen_you(lua_State *ls)
@@ -600,7 +676,7 @@ LUAFN(you_teleport_to)
 
 LUAFN(you_random_teleport)
 {
-    you_teleport_now(false);
+    you_teleport_now();
     return 0;
 }
 
@@ -618,7 +694,22 @@ static int _you_uniques(lua_State *ls)
     return 1;
 }
 
-LUAWRAP(_you_die,ouch(INSTANT_DEATH, NON_MONSTER, KILLED_BY_SOMETHING))
+static int _you_unrands(lua_State *ls)
+{
+    bool unrand_found = false;
+
+    if (lua_gettop(ls) >= 1 && lua_isstring(ls, 1))
+    {
+        int unrand_num = get_unrandart_num(lua_tostring(ls, 1));
+        if (unrand_num != SPWPN_NORMAL)
+            unrand_found = get_unique_item_status(unrand_num);
+    }
+
+    lua_pushboolean(ls, unrand_found);
+    return 1;
+}
+
+LUAWRAP(_you_die,ouch(INSTANT_DEATH, KILLED_BY_SOMETHING))
 
 static int _you_piety(lua_State *ls)
 {
@@ -630,6 +721,21 @@ static int _you_piety(lua_State *ls)
     PLUARET(number, you.piety);
 }
 
+static int you_dock_piety(lua_State *ls)
+{
+    const int piety_loss = luaL_checkint(ls, 1);
+    const int penance = luaL_checkint(ls, 2);
+    dock_piety(piety_loss, penance);
+    return 0;
+}
+
+static int you_lose_piety(lua_State *ls)
+{
+    const int piety_loss = luaL_checkint(ls, 1);
+    lose_piety(piety_loss);
+    return 0;
+}
+
 LUAFN(you_in_branch)
 {
     const char* name = luaL_checkstring(ls, 1);
@@ -720,15 +826,16 @@ LUAFN(you_init)
     ng.weapon = str_to_weapon(luaL_checkstring(ls, 2));
     setup_game(ng);
     you.save->unlink();
-    you.save = NULL;
+    you.save = nullptr;
     PLUARET(string, skill_name(item_attack_skill(OBJ_WEAPONS, ng.weapon)));
 }
 
-LUARET1(you_exp_needed, number, exp_needed(luaL_checkint(ls, 1)));
-LUAWRAP(you_exercise, exercise(str_to_skill(luaL_checkstring(ls, 1)), 1));
-LUARET1(you_skill_cost_level, number, you.skill_cost_level);
+LUARET1(you_exp_needed, number, exp_needed(luaL_checkint(ls, 1)))
+LUAWRAP(you_exercise, exercise(str_to_skill(luaL_checkstring(ls, 1)), 1))
+LUARET1(you_skill_cost_level, number, you.skill_cost_level)
 LUARET1(you_skill_points, number,
-        you.skill_points[str_to_skill(luaL_checkstring(ls, 1))]);
+        you.skill_points[str_to_skill(luaL_checkstring(ls, 1))])
+LUARET1(you_zigs_completed, number, you.zigs_completed)
 
 static const struct luaL_reg you_dlib[] =
 {
@@ -743,8 +850,11 @@ static const struct luaL_reg you_dlib[] =
 { "random_teleport",    you_random_teleport },
 { "teleport_to",        you_teleport_to },
 { "uniques",            _you_uniques },
+{ "unrands",            _you_unrands },
 { "die",                _you_die },
 { "piety",              _you_piety },
+{ "dock_piety",         you_dock_piety },
+{ "lose_piety",         you_lose_piety },
 { "in_branch",          you_in_branch },
 { "shopping_list_has",  _you_shopping_list_has },
 { "shopping_list_add",  _you_shopping_list_add },
@@ -757,8 +867,9 @@ static const struct luaL_reg you_dlib[] =
 { "exercise",           you_exercise },
 { "skill_cost_level",   you_skill_cost_level },
 { "skill_points",       you_skill_points },
+{ "zigs_completed",     you_zigs_completed },
 
-{ NULL, NULL }
+{ nullptr, nullptr }
 };
 
 void dluaopen_you(lua_State *ls)
