@@ -5,23 +5,22 @@
 
 #include "AppHdr.h"
 
-#include <algorithm>
-
 #include "lev-pand.h"
 
-#include "externs.h"
+#include <algorithm>
+
 #include "colour.h"
 #include "env.h"
-#include "mon-place.h"
-#include "mgen_data.h"
-#include "random.h"
+#include "tileview.h"
 
 static colour_t _pan_floor_colour()
 {
     colour_t col;
 
     do
+    {
         col = random_colour();
+    }
     // Don't use silence or halo colours for floors.
     while (col == DARKGREY || col == CYAN || col == YELLOW);
 
@@ -33,7 +32,9 @@ static colour_t _pan_rock_colour()
     colour_t col;
 
     do
+    {
         col = random_colour();
+    }
     // Don't use stone or metal colours for walls.
     while (col == DARKGREY || col == LIGHTGREY || col == CYAN);
 
@@ -44,49 +45,21 @@ void init_pandemonium()
 {
     for (int pc = 0; pc < PAN_MONS_ALLOC; ++pc)
     {
-        env.mons_alloc[pc] = random_choose_weighted(
-                               2, MONS_NEQOXEC,
-                               4, MONS_ORANGE_DEMON,
-                               4, MONS_HELLWING,
-                               4, MONS_SMOKE_DEMON,
-                               4, MONS_YNOXINUL,
-                               4, MONS_ABOMINATION_LARGE,
-                               2, MONS_MONSTROUS_DEMONSPAWN,
-                               2, MONS_GELID_DEMONSPAWN,
-                               2, MONS_INFERNAL_DEMONSPAWN,
-                               2, MONS_PUTRID_DEMONSPAWN,
-                               2, MONS_TORTUROUS_DEMONSPAWN,
-                               0);
-
-        if (one_chance_in(10))
-        {
-            env.mons_alloc[pc] = random_choose(
-                                    MONS_HELLION,
-                                    MONS_TORMENTOR,
-                                    MONS_REAPER,
-                                    MONS_SOUL_EATER,
-                                    MONS_ICE_DEVIL,
-                                    MONS_BLUE_DEVIL,
-                                    MONS_HELL_BEAST,
-                                    MONS_IRON_DEVIL,
-                                    -1);
-        }
-
-        if (one_chance_in(30))
-            env.mons_alloc[pc] = MONS_RED_DEVIL;
-
-        if (one_chance_in(20))
-            env.mons_alloc[pc] = MONS_SIXFIRHY;
-
-        if (one_chance_in(20))
-        {
-            env.mons_alloc[pc] = random_choose(
-                                    MONS_DEMONIC_CRAWLER,
-                                    MONS_SUN_DEMON,
-                                    MONS_SHADOW_DEMON,
-                                    MONS_LOROCYPROCA,
-                                    -1);
-        }
+        env.mons_alloc[pc] = random_choose_weighted(5, MONS_SMOKE_DEMON,
+                                                    5, MONS_YNOXINUL,
+                                                    5, MONS_ABOMINATION_LARGE,
+                                                    5, MONS_ICE_DEVIL,
+                                                    5, MONS_SOUL_EATER,
+                                                    5, MONS_DEMONIC_CRAWLER,
+                                                    5, MONS_SUN_DEMON,
+                                                    5, MONS_NEQOXEC,
+                                                    5, MONS_CHAOS_SPAWN,
+                                                    1, MONS_SHADOW_DEMON,
+                                                    1, MONS_LOROCYPROCA,
+                                                    1, MONS_HELLION,
+                                                    1, MONS_TORMENTOR,
+                                                    1, MONS_REAPER,
+                                                    0);
 
         // The last three slots have a good chance of big badasses.
         if (pc == 7 && one_chance_in(8)
@@ -124,4 +97,5 @@ void init_pandemonium()
 
     env.floor_colour = _pan_floor_colour();
     env.rock_colour  = _pan_rock_colour();
+    tile_init_default_flavour();
 }
