@@ -15,33 +15,26 @@
 
 #include "AppHdr.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-#include <stdarg.h>
-#include <ctype.h>
-#include <term.h>
 #define _LIBUNIX_IMPLEMENTATION
 #include "libunix.h"
-#include "defines.h"
+
+#include <cctype>
+#include <clocale>
+#include <cstdarg>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
+#include <cwchar>
+#include <langinfo.h>
+#include <term.h>
+#include <termios.h>
+#include <unistd.h>
 
 #include "cio.h"
 #include "crash.h"
-#include "enum.h"
-#include "externs.h"
-#include "libutil.h"
-#include "options.h"
-#include "files.h"
 #include "state.h"
 #include "unicode.h"
 #include "view.h"
-#include "viewgeom.h"
-
-#include <wchar.h>
-#include <locale.h>
-#include <langinfo.h>
-#include <termios.h>
 
 static struct termios def_term;
 static struct termios game_term;
@@ -63,7 +56,7 @@ static struct termios game_term;
     #include CURSES_INCLUDE_FILE
 #endif
 
-// Globals holding current text/backg. colors
+// Globals holding current text/backg. colours
 static short FG_COL = WHITE;
 static short BG_COL = BLACK;
 static int   Current_Colour = COLOR_PAIR(BG_COL * 8 + FG_COL);
@@ -349,7 +342,7 @@ static void unixcurses_defkeys()
     define_key("\033On", 1019); // .
     define_key("\033Oo", 1020); // -
 
-    // variants.  Ugly curses won't allow us to return the same code...
+    // variants. Ugly curses won't allow us to return the same code...
     define_key("\033[1~", 1031); // Home
     define_key("\033[4~", 1034); // End
     define_key("\033[E",  1040); // center arrow
@@ -385,7 +378,7 @@ int unixcurses_get_vi_key(int keyin)
     case KEY_BTAB:   return CK_SHIFT_TAB;
     case KEY_BACKSPACE:
         // If terminfo's entry for backspace (kbs) is ctrl-h, curses
-        // generates KEY_BACKSPACE for the ctrl-h key.  Work around that by
+        // generates KEY_BACKSPACE for the ctrl-h key. Work around that by
         // converting back to CK_BKSP.
         // Note that this mangling occurs entirely on the machine Crawl runs
         // on (and even within crawl's process) rather than where the user's
@@ -527,7 +520,7 @@ void puttext(int x1, int y1, const crawl_view_buffer &vbuf)
 
 // These next four are front functions so that we can reduce
 // the amount of curses special code that occurs outside this
-// this file.  This is good, since there are some issues with
+// this file. This is good, since there are some issues with
 // name space collisions between curses macros and the standard
 // C++ string class.  -- bwr
 void update_screen()
@@ -541,7 +534,7 @@ void update_screen()
 
 void clear_to_end_of_line()
 {
-    textcolor(LIGHTGREY);
+    textcolour(LIGHTGREY);
     textbackground(BLACK);
     clrtoeol();
 
@@ -567,7 +560,7 @@ int num_to_lines(int num)
 
 void clrscr()
 {
-    textcolor(LIGHTGREY);
+    textcolour(LIGHTGREY);
     textbackground(BLACK);
     clear();
 #ifdef DGAMELAUNCH
@@ -602,7 +595,7 @@ bool is_smart_cursor_enabled()
     return false;
 }
 
-void enable_smart_cursor(bool dummy)
+void enable_smart_cursor(bool /*cursor*/)
 {
 }
 
@@ -675,12 +668,12 @@ static int curs_fg_attr(int col)
     return COLOR_PAIR(pair) | flags;
 }
 
-void textcolor(int col)
+void textcolour(int col)
 {
     (void)attrset(Current_Colour = curs_fg_attr(col));
 
 #ifdef USE_TILE_WEB
-    tiles.textcolor(col);
+    tiles.textcolour(col);
 #endif
 }
 
