@@ -5,25 +5,22 @@
 
 #include "AppHdr.h"
 
-#include "colour.h"
-#include "defines.h"
-#include "itemname.h" // is_vowel()
 #include "libutil.h"
-#include "externs.h"
+
+#include <cctype>
+#include <cstdarg>
+#include <cstdio>
+#include <cstring>
+#include <sstream>
+
+#include "colour.h"
 #include "files.h"
 #include "message.h"
 #include "state.h"
 #include "stringutil.h"
 #include "unicode.h"
-#include "version.h"
 #include "viewgeom.h"
 
-#include <sstream>
-#include <stdio.h>
-#include <ctype.h>
-#include <stdarg.h>
-#include <string.h>
-
 #ifdef TARGET_OS_WINDOWS
     #undef ARRAYSZ
     #include <windows.h>
@@ -35,7 +32,7 @@
 #endif
 
 #ifdef UNIX
-    #include <signal.h>
+    #include <csignal>
 #endif
 
 #ifdef DGL_ENABLE_CORE_DUMP
@@ -43,9 +40,13 @@
     #include <sys/resource.h>
 #endif
 
-#ifdef __ANDROID__
-    #include <SDL_mixer.h>
-    Mix_Chunk* android_sound_to_play = NULL;
+#if defined(USE_SOUND) && defined(USE_SDL) && !defined(WINMM_PLAY_SOUNDS)
+    #ifdef __ANDROID__
+        #include <SDL_mixer.h>
+    #else
+        #include <SDL2/SDL_mixer.h>
+    #endif
+    Mix_Chunk* sdl_sound_to_play = nullptr;
 #endif
 
 unsigned int isqrt(unsigned int a)
@@ -98,29 +99,6 @@ description_level_type description_type_by_name(const char *desc)
     return DESC_PLAIN;
 }
 
-static string _number_to_string(unsigned number, bool in_words)
-{
-    return in_words? number_in_words(number) : make_stringf("%u", number);
-}
-
-string apply_description(description_level_type desc, const string &name,
-                         int quantity, bool in_words)
-{
-    switch (desc)
-    {
-    case DESC_THE:
-        return "the " + name;
-    case DESC_A:
-        return quantity > 1 ? _number_to_string(quantity, in_words) + name
-                            : article_a(name, true);
-    case DESC_YOUR:
-        return "your " + name;
-    case DESC_PLAIN:
-    default:
-        return name;
-    }
-}
-
 // Should return true if the filename contains nothing that
 // the shell can do damage with.
 bool shell_safe(const char *file)
@@ -129,6 +107,7 @@ bool shell_safe(const char *file)
     return match < 0 || !file[match];
 }
 
+#ifdef USE_SOUND
 void play_sound(const char *file)
 {
 #if defined(WINMM_PLAY_SOUNDS)
@@ -145,15 +124,16 @@ void play_sound(const char *file)
         snprintf(command, sizeof command, SOUND_PLAY_COMMAND, file);
         system(OUTS(command));
     }
-#elif defined(__ANDROID__)
+#elif defined(USE_SDL)
     if (Mix_Playing(0))
         Mix_HaltChannel(0);
-    if (android_sound_to_play != NULL)
-        Mix_FreeChunk(android_sound_to_play);
-    android_sound_to_play = Mix_LoadWAV(OUTS(file));
-    Mix_PlayChannel(0, android_sound_to_play, 0);
+    if (sdl_sound_to_play != nullptr)
+        Mix_FreeChunk(sdl_sound_to_play);
+    sdl_sound_to_play = Mix_LoadWAV(OUTS(file));
+    Mix_PlayChannel(0, sdl_sound_to_play, 0);
 #endif
 }
+#endif
 
 bool key_is_escape(int key)
 {
@@ -268,249 +248,6 @@ bool parse_int(const char *s, int &i)
     return true;
 }
 
-// Naively prefix A/an to a noun.
-string article_a(const string &name, bool lowercase)
-{
-    if (!name.length())
-        return name;
-
-    const char *a  = lowercase? "a "  : "A ";
-    const char *an = lowercase? "an " : "An ";
-    switch (name[0])
-    {
-        case 'a': case 'e': case 'i': case 'o': case 'u':
-        case 'A': case 'E': case 'I': case 'O': case 'U':
-            // XXX: Hack
-            if (starts_with(name, "one-"))
-                return a + name;
-            return an + name;
-        default:
-            return a + name;
-    }
-}
-
-const char *standard_plural_qualifiers[] =
-{
-    " of ", " labeled ", NULL
-};
-
-// Pluralises a monster or item name.  This'll need to be updated for
-// correctness whenever new monsters/items are added.
-string pluralise(const string &name, const char *qualifiers[],
-                 const char *no_qualifier[])
-{
-    string::size_type pos;
-
-    if (qualifiers)
-    {
-        for (int i = 0; qualifiers[i]; ++i)
-            if ((pos = name.find(qualifiers[i])) != string::npos
-                && !ends_with(name, no_qualifier))
-            {
-                return pluralise(name.substr(0, pos)) + name.substr(pos);
-            }
-    }
-
-    if (!name.empty() && name[name.length() - 1] == ')'
-        && (pos = name.rfind(" (")) != string::npos)
-    {
-        return pluralise(name.substr(0, pos)) + name.substr(pos);
-    }
-
-    if (!name.empty() && name[name.length() - 1] == ']'
-        && (pos = name.rfind(" [")) != string::npos)
-    {
-        return pluralise(name.substr(0, pos)) + name.substr(pos);
-    }
-
-    if (ends_with(name, "us"))
-    {
-        if (ends_with(name, "lotus"))
-            return name + "es";
-        else
-            // Fungus, ufetubus, for instance.
-            return name.substr(0, name.length() - 2) + "i";
-    }
-    else if (ends_with(name, "larva") || ends_with(name, "antenna"))
-        return name + "e";
-    else if (ends_with(name, "ex"))
-    {
-        // Vortex; vortexes is legal, but the classic plural is cooler.
-        return name.substr(0, name.length() - 2) + "ices";
-    }
-    else if (ends_with(name, "mosquito") || ends_with(name, "ss"))
-        return name + "es";
-    else if (ends_with(name, "cyclops"))
-        return name.substr(0, name.length() - 1) + "es";
-    else if (name == "catoblepas")
-        return "catoblepae";
-    else if (ends_with(name, "s"))
-        return name;
-    else if (ends_with(name, "y"))
-    {
-        if (name == "y")
-            return "ys";
-        // day -> days, boy -> boys, etc
-        else if (is_vowel(name[name.length() - 2]))
-            return name + "s";
-        // jelly -> jellies
-        else
-            return name.substr(0, name.length() - 1) + "ies";
-    }
-    else if (ends_with(name, "fe"))
-    {
-        // knife -> knives
-        return name.substr(0, name.length() - 2) + "ves";
-    }
-    else if (ends_with(name, "staff"))
-    {
-        // staff -> staves
-        return name.substr(0, name.length() - 2) + "ves";
-    }
-    else if (ends_with(name, "f") && !ends_with(name, "ff"))
-    {
-        // elf -> elves, but not hippogriff -> hippogrives.
-        // TODO: if someone defines a "goblin chief", this should be revisited.
-        return name.substr(0, name.length() - 1) + "ves";
-    }
-    else if (ends_with(name, "mage"))
-    {
-        // mage -> magi
-        return name.substr(0, name.length() - 1) + "i";
-    }
-    else if (name == "gold"                 || ends_with(name, "fish")
-             || ends_with(name, "folk")     || ends_with(name, "spawn")
-             || ends_with(name, "tengu")    || ends_with(name, "sheep")
-             || ends_with(name, "swine")    || ends_with(name, "efreet")
-             || ends_with(name, "jiangshi") || ends_with(name, "unborn")
-             || ends_with(name, "raiju")    )
-    {
-        return name;
-    }
-    else if (ends_with(name, "ch") || ends_with(name, "sh")
-             || ends_with(name, "x"))
-    {
-        // To handle cockroaches, sphinxes, and bushes.
-        return name + "es";
-    }
-    else if (ends_with(name, "simulacrum") || ends_with(name, "eidolon"))
-    {
-        // simulacrum -> simulacra (correct Latin pluralisation)
-        // also eidolon -> eidola (correct Greek pluralisation)
-        return name.substr(0, name.length() - 2) + "a";
-    }
-    else if (ends_with(name, "djinni"))
-    {
-        // djinni -> djinn.
-        return name.substr(0, name.length() - 1);
-    }
-    else if (name == "foot")
-        return "feet";
-    else if (name == "ophan" || name == "cherub" || name == "seraph")
-    {
-        // Unlike "angel" which is fully assimilated, and "cherub" and "seraph"
-        // which may be pluralised both ways, "ophan" always uses Hebrew
-        // pluralisation.
-        return name + "im";
-    }
-
-    return name + "s";
-}
-
-string apostrophise(const string &name)
-{
-    if (name.empty())
-        return name;
-
-    if (name == "you" || name == "You")
-        return name + "r";
-
-    if (name == "it" || name == "It")
-        return name + "s";
-
-    const char lastc = name[name.length() - 1];
-    return name + (lastc == 's' ? "'" : "'s");
-}
-
-string apostrophise_fixup(const string &msg)
-{
-    if (msg.empty())
-        return msg;
-
-    // XXX: This is rather hackish.
-    return replace_all(msg, "s's", "s'");
-}
-
-static string pow_in_words(int pow)
-{
-    switch (pow)
-    {
-    case 0:
-        return "";
-    case 3:
-        return " thousand";
-    case 6:
-        return " million";
-    case 9:
-        return " billion";
-    case 12:
-    default:
-        return " trillion";
-    }
-}
-
-static string tens_in_words(unsigned num)
-{
-    static const char *numbers[] =
-    {
-        "", "one", "two", "three", "four", "five", "six", "seven",
-        "eight", "nine", "ten", "eleven", "twelve", "thirteen", "fourteen",
-        "fifteen", "sixteen", "seventeen", "eighteen", "nineteen"
-    };
-    static const char *tens[] =
-    {
-        "", "", "twenty", "thirty", "forty", "fifty", "sixty", "seventy",
-        "eighty", "ninety"
-    };
-
-    if (num < 20)
-        return numbers[num];
-
-    int ten = num / 10, digit = num % 10;
-    return string(tens[ten]) + (digit ? string("-") + numbers[digit] : "");
-}
-
-static string join_strings(const string &a, const string &b)
-{
-    if (!a.empty() && !b.empty())
-        return a + " " + b;
-
-    return a.empty() ? b : a;
-}
-
-static string hundreds_in_words(unsigned num)
-{
-    unsigned dreds = num / 100, tens = num % 100;
-    string sdreds = dreds? tens_in_words(dreds) + " hundred" : "";
-    string stens  = tens? tens_in_words(tens) : "";
-    return join_strings(sdreds, stens);
-}
-
-string number_in_words(unsigned num, int pow)
-{
-    if (pow == 12)
-        return number_in_words(num, 0) + pow_in_words(pow);
-
-    unsigned thousands = num % 1000, rest = num / 1000;
-    if (!rest && !thousands)
-        return "zero";
-
-    return join_strings((rest? number_in_words(rest, pow + 3) : ""),
-                        (thousands? hundreds_in_words(thousands)
-                                    + pow_in_words(pow)
-                                  : ""));
-}
-
 /**
  * Compare two strings, sorting integer numeric parts according to their value.
  *
@@ -526,37 +263,34 @@ int numcmp(const char *a, const char *b, int limit)
 {
     int res;
 
-not_numeric:
-    while (*a && *a == *b && !isadigit(*a))
+    do
     {
-        a++;
-        b++;
-    }
-    if (!a && !b)
-        return 0;
-    if (!isadigit(*a) || !isadigit(*b))
-        return (*a < *b) ? -1 : (*a > *b) ? 1 : 0;
-    while (*a == '0')
-        a++;
-    while (*b == '0')
-        b++;
-    res = 0;
-    while (isadigit(*a))
-    {
-        if (!isadigit(*b))
-            return 1;
-        if (*a != *b && !res)
-            res = (*a < *b) ? -1 : 1;
-        a++;
-        b++;
-    }
-    if (isadigit(*b))
-        return -1;
-    if (res)
-        return res;
-
-    if (--limit)
-        goto not_numeric;
+        while (*a && *a == *b && !isadigit(*a))
+        {
+            a++;
+            b++;
+        }
+        if (!isadigit(*a) || !isadigit(*b))
+            return (*a < *b) ? -1 : (*a > *b) ? 1 : 0;
+        while (*a == '0')
+            a++;
+        while (*b == '0')
+            b++;
+        res = 0;
+        while (isadigit(*a))
+        {
+            if (!isadigit(*b))
+                return 1;
+            if (*a != *b && !res)
+                res = (*a < *b) ? -1 : 1;
+            a++;
+            b++;
+        }
+        if (isadigit(*b))
+            return -1;
+        if (res)
+            return res;
+    } while (--limit > 0);
     return 0;
 }
 
@@ -762,7 +496,7 @@ static bool _is_aero()
 taskbar_pos get_taskbar_pos()
 {
     RECT rect;
-    HWND taskbar = FindWindow("Shell_traywnd", NULL);
+    HWND taskbar = FindWindow("Shell_traywnd", nullptr);
     if (taskbar && GetWindowRect(taskbar, &rect))
     {
         if (rect.right - rect.left > rect.bottom - rect.top)
@@ -788,14 +522,14 @@ int get_taskbar_size()
     RECT rect;
     int size;
     taskbar_pos tpos = get_taskbar_pos();
-    HWND taskbar = FindWindow("Shell_traywnd", NULL);
+    HWND taskbar = FindWindow("Shell_traywnd", nullptr);
 
     if (taskbar && GetWindowRect(taskbar, &rect))
     {
         if (tpos & TASKBAR_H)
-                size = rect.bottom - rect.top;
+            size = rect.bottom - rect.top;
         else if (tpos & TASKBAR_V)
-                size = rect.right - rect.left;
+            size = rect.right - rect.left;
         else
             return 0;
 
