@@ -6,12 +6,12 @@
 #ifndef LIBUTIL_H
 #define LIBUTIL_H
 
-#include "enum.h"
 #include <cctype>
+#include <map>
 #include <string>
 #include <vector>
-#include <map>
-#include "stringutil.h" // :( for find_earliest_match
+
+#include "enum.h"
 
 bool key_is_escape(int key);
 
@@ -57,6 +57,12 @@ static inline ucs_t toalower(ucs_t c)
     return isaupper(c) ? c + 'a' - 'A' : c;
 }
 
+// Same thing with signed int, so we can pass though -1 undisturbed.
+static inline int toalower(int c)
+{
+    return isaupper(c) ? c + 'a' - 'A' : c;
+}
+
 int numcmp(const char *a, const char *b, int limit = 0);
 bool numcmpstr(string a, string b);
 
@@ -70,107 +76,44 @@ vector<string> strip_multiple_tag_prefix(string &s, const string &tagprefix);
 string strip_tag_prefix(string &s, const string &tagprefix);
 bool parse_int(const char *s, int &i);
 
-string number_in_words(unsigned number, int pow = 0);
-
 // String 'descriptions'
-
-extern const char *standard_plural_qualifiers[];
-
-// Applies a description type to a name, but does not pluralise! You
-// must pluralise the name if needed. The quantity is used to prefix the
-// name with a quantity if appropriate.
-string apply_description(description_level_type desc, const string &name,
-                         int quantity = 1, bool num_in_words = false);
-
 description_level_type description_type_by_name(const char *desc);
-string article_a(const string &name, bool lowercase = true);
-string pluralise(const string &name,
-                 const char *stock_plural_quals[] = standard_plural_qualifiers,
-                 const char *no_of[] = NULL);
-string apostrophise(const string &name);
-string apostrophise_fixup(const string &msg);
-
-
 
 bool shell_safe(const char *file);
 
+#ifdef USE_SOUND
 void play_sound(const char *file);
+#endif
 
 string unwrap_desc(string desc);
 
-template<class T> bool _always_true(T) { return true; }
-
-/**
- * Find the enumerator e between begin and end that satisfies pred(e) and
- * whose name, as given by namefunc(e), has the earliest occurrence of the
- * substring spec.
+/** Ignore an argument and return true.
  *
- * @param spec      The substring to search for.
- * @param begin     The beginning of the enumerator range to search in.
- * @param end       One past the end of the enum range to search in.
- * @param pred      A function from Enum to bool. Enumerators that do not
- *                  satisfy the predicate are ignored.
- * @param namefunc  A function from Enum to string or const char * giving
- *                  the name of the enumerator.
- * @return The enumerator that satisfies pred and whose name contains the
- *         spec substring beginning at the earliest position. If no such
- *         enumerator exists, returns end. If there are multiple strings
- *         containing the spec as a prefix, returns the shortest such string
- *         (so exact matches are preferred); otherwise ties are broken in
- *         an unspecified manner.
+ * @return true
  */
-template<class Enum, class Pred, class NameFunc>
-Enum find_earliest_match(string spec, Enum begin, Enum end,
-                         Pred pred, NameFunc namefunc)
-{
-    Enum selected = end;
-    const size_t speclen = spec.length();
-    size_t bestpos = string::npos;
-    size_t bestlen = string::npos;
-    for (size_t i = begin; i < (size_t) end; ++i)
-    {
-        const Enum curr = static_cast<Enum>(i);
-
-        if (!pred(curr))
-            continue;
-
-        const string name = lowercase_string(namefunc(curr));
-        const size_t pos = name.find(spec);
-        const size_t len = name.length();
-
-        if (pos < bestpos || pos == 0 && len < bestlen)
-        {
-            // Exit early if we found an exact match.
-            if (pos == 0 && len == speclen)
-                return curr;
-
-            // npos is never less than bestpos, so the spec was found.
-            bestpos = pos;
-            if (pos == 0)
-                bestlen = len;
-            selected = curr;
-        }
-    }
-    return selected;
-}
+template<class T> bool always_true(T) { return true; }
 
+/** Remove an element from a vector without preserving order.
+ *  The indicated element is replaced by the last element of the vector.
+ *
+ * @tparam Z The value type of the vector.
+ * @param vec The vector to modify.
+ * @param which The index of the element to remove.
+ */
 template <typename Z>
 void erase_any(vector<Z> &vec, unsigned long which)
 {
     if (which != vec.size() - 1)
-        vec[which] = vec[vec.size() - 1];
+        vec[which] = std::move(vec[vec.size() - 1]);
     vec.pop_back();
 }
 
-template <typename Z>
-static inline void swapv(Z &a, Z &b)
-{
-    Z tmp = a;
-    a = b;
-    b = tmp;
-}
-
-// A comparator for pairs.
+/** A comparator to compare pairs by their second elements.
+ *
+ * @tparam T  The type of the pairs (not the second element!)
+ * @param left,right  The pairs to be compared.
+ * @return true if left.second > right.second, false otherwise.
+ */
 template<typename T>
 struct greater_second
 {
@@ -180,6 +123,56 @@ struct greater_second
     }
 };
 
+/** Delete all the pointers in a container and clear the container.
+ *
+ * @tparam T The type of the container; must be a container of pointers
+ *           to non-arrays.
+ * @param collection The container to clear.
+ */
+template<class T>
+static void deleteAll(T& collection)
+{
+    for (auto ptr : collection)
+        delete ptr;
+    collection.clear();
+}
+
+/** Find a map element by key, and return a pointer to its value.
+ *
+ * @tparam M The type of the map; may be const.
+ * @param map The map in which to do the lookup.
+ * @param obj The key to search for.
+ * @return a pointer to the value associated with obj if found,
+ *         otherwise null. The pointer is const if the map is.
+ */
+template<class M>
+auto map_find(M &map, const typename M::key_type &obj)
+    -> decltype(&map.begin()->second)
+{
+    auto it = map.find(obj);
+    return it == map.end() ? nullptr : &it->second;
+}
+
+/** Find a map element by key, and return its value or a default.
+ *  Only intended for use with simple map values where copying is not
+ *  a concern; use map_find if the returned value should not be
+ *  copied.
+ *
+ * @tparam M The type of the map; may be const.
+ * @param map The map in which to do the lookup.
+ * @param key The key to search for.
+ * @param unfound The object to return if the key was not found.
+ *
+ * @return a copy of the value associated with key if found, otherwise
+ *         a copy of unfound.
+ */
+template<class M>
+typename M::mapped_type lookup(M &map, const typename M::key_type &key,
+                               const typename M::mapped_type &unfound)
+{
+    auto it = map.find(key);
+    return it == map.end() ? unfound : it->second;
+}
 
 static inline int sqr(int x)
 {
@@ -194,6 +187,19 @@ static inline bool testbits(uint64_t flags, uint64_t test)
     return (flags & test) == test;
 }
 
+template<class E, int Exp>
+static inline bool testbits(enum_bitfield<E, Exp> flags,
+                            enum_bitfield<E, Exp> test)
+{
+    return (flags & test) == test;
+}
+
+template<class E, int Exp>
+static inline bool testbits(enum_bitfield<E, Exp> flags, E test)
+{
+    return (flags & test) == test;
+}
+
 coord_def cgetsize(GotoRegion region = GOTO_CRT);
 void cscroll(int n, GotoRegion region);
 
