@@ -44,23 +44,14 @@
 
 #include "los.h"
 
-#include <cmath>
 #include <algorithm>
+#include <cmath>
 
 #include "areas.h"
-#include "bitary.h"
 #include "coord.h"
-#include "coord-circle.h"
 #include "coordit.h"
-#include "externs.h"
-#include "geom2d.h"
-#include "losglobal.h"
-#include "losparam.h"
-#include "mon-poly.h"
-#include "player.h"
-#include "ray.h"
 #include "env.h"
-#include "terrain.h"
+#include "losglobal.h"
 
 // These determine what rays are cast in the precomputation,
 // and affect start-up time significantly.
@@ -95,8 +86,8 @@ static FixedArray<vector<cellray>, LOS_MAX_RANGE+1, LOS_MAX_RANGE+1> min_cellray
 // Temporary arrays used in losight() to track which rays
 // are blocked or have seen a smoke cloud.
 // Allocated when doing the precomputations.
-static bit_vector *dead_rays     = NULL;
-static bit_vector *smoke_rays    = NULL;
+static bit_vector *dead_rays     = nullptr;
+static bit_vector *smoke_rays    = nullptr;
 
 class quadrant_iterator : public rectangle_iterator
 {
@@ -116,15 +107,15 @@ void clear_rays_on_exit()
         delete blockrays(*qi);
 }
 
-// Pre-squared LOS radius.
-int los_radius2 = LOS_RADIUS_SQ;
+// LOS radius.
+int los_radius = LOS_RADIUS;
 
 static void _handle_los_change();
 
 void set_los_radius(int r)
 {
     ASSERT(r <= LOS_RADIUS);
-    los_radius2 = r * r + 1;
+    los_radius = r;
     invalidate_los();
     _handle_los_change();
 }
@@ -168,7 +159,7 @@ struct los_ray : public ray_def
                 break;
             }
             c = copy.pos();
-            if (c.abs() > LOS_RADIUS_SQ)
+            if (c.rdist() > LOS_RADIUS)
                 break;
             cs.push_back(c);
             ASSERT((c - old).rdist() == 1);
@@ -197,8 +188,8 @@ static bool _is_same_ray(los_ray ray, vector<coord_def> newray)
 // Check if the passed ray has already been created.
 static bool _is_duplicate_ray(vector<coord_def> newray)
 {
-    for (unsigned int i = 0; i < fullrays.size(); ++i)
-        if (_is_same_ray(fullrays[i], newray))
+    for (los_ray lray : fullrays)
+        if (_is_same_ray(lray, newray))
             return true;
     return false;
 }
@@ -315,9 +306,8 @@ static vector<int> _find_minimal_cellrays()
     FixedArray<list<cellray>, LOS_MAX_RANGE+1, LOS_MAX_RANGE+1> minima;
     list<cellray>::iterator min_it;
 
-    for (unsigned int r = 0; r < fullrays.size(); ++r)
+    for (los_ray ray : fullrays)
     {
-        los_ray ray = fullrays[r];
         for (unsigned int i = 0; i < ray.length; ++i)
         {
             // Is the cellray ray[0..i] duplicated so far?
@@ -381,8 +371,8 @@ static void _register_ray(geom::ray r)
 
     ray.start = ray_coords.size();
     ray.length = coords.size();
-    for (unsigned int i = 0; i < coords.size(); i++)
-        ray_coords.push_back(coords[i]);
+    for (coord_def c : coords)
+        ray_coords.push_back(c);
     fullrays.push_back(ray);
 }
 
@@ -396,9 +386,8 @@ static void _create_blockrays()
     for (quadrant_iterator qi; qi; ++qi)
         all_blockrays(*qi) = new bit_vector(n_cellrays);
 
-    for (unsigned int r = 0; r < fullrays.size(); ++r)
+    for (los_ray ray : fullrays)
     {
-        los_ray ray = fullrays[r];
         for (unsigned int i = 0; i < ray.length; ++i)
         {
             // Every cell is contained in (thus blocks)
@@ -487,14 +476,14 @@ static void raycast()
         for (int yangle = 1; yangle <= LOS_MAX_ANGLE; ++yangle)
         {
             if (_gcd(xangle, yangle) == 1)
-                xyangles.push_back(pair<int,int>(xangle, yangle));
+                xyangles.emplace_back(xangle, yangle);
         }
 
     sort(xyangles.begin(), xyangles.end(), _complexity_lt);
-    for (unsigned int i = 0; i < xyangles.size(); ++i)
+    for (auto xyangle : xyangles)
     {
-        const int xangle = xyangles[i].first;
-        const int yangle = xyangles[i].second;
+        const int xangle = xyangle.first;
+        const int yangle = xyangle.second;
 
         for (int intercept = 1; intercept < LOS_INTERCEPT_MULT*yangle; ++intercept)
         {
@@ -555,10 +544,10 @@ static bool _find_ray_se(const coord_def& target, ray_def& ray,
     ASSERT(target.x >= 0);
     ASSERT(target.y >= 0);
     ASSERT(!target.origin());
-    if (target.abs() > range * range + 1)
+    if (target.rdist() > range)
         return false;
 
-    ASSERT(target.abs() <= LOS_RADIUS_SQ);
+    ASSERT(target.rdist() <= LOS_RADIUS);
 
     // Ensure the precalculations have been done.
     raycast();
@@ -608,7 +597,7 @@ struct opacity_trans : public opacity_func
 
     CLONE(opacity_trans)
 
-    opacity_type operator()(const coord_def &l) const
+    opacity_type operator()(const coord_def &l) const override
     {
         return orig(transform(l));
     }
@@ -773,11 +762,11 @@ bool cell_see_cell_nocache(const coord_def& p1, const coord_def& p2)
 // Look up which cells the surviving rays have, and that's your LOS!
 // OPTIMIZATIONS:
 // WLOG, we can assume that we're in a specific quadrant - say the
-// first quadrant - and just mirror everything after that.  We can
+// first quadrant - and just mirror everything after that. We can
 // likely get away with a single octant, but we don't do that. (To
 // do...)
 // Rays are actually split by each cell they pass. So each "ray" only
-// identifies a single cell, and we can do logical ORs.  Once a cell
+// identifies a single cell, and we can do logical ORs. Once a cell
 // kills a cellray, it will kill all remaining cellrays of that ray.
 // Also, rays are checked to see if they are duplicates of each
 // other. If they are, they're eliminated.
@@ -853,12 +842,12 @@ struct los_param_funcs : public los_param
     {
     }
 
-    bool los_bounds(const coord_def& p) const
+    bool los_bounds(const coord_def& p) const override
     {
         return map_bounds(p + center) && bounds.contains(p);
     }
 
-    opacity_type opacity(const coord_def& p) const
+    opacity_type opacity(const coord_def& p) const override
     {
         return opc(p + center);
     }
@@ -893,20 +882,6 @@ opacity_type mons_opacity(const monster* mon, los_type how)
         return OPC_HALF;
     }
 
-    if (mons_is_feat_mimic(mon->type))
-    {
-        dungeon_feature_type feat = get_mimic_feat(mon);
-        if (how == LOS_SOLID)
-            return feat_is_solid(feat) ? OPC_OPAQUE : OPC_CLEAR;
-        if (how == LOS_NO_TRANS
-            && (feat_is_wall(feat) || feat_is_tree(feat)))
-        {
-            return OPC_OPAQUE;
-        }
-        if (feat_is_opaque(get_mimic_feat(mon)))
-            return OPC_OPAQUE;
-    }
-
     return OPC_CLEAR;
 }
 
