@@ -9,9 +9,7 @@
 
 #include "cloud.h"
 #include "env.h"
-#include "externs.h"
 #include "los.h"
-#include "mon-util.h"
 #include "state.h"
 #include "terrain.h"
 
@@ -28,11 +26,9 @@ opacity_type opacity_default::operator()(const coord_def& p) const
     dungeon_feature_type f = grd(p);
     if (feat_is_opaque(f))
         return OPC_OPAQUE;
-    else if (feat_is_tree(f))
-        return OPC_HALF;
     else if (is_opaque_cloud(env.cgrid(p)))
         return OPC_HALF;
-    if (const monster *mon = monster_at(p))
+    else if (const monster *mon = monster_at(p))
         return mons_opacity(mon, LOS_DEFAULT);
     return OPC_CLEAR;
 }
@@ -48,36 +44,19 @@ opacity_type opacity_fullyopaque::operator()(const coord_def& p) const
 opacity_type opacity_no_trans::operator()(const coord_def& p) const
 {
     dungeon_feature_type f = grd(p);
-    if (feat_is_opaque(f) || feat_is_wall(f) || feat_is_tree(f))
+    if (feat_is_opaque(f) || feat_is_wall(f))
         return OPC_OPAQUE;
     else if (is_opaque_cloud(env.cgrid(p)))
         return OPC_HALF;
-    if (const monster *mon = monster_at(p))
+    else if (const monster *mon = monster_at(p))
         return mons_opacity(mon, LOS_NO_TRANS);
     return OPC_CLEAR;
 }
 
-static bool mons_block_immob(const monster* mons)
-{
-    if (mons == NULL)
-        return false;
-
-    // In Zotdef, plants don't block movement as critters
-    // will attack them
-    if (crawl_state.game_is_zotdef())
-        return false;
-
-    return mons->is_stationary();
-}
-
 opacity_type opacity_immob::operator()(const coord_def& p) const
 {
-    opacity_type base = opc_no_trans(p);
-
-    if (mons_block_immob(monster_at(p)))
-        return OPC_OPAQUE;
-    else
-        return base;
+    const monster* mons = monster_at(p);
+    return (mons && mons->is_stationary()) ? OPC_OPAQUE : opc_no_trans(p);
 }
 
 opacity_type opacity_solid::operator()(const coord_def& p) const
@@ -107,7 +86,7 @@ opacity_type opacity_solid_see::operator()(const coord_def& p) const
 opacity_type opacity_monmove::operator()(const coord_def& p) const
 {
     dungeon_feature_type feat = env.grid(p);
-    if (feat < DNGN_MINMOVE || !mon.can_pass_through_feat(feat))
+    if (feat_is_solid(feat) || !mon.can_pass_through_feat(feat))
         return OPC_OPAQUE;
     else
         return OPC_CLEAR;
