@@ -36,8 +36,8 @@ private:
     vector<bool> prev_pause_status;
 };
 
-int getchm(int (*rgetch)() = NULL);       // keymaps applied (ie for prompts)
-int getchm(KeymapContext context, int (*rgetch)() = NULL);
+int getchm(int (*rgetch)() = nullptr);       // keymaps applied (ie for prompts)
+int getchm(KeymapContext context, int (*rgetch)() = nullptr);
 
 int get_ch();
 
@@ -98,4 +98,7 @@ string command_to_string(command_type cmd, bool tutorial = false);
 void insert_commands(string &desc, vector<command_type> cmds,
                      bool formatted = true);
 void insert_commands(string &desc, const int first, ...);
+
+// Let rc files declare macros:
+string read_rc_file_macro(const string& field);
 #endif
