@@ -11,8 +11,7 @@
 int create_item_named(string name, coord_def pos, string *error);
 
 int items(bool allow_uniques, object_class_type force_class, int force_type,
-          bool dont_place, int item_level, uint32_t mapmask = 0,
-          int force_ego = 0, int agent = -1, bool mundane = false);
+          int item_level, int force_ego = 0, int agent = -1);
 
 void item_colour(item_def &item);
 void init_rod_mp(item_def &item, int ncharges = -1, int item_level = -1);
@@ -30,6 +29,7 @@ void reroll_brand(item_def &item, int item_level);
 
 bool is_high_tier_wand(int type);
 
+void squash_plusses(int item_slot);
 #if defined(DEBUG_DIAGNOSTICS) || defined(DEBUG_TESTS)
 void makeitem_tests();
 #endif
