@@ -15,6 +15,7 @@
 
 #include <cstdio>
 #include <memory>
+#include <stdexcept>
 #include <string>
 #include <vector>
 
@@ -24,11 +25,20 @@
 #include "makeitem.h"
 #include "matrix.h"
 #include "mon-ench.h"
+#include "mon-flags.h"
 #include "tags.h"
 #include "travel_defs.h"
 
 #define NEVER_CORPSE_KEY "never_corpse"
 
+#define RANDBK_OWNER_KEY "randbook_owner"
+#define RANDBK_TITLE_KEY "randbook_title"
+#define RANDBK_DISC1_KEY "randbook_disc1"
+#define RANDBK_DISC2_KEY "randbook_disc2"
+#define RANDBK_SPELLS_KEY "randbook_spells"
+#define RANDBK_SLVLS_KEY "randbook_slevels"
+#define RANDBK_NSPELLS_KEY "randbook_num_spells"
+
 class mon_enchant;
 extern const char *traversable_glyphs;
 
@@ -39,17 +49,11 @@ static const int BRANCH_END = 100;
 
 // Exception thrown when a map cannot be loaded from its .dsc file
 // because the .dsc file has changed under it.
-class map_load_exception : public exception
+struct map_load_exception : public runtime_error
 {
-public:
-    map_load_exception(const string &_mapname) : mapname(_mapname) { }
-    ~map_load_exception() throw () { }
-    const char *what() const throw()
-    {
-        return mapname.c_str();
-    }
-private:
-    string mapname;
+    // g++ 4.7 doesn't have inherited constructors, sadly
+    map_load_exception(const string &s) : runtime_error(s) { }
+    map_load_exception(const char *s) : runtime_error(s) { }
 };
 
 // [dshaligram] Maps can be mirrored; for every orientation, there must be
@@ -433,7 +437,6 @@ public:
                                 const Matrix<bool> &mask, const map_def &vault);
 private:
     void init_from(const map_lines &map);
-    template <typename V> void clear_vector(V &vect);
     void vmirror_markers();
     void hmirror_markers();
     void rotate_markers(bool clock);
@@ -660,13 +663,11 @@ public:
 
     bool explicit_spells;
     vector<monster_spells> spells;
-    uint64_t extra_monster_flags;
+    monster_flags_t extra_monster_flags;
     vector<mon_enchant> ench;
 
     monster_type initial_shifter;
 
-    vector<monster_type> chimera_mons;
-
     CrawlHashTable props;
 
     mons_spec(monster_type t = RANDOM_MONSTER,
@@ -678,7 +679,7 @@ public:
           colour(COLOUR_INHERIT), god(GOD_NO_GOD), god_gift(false), hd(0),
           hp(0), abjuration_duration(0), summon_type(0), items(), monname(""),
           non_actor_summoner(""), explicit_spells(false), spells(),
-          extra_monster_flags(0), initial_shifter(RANDOM_MONSTER), props()
+          extra_monster_flags(), initial_shifter(RANDOM_MONSTER), props()
     {
     }
 };
@@ -870,7 +871,7 @@ public:
 public:
     keyed_mapspec();
 
-    // Parse the string and set the given entry.  If fix is true,
+    // Parse the string and set the given entry. If fix is true,
     // then whatever is selected for the first feature will be
     // permanently fixed.
     string set_feat(const string &s, bool fix);
@@ -878,7 +879,7 @@ public:
     string set_item(const string &s, bool fix);
     string set_mask(const string &s, bool /*garbage*/);
 
-    // Copy from the given mapspec.  If that entry is fixed,
+    // Copy from the given mapspec. If that entry is fixed,
     // it should be pre-selected prior to the copy.
     void copy_feat(const keyed_mapspec &spec);
     void copy_mons(const keyed_mapspec &spec);
@@ -1060,8 +1061,8 @@ public:
     {
         writer_fn(outf, default_thing);
         marshallShort(outf, depth_range_Xs.size());
-        for (int i = 0, size = depth_range_Xs.size(); i < size; ++i)
-            depth_range_Xs[i].write(outf, writer_fn);
+        for (const auto &range : depth_range_Xs)
+            range.write(outf, writer_fn);
     }
 };
 
@@ -1096,10 +1097,9 @@ struct subvault_place
 //   they will not change.
 //
 // * Fields that do not determine placement and may change between
-//   different uses of the map (such as "mons", "items",
-//   "level_flags", etc.). Such fields must be reset to their default
-//   values in map_def::reinit(), which is called before the map is
-//   used.
+//   different uses of the map (such as "mons", "items", etc.). Such fields
+//   must be reset to their default values in map_def::reinit(), which is
+//   called before the map is used.
 //
 // If you do not do this, maps will not work correctly, and will break
 // in obscure, hard-to-find ways. The level-compiler will not (cannot)
@@ -1111,7 +1111,7 @@ public:
     string          name;
     // Description for the map that can be shown to players.
     string          description;
-    // Order among related maps; used only for tutorial/sprint/zotdef.
+    // Order among related maps; used only for tutorial/sprint.
     int             order;
     string          tags;
     depth_ranges    place;
@@ -1137,8 +1137,6 @@ public:
 
     vector<mons_spec> random_mons;
 
-    map_flags       level_flags, branch_flags;
-
     dlua_chunk      prelude, mapchunk, main, validate, veto, epilogue;
 
     map_file_place  place_loaded_from;
