@@ -3,6 +3,7 @@
 
 #include "coord.h"
 #include "mgen_enum.h"
+#include "mon-flags.h"
 
 // Hash key for passing a weapon to be given to
 // a dancing weapon.
@@ -18,11 +19,6 @@ struct mgen_data
     // Monster type.
     monster_type    cls;
 
-    // If the monster is zombie-like, or a specialised draconian, this
-    // is the base monster that the monster is based on - should be
-    // set to MONS_NO_MONSTER when not used.
-    monster_type    base_type;
-
     // Determines the behaviour of the monster after it is generated. This
     // behaviour is an unholy combination of monster attitude
     // (friendly, hostile) and monster initial state (asleep, wandering).
@@ -30,7 +26,7 @@ struct mgen_data
     beh_type        behaviour;
 
     // Who summoned this monster?  Important to know for death accounting
-    // and the summon cap, if and when it goes in.  NULL is no summoner.
+    // and the summon cap, if and when it goes in.  nullptr is no summoner.
     const actor*    summoner;
 
     // For summoned monsters, this is a measure of how long the summon will
@@ -46,14 +42,6 @@ struct mgen_data
     // Where the monster will be created.
     coord_def       pos;
 
-    // A grid feature to prefer when finding a place to create monsters.
-    // For instance, using DNGN_FLOOR when placing flying monsters or
-    // merfolk in the Shoals will force them to appear on land.
-    // preferred_grid_feature will be ignored if it is incompatible with
-    // the monster's native habitat (for instance, if trying to place
-    // a electric eel with preferred_grid_feature DNGN_FLOOR).
-    dungeon_feature_type preferred_grid_feature;
-
     // The monster's foe, i.e. which monster it will want to attack. foe
     // may be an index into the monster array (0 - (MAX_MONSTERS-1)), or
     // it may be MHITYOU to indicate that the monster wants to attack the
@@ -64,11 +52,16 @@ struct mgen_data
     // Generation flags from mgen_flag_type.
     uint32_t        flags;
 
-    // What god the monster worships, if any.  Used for monsters that
+    // What god the monster worships, if any. Used for monsters that
     // are god gifts, to indicate which god sent them, and by priest
     // monsters, to indicate whose priest they are.
     god_type        god;
 
+    // If the monster is zombie-like, or a specialised draconian, this
+    // is the base monster that the monster is based on - should be
+    // set to MONS_NO_MONSTER when not used.
+    monster_type    base_type;
+
     // The number of hydra heads, the number of manticore attack volleys,
     // or the number of merged slime creatures.
     //
@@ -76,7 +69,7 @@ struct mgen_data
     // base_type.
     int             number;
 
-    // The colour of the monster.
+    // The colour of the monster, or COLOUR_UNDEF for col:any
     int             colour;
 
     // How close to or far from the player the monster should be created.
@@ -87,6 +80,29 @@ struct mgen_data
     // the player is actually in.
     level_id        place;
 
+    int             hd;
+    int             hp;
+
+    // These flags will be appended to the monster's flags after placement.
+    monster_flags_t extra_flags;
+
+    string          mname;
+
+    // This is used to account for non-actor summoners. Blasted by an Ice
+    // Fiend ... summoned by the effects of Hell.
+    string          non_actor_summoner;
+
+    // This simply stores the initial shape-shifter type.
+    monster_type    initial_shifter;
+
+    // A grid feature to prefer when finding a place to create monsters.
+    // For instance, using DNGN_FLOOR when placing flying monsters or
+    // merfolk in the Shoals will force them to appear on land.
+    // preferred_grid_feature will be ignored if it is incompatible with
+    // the monster's native habitat (for instance, if trying to place
+    // a electric eel with preferred_grid_feature DNGN_FLOOR).
+    dungeon_feature_type preferred_grid_feature = DNGN_UNSEEN;
+
     // Some predefined vaults (aka maps) include flags to suppress random
     // generation of monsters. When generating monsters, this is a mask of
     // map flags to honour (such as MMT_NO_MONS to specify that we shouldn't
@@ -94,26 +110,7 @@ struct mgen_data
     // map flags are usually respected only when a dungeon level is being
     // constructed, since at future points vault information may no longer
     // be available (vault metadata is not preserved across game saves).
-    unsigned        map_mask;
-
-    int             hd;
-    int             hp;
-
-    // These flags will be appended to the monster's flags after placement.
-    // These flags are MF_XXX, rather than MG_XXX flags.
-    uint64_t        extra_flags;
-
-    string          mname;
-
-    // This is used to account for non-actor summoners.  Blasted by an Ice
-    // Fiend ... summoned by the effects of Hell.
-    string          non_actor_summoner;
-
-    // This simply stores the initial shape-shifter type.
-    monster_type    initial_shifter;
-
-    // This simply stores chimera base monsters.
-    vector<monster_type> chimera_mons;
+    unsigned        map_mask = 0;
 
     // This can eventually be used to store relevant information.
     CrawlHashTable  props;
@@ -129,22 +126,21 @@ struct mgen_data
               god_type which_god = GOD_NO_GOD,
               monster_type base = MONS_NO_MONSTER,
               int monnumber = 0,
-              int moncolour = BLACK,
+              int moncolour = COLOUR_INHERIT,
               proximity_type prox = PROX_ANYWHERE,
               level_id _place = level_id::current(),
               int mhd = 0, int mhp = 0,
-              uint64_t extflags = 0,
+              monster_flags_t extflags = MF_NO_FLAGS,
               string monname = "",
               string nas = "",
               monster_type is = RANDOM_MONSTER)
 
-        : cls(mt), base_type(base), behaviour(beh), summoner(sner),
-          abjuration_duration(abj), summon_type(st), pos(p),
-          preferred_grid_feature(DNGN_UNSEEN), foe(mfoe), flags(genflags),
-          god(which_god), number(monnumber), colour(moncolour),
-          proximity(prox), place(_place), map_mask(0),
-          hd(mhd), hp(mhp), extra_flags(extflags), mname(monname),
-          non_actor_summoner(nas), initial_shifter(is), props()
+        : cls(mt), behaviour(beh), summoner(sner), abjuration_duration(abj),
+          summon_type(st), pos(p), foe(mfoe), flags(genflags), god(which_god),
+          base_type(base), number(monnumber), colour(moncolour),
+          proximity(prox), place(_place), hd(mhd), hp(mhp),
+          extra_flags(extflags), mname(monname), non_actor_summoner(nas),
+          initial_shifter(is)
     {
         ASSERT(summon_type == 0 || (abj >= 1 && abj <= 6)
                || mt == MONS_BALL_LIGHTNING || mt == MONS_ORB_OF_DESTRUCTION
@@ -152,10 +148,16 @@ struct mgen_data
                || summon_type == SPELL_STICKS_TO_SNAKES
                || summon_type == SPELL_DEATH_CHANNEL
                || summon_type == SPELL_SIMULACRUM
-               || summon_type == SPELL_AWAKEN_VINES);
+               || summon_type == SPELL_AWAKEN_VINES
+               || summon_type == SPELL_FULMINANT_PRISM);
+    }
+
+    bool permit_bands() const
+    {
+        // The permit flag is set but the forbid flag is not.
+        return (flags & (MG_PERMIT_BANDS|MG_FORBID_BANDS)) == MG_PERMIT_BANDS;
     }
 
-    bool permit_bands() const       { return flags & MG_PERMIT_BANDS; }
     bool force_place() const        { return flags & MG_FORCE_PLACE; }
     bool needs_patrol_point() const { return flags & MG_PATROLLING; }
 
@@ -165,14 +167,12 @@ struct mgen_data
 
     bool summoned() const { return abjuration_duration > 0; }
 
-    void define_chimera(monster_type part1, monster_type part2,
-                        monster_type part3);
-
     static mgen_data sleeper_at(monster_type what,
                                 const coord_def &where,
                                 unsigned genflags = 0)
     {
-        return mgen_data(what, BEH_SLEEP, 0, 0, 0, where, MHITNOT, genflags);
+        return mgen_data(what, BEH_SLEEP, 0, 0, MF_NO_FLAGS, where,
+                         MHITNOT, genflags);
     }
 
     static mgen_data hostile_at(monster_type mt,
@@ -188,9 +188,9 @@ struct mgen_data
     {
         return mgen_data(mt, BEH_HOSTILE, 0, abj, st, p,
                          alert ? MHITYOU : MHITNOT,
-                         genflags, ngod, base, 0, BLACK,
-                         PROX_ANYWHERE, level_id::current(), 0, 0, 0, "", nsummoner,
-                         RANDOM_MONSTER);
+                         genflags, ngod, base, 0, COLOUR_INHERIT,
+                         PROX_ANYWHERE, level_id::current(), 0, 0, MF_NO_FLAGS,
+                         "", nsummoner, RANDOM_MONSTER);
     }
 };
 
