@@ -4,53 +4,47 @@
 **/
 
 #include "AppHdr.h"
-#include "mon-behv.h"
 
-#include "externs.h"
+#include "mon-behv.h"
 
 #include "ability.h"
 #include "act-iter.h"
 #include "areas.h"
 #include "attitude-change.h"
-#include "coord.h"
 #include "coordit.h"
 #include "database.h"
 #include "dgn-overview.h"
 #include "dungeon.h"
-#include "env.h"
-#include "fprop.h"
 #include "exclude.h"
+#include "hints.h"
 #include "itemprop.h"
-#include "libutil.h"
 #include "losglobal.h"
 #include "macro.h"
+#include "message.h"
 #include "mon-act.h"
 #include "mon-death.h"
 #include "mon-movetarget.h"
-#include "mon-pathfind.h"
 #include "mon-speak.h"
 #include "ouch.h"
-#include "random.h"
 #include "religion.h"
+#include "shout.h"
 #include "spl-summoning.h"
 #include "state.h"
 #include "stringutil.h"
 #include "terrain.h"
 #include "traps.h"
-#include "hints.h"
 #include "view.h"
-#include "shout.h"
 
 static void _guess_invis_foe_pos(monster* mon)
 {
     const actor* foe          = mon->get_foe();
-    const int    guess_radius = mons_sense_invis(mon) ? 3 : 2;
+    const int    guess_radius = 2;
 
     vector<coord_def> possibilities;
 
     // NOTE: This depends on ignoring clouds, so that cells hidden by
     // opaque clouds are included as a possibility for the foe's location.
-    for (radius_iterator ri(mon->pos(), guess_radius, C_ROUND, LOS_SOLID); ri; ++ri)
+    for (radius_iterator ri(mon->pos(), guess_radius, C_SQUARE, LOS_SOLID); ri; ++ri)
     {
         if (foe->is_habitable(*ri))
             possibilities.push_back(*ri);
@@ -90,7 +84,7 @@ static void _mon_check_foe_invalid(monster* mon)
 static bool _mon_tries_regain_los(monster* mon)
 {
     // Only intelligent monsters with ranged attack will try to regain LOS.
-    if (mons_intel(mon) < I_NORMAL || !mons_has_ranged_attack(mon))
+    if (mons_intel(mon) < I_HUMAN || !mons_has_ranged_attack(mon))
         return false;
 
     // Any special case should go here.
@@ -102,8 +96,7 @@ static bool _mon_tries_regain_los(monster* mon)
     }
 
     // Randomize it a bit to make it less predictable.
-    return mons_intel(mon) == I_NORMAL && !one_chance_in(10)
-           || mons_intel(mon) == I_HIGH && !one_chance_in(20);
+    return !one_chance_in(10);
 }
 
 // Monster tries to get into a firing position. Among the cells which have
@@ -169,8 +162,8 @@ static void _decide_monster_firing_position(monster* mon, actor* owner)
                 // Get to firing range even if we are close.
                 _set_firing_pos(mon, you.pos());
             }
-        else if (mon->type == MONS_SIREN && !ignore_special_firing_AI)
-            find_siren_water_target(mon);
+        else if (mon->type == MONS_MERFOLK_AVATAR && !ignore_special_firing_AI)
+            find_merfolk_avatar_water_target(mon);
         else if (!mon->firing_pos.zero()
                  && mon->see_cell_no_trans(mon->target))
         {
@@ -185,7 +178,7 @@ static void _decide_monster_firing_position(monster* mon, actor* owner)
             // (or wasn't even attempted) and we need to set our target
             // the traditional way.
 
-            mon->target = PLAYER_POS;
+            mon->target = you.pos();
         }
     }
     else
@@ -203,8 +196,8 @@ static void _decide_monster_firing_position(monster* mon, actor* owner)
         }
         // Hold position if we've reached our ideal range
         else if (mon->type == MONS_SPELLFORGED_SERVITOR
-                 && (mon->pos() - target->pos()).abs()
-                 <= dist_range(mon->props["ideal_range"].get_int())
+                 && (mon->pos() - target->pos()).rdist()
+                 <= mon->props["ideal_range"].get_int()
                  && !one_chance_in(8))
         {
             mon->firing_pos = mon->pos();
@@ -212,18 +205,39 @@ static void _decide_monster_firing_position(monster* mon, actor* owner)
     }
 }
 
+/**
+ * Should the player be treated as if they were normally visible to a given
+ * monster, even though they're currently invisible?
+ *
+ * Random per-call; depends on proximity, monster intelligence, and Ash wrath.
+ *
+ * @param mon     The monster in question.
+ * @param         Whether the monster correctly guessed the player's presence.
+ */
+static bool _monster_guesses_invis_player(const monster &mon)
+{
+    // Sometimes, if a player is right next to a monster, they will 'see'.
+    if (grid_distance(you.pos(), mon.pos()) == 1 && one_chance_in(3))
+        return true;
+
+    // [dshaligram] Smart monsters have a chance of clueing in to
+    // invisible players in various ways.
+    if (mons_intel(&mon) == I_HUMAN && one_chance_in(12))
+        return true;
+
+    // Ash penance makes monsters very likely to target you through invis.
+    if (player_under_penance(GOD_ASHENZARI) && coinflip())
+        return true;
+
+    return false;
+}
+
 //---------------------------------------------------------------
 //
 // handle_behaviour
 //
 // 1. Evaluates current AI state
 // 2. Sets monster target x,y based on current foe
-//
-// XXX: Monsters of I_NORMAL or above should select a new target
-// if their current target is another monster which is sitting in
-// a wall and is immune to most attacks while in a wall, unless
-// the monster has a spell or special/nearby ability which isn't
-// affected by the wall.
 //---------------------------------------------------------------
 void handle_behaviour(monster* mon)
 {
@@ -244,14 +258,6 @@ void handle_behaviour(monster* mon)
         }
     }
 
-    // Grand avatar targeting is handled through its triggerers
-    // and _grand_avatar_act in mon-act.cc.
-    if (mon->type == MONS_GRAND_AVATAR)
-    {
-        mon->behaviour = BEH_SEEK;
-        return;
-    }
-
     bool changed = true;
     bool isFriendly = mon->friendly();
     bool isNeutral  = mon->neutral();
@@ -266,34 +272,13 @@ void handle_behaviour(monster* mon)
 
     bool proxFoe;
     bool isHealthy  = (mon->hit_points > mon->max_hit_points / 2);
-    bool isSmart    = (mons_intel(mon) > I_ANIMAL);
+    bool isSmart    = (mons_intel(mon) >= I_HUMAN);
     bool isScared   = mon->has_ench(ENCH_FEAR);
     bool isPacified = mon->pacified();
     bool patrolling = mon->is_patrolling();
     static vector<level_exit> e;
     static int                e_index = -1;
 
-    // Zotdef rotting
-    if (crawl_state.game_is_zotdef())
-    {
-        if (!isFriendly && !isNeutral && env.orb_pos == mon->pos()
-            && mon->speed)
-        {
-            const int loss = div_rand_round(10, mon->speed);
-            if (loss)
-            {
-                mprf(MSGCH_DANGER, "Your flesh rots away as the Orb of Zot is desecrated.");
-
-                // If the rot would reduce us to <= 0 max HP, attribute the
-                // kill to the monster.
-                if (loss >= you.hp_max)
-                    ouch(loss, mon->mindex(), KILLED_BY_ROTTING);
-
-                rot_hp(loss);
-            }
-        }
-    }
-
     //mprf("AI debug: mon %d behv=%d foe=%d pos=%d %d target=%d %d",
     //     mon->mindex(), mon->behaviour, mon->foe, mon->pos().x,
     //     mon->pos().y, mon->target.x, mon->target.y);
@@ -334,7 +319,7 @@ void handle_behaviour(monster* mon)
         }
 
         // Only go after the target if it's still near the owner, and
-        // so are we.  The weapon is restricted to a leash range of 2,
+        // so are we. The weapon is restricted to a leash range of 2,
         // and things reachable within that leash range [qoala]
         const int leash = 2;
 
@@ -363,43 +348,7 @@ void handle_behaviour(monster* mon)
     // Change proxPlayer depending on invisibility and standing
     // in shallow water.
     if (proxPlayer && !you.visible_to(mon))
-    {
-        proxPlayer = false;
-
-        const int intel = mons_intel(mon);
-        // Sometimes, if a player is right next to a monster, they will 'see'.
-        if (grid_distance(you.pos(), mon->pos()) == 1
-            && one_chance_in(3))
-        {
-            proxPlayer = true;
-        }
-
-        // [dshaligram] Very smart monsters have a chance of clueing in to
-        // invisible players in various ways.
-        if (intel == I_NORMAL && one_chance_in(13)
-                 || intel == I_HIGH && one_chance_in(6))
-        {
-            proxPlayer = true;
-        }
-
-        // Ash penance makes monsters very likely to target you through
-        // invisibility, depending on their intelligence.
-        if (player_under_penance(GOD_ASHENZARI) && x_chance_in_y(intel, 7))
-            proxPlayer = true;
-    }
-
-    // Zotdef: immobile allies forget targets that are out of sight
-    if (crawl_state.game_is_zotdef())
-    {
-        if (isFriendly && mon->is_stationary()
-            && (mon->foe != MHITNOT && mon->foe != MHITYOU)
-            && !mon->can_see(&menv[mon->foe]))
-        {
-            mon->foe = MHITYOU;
-            //mprf("%s resetting target (cantSee)",
-            //     mon->name(DESC_THE,true).c_str());
-        }
-    }
+        proxPlayer = _monster_guesses_invis_player(*mon);
 
     // Set friendly target, if they don't already have one.
     // Berserking allies ignore your commands!
@@ -411,29 +360,10 @@ void handle_behaviour(monster* mon)
         && mon->type != MONS_BALL_LIGHTNING
         && !mons_is_avatar(mon->type))
     {
-        if (!crawl_state.game_is_zotdef())
-        {
-            if (you.pet_target != MHITNOT)
-                mon->foe = you.pet_target;
-            else if (mons_class_is_stationary(mon->type))
-                set_nearest_monster_foe(mon);
-        }
-        else    // Zotdef only
-        {
-            // Attack pet target if nearby
-            if (you.pet_target != MHITNOT && proxPlayer)
-            {
-                //mprf("%s setting target (player target)",
-                //     mon->name(DESC_THE,true).c_str());
-                mon->foe = you.pet_target;
-            }
-            else
-            {
-                // Zotdef - this is all new, for out-of-sight friendlies to do
-                // something useful.  If no current target, get the closest one.
-                set_nearest_monster_foe(mon);
-            }
-        }
+        if (you.pet_target != MHITNOT)
+            mon->foe = you.pet_target;
+        else if (mons_class_is_stationary(mon->type))
+            set_nearest_monster_foe(mon);
     }
 
     // Instead, berserkers attack nearest monsters.
@@ -450,7 +380,7 @@ void handle_behaviour(monster* mon)
         if (!isFriendly
             && !mon->has_ench(ENCH_INSANE)
             && proxPlayer
-            && mons_intel(mon) >= I_NORMAL)
+            && mons_intel(mon) >= I_HUMAN)
         {
             mon->foe = MHITYOU;
         }
@@ -467,8 +397,6 @@ void handle_behaviour(monster* mon)
         && (proxPlayer || one_chance_in(3)))
     {
         set_nearest_monster_foe(mon);
-        if (mon->foe == MHITNOT && crawl_state.game_is_zotdef())
-            mon->foe = MHITYOU;
     }
 
     // Friendly summons will come back to the player if they go out of sight.
@@ -498,14 +426,12 @@ void handle_behaviour(monster* mon)
 
     // Unfriendly monsters fighting other monsters will usually
     // target the player, if they're healthy.
-    // Zotdef: 2/3 chance of retargeting changed to 1/4
     if (!isFriendly && !isNeutral
         && !mons_is_avatar(mon->type)
         && mon->foe != MHITYOU && mon->foe != MHITNOT
         && proxPlayer && !mon->berserk_or_insane()
         && isHealthy
-        && (crawl_state.game_is_zotdef() ? one_chance_in(4)
-                                         : !one_chance_in(3)))
+        && !one_chance_in(3))
     {
         mon->foe = MHITYOU;
     }
@@ -526,11 +452,11 @@ void handle_behaviour(monster* mon)
     while (changed)
     {
         actor* afoe = mon->get_foe();
-        proxFoe = afoe && mon->can_see(afoe);
+        proxFoe = afoe && mon->can_see(*afoe);
 
         if (mon->foe == MHITYOU)
         {
-            // monster::get_foe returns NULL for friendly monsters with
+            // monster::get_foe returns nullptr for friendly monsters with
             // foe == MHITYOU, so make afoe point to the player here.
             // -cao
             afoe = &you;
@@ -541,13 +467,6 @@ void handle_behaviour(monster* mon)
         if (afoe)
             foepos = afoe->pos();
 
-        if (crawl_state.game_is_zotdef() && mon->foe == MHITYOU
-            && !mon->wont_attack())
-        {
-            foepos = PLAYER_POS;
-            proxFoe = true;
-        }
-
         if (mon->pos() == mon->firing_pos)
             mon->firing_pos.reset();
 
@@ -588,7 +507,7 @@ void handle_behaviour(monster* mon)
                 else
                 {
                     new_foe = MHITYOU;
-                    mon->target = PLAYER_POS;
+                    mon->target = you.pos();
                 }
                 break;
             }
@@ -628,7 +547,7 @@ void handle_behaviour(monster* mon)
                     }
                 }
 
-                if (mon->travel_target == MTRAV_SIREN)
+                if (mon->travel_target == MTRAV_MERFOLK_AVATAR)
                     mon->travel_target = MTRAV_NONE;
 
                 // Spectral weapons simply seek back to their owner if
@@ -664,30 +583,23 @@ void handle_behaviour(monster* mon)
                 if (mon->foe_memory > 0)
                 {
                     // If we've arrived at our target x,y
-                    // do a stealth check.  If the foe
+                    // do a stealth check. If the foe
                     // fails, monster will then start
                     // tracking foe's CURRENT position,
                     // but only for a few moves (smell and
                     // intuition only go so far).
 
-                    if (mon->pos() == mon->target
-                        && (!isFriendly || !crawl_state.game_is_zotdef()))
-                    {   // hostiles only in Zotdef
+                  if (mon->pos() == mon->target)
+                    {
                         if (mon->foe == MHITYOU)
                         {
-                            // infallible tracking in zotdef
-                            if (crawl_state.game_is_zotdef())
-                                mon->target = PLAYER_POS;
-                            else
+                            if (x_chance_in_y(50, you.stealth())
+                                || you.penance[GOD_ASHENZARI] && coinflip())
                             {
-                                if (x_chance_in_y(50, you.stealth())
-                                    || you.penance[GOD_ASHENZARI] && coinflip())
-                                {
-                                    mon->target = you.pos();
-                                }
-                                else
-                                    mon->foe_memory = 0;
+                                mon->target = you.pos();
                             }
+                            else
+                                mon->foe_memory = 0;
                         }
                         else
                         {
@@ -732,18 +644,13 @@ void handle_behaviour(monster* mon)
             // Hack: smarter monsters will tend to pursue the player longer.
             switch (mons_intel(mon))
             {
-            case I_HIGH:
-                mon->foe_memory = random_range(700, 1300);
-                break;
-            case I_NORMAL:
-                mon->foe_memory = random_range(300, 700);
+            case I_HUMAN:
+                mon->foe_memory = random_range(450, 1000);
                 break;
             case I_ANIMAL:
-            case I_INSECT:
-            case I_REPTILE:
                 mon->foe_memory = random_range(250, 550);
                 break;
-            case I_PLANT:
+            case I_BRAINLESS:
                 mon->foe_memory = random_range(100, 300);
                 break;
             }
@@ -757,7 +664,7 @@ void handle_behaviour(monster* mon)
             {
                 // If a pacified monster isn't travelling toward
                 // someplace from which it can leave the level, make it
-                // start doing so.  If there's no such place, either
+                // start doing so. If there's no such place, either
                 // search the level for such a place again, or travel
                 // randomly.
                 if (mon->travel_target != MTRAV_PATROL)
@@ -822,7 +729,7 @@ void handle_behaviour(monster* mon)
 
             // During their wanderings, monsters will eventually relax
             // their guard (stupid ones will do so faster, smart
-            // monsters have longer memories).  Pacified monsters will
+            // monsters have longer memories). Pacified monsters will
             // also eventually switch the place from which they want to
             // leave the level, in case their current choice is blocked.
             if (!proxFoe && !mons_is_avatar(mon->type) && mon->foe != MHITNOT
@@ -921,7 +828,7 @@ void handle_behaviour(monster* mon)
 
             bool stop_retreat = false;
             // We've approached our next destination, re-evaluate
-            if (distance2(mon->target, mon->pos()) <= 3)
+            if (grid_distance(mon->target, mon->pos()) <= 1)
             {
                 // Continue on to the rally point
                 if (mon->target != mon->patrol_point)
@@ -931,7 +838,7 @@ void handle_behaviour(monster* mon)
                     stop_retreat = true;
 
             }
-            else if (distance2(mon->pos(), you.pos()) > dist_range(LOS_RADIUS + 2))
+            else if (grid_distance(mon->pos(), you.pos()) > LOS_RADIUS + 2)
             {
                 // We're too far from the player. Idle around and wait for
                 // them to catch up.
@@ -955,7 +862,7 @@ void handle_behaviour(monster* mon)
                 // idling (to prevent it from repeatedly resetting idle
                 // time if its own wanderings bring it closer to the player)
                 if (mon->props.exists("idle_point")
-                    && distance2(mon->pos(), you.pos()) < dist_range(LOS_RADIUS))
+                    && grid_distance(mon->pos(), you.pos()) < LOS_RADIUS)
                 {
                     mon->props.erase("idle_point");
                     mon->props.erase("idle_deadline");
@@ -1021,24 +928,16 @@ static bool _mons_check_foe(monster* mon, const coord_def& p,
     // -- But why should they always attack monsters? -- 1KB
 
     monster* foe = monster_at(p);
-    if (foe && foe != mon
-        && (mon->has_ench(ENCH_INSANE)
-            || foe->friendly() != friendly
-            || neutral && !foe->neutral())
-        && (ignore_sight || mon->can_see(foe))
-        && !foe->is_projectile()
-        && summon_can_attack(mon, p)
-        && (friendly || !is_sanctuary(p))
-        && (crawl_state.game_is_zotdef() || !mons_is_firewood(foe)))
-            // Zotdef allies take out firewood
-    {
-        return true;
-    }
-
-    if (mon->has_ench(ENCH_INSANE) && p == you.pos())
-        return true;
-
-    return false;
+    return foe && foe != mon
+           && (mon->has_ench(ENCH_INSANE)
+               || foe->friendly() != friendly
+               || neutral && !foe->neutral())
+           && (ignore_sight || mon->can_see(*foe))
+           && !foe->is_projectile()
+           && summon_can_attack(mon, p)
+           && (friendly || !is_sanctuary(p))
+           && !mons_is_firewood(foe)
+           || mon->has_ench(ENCH_INSANE) && p == you.pos();
 }
 
 // Choose random nearest monster as a foe.
@@ -1120,15 +1019,17 @@ void behaviour_event(monster* mon, mon_event_type event, const actor *src,
 
     const beh_type old_behaviour = mon->behaviour;
 
-    int fleeThreshold = min(mon->max_hit_points / 4, 20);
-
-    bool isSmart          = (mons_intel(mon) > I_ANIMAL);
+    bool isSmart          = (mons_intel(mon) >= I_HUMAN);
     bool setTarget        = false;
     bool breakCharm       = false;
-    bool was_sleeping     = mon->asleep();
+    bool was_unaware      = mon->asleep() || mon->foe == MHITNOT;
     string msg;
     int src_idx           = src ? src->mindex() : MHITNOT; // AXE ME
 
+    // Monsters know to blame you for reflecting things at them.
+    if (src_idx == YOU_FAULTLESS)
+        src_idx = MHITYOU;
+
     if (is_sanctuary(mon->pos()) && mons_is_fleeing_sanctuary(mon))
     {
         mon->behaviour = BEH_FLEE;
@@ -1150,7 +1051,7 @@ void behaviour_event(monster* mon, mon_event_type event, const actor *src,
         }
 
         // A bit of code to make Projected Noise actually do
-        // something again.  Basically, dumb monsters and
+        // something again. Basically, dumb monsters and
         // monsters who aren't otherwise occupied will at
         // least consider the (apparent) source of the noise
         // interesting for a moment. -- bwr
@@ -1175,15 +1076,15 @@ void behaviour_event(monster* mon, mon_event_type event, const actor *src,
             break;
 
         // Monster types that you can't gain experience from cannot
-        // fight back, so don't bother having them do so.  If you
+        // fight back, so don't bother having them do so. If you
         // worship Fedhas, create a ring of friendly plants, and try
         // to break out of the ring by killing a plant, you'll get
-        // a warning prompt and penance only once.  Without the
+        // a warning prompt and penance only once. Without the
         // hostility check, the plant will remain friendly until it
         // dies, and you'll get a warning prompt and penance once
-        // *per hit*.  This may not be the best way to address the
+        // *per hit*. This may not be the best way to address the
         // issue, though. -cao
-        if (mons_class_flag(mon->type, M_NO_EXP_GAIN)
+        if (!mons_class_gives_xp(mon->type)
             && mon->attitude != ATT_FRIENDLY
             && mon->attitude != ATT_GOOD_NEUTRAL)
         {
@@ -1198,14 +1099,18 @@ void behaviour_event(monster* mon, mon_event_type event, const actor *src,
         // If the monster can't reach its target and can't attack it
         // either, retreat.
         try_pathfind(mon);
-        if (mons_intel(mon) > I_REPTILE && !mons_can_attack(mon)
+        if (mons_intel(mon) > I_BRAINLESS && !mons_can_attack(mon)
             && target_is_unreachable(mon))
         {
             mon->behaviour = BEH_RETREAT;
         }
         else if (mon->has_ench(ENCH_FEAR))
         {
-            if (you.can_see(mon))
+            // self-attacks probably shouldn't break fear.
+            if (src == mon)
+                break;
+
+            if (you.can_see(*mon))
             {
                 mprf("%s attack snaps %s out of %s fear.",
                         src ? src->name(DESC_ITS).c_str() : "the",
@@ -1372,26 +1277,6 @@ void behaviour_event(monster* mon, mon_event_type event, const actor *src,
             mon->behaviour = BEH_SEEK;
         break;
 
-    case ME_HURT:
-        // Monsters with the M_FLEES flag can flee at low HP.
-        // Cannot flee if cornered.
-        // Monster can flee if HP is less than 1/4 maxhp or less than 20 hp
-        // (whichever is lower). Chance starts quite low, and is near 100% at 1.
-        // Monsters with less than 8 maxhp are unable to flee.
-        // These numbers could still use some adjusting.
-        //
-        // Assuming fleeThreshold is 20:
-        //   at 19 hp: 5% chance of fleeing
-        //   at 10 hp: 50% chance of fleeing
-        //   (chance increases by 5% for every hp lost.)
-        if (mons_class_flag(mon->type, M_FLEES)
-            && !mon->berserk_or_insane()
-            && x_chance_in_y(fleeThreshold - mon->hit_points, fleeThreshold))
-        {
-            mon->behaviour = BEH_FLEE;
-        }
-        break;
-
     case ME_EVAL:
         break;
     }
@@ -1405,6 +1290,11 @@ void behaviour_event(monster* mon, mon_event_type event, const actor *src,
         {
             // Why only attacks by the player change attitude? -- 1KB
             mon->attitude = ATT_HOSTILE;
+            // Non-hostile uniques might be removed from dungeon annotation
+            // so we add them back.
+            if (mon->props.exists("no_annotate"))
+                mon->props["no_annotate"] = false;
+            set_unique_annotation(mon);
             mons_att_changed(mon);
         }
     }
@@ -1426,8 +1316,8 @@ void behaviour_event(monster* mon, mon_event_type event, const actor *src,
 
     ASSERT_IN_BOUNDS_OR_ORIGIN(mon->target);
 
-    // If it woke up and you're its new foe, it might shout.
-    if (was_sleeping && !mon->asleep() && allow_shout
+    // If it was unaware of you and you're its new foe, it might shout.
+    if (was_unaware && !mon->asleep() && allow_shout
         && mon->foe == MHITYOU && !mon->wont_attack())
     {
         handle_monster_shouts(mon);
@@ -1506,7 +1396,7 @@ void alert_nearby_monsters()
 {
     // Judging from the above comment, this function isn't
     // intended to wake up monsters, so we're only going to
-    // alert monsters that aren't sleeping.  For cases where an
+    // alert monsters that aren't sleeping. For cases where an
     // event should wake up monsters and alert them, I'd suggest
     // calling noisy() before calling this function. - bwr
     for (monster_near_iterator mi(you.pos()); mi; ++mi)
@@ -1562,7 +1452,7 @@ static void _mons_indicate_level_exit(const monster* mon)
     {
         simple_monster_message(mon,
             make_stringf(" %s the shaft.",
-                mons_flies(mon) ? "goes down"
+                mon->airborne() ? "goes down"
                                 : "jumps into").c_str());
     }
 }
@@ -1571,11 +1461,8 @@ void make_mons_leave_level(monster* mon)
 {
     if (mon->pacified())
     {
-        if (you.can_see(mon))
-        {
+        if (you.can_see(*mon))
             _mons_indicate_level_exit(mon);
-            remove_unique_annotation(mon);
-        }
 
         // Pacified monsters leaving the level take their stuff with
         // them.
@@ -1600,22 +1487,19 @@ bool monster_can_hit_monster(monster* mons, const monster* targ)
         return false;
 
     const item_def *weapon = mons->weapon();
-    return weapon && melee_skill(*weapon) == SK_POLEARMS;
+    return weapon && item_attack_skill(*weapon) == SK_POLEARMS;
 }
 
 // Friendly summons can't attack out of the player's LOS, it's too abusable.
 bool summon_can_attack(const monster* mons)
 {
-    if (crawl_state.game_is_arena() || crawl_state.game_is_zotdef())
-        return true;
-
-    return !mons->friendly() || !mons->is_summoned()
+    return crawl_state.game_is_arena() || !mons->friendly() || !mons->is_summoned()
            || you.see_cell_no_trans(mons->pos());
 }
 
 bool summon_can_attack(const monster* mons, const coord_def &p)
 {
-    if (crawl_state.game_is_arena() || crawl_state.game_is_zotdef())
+    if (crawl_state.game_is_arena())
         return true;
 
     // Spectral weapons only attack their target
