@@ -17,7 +17,6 @@ enum mon_event_type
     ME_WHACK,                           // physical attack
     ME_SCARE,                           // frighten monster
     ME_CORNERED,                        // cannot flee
-    ME_HURT,                            // lost some HP (by any mean)
 };
 
 struct level_exit
@@ -48,8 +47,6 @@ void alert_nearby_monsters();
 
 void make_mons_stop_fleeing(monster* mon);
 
-void set_random_target(monster* mon);
-
 void make_mons_leave_level(monster* mon);
 
 bool monster_can_hit_monster(monster* mons, const monster* targ);
@@ -62,8 +59,4 @@ void shake_off_monsters(const actor* target);
 
 void set_nearest_monster_foe(monster* mon, bool near_player = false);
 
-// For Zotdef: the target position of MHITYOU monsters is
-// the orb.
-#define PLAYER_POS (crawl_state.game_is_zotdef() ? env.orb_pos : you.pos())
-
 #endif
