@@ -7,6 +7,7 @@
 #define MONBOOK_H
 
 #include <vector>
+
 #include "defines.h"
 #include "enum.h"
 #include "mon-mst.h"
@@ -14,13 +15,12 @@
 struct mon_spellbook
 {
     mon_spellbook_type type;
-    spell_type spells[NUM_MONSTER_SPELL_SLOTS];
+    vector<mon_spell_slot> spells;
 };
 
-extern const mon_spellbook mspell_list[];
-
 typedef vector<vector<spell_type> > unique_books;
 
 vector<mon_spellbook_type> get_spellbooks(const monster_info &mon);
-unique_books get_unique_spells(const monster_info &mon);
+unique_books get_unique_spells(const monster_info &mon,
+                               mon_spell_slot_flags flags = MON_SPELL_NO_FLAGS);
 #endif
