@@ -4,25 +4,22 @@
 **/
 
 #include "AppHdr.h"
+
 #include "mon-clone.h"
 
 #include "act-iter.h"
 #include "arena.h"
 #include "artefact.h"
+#include "coordit.h"
 #include "directn.h"
-#include "externs.h"
 #include "env.h"
 #include "items.h"
-#include "libutil.h"
+#include "message.h"
 #include "mgen_data.h"
-#include "monster.h"
 #include "mon-behv.h"
 #include "mon-death.h"
-#include "mon-enum.h"
 #include "mon-place.h"
-#include "mon-util.h"
-#include "player.h"
-#include "random.h"
+#include "mon-tentacle.h"
 #include "state.h"
 #include "stringutil.h"
 #include "terrain.h"
@@ -64,7 +61,10 @@ static bool _mons_is_illusion(monster* mons)
 
 static bool _mons_is_illusion_cloneable(monster* mons)
 {
-    return !_mons_is_illusion(mons) && !_monster_clone_exists(mons);
+    return !mons_is_conjured(mons->type)
+           && !mons_is_tentacle_or_tentacle_segment(mons->type)
+           && !_mons_is_illusion(mons)
+           && !_monster_clone_exists(mons);
 }
 
 static bool _player_is_illusion_cloneable()
@@ -118,12 +118,13 @@ static void _mons_summon_monster_illusion(monster* caster,
         clone->del_ench(ENCH_STICKY_FLAME);
         clone->del_ench(ENCH_CORONA);
         clone->del_ench(ENCH_SILVER_CORONA);
+        clone->del_ench(ENCH_HEXED);
 
         behaviour_event(clone, ME_ALERT, 0, caster->pos());
 
         if (cloning_visible)
         {
-            if (!you.can_see(caster))
+            if (!you.can_see(*caster))
             {
                 mprf("%s seems to step out of %s!",
                      foe->name(DESC_THE).c_str(),
@@ -211,8 +212,6 @@ void mons_summon_illusion_from(monster* mons, actor *foe,
             else
                 mprf(MSGCH_WARN, "There is a horrible, sudden wrenching feeling in your soul!");
 
-            // Change type from player ghost.
-            clone->type = MONS_PLAYER_ILLUSION;
             _init_player_illusion_properties(
                 get_monster_data(MONS_PLAYER_ILLUSION));
             _mons_load_player_enchantments(mons, clone);
@@ -229,7 +228,7 @@ void mons_summon_illusion_from(monster* mons, actor *foe,
 
 bool mons_clonable(const monster* mon, bool needs_adjacent)
 {
-    // No uniques or ghost demon monsters.  Also, figuring out the name
+    // No uniques or ghost demon monsters. Also, figuring out the name
     // for the clone of a named monster isn't worth it, and duplicate
     // battlespheres with the same owner cause problems with the spell
     if (mons_is_unique(mon->type)
@@ -244,13 +243,11 @@ bool mons_clonable(const monster* mon, bool needs_adjacent)
     {
         // Is there space for the clone?
         bool square_found = false;
-        for (int i = 0; i < 8; i++)
+        for (adjacent_iterator ai(mon->pos()); ai; ++ai)
         {
-            const coord_def p = mon->pos() + Compass[i];
-
-            if (in_bounds(p)
-                && !actor_at(p)
-                && monster_habitable_grid(mon, grd(p)))
+            if (in_bounds(*ai)
+                && !actor_at(*ai)
+                && monster_habitable_grid(mon, grd(*ai)))
             {
                 square_found = true;
                 break;
@@ -282,38 +279,37 @@ monster* clone_mons(const monster* orig, bool quiet, bool* obvious,
     monster* mons = get_free_monster();
 
     if (!mons)
-        return 0;
+        return nullptr;
 
     if (!in_bounds(pos))
     {
-        // Find an adjacent square.
-        int squares = 0;
-        for (int i = 0; i < 8; i++)
+        for (fair_adjacent_iterator ai(orig->pos()); ai; ++ai)
         {
-            const coord_def p = orig->pos() + Compass[i];
-
-            if (in_bounds(p)
-                && !actor_at(p)
-                && monster_habitable_grid(orig, grd(p)))
+            if (in_bounds(*ai)
+                && !actor_at(*ai)
+                && monster_habitable_grid(orig, grd(*ai)))
             {
-                if (one_chance_in(++squares))
-                    pos = p;
+                pos = *ai;
             }
         }
 
-        if (squares == 0)
-            return 0;
+        if (!in_bounds(pos))
+            return nullptr;
     }
 
     ASSERT(!actor_at(pos));
+    ASSERT_IN_BOUNDS(pos);
 
     *mons          = *orig;
     mons->set_new_monster_id();
-    mons->set_position(pos);
+    mons->move_to_pos(pos);
     // The monster copy constructor doesn't copy constriction, so no need to
     // worry about that.
 
-    mgrd(pos)    = mons->mindex();
+    // Don't copy death triggers - phantom royal jellies should not open the
+    // Slime vaults on death.
+    if (mons->props.exists(MONSTER_DIES_LUA_KEY))
+        mons->props.erase(MONSTER_DIES_LUA_KEY);
 
     // Duplicate objects, or unequip them if they can't be duplicated.
     for (int i = 0; i < NUM_MONSTER_SLOTS; i++)
@@ -326,22 +322,22 @@ monster* clone_mons(const monster* orig, bool quiet, bool* obvious,
         const int new_index = get_mitm_slot(0);
         if (new_index == NON_ITEM)
         {
-            mons->unequip(mitm[old_index], i, 0, true);
+            mons->unequip(mitm[old_index], false, true);
             mons->inv[i] = NON_ITEM;
             continue;
         }
 
         mons->inv[i]      = new_index;
         mitm[new_index] = mitm[old_index];
-        mitm[new_index].set_holding_monster(mons->mindex());
+        mitm[new_index].set_holding_monster(*mons);
     }
 
     bool _obvious;
-    if (obvious == NULL)
+    if (obvious == nullptr)
         obvious = &_obvious;
     *obvious = false;
 
-    if (you.can_see(orig) && you.can_see(mons))
+    if (you.can_see(*orig) && you.can_see(*mons))
     {
         if (!quiet)
             simple_monster_message(orig, " is duplicated!");
@@ -349,7 +345,7 @@ monster* clone_mons(const monster* orig, bool quiet, bool* obvious,
     }
 
     mark_interesting_monst(mons, mons->behaviour);
-    if (you.can_see(mons))
+    if (you.can_see(*mons))
     {
         handle_seen_interrupt(mons);
         viewwindow();
