@@ -7,7 +7,7 @@
 // Formerly in mon-stuff:
 bool mons_clonable(const monster* orig, bool needs_adjacent = true);
 monster *clone_mons(const monster* orig, bool quiet = false,
-                    bool* obvious = NULL, coord_def pos = coord_def(0, 0));
+                    bool* obvious = nullptr, coord_def pos = coord_def(0, 0));
 
 void mons_summon_illusion_from(monster* mons, actor *foe,
                                spell_type spell_cast = SPELL_NO_SPELL,
