@@ -15,19 +15,8 @@ enum corpse_effect_type
 {
     CE_NOCORPSE,
     CE_CLEAN,
-    CE_POISONOUS,
-    CE_ROT,
+    CE_NOXIOUS,
     CE_MUTAGEN,
-    CE_ROTTEN,
-};
-
-enum gender_type
-{
-    GENDER_NEUTER,
-    GENDER_MALE,
-    GENDER_FEMALE,
-    GENDER_YOU, // A person, not a gender, but close enough.
-    NUM_GENDERS
 };
 
 // TODO: Unify this and a player_equivalent (if applicable)
@@ -36,6 +25,7 @@ enum attack_type
 {
     AT_NONE,
     AT_HIT,         // Including weapon attacks.
+    AT_FIRST_ATTACK = AT_HIT,
     AT_BITE,
     AT_STING,
     AT_SPORE,
@@ -58,6 +48,8 @@ enum attack_type
 #endif
     AT_POUNCE,
     AT_REACH_STING,
+    AT_LAST_REAL_ATTACK = AT_REACH_STING,
+
     AT_CHERUB,
 #if TAG_MAJOR_VERSION == 34
     AT_SHOOT,
@@ -128,6 +120,11 @@ enum attack_flavour
     AF_SHADOWSTAB,
     AF_DROWN,
     AF_FIREBRAND,
+    AF_CORRODE,
+    AF_SCARAB,
+    AF_KITE,  // Hops backwards if attacking with a polearm.
+    AF_SWOOP, // Swoops in to perform a melee attack if far away.
+    AF_TRAMPLE, // Trampling effect.
 };
 
 // Non-spell "summoning" types to give to monster::mark_summoned(), or
@@ -144,18 +141,17 @@ enum mon_summon_type
     MON_SUMM_WRATH,   // Divine wrath
     MON_SUMM_AID,     // Divine aid
     MON_SUMM_SCROLL,  // Scroll of summoning
+    MON_SUMM_SHADOW,  // Shadow trap
+    MON_SUMM_LANTERN, // Lantern of shadows
 };
 
 #include "mon-flags.h"
 
 enum mon_intel_type             // Must be in increasing intelligence order
 {
-    I_PLANT = 0,
-    I_INSECT,
-    I_REPTILE,
+    I_BRAINLESS = 0,
     I_ANIMAL,
-    I_NORMAL,
-    I_HIGH,
+    I_HUMAN,
 };
 
 enum habitat_type
@@ -181,18 +177,6 @@ enum mon_itemuse_type
     NUM_MONUSE
 };
 
-enum mon_itemeat_type
-{
-    MONEAT_NOTHING,
-    MONEAT_ITEMS,
-    MONEAT_CORPSES,
-#if TAG_MAJOR_VERSION == 34
-    MONEAT_FOOD,
-#endif
-
-    NUM_MONEAT
-};
-
 typedef uint32_t resists_t;
 #define mrd(res, lev) (resists_t)((res) * ((lev) & 7))
 
@@ -214,14 +198,17 @@ enum mon_resist_flags
 
     MR_LAST_MULTI, // must be >= any multi, < any boolean, exact value doesn't matter
 
-    MR_RES_ASPHYX        = 1 << 24,
+    MR_RES_TORMENT       = 1 << 22,
+    MR_RES_PETRIFY       = 1 << 23,
 #if TAG_MAJOR_VERSION == 34
+    MR_OLD_RES_ASPHYX    = 1 << 24,
     MR_OLD_RES_ACID      = 1 << 25,
 #else
+    // unused 1 << 24,
     // unused 1 << 25,
 #endif
     MR_RES_STICKY_FLAME  = 1 << 26,
-    // unused 1 << 27,
+    MR_RES_WIND          = 1 << 27,
     MR_RES_STEAM         = 1 << 28,
 
     // vulnerabilities
@@ -237,6 +224,7 @@ enum shout_type
     S_SILENT,               // silent
     S_SHOUT,                // shout
     S_BARK,                 // bark
+    S_HOWL,                 // howl
     S_SHOUT2,               // shout twice (e.g. two-headed ogres)
     S_ROAR,                 // roar
     S_SCREAM,               // scream
@@ -246,13 +234,15 @@ enum shout_type
     S_BUZZ,                 // buzz
     S_MOAN,                 // moan
     S_GURGLE,               // gurgle
-    S_WHINE,                // irritating whine (mosquitos)
     S_CROAK,                // frog croak
     S_GROWL,                // for bears
     S_HISS,                 // for snakes and lizards
     S_DEMON_TAUNT,          // for pandemonium lords
+#if TAG_MAJOR_VERSION == 34
     S_CAW,                  // for ravens
+#endif
     S_CHERUB,               // for cherubs
+    S_RUMBLE,               // for ushabti
     NUM_SHOUTS,
 
     // Loudness setting for shouts that are only defined in dat/shout.txt
@@ -280,12 +270,14 @@ enum zombie_size_type
  */
 enum mon_body_shape
 {
+    MON_SHAPE_BUGGY,
     MON_SHAPE_HUMANOID,
     MON_SHAPE_HUMANOID_WINGED,
     MON_SHAPE_HUMANOID_TAILED,
     MON_SHAPE_HUMANOID_WINGED_TAILED,
     MON_SHAPE_CENTAUR,
     MON_SHAPE_NAGA,
+    // Everything before this should have arms
     MON_SHAPE_QUADRUPED,
     MON_SHAPE_QUADRUPED_TAILLESS,
     MON_SHAPE_QUADRUPED_WINGED,
