@@ -8,7 +8,8 @@
 
 void give_specific_item(monster* mon, const item_def& tpl);
 void give_item(monster *mon, int level_number,
-               bool mons_summoned, bool spectral_orcs = false);
+               bool mons_summoned, bool spectral_orcs = false, bool merc = false);
 void give_weapon(monster *mon, int level_number, bool mons_summoned,
                  bool spectral_orcs = false);
+void give_armour(monster *mon, int level_number);
 #endif
