@@ -5,13 +5,12 @@
 
 #include "AppHdr.h"
 
-#include "enum.h"
 #include "mon-grow.h"
-#include "mon-message.h"
-#include "mon-util.h"
+
+#include "message.h"
 #include "mon-place.h"
 #include "monster.h"
-#include "random.h"
+#include "mon-util.h"
 
 // Base experience required by a monster to reach HD 1.
 const int monster_xp_base       = 15;
@@ -59,12 +58,10 @@ static const monster_level_up mon_grow[] =
     monster_level_up(MONS_DEEP_ELF_SUMMONER, MONS_DEEP_ELF_DEMONOLOGIST, 500),
     monster_level_up(MONS_DEEP_ELF_SUMMONER, MONS_DEEP_ELF_SORCERER),
 
-    monster_level_up(MONS_BABY_ALLIGATOR, MONS_ALLIGATOR),
-
     monster_level_up(MONS_GNOLL, MONS_GNOLL_SERGEANT),
 
     monster_level_up(MONS_MERFOLK, MONS_MERFOLK_IMPALER),
-    monster_level_up(MONS_MERMAID, MONS_SIREN),
+    monster_level_up(MONS_SIREN, MONS_MERFOLK_AVATAR),
 
     // Spriggan -> rider is no good (no mount), -> defender would be an insane
     // power jump, -> druid or -> air mage would require magic training,
@@ -97,9 +94,8 @@ mons_experience_levels::mons_experience_levels()
 static const monster_level_up *_monster_level_up_target(monster_type type,
                                                         int hit_dice)
 {
-    for (unsigned i = 0; i < ARRAYSZ(mon_grow); ++i)
+    for (const monster_level_up &mlup : mon_grow)
     {
-        const monster_level_up &mlup(mon_grow[i]);
         if (mlup.before == type)
         {
             const monsterentry *me = get_monster_data(mlup.after);
@@ -110,13 +106,12 @@ static const monster_level_up *_monster_level_up_target(monster_type type,
             }
         }
     }
-    return NULL;
+    return nullptr;
 }
 
 void monster::upgrade_type(monster_type after, bool adjust_hd,
                             bool adjust_hp)
 {
-    const monsterentry *orig = get_monster_data(type);
     // Ta-da!
     type   = after;
 
@@ -132,10 +127,6 @@ void monster::upgrade_type(monster_type after, bool adjust_hd,
     spells = dummy.spells;
     calc_speed();
 
-    const monsterentry *m = get_monster_data(after);
-    ac += m->AC - orig->AC;
-    ev += m->ev - orig->ev;
-
     if (adjust_hd)
         set_hit_dice(max(get_experience_level(), dummy.get_hit_dice()));
 
@@ -158,8 +149,8 @@ void monster::upgrade_type(monster_type after, bool adjust_hd,
 
 bool monster::level_up_change()
 {
-    const monster_level_up *lup = _monster_level_up_target(type,
-                                                           get_experience_level());
+    const monster_level_up *lup =
+        _monster_level_up_target(type, get_experience_level());
     if (lup)
     {
         upgrade_type(lup->after, false, lup->adjust_hp);
@@ -168,7 +159,8 @@ bool monster::level_up_change()
     else if (mons_is_base_draconian(type))
     {
         base_monster = type;
-        monster_type upgrade = resolve_monster_type(RANDOM_NONBASE_DRACONIAN, type);
+        monster_type upgrade = resolve_monster_type(RANDOM_NONBASE_DRACONIAN,
+                                                    type);
         if (static_cast<int>(get_monster_data(upgrade)->hpdice[0]) == get_experience_level())
             upgrade_type(upgrade, false, true);
     }
@@ -228,7 +220,7 @@ bool monster::gain_exp(int exp, int max_levels_to_gain)
         return false;
 
     // Only monsters that you can gain XP from can level-up.
-    if (mons_class_flag(type, M_NO_EXP_GAIN) || is_summoned())
+    if (!mons_class_gives_xp(type) || is_summoned())
         return false;
 
     // Avoid wrap-around.
@@ -247,7 +239,7 @@ bool monster::gain_exp(int exp, int max_levels_to_gain)
 
     if (levels_gained)
     {
-        if (mons_intel(this) >= I_NORMAL)
+        if (mons_intel(this) >= I_HUMAN)
             simple_monster_message(&mcopy, " looks more experienced.");
         else
             simple_monster_message(&mcopy, " looks stronger.");
