@@ -8,39 +8,33 @@
 #include "coordit.h"
 #include "env.h"
 #include "fprop.h"
-#include "items.h"
 #include "itemprop.h"
+#include "items.h"
 #include "libutil.h"
 #include "los_def.h"
 #include "losglobal.h"
 #include "mon-behv.h"
 #include "mon-pathfind.h"
 #include "mon-place.h"
-#include "random.h"
 #include "state.h"
 #include "terrain.h"
 #include "traps.h"
 
 // If a monster can see but not directly reach the target, and then fails to
-// find a path to get there, mark all surrounding (in a radius of √8) monsters
+// find a path to get there, mark all surrounding (in a radius of 2) monsters
 // of the same (or greater) movement restrictions as also being unable to
 // find a path, so we won't need to calculate again.
 // Should there be a direct path to the target for a monster thus marked, it
 // will still be able to come nearer (and the mark will then be cleared).
 static void _mark_neighbours_target_unreachable(monster* mon)
 {
-    // Highly intelligent monsters are perfectly capable of pathfinding
-    // and don't need their neighbour's advice.
     const mon_intel_type intel = mons_intel(mon);
-    if (intel > I_NORMAL)
-        return;
-
-    const bool flies         = mons_flies(mon);
+    const bool flies         = mon->airborne();
     const bool amphibious    = (mons_habitat(mon) == HT_AMPHIBIOUS);
     const bool amph_lava     = (mons_habitat(mon) == HT_AMPHIBIOUS_LAVA);
     const habitat_type habit = mons_primary_habitat(mon);
 
-    for (radius_iterator ri(mon->pos(), 8, C_CIRCLE); ri; ++ri)
+    for (radius_iterator ri(mon->pos(), 2, C_SQUARE); ri; ++ri)
     {
         if (*ri == mon->pos())
             continue;
@@ -51,7 +45,7 @@ static void _mark_neighbours_target_unreachable(monster* mon)
             continue;
 
         monster* const m = monster_at(*ri);
-        if (m == NULL)
+        if (m == nullptr)
             continue;
 
         // Don't restrict smarter monsters as they might find a path
@@ -69,7 +63,7 @@ static void _mark_neighbours_target_unreachable(monster* mon)
 
         // A flying monster has an advantage over a non-flying one.
         // Same for a swimming one.
-        if (!flies && mons_flies(m)
+        if (!flies && m->airborne()
             || !amphibious && mons_habitat(m) == HT_AMPHIBIOUS
             || !amph_lava  && mons_habitat(m) == HT_AMPHIBIOUS_LAVA)
         {
@@ -88,33 +82,6 @@ static void _set_no_path_found(monster* mon)
 #ifdef DEBUG_PATHFIND
     mpr("No path found!");
 #endif
-    if (crawl_state.game_is_zotdef() && player_in_branch(root_branch)
-        && !testbits(env.pgrid(mon->pos()), FPROP_NO_RTELE_INTO))
-    {
-        if (you.wizard)
-        {
-            // You might have used a wizard power to teleport into a wall or
-            // a loot chamber.
-            mprf(MSGCH_ERROR, "Monster %s failed to pathfind to (%d,%d) (%s)",
-                mon->name(DESC_PLAIN, true).c_str(),
-                env.orb_pos.x, env.orb_pos.y,
-                orb_position().origin() ? "you" : "the Orb");
-        }
-        else
-        {
-            // None of the maps allows the goal to ever become unreachable,
-            // and when that happens, let's crash rather than a give an
-            // effortless win with all the opposition doing nothing.
-
-            // This is only appropriate in the zotdef map itself, though,
-            // which is why we check for BRANCH_DUNGEON above.
-            // (This kind of thing is totally normal in, say, a Bazaar.)
-            die("ZotDef: monster %s failed to pathfind to (%d,%d) (%s)",
-                mon->name(DESC_PLAIN, true).c_str(),
-                env.orb_pos.x, env.orb_pos.y,
-                orb_position().origin() ? "you" : "the Orb");
-        }
-    }
 
     mon->travel_target = MTRAV_UNREACHABLE;
     // Pass information on to nearby monsters.
@@ -156,7 +123,7 @@ bool try_pathfind(monster* mon)
     // pathfinding.
     if (need_pathfind
         && !mon->friendly()
-        && mon->can_see(foe)
+        && mon->can_see(*foe)
         && mons_has_los_ability(mon->type))
     {
         need_pathfind = false;
@@ -165,7 +132,7 @@ bool try_pathfind(monster* mon)
     // Also don't use pathfinding if the monster can shoot
     // across the blocking terrain, and is smart enough to
     // realise that.
-    if ((!crawl_state.game_is_zotdef()) && need_pathfind
+    if (need_pathfind
         && !mon->friendly()
         && mons_has_ranged_attack(mon)
         && cell_see_cell(mon->pos(), targpos, LOS_SOLID_SEE))
@@ -236,8 +203,7 @@ bool try_pathfind(monster* mon)
 #ifdef DEBUG_PATHFIND
     mprf("Need to calculate a path... (dist = %d)", dist);
 #endif
-    // All monsters can find the Orb in Zotdef
-    const int range = (crawl_state.game_is_zotdef() || mon->friendly() ? 1000 : mons_tracking_range(mon));
+    const int range = mon->friendly() ? 1000 : mons_tracking_range(mon);
 
     if (range > 0 && dist > range)
     {
@@ -263,7 +229,7 @@ bool try_pathfind(monster* mon)
         mon->travel_path = mp.calc_waypoints();
         if (!mon->travel_path.empty())
         {
-            // Okay then, we found a path.  Let's use it!
+            // Okay then, we found a path. Let's use it!
             mon->target = mon->travel_path[0];
             mon->travel_target = MTRAV_FOE;
             return true;
@@ -301,7 +267,7 @@ bool pacified_leave_level(monster* mon, vector<level_exit> e, int e_index)
     // player, make it leave the level.
     if (_is_level_exit(mon->pos())
         || (e_index != -1 && mon->pos() == e[e_index].target)
-        || distance2(mon->pos(), you.pos()) >= dist_range(LOS_RADIUS * 4))
+        || grid_distance(mon->pos(), you.pos()) >= LOS_RADIUS * 3)
     {
         make_mons_leave_level(mon);
         return true;
@@ -310,10 +276,10 @@ bool pacified_leave_level(monster* mon, vector<level_exit> e, int e_index)
     return false;
 }
 
-// Assesses how desirable a spot is to a siren (preferring spaces surrounded
-// by water, at least one of which is deep, and with at least one neighbour
-// which is inhabitable without swimming or flight)
-static int _siren_water_score(coord_def p, bool& deep)
+// Assesses how desirable a spot is to a merfolk avatar (preferring spaces
+// surrounded by water, at least one of which is deep, and with at least one
+// neighbour which is inhabitable without swimming or flight)
+static int _merfolk_avatar_water_score(coord_def p, bool& deep)
 {
     int score = 0;
     bool near_floor = false;
@@ -352,12 +318,12 @@ static int _siren_water_score(coord_def p, bool& deep)
 
 // Pick the nearest water grid that is surrounded by the most
 // water squares within LoS.
-bool find_siren_water_target(monster* mon)
+bool find_merfolk_avatar_water_target(monster* mon)
 {
-    ASSERT(mon->type == MONS_SIREN);
+    ASSERT(mon->type == MONS_MERFOLK_AVATAR);
 
     // Moving away could break the entrancement, so don't do this.
-    if (distance2(mon->pos(), you.pos()) >= 6 * 6 + 1)
+    if (grid_distance(mon->pos(), you.pos()) >= 5)
     {
         mon->firing_pos.reset();
         return false;
@@ -367,18 +333,18 @@ bool find_siren_water_target(monster* mon)
 
     // If our current location is good enough, don't bother moving towards
     // some other spot which might be somewhat better
-    if (_siren_water_score(mon->pos(), deep) >= 12 && deep
+    if (_merfolk_avatar_water_score(mon->pos(), deep) >= 12 && deep
         && grd(mon->pos()) == DNGN_DEEP_WATER)
     {
         mon->firing_pos = mon->pos();
         return true;
     }
 
-    if (mon->travel_target == MTRAV_SIREN)
+    if (mon->travel_target == MTRAV_MERFOLK_AVATAR)
     {
         coord_def targ_pos(mon->travel_path[mon->travel_path.size() - 1]);
 #ifdef DEBUG_PATHFIND
-        mprf("siren target is (%d, %d), dist = %d",
+        mprf("merfolk avatar target is (%d, %d), dist = %d",
              targ_pos.x, targ_pos.y, (int) (mon->pos() - targ_pos).rdist());
 #endif
         if ((mon->pos() - targ_pos).rdist() > 2)
@@ -401,12 +367,12 @@ bool find_siren_water_target(monster* mon)
                 continue;
 
             // In the first iteration only count water grids that are
-            // not closer to the player than to the siren.
+            // not closer to the player than to the merfolk avatar.
             if (first && (mon->pos() - *ri).rdist() > (you.pos() - *ri).rdist())
                 continue;
 
             // Counts deep water twice.
-            const int water_count = _siren_water_score(*ri, deep);
+            const int water_count = _merfolk_avatar_water_score(*ri, deep);
             if (water_count < best_water_count)
                 continue;
 
@@ -466,16 +432,18 @@ bool find_siren_water_target(monster* mon)
         {
 #ifdef WIZARD
             if (you.wizard)
-                for (unsigned int i = 0; i < mon->travel_path.size(); i++)
-                    env.pgrid(mon->travel_path[i]) |= FPROP_HIGHLIGHT;
+            {
+                for (coord_def pos : mon->travel_path)
+                    env.pgrid(pos) |= FPROP_HIGHLIGHT;
+            }
 #endif
 #ifdef DEBUG_PATHFIND
             mprf("Found a path to (%d, %d) with %d surrounding water squares",
                  best_target.x, best_target.y, best_water_count);
 #endif
-            // Okay then, we found a path.  Let's use it!
+            // Okay then, we found a path. Let's use it!
             mon->firing_pos = mon->travel_path[0];
-            mon->travel_target = MTRAV_SIREN;
+            mon->travel_target = MTRAV_MERFOLK_AVATAR;
             return true;
         }
     }
@@ -596,38 +564,39 @@ static bool _handle_monster_travelling(monster* mon)
 
 static bool _choose_random_patrol_target_grid(monster* mon)
 {
-    const int intel = mons_intel(mon);
+    const mon_intel_type intel = mons_intel(mon);
 
     // Zombies will occasionally just stand around.
     // This does not mean that they don't move every second turn. Rather,
     // once they reach their chosen target, there's a 50% chance they'll
     // just remain there until next turn when this function is called
     // again.
-    if (intel == I_PLANT && coinflip())
+    if (intel == I_BRAINLESS && coinflip())
         return true;
 
     // If there's no chance we'll find the patrol point, quit right away.
-    if (distance2(mon->pos(), mon->patrol_point) > dist_range(2 * LOS_RADIUS))
+    if (grid_distance(mon->pos(), mon->patrol_point) > 2 * LOS_RADIUS)
         return false;
 
     // Can the monster see the patrol point from its current position?
     const bool patrol_seen = mon->see_cell(mon->patrol_point);
 
-    if (intel == I_PLANT && !patrol_seen)
+    if (intel == I_BRAINLESS && !patrol_seen)
     {
         // Really stupid monsters won't even try to get back into the
         // patrol zone.
         return false;
     }
 
-    // While the patrol point is in easy reach, monsters of insect/plant
-    // intelligence will only use a range of 5 (distance from the patrol point).
+    // While the patrol point is in easy reach, monsters of brainless
+    // intelligence will only use a range of 4 (distance from the patrol point).
     // Otherwise, try to get back using the full LOS.
-    const int  rad      = (intel >= I_ANIMAL || !patrol_seen) ? LOS_RADIUS : 5;
-    const bool is_smart = (intel >= I_NORMAL);
+    const int  rad      = (intel > I_BRAINLESS || !patrol_seen) ? LOS_RADIUS
+                                                                : 4;
+    const bool is_smart = (intel >= I_HUMAN);
 
     los_def patrol(mon->patrol_point, opacity_monmove(*mon),
-                   circle_def(rad, C_ROUND));
+                   circle_def(rad, C_SQUARE));
     patrol.update();
     los_def lm(mon->pos(), opacity_monmove(*mon));
     if (is_smart || !patrol_seen)
@@ -639,7 +608,7 @@ static bool _choose_random_patrol_target_grid(monster* mon)
     int count_grids = 0;
     // Don't bother for the current position. If everything fails,
     // we'll stay here anyway.
-    for (radius_iterator ri(mon->patrol_point, you.current_vision, C_ROUND, true);
+    for (radius_iterator ri(mon->patrol_point, you.current_vision, C_SQUARE, true);
          ri; ++ri)
     {
         if (!in_bounds(*ri) || !mon->can_pass_through_feat(grd(*ri)))
@@ -690,7 +659,7 @@ static bool _choose_random_patrol_target_grid(monster* mon)
         }
 
         bool set_target = false;
-        if (intel == I_PLANT && *ri == mon->patrol_point)
+        if (intel == I_BRAINLESS && *ri == mon->patrol_point)
         {
             // Slightly greater chance to simply head for the centre.
             count_grids += 3;
@@ -778,6 +747,9 @@ void set_random_target(monster* mon)
         if (!summon_can_attack(mon, newtarget))
             continue;
 
+        if (!mon->is_location_safe(newtarget))
+            continue;
+
         mon->target = newtarget;
         break;
     }
@@ -788,7 +760,7 @@ static monster * _active_band_leader(monster * mon)
 {
     // Not a band member
     if (!mon->props.exists("band_leader"))
-        return NULL;
+        return nullptr;
 
     // Try to find our fearless leader.
     unsigned leader_mid = mon->props["band_leader"].get_int();
@@ -802,7 +774,7 @@ static bool _band_wander_target(monster * mon)
 {
     int dist_thresh = LOS_RADIUS + HERD_COMFORT_RANGE;
     monster * band_leader = _active_band_leader(mon);
-    if (band_leader == NULL)
+    if (band_leader == nullptr)
         return true;
 
     int leader_dist = grid_distance(mon->pos(), band_leader->pos());
@@ -816,7 +788,7 @@ static bool _band_wander_target(monster * mon)
             mon->travel_path = mp.calc_waypoints();
             if (!mon->travel_path.empty())
             {
-                // Okay then, we found a path.  Let's use it!
+                // Okay then, we found a path. Let's use it!
                 mon->target = mon->travel_path[0];
                 mon->travel_target = MTRAV_PATROL;
                 return false;
@@ -835,6 +807,9 @@ static bool _band_wander_target(monster * mon)
         if (!in_bounds(*r_it))
             continue;
 
+        if (!band_leader->is_location_safe(*r_it))
+            continue;
+
         int dist = grid_distance(*r_it, band_leader->pos());
         if (dist < HERD_COMFORT_RANGE)
             positions.push_back(*r_it);
@@ -859,7 +834,7 @@ static bool _herd_wander_target(monster * mon)
 
     for (monster_iterator mit; mit; ++mit)
     {
-        if (mit->mindex() == mon->mindex()
+        if (*mit == mon
             || mons_genus(mit->type) != mons_genus(mon->type)
             || grid_distance(mit->pos(), mon->pos()) > dist_thresh)
         {
@@ -877,11 +852,14 @@ static bool _herd_wander_target(monster * mon)
         if (!in_bounds(*r_it))
             continue;
 
+        if (!mon->is_location_safe(*r_it))
+            continue;
+
         int count = 0;
-        for (unsigned i = 0; i < friends.size(); i++)
+        for (monster_iterator &fr_it : friends)
         {
-            if (grid_distance(friends[i]->pos(), *r_it) < HERD_COMFORT_RANGE
-                && friends[i]->see_cell_no_trans(*r_it))
+            if (grid_distance(fr_it->pos(), *r_it) < HERD_COMFORT_RANGE
+                && fr_it->see_cell_no_trans(*r_it))
             {
                 count++;
             }
@@ -908,7 +886,7 @@ static bool _herd_ok(monster * mon)
     // herdlings magically know others even out of LOS
     for (monster_iterator mit; mit; ++mit)
     {
-        if (mit->mindex() == mon->mindex())
+        if (*mit == mon)
             continue;
 
         if (mons_genus(mit->type) == mons_genus(mon->type))
@@ -975,7 +953,7 @@ void check_wander_target(monster* mon, bool isPacified)
             need_target = _herd_wander_target(mon);
 
         if (need_target
-            && _active_band_leader(mon) != NULL)
+            && _active_band_leader(mon) != nullptr)
         {
             need_target = _band_wander_target(mon);
         }
@@ -984,7 +962,7 @@ void check_wander_target(monster* mon, bool isPacified)
         // changing the goal square every turn, better would be to
         // have the monster store a direction and have the monster
         // head in that direction for a while, then shift the
-        // direction to the left or right.  We're changing this so
+        // direction to the left or right. We're changing this so
         // wandering monsters at least appear to have some sort of
         // attention span.  -- bwr
         if (need_target)
@@ -1020,7 +998,7 @@ int mons_find_nearest_level_exit(const monster* mon, vector<level_exit> &e,
         if (e[i].unreachable)
             continue;
 
-        int dist = distance2(mon->pos(), e[i].target);
+        int dist = grid_distance(mon->pos(), e[i].target);
 
         if (old_dist == -1 || old_dist >= dist)
         {
@@ -1075,7 +1053,7 @@ static bool _can_safely_go_through(const monster * mon, const coord_def p)
     // Stupid monsters don't pathfind around shallow water
     // except the clinging ones.
     if (mon->floundering_at(p)
-        && (mons_intel(mon) >= I_NORMAL || mon->can_cling_to_walls()))
+        && (mons_intel(mon) >= I_HUMAN || mon->can_cling_to_walls()))
     {
         return false;
     }
@@ -1098,7 +1076,7 @@ bool can_go_straight(const monster* mon, const coord_def& p1,
     if (p1 == p2)
         return true;
 
-    if (distance2(p1, p2) > los_radius2)
+    if (grid_distance(p1, p2) > los_radius)
         return false;
 
     // XXX: Hack to improve results for now. See FIXME above.
