@@ -6,11 +6,8 @@
 #ifndef MONPICK_H
 #define MONPICK_H
 
-#include "externs.h"
 #include "random-pick.h"
 
-#define DEPTH_NOWHERE 999
-
 #define pop_entry random_pick_entry<monster_type>
 
 typedef bool (*mon_pick_vetoer)(monster_type);
@@ -26,6 +23,7 @@ monster_type pick_monster_no_rarity(branch_type branch);
 monster_type pick_monster_by_hash(branch_type branch, uint32_t hash);
 monster_type pick_monster_all_branches(int absdepth0, mon_pick_vetoer veto = nullptr);
 int branch_ood_cap(branch_type branch);
+bool branch_has_monsters(branch_type branch);
 const pop_entry* fish_population(branch_type br, bool lava);
 const pop_entry* zombie_population(branch_type br);
 
@@ -44,7 +42,7 @@ public:
                                 monster_type none,
                                 mon_pick_vetoer vetoer = nullptr);
 
-    virtual bool veto(monster_type mon);
+    virtual bool veto(monster_type mon) override;
 
 private:
     mon_pick_vetoer _veto;
@@ -57,7 +55,7 @@ public:
                               mon_pick_pos_vetoer _posveto = nullptr)
         : monster_picker(), pos(_pos), posveto(_posveto) { };
 
-    virtual bool veto(monster_type mon);
+    virtual bool veto(monster_type mon) override;
 
 protected:
     const coord_def &pos;
@@ -74,7 +72,7 @@ public:
           zombie_kind(_ztype)
           { };
 
-    virtual bool veto(monster_type mon);
+    virtual bool veto(monster_type mon) override;
 
 private:
     monster_type zombie_kind;
