@@ -4,45 +4,35 @@
 **/
 
 #include "AppHdr.h"
+
 #include "mon-poly.h"
 
 #include "artefact.h"
 #include "attitude-change.h"
-#include "butcher.h"
 #include "delay.h"
 #include "describe.h"
 #include "dgn-overview.h"
 #include "dungeon.h"
-#include "env.h"
 #include "exclude.h"
+#include "fineff.h"
 #include "godconduct.h"
 #include "hints.h"
 #include "itemprop.h"
 #include "items.h"
 #include "libutil.h"
-#include "monster.h"
-#include "mon-flags.h"
+#include "message.h"
 #include "mon-death.h"
-#include "mon-message.h"
 #include "mon-place.h"
-#include "mon-util.h"
+#include "mon-tentacle.h"
 #include "notes.h"
-#include "player.h"
 #include "religion.h"
 #include "state.h"
 #include "stringutil.h"
 #include "terrain.h"
 #include "traps.h"
+#include "view.h"
 #include "xom.h"
 
-dungeon_feature_type get_mimic_feat(const monster* mimic)
-{
-    if (mimic->props.exists("feat_type"))
-        return static_cast<dungeon_feature_type>(mimic->props["feat_type"].get_short());
-    else
-        return DNGN_FLOOR;
-}
-
 bool feature_mimic_at(const coord_def &c)
 {
     return map_masked(c, MMT_MIMIC);
@@ -53,7 +43,7 @@ item_def* item_mimic_at(const coord_def &c)
     for (stack_iterator si(c); si; ++si)
         if (si->flags & ISFLAG_MIMIC)
             return &*si;
-    return NULL;
+    return nullptr;
 }
 
 bool mimic_at(const coord_def &c)
@@ -61,16 +51,9 @@ bool mimic_at(const coord_def &c)
     return feature_mimic_at(c) || item_mimic_at(c);
 }
 
-// The default suitable() function for monster_drop_things().
-bool is_any_item(const item_def& item)
-{
-    return true;
-}
-
 void monster_drop_things(monster* mons,
                           bool mark_item_origins,
-                          bool (*suitable)(const item_def& item),
-                          int owner_id)
+                          bool (*suitable)(const item_def& item))
 {
     // Drop weapons and missiles last (i.e., on top), so others pick up.
     for (int i = NUM_MONSTER_SLOTS - 1; i >= 0; --i)
@@ -83,7 +66,7 @@ void monster_drop_things(monster* mons,
                 testbits(mitm[item].flags, ISFLAG_SUMMONED);
             if (summoned_item)
             {
-                item_was_destroyed(mitm[item], mons->mindex());
+                item_was_destroyed(mitm[item]);
                 destroy_item(item);
             }
             else
@@ -129,13 +112,23 @@ static bool _valid_morph(monster* mons, monster_type new_mclass)
         return false;
     }
 
+    // [hm] Lower base draconian chances since there are nine of them,
+    // and they shouldn't each count for a full chance.
+    if (mons_genus(new_mclass) == MONS_DRACONIAN
+        && new_mclass != MONS_DRACONIAN
+        && !one_chance_in(9))
+    {
+        return false;
+    }
+
     // Various inappropriate polymorph targets.
     if (mons_class_holiness(new_mclass) != mons_class_holiness(old_mclass)
         || mons_class_flag(new_mclass, M_UNFINISHED)  // no unfinished monsters
         || mons_class_flag(new_mclass, M_CANT_SPAWN)  // no dummy monsters
         || mons_class_flag(new_mclass, M_NO_POLY_TO)  // explicitly disallowed
         || mons_class_flag(new_mclass, M_UNIQUE)      // no uniques
-        || mons_class_flag(new_mclass, M_NO_EXP_GAIN) // not helpless
+        || !mons_class_gives_xp(new_mclass)           // no tentacle parts or
+                                                      // harmless things
         || new_mclass == MONS_PROGRAM_BUG
 
         // 'morph targets are _always_ "base" classes, not derived ones.
@@ -156,10 +149,7 @@ static bool _valid_morph(monster* mons, monster_type new_mclass)
         // Other poly-unsuitable things.
         || mons_is_statue(new_mclass)
         || mons_is_projectile(new_mclass)
-        || mons_is_tentacle_or_tentacle_segment(new_mclass)
 
-        // Don't polymorph things without Gods into priests.
-        || (mons_class_flag(new_mclass, MF_PRIEST) && mons->god == GOD_NO_GOD)
         // The spell on Prince Ribbit can't be broken so easily.
         || (new_mclass == MONS_HUMAN
             && (mons->type == MONS_PRINCE_RIBBIT
@@ -203,11 +193,12 @@ static bool _jiyva_slime_target(monster_type targetc)
 
 void change_monster_type(monster* mons, monster_type targetc)
 {
-    bool could_see     = you.can_see(mons);
+    ASSERT(mons); // XXX: change to monster &mons
+    bool could_see     = you.can_see(*mons);
     bool slimified = _jiyva_slime_target(targetc);
 
     // Quietly remove the old monster's invisibility before transforming
-    // it.  If we don't do this, it'll stay invisible even after losing
+    // it. If we don't do this, it'll stay invisible even after losing
     // the invisibility enchantment below.
     mons->del_ench(ENCH_INVIS, false, false);
 
@@ -218,23 +209,32 @@ void change_monster_type(monster* mons, monster_type targetc)
 
     // Even if the monster transforms from one type that can behold the
     // player into a different type which can also behold the player,
-    // the polymorph disrupts the beholding process.  Do this before
+    // the polymorph disrupts the beholding process. Do this before
     // changing mons->type, since unbeholding can only happen while
-    // the monster is still a mermaid/siren.
+    // the monster is still a siren/merfolk avatar.
     you.remove_beholder(mons);
     you.remove_fearmonger(mons);
 
     if (mons_is_tentacle_head(mons_base_type(mons)))
         destroy_tentacles(mons);
 
+    // trj spills out jellies when polied, as if he'd been hit for mhp.
+    if (mons->type == MONS_ROYAL_JELLY)
+    {
+        simple_monster_message(mons, "'s form twists and warps, and jellies "
+                               "spill out!");
+        trj_spawn_fineff::schedule(nullptr, mons, mons->pos(),
+                                   mons->hit_points);
+    }
+
     // Inform listeners that the original monster is gone.
     fire_monster_death_event(mons, KILL_MISC, NON_MONSTER, true);
 
     // the actual polymorphing:
-    uint64_t flags =
+    auto flags =
         mons->flags & ~(MF_INTERESTING | MF_SEEN | MF_ATT_CHANGE_ATTEMPT
                            | MF_WAS_IN_VIEW | MF_BAND_MEMBER | MF_KNOWN_SHIFTER
-                           | MF_MELEE_MASK | MF_SPELL_MASK);
+                           | MF_MELEE_MASK);
     flags |= MF_POLYMORPHED;
     string name;
 
@@ -300,15 +300,16 @@ void change_monster_type(monster* mons, monster_type targetc)
     const bool old_mon_caught     = mons->caught();
     const char old_ench_countdown = mons->ench_countdown;
 
-    // XXX: mons_is_unique should be converted to monster::is_unique, and that
-    // function should be testing the value of props["original_was_unique"]
-    // which would make things a lot simpler.
-    // See also record_monster_defeat.
-    bool old_mon_unique           = mons_is_unique(mons->type);
-    if (mons->props.exists("original_was_unique")
-        && mons->props["original_was_unique"].get_bool())
+    const bool old_mon_unique = mons_is_or_was_unique(*mons);
+
+    if (!mons->props.exists(ORIGINAL_TYPE_KEY))
     {
-        old_mon_unique = true;
+        const monster_type type = mons_is_job(mons->type)
+                                ? draco_or_demonspawn_subspecies(mons)
+                                : mons->type;
+        mons->props[ORIGINAL_TYPE_KEY].get_int() = type;
+        if (mons->mons_species() == MONS_HYDRA)
+            mons->props["old_heads"].get_int() = mons->num_heads;
     }
 
     mon_enchant abj       = mons->get_ench(ENCH_ABJ);
@@ -321,12 +322,13 @@ void change_monster_type(monster* mons, monster_type targetc)
     mon_enchant tp        = mons->get_ench(ENCH_TP);
     mon_enchant vines     = mons->get_ench(ENCH_AWAKEN_VINES);
     mon_enchant forest    = mons->get_ench(ENCH_AWAKEN_FOREST);
+    mon_enchant hexed     = mons->get_ench(ENCH_HEXED);
 
     monster_spells spl    = mons->spells;
     const bool need_save_spells
-            = (!name.empty()
-               && (!mons->can_use_spells() || mons->is_actual_spellcaster())
-               && !slimified);
+            =  old_mon_unique && !slimified
+               && mons_class_intel(targetc) >= I_HUMAN
+               && (!mons->has_spells() || mons->is_actual_spellcaster());
 
     mons->number       = 0;
 
@@ -341,9 +343,7 @@ void change_monster_type(monster* mons, monster_type targetc)
     }
 
     mons->mname = name;
-    mons->props["original_name"] = name;
-    mons->props["original_was_unique"] = old_mon_unique;
-    mons->god   = god;
+    mons->props["no_annotate"] = slimified && old_mon_unique;
     mons->props.erase("dbname");
 
     mons->flags = flags;
@@ -354,20 +354,20 @@ void change_monster_type(monster* mons, monster_type targetc)
     // Forget various speech/shout Lua functions.
     mons->props.erase("speech_prefix");
 
-    // Don't allow polymorphing monsters for hides.
-    mons->props[NEVER_HIDE_KEY] = true;
-
     // Keep spells for named monsters, but don't override innate ones
     // for dragons and the like. This means that Sigmund polymorphed
     // into a goblin will still cast spells, but if he ends up as a
     // swamp drake he'll breathe fumes and, if polymorphed further,
     // won't remember his spells anymore.
     if (need_save_spells
-        && (!mons->can_use_spells() || mons->is_actual_spellcaster()))
+        && (!mons->has_spells() || mons->is_actual_spellcaster()))
     {
         mons->spells = spl;
     }
 
+    // Make sure we have a god if we've been polymorphed into a priest.
+    mons->god = mons->is_priest() ? GOD_NAMELESS : god;
+
     mons->add_ench(abj);
     mons->add_ench(fabj);
     mons->add_ench(charm);
@@ -377,6 +377,7 @@ void change_monster_type(monster* mons, monster_type targetc)
     mons->add_ench(tp);
     mons->add_ench(vines);
     mons->add_ench(forest);
+    mons->add_ench(hexed);
 
     // Allows for handling of submerged monsters which polymorph into
     // monsters that can't submerge on this square.
@@ -407,7 +408,7 @@ void change_monster_type(monster* mons, monster_type targetc)
     mark_interesting_monst(mons);
 
     // If new monster is visible to player, then we've seen it.
-    if (you.can_see(mons))
+    if (you.can_see(*mons))
     {
         seen_monster(mons);
         // If the player saw both the beginning and end results of a
@@ -420,9 +421,9 @@ void change_monster_type(monster* mons, monster_type targetc)
         check_net_will_hold_monster(mons);
 
     // Even if the new form can constrict, it might be with a different
-    // body part.  Likewise, the new form might be too large for its
-    // current constrictor.  Rather than trying to handle these as special
-    // cases, just stop the constriction entirely.  The usual message about
+    // body part. Likewise, the new form might be too large for its
+    // current constrictor. Rather than trying to handle these as special
+    // cases, just stop the constriction entirely. The usual message about
     // evaporating and reforming justifies this behaviour.
     mons->stop_constricting_all(false);
     mons->stop_being_constricted();
@@ -448,7 +449,7 @@ bool monster_polymorph(monster* mons, monster_type targetc,
     int tries = 1000;
 
     // Used to be mons_power, but that just returns hit_dice
-    // for the monster class.  By using the current hit dice
+    // for the monster class. By using the current hit dice
     // the player gets the opportunity to use draining more
     // effectively against shapeshifters. - bwr
     source_power = mons->get_hit_dice();
@@ -487,7 +488,7 @@ bool monster_polymorph(monster* mons, monster_type targetc,
                                                         target_power, relax)));
     }
 
-    bool could_see = you.can_see(mons);
+    bool could_see = you.can_see(*mons);
     bool need_note = (could_see && MONST_INTERESTING(mons));
     string old_name_a = mons->full_name(DESC_A);
     string old_name_the = mons->full_name(DESC_THE);
@@ -496,21 +497,20 @@ bool monster_polymorph(monster* mons, monster_type targetc,
     if (targetc == RANDOM_TOUGHER_MONSTER)
     {
         vector<monster_type> target_types;
-        for (int mc = 0; mc < NUM_MONSTERS; ++mc)
+        for (monster_type mc = MONS_0; mc < NUM_MONSTERS; ++mc)
         {
-            const monsterentry *me = get_monster_data((monster_type) mc);
+            const monsterentry *me = get_monster_data(mc);
             int delta = (int) me->hpdice[0] - mons->get_hit_dice();
             if (delta != 1)
                 continue;
-            if (!_valid_morph(mons, (monster_type) mc))
+            if (!_valid_morph(mons, mc))
                 continue;
-            target_types.push_back((monster_type) mc);
+            target_types.push_back(mc);
         }
         if (target_types.empty())
             return false;
 
-        shuffle_array(target_types);
-        targetc = target_types[0];
+        targetc = target_types[random2(target_types.size())];
     }
 
     if (!_valid_morph(mons, targetc))
@@ -518,7 +518,7 @@ bool monster_polymorph(monster* mons, monster_type targetc,
 
     change_monster_type(mons, targetc);
 
-    bool can_see = you.can_see(mons);
+    bool can_see = you.can_see(*mons);
 
     // Messaging
     bool player_messaged = true;
@@ -557,8 +557,7 @@ bool monster_polymorph(monster* mons, monster_type targetc,
         string new_name = can_see ? mons->full_name(DESC_A)
                                   : "something unseen";
 
-        take_note(Note(NOTE_POLY_MONSTER, 0, 0, old_name_a.c_str(),
-                       new_name.c_str()));
+        take_note(Note(NOTE_POLY_MONSTER, 0, 0, old_name_a, new_name));
 
         if (can_see)
             mons->flags |= MF_SEEN;
@@ -622,6 +621,9 @@ void slimify_monster(monster* mon, bool hostile)
         return;
     }
 
+    record_monster_defeat(mon, KILL_SLIMIFIED);
+    remove_unique_annotation(mon);
+
     monster_polymorph(mon, target);
 
     if (!mons_eats_items(mon))
@@ -651,50 +653,34 @@ void seen_monster(monster* mons)
     item_def* weapon = mons->weapon();
     if (weapon && is_range_weapon(*weapon))
         mons->flags |= MF_SEEN_RANGED;
+    mark_mon_equipment_seen(mons);
 
     // Monster was viewed this turn
     mons->flags |= MF_WAS_IN_VIEW;
 
-    // mark items as seen.
-    for (int slot = MSLOT_WEAPON; slot <= MSLOT_LAST_VISIBLE_SLOT; slot++)
-    {
-        int item_id = mons->inv[slot];
-        if (item_id != NON_ITEM)
-            mitm[item_id].flags |= ISFLAG_SEEN;
-    }
-
     if (mons->flags & MF_SEEN)
         return;
 
     // First time we've seen this particular monster.
     mons->flags |= MF_SEEN;
 
-    if (!mons_is_mimic(mons->type))
-    {
-        if (crawl_state.game_is_hints())
-            hints_monster_seen(*mons);
+    if (crawl_state.game_is_hints())
+        hints_monster_seen(*mons);
 
-        if (MONST_INTERESTING(mons))
+    if (MONST_INTERESTING(mons))
+    {
+        string name = mons->name(DESC_A, true);
+        if (mons->type == MONS_PLAYER_GHOST)
         {
-            string name = mons->name(DESC_A, true);
-            if (mons->type == MONS_PLAYER_GHOST)
-            {
-                name += make_stringf(" (%s)",
-                        short_ghost_description(mons, true).c_str());
-            }
-            take_note(
-                      Note(NOTE_SEEN_MONSTER, mons->type, 0,
-                           name.c_str()));
+            name += make_stringf(" (%s)",
+                                 short_ghost_description(mons, true).c_str());
         }
+        take_note(Note(NOTE_SEEN_MONSTER, mons->type, 0, name));
     }
 
     if (!(mons->flags & MF_TSO_SEEN))
     {
-        if (!mons->has_ench(ENCH_ABJ)
-            && !mons->has_ench(ENCH_FAKE_ABJURATION)
-            && !testbits(mons->flags, MF_NO_REWARD)
-            && !mons_class_flag(mons->type, M_NO_EXP_GAIN)
-            && !crawl_state.game_is_arena())
+        if (mons_gives_xp(mons, &you) && !crawl_state.game_is_arena())
         {
             did_god_conduct(DID_SEE_MONSTER, mons->get_experience_level(),
                             true, mons);
