@@ -7,22 +7,21 @@
 
 #include "mon-project.h"
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
 #include <cmath>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 
-#include "externs.h"
-
+#include "act-iter.h"
 #include "areas.h"
 #include "cloud.h"
 #include "directn.h"
 #include "env.h"
 #include "itemprop.h"
+#include "message.h"
 #include "mgen_data.h"
 #include "mon-death.h"
 #include "mon-place.h"
-#include "mon-util.h"
 #include "ouch.h"
 #include "shout.h"
 #include "stepdown.h"
@@ -69,10 +68,10 @@ spret_type cast_iood(actor *caster, int pow, bolt *beam, float vx, float vy,
             beam->ray.r.start.x - 0.5, beam->ray.r.start.y - 0.5,
             beam->ray.r.dir.x, beam->ray.r.dir.y);
 #endif
-        mon->props["iood_x"].get_float() = beam->ray.r.start.x - 0.5;
-        mon->props["iood_y"].get_float() = beam->ray.r.start.y - 0.5;
-        mon->props["iood_vx"].get_float() = beam->ray.r.dir.x;
-        mon->props["iood_vy"].get_float() = beam->ray.r.dir.y;
+        mon->props[IOOD_X].get_float() = beam->ray.r.start.x - 0.5;
+        mon->props[IOOD_Y].get_float() = beam->ray.r.start.y - 0.5;
+        mon->props[IOOD_VX].get_float() = beam->ray.r.dir.x;
+        mon->props[IOOD_VY].get_float() = beam->ray.r.dir.y;
         _fuzz_direction(caster, *mon, pow);
     }
     else
@@ -80,17 +79,17 @@ spret_type cast_iood(actor *caster, int pow, bolt *beam, float vx, float vy,
         // Multi-orb: spread the orbs a bit, otherwise diagonal ones might
         // fail to leave the cardinal direction: orb A moves -0.4,+0.9 and
         // orb B +0.4,+0.9, both rounded to 0,1.
-        mon->props["iood_x"].get_float() = caster->pos().x + 0.4 * vx;
-        mon->props["iood_y"].get_float() = caster->pos().y + 0.4 * vy;
-        mon->props["iood_vx"].get_float() = vx;
-        mon->props["iood_vy"].get_float() = vy;
+        mon->props[IOOD_X].get_float() = caster->pos().x + 0.4 * vx;
+        mon->props[IOOD_Y].get_float() = caster->pos().y + 0.4 * vy;
+        mon->props[IOOD_VX].get_float() = vx;
+        mon->props[IOOD_VY].get_float() = vy;
     }
 
-    mon->props["iood_kc"].get_byte() = (is_player) ? KC_YOU :
+    mon->props[IOOD_KC].get_byte() = (is_player) ? KC_YOU :
         ((monster*)caster)->friendly() ? KC_FRIENDLY : KC_OTHER;
-    mon->props["iood_pow"].get_short() = pow;
+    mon->props[IOOD_POW].get_short() = pow;
     mon->flags &= ~MF_JUST_SUMMONED;
-    mon->props["iood_caster"].get_string() = caster->as_monster()
+    mon->props[IOOD_CASTER].get_string() = caster->as_monster()
         ? caster->name(DESC_A, true)
         : (caster->is_player()) ? "you" : "";
     mon->summoner = caster->mid;
@@ -98,40 +97,120 @@ spret_type cast_iood(actor *caster, int pow, bolt *beam, float vx, float vy,
     if (caster->is_player() || caster->type == MONS_PLAYER_GHOST
         || caster->type == MONS_PLAYER_ILLUSION)
     {
-        mon->props["iood_flawed"].get_byte() = true;
+        mon->props[IOOD_FLAWED].get_byte() = true;
     }
 
     // Move away from the caster's square.
     iood_act(*mon, true);
-    // We need to take at least one full move (for the above), but let's
-    // randomize it and take more so players won't get guaranteed instant
-    // damage.
-    mon->lose_energy(EUT_MOVE, 2, random2(3)+2);
 
-    // Multi-orbs don't home during the first move, they'd likely
-    // immediately explode otherwise.
-    if (foe != MHITNOT)
-        mon->foe = foe;
+    // If the foe was adjacent to the caster, that might have destroyed it.
+    if (mon->alive())
+    {
+        // We need to take at least one full move (for the above), but let's
+        // randomize it and take more so players won't get guaranteed instant
+        // damage.
+        mon->lose_energy(EUT_MOVE, 2, random2(3)+2);
+
+        // Multi-orbs don't home during the first move, they'd likely
+        // immediately explode otherwise.
+        if (foe != MHITNOT)
+            mon->foe = foe;
+    }
 
     return SPRET_SUCCESS;
 }
 
-void cast_iood_burst(int pow, coord_def target)
+/**
+ * Find a target for a bursty (non-player-targeted) IOOD.
+ *
+ * Try to find an enemy that's at a reasonable angle from the angle the IOOD
+ * is fired at, preferring the given foe (if a non-MHITNOT foe is given) if
+ * they're valid, and otherwise preferring the closest valid foe.
+ *
+ * @param angle             The angle that the IOOD will be fired at, relative
+ *                          to the player's position.
+ * @param preferred_foe     The mindex of a target to choose if possible; may
+ *                          be MHITNOT (no preferred target)
+ * @return                  The mindex of a valid target for the IOOD.
+ */
+static int _burst_iood_target(double iood_angle, int preferred_foe)
 {
-    int foe = MHITNOT;
-    if (const monster* mons = monster_at(target))
+    int closest_foe = MHITNOT;
+    int closest_dist = INT_MAX;
+
+    for (monster_near_iterator mi(you.pos(), LOS_SOLID); mi; ++mi)
     {
-        if (mons && you.can_see(mons))
-            foe = mons->mindex();
+        const monster* m = *mi;
+        ASSERT(m);
+
+        if (!you.can_see(*m) || mons_is_projectile(m))
+            continue;
+
+        // is this position at a valid angle?
+        const coord_def delta = mi->pos() - you.pos();
+        const double angle = atan2(delta.x, delta.y);
+        const double abs_angle_diff = abs(angle - fmod(iood_angle, PI * 2));
+        const double angle_diff = (abs_angle_diff > PI) ?
+                                        2 * PI - abs_angle_diff :
+                                        abs_angle_diff;
+        if (angle_diff >= PI / 3)
+        {
+            dprf("can't target %s; angle diff %f",
+                 m->name(DESC_PLAIN).c_str(), angle_diff);
+            continue;
+        }
+
+        // if preferred foe is valid, choose it.
+        if (m->mindex() == preferred_foe)
+        {
+            dprf("preferred target %s is valid burst target (delta %f)",
+                 m->name(DESC_PLAIN).c_str(), angle_diff);
+            return preferred_foe;
+        }
+
+        if (mons_aligned(m, &you) || mons_is_firewood(m))
+        {
+            dprf("skipping invalid burst target %s (%s)",
+                 m->name(DESC_PLAIN).c_str(),
+                 mons_aligned(m, &you) ? "aligned" : "firewood");
+            continue;
+        }
+
+        const int dist = grid_distance(m->pos(), you.pos());
+        // on distance ties, bias by iterator order (mindex)
+        if (dist >= closest_dist)
+        {
+            dprf("%s not closer to target than closest (%d vs %d)",
+                 m->name(DESC_PLAIN).c_str(), dist, closest_dist);
+            continue;
+        }
+
+        dprf("%s is valid burst target (delta %f, dist %d)",
+             m->name(DESC_PLAIN).c_str(), angle_diff, dist);
+        closest_dist = dist;
+        closest_foe = m->mindex();
     }
 
-    int n_orbs = random_range(3, 7);
+    const int foe = closest_foe != MHITNOT ? closest_foe : preferred_foe;
+    dprf("targeting %d", foe);
+    return foe;
+}
+
+void cast_iood_burst(int pow, coord_def target)
+{
+    const monster* mons = monster_at(target);
+    const int preferred_foe = mons && you.can_see(*mons) ?
+                              mons->mindex() :
+                              MHITNOT;
+
+    const int n_orbs = random_range(3, 7);
     dprf("Bursting %d orbs.", n_orbs);
-    double angle0 = random2(2097152) * PI / 1048576;
+    const double angle0 = random2(2097152) * PI * 2 / 2097152;
 
     for (int i = 0; i < n_orbs; i++)
     {
-        double angle = angle0 + i * PI * 2 / n_orbs;
+        const double angle = angle0 + i * PI * 2 / n_orbs;
+        const int foe = _burst_iood_target(angle, preferred_foe);
         cast_iood(&you, pow, 0, sin(angle), cos(angle), foe);
     }
 }
@@ -173,10 +252,10 @@ static void _iood_stop(monster& mon, bool msg = true)
 
 static void _fuzz_direction(const actor *caster, monster& mon, int pow)
 {
-    const float x = mon.props["iood_x"];
-    const float y = mon.props["iood_y"];
-    float vx = mon.props["iood_vx"];
-    float vy = mon.props["iood_vy"];
+    const float x = mon.props[IOOD_X];
+    const float y = mon.props[IOOD_Y];
+    float vx = mon.props[IOOD_VX];
+    float vy = mon.props[IOOD_VY];
 
     _normalize(vx, vy);
 
@@ -185,15 +264,16 @@ static void _fuzz_direction(const actor *caster, monster& mon, int pow)
     const float off = (coinflip() ? -1 : 1) * 0.25;
     float tan = (random2(31) - 15) * 0.019; // approx from degrees
     tan *= 75.0 / pow;
-    if (caster && caster->inaccuracy())
-        tan *= 2;
+    int inaccuracy = caster->inaccuracy();
+    if (caster && inaccuracy > 0)
+        tan *= 2 * inaccuracy;
 
     // Cast either from left or right hand.
-    mon.props["iood_x"] = x + vy*off;
-    mon.props["iood_y"] = y - vx*off;
+    mon.props[IOOD_X] = x + vy*off;
+    mon.props[IOOD_Y] = y - vx*off;
     // And off the direction a bit.
-    mon.props["iood_vx"] = vx + vy*tan;
-    mon.props["iood_vy"] = vy - vx*tan;
+    mon.props[IOOD_VX] = vx + vy*tan;
+    mon.props[IOOD_VY] = vy - vx*tan;
 }
 
 // Alas, too much differs to reuse beam shield blocks :(
@@ -203,7 +283,7 @@ static bool _iood_shielded(monster& mon, actor &victim)
         return false;
 
     const int to_hit = 15 + (mons_is_projectile(mon.type) ?
-        mon.props["iood_pow"].get_short()/12 : mon.get_hit_dice()/2);
+        mon.props[IOOD_POW].get_short()/12 : mon.get_hit_dice()/2);
     const int con_block = random2(to_hit + victim.shield_block_penalty());
     const int pro_block = victim.shield_bonus();
     dprf("iood shield: pro %d, con %d", pro_block, con_block);
@@ -219,10 +299,7 @@ static bool _boulder_hit(monster& mon, const coord_def &pos)
                                + victim->name(DESC_THE) + "!").c_str());
 
         int dam = victim->apply_ac(roll_dice(3, 20));
-        if (victim->is_player())
-            ouch(dam, mon.mindex(), KILLED_BY_ROLLING);
-        else
-            victim->hurt(&mon, dam);
+        victim->hurt(&mon, dam, BEAM_MISSILE, KILLED_BY_ROLLING);
     }
 
     noisy(5, pos);
@@ -243,19 +320,19 @@ static bool _iood_hit(monster& mon, const coord_def &pos, bool big_boom = false)
     if (!caster)        // caster is dead/gone, blame the orb itself (as its
         caster = &mon;  // friendliness is correct)
     beam.set_agent(caster);
-    if (mon.props.exists("iood_reflector"))
+    if (mon.props.exists(IOOD_REFLECTOR))
     {
         beam.reflections = 1;
 
-        const mid_t refl_mid = mon.props["iood_reflector"].get_int64();
+        const mid_t refl_mid = mon.props[IOOD_REFLECTOR].get_int64();
 
         if (refl_mid == MID_PLAYER)
-            beam.reflector = NON_MONSTER;
+            beam.reflector = MID_PLAYER;
         else
         {
             // If the reflecting monster has died, credit the original caster.
             const monster * const rmon = monster_by_mid(refl_mid);
-            beam.reflector = rmon ? rmon->mindex() : caster->mindex();
+            beam.reflector = rmon ? refl_mid : caster->mid;
         }
     }
     beam.colour = WHITE;
@@ -264,11 +341,11 @@ static bool _iood_hit(monster& mon, const coord_def &pos, bool big_boom = false)
     beam.source = pos;
     beam.target = pos;
     beam.hit = AUTOMATIC_HIT;
-    beam.source_name = mon.props["iood_caster"].get_string();
+    beam.source_name = mon.props[IOOD_CASTER].get_string();
 
-    int pow = mon.props["iood_pow"].get_short();
+    int pow = mon.props[IOOD_POW].get_short();
     pow = stepdown_value(pow, 30, 30, 200, -1);
-    const int dist = mon.props["iood_distance"].get_int();
+    const int dist = mon.props[IOOD_DIST].get_int();
     ASSERT(dist >= 0);
     if (dist < 4)
         pow = pow * (dist*2+3) / 10;
@@ -297,10 +374,10 @@ bool iood_act(monster& mon, bool no_trail)
     bool iood = mons_is_projectile(mon.type);
     ASSERT(iood || mons_is_boulder(&mon));
 
-    float x = mon.props["iood_x"];
-    float y = mon.props["iood_y"];
-    float vx = mon.props["iood_vx"];
-    float vy = mon.props["iood_vy"];
+    float x = mon.props[IOOD_X];
+    float y = mon.props[IOOD_Y];
+    float vx = mon.props[IOOD_VX];
+    float vy = mon.props[IOOD_VY];
 
     dprf("iood_act: pos=(%d,%d) rpos=(%f,%f) v=(%f,%f) foe=%d",
          mon.pos().x, mon.pos().y,
@@ -318,7 +395,7 @@ bool iood_act(monster& mon, bool no_trail)
     // If the target is gone, the orb continues on a ballistic course since
     // picking a new one would require intelligence.
 
-    // Boulders don't home onto their targets.  IOODs can't home in on a
+    // Boulders don't home onto their targets. IOODs can't home in on a
     // submerged creature.
     if (iood && foe && !foe->submerged())
     {
@@ -331,10 +408,10 @@ bool iood_act(monster& mon, bool no_trail)
         // Moving diagonally when the orb is just about to hit you
         //      2
         //    ->*1
-        // (from 1 to 2) would be a guaranteed escape.  This may be
+        // (from 1 to 2) would be a guaranteed escape. This may be
         // realistic (strafing!), but since the game has no non-cheesy
         // means of waiting a small fraction of a turn, we don't want it.
-        const int old_t_pos = mon.props["iood_tpos"].get_short();
+        const int old_t_pos = mon.props[IOOD_TPOS].get_short();
         const coord_def rpos(static_cast<int>(round(x)), static_cast<int>(round(y)));
         if (old_t_pos && old_t_pos != (256 * target.x + target.y)
             && (rpos - target).rdist() <= 1
@@ -344,7 +421,7 @@ bool iood_act(monster& mon, bool no_trail)
             vx = dx;
             vy = dy;
         }
-        mon.props["iood_tpos"].get_short() = 256 * target.x + target.y;
+        mon.props[IOOD_TPOS].get_short() = 256 * target.x + target.y;
 
         if (!_in_front(vx, vy, dx, dy, 0.3)) // ~17 degrees
         {
@@ -360,8 +437,8 @@ bool iood_act(monster& mon, bool no_trail)
             dprf("iood: keeping course");
 
         _normalize(vx, vy);
-        mon.props["iood_vx"] = vx;
-        mon.props["iood_vy"] = vy;
+        mon.props[IOOD_VX] = vx;
+        mon.props[IOOD_VY] = vy;
     }
 
 move_again:
@@ -369,9 +446,9 @@ move_again:
     x += vx;
     y += vy;
 
-    mon.props["iood_x"] = x;
-    mon.props["iood_y"] = y;
-    mon.props["iood_distance"].get_int()++;
+    mon.props[IOOD_X] = x;
+    mon.props[IOOD_Y] = y;
+    mon.props[IOOD_DIST].get_int()++;
 
     const coord_def pos(static_cast<int>(round(x)), static_cast<int>(round(y)));
     if (!in_bounds(pos))
@@ -380,7 +457,7 @@ move_again:
         return true;
     }
 
-    if (iood && mon.props.exists("iood_flawed"))
+    if (iood && mon.props.exists(IOOD_FLAWED))
     {
         const actor *caster = actor_by_mid(mon.summoner);
         if (!caster || caster->pos().origin() ||
@@ -429,10 +506,10 @@ move_again:
         if (mons && iood && mons_is_projectile(victim->type))
         {
             // Weak orbs just fizzle instead of exploding.
-            if (mons->props["iood_distance"].get_int() < 2
-                || mon.props["iood_distance"].get_int() < 2)
+            if (mons->props[IOOD_DIST].get_int() < 2
+                || mon.props[IOOD_DIST].get_int() < 2)
             {
-                if (mons->props["iood_distance"].get_int() < 2)
+                if (mons->props[IOOD_DIST].get_int() < 2)
                 {
                     if (you.see_cell(pos))
                         mpr("The orb fizzles.");
@@ -440,7 +517,7 @@ move_again:
                 }
 
                 // Return, if the acting orb fizzled.
-                if (mon.props["iood_distance"].get_int() < 2)
+                if (mon.props[IOOD_DIST].get_int() < 2)
                 {
                     if (you.see_cell(pos))
                         mpr("The orb fizzles.");
@@ -484,16 +561,9 @@ move_again:
         if (mons && (mons->submerged() || mons->type == MONS_BATTLESPHERE))
         {
             // Try to swap with the submerged creature.
-            if (mons->is_habitable(mon.pos()))
+            if (mon.swap_with(mons))
             {
                 dprf("iood: Swapping with a submerged monster.");
-                mons->set_position(mon.pos());
-                mon.set_position(pos);
-                ASSERT(!mons->is_constricted());
-                ASSERT(!mons->is_constricting());
-                mgrd(mons->pos()) = mons->mindex();
-                mgrd(pos) = mon.mindex();
-
                 return false;
             }
             else // if swap fails, move ahead
@@ -548,9 +618,9 @@ move_again:
             victim->shield_block_succeeded(&mon);
 
             // mid_t is unsigned so won't fit in a plain int
-            mon.props["iood_reflector"] = (int64_t) victim->mid;
-            mon.props["iood_vx"] = vx = -vx;
-            mon.props["iood_vy"] = vy = -vy;
+            mon.props[IOOD_REFLECTOR] = (int64_t) victim->mid;
+            mon.props[IOOD_VX] = vx = -vx;
+            mon.props[IOOD_VY] = vy = -vy;
 
             // Need to get out of the victim's square.
 
@@ -586,8 +656,8 @@ move_again:
     }
 
     // move_to_pos() just trashed the coords, set them again
-    mon.props["iood_x"] = x;
-    mon.props["iood_y"] = y;
+    mon.props[IOOD_X] = x;
+    mon.props[IOOD_Y] = y;
 
     return false;
 }
@@ -597,10 +667,10 @@ move_again:
 // hitting anything.
 static bool _iood_catchup_move(monster& mon)
 {
-    float x = mon.props["iood_x"];
-    float y = mon.props["iood_y"];
-    float vx = mon.props["iood_vx"];
-    float vy = mon.props["iood_vy"];
+    float x = mon.props[IOOD_X];
+    float y = mon.props[IOOD_Y];
+    float vx = mon.props[IOOD_VX];
+    float vy = mon.props[IOOD_VY];
 
     if (!vx && !vy) // not initialized
     {
@@ -613,9 +683,9 @@ static bool _iood_catchup_move(monster& mon)
     x += vx;
     y += vy;
 
-    mon.props["iood_x"] = x;
-    mon.props["iood_y"] = y;
-    mon.props["iood_distance"].get_int()++;
+    mon.props[IOOD_X] = x;
+    mon.props[IOOD_Y] = y;
+    mon.props[IOOD_DIST].get_int()++;
 
     const coord_def pos(static_cast<int>(round(x)), static_cast<int>(round(y)));
     if (!in_bounds(pos))
@@ -649,8 +719,8 @@ static bool _iood_catchup_move(monster& mon)
     }
 
     // move_to_pos() just trashed the coords, set them again
-    mon.props["iood_x"] = x;
-    mon.props["iood_y"] = y;
+    mon.props[IOOD_X] = x;
+    mon.props[IOOD_Y] = y;
 
     return false;
 }
@@ -671,7 +741,7 @@ void iood_catchup(monster* mons, int pturns)
             return;
         }
 
-        if (mon.props["iood_kc"].get_byte() == KC_YOU)
+        if (mon.props[IOOD_KC].get_byte() == KC_YOU)
         {
             // Left player's vision.
             _iood_stop(mon, false);
@@ -689,11 +759,11 @@ void boulder_start(monster *mon, bolt *beam)
     mon->add_ench(ENCH_ROLLING);
     // Work out x/y/vx/vy from beam
     beam->choose_ray();
-    mon->props["iood_x"].get_float() = beam->ray.r.start.x - 0.5;
-    mon->props["iood_y"].get_float() = beam->ray.r.start.y - 0.5;
-    mon->props["iood_vx"].get_float() = mons_is_fleeing(mon) ?
+    mon->props[IOOD_X].get_float() = beam->ray.r.start.x - 0.5;
+    mon->props[IOOD_Y].get_float() = beam->ray.r.start.y - 0.5;
+    mon->props[IOOD_VX].get_float() = mons_is_fleeing(mon) ?
         -beam->ray.r.dir.x : beam->ray.r.dir.x;
-    mon->props["iood_vy"].get_float() = mons_is_fleeing(mon) ?
+    mon->props[IOOD_VY].get_float() = mons_is_fleeing(mon) ?
         -beam->ray.r.dir.y : beam->ray.r.dir.y;
     iood_act(*mon);
 }
