@@ -6,6 +6,7 @@
 #include "AppHdr.h"
 
 #include "monster.h"
+
 #include "mon-util.h"
 
 static int _clamp_stealth(int stealth)
@@ -62,15 +63,15 @@ int monster::stealth() const
 
     // Not an issue with invisibility, but glowing or haloes make you
     // unstealthy.
-    if (glows_naturally() || halo_radius2() != -1)
+    if (glows_naturally() || halo_radius() != -1)
         actual_stealth -= 3;
 
     // Having an umbra makes you more stealthy, on the other hand.
-    if (mons_class_flag(type, M_SHADOW) || umbra_radius2() != -1)
+    if (umbra_radius() != -1)
         actual_stealth += 3;
 
     // Some specific overrides
-    if (type == MONS_UNSEEN_HORROR)
+    if (type == MONS_UNSEEN_HORROR || type == MONS_SHADOW)
         actual_stealth = 3;
 
     return _clamp_stealth(actual_stealth);
