@@ -6,7 +6,6 @@
 #ifndef MONUTIL_H
 #define MONUTIL_H
 
-#include "externs.h"
 #include "enum.h"
 #include "mon-enum.h"
 #include "player.h"
@@ -115,10 +114,9 @@ struct monsterentry
     colour_t colour;
     const char *name;
 
-    uint64_t bitfields;
+    monclass_flags_t bitfields;
     resists_t resists;
 
-    short weight;
     // [Obsolete] Experience used to be calculated like this:
     // ((((max_hp / 7) + 1) * (mHD * mHD) + 1) * exp_mod) / 10
     //     ^^^^^^ see below at hpdice
@@ -130,9 +128,9 @@ struct monsterentry
 
     mon_holy_type holiness;
 
-    short resist_magic;  // (positive is ??)
-    // max damage in a turn is total of these four?
+    short resist_magic;  // (positive sets value, negative is relative to hd)
 
+    // max damage in a turn is total of these four?
     mon_attack_def attack[MAX_NUM_ATTACKS];
 
     // hpdice[4]: [0]=HD [1]=min_hp [2]=rand_hp [3]=add_hp
@@ -147,15 +145,12 @@ struct monsterentry
     int8_t ev; // evasion
     int sec;   // actually mon_spellbook_type
     corpse_effect_type corpse_thingy;
-    zombie_size_type   zombie_size;
     shout_type         shouts;
     mon_intel_type     intel;
     habitat_type     habitat;
-    flight_type      fly;
     int8_t           speed;        // How quickly speed_increment increases
     mon_energy_usage energy_usage; // And how quickly it decreases
     mon_itemuse_type gmon_use;
-    mon_itemeat_type gmon_eat;
     size_type size;
     mon_body_shape shape;
 };
@@ -186,6 +181,8 @@ static inline int get_resist(resists_t all, mon_resist_flags res)
 dungeon_feature_type habitat2grid(habitat_type ht);
 
 monsterentry *get_monster_data(monster_type mc) IMMUTABLE;
+int get_mons_class_ac(monster_type mc) IMMUTABLE;
+int get_mons_class_ev(monster_type mc) IMMUTABLE;
 resists_t get_mons_class_resists(monster_type mc) IMMUTABLE;
 resists_t get_mons_resists(const monster* mon);
 int get_mons_resist(const monster* mon, mon_resist_flags res);
@@ -200,24 +197,27 @@ string mons_type_name(monster_type type, description_level_type desc);
 
 bool give_monster_proper_name(monster* mon, bool orcs_only = true);
 
-flight_type mons_class_flies(monster_type mc);
-flight_type mons_flies(const monster* mon, bool temp = true);
-
 bool mons_flattens_trees(const monster* mon);
-int mons_class_res_wind(monster_type mc);
+bool mons_class_res_wind(monster_type mc);
 
 mon_itemuse_type mons_class_itemuse(monster_type mc);
 mon_itemuse_type mons_itemuse(const monster* mon);
-mon_itemeat_type mons_itemeat(const monster* mon);
 
-bool mons_sense_invis(const monster* mon);
+bool mons_can_be_blinded(monster_type mc);
+bool mons_can_be_dazzled(monster_type mc);
 
 int get_shout_noise_level(const shout_type shout);
 shout_type mons_shouts(monster_type mclass, bool demon_shout = false);
 
 bool mons_is_ghost_demon(monster_type mc);
 bool mons_is_unique(monster_type mc);
+bool mons_is_or_was_unique(const monster& mon);
 bool mons_is_pghost(monster_type mc);
+bool mons_is_draconian_job(monster_type mc);
+bool mons_is_demonspawn_job(monster_type mc);
+bool mons_is_job(monster_type mc);
+
+int mutant_beast_tier(int xl);
 
 int mons_avg_hp(monster_type mc);
 int exper_value(const monster* mon, bool real = true);
@@ -227,21 +227,16 @@ int hit_points(int hit_dice, int min_hp, int rand_hp);
 int mons_class_hit_dice(monster_type mc);
 
 bool mons_immune_magic(const monster* mon);
-const char* mons_resist_string(const monster* mon, int res_margin);
-const char* resist_margin_phrase(int margin);
 
 mon_attack_def mons_attack_spec(const monster* mon, int attk_number, bool base_flavour = false);
 
 corpse_effect_type mons_corpse_effect(monster_type mc);
 
-bool mons_class_flag(monster_type mc, uint64_t bf);
+bool mons_class_flag(monster_type mc, monclass_flags_t bits);
 
 mon_holy_type mons_class_holiness(monster_type mc);
 
-bool mons_is_mimic(monster_type mc);
-bool mons_is_item_mimic(monster_type mc);
-bool mons_is_feat_mimic(monster_type mc);
-void discover_mimic(const coord_def& pos, bool wake = true);
+void discover_mimic(const coord_def& pos);
 void discover_shifter(monster* shifter);
 
 bool mons_is_statue(monster_type mc);
@@ -274,13 +269,12 @@ bool intelligent_ally(const monster* mon);
 bool mons_skeleton(monster_type mc);
 bool mons_zombifiable(monster_type mc);
 
-int mons_weight(monster_type mc);
+int max_corpse_chunks(monster_type mc);
 int mons_class_base_speed(monster_type mc);
 mon_energy_usage mons_class_energy(monster_type mc);
+mon_energy_usage mons_energy(const monster* mon);
 int mons_class_zombie_base_speed(monster_type zombie_base_mc);
-int mons_base_speed(const monster* mon);
-
-bool get_tentacle_head(const monster*& mon);
+int mons_base_speed(const monster* mon, bool known = false);
 
 bool mons_class_can_regenerate(monster_type mc);
 bool mons_can_regenerate(const monster* mon);
@@ -289,7 +283,6 @@ int mons_zombie_size(monster_type mc);
 monster_type mons_zombie_base(const monster* mon);
 bool mons_class_is_zombified(monster_type mc);
 bool mons_class_is_hybrid(monster_type mc);
-bool mons_class_is_chimeric(monster_type mc);
 bool mons_class_is_animated_weapon(monster_type type);
 monster_type mons_base_type(const monster* mon);
 bool mons_class_can_leave_corpse(monster_type mc);
@@ -298,7 +291,8 @@ bool mons_is_zombified(const monster* mons);
 bool mons_class_can_be_zombified(monster_type mc);
 bool mons_can_be_zombified(const monster* mon);
 bool mons_class_can_use_stairs(monster_type mc);
-bool mons_can_use_stairs(const monster* mon);
+bool mons_can_use_stairs(const monster* mon,
+                         dungeon_feature_type stair = DNGN_UNSEEN);
 bool mons_enslaved_body_and_soul(const monster* mon);
 bool mons_enslaved_soul(const monster* mon);
 bool name_zombie(monster* mon, monster_type mc, const string &mon_name);
@@ -321,7 +315,7 @@ void define_monster(monster* mons);
 void mons_pacify(monster* mon, mon_attitude_type att = ATT_GOOD_NEUTRAL,
                  bool no_xp = false);
 
-bool mons_should_fire(bolt &beam);
+bool mons_should_fire(bolt &beam, bool ignore_good_idea = false);
 
 bool mons_has_los_ability(monster_type mon_type);
 bool mons_has_ranged_spell(const monster* mon, bool attack_only = false,
@@ -332,8 +326,6 @@ bool mons_can_attack(const monster* mon);
 bool mons_has_incapacitating_spell(const monster* mon, const actor* foe);
 bool mons_has_incapacitating_ranged_attack(const monster* mon, const actor* foe);
 
-// FIXME: move decline_pronoun somewhere more generic.
-const char *decline_pronoun(gender_type gender, pronoun_type variant);
 const char *mons_pronoun(monster_type mon_type, pronoun_type variant,
                          bool visible = true);
 
@@ -343,8 +335,6 @@ bool mons_atts_aligned(mon_attitude_type fr1, mon_attitude_type fr2);
 bool mons_att_wont_attack(mon_attitude_type fr);
 mon_attitude_type mons_attitude(const monster* m);
 
-bool mons_foe_is_mons(const monster* mons);
-
 bool mons_is_native_in_branch(const monster* mons,
                               const branch_type branch = you.where_are_you);
 bool mons_is_poisoner(const monster* mon);
@@ -368,11 +358,12 @@ bool mons_is_slime(const monster* mon);
 bool mons_class_is_plant(monster_type mc);
 bool mons_is_plant(const monster* mon);
 bool mons_eats_items(const monster* mon);
-bool mons_eats_corpses(const monster* mon);
 monster_type mons_genus(monster_type mc);
 monster_type mons_species(monster_type mc);
 monster_type draco_or_demonspawn_subspecies(const monster* mon);
 monster_type mons_detected_base(monster_type mt);
+bool mons_is_siren_beholder(monster_type mc);
+bool mons_is_siren_beholder(const monster* mons);
 
 bool mons_looks_stabbable(const monster* m);
 bool mons_looks_distracted(const monster* m);
@@ -382,10 +373,10 @@ void mons_stop_fleeing_from_sanctuary(monster* mons);
 
 bool mons_landlubbers_in_reach(const monster* mons);
 
-bool mons_class_is_confusable(monster_type mc);
 bool mons_class_is_stationary(monster_type mc);
 bool mons_class_is_firewood(monster_type mc);
 bool mons_is_firewood(const monster* mon);
+bool mons_is_active_ballisto(const monster* mon);
 bool mons_has_body(const monster* mon);
 bool mons_has_flesh(const monster* mon);
 bool mons_is_abyssal_only(monster_type mc);
@@ -394,7 +385,6 @@ bool herd_monster(const monster* mon);
 
 int cheibriados_monster_player_speed_delta(const monster* mon);
 bool cheibriados_thinks_mons_is_fast(const monster* mon);
-bool mons_is_illuminating(const monster* mon);
 bool mons_is_fiery(const monster* mon);
 bool mons_is_projectile(monster_type mc);
 bool mons_is_projectile(const monster* mon);
@@ -410,6 +400,8 @@ bool invalid_monster(const monster* mon);
 bool invalid_monster_type(monster_type mt);
 bool invalid_monster_index(int i);
 
+void mons_load_spells(monster* mon);
+
 void mons_remove_from_grid(const monster* mon);
 
 bool monster_shover(const monster* m);
@@ -425,6 +417,7 @@ string  draconian_colour_name(monster_type mon_type);
 monster_type draconian_colour_by_name(const string &colour);
 string  demonspawn_base_name(monster_type mon_type);
 monster_type demonspawn_base_by_name(const string &colour);
+mon_spell_slot drac_breath(monster_type drac_type);
 
 monster_type random_monster_at_grid(const coord_def& p, bool species = false);
 
@@ -442,6 +435,7 @@ string get_mon_shape_str(const mon_body_shape shape);
 bool mons_class_can_pass(monster_type mc, const dungeon_feature_type grid);
 bool mons_can_open_door(const monster* mon, const coord_def& pos);
 bool mons_can_eat_door(const monster* mon, const coord_def& pos);
+bool mons_can_destroy_door(const monster* mon, const coord_def& pos);
 bool mons_can_traverse(const monster* mon, const coord_def& pos,
                        bool only_in_sight = false,
                        bool checktraps = true);
@@ -454,23 +448,12 @@ bool mons_is_immotile(const monster* mons);
 
 int get_dist_to_nearest_monster();
 bool monster_nearby();
-actor *actor_by_mid(mid_t m);
-monster *monster_by_mid(mid_t m);
-bool mons_is_tentacle_head(monster_type mc);
-bool mons_is_child_tentacle(monster_type mc);
-bool mons_is_child_tentacle_segment(monster_type mc);
-bool mons_is_tentacle(monster_type mc);
-bool mons_is_tentacle_segment(monster_type mc);
-bool mons_is_tentacle_or_tentacle_segment(monster_type mc);
+actor *actor_by_mid(mid_t m, bool require_valid = false);
+monster *monster_by_mid(mid_t m, bool require_valid = false);
 bool mons_is_recallable(actor* caller, monster* targ);
-monster* mons_get_parent_monster(monster* mons);
 void init_anon();
 actor *find_agent(mid_t m, kill_category kc);
 const char* mons_class_name(monster_type mc);
-void check_clinging();
-monster_type mons_tentacle_parent_type(const monster* mons);
-monster_type mons_tentacle_child_type(const monster* mons);
-bool mons_tentacle_adjacent(const monster* parent, const monster* child);
 mon_threat_level_type mons_threat_level(const monster *mon,
                                         bool real = false);
 int count_monsters(monster_type mtyp, bool friendly_only);
@@ -482,10 +465,10 @@ vector<monster* > get_on_level_followers();
 bool mons_stores_tracking_data(const monster* mons);
 
 bool mons_is_player_shadow(const monster* mon);
-bool mons_antimagic_affected(const monster* mons);
 
 void reset_all_monsters();
 void debug_mondata();
+void debug_monspells();
 
 bool choose_any_monster(const monster* mon);
 monster *choose_random_nearby_monster(
@@ -497,6 +480,35 @@ monster *choose_random_nearby_monster(
 monster *choose_random_monster_on_level(
     int weight,
     bool (*suitable)(const monster* mon) =
-        choose_any_monster);
+        choose_any_monster,
+    bool prefer_named_or_priest = false);
 
+void update_monster_symbol(monster_type mtype, cglyph_t md);
+
+void normalize_spell_freq(monster_spells &spells, int hd);
+
+enum mon_dam_level_type
+{
+    MDAM_OKAY,
+    MDAM_LIGHTLY_DAMAGED,
+    MDAM_MODERATELY_DAMAGED,
+    MDAM_HEAVILY_DAMAGED,
+    MDAM_SEVERELY_DAMAGED,
+    MDAM_ALMOST_DEAD,
+    MDAM_DEAD,
+};
+
+void print_wounds(const monster* mons);
+bool wounded_damaged(mon_holy_type holi);
+
+mon_dam_level_type mons_get_damage_level(const monster* mons);
+
+string get_damage_level_string(mon_holy_type holi, mon_dam_level_type mdam);
+bool mons_class_can_display_wounds(monster_type mc);
+bool mons_can_display_wounds(const monster* mon);
+bool mons_class_gives_xp(monster_type mc, bool indirect = false);
+bool mons_gives_xp(const monster* mon, const actor* agent);
+
+void init_mutant_beast(monster &mon, short HD, vector<int> beast_facets,
+                       set<int> avoid_facets);
 #endif
