@@ -186,6 +186,26 @@ static void _give_ranged_weapon(weapon_type weapon, int plus)
 {
     ASSERT(weapon != NUM_WEAPONS);
 
+    switch (weapon)
+    {
+    case WPN_SHORTBOW:
+        newgame_make_item(OBJ_WEAPONS, WPN_SHORTBOW, 1, plus);
+        break;
+    case WPN_HAND_CROSSBOW:
+        newgame_make_item(OBJ_WEAPONS, WPN_HAND_CROSSBOW, 1, plus);
+        break;
+    case WPN_HUNTING_SLING:
+        newgame_make_item(OBJ_WEAPONS, WPN_HUNTING_SLING, 1, plus);
+        break;
+    default:
+        break;
+    }
+}
+
+static void _give_ammo(weapon_type weapon, int plus)
+{
+    ASSERT(weapon != NUM_WEAPONS);
+
     switch (weapon)
     {
     case WPN_THROWN:
@@ -198,15 +218,12 @@ static void _give_ranged_weapon(weapon_type weapon, int plus)
         newgame_make_item(OBJ_MISSILES, MI_THROWING_NET, 2);
         break;
     case WPN_SHORTBOW:
-        newgame_make_item(OBJ_WEAPONS, WPN_SHORTBOW, 1, plus);
         newgame_make_item(OBJ_MISSILES, MI_ARROW, 20);
         break;
     case WPN_HAND_CROSSBOW:
-        newgame_make_item(OBJ_WEAPONS, WPN_HAND_CROSSBOW, 1, plus);
         newgame_make_item(OBJ_MISSILES, MI_BOLT, 20);
         break;
     case WPN_HUNTING_SLING:
-        newgame_make_item(OBJ_WEAPONS, WPN_HUNTING_SLING, 1, plus);
         newgame_make_item(OBJ_MISSILES, MI_SLING_BULLET, 20);
         break;
     default:
@@ -276,6 +293,9 @@ static void _give_items_skills(const newgame_def& ng)
     give_job_equipment(you.char_class);
     give_job_skills(you.char_class);
 
+    if (job_gets_ranged_weapons(you.char_class))
+        _give_ammo(ng.weapon, you.char_class == JOB_HUNTER ? 1 : 0);
+
     // Deep Dwarves get a wand of heal wounds (5).
     if (you.species == SP_DEEP_DWARF)
         newgame_make_item(OBJ_WANDS, WAND_HEAL_WOUNDS, 1, 5);
