@@ -5,48 +5,29 @@
 
 #include "AppHdr.h"
 
-#include <vector>
-#include <sstream>
-#include <iomanip>
-
 #include "notes.h"
 
+#include <iomanip>
+#include <sstream>
+#include <vector>
+
 #include "branch.h"
-#include "files.h"
-#include "kills.h"
+#include "english.h"
 #include "hiscores.h"
-#include "libutil.h"
 #include "message.h"
 #include "mutation.h"
 #include "options.h"
-#include "place.h"
 #include "religion.h"
-#include "skills2.h"
+#include "skills.h"
 #include "spl-util.h"
 #include "state.h"
 #include "stringutil.h"
-#include "tags.h"
 #include "unicode.h"
 
 #define NOTES_VERSION_NUMBER 1002
 
 vector<Note> note_list;
 
-// return the real number of the power (casting out nonexistent powers),
-// starting from 0, or -1 if the power doesn't exist
-static int _real_god_power(int religion, int idx)
-{
-    if (god_gain_power_messages[religion][idx][0] == 0)
-        return -1;
-
-    int count = 0;
-    for (int j = 0; j < idx; ++j)
-        if (god_gain_power_messages[religion][j][0])
-            ++count;
-
-    return count;
-}
-
 static bool _is_highest_skill(int skill)
 {
     for (int i = 0; i < NUM_SKILLS; ++i)
@@ -72,10 +53,10 @@ static int _dungeon_branch_depth(uint8_t branch)
     return brdepth[branch];
 }
 
-static bool _is_noteworthy_dlevel(unsigned short place)
+static bool _is_noteworthy_dlevel(level_id place)
 {
-    branch_type branch = place_branch(place);
-    int lev = place_depth(place);
+    branch_type branch = place.branch;
+    int lev = place.depth;
 
     // Entering the Abyss is noted a different way, since we care mostly about
     // the cause.
@@ -87,7 +68,7 @@ static bool _is_noteworthy_dlevel(unsigned short place)
         return false;
 
     // Other portal levels are always interesting.
-    if (!is_connected_branch(static_cast<branch_type>(branch)))
+    if (!is_connected_branch(branch))
         return true;
 
     return lev == _dungeon_branch_depth(branch)
@@ -126,7 +107,8 @@ static bool _is_noteworthy(const Note& note)
         || note.type == NOTE_NAMED_ALLY
         || note.type == NOTE_ALLY_DEATH
         || note.type == NOTE_FEAT_MIMIC
-        || note.type == NOTE_OFFERED_SPELL)
+        || note.type == NOTE_OFFERED_SPELL
+        || note.type == NOTE_FOCUS_CARD)
     {
         return true;
     }
@@ -143,13 +125,6 @@ static bool _is_noteworthy(const Note& note)
     if (note.type == NOTE_XOM_EFFECT)
         return Options.note_xom_effects;
 
-    // God powers might be noteworthy if it's an actual power.
-    if (note.type == NOTE_GOD_POWER
-        && _real_god_power(note.first, note.second) == -1)
-    {
-        return false;
-    }
-
     // HP noteworthiness is handled in its own function.
     if (note.type == NOTE_HP_CHANGE
         && !_is_noteworthy_hp(note.first, note.second))
@@ -171,17 +146,17 @@ static bool _is_noteworthy(const Note& note)
     }
 
     if (note.type == NOTE_DUNGEON_LEVEL_CHANGE)
-        return _is_noteworthy_dlevel(note.packed_place);
+        return _is_noteworthy_dlevel(note.place);
 
-    for (unsigned i = 0; i < note_list.size(); ++i)
+    for (const Note &oldnote : note_list)
     {
-        if (note_list[i].type != note.type)
+        if (oldnote.type != note.type)
             continue;
 
-        const Note& rnote(note_list[i]);
+        const Note& rnote(oldnote);
         switch (note.type)
         {
-        case NOTE_GOD_POWER:
+        case NOTE_PIETY_RANK:
             if (rnote.first == note.first && rnote.second == note.second)
                 return false;
             break;
@@ -206,17 +181,6 @@ static bool _is_noteworthy(const Note& note)
     return true;
 }
 
-static const char* _number_to_ordinal(int number)
-{
-    const char* ordinals[5] = { "first", "second", "third", "fourth", "fifth" };
-
-    if (number < 1)
-        return "[unknown ordinal (too small)]";
-    if (number > 5)
-        return "[unknown ordinal (too big)]";
-    return ordinals[number-1];
-}
-
 string Note::describe(bool when, bool where, bool what) const
 {
     ostringstream result;
@@ -226,8 +190,9 @@ string Note::describe(bool when, bool where, bool what) const
 
     if (where)
     {
-        result << "| " << chop_string(short_place_name(packed_place),
-                                      MAX_NOTE_PLACE_LEN) << " | ";
+        result << "| "
+               << chop_string(place.describe(), MAX_NOTE_PLACE_LEN)
+               << " | ";
     }
 
     if (what)
@@ -259,7 +224,8 @@ string Note::describe(bool when, bool where, bool what) const
             if (!desc.empty())
                 result << desc;
             else
-                result << "Entered " << place_name(packed_place, true, true);
+                result << "Entered "
+                       << place.describe(true, true);
             break;
         case NOTE_LEARN_SPELL:
             result << "Learned a level "
@@ -326,12 +292,11 @@ string Note::describe(bool when, bool where, bool what) const
         case NOTE_POLY_MONSTER:
             result << name << " changed into " << desc;
             break;
-        case NOTE_GOD_POWER:
-            result << "Acquired "
-                   << apostrophise(god_name(static_cast<god_type>(first)))
-                   << " "
-                   << _number_to_ordinal(_real_god_power(first, second)+1)
-                   << " power";
+        case NOTE_PIETY_RANK:
+            result << "Reached "
+                   << string(second, '*')
+                   << " piety under "
+                   << god_name(static_cast<god_type>(first));
             break;
         case NOTE_GET_MUTATION:
             result << "Gained mutation: "
@@ -393,6 +358,10 @@ string Note::describe(bool when, bool where, bool what) const
                    << spell_title(static_cast<spell_type>(first))
                    << " by Vehumet.";
             break;
+        case NOTE_FOCUS_CARD:
+            result << "Drew Focus: " << name << " increased to " << first << ", "
+                   << desc << " decreased to " << second;
+            break;
         default:
             result << "Buggy note description: unknown note type";
             break;
@@ -407,24 +376,6 @@ string Note::describe(bool when, bool where, bool what) const
     return result.str();
 }
 
-Note::Note()
-{
-    turn         = you.num_turns;
-    packed_place = get_packed_place();
-}
-
-Note::Note(NOTE_TYPES t, int f, int s, const char* n, const char* d) :
-    type(t), first(f), second(s)
-{
-    if (n)
-        name = string(n);
-    if (d)
-        desc = string(d);
-
-    turn         = you.num_turns;
-    packed_place = get_packed_place();
-}
-
 void Note::check_milestone() const
 {
     if (crawl_state.game_is_arena())
@@ -432,14 +383,15 @@ void Note::check_milestone() const
 
     if (type == NOTE_DUNGEON_LEVEL_CHANGE)
     {
-        const int br = place_branch(packed_place),
-                 dep = place_depth(packed_place);
+        const int br = place.branch,
+                 dep = place.depth;
 
         // Wizlabs report their milestones on their own.
         if (br != -1 && br != BRANCH_WIZLAB)
         {
             ASSERT_RANGE(br, 0, NUM_BRANCHES);
-            string branch = place_name(packed_place, true, false).c_str();
+            string branch = place.describe(true, false);
+
             if (branch.find("The ") == 0)
                 branch[0] = tolower(branch[0]);
 
@@ -451,14 +403,12 @@ void Note::check_milestone() const
             else if (dep == _dungeon_branch_depth(br)
                      || br == BRANCH_ZIGGURAT)
             {
-                string level = place_name(packed_place, true, true);
+                string level = place.describe(true, true);
                 if (level.find("Level ") == 0)
                     level[0] = tolower(level[0]);
 
-                ostringstream branch_finale;
-                branch_finale << "reached " << level << ".";
                 mark_milestone(br == BRANCH_ZIGGURAT ? "zig" : "br.end",
-                               branch_finale.str());
+                               "reached " + level + ".");
             }
         }
     }
@@ -468,7 +418,7 @@ void Note::save(writer& outf) const
 {
     marshallInt(outf, type);
     marshallInt(outf, turn);
-    marshallShort(outf, packed_place);
+    place.save(outf);
     marshallInt(outf, first);
     marshallInt(outf, second);
     marshallString4(outf, name);
@@ -479,7 +429,12 @@ void Note::load(reader& inf)
 {
     type = static_cast<NOTE_TYPES>(unmarshallInt(inf));
     turn = unmarshallInt(inf);
-    packed_place = unmarshallShort(inf);
+#if TAG_MAJOR_VERSION == 34
+    if (inf.getMinorVersion() < TAG_MINOR_PLACE_UNPACK)
+        place = level_id::from_packed_place(unmarshallShort(inf));
+    else
+#endif
+    place.load(inf);
     first  = unmarshallInt(inf);
     second = unmarshallInt(inf);
     unmarshallString4(inf, name);
@@ -511,8 +466,8 @@ void save_notes(writer& outf)
 {
     marshallInt(outf, NOTES_VERSION_NUMBER);
     marshallInt(outf, note_list.size());
-    for (unsigned i = 0; i < note_list.size(); ++i)
-        note_list[i].save(outf);
+    for (const Note &note : note_list)
+        note.save(outf);
 }
 
 void load_notes(reader& inf)
