@@ -8,7 +8,9 @@
 
 #include <string>
 #include <vector>
-#include <stdio.h>
+#include <cstdio>
+
+#include "player.h"
 
 #define MAX_NOTE_PLACE_LEN 8
 
@@ -26,7 +28,7 @@ enum NOTE_TYPES
     NOTE_LEARN_SPELL,           /* needs: spell idx */
     NOTE_GET_GOD,               /* needs: god id */
     NOTE_GOD_GIFT,              /* needs: god id */
-    NOTE_GOD_POWER,             /* needs: god id, idx */
+    NOTE_PIETY_RANK,            /* needs: god id, rank */
     NOTE_GET_MUTATION,          /* needs: mutation idx, reason (string) */
     NOTE_LOSE_MUTATION,         /* needs: mutation idx, reason (string) */
     NOTE_ID_ITEM,               /* needs: item name (string) */
@@ -53,14 +55,17 @@ enum NOTE_TYPES
     NOTE_FEAT_MIMIC,            /* needs: mimiced feature (string) */
     NOTE_OFFERED_SPELL,         /* needs: spell idx */
     NOTE_PERM_MUTATION,         /* needs: mutation idx, reason (string) */
+    NOTE_FOCUS_CARD,            /* needs: increased stat value, decreased stat value,
+                                   increased stat name (string), decreased stat name (string) */
     NOTE_NUM_TYPES
 };
 
 struct Note
 {
-    Note();
-    Note(NOTE_TYPES t, int f = 0, int s = 0, const char* n = 0,
-          const char* d = 0);
+    Note() {}
+    Note(NOTE_TYPES t, int f = 0, int s = 0, const string& n = "",
+                                             const string& d = "") :
+        type(t), first(f), second(s), name(n), desc(d) {}
     void save(writer& outf) const;
     void load(reader& inf);
     string describe(bool when = true, bool where = true, bool what = true) const;
@@ -68,8 +73,8 @@ struct Note
 
     NOTE_TYPES type;
     int first, second;
-    int turn;
-    unsigned short packed_place;
+    int turn = you.num_turns;
+    level_id place = level_id::current();
 
     string name;
     string desc;
@@ -83,4 +88,21 @@ void save_notes(writer&);
 void load_notes(reader&);
 void make_user_note();
 
+/**
+ * Disable notes in a dynamic scope. Restores the original note status when
+ * the object goes out of scope or is otherwise destroyed.
+ */
+struct no_notes
+{
+    no_notes() : saved(notes_are_active())
+    {
+        activate_notes(false);
+    }
+    ~no_notes()
+    {
+        activate_notes(saved);
+    }
+    bool saved;
+};
+
 #endif
