@@ -1,9 +1,11 @@
 #ifndef OPTIONS_H
 #define OPTIONS_H
 
+#include <algorithm>
+
 #include "feature.h"
-#include "pattern.h"
 #include "newgame_def.h"
+#include "pattern.h"
 
 enum autosac_type
 {
@@ -54,7 +56,7 @@ struct colour_mapping
 {
     string tag;
     text_pattern pattern;
-    int colour;
+    colour_t colour;
     bool operator== (const colour_mapping &o) const
     {
         return tag == o.tag && pattern == o.pattern && colour == o.colour;
@@ -71,6 +73,37 @@ struct message_colour_mapping
     }
 };
 
+struct flang_entry
+{
+    flang_t lang_id;
+    int value;
+};
+
+enum use_animation_type
+{
+    UA_NONE             = 0,
+    // projectile animations, from throwing weapons, fireball, etc
+    UA_BEAM             = (1 << 0),
+    // flashes the screen when trying to cast a spell beyond its range
+    UA_RANGE            = (1 << 1),
+    // flashes the screen on low hitpoint warning
+    UA_HP               = (1 << 2),
+    // flashes the screen on attempt to travel or rest with a monster in view
+    UA_MONSTER_IN_SIGHT = (1 << 3),
+    // various animations for picking up runes and the orb
+    UA_PICKUP           = (1 << 4),
+    // various monster spell/ability effects (slime creature merging, etc)
+    UA_MONSTER          = (1 << 5),
+    // various player spell/ability animation effects (shatter, etc)
+    UA_PLAYER           = (1 << 6),
+    // animation when entering certain branches (abyss, zot, etc)
+    UA_BRANCH_ENTRY     = (1 << 7),
+    // animations that can't be turned off (please don't use unless they can
+    // be turned off some other way)
+    UA_ALWAYS_ON        = (1 << 8),
+};
+DEF_BITFIELD(use_animations_type, use_animation_type);
+
 class LineInput;
 struct game_options
 {
@@ -90,7 +123,7 @@ public:
     bool was_included(const string &file) const;
 
     static string resolve_include(string including_file, string included_file,
-                            const vector<string> *rcdirs = NULL) throw (string);
+                            const vector<string> *rcdirs = nullptr) throw (string);
 
 #ifdef USE_TILE_WEB
     void write_webtiles_options(const string &name);
@@ -102,10 +135,12 @@ public:
     int         line_num;     // Current line number being processed.
 
     // View options
-    map<dungeon_feature_type, feature_def> feature_overrides;
+    map<dungeon_feature_type, feature_def> feature_colour_overrides;
+    map<dungeon_feature_type, FixedVector<ucs_t, 2> > feature_symbol_overrides;
     map<monster_type, cglyph_t> mon_glyph_overrides;
     ucs_t cset_override[NUM_DCHAR_TYPES];
-    vector<pair<string, cglyph_t> > item_glyph_overrides;
+    typedef pair<string, cglyph_t> item_glyph_override_type;
+    vector<item_glyph_override_type > item_glyph_overrides;
     map<string, cglyph_t> item_glyph_cache;
 
     string      save_dir;       // Directory where saves and bones go.
@@ -113,7 +148,7 @@ public:
     string      morgue_dir;     // Directory where character dumps and morgue
                                 // dumps are saved. Overrides crawl_dir.
     string      shared_dir;     // Directory where the logfile, scores and bones
-                                // are stored.  On a multi-user system, this dir
+                                // are stored. On a multi-user system, this dir
                                 // should be accessible by different people.
     vector<string> additional_macro_files;
 
@@ -155,6 +190,9 @@ public:
     int         scroll_margin_x;
     int         scroll_margin_y;
 
+    // Whether exclusions and exclusion radius are visible in the viewport.
+    bool        always_show_exclusions;
+
     int         autopickup_on;
     bool        autopickup_starting_ammo;
     bool        default_manual_training;
@@ -169,11 +207,10 @@ public:
     bool        easy_unequip;    // allow auto-removing of armour / jewellery
     bool        equip_unequip;   // Make 'W' = 'T', and 'P' = 'R'.
     bool        jewellery_prompt; // Always prompt for slot when changing jewellery.
+    bool        easy_door;       // 'O', 'C' don't prompt with just one door.
+    bool        warn_hatches;    // offer a y/n prompt when the player uses an escape hatch
+    bool        enable_recast_spell; // Allow recasting spells with 'z' Enter.
     int         confirm_butcher; // When to prompt for butchery
-    bool        chunks_autopickup; // Autopickup chunks after butchering
-    bool        prompt_for_swap; // Prompt to switch back from butchering
-                                 // tool if hostile monsters are around.
-    chunk_drop_type auto_drop_chunks; // drop chunks when overburdened
     bool        easy_eat_chunks; // make 'e' auto-eat the oldest safe chunk
     bool        auto_eat_chunks; // allow eating chunks while resting or travelling
     skill_focus_mode skill_focus; // is the focus skills available
@@ -192,7 +229,7 @@ public:
     int         colour[16];      // macro fg colours to other colours
     int         background_colour; // select default background colour
     msg_colour_type channels[NUM_MESSAGE_CHANNELS];  // msg channel colouring
-    bool        darken_beyond_range; // for whether targeting is out of range
+    use_animations_type use_animations; // which animations to show
 
     int         hp_warning;      // percentage hp for danger warning
     int         magic_point_warning;    // percentage mp for danger warning
@@ -204,6 +241,7 @@ public:
     bool        no_dark_brand;    // Attribute for branding friendly monsters
     bool        macro_meta_entry; // Allow user to use numeric sequences when
                                   // creating macros
+    bool        cloud_status;     // Whether to show a cloud status light
 
     int         fire_items_start; // index of first item for fire command
     vector<unsigned> fire_order;  // missile search order for 'f' command
@@ -215,8 +253,11 @@ public:
 
     int         num_colours;     // used for setting up curses colour table (8 or 16)
 
+    vector<string> pizzas;
+
 #ifdef WIZARD
-    int            wiz_mode;   // no, never, start in wiz mode
+    int            wiz_mode;      // no, never, start in wiz mode
+    int            explore_mode;  // no, never, start in explore mode
 #endif
     vector<string> terp_files; // Lua files to load for luaterp
     bool           no_save;    // don't use persistent save files
@@ -237,10 +278,11 @@ public:
     vector<text_pattern> note_messages;  // Interesting messages
     vector<pair<text_pattern, string> > autoinscriptions;
     vector<text_pattern> note_items;     // Objects to note
-    FixedBitVector<27+1> note_skill_levels;   // Skill levels to note
-    vector<pair<text_pattern, string> > auto_spell_letters;
-
-    bool        autoinscribe_cursed; // Auto-inscribe previosly cursed items.
+    // Skill levels to note
+    FixedBitVector<MAX_SKILL_LEVEL + 1> note_skill_levels;
+    vector<pair<text_pattern, string>> auto_spell_letters;
+    vector<pair<text_pattern, string>> auto_item_letters;
+    vector<pair<text_pattern, string>> auto_ability_letters;
 
     bool        pickup_thrown;  // Pickup thrown missiles
     int         travel_delay;   // How long to pause between travel moves
@@ -256,6 +298,7 @@ public:
     bool        arena_list_eq;
 
     vector<message_filter> force_more_message;
+    vector<message_filter> flash_screen_message;
 
     int         tc_reachable;   // Colour for squares that are reachable
     int         tc_excluded;    // Colour for excluded squares.
@@ -303,6 +346,9 @@ public:
     // How much autoexplore favors visiting squares next to walls.
     int         explore_wall_bias;
 
+    // Wait for rest wait percent HP and MP before exploring.
+    bool        explore_auto_rest;
+
     // Some experimental improvements to explore
     bool        explore_improved;
 
@@ -330,11 +376,13 @@ public:
     int         pickup_menu_limit;  // Over this number of items, menu for
                                     // pickup
     bool        easy_exit_menu;     // Menus are easier to get out of
+    bool        ability_menu;       // 'a'bility starts with a full-screen menu
 
     int         assign_item_slot;   // How free slots are assigned
     maybe_bool  show_god_gift;      // Show {god gift} in item names
 
     bool        restart_after_game; // If true, Crawl will not close on game-end
+    bool        restart_after_save; // .. or on save
 
     vector<text_pattern> drop_filter;
 
@@ -362,10 +410,21 @@ public:
 
     bool        rest_wait_both; // Stop resting only when both HP and MP are
                                 // fully restored.
+    int         rest_wait_percent; // Stop resting after restoring this
+                                   // fraction of HP or MP
 
-    lang_t      lang;                // Translation to use.
-    const char* lang_name;           // Database name of the language.
+    lang_t              language;         // Translation to use.
+    const char*         lang_name;        // Database name of the language.
+    vector<flang_entry> fake_langs;       // The fake language(s) in use.
+    bool has_fake_lang(flang_t flang)
+    {
+        return any_of(begin(fake_langs), end(fake_langs),
+                      [flang] (const flang_entry &f)
+                      { return f.lang_id == flang; });
+    }
 
+    // -1 and 0 mean no confirmation, other possible values are 1,2,3 (see fail_severity())
+    int         fail_severity_to_confirm;
 #ifdef WIZARD
     // Parameters for fight simulations.
     string      fsim_mode;
@@ -384,9 +443,6 @@ public:
     // minimap colours
     VColour     tile_player_col;
     VColour     tile_monster_col;
-    VColour     tile_neutral_col;
-    VColour     tile_peaceful_col;
-    VColour     tile_friendly_col;
     VColour     tile_plant_col;
     VColour     tile_item_col;
     VColour     tile_unseen_col;
@@ -453,8 +509,11 @@ public:
     bool        tile_show_demon_tier;
     bool        tile_water_anim;
     bool        tile_misc_anim;
-    bool        tile_show_player_species;
     vector<string> tile_layout_priority;
+    monster_type tile_use_monster;
+    tileidx_t   tile_player_tile;
+    pair<int, int> tile_weapon_offsets;
+    pair<int, int> tile_shield_offsets;
 #ifdef USE_TILE_WEB
     bool        tile_realtime_anim;
     string      tile_display_mode;
@@ -480,7 +539,7 @@ public:
     // Convenience accessors for the second-class options in named_options.
     int         o_int(const char *name, int def = 0) const;
     bool        o_bool(const char *name, bool def = false) const;
-    string      o_str(const char *name, const char *def = NULL) const;
+    string      o_str(const char *name, const char *def = nullptr) const;
     int         o_colour(const char *name, int def = LIGHTGREY) const;
 
     // Fix option values if necessary, specifically file paths.
@@ -493,9 +552,9 @@ private:
 
     void clear_feature_overrides();
     void clear_cset_overrides();
-    void add_cset_override(char_set_type set, const string &overrides);
-    void add_cset_override(char_set_type set, dungeon_char_type dc, int symbol);
-    void add_feature_override(const string &);
+    void add_cset_override(dungeon_char_type dc, int symbol);
+    void add_feature_override(const string &, bool prepend);
+    void remove_feature_override(const string &, bool prepend);
 
     void add_message_colour_mappings(const string &, bool, bool);
     void add_message_colour_mapping(const string &, bool, bool);
@@ -516,15 +575,21 @@ private:
                          bool prepend = false);
     void do_kill_map(const string &from, const string &to);
     int  read_explore_stop_conditions(const string &) const;
+    use_animations_type read_use_animations(const string &) const;
 
     void split_parse(const string &s, const string &separator,
-                     void (game_options::*add)(const string &));
-    void add_mon_glyph_overrides(const string &mons, cglyph_t &mdisp);
-    void add_mon_glyph_override(const string &);
+                     void (game_options::*add)(const string &, bool),
+                     bool prepend = false);
+    void add_mon_glyph_override(const string &, bool prepend);
+    void remove_mon_glyph_override(const string &, bool prepend);
     cglyph_t parse_mon_glyph(const string &s) const;
-    void add_item_glyph_override(const string &);
-    void set_option_fragment(const string &s);
+    void add_item_glyph_override(const string &, bool prepend);
+    void remove_item_glyph_override(const string &, bool prepend);
+    void set_option_fragment(const string &s, bool prepend);
     bool set_lang(const char *s);
+    void set_fake_langs(const string &input);
+    void set_player_tile(const string &s);
+    void set_tile_offsets(const string &s, bool set_shield);
 
     static const string interrupt_prefix;
 };
