@@ -5,37 +5,33 @@
 
 #include "AppHdr.h"
 
-#include <cstring>
-#include <string>
-#include <iostream>
-#include <cstdio>
-#include <cstdlib>
+#include "ouch.h"
+
 #include <cctype>
 #include <cmath>
-
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
+#include <iostream>
+#include <string>
 #ifdef UNIX
-#include <sys/types.h>
 #include <fcntl.h>
+#include <sys/types.h>
 #include <unistd.h>
 #endif
 
-#include "ouch.h"
-
-#include "externs.h"
-#include "options.h"
-
-#include "art-enum.h"
 #include "artefact.h"
+#include "art-enum.h"
 #include "beam.h"
 #include "chardump.h"
+#include "colour.h"
 #include "delay.h"
 #include "describe.h"
 #include "dgnevent.h"
-#include "effects.h"
 #include "end.h"
 #include "env.h"
-#include "files.h"
 #include "fight.h"
+#include "files.h"
 #include "fineff.h"
 #include "godabil.h"
 #include "godpassive.h"
@@ -51,10 +47,11 @@
 #include "mgen_data.h"
 #include "misc.h"
 #include "mon-death.h"
-#include "mon-util.h"
 #include "mon-place.h"
+#include "mon-util.h"
 #include "mutation.h"
 #include "notes.h"
+#include "options.h"
 #include "output.h"
 #include "player.h"
 #include "player-stats.h"
@@ -63,16 +60,16 @@
 #include "random.h"
 #include "religion.h"
 #include "shopping.h"
-#include "skills2.h"
+#include "shout.h"
+#include "skills.h"
 #include "spl-clouds.h"
-#include "spl-selfench.h"
 #include "spl-other.h"
+#include "spl-selfench.h"
 #include "state.h"
 #include "stringutil.h"
 #include "transform.h"
 #include "tutorial.h"
 #include "view.h"
-#include "shout.h"
 #include "xom.h"
 
 void maybe_melt_player_enchantments(beam_type flavour, int damage)
@@ -81,15 +78,6 @@ void maybe_melt_player_enchantments(beam_type flavour, int damage)
         || flavour == BEAM_HELLFIRE || flavour == BEAM_STICKY_FLAME
         || flavour == BEAM_STEAM)
     {
-        if (you.duration[DUR_CONDENSATION_SHIELD] > 0)
-        {
-            you.duration[DUR_CONDENSATION_SHIELD] -= damage * BASELINE_DELAY;
-            if (you.duration[DUR_CONDENSATION_SHIELD] <= 0)
-                remove_condensation_shield();
-            else
-                you.props["melt_shield"] = true;
-        }
-
         if (you.mutation[MUT_ICEMAIL])
         {
             if (!you.duration[DUR_ICEMAIL_DEPLETED])
@@ -113,7 +101,6 @@ void maybe_melt_player_enchantments(beam_type flavour, int damage)
 int check_your_resists(int hurted, beam_type flavour, string source,
                        bolt *beam, bool doEffects)
 {
-    int resist;
     int original = hurted;
 
     dprf("checking resistance: flavour=%d", flavour);
@@ -131,8 +118,7 @@ int check_your_resists(int hurted, beam_type flavour, string source,
     switch (flavour)
     {
     case BEAM_WATER:
-        hurted = resist_adjust_damage(&you, flavour,
-                                      you.res_water_drowning(), hurted, true);
+        hurted = resist_adjust_damage(&you, flavour, hurted);
         if (!hurted && doEffects)
             mpr("You shrug off the wave.");
         else if (hurted > original && doEffects)
@@ -143,8 +129,7 @@ int check_your_resists(int hurted, beam_type flavour, string source,
         break;
 
     case BEAM_STEAM:
-        hurted = resist_adjust_damage(&you, flavour,
-                                      player_res_steam(), hurted, true);
+        hurted = resist_adjust_damage(&you, flavour, hurted);
         if (hurted < original && doEffects)
             canned_msg(MSG_YOU_RESIST);
         else if (hurted > original && doEffects)
@@ -155,8 +140,7 @@ int check_your_resists(int hurted, beam_type flavour, string source,
         break;
 
     case BEAM_FIRE:
-        hurted = resist_adjust_damage(&you, flavour,
-                                      player_res_fire(), hurted, true);
+        hurted = resist_adjust_damage(&you, flavour, hurted);
         if (hurted < original && doEffects)
             canned_msg(MSG_YOU_RESIST);
         else if (hurted > original && doEffects)
@@ -175,13 +159,10 @@ int check_your_resists(int hurted, beam_type flavour, string source,
                 mpr("You resist completely.");
         }
 #endif
-        // Inconsistency: no penalty for rF-, unlike monsters.  That's
-        // probably good, and monsters should be changed.
         break;
 
     case BEAM_COLD:
-        hurted = resist_adjust_damage(&you, flavour,
-                                      player_res_cold(), hurted, true);
+        hurted = resist_adjust_damage(&you, flavour, hurted);
         if (hurted < original && doEffects)
             canned_msg(MSG_YOU_RESIST);
         else if (hurted > original && doEffects)
@@ -192,9 +173,7 @@ int check_your_resists(int hurted, beam_type flavour, string source,
         break;
 
     case BEAM_ELECTRICITY:
-        hurted = resist_adjust_damage(&you, flavour,
-                                      player_res_electricity(),
-                                      hurted, true);
+        hurted = resist_adjust_damage(&you, flavour, hurted);
 
         if (hurted < original && doEffects)
             canned_msg(MSG_YOU_RESIST);
@@ -206,6 +185,8 @@ int check_your_resists(int hurted, beam_type flavour, string source,
         break;
 
     case BEAM_POISON:
+        hurted = resist_adjust_damage(&you, flavour, hurted);
+
         if (doEffects)
         {
             // Ensure that we received a valid beam object before proceeding.
@@ -214,24 +195,15 @@ int check_your_resists(int hurted, beam_type flavour, string source,
             ASSERT(beam);
             int pois = div_rand_round(beam->damage.num * beam->damage.size, 3);
             pois = 3 + random_range(pois * 2 / 3, pois * 4 / 3);
+            poison_player(pois, source, kaux);
 
-            resist = poison_player(pois, source, kaux) ? 0 : 1;
-
-            hurted = resist_adjust_damage(&you, flavour, resist,
-                                          hurted, true);
-            if (resist > 0)
+            if (player_res_poison() > 0)
                 canned_msg(MSG_YOU_RESIST);
         }
-        else
-        {
-            hurted = resist_adjust_damage(&you, flavour, player_res_poison(),
-                                          hurted, true);
-        }
+
         break;
 
     case BEAM_POISON_ARROW:
-        resist = player_res_poison();
-
         if (doEffects)
         {
             // Ensure that we received a valid beam object before proceeding.
@@ -241,25 +213,32 @@ int check_your_resists(int hurted, beam_type flavour, string source,
             int pois = div_rand_round(beam->damage.num * beam->damage.size, 3);
             pois = 3 + random_range(pois * 2 / 3, pois * 4 / 3);
 
+            const int resist = player_res_poison();
             poison_player((resist ? pois / 2 : pois), source, kaux, true);
         }
 
-        hurted = resist_adjust_damage(&you, flavour, resist, hurted);
+        hurted = resist_adjust_damage(&you, flavour, hurted);
         if (hurted < original && doEffects)
             canned_msg(MSG_YOU_PARTIALLY_RESIST);
         break;
 
     case BEAM_NEG:
-        hurted = resist_adjust_damage(&you, flavour, player_prot_life(),
-                                      hurted, true);
+        hurted = resist_adjust_damage(&you, flavour, hurted);
 
         if (doEffects)
+        {
+            // drain_player handles the messaging here
+            if (hurted > original)
+            {
+                mpr("The negative energy saps you greatly!");
+                xom_is_stimulated(200);
+            }
             drain_player(min(75, 35 + original * 2 / 3), true);
+        }
         break;
 
     case BEAM_ICE:
-        hurted = resist_adjust_damage(&you, flavour, player_res_cold(),
-                                      hurted, true);
+        hurted = resist_adjust_damage(&you, flavour, hurted);
 
         if (hurted < original && doEffects)
             canned_msg(MSG_YOU_PARTIALLY_RESIST);
@@ -271,8 +250,7 @@ int check_your_resists(int hurted, beam_type flavour, string source,
         break;
 
     case BEAM_LAVA:
-        hurted = resist_adjust_damage(&you, flavour, player_res_fire(),
-                                      hurted, true);
+        hurted = resist_adjust_damage(&you, flavour, hurted);
 
         if (hurted < original && doEffects)
             canned_msg(MSG_YOU_PARTIALLY_RESIST);
@@ -284,8 +262,7 @@ int check_your_resists(int hurted, beam_type flavour, string source,
         break;
 
     case BEAM_ACID:
-        hurted = resist_adjust_damage(&you, flavour, player_res_acid(),
-                                      hurted, true);
+        hurted = resist_adjust_damage(&you, flavour, hurted);
         if (hurted < original && doEffects)
             canned_msg(MSG_YOU_RESIST);
         break;
@@ -302,7 +279,7 @@ int check_your_resists(int hurted, beam_type flavour, string source,
     case BEAM_HOLY:
     {
         // Cleansing flame.
-        const int rhe = you.res_holy_energy(NULL);
+        const int rhe = you.res_holy_energy(nullptr);
         if (rhe > 0)
             hurted = 0;
         else if (rhe == 0)
@@ -318,9 +295,9 @@ int check_your_resists(int hurted, beam_type flavour, string source,
     case BEAM_AIR:
     {
         // Airstrike.
-        if (you.res_wind() > 0)
+        if (you.res_wind())
             hurted = 0;
-        else if (you.flight_mode())
+        else if (you.airborne())
             hurted += hurted / 2;
         break;
     }
@@ -338,11 +315,14 @@ int check_your_resists(int hurted, beam_type flavour, string source,
         }
         else
         {
-            hurted = resist_adjust_damage(&you, flavour,
-                                          you.res_negative_energy(),
-                                          hurted, true);
+            hurted = resist_adjust_damage(&you, flavour, hurted);
             if (hurted < original && doEffects)
                 canned_msg(MSG_YOU_PARTIALLY_RESIST);
+            else if (hurted > original && doEffects)
+            {
+                mpr("The flames sap you greatly!");
+                xom_is_stimulated(200);
+            }
         }
     }
 
@@ -353,70 +333,6 @@ int check_your_resists(int hurted, beam_type flavour, string source,
     return hurted;
 }
 
-/**
- * Attempts to apply corrosion to the player and deals acid damage.
- *
- * Each full equipment slot gives a chance of applying the corrosion debuff,
- * and each empty equipment slot increases the amount of acid damage taken.
- *
- * @param acid_strength The strength of the acid.
- * @param death_source The monster index of the acid's source.
- * @param allow_corrosion Whether to try and apply the corrosion debuff.
- * @param hurt_msg A message to display when dealing damage.
- */
-void splash_with_acid(int acid_strength, int death_source, bool allow_corrosion,
-                      const char* hurt_msg)
-{
-    int dam = 0;
-    bool do_corrosion = false;
-    const bool wearing_cloak = player_wearing_slot(EQ_CLOAK);
-
-    for (int slot = EQ_MIN_ARMOUR; slot <= EQ_MAX_ARMOUR; slot++)
-    {
-        const bool cloak_protects = wearing_cloak && coinflip()
-                                    && slot != EQ_SHIELD && slot != EQ_CLOAK;
-
-        if (!cloak_protects)
-        {
-            item_def *item = you.slot_item(static_cast<equipment_type>(slot));
-            if (!item && slot != EQ_SHIELD)
-                dam++;
-
-            if (item && allow_corrosion && x_chance_in_y(acid_strength + 1, 30))
-                do_corrosion = true;
-        }
-    }
-
-    if (do_corrosion)
-        corrode_actor(&you);
-
-    // Covers head, hands and feet.
-    if (player_equip_unrand(UNRAND_LEAR))
-        dam = !wearing_cloak;
-
-    // Without fur, clothed people have dam 0 (+2 later), Sp/Tr/Dr/Og ~1
-    // (randomized), Fe 5.  Fur helps only against naked spots.
-    const int fur = player_mutation_level(MUT_SHAGGY_FUR);
-    dam -= fur * dam / 5;
-
-    // two extra virtual slots so players can't be immune
-    dam += 2;
-    dam = roll_dice(dam, acid_strength);
-
-    const int post_res_dam = resist_adjust_damage(&you, BEAM_ACID,
-                                                  you.res_acid(), dam);
-
-    if (post_res_dam > 0)
-    {
-        mpr(hurt_msg ? hurt_msg : "The acid burns!");
-
-        if (post_res_dam < dam)
-            canned_msg(MSG_YOU_RESIST);
-
-        ouch(post_res_dam, death_source, KILLED_BY_ACID);
-    }
-}
-
 /**
  * Handle side-effects for exposure to element other than damage.
  *
@@ -459,13 +375,13 @@ static void _lose_level_abilities()
     }
 }
 
-void lose_level(int death_source, const char *aux)
+void lose_level()
 {
     // Because you.experience is unsigned long, if it's going to be
     // negative, must die straightaway.
     if (you.experience_level == 1)
     {
-        ouch(INSTANT_DEATH, death_source, KILLED_BY_DRAINING, aux);
+        ouch(INSTANT_DEATH, KILLED_BY_DRAINING);
         // Return in case death was cancelled via wizard mode
         return;
     }
@@ -475,9 +391,6 @@ void lose_level(int death_source, const char *aux)
     mprf(MSGCH_WARN,
          "You are now level %d!", you.experience_level);
 
-    ouch(4, death_source, KILLED_BY_DRAINING, aux);
-    dec_mp(1);
-
     calc_hp();
     calc_mp();
     _lose_level_abilities();
@@ -497,9 +410,9 @@ void lose_level(int death_source, const char *aux)
 
     xom_is_stimulated(200);
 
-    // Kill the player if maxhp <= 0.  We can't just move the ouch() call past
+    // Kill the player if maxhp <= 0. We can't just move the ouch() call past
     // dec_max_hp() since it would decrease hp twice, so here's another one.
-    ouch(0, death_source, KILLED_BY_DRAINING, aux);
+    ouch(0, KILLED_BY_DRAINING);
 }
 
 /**
@@ -513,9 +426,12 @@ void lose_level(int death_source, const char *aux)
  */
 bool drain_player(int power, bool announce_full, bool ignore_protection)
 {
-    const int protection = player_prot_life();
+    if (crawl_state.disables[DIS_AFFLICTIONS])
+        return false;
 
-    if (protection == 3 && !ignore_protection)
+    const int protection = ignore_protection ? 0 : player_prot_life();
+
+    if (protection == 3)
     {
         if (announce_full)
             canned_msg(MSG_YOU_RESIST);
@@ -523,7 +439,7 @@ bool drain_player(int power, bool announce_full, bool ignore_protection)
         return false;
     }
 
-    if (protection > 0 && !ignore_protection)
+    if (protection > 0)
     {
         canned_msg(MSG_YOU_PARTIALLY_RESIST);
         power /= (protection * 2);
@@ -548,7 +464,7 @@ bool drain_player(int power, bool announce_full, bool ignore_protection)
 }
 
 static void _xom_checks_damage(kill_method_type death_type,
-                               int dam, int death_source)
+                               int dam, mid_t death_source)
 {
     if (you_worship(GOD_XOM))
     {
@@ -582,13 +498,13 @@ static void _xom_checks_damage(kill_method_type death_type,
         else if (death_type != KILLED_BY_MONSTER
                     && death_type != KILLED_BY_BEAM
                     && death_type != KILLED_BY_DISINT
-                 || invalid_monster_index(death_source))
+                 || !monster_by_mid(death_source))
         {
             return;
         }
 
         int amusementvalue = 1;
-        const monster* mons = &menv[death_source];
+        const monster* mons = monster_by_mid(death_source);
 
         if (!mons->alive())
             return;
@@ -632,62 +548,99 @@ static void _xom_checks_damage(kill_method_type death_type,
     }
 }
 
-static void _yred_mirrors_injury(int dam, int death_source)
+static void _yred_mirrors_injury(int dam, mid_t death_source)
 {
     if (yred_injury_mirror())
     {
-        // Cap damage to what was enough to kill you.  Can matter if
+        // Cap damage to what was enough to kill you. Can matter if
         // Yred saves your life or you have an extra kitty.
         if (you.hp < 0)
             dam += you.hp;
 
-        if (dam <= 0 || invalid_monster_index(death_source))
+        monster* mons = monster_by_mid(death_source);
+        if (dam <= 0 || !mons)
             return;
 
-        (new mirror_damage_fineff(&menv[death_source], &you, dam))->schedule();
+        mirror_damage_fineff::schedule(mons, &you, dam);
     }
 }
 
-static void _maybe_spawn_jellies(int dam, const char* aux,
-                                  kill_method_type death_type, int death_source)
+static void _maybe_ru_retribution(int dam, mid_t death_source)
 {
+    if (will_ru_retaliate())
+    {
+        // Cap damage to what was enough to kill you. Can matter if
+        // you have an extra kitty.
+        if (you.hp < 0)
+            dam += you.hp;
+
+        monster* mons = monster_by_mid(death_source);
+        if (dam <= 0 || !mons)
+            return;
+
+        ru_retribution_fineff::schedule(mons, &you, dam);
+    }
+}
+
+static void _maybe_spawn_monsters(int dam, const char* aux,
+                                  kill_method_type death_type,
+                                  mid_t death_source)
+{
+    monster* damager = monster_by_mid(death_source);
     // We need to exclude acid damage and similar things or this function
     // will crash later.
-    if (death_source == NON_MONSTER)
+    if (!damager)
         return;
 
-    monster_type mon = royal_jelly_ejectable_monster();
-
-    // Exclude torment damage.
-    const bool torment = aux && strstr(aux, "torment");
-    if (you_worship(GOD_JIYVA)
-        && you.piety >= piety_breakpoint(5)
-        && !torment)
+    // Exclude torment damage. Ugh.
+    if (aux && (strstr(aux, "torment") || strstr(aux, "Torment")
+                || strstr(aux, "exploding lurking horror")))
     {
-        int how_many = 0;
+        return;
+    }
+
+    monster_type mon;
+    int how_many = 0;
+
+    if (you_worship(GOD_JIYVA)
+        && you.piety >= piety_breakpoint(5))
+    {
+        mon = royal_jelly_ejectable_monster();
         if (dam >= you.hp_max * 3 / 4)
             how_many = random2(4) + 2;
         else if (dam >= you.hp_max / 2)
             how_many = random2(2) + 2;
         else if (dam >= you.hp_max / 4)
             how_many = 1;
+    }
+    else if (you_worship(GOD_XOM)
+             && dam >= you.hp_max / 4
+             && x_chance_in_y(dam, 3 * you.hp_max))
+    {
+        mon = MONS_BUTTERFLY;
+        how_many = 2 + random2(5);
+    }
 
-        if (how_many > 0)
+    if (how_many > 0)
+    {
+        int count_created = 0;
+        for (int i = 0; i < how_many; ++i)
         {
-            int count_created = 0;
-            for (int i = 0; i < how_many; ++i)
+            mgen_data mg(mon, BEH_FRIENDLY, &you, 2, 0, you.pos(),
+                         damager->mindex(), 0, you.religion);
+
+            if (create_monster(mg))
+                count_created++;
+        }
+
+        if (count_created > 0)
+        {
+            if (mon == MONS_BUTTERFLY)
             {
-                int foe = death_source;
-                if (invalid_monster_index(foe))
-                    foe = MHITNOT;
-                mgen_data mg(mon, BEH_FRIENDLY, &you, 2, 0, you.pos(),
-                             foe, 0, GOD_JIYVA);
-
-                if (create_monster(mg))
-                    count_created++;
+                mprf(MSGCH_GOD, "A shower of butterflies erupts from you!");
+                take_note(Note(NOTE_XOM_EFFECT, you.piety, -1, "butterfly on damage"), true);
             }
-
-            if (count_created > 0)
+            else
             {
                 mprf("You shudder from the %s and a %s!",
                      death_type == KILLED_BY_MONSTER ? "blow" : "blast",
@@ -703,7 +656,7 @@ static void _powered_by_pain(int dam)
     const int level = player_mutation_level(MUT_POWERED_BY_PAIN);
 
     if (you.mutation[MUT_POWERED_BY_PAIN]
-        && (random2(dam) > 2 + 3 * level
+        && (random2(dam) > 4 + div_rand_round(you.experience_level, 4)
             || dam >= you.hp_max / 2))
     {
         switch (random2(4))
@@ -715,7 +668,7 @@ static void _powered_by_pain(int dam)
             {
                 mpr("You focus on the pain.");
                 int mp = roll_dice(3, 2 + 3 * level);
-                mpr("You feel your power returning.");
+                canned_msg(MSG_GAIN_MAGIC);
                 inc_mp(mp);
                 break;
             }
@@ -723,11 +676,11 @@ static void _powered_by_pain(int dam)
         }
         case 2:
             mpr("You focus on the pain.");
-            potion_effect(POT_MIGHT, level * 20);
+            potionlike_effect(POT_MIGHT, level * 20);
             break;
         case 3:
             mpr("You focus on the pain.");
-            potion_effect(POT_AGILITY, level * 20);
+            potionlike_effect(POT_AGILITY, level * 20);
             break;
         }
     }
@@ -750,6 +703,48 @@ static void _maybe_fog(int dam)
         mpr("You emit a cloud of dark smoke.");
         big_cloud(CLOUD_BLACK_SMOKE, &you, you.pos(), 50, 4 + random2(5));
     }
+    else if (you_worship(GOD_XOM) && x_chance_in_y(dam, 30 * upper_threshold))
+    {
+        mprf(MSGCH_GOD, "You emit a cloud of colourful smoke!");
+        big_cloud(CLOUD_MAGIC_TRAIL, &you, you.pos(), 50, 4 + random2(5),
+                  -1, ETC_RANDOM);
+        take_note(Note(NOTE_XOM_EFFECT, you.piety, -1, "smoke on damage"), true);
+    }
+}
+
+static void _deteriorate(int dam)
+{
+    if (x_chance_in_y(player_mutation_level(MUT_DETERIORATION), 4)
+        && dam > you.hp_max / 10)
+    {
+        mprf(MSGCH_WARN, "Your body deteriorates!");
+        lose_stat(STAT_RANDOM, 1);
+    }
+}
+
+/**
+ * Maybe corrode the player after taking damage if they're wearing *Corrode.
+ **/
+static void _maybe_corrode()
+{
+    int corrosion_sources = you.scan_artefacts(ARTP_CORRODE);
+    int degree = binomial(corrosion_sources, 3);
+    if (degree > 0)
+        you.corrode_equipment("Your corrosive artefact", degree);
+}
+
+/**
+ * Maybe confuse the player after taking damage if they're wearing *Confuse.
+ **/
+static void _maybe_confuse()
+{
+    int confusion_sources = you.scan_artefacts(ARTP_CONFUSE);
+    if (x_chance_in_y(confusion_sources, 100)) {
+        const bool conf = you.confused();
+
+        if (confuse_player(5 + random2(3), true))
+            mprf(MSGCH_WARN, "You are %sconfused.", conf ? "more " : "");
+    }
 }
 
 static void _place_player_corpse(bool explode)
@@ -757,30 +752,20 @@ static void _place_player_corpse(bool explode)
     if (!in_bounds(you.pos()))
         return;
 
-    item_def corpse;
-    if (fill_out_corpse(0, player_mons(), corpse) == MONS_NO_MONSTER)
-        return;
+    monster dummy;
+    dummy.type = player_mons(false);
+    define_monster(&dummy);
+    dummy.position = you.pos();
+    dummy.props["always_corpse"] = true;
+    dummy.mname = you.your_name;
+    dummy.set_hit_dice(you.experience_level);
+    if (explode)
+        dummy.flags &= MF_EXPLODE_KILL;
 
-    if (explode && explode_corpse(corpse, you.pos()))
-        return;
+    if (you.form != TRAN_NONE)
+        mpr("Your shape twists and changes as you die.");
 
-    int o = get_mitm_slot();
-    if (o == NON_ITEM)
-    {
-        item_was_destroyed(corpse);
-        return;
-    }
-
-    corpse.props[MONSTER_HIT_DICE].get_short() = you.experience_level;
-    corpse.props[CORPSE_NAME_KEY] = you.your_name;
-    corpse.props[CORPSE_NAME_TYPE_KEY].get_int64() = 0;
-    corpse.props["ev"].get_int() = player_evasion(static_cast<ev_ignore_type>(
-                                   EV_IGNORE_HELPLESS | EV_IGNORE_PHASESHIFT));
-    // mostly mutations here.  At least there's no need to handle armour.
-    corpse.props["ac"].get_int() = you.armour_class();
-    mitm[o] = corpse;
-
-    move_item_to_grid(&o, you.pos(), !you.in_water());
+    place_monster_corpse(dummy, false);
 }
 
 #if defined(WIZARD) || defined(DEBUG)
@@ -802,10 +787,52 @@ void reset_damage_counters()
     you.source_damage = 0;
 }
 
-// death_source should be set to NON_MONSTER for non-monsters. {dlb}
-void ouch(int dam, int death_source, kill_method_type death_type,
-          const char *aux, bool see_source, const char *death_source_name,
-          bool attacker_effects)
+bool can_shave_damage()
+{
+    return you.species == SP_DEEP_DWARF || you.duration[DUR_FORTITUDE];
+}
+
+int do_shave_damage(int dam)
+{
+    if (you.species == SP_DEEP_DWARF)
+    {
+        // Deep Dwarves get to shave any hp loss.
+        int shave = 1 + random2(2 + random2(1 + you.experience_level / 3));
+        dprf("HP shaved: %d.", shave);
+        dam -= shave;
+    }
+
+    if (you.duration[DUR_FORTITUDE])
+        dam -= random2(10);
+
+    return dam;
+}
+
+// Determine what's threatening for purposes of sacrifice drink and reading.
+// the statuses are guaranteed not to happen if the incoming damage is less
+// than 4% max hp. Otherwise, they scale up with damage taken and with lower
+// health, becoming certain at 20% max health damage or <30% max health
+// current hp.
+static bool _is_damage_threatening (int damage_fraction_of_hp)
+{
+    int hp_fraction = you.hp * 100 / you.hp_max;
+    return damage_fraction_of_hp > 5
+            && hp_fraction <= 85
+            && (damage_fraction_of_hp + random2(20) >= 20
+                || random2(100) < hp_fraction);
+}
+
+/** Hurt the player. Isn't it fun?
+ *
+ *  @param dam How much damage -- may be INSTANT_DEATH.
+ *  @param death_type how did you get hurt?
+ *  @param source who could do such a thing?
+ *  @param aux what did they do it with?
+ *  @param see_source whether the attacker was visible to you
+ *  @param death_source_name the attacker's name if it is already dead.
+ */
+void ouch(int dam, kill_method_type death_type, mid_t source, const char *aux,
+          bool see_source, const char *death_source_name)
 {
     ASSERT(!crawl_state.game_is_arena());
     if (you.duration[DUR_TIME_STEP])
@@ -816,39 +843,10 @@ void ouch(int dam, int death_source, kill_method_type death_type,
 
     int drain_amount = 0;
 
-    if (attacker_effects && dam != INSTANT_DEATH
-        && !invalid_monster_index(death_source)
-        && menv[death_source].has_ench(ENCH_WRETCHED))
+    if (can_shave_damage() && dam != INSTANT_DEATH
+        && death_type != KILLED_BY_POISON)
     {
-        // An abstract boring simulation of reduced stats/etc due to bad muts
-        // reducing the monster's melee damage, spell power, etc.  This is
-        // wrong eg. for clouds that would be smaller and with a shorter
-        // duration but do the same damage, etc.
-        int degree = menv[death_source].get_ench(ENCH_WRETCHED).degree;
-        dam = div_rand_round(dam * (10 - min(degree, 5)), 10);
-    }
-
-    if ((you.duration[DUR_FORTITUDE] || you.species == SP_DEEP_DWARF)
-         && dam != INSTANT_DEATH && death_type != KILLED_BY_POISON)
-    {
-        if (you.species == SP_DEEP_DWARF)
-        {
-            // Deep Dwarves get to shave any hp loss.
-            int shave = 1 + random2(2 + random2(1 + you.experience_level / 3));
-            dprf("HP shaved: %d.", shave);
-            dam -= shave;
-        }
-
-        if (you.duration[DUR_FORTITUDE])
-            dam -= random2(10);
-
-        if (dam <= 0)
-        {
-            // Rotting and costs may lower hp directly.
-            if (you.hp > 0)
-                return;
-            dam = 0;
-        }
+        dam = max(0, do_shave_damage(dam));
     }
 
     if (dam != INSTANT_DEATH)
@@ -870,17 +868,57 @@ void ouch(int dam, int death_source, kill_method_type death_type,
         you.check_awaken(500);
 
     const bool non_death = death_type == KILLED_BY_QUITTING
-        || death_type == KILLED_BY_WINNING
-        || death_type == KILLED_BY_LEAVING;
+                        || death_type == KILLED_BY_WINNING
+                        || death_type == KILLED_BY_LEAVING;
 
-    if (you.duration[DUR_DEATHS_DOOR] && death_type != KILLED_BY_LAVA
-        && death_type != KILLED_BY_WATER && !non_death && you.hp_max > 0)
+    // certain effects (e.g. drowned souls) use KILLED_BY_WATER for flavour
+    // reasons (morgue messages?), with regrettable consequences if we don't
+    // double-check.
+    const bool env_death = source == MID_NOBODY
+                           && (death_type == KILLED_BY_LAVA
+                               || death_type == KILLED_BY_WATER);
+
+    // death's door protects against everything but falling into water/lava,
+    // excessive rot, leaving the dungeon, or quitting.
+    if (you.duration[DUR_DEATHS_DOOR] && !env_death && !non_death
+        && you.hp_max > 0)
     {
         return;
     }
 
+    if (dam > 0 && death_type != KILLED_BY_POISON)
+    {
+        int damage_fraction_of_hp = dam * 100 / you.hp_max;
+
+        // Check _is_damage_threatening separately for read and drink so they
+        // don't always trigger in unison when you have both.
+        if (player_mutation_level(MUT_NO_READ))
+        {
+            if (_is_damage_threatening(damage_fraction_of_hp))
+            {
+                if (!you.duration[DUR_NO_SCROLLS])
+                    mpr("You feel threatened and lose the ability to read scrolls!");
+
+                you.increase_duration(DUR_NO_SCROLLS, 1 + random2(dam), 30);
+            }
+        }
+
+        if (player_mutation_level(MUT_NO_DRINK))
+        {
+            if (_is_damage_threatening(damage_fraction_of_hp))
+            {
+                if (!you.duration[DUR_NO_POTIONS])
+                    mpr("You feel threatened and lose the ability to drink potions!");
+
+                you.increase_duration(DUR_NO_POTIONS, 1 + random2(dam), 30);
+            }
+        }
+    }
+
     if (dam != INSTANT_DEATH)
     {
+        you.maybe_degrade_bone_armour(BONE_ARMOUR_HIT_RATIO);
+
         if (you.spirit_shield() && death_type != KILLED_BY_POISON
             && !(aux && strstr(aux, "flay_damage")))
         {
@@ -904,9 +942,9 @@ void ouch(int dam, int death_source, kill_method_type death_type,
         }
 
         you.turn_damage += dam;
-        if (you.damage_source != death_source)
+        if (you.damage_source != source)
         {
-            you.damage_source = death_source;
+            you.damage_source = source;
             you.source_damage = 0;
         }
         you.source_damage += dam;
@@ -924,13 +962,14 @@ void ouch(int dam, int death_source, kill_method_type death_type,
                 && you.hp <= (you.hp_max * Options.hp_warning) / 100
                 && (death_type != KILLED_BY_POISON || poison_is_lethal()))
             {
+                flash_view_delay(UA_HP, RED, 50);
                 mprf(MSGCH_DANGER, "* * * LOW HITPOINT WARNING * * *");
                 dungeon_events.fire_event(DET_HP_WARNING);
             }
 
             hints_healing_check();
 
-            _xom_checks_damage(death_type, dam, death_source);
+            _xom_checks_damage(death_type, dam, source);
 
             // for note taking
             string damage_desc;
@@ -938,7 +977,7 @@ void ouch(int dam, int death_source, kill_method_type death_type,
                 damage_desc = make_stringf("something (%d)", dam);
             else
             {
-                damage_desc = scorefile_entry(dam, death_source,
+                damage_desc = scorefile_entry(dam, source,
                                               death_type, aux, true)
                     .death_description(scorefile_entry::DDV_TERSE);
             }
@@ -946,10 +985,17 @@ void ouch(int dam, int death_source, kill_method_type death_type,
             take_note(Note(NOTE_HP_CHANGE, you.hp, you.hp_max,
                            damage_desc.c_str()));
 
-            _yred_mirrors_injury(dam, death_source);
-            _maybe_spawn_jellies(dam, aux, death_type, death_source);
+            _deteriorate(dam);
+            _yred_mirrors_injury(dam, source);
+            _maybe_ru_retribution(dam, source);
+            _maybe_spawn_monsters(dam, aux, death_type, source);
             _maybe_fog(dam);
             _powered_by_pain(dam);
+            if (death_type != KILLED_BY_POISON)
+            {
+                _maybe_corrode();
+                _maybe_confuse();
+            }
             if (drain_amount > 0)
                 drain_player(drain_amount, true, true);
         }
@@ -963,7 +1009,7 @@ void ouch(int dam, int death_source, kill_method_type death_type,
         && crawl_state.other_gods_acting().empty())
     {
         you.escaped_death_cause = death_type;
-        you.escaped_death_aux   = aux == NULL ? "" : aux;
+        you.escaped_death_aux   = aux == nullptr ? "" : aux;
 
         // Xom should only kill his worshippers if they're under penance
         // or Xom is bored.
@@ -985,12 +1031,12 @@ void ouch(int dam, int death_source, kill_method_type death_type,
         you.reset_escaped_death();
 
         // Ensure some minimal information about Xom's involvement.
-        if (aux == NULL || !*aux)
+        if (aux == nullptr || !*aux)
         {
             if (death_type != KILLED_BY_XOM)
                 aux = "Xom";
         }
-        else if (strstr(aux, "Xom") == NULL)
+        else if (strstr(aux, "Xom") == nullptr)
             death_type = KILLED_BY_XOM;
     }
     // Xom may still try to save your life.
@@ -1007,14 +1053,17 @@ void ouch(int dam, int death_source, kill_method_type death_type,
 
     crawl_state.cancel_cmd_all();
 
+    if (non_death)
+        you.delay_queue.clear(); // don't lose ev for taking the exit...
+
     // Construct scorefile entry.
-    scorefile_entry se(dam, death_source, death_type, aux, false,
+    scorefile_entry se(dam, source, death_type, aux, false,
                        death_source_name);
 
 #ifdef WIZARD
     if (!non_death)
     {
-        if (crawl_state.test || you.wizard)
+        if (crawl_state.test || you.wizard || (you.explore && !you.lives))
         {
             const string death_desc
                 = se.death_description(scorefile_entry::DDV_VERBOSE);
@@ -1077,7 +1126,7 @@ void ouch(int dam, int death_source, kill_method_type death_type,
     activate_notes(false);
 
 #ifndef SCORE_WIZARD_CHARACTERS
-    if (!you.wizard)
+    if (!you.wizard && !you.explore)
 #endif
     {
         // Add this highscore to the score file.
@@ -1105,12 +1154,7 @@ string morgue_name(string char_name, time_t when_crawl_got_even)
 
 int actor_to_death_source(const actor* agent)
 {
-    if (agent->is_player())
-        return NON_MONSTER;
-    else if (agent->is_monster())
-        return agent->as_monster()->mindex();
-    else
-        return NON_MONSTER;
+    return agent ? agent->mindex() : NON_MONSTER;
 }
 
 int timescale_damage(const actor *act, int damage)
