@@ -15,13 +15,13 @@ class text_pattern : public base_pattern
 {
 public:
     text_pattern(const string &s, bool icase = false)
-        : pattern(s), compiled_pattern(NULL),
+        : pattern(s), compiled_pattern(nullptr),
           isvalid(true), ignore_case(icase)
     {
     }
 
     text_pattern()
-        : pattern(), compiled_pattern(NULL),
+        : pattern(), compiled_pattern(nullptr),
          isvalid(false), ignore_case(false)
     {
     }
@@ -29,7 +29,7 @@ public:
     text_pattern(const text_pattern &tp)
         : base_pattern(tp),
           pattern(tp.pattern),
-          compiled_pattern(NULL),
+          compiled_pattern(nullptr),
           isvalid(tp.isvalid),
           ignore_case(tp.ignore_case)
     {
@@ -43,7 +43,7 @@ public:
 
     bool empty() const { return !pattern.length(); }
 
-    bool valid() const
+    bool valid() const override
     {
         return isvalid
             && (compiled_pattern || (isvalid = compile()));
@@ -56,12 +56,12 @@ public:
         return matches(s, strlen(s));
     }
 
-    bool matches(const string &s) const
+    bool matches(const string &s) const override
     {
         return matches(s.c_str(), s.length());
     }
 
-    const string &tostring() const
+    const string &tostring() const override
     {
         return pattern;
     }
