@@ -128,6 +128,6 @@ PlaceInfo& player::get_place_info(branch_type branch) const
 void player::clear_place_info()
 {
     global_info = PlaceInfo();
-    for (unsigned int i = 0; i < NUM_BRANCHES; ++i)
-        branch_info[i] = PlaceInfo();
+    for (branch_iterator it; it; ++it)
+        branch_info[it->id] = PlaceInfo();
 }
