@@ -1,48 +1,53 @@
 #include "AppHdr.h"
 
-#include <math.h>
-#include "asg.h"
 #include "random.h"
-#include "syscalls.h"
 
-#ifdef UNIX
-// for times()
-#include <sys/times.h>
-#endif
-
-// for getpid()
-#include <sys/types.h>
+#include <cmath>
 #ifndef TARGET_COMPILER_VC
 # include <unistd.h>
 #else
 # include <process.h>
 #endif
 
+#include "pcg.h"
+#include "syscalls.h"
+
+static FixedVector<PcgRNG, 2> rngs;
+
+uint32_t get_uint32(int generator)
+{
+    return rngs[generator].get_uint32();
+}
+
+uint64_t get_uint64(int generator)
+{
+    return rngs[generator].get_uint64();
+}
+
+static void _seed_rng(uint64_t seed_array[], int seed_len)
+{
+    PcgRNG seeded(seed_array, seed_len);
+    // Use the just seeded RNG to initialize the rest.
+    for (PcgRNG& rng : rngs)
+    {
+        uint64_t key[2] = { seeded.get_uint64(), seeded.get_uint64() };
+        rng = PcgRNG(key, ARRAYSZ(key));
+    }
+}
+
 void seed_rng(uint32_t seed)
 {
-    uint32_t sarg[1] = { seed };
-    seed_asg(sarg, 1);
+    uint64_t sarg[1] = { seed };
+    _seed_rng(sarg, ARRAYSZ(sarg));
 }
 
 void seed_rng()
 {
-    /* Use a 160-bit wide seed */
-    uint32_t seed_key[5];
-    read_urandom((char*)(&seed_key), sizeof(seed_key));
-
-#ifdef UNIX
-    struct tms buf;
-    seed_key[0] += times(&buf);
-#endif
-    seed_key[1] += getpid();
-    seed_key[2] += time(NULL);
-
-    seed_asg(seed_key, 5);
-}
-
-uint32_t random_int()
-{
-    return get_uint32();
+    /* Use a 128-bit wide seed */
+    uint64_t seed_key[2];
+    bool seeded = read_urandom((char*)(&seed_key), sizeof(seed_key));
+    ASSERT(seeded);
+    _seed_rng(seed_key, ARRAYSZ(seed_key));
 }
 
 // [low, high]
@@ -60,33 +65,6 @@ int random_range(int low, int high, int nrolls)
     return low + roll;
 }
 
-// Chooses one of the strings passed in at random. The list of strings
-// must be terminated with NULL.  NULL is not -1, and 0 is popular
-// value for enums, so we need to copy the function.
-template <>
-const char* random_choose<const char*>(const char* first, ...)
-{
-    va_list args;
-    va_start(args, first);
-
-    const char* chosen = first;
-    int count = 1, nargs = 100;
-
-    while (nargs-- > 0)
-    {
-        char* pick = va_arg(args, char*);
-        if (pick == NULL)
-            break;
-        if (one_chance_in(++count))
-            chosen = pick;
-    }
-
-    ASSERT(nargs > 0);
-
-    va_end(args);
-    return chosen;
-}
-
 const char* random_choose_weighted(int weight, const char* first, ...)
 {
     va_list args;
@@ -111,16 +89,12 @@ const char* random_choose_weighted(int weight, const char* first, ...)
     return chosen;
 }
 
-#ifndef UINT32_MAX
-#define UINT32_MAX ((uint32_t)(-1))
-#endif
-
 static int _random2(int max, int rng)
 {
     if (max <= 1)
         return 0;
 
-    uint32_t partn = UINT32_MAX / max;
+    uint32_t partn = PcgRNG::max() / max;
 
     while (true)
     {
@@ -135,13 +109,13 @@ static int _random2(int max, int rng)
 // [0, max)
 int random2(int max)
 {
-    return _random2(max, 0);
+    return _random2(max, RNG_GAMEPLAY);
 }
 
 // [0, max), separate RNG state
 int ui_random(int max)
 {
-    return _random2(max, 1);
+    return _random2(max, RNG_UI);
 }
 
 // [0, 1]
@@ -188,7 +162,7 @@ int roll_dice(int num, int size)
     int ret = 0;
 
     // If num <= 0 or size <= 0, then we'll just return the default
-    // value of zero.  This is good behaviour in that it will be
+    // value of zero. This is good behaviour in that it will be
     // appropriate for calculated values that might be passed in.
     if (num > 0 && size > 0)
     {
@@ -294,39 +268,56 @@ int biased_random2(int max, int n)
 // [0, max]
 int random2limit(int max, int limit)
 {
-    int i;
     int sum = 0;
 
     if (max < 1)
         return 0;
 
-    for (i = 0; i < max; i++)
+    for (int i = 0; i < max; i++)
         if (random2(limit) >= i)
             sum++;
 
     return sum;
 }
 
-// Generate samples from a binomial distribution with n_trials and trial_prob
-// probability of success per trial. trial_prob is an integer less than 100
-// representing the % chance of success.
-// This just evaluates all n trials, there is probably an efficient way of
-// doing this but I'm not much of a statistician. -CAO
-// [0, n_trials]
-int binomial_generator(unsigned n_trials, unsigned trial_prob)
+/** Sample from a binomial distribution.
+ *
+ * This is the number of successes in a sequence of independent trials with
+ * fixed probability.
+ *
+ * @param n_trials The number of trials.
+ * @param trial_prob The numerator of the probability of success of each trial.
+ *                   If greater than scale, the probability is 1.0.
+ * @param scale The denominator of trial_prob, default 100.
+ * @return the number of successes, range [0, n_trials]
+ */
+int binomial(unsigned n_trials, unsigned trial_prob, unsigned scale)
 {
     int count = 0;
     for (unsigned i = 0; i < n_trials; ++i)
-        if (::x_chance_in_y(trial_prob, 100))
+        if (::x_chance_in_y(trial_prob, scale))
             count++;
 
     return count;
 }
 
 // range [0, 1.0)
+// This uses a technique described by Saito and Matsumoto at
+// MCQMC'08. Given that the IEEE floating point numbers are
+// uniformly distributed over [1,2), we generate a number in
+// this range and then offset it onto the range [0,1). The
+// choice of bits (masking v. shifting) is arbitrary and
+// should be immaterial for high quality generators.
 double random_real()
 {
-    return get_uint32() / 4294967296.0;
+    static const uint64_t UPPER_MASK = 0x3FF0000000000000ULL;
+    static const uint64_t LOWER_MASK = 0xFFFFFFFFFFFFFULL;
+    const uint64_t value = UPPER_MASK | (get_uint64() & LOWER_MASK);
+    double result;
+    // Portable memory transmutation. The union trick almost always
+    // works, but this is safer.
+    memcpy(&result, &value, sizeof(value));
+    return result - 1.0;
 }
 
 // Roll n_trials, return true if at least one succeeded.  n_trials might be
@@ -363,12 +354,12 @@ int fuzz_value(int val, int lowfuzz, int highfuzz, int naverage)
     return val + random2avg(lfuzz + hfuzz + 1, naverage) - lfuzz;
 }
 
-bool decimal_chance(double percent)
+bool decimal_chance(double chance)
 {
-    return random_real() < percent;
+    return random_real() < chance;
 }
 
-// This is used when the front-end randomness is inconclusive.  There are
+// This is used when the front-end randomness is inconclusive. There are
 // never more than two possibilities, which simplifies things.
 bool defer_rand::x_chance_in_y_contd(int x, int y, int index)
 {
