@@ -1,13 +1,22 @@
 #ifndef RANDOM_H
 #define RANDOM_H
 
+#include <algorithm>  // iter_swap
+#include <iterator>   // advance
 #include <map>
 #include <vector>
+
 #include "hash.h"
 
+#define RNG_GAMEPLAY 0
+#define RNG_UI 1
+
 void seed_rng();
 void seed_rng(uint32_t seed);
+void seed_rng(uint64_t[], int);
 
+uint32_t get_uint32(int generator = RNG_GAMEPLAY);
+uint64_t get_uint64(int generator = RNG_GAMEPLAY);
 bool coinflip();
 int div_rand_round(int num, int den);
 int div_round_up(int num, int den);
@@ -19,14 +28,13 @@ int maybe_random_div(int nom, int denom, bool random_factor);
 int maybe_roll_dice(int num, int size, bool random);
 int random_range(int low, int high);
 int random_range(int low, int high, int nrolls);
-uint32_t random_int();
 double random_real();
 
 int random2avg(int max, int rolls);
 int bestroll(int max, int rolls);
 int biased_random2(int max, int n);
 int random2limit(int max, int limit);
-int binomial_generator(unsigned n_trials, unsigned trial_prob);
+int binomial(unsigned n_trials, unsigned trial_prob, unsigned scale = 100);
 bool bernoulli(double n_trials, double trial_prob);
 int fuzz_value(int val, int lowfuzz, int highfuzz, int naverage = 2);
 int roll_dice(int num, int size);
@@ -34,36 +42,24 @@ bool decimal_chance(double percent);
 
 int ui_random(int max);
 
-/**
- * Chooses one of the numbers passed in at random. The list of numbers
- * must be terminated with -1.
+/** Chooses one of the objects passed in at random (by value).
+ *  @return One of the arguments.
  */
-template <typename T>
-T random_choose(T first, ...)
+template <typename T, typename... Ts>
+T random_choose(T first, Ts... rest)
 {
-    va_list args;
-    va_start(args, first);
-
-    T chosen = first;
-    int count = 1;
-    int nargs = 100; // a hard limit to catch unterminated uses
-
-    while (nargs-- > 0)
-    {
-        const int pick = va_arg(args, int);
-        if (pick == -1)
-            break;
-        if (one_chance_in(++count))
-            chosen = static_cast<T>(pick);
-    }
-
-    va_end(args);
-    ASSERT(nargs > 0);
-    return chosen;
+    const T elts[] = { first, rest... };
+    return elts[random2(1 + sizeof...(rest))];
 }
 
-template <>
-const char* random_choose<const char*>(const char* first, ...);
+template <typename C>
+auto random_iterator(C &container) -> decltype(container.begin())
+{
+    int pos = random2(container.size());
+    auto it = container.begin();
+    advance(it, pos);
+    return it;
+}
 
 template <typename T>
 T random_choose_weighted(int weight, T first, ...)
@@ -104,31 +100,35 @@ struct dice_def
 
 dice_def calc_dice(int num_dice, int max_damage);
 
-template <typename T>
-void shuffle_array(T* arr, int n)
+// I must be a random-access iterator.
+template <typename I>
+void shuffle_array(I begin, I end)
 {
+    size_t n = end - begin;
     while (n > 1)
     {
-        int i = random2(n);
+        const int i = random2(n);
         n--;
-        T tmp = arr[i];
-        arr[i] = arr[n];
-        arr[n] = tmp;
+        iter_swap(begin + i, begin + n);
     }
 }
 
 template <typename T>
-void shuffle_array(vector<T> &vec)
+void shuffle_array(T &vec)
 {
-    // &vec[0] is undefined behaviour, and vec.data() is C++11-only.
-    if (!vec.empty())
-        shuffle_array(&vec[0], vec.size());
+    shuffle_array(begin(vec), end(vec));
+}
+
+template <typename T>
+void shuffle_array(T *arr, int n)
+{
+    shuffle_array(arr, arr + n);
 }
 
 /**
  * A defer_rand object represents an infinite tree of random values, allowing
  * for a much more functional approach to randomness.  defer_rand values which
- * have been used should not be copy-constructed.  Querying the same path
+ * have been used should not be copy-constructed. Querying the same path
  * multiple times will always give the same result.
  *
  * An important property of defer_rand is that, except for rounding,
@@ -142,10 +142,10 @@ void shuffle_array(vector<T> &vec)
  * Rephrased: The first time any node in the tree has a method called on
  * it, a random float between 0 and 1 (the fraction) is generated and stored,
  * and this float is combined with the method's parameters to arrive at
- * the result.  Calling the same method on the same node with the same
+ * the result. Calling the same method on the same node with the same
  * parameters will always give the same result, while different parameters
  * or methods will give different results (though they'll all use the same
- * float which was generated by the first method call).  Each node in the
+ * float which was generated by the first method call). Each node in the
  * tree has it's own float, so the same method+parameters on different
  * nodes will get different results.
  */
