@@ -97,7 +97,7 @@ bool ranged_attack::attack()
         return true;
     }
 
-    const int ev = defender->melee_evasion(attacker);
+    const int ev = defender->evasion(EV_IGNORE_NONE, attacker);
     ev_margin = test_hit(to_hit, ev, !attacker->is_player());
     bool shield_blocked = attack_shield_blocked(false);
 
@@ -121,6 +121,12 @@ bool ranged_attack::attack()
     {
         if (ev_margin >= 0)
         {
+            if (attacker != defender && attack_warded_off())
+            {
+                handle_phase_end();
+                return false;
+            }
+
             if (!handle_phase_hit())
             {
                 if (!defender->alive())
@@ -208,7 +214,7 @@ bool ranged_attack::handle_phase_dodged()
 {
     did_hit = false;
 
-    const int ev = defender->melee_evasion(attacker);
+    const int ev = defender->evasion(EV_IGNORE_NONE, attacker);
 
     const int orig_ev_margin =
         test_hit(orig_to_hit, ev, !attacker->is_player());
@@ -233,8 +239,8 @@ bool ranged_attack::handle_phase_dodged()
         return true;
     }
 
-    const int ev_nophase = defender->melee_evasion(attacker,
-                                                   EV_IGNORE_PHASESHIFT);
+    const int ev_nophase = defender->evasion(EV_IGNORE_PHASESHIFT, attacker);
+
     if (ev_margin + (ev - ev_nophase) > 0)
     {
         if (needs_message && defender_visible)
@@ -377,8 +383,7 @@ int ranged_attack::calc_mon_to_hit_base()
     return 18 + attacker->get_hit_dice() * hd_mult / 6;
 }
 
-int ranged_attack::apply_damage_modifiers(int damage, int damage_max,
-                                          bool &half_ac)
+int ranged_attack::apply_damage_modifiers(int damage, int damage_max)
 {
     ASSERT(attacker->is_monster());
     if (attacker->as_monster()->is_archer())
@@ -386,7 +391,6 @@ int ranged_attack::apply_damage_modifiers(int damage, int damage_max,
         const int bonus = attacker->get_hit_dice() * 4 / 3;
         damage += random2avg(bonus, 2);
     }
-    half_ac = false;
     return damage;
 }
 
@@ -536,7 +540,7 @@ special_missile_type ranged_attack::random_chaos_missile_brand()
     // Pretty much duplicated by the chaos effect note,
     // which will be much more informative.
     if (brand != SPMSL_CHAOS)
-        take_note(Note(NOTE_MESSAGE, 0, 0, brand_name.c_str()), true);
+        take_note(Note(NOTE_MESSAGE, 0, 0, brand_name), true);
 #endif
     return brand;
 }
@@ -741,7 +745,7 @@ bool ranged_attack::apply_missile_brand()
                                          no_sanct))
                 {
                     const coord_def from = attacker->pos();
-                    if (distance2(pos2, from) > distance2(pos, from))
+                    if (grid_distance(pos2, from) > grid_distance(pos, from))
                         pos = pos2;
 
                     if (defender->is_player())
