@@ -7,45 +7,35 @@
 
 #include "shout.h"
 
+#include <sstream>
+
 #include "act-iter.h"
-#include "art-enum.h"
-#include "artefact.h"
 #include "areas.h"
+#include "artefact.h"
+#include "art-enum.h"
 #include "branch.h"
-#include "cluautil.h"
-#include "coord.h"
 #include "database.h"
-#include "dlua.h"
+#include "directn.h"
+#include "english.h"
 #include "env.h"
 #include "exercise.h"
 #include "ghost.h"
+#include "godabil.h"
 #include "hints.h"
 #include "jobs.h"
 #include "libutil.h"
-#include "los.h"
+#include "macro.h"
 #include "message.h"
-#include "misc.h"
 #include "mon-behv.h"
-#include "mon-chimera.h"
 #include "mon-place.h"
 #include "mon-poly.h"
-#include "mon-pathfind.h"
-#include "mon-util.h"
-#include "monster.h"
-#include "noise.h"
-#include "player.h"
-#include "random.h"
+#include "prompt.h"
 #include "religion.h"
-#include "skills.h"
 #include "state.h"
 #include "stringutil.h"
 #include "terrain.h"
 #include "view.h"
 
-#include <sstream>
-
-extern int stealth;             // defined in main.cc
-
 static noise_grid _noise_grid;
 static void _actor_apply_noise(actor *act,
                                const coord_def &apparent_source,
@@ -68,14 +58,6 @@ void handle_monster_shouts(monster* mons, bool force)
     // choose a random verb and loudness for them.
     shout_type  s_type = mons_shouts(mons->type, false);
 
-    // Chimera can take a random shout type from any of their
-    // three components
-    if (mons->type == MONS_CHIMERA)
-    {
-        monster_type acting = mons->ghost->acting_part != MONS_0
-            ? mons->ghost->acting_part : random_chimera_part(mons);
-        s_type = mons_shouts(acting, false);
-    }
     // Silent monsters can give noiseless "visual shouts" if the
     // player can see them, in which case silence isn't checked for.
     // Muted monsters can't shout at all.
@@ -101,6 +83,9 @@ void handle_monster_shouts(monster* mons, bool force)
     case S_BARK:
         default_msg_key = "__BARK";
         break;
+    case S_HOWL:
+        default_msg_key = "__HOWL";
+        break;
     case S_SHOUT2:
         default_msg_key = "__TWO_SHOUTS";
         break;
@@ -116,6 +101,9 @@ void handle_monster_shouts(monster* mons, bool force)
     case S_TRUMPET:
         default_msg_key = "__TRUMPET";
         break;
+#if TAG_MAJOR_VERSION == 34
+    case S_CAW:
+#endif
     case S_SCREECH:
         default_msg_key = "__SCREECH";
         break;
@@ -128,9 +116,6 @@ void handle_monster_shouts(monster* mons, bool force)
     case S_GURGLE:
         default_msg_key = "__GURGLE";
         break;
-    case S_WHINE:
-        default_msg_key = "__WHINE";
-        break;
     case S_CROAK:
         default_msg_key = "__CROAK";
         break;
@@ -143,12 +128,12 @@ void handle_monster_shouts(monster* mons, bool force)
     case S_DEMON_TAUNT:
         default_msg_key = "__DEMON_TAUNT";
         break;
-    case S_CAW:
-        default_msg_key = "__CAW";
-        break;
     case S_CHERUB:
         default_msg_key = "__CHERUB";
         break;
+    case S_RUMBLE:
+        default_msg_key = "__RUMBLE";
+        break;
     default:
         default_msg_key = "__BUGGY"; // S_LOUD, S_VERY_SOFT, etc. (loudness)
     }
@@ -181,7 +166,7 @@ void handle_monster_shouts(monster* mons, bool force)
     // false for submerged monsters, but submerged monsters will be forced
     // to surface before they shout, thus removing that source of
     // non-visibility.
-    if (you.can_see(mons))
+    if (you.can_see(*mons))
         suffix = " seen";
     else
         suffix = " unseen";
@@ -252,7 +237,7 @@ void handle_monster_shouts(monster* mons, bool force)
                 return;
             }
 
-            if (you.can_see(mons))
+            if (you.can_see(*mons))
             {
                 if (!monster_habitable_grid(mons, DNGN_FLOOR))
                     mons->seen_context = SC_FISH_SURFACES_SHOUT;
@@ -264,10 +249,10 @@ void handle_monster_shouts(monster* mons, bool force)
             }
         }
 
-        if (channel != MSGCH_TALK_VISUAL || you.can_see(mons))
+        if (channel != MSGCH_TALK_VISUAL || you.can_see(*mons))
         {
             // Otherwise it can move away with no feedback.
-            if (you.can_see(mons))
+            if (you.can_see(*mons))
             {
                 if (!(mons->flags & MF_WAS_IN_VIEW))
                     handle_seen_interrupt(mons);
@@ -280,13 +265,13 @@ void handle_monster_shouts(monster* mons, bool force)
     }
 
     const int  noise_level = get_shout_noise_level(s_type);
-    const bool heard       = noisy(noise_level, mons->pos(), mons->mindex());
+    const bool heard       = noisy(noise_level, mons->pos(), mons->mid);
 
-    if (crawl_state.game_is_hints() && (heard || you.can_see(mons)))
+    if (crawl_state.game_is_hints() && (heard || you.can_see(*mons)))
         learned_something_new(HINT_MONSTER_SHOUT, mons->pos());
 }
 
-bool check_awaken(monster* mons)
+bool check_awaken(monster* mons, int stealth)
 {
     // Usually redundant because we iterate over player LOS,
     // but e.g. for you.xray_vision.
@@ -302,9 +287,12 @@ bool check_awaken(monster* mons)
     if (you.berserk())
         return true;
 
-    // I assume that creatures who can sense invisible are very perceptive.
-    int mons_perc = 10 + (mons_intel(mons) * 4) + mons->get_hit_dice()
-                       + mons_sense_invis(mons) * 5;
+    // If you've sacrificed stealth, you always alert monsters.
+    if (player_mutation_level(MUT_NO_STEALTH))
+        return true;
+
+
+    int mons_perc = 10 + (mons_intel(mons) * 4) + mons->get_hit_dice();
 
     bool unnatural_stealthy = false; // "stealthy" only because of invisibility?
 
@@ -338,17 +326,17 @@ bool check_awaken(monster* mons)
         }
     }
 
-    if (mons_perc < 0)
-        mons_perc = 0;
+    if (mons_perc < 4)
+        mons_perc = 4;
 
     if (x_chance_in_y(mons_perc + 1, stealth))
         return true; // Oops, the monster wakes up!
 
     // You didn't wake the monster!
-    if (you.can_see(mons) // to avoid leaking information
+    if (you.can_see(*mons) // to avoid leaking information
         && !mons->wont_attack()
         && !mons->neutral() // include pacified monsters
-        && !mons_class_flag(mons->type, M_NO_EXP_GAIN))
+        && mons_class_gives_xp(mons->type))
     {
         practise(unnatural_stealthy ? EX_SNEAK_INVIS : EX_SNEAK);
     }
@@ -406,7 +394,7 @@ void item_noise(const item_def &item, string msg, int loudness)
         msg = "You hear a strange noise.";
     }
 
-    // Replace weapon references.  Can't use DESC_THE because that includes
+    // Replace weapon references. Can't use DESC_THE because that includes
     // pluses etc. and we want just the basename.
     msg = replace_all(msg, "@The_weapon@", "The @weapon@");
     msg = replace_all(msg, "@the_weapon@", "the @weapon@");
@@ -419,11 +407,14 @@ void item_noise(const item_def &item, string msg, int loudness)
     msg = replace_all(msg, "@player_god@",
                       you_worship(GOD_NO_GOD) ? "atheism"
                       : god_name(you.religion, coinflip()));
-    msg = replace_all(msg, "@player_genus@", species_name(you.species, true));
+    msg = replace_all(msg, "@player_genus@", species_name(you.species, SPNAME_GENUS));
     msg = replace_all(msg, "@a_player_genus@",
-                          article_a(species_name(you.species, true)));
+                          article_a(species_name(you.species, SPNAME_GENUS)));
     msg = replace_all(msg, "@player_genus_plural@",
-                      pluralise(species_name(you.species, true)));
+                      pluralise(species_name(you.species, SPNAME_GENUS)));
+
+    msg = maybe_pick_random_substring(msg);
+    msg = maybe_capitalise_substring(msg);
 
     mprf(channel, "%s", msg.c_str());
 
@@ -455,12 +446,301 @@ void noisy_equipment()
     if (msg.empty())
         msg = getSpeakString("noisy weapon");
 
-    msg = maybe_pick_random_substring(msg);
-    msg = maybe_capitalise_substring(msg);
-
     item_noise(*weapon, msg, 20);
 }
 
+// Berserking monsters cannot be ordered around.
+static bool _follows_orders(monster* mon)
+{
+    return mon->friendly()
+           && mon->type != MONS_GIANT_SPORE
+           && !mon->berserk_or_insane()
+           && !mons_is_conjured(mon->type)
+           && !mon->has_ench(ENCH_HAUNTING);
+}
+
+// Sets foe target of friendly monsters.
+// If allow_patrol is true, patrolling monsters get MHITNOT instead.
+static void _set_friendly_foes(bool allow_patrol = false)
+{
+    for (monster_near_iterator mi(you.pos()); mi; ++mi)
+    {
+        if (!_follows_orders(*mi))
+            continue;
+        if (you.pet_target != MHITNOT && mi->behaviour == BEH_WITHDRAW)
+        {
+            mi->behaviour = BEH_SEEK;
+            mi->patrol_point = coord_def(0, 0);
+        }
+        mi->foe = (allow_patrol && mi->is_patrolling() ? MHITNOT
+                                                         : you.pet_target);
+    }
+}
+
+static void _set_allies_patrol_point(bool clear = false)
+{
+    for (monster_near_iterator mi(you.pos()); mi; ++mi)
+    {
+        if (!_follows_orders(*mi))
+            continue;
+        mi->patrol_point = (clear ? coord_def(0, 0) : mi->pos());
+        if (!clear)
+            mi->behaviour = BEH_WANDER;
+        else
+            mi->behaviour = BEH_SEEK;
+    }
+}
+
+static void _set_allies_withdraw(const coord_def &target)
+{
+    coord_def delta = target - you.pos();
+    float mult = float(LOS_RADIUS * 2) / (float)max(abs(delta.x), abs(delta.y));
+    coord_def rally_point = clamp_in_bounds(coord_def(delta.x * mult, delta.y * mult) + you.pos());
+
+    for (monster_near_iterator mi(you.pos()); mi; ++mi)
+    {
+        if (!_follows_orders(*mi))
+            continue;
+        if (mons_class_flag(mi->type, M_STATIONARY))
+            continue;
+        mi->behaviour = BEH_WITHDRAW;
+        mi->target = clamp_in_bounds(target);
+        mi->patrol_point = rally_point;
+        mi->foe = MHITNOT;
+
+        mi->props.erase("last_pos");
+        mi->props.erase("idle_point");
+        mi->props.erase("idle_deadline");
+        mi->props.erase("blocked_deadline");
+    }
+}
+
+void yell(const actor* mon)
+{
+    ASSERT(!crawl_state.game_is_arena());
+
+    bool targ_prev = false;
+    int mons_targd = MHITNOT;
+    dist targ;
+
+    const string shout_verb = you.shout_verb(mon != nullptr);
+    string cap_shout = shout_verb;
+    cap_shout[0] = toupper(cap_shout[0]);
+    const int noise_level = you.shout_volume();
+
+    if (you.cannot_speak())
+    {
+        if (mon)
+        {
+            if (you.paralysed() || you.duration[DUR_WATER_HOLD])
+            {
+                mprf("You feel a strong urge to %s, but "
+                     "you are unable to make a sound!",
+                     shout_verb.c_str());
+            }
+            else
+            {
+                mprf("You feel a %s rip itself from your throat, "
+                     "but you make no sound!",
+                     shout_verb.c_str());
+            }
+        }
+        else
+            mpr("You are unable to make a sound!");
+
+        return;
+    }
+
+    if (mon)
+    {
+        mprf("You %s%s at %s!",
+             shout_verb.c_str(),
+             you.duration[DUR_RECITE] ? " your recitation" : "",
+             mon->name(DESC_THE).c_str());
+        noisy(noise_level, you.pos());
+        return;
+    }
+
+    mprf(MSGCH_PROMPT, "What do you say?");
+    mprf(" t - %s!", cap_shout.c_str());
+
+    if (!you.berserk())
+    {
+        string previous;
+        if (!(you.prev_targ == MHITNOT || you.prev_targ == MHITYOU))
+        {
+            const monster* target = &menv[you.prev_targ];
+            if (target->alive() && you.can_see(*target))
+            {
+                previous = "   p - Attack previous target.";
+                targ_prev = true;
+            }
+        }
+
+        mprf("Orders for allies: a - Attack new target.%s", previous.c_str());
+        mpr("                   r - Retreat!             s - Stop attacking.");
+        mpr("                   w - Wait here.           f - Follow me.");
+    }
+    mpr(" Anything else - Stay silent.");
+
+    int keyn = get_ch();
+    clear_messages();
+
+    switch (keyn)
+    {
+    case '!':    // for players using the old keyset
+    case 't':
+        mprf(MSGCH_SOUND, "You %s%s!",
+             shout_verb.c_str(),
+             you.berserk() ? " wildly" : " for attention");
+        noisy(noise_level, you.pos());
+        zin_recite_interrupt();
+        you.turn_is_over = true;
+        return;
+
+    case 'f':
+    case 's':
+        if (you.berserk())
+        {
+            canned_msg(MSG_TOO_BERSERK);
+            return;
+        }
+
+        mons_targd = MHITYOU;
+        if (keyn == 'f')
+        {
+            // Don't reset patrol points for 'Stop fighting!'
+            _set_allies_patrol_point(true);
+            mpr("Follow me!");
+        }
+        else
+            mpr("Stop fighting!");
+        break;
+
+    case 'w':
+        if (you.berserk())
+        {
+            canned_msg(MSG_TOO_BERSERK);
+            return;
+        }
+
+        mpr("Wait here!");
+        mons_targd = MHITNOT;
+        _set_allies_patrol_point();
+        break;
+
+    case 'p':
+        if (you.berserk())
+        {
+            canned_msg(MSG_TOO_BERSERK);
+            return;
+        }
+
+        if (targ_prev)
+        {
+            mons_targd = you.prev_targ;
+            break;
+        }
+
+    // fall through
+    case 'a':
+        if (you.berserk())
+        {
+            canned_msg(MSG_TOO_BERSERK);
+            return;
+        }
+
+        if (env.sanctuary_time > 0)
+        {
+            if (!yesno("An ally attacking under your orders might violate "
+                       "sanctuary; order anyway?", false, 'n'))
+            {
+                canned_msg(MSG_OK);
+                return;
+            }
+        }
+
+        {
+            direction_chooser_args args;
+            args.restricts = DIR_TARGET;
+            args.mode = TARG_HOSTILE;
+            args.needs_path = false;
+            args.top_prompt = "Gang up on whom?";
+            direction(targ, args);
+        }
+
+        if (targ.isCancel)
+        {
+            canned_msg(MSG_OK);
+            return;
+        }
+
+        {
+            bool cancel = !targ.isValid;
+            if (!cancel)
+            {
+                const monster* m = monster_at(targ.target);
+                cancel = (m == nullptr || !you.can_see(*m));
+                if (!cancel)
+                    mons_targd = m->mindex();
+            }
+
+            if (cancel)
+            {
+                canned_msg(MSG_NOTHING_THERE);
+                return;
+            }
+        }
+        break;
+
+    case 'r':
+        if (you.berserk())
+        {
+            canned_msg(MSG_TOO_BERSERK);
+            return;
+        }
+
+        {
+            direction_chooser_args args;
+            args.restricts = DIR_TARGET;
+            args.mode = TARG_ANY;
+            args.needs_path = false;
+            args.top_prompt = "Retreat in which direction?";
+            direction(targ, args);
+        }
+
+        if (targ.isCancel)
+        {
+            canned_msg(MSG_OK);
+            return;
+        }
+
+        if (targ.isValid)
+        {
+            mpr("Fall back!");
+            mons_targd = MHITNOT;
+        }
+
+        _set_allies_withdraw(targ.target);
+        break;
+
+    default:
+        canned_msg(MSG_OK);
+        return;
+    }
+
+    zin_recite_interrupt();
+    you.turn_is_over = true;
+    you.pet_target = mons_targd;
+    // Allow patrolling for "Stop fighting!" and "Wait here!"
+    _set_friendly_foes(keyn == 's' || keyn == 'w');
+
+    if (mons_targd != MHITNOT && mons_targd != MHITYOU)
+        mpr("Attack!");
+
+    noisy(noise_level, you.pos());
+}
+
 void apply_noises()
 {
     // [ds] This copying isn't awesome, but we cannot otherwise handle
@@ -479,8 +759,8 @@ void apply_noises()
 // noisy() has a messaging service for giving messages to the player
 // as appropriate.
 bool noisy(int original_loudness, const coord_def& where,
-           const char *msg, int who,
-           bool mermaid, bool message_if_unseen, bool fake_noise)
+           const char *msg, mid_t who, noise_flag_type flags,
+           bool fake_noise)
 {
     ASSERT_IN_BOUNDS(where);
 
@@ -493,7 +773,7 @@ bool noisy(int original_loudness, const coord_def& where,
         ambient < 0? original_loudness + random2avg(abs(ambient), 3)
                    : original_loudness - random2avg(abs(ambient), 3);
 
-    dprf("Noise %d (orig: %d; ambient: %d) at pos(%d,%d)",
+    dprf(DIAG_NOISE, "Noise %d (orig: %d; ambient: %d) at pos(%d,%d)",
          loudness, original_loudness, ambient, where.x, where.y);
 
     if (loudness <= 0)
@@ -513,20 +793,15 @@ bool noisy(int original_loudness, const coord_def& where,
     // sound of loudness 1 will hear the sound.
     const string noise_msg(msg? msg : "");
     _noise_grid.register_noise(
-        noise_t(where,
-                noise_msg,
-                (scaled_loudness + 1) * 1000,
-                who,
-                0 | (mermaid ? NF_MERMAID : 0)
-                | (message_if_unseen ? NF_MESSAGE_IF_UNSEEN : 0)));
+        noise_t(where, noise_msg, (scaled_loudness + 1) * 1000, who, flags));
 
     // Some users of noisy() want an immediate answer to whether the
     // player heard the noise. The deferred noise system also means
     // that soft noises can be drowned out by loud noises. For both
     // these reasons, use the simple old noise system to check if the
     // player heard the noise:
-    const int dist = loudness * loudness + 1;
-    const int player_distance = distance2(you.pos(), where);
+    const int dist = loudness;
+    const int player_distance = grid_distance(you.pos(), where);
 
     // Message the player.
     if (player_distance <= dist && player_can_hear(where))
@@ -538,23 +813,23 @@ bool noisy(int original_loudness, const coord_def& where,
     return false;
 }
 
-bool noisy(int loudness, const coord_def& where, int who,
-           bool mermaid, bool message_if_unseen)
+bool noisy(int loudness, const coord_def& where, mid_t who,
+           noise_flag_type flags)
 {
-    return noisy(loudness, where, NULL, who, mermaid, message_if_unseen);
+    return noisy(loudness, where, nullptr, who, flags);
 }
 
 // This fakes noise even through silence.
 bool fake_noisy(int loudness, const coord_def& where)
 {
-    return noisy(loudness, where, NULL, -1, false, false, true);
+    return noisy(loudness, where, nullptr, MID_NOBODY, NF_NONE, true);
 }
 
 void check_monsters_sense(sense_type sense, int range, const coord_def& where)
 {
     for (monster_iterator mi; mi; ++mi)
     {
-        if (distance2(mi->pos(), where) > range)
+        if (grid_distance(mi->pos(), where) > range)
             continue;
 
         switch (sense)
@@ -574,7 +849,7 @@ void check_monsters_sense(sense_type sense, int range, const coord_def& where)
                 {
                     if (coinflip())
                     {
-                        dprf("disturbing %s (%d, %d)",
+                        dprf(DIAG_NOISE, "disturbing %s (%d, %d)",
                              mi->name(DESC_PLAIN).c_str(),
                              mi->pos().x, mi->pos().y);
                         behaviour_event(*mi, ME_DISTURB, 0, where);
@@ -582,29 +857,28 @@ void check_monsters_sense(sense_type sense, int range, const coord_def& where)
                     break;
                 }
             }
-            dprf("alerting %s (%d, %d)",
+            dprf(DIAG_NOISE, "alerting %s (%d, %d)",
                             mi->name(DESC_PLAIN).c_str(),
                             mi->pos().x, mi->pos().y);
             behaviour_event(*mi, ME_ALERT, 0, where);
+            break;
 
         case SENSE_WEB_VIBRATION:
-            if (!mons_class_flag(mi->type, M_WEB_SENSE)
-                && !mons_class_flag(get_chimera_legs(*mi), M_WEB_SENSE))
-            {
+            if (!mons_class_flag(mi->type, M_WEB_SENSE))
                 break;
-            }
+
             if (!one_chance_in(4))
             {
                 if (coinflip())
                 {
-                    dprf("disturbing %s (%d, %d)",
+                    dprf(DIAG_NOISE, "disturbing %s (%d, %d)",
                          mi->name(DESC_PLAIN).c_str(),
                          mi->pos().x, mi->pos().y);
                     behaviour_event(*mi, ME_DISTURB, 0, where);
                 }
                 else
                 {
-                    dprf("alerting %s (%d, %d)",
+                    dprf(DIAG_NOISE, "alerting %s (%d, %d)",
                          mi->name(DESC_PLAIN).c_str(),
                          mi->pos().x, mi->pos().y);
                     behaviour_event(*mi, ME_ALERT, 0, where);
@@ -617,7 +891,7 @@ void check_monsters_sense(sense_type sense, int range, const coord_def& where)
 
 void blood_smell(int strength, const coord_def& where)
 {
-    const int range = strength * strength;
+    const int range = strength;
     dprf("blood stain at (%d, %d), range of smell = %d",
          where.x, where.y, range);
 
@@ -632,24 +906,16 @@ void blood_smell(int strength, const coord_def& where)
 static int _noise_attenuation_millis(const coord_def &pos)
 {
     const dungeon_feature_type feat = grd(pos);
-    switch (feat)
-    {
-    // Closed doors are excellent at cutting off sound.
-    case DNGN_CLOSED_DOOR:
-    case DNGN_RUNED_DOOR:
-    case DNGN_SEALED_DOOR:
-        return BASE_NOISE_ATTENUATION_MILLIS * 8;
-    case DNGN_TREE:
-        return BASE_NOISE_ATTENUATION_MILLIS * 3;
-    default:
-        if (feat_is_statue_or_idol(feat))
-            return BASE_NOISE_ATTENUATION_MILLIS * 2;
-        if (feat_is_permarock(feat))
-            return NOISE_ATTENUATION_COMPLETE;
-        if (feat_is_wall(feat))
-            return BASE_NOISE_ATTENUATION_MILLIS * 12;
-        return BASE_NOISE_ATTENUATION_MILLIS;
-    }
+
+    if (feat_is_permarock(feat))
+        return NOISE_ATTENUATION_COMPLETE;
+
+    return BASE_NOISE_ATTENUATION_MILLIS *
+            (feat_is_wall(feat)        ? 12 :
+             feat_is_closed_door(feat) ?  8 :
+             feat_is_tree(feat)        ?  3 :
+             feat_is_statuelike(feat)  ?  2 :
+                                          1);
 }
 
 noise_cell::noise_cell()
@@ -729,13 +995,14 @@ void noise_grid::propagate_noise()
         return;
 
 #ifdef DEBUG_NOISE_PROPAGATION
-    dprf("noise_grid: %u noises to apply", (unsigned int)noises.size());
+    dprf(DIAG_NOISE, "noise_grid: %u noises to apply",
+         (unsigned int)noises.size());
 #endif
     vector<coord_def> noise_perimeter[2];
     int circ_index = 0;
 
-    for (int i = 0, size = noises.size(); i < size; ++i)
-        noise_perimeter[circ_index].push_back(noises[i].noise_source);
+    for (const noise_t &noise : noises)
+        noise_perimeter[circ_index].push_back(noise.noise_source);
 
     int travel_distance = 0;
     while (!noise_perimeter[circ_index].empty())
@@ -743,9 +1010,8 @@ void noise_grid::propagate_noise()
         const vector<coord_def> &perimeter(noise_perimeter[circ_index]);
         vector<coord_def> &next_perimeter(noise_perimeter[!circ_index]);
         ++travel_distance;
-        for (int i = 0, size = perimeter.size(); i < size; ++i)
+        for (const coord_def p : perimeter)
         {
-            const coord_def p(perimeter[i]);
             const noise_cell &cell(cells(p));
 
             if (!cell.silent())
@@ -815,10 +1081,6 @@ bool noise_grid::propagate_noise_to_neighbour(int base_attenuation,
         return false;
     }
 
-    // Diagonals cost more.
-    if ((next_pos - current_pos).abs() == 2)
-        base_attenuation = base_attenuation * 141 / 100;
-
     const int noise_turn_angle = cell.turn_angle(next_pos - current_pos);
     const int turn_attenuation =
         noise_turn_angle? (base_attenuation * (100 + noise_turn_angle * 25)
@@ -857,7 +1119,7 @@ void noise_grid::apply_noise_effects(const coord_def &pos,
     {
         if (mons->alive()
             && !mons_just_slept(mons)
-            && mons->mindex() != noise.noise_producer_id)
+            && mons->mid != noise.noise_producer_mid)
         {
             const coord_def perceived_position =
                 noise_perceived_position(mons, pos, noise);
@@ -936,8 +1198,8 @@ coord_def noise_grid::noise_perceived_position(actor *act,
         : perceived_point;
 
 #ifdef DEBUG_NOISE_PROPAGATION
-    dprf("[NOISE] Noise perceived by %s at (%d,%d) centroid (%d,%d) "
-         "source (%d,%d) "
+    dprf(DIAG_NOISE, "[NOISE] Noise perceived by %s at (%d,%d) "
+         "centroid (%d,%d) source (%d,%d) "
          "heard at (%d,%d), distance: %d (traveled %d)",
          act->name(DESC_PLAIN, true).c_str(),
          final_perceived_point.x, final_perceived_point.y,
@@ -951,9 +1213,7 @@ coord_def noise_grid::noise_perceived_position(actor *act,
 
 #ifdef DEBUG_NOISE_PROPAGATION
 
-#include "options.h"
-#include "viewchar.h"
-#include <math.h>
+#include <cmath>
 
 // Return HTML RGB triple given a hue and assuming chroma of 0.86 (220)
 static string _hue_rgb(int hue)
@@ -1035,7 +1295,7 @@ void noise_grid::write_noise_grid(FILE *outf) const
             if (you.pos() == coord_def(x, y))
                 write_cell(outf, p, '@');
             else
-                write_cell(outf, p, get_feature_def(grd[x][y]).symbol);
+                write_cell(outf, p, get_feature_def(grd[x][y]).symbol());
         }
         fprintf(outf, "<br>\n");
     }
@@ -1061,7 +1321,7 @@ static void _actor_apply_noise(actor *act,
                                int noise_travel_distance)
 {
 #ifdef DEBUG_NOISE_PROPAGATION
-    dprf("[NOISE] Actor %s (%d,%d) perceives noise (%d) "
+    dprf(DIAG_NOISE, "[NOISE] Actor %s (%d,%d) perceives noise (%d) "
          "from (%d,%d), real source (%d,%d), distance: %d, noise traveled: %d",
          act->name(DESC_PLAIN, true).c_str(),
          act->pos().x, act->pos().y,
@@ -1077,7 +1337,7 @@ static void _actor_apply_noise(actor *act,
     {
         const int loudness = div_rand_round(noise_intensity_millis, 1000);
         act->check_awaken(loudness);
-        if (!(noise.noise_flags & NF_MERMAID))
+        if (!(noise.noise_flags & NF_SIREN))
         {
             you.beholders_check_noise(loudness, player_equip_unrand(UNRAND_DEMON_AXE));
             you.fearmongers_check_noise(loudness, player_equip_unrand(UNRAND_DEMON_AXE));
@@ -1086,15 +1346,26 @@ static void _actor_apply_noise(actor *act,
     else
     {
         monster *mons = act->as_monster();
+        actor *source = actor_by_mid(noise.noise_producer_mid);
         // If the noise came from the character, any nearby monster
         // will be jumping on top of them.
         if (grid_distance(apparent_source, you.pos()) <= 3)
             behaviour_event(mons, ME_ALERT, &you, apparent_source);
-        else if ((noise.noise_flags & NF_MERMAID)
+        else if ((noise.noise_flags & NF_SIREN)
                  && mons_secondary_habitat(mons) == HT_WATER
                  && !mons->friendly())
         {
-            // Mermaids/sirens call (hostile) aquatic monsters.
+            // Sirens/merfolk avatar call (hostile) aquatic monsters.
+            behaviour_event(mons, ME_ALERT, 0, apparent_source);
+        }
+        else if ((noise.noise_flags & NF_HUNTING_CRY)
+                 && source
+                 && (mons_genus(mons->type) == mons_genus(source->type)
+                     || mons->holiness() == MH_HOLY
+                        && source->holiness() == MH_HOLY))
+        {
+            // Hunting cries alert monsters of the same genus, or other
+            // holy creatures if the source is holy.
             behaviour_event(mons, ME_ALERT, 0, apparent_source);
         }
         else
