@@ -13,26 +13,17 @@
 #include "coordit.h"
 #include "dgnevent.h"
 #include "dgn-overview.h"
-#include "directn.h"
 #include "dungeon.h"
-#include "env.h"
-#include "exclude.h"
-#include "fprop.h"
 #include "itemprop.h"
 #include "libutil.h"
 #include "mon-place.h"
-#include "mon-util.h"
-#include "monster.h"
 #include "options.h"
-#include "random.h"
-#include "stash.h"
 #include "state.h"
-#include "areas.h"
 #include "terrain.h"
+#include "tiledef-main.h"
 #ifdef USE_TILE
  #include "tileview.h"
 #endif
-#include "tiledef-main.h"
 #include "traps.h"
 #include "travel.h"
 #include "viewgeom.h"
@@ -153,9 +144,6 @@ static void _update_feat_at(const coord_def &gp)
         env.map_knowledge(gp).flags |= MAP_HOT;
 #endif
 
-    if (golden(gp))
-        env.map_knowledge(gp).flags |= MAP_GOLDEN;
-
     if (is_sanctuary(gp))
     {
         if (testbits(env.pgrid(gp), FPROP_SANCTUARY_1))
@@ -173,8 +161,8 @@ static void _update_feat_at(const coord_def &gp)
     if (you.made_nervous_by(gp))
         env.map_knowledge(gp).flags |= MAP_WITHHELD;
 
-    if (feat >= DNGN_STONE_STAIRS_DOWN_I
-        && feat <= DNGN_ESCAPE_HATCH_UP
+    if ((feat_is_stone_stair(feat)
+         || feat_is_escape_hatch(feat))
         && is_exclude_root(gp))
     {
         env.map_knowledge(gp).flags |= MAP_EXCLUDED_STAIRS;
@@ -220,16 +208,17 @@ static show_item_type _item_to_show_code(const item_def &item)
         return jewellery_is_amulet(item) ? SHOW_ITEM_AMULET : SHOW_ITEM_RING;
     case OBJ_POTIONS:    return SHOW_ITEM_POTION;
     case OBJ_BOOKS:      return SHOW_ITEM_BOOK;
-    case OBJ_STAVES:     return SHOW_ITEM_STAVE;
-    case OBJ_RODS:       return SHOW_ITEM_STAVE;
-    case OBJ_MISCELLANY:
-        if (item.sub_type == MISC_RUNE_OF_ZOT)
-            return SHOW_ITEM_RUNE;
+    case OBJ_STAVES:     return SHOW_ITEM_STAFF;
+    case OBJ_RODS:       return SHOW_ITEM_ROD;
+    case OBJ_MISCELLANY: return SHOW_ITEM_MISCELLANY;
+    case OBJ_CORPSES:
+        if (item.sub_type == CORPSE_SKELETON)
+            return SHOW_ITEM_SKELETON;
         else
-            return SHOW_ITEM_MISCELLANY;
-    case OBJ_CORPSES:    return SHOW_ITEM_CORPSE;
+            return SHOW_ITEM_CORPSE;
     case OBJ_GOLD:       return SHOW_ITEM_GOLD;
     case OBJ_DETECTED:   return SHOW_ITEM_DETECTED;
+    case OBJ_RUNES:      return SHOW_ITEM_RUNE;
     default:             return SHOW_ITEM_ORB; // bad item character
     }
 }
@@ -311,7 +300,7 @@ static void _check_monster_pos(const monster* mons)
 
     // [rob] The following in case asserts aren't enabled.
     // [enne] - It's possible that mgrd and mons->x/y are out of
-    // sync because they are updated separately.  If we can see this
+    // sync because they are updated separately. If we can see this
     // monster, then make sure that the mgrd is set correctly.
     if (mgrd(mons->pos()) != s)
     {
@@ -319,7 +308,7 @@ static void _check_monster_pos(const monster* mons)
         // circumstances so we can track down where this is coming
         // from.
         mprf(MSGCH_ERROR, "monster %s (%d) at (%d, %d) was "
-             "improperly placed.  Updating mgrd.",
+             "improperly placed. Updating mgrd.",
              mons->name(DESC_PLAIN, true).c_str(), s,
              mons->pos().x, mons->pos().y);
         mgrd(mons->pos()) = s;
@@ -373,7 +362,7 @@ static int _hashed_rand(const monster* mons, uint32_t id, uint32_t die)
  * Mark the estimated position of an invisible monster.
  *
  * Marks a spot on the map as possibly containing an unseen monster
- * (showing up as a disturbance in the air).  Also flags the square as
+ * (showing up as a disturbance in the air). Also flags the square as
  * updated for invisible monster, which is used by show_init().
  *
  * @param where          The disturbance's map position.
@@ -390,30 +379,31 @@ static void _mark_invisible_at(const coord_def &where,
 }
 
 /**
- * Mark monsters that have transitioned from seen to unseen in the last turn get
- * an invisible monster indicator.
+ * Mark invisible monsters with a known position with an invisible monster
+ * indicator.
  * @param mons      The monster to check.
  * @param hash_ind  The random hash index, combined with the mid to make a
- *                  unique hash for this roll.  Needed for when we can't mark
+ *                  unique hash for this roll. Needed for when we can't mark
  *                  the monster's true position and instead mark an adjacent
  *                  one.
 */
 static void _handle_unseen_mons(monster* mons, uint32_t hash_ind)
 {
-    // Monster didn't go unseen last turn.
+    // Monster position is unknown.
     if (mons->unseen_pos.origin())
         return;
 
-
-    // We expire these unseen invis markers after one turn.
-    if (you.turn_is_over && !mons->went_unseen_this_turn)
+    // We expire these unseen invis markers after one turn if the monster
+    // has moved away.
+    if (you.turn_is_over && !mons->went_unseen_this_turn
+        && mons->pos() != mons->unseen_pos)
     {
         mons->unseen_pos = coord_def(0, 0);
         return;
     }
 
     bool do_tiles_draw;
-    // Try to use the original position where the monster became unseen.
+    // Try to use the unseen position.
     if (_valid_invisible_spot(mons->unseen_pos, mons))
     {
         do_tiles_draw = mons->unseen_pos != mons->pos();
@@ -422,6 +412,7 @@ static void _handle_unseen_mons(monster* mons, uint32_t hash_ind)
     }
 
     // Fall back to a random position adjacent to the unseen position.
+    // This can only happen if the monster just became unseen.
     vector <coord_def> adj_unseen;
     for (adjacent_iterator ai(mons->unseen_pos, false); ai; ai++)
     {
@@ -462,27 +453,29 @@ static void _update_monster(monster* mons)
     // invisible monster indicator.
 
     // We cannot use regular randomness here, otherwise redrawing the screen
-    // would give out the real position.  We need to save the seed too -- but it
+    // would give out the real position. We need to save the seed too -- but it
     // needs to be regenerated every turn.
     if (you.attribute[ATTR_SEEN_INVIS_TURN] != you.num_turns)
     {
         you.attribute[ATTR_SEEN_INVIS_TURN] = you.num_turns;
-        you.attribute[ATTR_SEEN_INVIS_SEED] = random_int();
+        you.attribute[ATTR_SEEN_INVIS_SEED] = get_uint32();
     }
     // After the player finishes this turn, the monster's unseen pos (and
     // its invis indicator due to going unseen) will be erased.
     if (!you.turn_is_over)
         mons->went_unseen_this_turn = false;
 
-    // ripple effect?
+    // Ripple effect?
+    // Should match directn.cc's _mon_exposed
     if (grd(gp) == DNGN_SHALLOW_WATER
-        && !mons_flies(mons)
-        && env.cgrid(gp) == EMPTY_CLOUD
+            && !mons->airborne()
+            && env.cgrid(gp) == EMPTY_CLOUD
         || is_opaque_cloud(env.cgrid(gp))
-        && !mons->submerged()
-        && !mons->is_insubstantial())
+            && !mons->submerged()
+            && !mons->is_insubstantial())
     {
         _mark_invisible_at(gp);
+        mons->unseen_pos = gp;
         return;
     }
 
@@ -490,11 +483,12 @@ static void _update_monster(monster* mons)
     if (mons->submerged())
         return;
 
-    int range2 = dist_range(player_monster_detect_radius());
+    int range = player_monster_detect_radius();
     if (mons->constricted_by == MID_PLAYER
-        || (range2 > 1 && (you.pos() - mons->pos()).abs() <= range2))
+        || (range > 0 && (you.pos() - mons->pos()).rdist() <= range))
     {
         _mark_invisible_at(gp);
+        mons->unseen_pos = gp;
         return;
     }
 
@@ -505,6 +499,7 @@ static void _update_monster(monster* mons)
         && (mons->stealth() <= -2 || !_hashed_rand(mons, 1, 4)))
     {
         _mark_invisible_at(gp);
+        mons->unseen_pos = gp;
     }
     else
         _handle_unseen_mons(mons, 2);
@@ -512,10 +507,10 @@ static void _update_monster(monster* mons)
 
 /**
  * Update map knowledge and set the map tiles at a location.
- * @param gp            The location to update.
- * @param terrain_only  If True, only the feature information/tiles are updated.
+ * @param gp      The location to update.
+ * @param layers  The information layers to display.
 **/
-void show_update_at(const coord_def &gp, bool terrain_only)
+void show_update_at(const coord_def &gp, layers_type layers)
 {
     if (you.see_cell(gp))
         env.map_knowledge(gp).clear_data();
@@ -523,28 +518,34 @@ void show_update_at(const coord_def &gp, bool terrain_only)
         return;
     else
         env.map_knowledge(gp).clear_monster();
-
     // The sequence is grid, items, clouds, monsters.
     _update_feat_at(gp);
 
     // If there's items on the boundary (shop inventory),
     // we don't show them.
-    if (!terrain_only && in_bounds(gp))
+    if (in_bounds(gp))
     {
-        monster* mons = monster_at(gp);
-        if (mons && mons->alive())
-            _update_monster(mons);
-        else if (env.map_knowledge(gp).flags & MAP_INVISIBLE_UPDATE)
-            _mark_invisible_at(gp);
-
-        const int cloud = env.cgrid(gp);
-        if (cloud != EMPTY_CLOUD && env.cloud[cloud].type != CLOUD_NONE
-            && env.cloud[cloud].pos == gp)
+        if (layers & LAYER_MONSTERS)
         {
-            _update_cloud(cloud);
+            monster* mons = monster_at(gp);
+            if (mons && mons->alive())
+                _update_monster(mons);
+            else if (env.map_knowledge(gp).flags & MAP_INVISIBLE_UPDATE)
+                _mark_invisible_at(gp);
         }
 
-        update_item_at(gp);
+        if (layers & LAYER_CLOUDS)
+        {
+            const int cloud = env.cgrid(gp);
+            if (cloud != EMPTY_CLOUD && env.cloud[cloud].type != CLOUD_NONE
+                && env.cloud[cloud].pos == gp)
+            {
+                _update_cloud(cloud);
+            }
+        }
+
+        if (layers & LAYER_ITEMS)
+            update_item_at(gp);
     }
 
 #ifdef USE_TILE
@@ -555,14 +556,14 @@ void show_update_at(const coord_def &gp, bool terrain_only)
 #endif
 }
 
-void show_init(bool terrain_only)
+void show_init(layers_type layers)
 {
     clear_terrain_visibility();
     if (crawl_state.game_is_arena())
     {
         for (rectangle_iterator ri(crawl_view.vgrdc, LOS_MAX_RANGE); ri; ++ri)
         {
-            show_update_at(*ri, terrain_only);
+            show_update_at(*ri, layers);
             // Invis indicators and update flags not used in Arena.
             env.map_knowledge(*ri).flags &= ~MAP_INVISIBLE_UPDATE;
         }
@@ -572,13 +573,13 @@ void show_init(bool terrain_only)
     vector <coord_def> update_locs;
     for (radius_iterator ri(you.pos(), you.xray_vision ? LOS_NONE : LOS_DEFAULT); ri; ++ri)
     {
-        show_update_at(*ri, terrain_only);
+        show_update_at(*ri, layers);
         update_locs.push_back(*ri);
     }
 
     // Need to clear these update flags now so they don't persist.
-    for (unsigned int i = 0; i < update_locs.size(); i++)
-        env.map_knowledge(update_locs[i]).flags &= ~MAP_INVISIBLE_UPDATE;
+    for (coord_def loc : update_locs)
+        env.map_knowledge(loc).flags &= ~MAP_INVISIBLE_UPDATE;
 }
 
 // Emphasis may change while off-level (precisely, after
@@ -593,7 +594,7 @@ void show_update_emphasis()
     // stairs are now known. (see is_unknown_stair(), emphasise())
     LevelInfo& level_info = travel_cache.get_level_info(level_id::current());
     vector<stair_info> stairs = level_info.get_stairs();
-    for (unsigned i = 0; i < stairs.size(); ++i)
-        if (stairs[i].destination.is_valid())
-            env.map_knowledge(stairs[i].position).flags &= ~MAP_EMPHASIZE;
+    for (const stair_info &stair : stairs)
+        if (stair.destination.is_valid())
+            env.map_knowledge(stair.position).flags &= ~MAP_EMPHASIZE;
 }
