@@ -1,6 +1,8 @@
 #ifndef SHOW_H
 #define SHOW_H
 
+#include "enum.h"
+
 // same order as DCHAR_*
 enum show_item_type
 {
@@ -17,9 +19,11 @@ enum show_item_type
     SHOW_ITEM_POTION,
     SHOW_ITEM_MISSILE,
     SHOW_ITEM_BOOK,
-    SHOW_ITEM_STAVE,
+    SHOW_ITEM_STAFF,
+    SHOW_ITEM_ROD,
     SHOW_ITEM_MISCELLANY,
     SHOW_ITEM_CORPSE,
+    SHOW_ITEM_SKELETON,
     SHOW_ITEM_GOLD,
     SHOW_ITEM_AMULET,
     NUM_SHOW_ITEMS
@@ -42,7 +46,7 @@ struct show_type
     dungeon_feature_type feat;
     show_item_type item;
     monster_type mons;
-    unsigned short colour;
+    colour_t colour;
 
     show_type();
     show_type(dungeon_feature_type f);
@@ -65,9 +69,21 @@ struct show_info
 
 class monster;
 
-void show_init(bool terrain_only = false);
+enum layer_type
+{
+    LAYERS_NONE    = 0,
+    LAYER_MONSTERS = (1 << 0),
+    LAYER_PLAYER   = (1 << 1),
+    LAYER_ITEMS    = (1 << 2),
+    LAYER_CLOUDS   = (1 << 3),
+};
+DEF_BITFIELD(layers_type, layer_type, 3);
+static const layers_type LAYERS_ALL = LAYER_MONSTERS | LAYER_PLAYER
+                                    | LAYER_ITEMS | LAYER_CLOUDS;
+
+void show_init(layers_type layers = LAYERS_ALL);
 void update_item_at(const coord_def &gp, bool detected = false);
-void show_update_at(const coord_def &gp, bool terrain_only = false);
+void show_update_at(const coord_def &gp, layers_type layers = LAYERS_ALL);
 void show_update_emphasis();
 
 #endif
