@@ -6,52 +6,48 @@
 #include "AppHdr.h"
 
 #include "spl-clouds.h"
-#include "externs.h"
 
 #include <algorithm>
 
-#include "act-iter.h"
-#include "beam.h"
 #include "butcher.h"
 #include "cloud.h"
-#include "coord.h"
 #include "coordit.h"
+#include "english.h"
 #include "env.h"
 #include "fprop.h"
 #include "godconduct.h"
 #include "items.h"
-#include "libutil.h"
 #include "losglobal.h"
 #include "message.h"
 #include "misc.h"
-#include "mon-behv.h"
-#include "mon-util.h"
 #include "ouch.h"
-#include "player.h"
 #include "prompt.h"
 #include "random-pick.h"
+#include "shout.h"
 #include "spl-util.h"
 #include "target.h"
 #include "terrain.h"
-#include "transform.h"
 #include "viewchar.h"
-#include "shout.h"
 
-spret_type conjure_flame(int pow, const coord_def& where, bool fail)
+spret_type conjure_flame(const actor *agent, int pow, const coord_def& where,
+                         bool fail)
 {
     // FIXME: This would be better handled by a flag to enforce max range.
-    if (distance2(where, you.pos()) > dist_range(spell_range(SPELL_CONJURE_FLAME,
-                                                      pow))
+    if (grid_distance(where, agent->pos()) > spell_range(SPELL_CONJURE_FLAME, pow)
         || !in_bounds(where))
     {
-        mpr("That's too far away.");
+        if (agent->is_player())
+            mpr("That's too far away.");
         return SPRET_ABORT;
     }
 
     if (cell_is_solid(where))
     {
-        const char *feat = feat_type_name(grd(where));
-        mprf("You can't place the cloud on %s.", article_a(feat).c_str());
+        if (agent->is_player())
+        {
+            const char *feat = feat_type_name(grd(where));
+            mprf("You can't place the cloud on %s.", article_a(feat).c_str());
+        }
         return SPRET_ABORT;
     }
 
@@ -59,96 +55,69 @@ spret_type conjure_flame(int pow, const coord_def& where, bool fail)
 
     if (cloud != EMPTY_CLOUD && env.cloud[cloud].type != CLOUD_FIRE)
     {
-        mpr("There's already a cloud there!");
+        if (agent->is_player())
+            mpr("There's already a cloud there!");
         return SPRET_ABORT;
     }
 
-    // Note that self-targeting is handled by SPFLAG_NOT_SELF.
-    monster* mons = monster_at(where);
-    if (mons)
+    actor* victim = actor_at(where);
+    if (victim)
     {
-        if (you.can_see(mons))
+        if (agent->can_see(*victim))
         {
-            mpr("You can't place the cloud on a creature.");
+            if (agent->is_player())
+                mpr("You can't place the cloud on a creature.");
             return SPRET_ABORT;
         }
 
         fail_check();
 
         // FIXME: maybe should do _paranoid_option_disable() here?
-        mpr("You see a ghostly outline there, and the spell fizzles.");
+        if (agent->is_player())
+            canned_msg(MSG_GHOSTLY_OUTLINE);
         return SPRET_SUCCESS;      // Don't give free detection!
     }
 
     fail_check();
 
-    did_god_conduct(DID_FIRE, min(5 + pow/2, 23));
+    if (agent->is_player())
+        did_god_conduct(DID_FIRE, min(5 + pow/2, 23));
 
     if (cloud != EMPTY_CLOUD)
     {
         // Reinforce the cloud - but not too much.
         // It must be a fire cloud from a previous test.
-        mpr("The fire roars with new energy!");
+        if (you.see_cell(where))
+            mpr("The fire roars with new energy!");
         const int extra_dur = 2 + min(random2(pow) / 2, 20);
         env.cloud[cloud].decay += extra_dur * 5;
-        env.cloud[cloud].set_whose(KC_YOU);
+        env.cloud[cloud].source = agent->mid;
+        if (agent->is_player())
+            env.cloud[cloud].set_whose(KC_YOU);
+        else
+            env.cloud[cloud].set_killer(KILL_MON_MISSILE);
     }
     else
     {
         const int durat = min(5 + (random2(pow)/2) + (random2(pow)/2), 23);
-        place_cloud(CLOUD_FIRE, where, durat, &you);
-        mpr("The fire roars!");
+        place_cloud(CLOUD_FIRE, where, durat, agent);
+        if (you.see_cell(where))
+        {
+            if (agent->is_player())
+                mpr("The fire roars!");
+            else
+                mpr("A cloud of flames roars to life!");
+        }
     }
-    noisy(2, where);
+    noisy(spell_effect_noise(SPELL_CONJURE_FLAME), where);
 
     return SPRET_SUCCESS;
 }
 
-// Assumes beem.range has already been set. -cao
-spret_type stinking_cloud(int pow, bolt &beem, bool fail)
-{
-    beem.name        = "stinking cloud";
-    beem.colour      = GREEN;
-    beem.damage      = dice_def(1, 0);
-    beem.hit         = 20;
-    beem.glyph       = dchar_glyph(DCHAR_FIRED_ZAP);
-    beem.flavour     = BEAM_MEPHITIC;
-    beem.ench_power  = pow;
-    beem.beam_source = MHITYOU;
-    beem.thrower     = KILL_YOU;
-    beem.is_beam     = false;
-    beem.is_explosion = true;
-    beem.aux_source.clear();
-
-    // Fire tracer.
-    beem.source            = you.pos();
-    beem.can_see_invis     = you.can_see_invisible();
-    beem.smart_monster     = true;
-    beem.attitude          = ATT_FRIENDLY;
-    beem.friend_info.count = 0;
-    beem.is_tracer         = true;
-    beem.fire();
-
-    if (beem.beam_cancelled)
-    {
-        // We don't want to fire through friendlies.
-        canned_msg(MSG_OK);
-        return SPRET_ABORT;
-    }
-
-    fail_check();
-
-    // Really fire.
-    beem.is_tracer = false;
-    beem.fire();
-
-    return SPRET_SUCCESS;
-}
-
-spret_type cast_big_c(int pow, cloud_type cty, const actor *caster, bolt &beam,
+spret_type cast_big_c(int pow, spell_type spl, const actor *caster, bolt &beam,
                       bool fail)
 {
-    if (distance2(beam.target, you.pos()) > dist_range(beam.range)
+    if (grid_distance(beam.target, you.pos()) > beam.range
         || !in_bounds(beam.target))
     {
         mpr("That is beyond the maximum range.");
@@ -162,20 +131,24 @@ spret_type cast_big_c(int pow, cloud_type cty, const actor *caster, bolt &beam,
         return SPRET_ABORT;
     }
 
+    cloud_type cty = CLOUD_NONE;
     //XXX: there should be a better way to specify beam cloud types
-    switch (cty)
+    switch (spl)
     {
-        case CLOUD_POISON:
+        case SPELL_POISONOUS_CLOUD:
             beam.flavour = BEAM_POISON;
             beam.name = "blast of poison";
+            cty = CLOUD_POISON;
             break;
-        case CLOUD_HOLY_FLAMES:
+        case SPELL_HOLY_BREATH:
             beam.flavour = BEAM_HOLY;
             beam.origin_spell = SPELL_HOLY_BREATH;
+            cty = CLOUD_HOLY_FLAMES;
             break;
-        case CLOUD_COLD:
+        case SPELL_FREEZING_CLOUD:
             beam.flavour = BEAM_COLD;
             beam.name = "freezing blast";
+            cty = CLOUD_COLD;
             break;
         default:
             mpr("That kind of cloud doesn't exist!");
@@ -185,9 +158,9 @@ spret_type cast_big_c(int pow, cloud_type cty, const actor *caster, bolt &beam,
     beam.thrower           = KILL_YOU;
     beam.hit               = AUTOMATIC_HIT;
     beam.damage            = dice_def(42, 1); // just a convenient non-zero
-    beam.is_big_cloud      = true;
     beam.is_tracer         = true;
     beam.use_target_as_pos = true;
+    beam.origin_spell      = spl;
     beam.affect_endpoint();
     if (beam.beam_cancelled)
         return SPRET_ABORT;
@@ -195,7 +168,7 @@ spret_type cast_big_c(int pow, cloud_type cty, const actor *caster, bolt &beam,
     fail_check();
 
     big_cloud(cty, caster, beam.target, pow, 8 + random2(3), -1);
-    noisy(2, beam.target);
+    noisy(spell_effect_noise(spl), beam.target);
     return SPRET_SUCCESS;
 }
 
@@ -274,7 +247,7 @@ spret_type cast_corpse_rot(bool fail)
     {
         for (stack_iterator si(you.pos()); si; ++si)
         {
-            if (si->base_type == OBJ_CORPSES && si->sub_type == CORPSE_BODY)
+            if (si->is_type(OBJ_CORPSES, CORPSE_BODY))
             {
                 if (!yesno(("Really cast Corpse Rot while standing on " + si->name(DESC_A) + "?").c_str(), false, 'n'))
                 {
@@ -294,15 +267,15 @@ void corpse_rot(actor* caster)
 {
     // If there is no caster (god wrath), centre the effect on the player.
     const coord_def center = caster ? caster->pos() : you.pos();
-    bool saw_rot = caster && (caster->is_player() || you.can_see(caster));
+    bool saw_rot = false;
 
-    for (radius_iterator ri(center, 6, C_ROUND, LOS_NO_TRANS); ri; ++ri)
+    for (radius_iterator ri(center, LOS_NO_TRANS); ri; ++ri)
     {
         if (!is_sanctuary(*ri) && env.cgrid(*ri) == EMPTY_CLOUD)
             for (stack_iterator si(*ri); si; ++si)
-                if (si->base_type == OBJ_CORPSES && si->sub_type == CORPSE_BODY)
+                if (si->is_type(OBJ_CORPSES, CORPSE_BODY))
                 {
-                    // Found a corpse.  Skeletonise it if possible.
+                    // Found a corpse. Skeletonise it if possible.
                     if (!mons_skeleton(si->mon_type))
                     {
                         item_was_destroyed(*si);
@@ -323,11 +296,11 @@ void corpse_rot(actor* caster)
 
     if (saw_rot)
         mprf("You %s decay.", you.can_smell() ? "smell" : "sense");
-
-    // Should make zombies decay into skeletons?
+    else
+        canned_msg(MSG_NOTHING_HAPPENS);
 }
 
-int holy_flames(monster* caster, actor* defender)
+void holy_flames(monster* caster, actor* defender)
 {
     const coord_def pos = defender->pos();
     int cloud_count = 0;
@@ -349,17 +322,15 @@ int holy_flames(monster* caster, actor* defender)
         cloud_count++;
     }
 
-    return cloud_count;
-}
-
-struct dist2_sorter
-{
-    coord_def pos;
-    bool operator()(const actor* a, const actor* b)
+    if (cloud_count)
     {
-        return distance2(a->pos(), pos) > distance2(b->pos(), pos);
+        if (defender->is_player())
+            mpr("Blessed fire suddenly surrounds you!");
+        else
+            simple_monster_message(defender->as_monster(),
+                                   " is surrounded by blessed fire!");
     }
-};
+}
 
 static bool _safe_cloud_spot(const monster* mon, coord_def p)
 {
@@ -388,7 +359,7 @@ void apply_control_winds(const monster* mon)
 
     bolt wind_beam;
     wind_beam.hit = AUTOMATIC_HIT;
-    wind_beam.is_beam = true;
+    wind_beam.pierce  = true;
     wind_beam.affects_nothing = true;
     wind_beam.source = mon->pos();
     wind_beam.range = LOS_RADIUS;
@@ -450,15 +421,15 @@ void apply_control_winds(const monster* mon)
 
 random_pick_entry<cloud_type> cloud_cone_clouds[] =
 {
-  { 0,   50,  80, DOWN, CLOUD_RAIN },
-  { 0,   50, 100, DOWN, CLOUD_MIST },
-  { 0,   50, 150, DOWN, CLOUD_MEPHITIC },
+  { 0,   50,  80, FALL, CLOUD_RAIN },
+  { 0,   50, 100, FALL, CLOUD_MIST },
+  { 0,   50, 150, FALL, CLOUD_MEPHITIC },
   { 0,  100, 100, PEAK, CLOUD_FIRE },
   { 0,  100, 100, PEAK, CLOUD_COLD },
   { 0,  100, 100, PEAK, CLOUD_POISON },
-  { 30, 100, 125, UP,   CLOUD_NEGATIVE_ENERGY },
-  { 40, 100, 135, UP,   CLOUD_ACID },
-  { 50, 100, 175, UP,   CLOUD_STORM },
+  { 30, 100, 125, RISE, CLOUD_NEGATIVE_ENERGY },
+  { 40, 100, 135, RISE, CLOUD_STORM },
+  { 50, 100, 175, RISE, CLOUD_ACID },
   { 0,0,0,FLAT,CLOUD_NONE }
 };
 
@@ -470,7 +441,7 @@ spret_type cast_cloud_cone(const actor *caster, int pow, const coord_def &pos,
 
     const int range = spell_range(SPELL_CLOUD_CONE, pow);
 
-    targetter_shotgun hitfunc(caster, range);
+    targetter_shotgun hitfunc(caster, CLOUD_CONE_BEAM_COUNT, range);
 
     hitfunc.set_aim(pos);
 
@@ -485,12 +456,11 @@ spret_type cast_cloud_cone(const actor *caster, int pow, const coord_def &pos,
     random_picker<cloud_type, NUM_CLOUD_TYPES> cloud_picker;
     cloud_type cloud = cloud_picker.pick(cloud_cone_clouds, pow, CLOUD_NONE);
 
-    for (map<coord_def, int>::const_iterator p = hitfunc.zapped.begin();
-         p != hitfunc.zapped.end(); ++p)
+    for (const auto &entry : hitfunc.zapped)
     {
-        if (p->second <= 0)
+        if (entry.second <= 0)
             continue;
-        place_cloud(cloud, p->first,
+        place_cloud(cloud, entry.first,
                     5 + random2avg(12 + div_rand_round(pow * 3, 4), 3),
                     caster);
     }
