@@ -7,7 +7,6 @@
 #define SPL_MISCAST_H
 
 #include "enum.h"
-
 #include "monster.h"
 #include "mpr.h"
 #include "spl-util.h"
@@ -19,23 +18,41 @@ enum nothing_happens_when_type
     NH_ALWAYS,
 };
 
+enum miscast_source
+{
+    ZOT_TRAP_MISCAST,
+    HELL_EFFECT_MISCAST,
+    WIELD_MISCAST,
+    MELEE_MISCAST,
+    SPELL_MISCAST,
+    ABIL_MISCAST,
+    WIZARD_MISCAST,
+    MUMMY_MISCAST,
+    DECK_MISCAST,
+    // This should always be last.
+    GOD_MISCAST
+};
+
 class actor;
 // class monster;
 
 class MiscastEffect
 {
 public:
-    MiscastEffect(actor* _target, int _source, spell_type _spell, int _pow,
+    MiscastEffect(actor* _target, actor* _act_source,
+                  int _source, spell_type _spell, int _pow,
                   int _fail, string _cause = "",
                   nothing_happens_when_type _nothing_happens = NH_DEFAULT,
                   int _lethality_margin = 0,
                   string _hand_str = "", bool _can_plural_hand = true);
-    MiscastEffect(actor* _target, int _source, spschool_flag_type _school,
+    MiscastEffect(actor* _target, actor* _act_source,
+                  int _source, spschool_flag_type _school,
                   int _level, string _cause,
                   nothing_happens_when_type _nothing_happens = NH_DEFAULT,
                   int _lethality_margin = 0,
                   string _hand_str = "", bool _can_plural_hand = true);
-    MiscastEffect(actor* _target, int _source, spschool_flag_type _school,
+    MiscastEffect(actor* _target, actor* _act_source,
+                  int _source, spschool_flag_type _school,
                   int _pow, int _fail, string _cause,
                   nothing_happens_when_type _nothing_happens = NH_DEFAULT,
                   int _lethality_margin = 0,
@@ -47,7 +64,10 @@ public:
 
 private:
     actor* target;
-    int    source;
+    // May be nullptr.
+    actor* act_source;
+    // Either a miscast_source, or GOD_MISCAST + god_type enum.
+    int    special_source;
 
     string cause;
 
@@ -59,8 +79,8 @@ private:
     int level;
 
 private:
-    kill_category kc;
-    killer_type   kt;
+    // init() sets this to a proper value
+    killer_type kt = KILL_NONE;
 
     nothing_happens_when_type nothing_happens_when;
 
@@ -69,10 +89,6 @@ private:
     string hand_str;
     bool        can_plural_hand;
 
-    int    kill_source;
-    actor* act_source;
-    actor* guilty;
-
     bool source_known;
     bool target_known;
 
@@ -95,16 +111,6 @@ private:
     void init();
     string get_default_cause(bool attribute_to_user) const;
 
-    monster* target_as_monster()
-    {
-        return dynamic_cast<monster* >(target);
-    }
-
-    monster* source_as_monster()
-    {
-        return dynamic_cast<monster* >(act_source);
-    }
-
     bool neither_end_silenced();
 
     void do_msg(bool suppress_nothing_happens = false);
@@ -112,18 +118,19 @@ private:
     bool _explosion();
     bool _big_cloud(cloud_type cl_type, int cloud_pow, int size,
                     int spread_rate = -1);
-    bool _lose_stat(stat_type which_stat, int8_t stat_loss);
-    void _potion_effect(potion_type pot_eff, int pot_pow);
     bool _paralyse(int dur);
+    bool _sleep(int dur);
     bool _create_monster(monster_type what, int abj_deg, bool alert = false);
     bool _send_to_abyss();
-    bool _malign_gateway();
+    bool _malign_gateway(bool hostile = true);
     void _do_poison(int amount);
+    void _malmutate();
 
     bool avoid_lethal(int dam);
 
     void _conjuration(int severity);
-    void _enchantment(int severity);
+    void _hexes(int severity);
+    void _charms(int severity);
     void _translocation(int severity);
     void _summoning(int severity);
     void _divination_you(int severity);
