@@ -7,8 +7,16 @@ spret_type cast_cure_poison(int pow, bool fail);
 
 spret_type cast_sublimation_of_blood(int pow, bool fail);
 spret_type cast_death_channel(int pow, god_type god, bool fail);
+
+enum recall_t
+{
+    RECALL_SPELL,
+    RECALL_YRED,
+    RECALL_BEOGH,
+};
+
 spret_type cast_recall(bool fail);
-void start_recall(int type);
+void start_recall(recall_t type);
 void recall_orders(monster *mons);
 void do_recall(int time);
 void end_recall();
