@@ -6,32 +6,33 @@
 #include "AppHdr.h"
 
 #include "spl-selfench.h"
-#include "externs.h"
+
+#include <cmath>
 
 #include "areas.h"
 #include "art-enum.h"
-#include "env.h"
+#include "butcher.h" // butcher_corpse
+#include "coordit.h" // radius_iterator
 #include "godconduct.h"
 #include "hints.h"
+#include "items.h" // stack_iterator
 #include "libutil.h"
 #include "macro.h"
 #include "message.h"
-#include "misc.h"
-#include "options.h"
 #include "output.h"
 #include "religion.h"
-#include "spl-cast.h"
 #include "spl-transloc.h"
 #include "spl-util.h"
-
 #include "transform.h"
+#include "tilepick.h"
 #include "view.h"
+#include "viewchar.h"
 
 int allowed_deaths_door_hp()
 {
     int hp = calc_spell_power(SPELL_DEATHS_DOOR, true) / 10;
 
-    if (you_worship(GOD_KIKUBAAQUDGHA) && !player_under_penance())
+    if (in_good_standing(GOD_KIKUBAAQUDGHA))
         hp += you.piety / 15;
 
     return max(hp, 1);
@@ -39,7 +40,7 @@ int allowed_deaths_door_hp()
 
 spret_type cast_deaths_door(int pow, bool fail)
 {
-    if (you.is_undead)
+    if (you.undead_state())
         mpr("You're already dead!");
     else if (you.duration[DUR_EXHAUSTED])
         mpr("You are too exhausted to enter Death's door!");
@@ -80,7 +81,7 @@ spret_type ice_armour(int pow, bool fail)
         return SPRET_ABORT;
     }
 
-    if (player_stoneskin() || you.form == TRAN_STATUE)
+    if (you.duration[DUR_STONESKIN] || you.form == TRAN_STATUE)
     {
         mpr("The film of ice won't work on stone.");
         return SPRET_ABORT;
@@ -101,15 +102,123 @@ spret_type ice_armour(int pow, bool fail)
     else
         mpr("A film of ice covers your body!");
 
+    if (you.attribute[ATTR_BONE_ARMOUR] > 0)
+    {
+        you.attribute[ATTR_BONE_ARMOUR] = 0;
+        mpr("Your corpse armour falls away.");
+    }
 
     you.increase_duration(DUR_ICY_ARMOUR, 20 + random2(pow) + random2(pow), 50,
-                          NULL);
+                          nullptr);
     you.props[ICY_ARMOUR_KEY] = pow;
     you.redraw_armour_class = true;
 
     return SPRET_SUCCESS;
 }
 
+/**
+ * Iterate over all corpses in LOS and harvest them (unless it's just a test
+ * run)
+ *
+ * @param harvester   The entity planning to do the harvesting.
+ * @param dry_run     Whether this is a test run & no corpses should be
+ *                    actually destroyed.
+ * @return            The total number of corpses (available to be) destroyed.
+ */
+int harvest_corpses(const actor &harvester, bool dry_run)
+{
+    int harvested = 0;
+
+    for (radius_iterator ri(harvester.pos(), LOS_NO_TRANS); ri; ++ri)
+    {
+        for (stack_iterator si(*ri, true); si; ++si)
+        {
+            item_def &item = *si;
+            if (item.base_type != OBJ_CORPSES)
+                continue;
+
+            ++harvested;
+
+            if (dry_run)
+                continue;
+
+            // don't spam animations
+            if (harvested <= 5)
+            {
+                bolt beam;
+                beam.source = *ri;
+                beam.target = harvester.pos();
+                beam.glyph = dchar_glyph(DCHAR_FIRED_CHUNK);
+                beam.colour = item.get_colour();
+                beam.range = LOS_RADIUS;
+                beam.aimed_at_spot = true;
+                beam.item = &item;
+                beam.flavour = BEAM_VISUAL;
+                beam.draw_delay = 3;
+                beam.fire();
+                viewwindow();
+            }
+
+            destroy_item(item.index());
+        }
+    }
+
+    return harvested;
+}
+
+
+/**
+ * Casts the player spell "Cigotuvi's Embrace", pulling all corpses into LOS
+ * around the caster to serve as armour.
+ *
+ * @param pow   The spellpower at which the spell is being cast.
+ * @param fail  Whether the casting failed.
+ * @return      SPRET_ABORT if you already have an incompatible buff running,
+ *              SPRET_FAIL if fail is true, and SPRET_SUCCESS otherwise.
+ */
+spret_type corpse_armour(int pow, bool fail)
+{
+    if (you.duration[DUR_STONESKIN] || you.form == TRAN_STATUE)
+    {
+        mpr("The corpses won't embrace your stony flesh.");
+        return SPRET_ABORT;
+    }
+
+    if (you.duration[DUR_ICY_ARMOUR])
+    {
+        mpr("The corpses won't embrace your icy flesh.");
+        return SPRET_ABORT;
+    }
+
+    // Could check carefully to see if it's even possible that there are any
+    // valid corpses/skeletons in LOS (any piles with stuff under them, etc)
+    // before failing, but it's better to be simple + predictable from the
+    // player's perspective.
+    fail_check();
+
+    const int harvested = harvest_corpses(you);
+    dprf("Harvested: %d", harvested);
+
+    if (!harvested)
+    {
+        canned_msg(MSG_NOTHING_HAPPENS);
+        return SPRET_SUCCESS; // still takes a turn, etc
+    }
+
+    if (you.attribute[ATTR_BONE_ARMOUR] <= 0)
+        mpr("The bodies of the dead rush to embrace you!");
+    else
+        mpr("Your shell of carrion and bone grows thicker.");
+
+    // value of ATTR_BONE_ARMOUR will be sqrt(9*harvested), rounded randomly
+    int squared = sqr(you.attribute[ATTR_BONE_ARMOUR]) + 9 * harvested;
+    you.attribute[ATTR_BONE_ARMOUR] = sqrt(squared) + random_real();
+    you.redraw_armour_class = true;
+
+    return SPRET_SUCCESS;
+}
+
+
 spret_type missile_prot(int pow, bool fail)
 {
     if (you.attribute[ATTR_REPEL_MISSILES]
@@ -162,11 +271,7 @@ spret_type cast_revivification(int pow, bool fail)
         fail_check();
         mpr("Your body is healed in an amazingly painful way.");
 
-        int loss = 6;
-        for (int i = 0; i < 9; ++i)
-            if (x_chance_in_y(8, pow))
-                loss++;
-
+        const int loss = 6 + binomial(9, 8, pow);
         dec_max_hp(loss * you.hp_max / 100);
         set_hp(you.hp_max);
 
@@ -177,6 +282,7 @@ spret_type cast_revivification(int pow, bool fail)
             paralyse_player("Death's Door abortion", 5 + random2(5));
             confuse_player(10 + random2(10));
             you.duration[DUR_DEATHS_DOOR] = 0;
+            you.increase_duration(DUR_EXHAUSTED, roll_dice(1,3));
         }
         return SPRET_SUCCESS;
     }
@@ -217,50 +323,11 @@ spret_type cast_swiftness(int power, bool fail)
     you.set_duration(DUR_SWIFTNESS, 12 + random2(power)/2, 30,
                      "You feel quick.");
     you.attribute[ATTR_SWIFTNESS] = you.duration[DUR_SWIFTNESS];
-    did_god_conduct(DID_HASTY, 8, true);
 
     return SPRET_SUCCESS;
 }
 
-spret_type cast_fly(int power, bool fail)
-{
-    if (!flight_allowed())
-        return SPRET_ABORT;
-
-    fail_check();
-    const int dur_change = 25 + random2(power) + random2(power);
-    const bool was_flying = you.airborne();
-
-    you.increase_duration(DUR_FLIGHT, dur_change, 100);
-    you.attribute[ATTR_FLIGHT_UNCANCELLABLE] = 1;
-
-    if (!was_flying)
-        float_player();
-    else
-        mpr("You feel more buoyant.");
-    return SPRET_SUCCESS;
-}
-
-spret_type cast_teleport_control(int power, bool fail)
-{
-    fail_check();
-    if (allow_control_teleport(true))
-        mpr("You feel in control.");
-    else
-        mpr("You feel your control is inadequate.");
-
-    if (you.duration[DUR_TELEPORT] && !player_control_teleport())
-    {
-        mpr("You feel your translocation being delayed.");
-        you.increase_duration(DUR_TELEPORT, 1 + random2(3));
-    }
-
-    you.increase_duration(DUR_CONTROL_TELEPORT, 10 + random2(power), 50);
-
-    return SPRET_SUCCESS;
-}
-
-int cast_selective_amnesia(string *pre_msg)
+int cast_selective_amnesia(const string &pre_msg)
 {
     if (you.spell_no == 0)
     {
@@ -312,8 +379,8 @@ int cast_selective_amnesia(string *pre_msg)
             break;
     }
 
-    if (pre_msg)
-        mpr(pre_msg->c_str());
+    if (!pre_msg.empty())
+        mpr(pre_msg);
 
     del_spell_from_memory_by_slot(slot);
 
@@ -345,7 +412,7 @@ spret_type cast_song_of_slaying(int pow, bool fail)
 
     you.set_duration(DUR_SONG_OF_SLAYING, 20 + random2avg(pow, 2));
 
-    you.props["song_of_slaying_bonus"] = 0;
+    you.props[SONG_OF_SLAYING_KEY] = 0;
     return SPRET_SUCCESS;
 }
 
@@ -382,9 +449,9 @@ spret_type cast_liquefaction(int pow, bool fail)
     }
 
     fail_check();
-    flash_view_delay(BROWN, 80);
-    flash_view_delay(YELLOW, 80);
-    flash_view_delay(BROWN, 140);
+    flash_view_delay(UA_PLAYER, BROWN, 80);
+    flash_view_delay(UA_PLAYER, YELLOW, 80);
+    flash_view_delay(UA_PLAYER, BROWN, 140);
 
     mpr("The ground around you becomes liquefied!");
 
@@ -407,8 +474,11 @@ spret_type cast_shroud_of_golubria(int pow, bool fail)
 
 spret_type cast_transform(int pow, transformation_type which_trans, bool fail)
 {
-    if (!transform(pow, which_trans, false, true))
+    if (!transform(pow, which_trans, false, true)
+        || !check_form_stat_safety(which_trans))
+    {
         return SPRET_ABORT;
+    }
 
     fail_check();
     transform(pow, which_trans);
