@@ -861,13 +861,14 @@ spret_type cast_golubrias_passage(const coord_def& where, bool fail)
         randomized_where = where;
         randomized_where.x += random_range(-2, 2);
         randomized_where.y += random_range(-2, 2);
-    } while ((!in_bounds(randomized_where) ||
-             grd(randomized_where) != DNGN_FLOOR ||
-             monster_at(randomized_where) ||
-             !you.see_cell(randomized_where) ||
-             you.trans_wall_blocking(randomized_where) ||
-             randomized_where == you.pos()) &&
-            tries < 100);
+    }
+    while ((!in_bounds(randomized_where)
+            || grd(randomized_where) != DNGN_FLOOR
+            || monster_at(randomized_where)
+            || !you.see_cell(randomized_where)
+            || you.trans_wall_blocking(randomized_where)
+            || randomized_where == you.pos())
+           && tries < 100);
 
     do
     {
@@ -875,14 +876,14 @@ spret_type cast_golubrias_passage(const coord_def& where, bool fail)
         randomized_here = you.pos();
         randomized_here.x += random_range(-2, 2);
         randomized_here.y += random_range(-2, 2);
-    } while ((!in_bounds(randomized_here) ||
-             grd(randomized_here) != DNGN_FLOOR ||
-             monster_at(randomized_here) ||
-             !you.see_cell(randomized_here) ||
-             you.trans_wall_blocking(randomized_here) ||
-             randomized_here == you.pos() ||
-             randomized_here == randomized_where) &&
-            tries2 < 100);
+    }
+    while ((!in_bounds(randomized_here)
+            || grd(randomized_here) != DNGN_FLOOR
+            || monster_at(randomized_here)
+            || !you.see_cell(randomized_here)
+            || you.trans_wall_blocking(randomized_here)
+            || randomized_here == randomized_where)
+           && tries2 < 100);
 
     if (tries >= 100 || tries2 >= 100)
     {
@@ -890,7 +891,7 @@ spret_type cast_golubrias_passage(const coord_def& where, bool fail)
             mpr("You cannot create a passage on the other side of the transparent wall.");
         else
             // XXX: bleh, dumb message
-            mpr("Creating passages of Golubria requires sufficient empty space.");
+            mpr("Creating a passage of Golubria requires sufficient empty space.");
         return SPRET_ABORT;
     }
 
@@ -953,8 +954,8 @@ int gravitas_range(int pow, int strength)
 
 #define GRAVITY "by gravitational forces"
 
-void attract_actor(const actor* agent, actor* victim, const coord_def pos,
-                   int pow, int strength)
+static void _attract_actor(const actor* agent, actor* victim,
+                           const coord_def pos, int pow, int strength)
 {
     ASSERT(victim); // XXX: change to actor &victim
 
@@ -989,6 +990,8 @@ void attract_actor(const actor* agent, actor* victim, const coord_def pos,
                 victim->collide(newpos, agent, pow);
             break;
         }
+        else if (!victim->is_habitable(newpos))
+            break;
         else
             victim->move_to_pos(newpos, false);
 
@@ -1018,7 +1021,7 @@ bool fatal_attraction(const coord_def& pos, actor *agent, int pow)
             continue;
 
         affected = true;
-        attract_actor(agent, *ai, pos, pow, strength);
+        _attract_actor(agent, *ai, pos, pow, strength);
     }
 
     return affected;
