@@ -2,26 +2,21 @@
 
 #include "sprint.h"
 
-#include "externs.h"
-#include "monster.h"
-#include "mpr.h"
-#include "player.h"
 #include "religion.h"
-#include "random.h"
 
 int sprint_modify_exp(int exp)
 {
-    return exp * 9;
+    return exp * SPRINT_MULTIPLIER;
 }
 
 int sprint_modify_exp_inverse(int exp)
 {
-    return div_rand_round(exp, 9);
+    return div_rand_round(exp, SPRINT_MULTIPLIER);
 }
 
 int sprint_modify_piety(int piety)
 {
     if (you_worship(GOD_OKAWARU))
         return piety;
-    return piety * 9;
+    return piety * SPRINT_MULTIPLIER;
 }
