@@ -6,13 +6,11 @@
 #ifndef STASH_H
 #define STASH_H
 
-#include "shopping.h"
-#include <string>
-
 #include <map>
+#include <string>
 #include <vector>
 
-#include "externs.h"
+#include "shopping.h"
 
 class input_history;
 class reader;
@@ -39,7 +37,7 @@ public:
     vector<item_def> get_items() const;
 
     bool show_menu(const level_pos &place, bool can_travel,
-                   const vector<item_def>* matching_items = NULL) const;
+                   const vector<item_def>* matching_items = nullptr) const;
 
     // Returns true if this Stash contains items that are eligible for
     // autopickup.
@@ -190,7 +188,7 @@ struct stash_search_result
     vector<item_def> matching_items;
 
     stash_search_result() : pos(), player_distance(0), matches(0),
-                            count(0), match(), stash(NULL), shop(NULL),
+                            count(0), match(), stash(nullptr), shop(nullptr),
                             matching_items()
     {
     }
@@ -420,6 +418,5 @@ string stash_annotate_item(const char *s, const item_def *item,
 
 #define STASH_LUA_SEARCH_ANNOTATE "ch_stash_search_annotate_item"
 #define STASH_LUA_DUMP_ANNOTATE   "ch_stash_dump_annotate_item"
-#define STASH_LUA_VIEW_ANNOTATE   "ch_stash_view_annotate_item"
 
 #endif
