@@ -5,16 +5,17 @@
 
 #include "AppHdr.h"
 
+#include "state.h"
+
 #ifndef TARGET_OS_WINDOWS
 #include <unistd.h>
 #endif
 
-#include "externs.h"
-
 #include "dbg-util.h"
 #include "delay.h"
 #include "directn.h"
 #include "exclude.h"
+#include "hints.h"
 #include "macro.h"
 #include "menu.h"
 #include "message.h"
@@ -23,42 +24,43 @@
 #include "player.h"
 #include "religion.h"
 #include "showsymb.h"
-#include "state.h"
-#include "hints.h"
 #include "unwind.h"
 
 game_state::game_state()
-    : game_crashed(false),
-      mouse_enabled(false), waiting_for_command(false),
+    : game_crashed(false), mouse_enabled(false), waiting_for_command(false),
       terminal_resized(false), last_winch(0), io_inited(false),
-      need_save(false),
-      saving_game(false), updating_scores(false), seen_hups(0),
-      map_stat_gen(false), type(GAME_TYPE_NORMAL),
-      last_type(GAME_TYPE_UNSPECIFIED), arena_suspended(false),
-      generating_level(false),
-      dump_maps(false), test(false), script(false), build_db(false),
-      tests_selected(), show_more_prompt(true),
-      terminal_resize_handler(NULL), terminal_resize_check(NULL),
-      doing_prev_cmd_again(false), prev_cmd(CMD_NO_CMD),
-      repeat_cmd(CMD_NO_CMD),cmd_repeat_started_unsafe(false),
-      lua_calls_no_turn(0), stat_gain_prompt(false),
-      level_annotation_shown(false), viewport_monster_hp(false),
-      viewport_weapons(false),
+      need_save(false), saving_game(false), updating_scores(false),
+      seen_hups(0), map_stat_gen(false), obj_stat_gen(false),
+      type(GAME_TYPE_NORMAL), last_type(GAME_TYPE_UNSPECIFIED),
+      arena_suspended(false), generating_level(false), dump_maps(false),
+      test(false), script(false), build_db(false), tests_selected(),
+#ifdef DGAMELAUNCH
+      throttle(true),
+#else
+      throttle(false),
+#endif
+      show_more_prompt(true), terminal_resize_handler(nullptr),
+      terminal_resize_check(nullptr), doing_prev_cmd_again(false),
+      prev_cmd(CMD_NO_CMD), repeat_cmd(CMD_NO_CMD),
+      cmd_repeat_started_unsafe(false), lua_calls_no_turn(0),
+      stat_gain_prompt(false), level_annotation_shown(false),
+      viewport_monster_hp(false), viewport_weapons(false),
 #ifndef USE_TILE_LOCAL
       mlist_targeting(false),
 #else
+      tiles_disabled(false),
       title_screen(true),
 #endif
-      darken_range(NULL), unsaved_macros(false), mon_act(NULL)
+      invisible_targeting(false),
+      darken_range(nullptr), unsaved_macros(false), disables(),
+      minor_version(-1), save_rcs_version(), mon_act(nullptr)
 {
     reset_cmd_repeat();
     reset_cmd_again();
 #ifdef TARGET_OS_WINDOWS
     no_gdb = "Non-UNIX Platform -> not running gdb.";
-#elif defined DEBUG_DIAGNOSTICS
-    no_gdb = "Debug build -> run gdb yourself.";
 #else
-    no_gdb = access("/usr/bin/gdb", 1) ? "GDB not installed." : 0;
+    no_gdb = access("/usr/bin/gdb", 1) ? "/usr/bin/gdb not executable." : 0;
 #endif
 }
 
@@ -79,8 +81,8 @@ void game_state::show_startup_errors()
     error_menu.set_title(
         new MenuEntry("Warning: Crawl encountered errors during startup:",
                       MEL_TITLE));
-    for (int i = 0, size = startup_errors.size(); i < size; ++i)
-        error_menu.add_entry(new MenuEntry(startup_errors[i]));
+    for (const string &err : startup_errors)
+        error_menu.add_entry(new MenuEntry(err));
     error_menu.show();
 }
 
@@ -129,7 +131,7 @@ void game_state::cancel_cmd_repeat(string reason)
     reset_cmd_repeat();
 
     if (!reason.empty())
-        mpr(reason.c_str());
+        mpr(reason);
 }
 
 void game_state::cancel_cmd_again(string reason)
@@ -145,7 +147,7 @@ void game_state::cancel_cmd_again(string reason)
     reset_cmd_again();
 
     if (!reason.empty())
-        mpr(reason.c_str());
+        mpr(reason);
 }
 
 void game_state::cancel_cmd_all(string reason)
@@ -179,7 +181,7 @@ void game_state::cant_cmd_any(string reason)
 // The method is called to prevent the "no repeating zero turns
 // commands" message that input() generates (in the absence of
 // cancelling the repetition) for a repeated command that took no
-// turns.  A wrapper around cancel_cmd_repeat(), its only purpose is
+// turns. A wrapper around cancel_cmd_repeat(), its only purpose is
 // to make it clear why cancel_cmd_repeat() is being called.
 void game_state::zero_turns_taken()
 {
@@ -204,6 +206,7 @@ bool interrupt_cmd_repeat(activity_interrupt_type ai,
     case AI_FORCE_INTERRUPT:
     case AI_HP_LOSS:
     case AI_MONSTER_ATTACKS:
+    case AI_MIMIC:
         crawl_state.cancel_cmd_repeat("Command repetition interrupted.");
         return true;
 
@@ -213,8 +216,9 @@ bool interrupt_cmd_repeat(activity_interrupt_type ai,
 
     if (ai == AI_SEE_MONSTER)
     {
-        const monster* mon = static_cast<const monster* >(at.data);
-        if (!you.can_see(mon))
+        const monster* mon = at.mons_data;
+        ASSERT(mon);
+        if (!you.can_see(*mon))
             return false;
 
         if (crawl_state.cmd_repeat_started_unsafe
@@ -253,8 +257,7 @@ bool interrupt_cmd_repeat(activity_interrupt_type ai,
 
     // If command repetition is being used to imitate the rest command,
     // then everything interrupts it.
-    if (crawl_state.repeat_cmd == CMD_MOVE_NOWHERE
-        || crawl_state.repeat_cmd == CMD_WAIT)
+    if (crawl_state.repeat_cmd == CMD_WAIT)
     {
         if (ai == AI_FULL_MP)
             crawl_state.cancel_cmd_repeat("Magic restored.");
@@ -274,13 +277,10 @@ bool interrupt_cmd_repeat(activity_interrupt_type ai,
         // This check is for when command repetition is used to
         // whack away at a 0xp monster, since the player feels safe
         // when the only monsters around are 0xp.
-        const monster* mon = static_cast<const monster* >(at.data);
+        const monster* mon = at.mons_data;
 
-        if (mons_class_flag(mon->type, M_NO_EXP_GAIN)
-            && mon->visible_to(&you))
-        {
+        if (!mons_class_gives_xp(mon->type) && mon->visible_to(&you))
             return false;
-        }
 
         crawl_state.cancel_cmd_repeat("Command repetition interrupted.");
         return true;
@@ -324,11 +324,12 @@ void god_act_state::reset()
 bool game_state::is_god_acting() const
 {
     ASSERT(god_act.depth >= 0);
-    ASSERT(!(god_act.depth > 0 && god_act.which_god == GOD_NO_GOD));
-    ASSERT(!(god_act.depth == 0 && god_act.which_god != GOD_NO_GOD));
-    ASSERT(!(god_act.depth == 0 && !god_act_stack.empty()));
+    bool god_is_acting = god_act.depth > 0;
+    ASSERT(!(god_is_acting && god_act.which_god == GOD_NO_GOD));
+    ASSERT(god_is_acting || god_act.which_god == GOD_NO_GOD);
+    ASSERT(god_is_acting || god_act_stack.empty());
 
-    return god_act.depth > 0;
+    return god_is_acting;
 }
 
 bool game_state::is_god_retribution() const
@@ -409,7 +410,7 @@ vector<god_act_state> game_state::other_gods_acting() const
 
 bool game_state::is_mon_acting() const
 {
-    return mon_act != NULL;
+    return mon_act != nullptr;
 }
 
 monster* game_state::which_mon_acting() const
@@ -421,7 +422,7 @@ void game_state::inc_mon_acting(monster* mon)
 {
     ASSERT(!invalid_monster(mon));
 
-    if (mon_act != NULL)
+    if (mon_act != nullptr)
         mon_act_stack.push_back(mon_act);
 
     mon_act = mon;
@@ -431,7 +432,7 @@ void game_state::dec_mon_acting(monster* mon)
 {
     ASSERT(mon_act == mon);
 
-    mon_act = NULL;
+    mon_act = nullptr;
 
     const unsigned int size = mon_act_stack.size();
     if (size > 0)
@@ -444,7 +445,7 @@ void game_state::dec_mon_acting(monster* mon)
 
 void game_state::clear_mon_acting()
 {
-    mon_act = NULL;
+    mon_act = nullptr;
     mon_act_stack.clear();
 }
 
@@ -493,8 +494,8 @@ void game_state::dump()
     if (!startup_errors.empty())
     {
         fprintf(stderr, "Startup errors:\n");
-        for (unsigned int i = 0; i < startup_errors.size(); i++)
-            fprintf(stderr, "%s\n", startup_errors[i].c_str());
+        for (const string &err : startup_errors)
+            fprintf(stderr, "%s\n", err.c_str());
         fprintf(stderr, "\n");
     }
 
@@ -503,12 +504,12 @@ void game_state::dump()
     if (doing_prev_cmd_again)
     {
         fprintf(stderr, "Doing prev_cmd again with keys: ");
-        for (unsigned int i = 0; i < prev_cmd_keys.size(); i++)
-            fprintf(stderr, "%d, ", prev_cmd_keys[i]);
+        for (int key : prev_cmd_keys)
+            fprintf(stderr, "%d, ", key);
         fprintf(stderr, "\n");
         fprintf(stderr, "As ASCII keys: ");
-        for (unsigned int i = 0; i < prev_cmd_keys.size(); i++)
-            fprintf(stderr, "%c", (char) prev_cmd_keys[i]);
+        for (int key : prev_cmd_keys)
+            fprintf(stderr, "%c", (char) key);
         fprintf(stderr, "\n\n");
     }
     fprintf(stderr, "repeat_cmd = %s\n", command_to_name(repeat_cmd).c_str());
@@ -524,16 +525,15 @@ void game_state::dump()
     if (!god_act_stack.empty())
     {
         fprintf(stderr, "Other gods acting:\n");
-        for (unsigned int i = 0; i < god_act_stack.size(); i++)
+        for (const god_act_state &godact : god_act_stack)
         {
             fprintf(stderr, "God %s with depth %d\n",
-                    god_name(god_act_stack[i].which_god).c_str(),
-                    god_act_stack[i].depth);
+                    god_name(godact.which_god).c_str(), godact.depth);
         }
         fprintf(stderr, "\n\n");
     }
 
-    if (mon_act != NULL)
+    if (mon_act != nullptr)
     {
         fprintf(stderr, "%s currently acting:\n\n",
                 debug_mon_str(mon_act).c_str());
@@ -543,11 +543,8 @@ void game_state::dump()
     if (!mon_act_stack.empty())
     {
         fprintf(stderr, "Others monsters acting:\n");
-        for (unsigned int i = 0; i < mon_act_stack.size(); i++)
-        {
-            fprintf(stderr, "    %s\n",
-                    debug_mon_str(mon_act_stack[i]).c_str());
-        }
+        for (const monster *mon : mon_act_stack)
+            fprintf(stderr, "    %s\n", debug_mon_str(mon).c_str());
     }
 }
 
@@ -585,12 +582,6 @@ bool game_state::game_is_sprint() const
     return type == GAME_TYPE_SPRINT;
 }
 
-bool game_state::game_is_zotdef() const
-{
-    ASSERT(type < NUM_GAME_TYPE);
-    return type == GAME_TYPE_ZOTDEF;
-}
-
 bool game_state::game_is_hints() const
 {
     ASSERT(type < NUM_GAME_TYPE);
@@ -623,23 +614,18 @@ string game_state::game_type_name_for(game_type _type)
         return "Arena";
     case GAME_TYPE_SPRINT:
         return "Dungeon Sprint";
-    case GAME_TYPE_ZOTDEF:
-        return "Zot Defence";
     }
 }
 
 string game_state::game_savedir_path() const
 {
-    return game_is_sprint()? "sprint/" :
-           game_is_zotdef()? "zotdef/" : "";
+    return game_is_sprint()? "sprint/" : "";
 }
 
 string game_state::game_type_qualifier() const
 {
     if (crawl_state.game_is_sprint())
         return "-sprint";
-    if (crawl_state.game_is_zotdef())
-        return "-zotdef";
     if (crawl_state.game_is_tutorial())
         return "-tutorial";
     if (crawl_state.game_is_hints())
