@@ -1,29 +1,26 @@
 #include "AppHdr.h"
 
 #include "status.h"
+#include "duration-data.h"
 
 #include "areas.h"
 #include "branch.h"
+#include "cloud.h"
 #include "env.h"
 #include "evoke.h"
 #include "food.h"
 #include "godabil.h"
-#include "godpassive.h"
-#include "libutil.h"
-#include "misc.h"
+#include "itemprop.h"
 #include "mutation.h"
-#include "player.h"
+#include "options.h"
 #include "player-stats.h"
 #include "religion.h"
-#include "skills2.h"
 #include "spl-transloc.h"
 #include "stringutil.h"
-#include "terrain.h"
+#include "throw.h"
 #include "transform.h"
 #include "traps.h"
 
-#include "duration-data.h"
-
 static int duration_index[NUM_DURATIONS];
 
 void init_duration_index()
@@ -46,7 +43,7 @@ static const duration_def* _lookup_duration(duration_type dur)
 {
     ASSERT_RANGE(dur, 0, NUM_DURATIONS);
     if (duration_index[dur] == -1)
-        return NULL;
+        return nullptr;
     else
         return &duration_data[duration_index[dur]];
 }
@@ -56,13 +53,18 @@ const char *duration_name(duration_type dur)
     return _lookup_duration(dur)->name();
 }
 
+bool duration_dispellable(duration_type dur)
+{
+    return _lookup_duration(dur)->duration_has_flag(D_DISPELLABLE);
+}
+
 static void _reset_status_info(status_info* inf)
 {
     inf->light_colour = 0;
     inf->light_text = "";
     inf->short_text = "";
     inf->long_text = "";
-};
+}
 
 static int _bad_ench_colour(int lvl, int orange, int red)
 {
@@ -107,6 +109,33 @@ static void _mark_expiring(status_info* inf, bool expiring)
     }
 }
 
+/**
+ * Populate a status_info struct from the duration_data struct corresponding
+ * to the given duration_type.
+ *
+ * @param[in] dur    The duration in question.
+ * @param[out] inf   The status_info struct to be filled.
+ * @return           Whether a duration_data struct was found.
+ */
+static bool _fill_inf_from_ddef(duration_type dur, status_info* inf)
+{
+    const duration_def* ddef = _lookup_duration(dur);
+    if (!ddef)
+        return false;
+
+    inf->light_colour = ddef->light_colour;
+    inf->light_text   = ddef->light_text;
+    inf->short_text   = ddef->short_text;
+    inf->long_text    = ddef->long_text;
+    if (ddef->duration_has_flag(D_EXPIRES))
+    {
+        inf->light_colour = _dur_colour(inf->light_colour, dur_expiring(dur));
+        _mark_expiring(inf, dur_expiring(dur));
+    }
+
+    return true;
+}
+
 static void _describe_airborne(status_info* inf);
 static void _describe_glow(status_info* inf);
 static void _describe_hunger(status_info* inf);
@@ -136,21 +165,7 @@ bool fill_status_info(int status, status_info* inf)
         if (!you.duration[dur])
             return false;
 
-        const duration_def* ddef = _lookup_duration(dur);
-        if (ddef)
-        {
-            found = true;
-            inf->light_colour = ddef->light_colour;
-            inf->light_text   = ddef->light_text;
-            inf->short_text   = ddef->short_text;
-            inf->long_text    = ddef->long_text;
-            if (ddef->expire)
-            {
-                inf->light_colour = _dur_colour(inf->light_colour,
-                                                 dur_expiring(dur));
-                _mark_expiring(inf, dur_expiring(dur));
-            }
-        }
+        found = _fill_inf_from_ddef(dur, inf);
     }
 
     // Now treat special status types and durations, possibly
@@ -160,12 +175,7 @@ bool fill_status_info(int status, status_info* inf)
 
     case DUR_CORROSION:
         inf->light_text = make_stringf("Corr (%d)",
-                          (-3 * you.props["corrosion_amount"].get_int()));
-        break;
-
-    case DUR_CONTROL_TELEPORT:
-        if (!allow_control_teleport(true))
-            inf->light_colour = DARKGREY;
+                          (-4 * you.props["corrosion_amount"].get_int()));
         break;
 
     case DUR_NO_POTIONS:
@@ -279,8 +289,7 @@ bool fill_status_info(int status, status_info* inf)
 
     case DUR_CONFUSING_TOUCH:
     {
-        inf->long_text = string("Your ") + you.hand_name(true)
-                         + " are glowing red.";
+        inf->long_text = you.hands_act("are", "glowing red.");
         break;
     }
 
@@ -302,12 +311,15 @@ bool fill_status_info(int status, status_info* inf)
         break;
 
     case DUR_POWERED_BY_DEATH:
-        if (handle_pbd_corpses() > 0)
+    {
+        const int pbd_str = you.props[POWERED_BY_DEATH_KEY].get_int();
+        if (pbd_str > 0)
         {
             inf->light_colour = LIGHTMAGENTA;
-            inf->light_text   = "Regen+";
+            inf->light_text   = make_stringf("Regen (%d)", pbd_str);
         }
         break;
+    }
 
     case STATUS_MISSILES:
         _describe_missiles(inf);
@@ -328,22 +340,6 @@ bool fill_status_info(int status, status_info* inf)
         break;
     }
 
-    case DUR_SURE_BLADE:
-    {
-        inf->light_colour = BLUE;
-        inf->light_text   = "Blade";
-        inf->short_text   = "bonded with blade";
-        string desc;
-        if (you.duration[DUR_SURE_BLADE] > 15 * BASELINE_DELAY)
-            desc = "strong ";
-        else if (you.duration[DUR_SURE_BLADE] >  5 * BASELINE_DELAY)
-            desc = "";
-        else
-            desc = "weak";
-        inf->long_text = "You have a " + desc + "bond with your blade.";
-        break;
-    }
-
     case DUR_TRANSFORMATION:
         _describe_transform(inf);
         break;
@@ -368,6 +364,14 @@ bool fill_status_info(int status, status_info* inf)
         }
         break;
 
+    case STATUS_BONE_ARMOUR:
+        if (you.attribute[ATTR_BONE_ARMOUR] > 0)
+        {
+            inf->short_text = "corpse armour";
+            inf->long_text = "You are enveloped in carrion and bones.";
+        }
+        break;
+
     case STATUS_CONSTRICTED:
         if (you.is_constricted())
         {
@@ -393,16 +397,9 @@ bool fill_status_info(int status, status_info* inf)
         break;
 
     case DUR_SONG_OF_SLAYING:
-        inf->light_text = make_stringf("Slay (%u)",
-                                       you.props["song_of_slaying_bonus"].get_int());
-        break;
-
-    case STATUS_NO_CTELE:
-        if (!allow_control_teleport(true))
-        {
-            inf->light_colour = RED;
-            inf->light_text = "-cTele";
-        }
+        inf->light_text
+            = make_stringf("Slay (%u)",
+                           you.props[SONG_OF_SLAYING_KEY].get_int());
         break;
 
     case STATUS_BEOGH:
@@ -525,13 +522,15 @@ bool fill_status_info(int status, status_info* inf)
     case STATUS_BRIBE:
     {
         int bribe = 0;
-        vector<string> places;
+        vector<const char *> places;
         for (int i = 0; i < NUM_BRANCHES; i++)
         {
-            if (branch_bribe[i] > 0)
+            branch_type br = gozag_fixup_branch(static_cast<branch_type>(i));
+
+            if (branch_bribe[br] > 0)
             {
                 if (player_in_branch(static_cast<branch_type>(i)))
-                    bribe = branch_bribe[i];
+                    bribe = branch_bribe[br];
 
                 places.push_back(branches[static_cast<branch_type>(i)]
                                  .longname);
@@ -558,6 +557,69 @@ bool fill_status_info(int status, status_info* inf)
         break;
     }
 
+    case DUR_HORROR:
+    {
+        const int horror = you.props[HORROR_PENALTY_KEY].get_int();
+        inf->light_text = make_stringf("Horr(%d)", -1 * horror);
+        if (horror >= HORROR_LVL_OVERWHELMING)
+        {
+            inf->light_colour = RED;
+            inf->short_text   = "overwhelmed with horror";
+            inf->long_text    = "Horror overwhelms you!";
+        }
+        else if (horror >= HORROR_LVL_EXTREME)
+        {
+            inf->light_colour = LIGHTRED;
+            inf->short_text   = "extremely horrified";
+            inf->long_text    = "You are extremely horrified!";
+        }
+        else if (horror)
+        {
+            inf->light_colour = YELLOW;
+            inf->short_text   = "horrified";
+            inf->long_text    = "You are horrified!";
+        }
+        break;
+    }
+
+    case STATUS_CLOUD:
+    {
+        cloud_type cloud = cloud_type_at(you.pos());
+        if (Options.cloud_status && cloud != CLOUD_NONE)
+        {
+            inf->light_text = "Cloud";
+            // TODO: make the colour based on the cloud's color; requires elemental
+            // status lights, though.
+            inf->light_colour =
+                is_damaging_cloud(cloud, true, cloud_is_yours_at(you.pos())) ? LIGHTRED : DARKGREY;
+        }
+        break;
+    }
+
+    case DUR_CLEAVE:
+    {
+        const item_def* weapon = you.weapon();
+
+        if (weapon && item_attack_skill(*weapon) == SK_AXES)
+            inf->light_colour = DARKGREY;
+
+        break;
+    }
+
+    case DUR_PORTAL_PROJECTILE:
+    {
+        if (!is_pproj_active())
+            inf->light_colour = DARKGREY;
+        break;
+    }
+
+    case DUR_INFUSION:
+    {
+        if (!enough_mp(1, true, false))
+            inf->light_colour = DARKGREY;
+        break;
+    }
+
     default:
         if (!found)
         {
@@ -604,10 +666,15 @@ static void _describe_hunger(status_info* inf)
         inf->light_text   = (vamp ? "Near Bloodless" : "Near Starving");
         break;
     case HS_STARVING:
-        inf->light_colour = RED;
+        inf->light_colour = LIGHTRED;
         inf->light_text   = (vamp ? "Bloodless" : "Starving");
         inf->short_text   = (vamp ? "bloodless" : "starving");
         break;
+    case HS_FAINTING:
+        inf->light_colour = RED;
+        inf->light_text   = (vamp ? "Bloodless" : "Fainting");
+        inf->short_text   = (vamp ? "bloodless" : "fainting");
+        break;
     case HS_SATIATED: // no status light
     default:
         break;
@@ -616,27 +683,35 @@ static void _describe_hunger(status_info* inf)
 
 static void _describe_glow(status_info* inf)
 {
-    const int cont = get_contamination_level();
-    if (cont > 0)
-    {
-        inf->light_colour = DARKGREY;
-        if (cont > 1)
-            inf->light_colour = _bad_ench_colour(cont, 3, 4);
-#if TAG_MAJOR_VERSION == 34
-        if (cont > 1 || you.species != SP_DJINNI)
-#endif
-        inf->light_text = "Contam";
+    const int signed_cont = get_contamination_level();
+    if (signed_cont <= 0)
+        return;
 
-        inf->short_text =
-                 (cont == 1) ? "very slightly " :
-                 (cont == 2) ? "slightly " :
-                 (cont == 3) ? "" :
-                 (cont == 4) ? "moderately " :
-                 (cont == 5) ? "heavily "
-                             : "really heavily ";
-        inf->short_text += "contaminated";
-        inf->long_text = describe_contamination(cont);
-    }
+    const unsigned int cont = signed_cont; // so we don't get compiler warnings
+    inf->light_colour = DARKGREY;
+    if (cont > 1)
+        inf->light_colour = _bad_ench_colour(cont, 3, 4);
+#if TAG_MAJOR_VERSION == 34
+    if (cont > 1 || you.species != SP_DJINNI)
+#endif
+    inf->light_text = "Contam";
+
+    /// Mappings from contamination levels to descriptions.
+    static const string contam_adjectives[] =
+    {
+        "",
+        "very slightly ",
+        "",
+        "heavily ",
+        "very heavily ",
+        "very very heavily ", // this is silly but no one will ever see it
+        "impossibly ",        // (likewise)
+    };
+    ASSERT(cont >= 0);
+
+    const int adj_i = min((size_t) cont, ARRAYSZ(contam_adjectives) - 1);
+    inf->short_text = contam_adjectives[adj_i] + "contaminated";
+    inf->long_text = describe_contamination(cont);
 }
 
 static void _describe_regen(status_info* inf)
@@ -644,8 +719,8 @@ static void _describe_regen(status_info* inf)
     const bool regen = (you.duration[DUR_REGENERATION] > 0
                         || you.duration[DUR_TROGS_HAND] > 0);
     const bool no_heal =
-            (you.species == SP_VAMPIRE && you.hunger_state == HS_STARVING)
-            || (player_mutation_level(MUT_SLOW_HEALING) == 3);
+            (you.species == SP_VAMPIRE && you.hunger_state <= HS_STARVING)
+            || (player_mutation_level(MUT_SLOW_REGENERATION) == 3);
     // Does vampire hunger level affect regeneration rate significantly?
     const bool vampmod = !no_heal && !regen && you.species == SP_VAMPIRE
                          && you.hunger_state != HS_SATIATED;
@@ -707,27 +782,31 @@ static void _describe_poison(status_info* inf)
          (pois_perc > 65)   ? "seriously" :
          (pois_perc > 35)   ? "quite"
                             : "mildly";
-    inf->short_text   = adj + " poisoned";
-    inf->long_text    = "You are " + inf->short_text + make_stringf(" (%d -> %d).", you.hp, poison_survival());
+    inf->short_text   = adj + " poisoned"
+        + make_stringf(" (%d -> %d)", you.hp, poison_survival());
+    inf->long_text    = "You are " + inf->short_text + ".";
 }
 
 static void _describe_speed(status_info* inf)
 {
-    if (you.duration[DUR_SLOW] && you.duration[DUR_HASTE])
+    bool slow = you.duration[DUR_SLOW] || have_stat_zero();
+    bool fast = you.duration[DUR_HASTE];
+
+    if (slow && fast)
     {
         inf->light_colour = MAGENTA;
         inf->light_text   = "Fast+Slow";
         inf->short_text   = "hasted and slowed";
         inf->long_text = "You are under both slowing and hasting effects.";
     }
-    else if (you.duration[DUR_SLOW])
+    else if (slow)
     {
         inf->light_colour = RED;
         inf->light_text   = "Slow";
         inf->short_text   = "slowed";
         inf->long_text    = "You are slowed.";
     }
-    else if (you.duration[DUR_HASTE])
+    else if (fast)
     {
         inf->light_colour = _dur_colour(BLUE, dur_expiring(DUR_HASTE));
         inf->light_text   = "Fast";
@@ -756,30 +835,21 @@ static void _describe_airborne(status_info* inf)
 
 static void _describe_rotting(status_info* inf)
 {
-    if (you.rotting)
-    {
-        inf->light_colour = _bad_ench_colour(you.rotting, 5, 9);
-        inf->light_text   = "Rot";
-    }
-
-    if (you.rotting || you.species == SP_GHOUL)
+    if (you.species == SP_GHOUL)
     {
         inf->short_text = "rotting";
         inf->long_text = "Your flesh is rotting";
-        int rot = you.rotting;
-        if (you.species == SP_GHOUL)
-            rot += 1 + (1 << max(0, HS_SATIATED - you.hunger_state));
+        int rot = 1 + (1 << max(0, HS_SATIATED - you.hunger_state));
         if (rot > 15)
             inf->long_text += " before your eyes";
         else if (rot > 8)
             inf->long_text += " away quickly";
         else if (rot > 4)
             inf->long_text += " badly";
-        else if (you.species == SP_GHOUL)
-            if (rot > 2)
-                inf->long_text += " faster than usual";
-            else
-                inf->long_text += " at the usual pace";
+        else if (rot > 2)
+            inf->long_text += " faster than usual";
+        else
+            inf->long_text += " at the usual pace";
         inf->long_text += ".";
     }
 }
@@ -803,97 +873,25 @@ static void _describe_sickness(status_info* inf)
     }
 }
 
+/**
+ * Populate a status info struct with a description of the player's current
+ * form.
+ *
+ * @param[out] inf  The status info struct to be populated.
+ */
 static void _describe_transform(status_info* inf)
 {
+    if (you.form == TRAN_NONE)
+        return;
+
+    const Form * const form = get_form();
+    inf->light_text = form->short_name;
+    inf->short_text = form->get_long_name();
+    inf->long_text = form->get_description();
+
     const bool vampbat = (you.species == SP_VAMPIRE && you.form == TRAN_BAT);
     const bool expire  = dur_expiring(DUR_TRANSFORMATION) && !vampbat;
 
-    switch (you.form)
-    {
-    case TRAN_BAT:
-        inf->light_text     = "Bat";
-        inf->short_text     = "bat-form";
-        inf->long_text      = "You are in ";
-        if (vampbat)
-            inf->long_text += "vampire ";
-        inf->long_text     += "bat-form.";
-        break;
-    case TRAN_BLADE_HANDS:
-        inf->light_text = "Blades";
-        inf->short_text = "blade " + blade_parts(true);
-        inf->long_text  = "You have blades for " + blade_parts() + ".";
-        break;
-    case TRAN_DRAGON:
-        inf->light_text = "Dragon";
-        inf->short_text = "dragon-form";
-        inf->long_text  = "You are in dragon-form.";
-        break;
-    case TRAN_ICE_BEAST:
-        inf->light_text = "Ice";
-        inf->short_text = "ice-form";
-        inf->long_text  = "You are an ice creature.";
-        break;
-    case TRAN_LICH:
-        inf->light_text = "Lich";
-        inf->short_text = "lich-form";
-        inf->long_text  = "You are in lich-form.";
-        break;
-    case TRAN_PIG:
-        inf->light_text = "Pig";
-        inf->short_text = "pig-form";
-        inf->long_text  = "You are a filthy swine.";
-        break;
-    case TRAN_SPIDER:
-        inf->light_text = "Spider";
-        inf->short_text = "spider-form";
-        inf->long_text  = "You are in spider-form.";
-        break;
-    case TRAN_STATUE:
-        inf->light_text = "Statue";
-        inf->short_text = "statue-form";
-        inf->long_text  = "You are a statue.";
-        break;
-    case TRAN_APPENDAGE:
-        inf->light_text = "App";
-        inf->short_text = "appendage";
-        inf->long_text  = "You have a beastly appendage.";
-        break;
-    case TRAN_FUNGUS:
-        inf->light_text = "Fungus";
-        inf->short_text = "fungus-form";
-        inf->long_text  = "You are a sentient fungus.";
-        break;
-    case TRAN_TREE:
-        inf->light_text = "Tree";
-        inf->short_text = "tree-form";
-        inf->long_text  = "You are an animated tree.";
-        break;
-#if TAG_MAJOR_VERSION == 34
-    case TRAN_JELLY:
-        inf->light_text = "Jelly";
-        inf->short_text = "jelly-form";
-        inf->long_text  = "You are a lump of jelly.";
-        break;
-#endif
-    case TRAN_PORCUPINE:
-        inf->light_text = "Porc";
-        inf->short_text = "porcupine-form";
-        inf->long_text  = "You are a porcupine.";
-        break;
-    case TRAN_WISP:
-        inf->light_text = "Wisp";
-        inf->short_text = "wisp-form";
-        inf->long_text  = "You are an insubstantial wisp.";
-        break;
-    case TRAN_SHADOW:
-        inf->light_text = "Shadow",
-        inf->short_text = "shadow form";
-        inf->long_text  = "You are a swirling mass of dark shadows.";
-        break;
-    case TRAN_NONE:
-        break;
-    }
-
     inf->light_colour = _dur_colour(GREEN, expire);
     _mark_expiring(inf, expire);
 }
@@ -902,7 +900,7 @@ static const char* s0_names[NUM_STATS] = { "Collapse", "Brainless", "Clumsy", };
 
 static void _describe_stat_zero(status_info* inf, stat_type st)
 {
-    if (you.stat_zero[st])
+    if (you.duration[stat_zero_duration(st)])
     {
         inf->light_colour = you.stat(st) ? LIGHTRED : RED;
         inf->light_text   = s0_names[st];
@@ -942,10 +940,7 @@ static void _describe_missiles(status_info* inf)
 
     if (level > 1)
     {
-        bool perm = false;
-                    /* you_worship(GOD_QAZLAL)
-                       && !player_under_penance(GOD_QAZLAL)
-                       && you.piety >= piety_breakpoint(4); */
+        bool perm = false; /* in_good_standing(GOD_QAZLAL, 4) */
         inf->light_colour = perm ? WHITE : LIGHTMAGENTA;
         inf->light_text   = "DMsl";
         inf->short_text   = "deflect missiles";
@@ -955,9 +950,7 @@ static void _describe_missiles(status_info* inf)
     {
         bool perm = player_mutation_level(MUT_DISTORTION_FIELD) == 3
                     || you.scan_artefacts(ARTP_RMSL)
-                    || you_worship(GOD_QAZLAL)
-                       && !player_under_penance(GOD_QAZLAL)
-                       && you.piety >= piety_breakpoint(3);
+                    || in_good_standing(GOD_QAZLAL, 3);
         inf->light_colour = perm ? WHITE : LIGHTBLUE;
         inf->light_text   = "RMsl";
         inf->short_text   = "repel missiles";
