@@ -18,6 +18,6 @@ enum rounding_type
 double stepdown(double value, double step);
 int stepdown(int value, int step, rounding_type = ROUND_CLOSE, int max = 0);
 int stepdown_value(int base_value, int stepping, int first_step,
-                   int last_step, int ceiling_value);
+                   int /*last_step*/, int ceiling_value);
 
 #endif
