@@ -8,14 +8,12 @@
 
 #include "store.h"
 
+#include <algorithm>
+
 #include "dlua.h"
-#include "externs.h"
+#include "libutil.h" // map_find
 #include "monster.h"
 #include "stringutil.h"
-#include "tags.h"
-#include "travel.h"
-
-#include <algorithm>
 
 // These tend to be called from tight loops, and C++ method calls don't
 // get optimized away except for LTO -fwhole-program builds, so merely
@@ -29,14 +27,14 @@
 CrawlStoreValue::CrawlStoreValue()
     : type(SV_NONE), flags(SFLAG_UNSET)
 {
-    val.ptr = NULL;
+    val.ptr = nullptr;
 }
 
 CrawlStoreValue::CrawlStoreValue(const CrawlStoreValue &other)
 {
     ASSERT_RANGE(other.type, SV_NONE, NUM_STORE_VAL_TYPES);
 
-    val.ptr = NULL;
+    val.ptr = nullptr;
 
     type  = other.type;
     flags = other.flags;
@@ -146,7 +144,7 @@ CrawlStoreValue::CrawlStoreValue(const store_flags _flags,
     ASSERT(!(flags & SFLAG_UNSET));
 
     flags   |= SFLAG_UNSET;
-    val.ptr  = NULL;
+    val.ptr  = nullptr;
 }
 
 // Conversion constructors
@@ -189,70 +187,70 @@ CrawlStoreValue::CrawlStoreValue(const float &_val)
 CrawlStoreValue::CrawlStoreValue(const string &_val)
     : type(SV_STR), flags(SFLAG_UNSET)
 {
-    val.ptr = NULL;
+    val.ptr = nullptr;
     get_string() = _val;
 }
 
 CrawlStoreValue::CrawlStoreValue(const char* _val)
     : type(SV_STR), flags(SFLAG_UNSET)
 {
-    val.ptr = NULL;
+    val.ptr = nullptr;
     get_string() = _val;
 }
 
 CrawlStoreValue::CrawlStoreValue(const coord_def &_val)
     : type(SV_COORD), flags(SFLAG_UNSET)
 {
-    val.ptr = NULL;
+    val.ptr = nullptr;
     get_coord() = _val;
 }
 
 CrawlStoreValue::CrawlStoreValue(const item_def &_val)
     : type(SV_ITEM), flags(SFLAG_UNSET)
 {
-    val.ptr = NULL;
+    val.ptr = nullptr;
     get_item() = _val;
 }
 
 CrawlStoreValue::CrawlStoreValue(const CrawlHashTable &_val)
     : type(SV_HASH), flags(SFLAG_UNSET)
 {
-    val.ptr = NULL;
+    val.ptr = nullptr;
     get_table() = _val;
 }
 
 CrawlStoreValue::CrawlStoreValue(const CrawlVector &_val)
     : type(SV_VEC), flags(SFLAG_UNSET)
 {
-    val.ptr = NULL;
+    val.ptr = nullptr;
     get_vector() = _val;
 }
 
 CrawlStoreValue::CrawlStoreValue(const level_id &_val)
     : type(SV_LEV_ID), flags(SFLAG_UNSET)
 {
-    val.ptr = NULL;
+    val.ptr = nullptr;
     get_level_id() = _val;
 }
 
 CrawlStoreValue::CrawlStoreValue(const level_pos &_val)
     : type(SV_LEV_POS), flags(SFLAG_UNSET)
 {
-    val.ptr = NULL;
+    val.ptr = nullptr;
     get_level_pos() = _val;
 }
 
 CrawlStoreValue::CrawlStoreValue(const monster& _val)
     : type(SV_MONST), flags(SFLAG_UNSET)
 {
-    val.ptr = NULL;
+    val.ptr = nullptr;
     get_monster() = _val;
 }
 
 CrawlStoreValue::CrawlStoreValue(const dlua_chunk &_val)
     : type(SV_LUA), flags(SFLAG_UNSET)
 {
-    val.ptr = NULL;
+    val.ptr = nullptr;
     get_lua() = _val;
 }
 
@@ -301,7 +299,7 @@ void CrawlStoreValue::unset(bool force)
     {
         string* str = static_cast<string*>(val.ptr);
         delete str;
-        val.ptr = NULL;
+        val.ptr = nullptr;
         break;
     }
 
@@ -309,7 +307,7 @@ void CrawlStoreValue::unset(bool force)
     {
         coord_def* coord = static_cast<coord_def*>(val.ptr);
         delete coord;
-        val.ptr = NULL;
+        val.ptr = nullptr;
         break;
     }
 
@@ -317,7 +315,7 @@ void CrawlStoreValue::unset(bool force)
     {
         item_def* item = static_cast<item_def*>(val.ptr);
         delete item;
-        val.ptr = NULL;
+        val.ptr = nullptr;
         break;
     }
 
@@ -325,7 +323,7 @@ void CrawlStoreValue::unset(bool force)
     {
         CrawlHashTable* hash = static_cast<CrawlHashTable*>(val.ptr);
         delete hash;
-        val.ptr = NULL;
+        val.ptr = nullptr;
         break;
     }
 
@@ -333,7 +331,7 @@ void CrawlStoreValue::unset(bool force)
     {
         CrawlVector* vec = static_cast<CrawlVector*>(val.ptr);
         delete vec;
-        val.ptr = NULL;
+        val.ptr = nullptr;
         break;
     }
 
@@ -341,7 +339,7 @@ void CrawlStoreValue::unset(bool force)
     {
         level_id* id = static_cast<level_id*>(val.ptr);
         delete id;
-        val.ptr = NULL;
+        val.ptr = nullptr;
         break;
     }
 
@@ -349,7 +347,7 @@ void CrawlStoreValue::unset(bool force)
     {
         level_pos* pos = static_cast<level_pos*>(val.ptr);
         delete pos;
-        val.ptr = NULL;
+        val.ptr = nullptr;
         break;
     }
 
@@ -357,7 +355,7 @@ void CrawlStoreValue::unset(bool force)
     {
         monster* mon = static_cast<monster* >(val.ptr);
         delete mon;
-        val.ptr = NULL;
+        val.ptr = nullptr;
         break;
     }
 
@@ -365,7 +363,7 @@ void CrawlStoreValue::unset(bool force)
     {
         dlua_chunk* chunk = static_cast<dlua_chunk*>(val.ptr);
         delete chunk;
-        val.ptr = NULL;
+        val.ptr = nullptr;
         break;
     }
 
@@ -384,7 +382,7 @@ void CrawlStoreValue::unset(bool force)
 #define COPY_PTR(ptr_type) \
     { \
         ptr_type *ptr = static_cast<ptr_type*>(val.ptr); \
-        if (ptr != NULL) \
+        if (ptr != nullptr) \
             delete ptr; \
         ptr = static_cast<ptr_type*>(other.val.ptr); \
         val.ptr = (void*) new ptr_type (*ptr);  \
@@ -1237,14 +1235,14 @@ string &CrawlStoreValue::operator += (const string &_val)
 
 CrawlHashTable::CrawlHashTable()
 {
-    hash_map = NULL;
+    hash_map = nullptr;
 }
 
 CrawlHashTable::CrawlHashTable(const CrawlHashTable& other)
 {
-    if (other.hash_map == NULL)
+    if (other.hash_map == nullptr)
     {
-        hash_map = NULL;
+        hash_map = nullptr;
         return;
     }
 
@@ -1255,21 +1253,21 @@ CrawlHashTable::~CrawlHashTable()
 {
     // NOTE: Not using unique_ptr because making hash_map an unique_ptr
     // causes compile weirdness in externs.h
-    if (hash_map == NULL)
+    if (hash_map == nullptr)
         return;
 
     delete hash_map;
-    hash_map = NULL;
+    hash_map = nullptr;
 }
 
 CrawlHashTable &CrawlHashTable::operator = (const CrawlHashTable &other)
 {
-    if (hash_map != NULL)
+    if (hash_map != nullptr)
         delete hash_map;
 
-    if (other.hash_map == NULL)
+    if (other.hash_map == nullptr)
     {
-        hash_map = NULL;
+        hash_map = nullptr;
         return *this;
     }
 
@@ -1291,12 +1289,10 @@ void CrawlHashTable::write(writer &th) const
 
     marshallUnsigned(th, size());
 
-    CrawlHashTable::hash_map_type::const_iterator i = hash_map->begin();
-
-    for (; i != hash_map->end(); ++i)
+    for (const auto &entry : *hash_map)
     {
-        marshallString(th, i->first);
-        i->second.write(th);
+        marshallString(th, entry.first);
+        entry.second.write(th);
     }
 
     ASSERT_VALIDITY();
@@ -1346,32 +1342,28 @@ static map<string, int> accesses;
 
 bool CrawlHashTable::exists(const string &key) const
 {
-    if (hash_map == NULL)
+    if (!hash_map)
         return false;
 
     ACCESS(key);
     ASSERT_VALIDITY();
-    hash_map_type::const_iterator i = hash_map->find(key);
-
-    return i != hash_map->end();
+    return hash_map->find(key) != hash_map->end();
 }
 
 void CrawlHashTable::assert_validity() const
 {
 #ifdef DEBUG
-    if (hash_map == NULL)
+    if (hash_map == nullptr)
         return;
 
-    hash_map_type::const_iterator i = hash_map->begin();
-
     size_t actual_size = 0;
 
-    for (; i != hash_map->end(); ++i)
+    for (const auto &entry : *hash_map)
     {
         actual_size++;
 
-        const string          &key = i->first;
-        const CrawlStoreValue &val = i->second;
+        const string          &key = entry.first;
+        const CrawlStoreValue &val = entry.second;
 
         ASSERT(!key.empty());
         string trimmed = trimmed_string(key);
@@ -1387,12 +1379,12 @@ void CrawlHashTable::assert_validity() const
         case SV_ITEM:
         case SV_LEV_ID:
         case SV_LEV_POS:
-            ASSERT(val.val.ptr != NULL);
+            ASSERT(val.val.ptr != nullptr);
             break;
 
         case SV_HASH:
         {
-            ASSERT(val.val.ptr != NULL);
+            ASSERT(val.val.ptr != nullptr);
 
             CrawlHashTable* nested;
             nested = static_cast<CrawlHashTable*>(val.val.ptr);
@@ -1403,7 +1395,7 @@ void CrawlHashTable::assert_validity() const
 
         case SV_VEC:
         {
-            ASSERT(val.val.ptr != NULL);
+            ASSERT(val.val.ptr != nullptr);
 
             CrawlVector* nested;
             nested = static_cast<CrawlVector*>(val.val.ptr);
@@ -1430,45 +1422,31 @@ CrawlStoreValue& CrawlHashTable::get_value(const string &key)
     init_hash_map();
 
     ACCESS(key);
-    iterator i = hash_map->find(key);
-
-    if (i == hash_map->end())
-    {
-        (*hash_map)[key]     = CrawlStoreValue();
-        CrawlStoreValue &val = (*hash_map)[key];
-
-        return val;
-    }
-
-    return i->second;
+    // Inserts CrawlStoreValue() if the key was not found.
+    return (*hash_map)[key];
 }
 
 const CrawlStoreValue& CrawlHashTable::get_value(const string &key) const
 {
-#ifdef ASSERTS
-    if (!hash_map)
-        die("trying to read non-existent property \"%s\"", key.c_str());
-#endif
+    ASSERTM(hash_map,
+            "trying to read non-existent property \"%s\"", key.c_str());
     ASSERT_VALIDITY();
 
     ACCESS(key);
-    hash_map_type::const_iterator i = hash_map->find(key);
+    CrawlStoreValue *store = map_find(*hash_map, key);
 
-#ifdef ASSERTS
-    if (i == hash_map->end())
-        die("trying to read non-existent property \"%s\"", key.c_str());
-#endif
-    ASSERT(i->second.type != SV_NONE);
-    ASSERT(!(i->second.flags & SFLAG_UNSET));
+    ASSERTM(store, "trying to read non-existent property \"%s\"", key.c_str());
+    ASSERT(store->type != SV_NONE);
+    ASSERT(!(store->flags & SFLAG_UNSET));
 
-    return i->second;
+    return *store;
 }
 
 ///////////////////////////
 // std::map style interface
 unsigned int CrawlHashTable::size() const
 {
-    if (hash_map == NULL)
+    if (hash_map == nullptr)
         return 0;
 
     return hash_map->size();
@@ -1476,7 +1454,7 @@ unsigned int CrawlHashTable::size() const
 
 bool CrawlHashTable::empty() const
 {
-    if (hash_map == NULL)
+    if (hash_map == nullptr)
         return true;
 
     return hash_map->empty();
@@ -1504,11 +1482,11 @@ void CrawlHashTable::erase(const string key)
 void CrawlHashTable::clear()
 {
     ASSERT_VALIDITY();
-    if (hash_map == NULL)
+    if (hash_map == nullptr)
         return;
 
     delete hash_map;
-    hash_map = NULL;
+    hash_map = nullptr;
 }
 
 CrawlHashTable::iterator CrawlHashTable::begin()
@@ -1529,7 +1507,7 @@ CrawlHashTable::iterator CrawlHashTable::end()
 
 CrawlHashTable::const_iterator CrawlHashTable::begin() const
 {
-    ASSERT(hash_map != NULL);
+    ASSERT(hash_map != nullptr);
     ASSERT_VALIDITY();
 
     return hash_map->begin();
@@ -1537,7 +1515,7 @@ CrawlHashTable::const_iterator CrawlHashTable::begin() const
 
 CrawlHashTable::const_iterator CrawlHashTable::end() const
 {
-    ASSERT(hash_map != NULL);
+    ASSERT(hash_map != nullptr);
     ASSERT_VALIDITY();
 
     return hash_map->end();
@@ -1545,7 +1523,7 @@ CrawlHashTable::const_iterator CrawlHashTable::end() const
 
 void CrawlHashTable::init_hash_map()
 {
-    if (hash_map != NULL)
+    if (hash_map != nullptr)
         return;
 
     hash_map = new hash_map_type();
@@ -1591,8 +1569,8 @@ void CrawlVector::write(writer &th) const
         return;
     }
 
-    marshallUnsigned(th, (char) size());
-    marshallUnsigned(th, (char) max_size);
+    marshallUnsigned(th, size());
+    marshallUnsigned(th, max_size);
     marshallByte(th, static_cast<char>(type));
     marshallByte(th, (char) default_flags);
 
@@ -1618,6 +1596,11 @@ void CrawlVector::read(reader &th)
     vec_size _size;
     if (th.getMinorVersion() < TAG_MINOR_16_BIT_TABLE)
         _size = (vec_size) unmarshallByte(th);
+    else if (th.getMinorVersion() < TAG_MINOR_REALLY_16_BIT_VEC)
+    {
+        // Fix bad sign extensions.
+        _size = (vec_size) (unsigned char) unmarshallUnsigned(th);
+    }
     else
         _size = (vec_size) unmarshallUnsigned(th);
 #else
@@ -1630,9 +1613,21 @@ void CrawlVector::read(reader &th)
 #if TAG_MAJOR_VERSION == 34
     if (th.getMinorVersion() < TAG_MINOR_16_BIT_TABLE)
         max_size = static_cast<vec_size>(unmarshallByte(th));
+    else if (th.getMinorVersion() < TAG_MINOR_REALLY_16_BIT_VEC)
+    {
+        // Fix bad sign extensions.
+        max_size = (vec_size) (unsigned char) unmarshallUnsigned(th);
+    }
     else
 #endif
     max_size      = static_cast<vec_size>(unmarshallUnsigned(th));
+#if TAG_MAJOR_VERSION == 34
+    if (th.getMinorVersion() < TAG_MINOR_FIX_8_BIT_VEC_MAX
+        && max_size == 0xFF)
+    {
+        max_size = VEC_MAX_SIZE;
+    }
+#endif
     type          = static_cast<store_val_type>(unmarshallByte(th));
     default_flags = static_cast<store_flags>(unmarshallByte(th));
 
@@ -1707,12 +1702,12 @@ void CrawlVector::assert_validity() const
         case SV_ITEM:
         case SV_LEV_ID:
         case SV_LEV_POS:
-            ASSERT(val.val.ptr != NULL);
+            ASSERT(val.val.ptr != nullptr);
             break;
 
         case SV_HASH:
         {
-            ASSERT(val.val.ptr != NULL);
+            ASSERT(val.val.ptr != nullptr);
 
             CrawlHashTable* nested;
             nested = static_cast<CrawlHashTable*>(val.val.ptr);
@@ -1723,7 +1718,7 @@ void CrawlVector::assert_validity() const
 
         case SV_VEC:
         {
-            ASSERT(val.val.ptr != NULL);
+            ASSERT(val.val.ptr != nullptr);
 
             CrawlVector* nested;
             nested = static_cast<CrawlVector*>(val.val.ptr);
@@ -1811,12 +1806,12 @@ void CrawlVector::push_back(CrawlStoreValue val)
     case SV_ITEM:
     case SV_LEV_ID:
     case SV_LEV_POS:
-        ASSERT(val.val.ptr != NULL);
+        ASSERT(val.val.ptr != nullptr);
         break;
 
     case SV_HASH:
     {
-        ASSERT(val.val.ptr != NULL);
+        ASSERT(val.val.ptr != nullptr);
 
         CrawlHashTable* nested;
         nested = static_cast<CrawlHashTable*>(val.val.ptr);
@@ -1827,7 +1822,7 @@ void CrawlVector::push_back(CrawlStoreValue val)
 
     case SV_VEC:
     {
-        ASSERT(val.val.ptr != NULL);
+        ASSERT(val.val.ptr != nullptr);
 
         CrawlVector* nested;
         nested = static_cast<CrawlVector*>(val.val.ptr);
@@ -1947,18 +1942,12 @@ void dump_prop_accesses()
 
     vector<string> props;
 
-    for (map<string, int>::const_iterator i = accesses.begin();
-         i != accesses.end(); ++i)
-    {
-        props.push_back(i->first);
-    }
+    for (auto &entry : accesses)
+        props.push_back(entry.first);
 
     sort(props.begin(), props.end(), _cmp);
-    for (vector<string>::const_iterator i = props.begin();
-         i != props.end(); ++i)
-    {
-        fprintf(f, "%10d %s\n", accesses[*i], i->c_str());
-    }
+    for (const auto &prop : props)
+        fprintf(f, "%10d %s\n", accesses[prop], prop.c_str());
     fclose(f);
 }
 #endif
