@@ -10,7 +10,7 @@
 
 // Let CDO updaters know if the syntax changes.
 // Really, really, REALLY _never_ ever bump this and clean up old #ifdefs
-// in a single commit, please.  Making clean-up and actual code changes,
+// in a single commit, please. Making clean-up and actual code changes,
 // especially of this size, separated is vital for sanity.
 #ifndef TAG_MAJOR_VERSION
 #define TAG_MAJOR_VERSION 34
@@ -114,9 +114,74 @@ enum tag_minor_version
     TAG_MINOR_SAVE_TERRAIN_COLOUR, // Save colour in terrain-change markers.
     TAG_MINOR_REMOVE_BASE_MP,      // Remove base MP bonus.
     TAG_MINOR_METABOLISM,          // Remove random fast/slow meta mutations
+    TAG_MINOR_RU_SACRIFICES,       // Store Ru sacrifices in an array for coloration
+    TAG_MINOR_IS_UNDEAD,           // Remove the old "is_undead" player field
+    TAG_MINOR_REMOVE_MON_AC_EV,    // Remove the old "ac" & "ev" monster fields
+    TAG_MINOR_DISPLAY_MON_AC_EV,   // Marshall & unmarshall ac/ev in monster_info
+    TAG_MINOR_PLACE_UNPACK,        // Some packed places are level_ids.
+    TAG_MINOR_NO_JUMP,             // Removal of jump-attack.
+    TAG_MINOR_MONSTER_SPELL_SLOTS, // Introduce monster spell slot flags/freqs
+    TAG_MINOR_ARB_SPELL_SLOTS,     // Arbitrary number of monster spell slots.
+    TAG_MINOR_CUT_CUTLASSES,       // Turn crummy cutlasses into real rapiers.
+    TAG_MINOR_NO_GHOST_SPELLCASTER,// Remove an unused field in ghost_demon
+    TAG_MINOR_MID_BEHOLDERS,       // you.beholders and fearmongers store mids
+    TAG_MINOR_REMOVE_ITEM_COLOUR,  // don't store item colour as state
+    TAG_MINOR_CORPSE_CRASH,        // don't crash when loading corpses
+    TAG_MINOR_INIT_RND,            // initialize rnd in more places
+    TAG_MINOR_RING_PLUSSES,        // don't generate +144 rings
+    TAG_MINOR_BLESSED_WPNS,        // Remove blessed long blade base types
+    TAG_MINOR_MON_COLOUR_LOOKUP,   // monster colour lookup when possible
+    TAG_MINOR_CONSUM_APPEARANCE,   // Stop storing item appearance in .plus
+    TAG_MINOR_NEG_IDESC,           // Fix a sign conversion error
+    TAG_MINOR_GHOST_ENERGY,        // ghost_demon has move_energy field
+    TAG_MINOR_TENTACLE_MID,        // Use mids for tentacle code
+    TAG_MINOR_CORPSE_COLOUR,       // Fix corpses with invalid colour.
+    TAG_MINOR_MANGLE_CORPSES,      // Turn NEVER_HIDE corpses into MANGLED_CORPSEs
+    TAG_MINOR_ZOT_OPEN,            // Don't store whether you opened Zot
+    TAG_MINOR_EXPLORE_MODE,        // Store whether you are in explore mode
+    TAG_MINOR_RANDLICHES,          // Liches are now GHOST_DEMONs
+    TAG_MINOR_ISFLAG_HANDLED,      // Game tracks which items player has handled
+    TAG_MINOR_SHOP_HACK,           // The shop hack is dead!
+    TAG_MINOR_STACKABLE_EVOKERS,   // XP evokers stack
+    TAG_MINOR_REALLY_16_BIT_VEC,   // CrawlVector size really saved as 16-bit
+    TAG_MINOR_FIX_8_BIT_VEC_MAX,   // Fix up bad CrawlVector max size
+    TAG_MINOR_TRACK_BANISHER,      // Persist the name of the last thing that banished the player
+    TAG_MINOR_SHOALS_LITE,         // Remove deep water from old Shoals
+    TAG_MINOR_FIX_EXPLORE_MODE,    // Fix char selection screen on old versions
+    TAG_MINOR_UNSTACKABLE_EVOKERS, // XP evokers no longer stack
+    TAG_MINOR_NO_NEGATIVE_VULN,    // Remove negative energy vulnerability
+    TAG_MINOR_MAX_XL,              // Store max XL instead of hardcoding it
+    TAG_MINOR_NO_RPOIS_MINUS,      // Remove rPois- artefacts
+    TAG_MINOR_XP_PENANCE,          // Let gods other than Ash use xp penance
+    TAG_MINOR_SPIT_POISON,         // Give Nagas MUT_SPIT_POISON
+    TAG_MINOR_REAL_MUTS,           // Give some species proper mutations
+    TAG_MINOR_NO_FORLORN,          // Remove Forlorn mutation
+    TAG_MINOR_MP_WANDS,            // Make MP wands a single-level mutation
+    TAG_MINOR_TELEPORTITIS,        // Rescale teleportitis on artefacts
+    TAG_MINOR_ROTTING,             // Remove rot-over-time
+    TAG_MINOR_STAT_ZERO_DURATION,  // Stat zero uses a duration
+    TAG_MINOR_INT_REGEN,           // hp/mp regen are ints
+    TAG_MINOR_NAGA_METABOLISM,     // nagas have slow metabolism
+    TAG_MINOR_BOOL_FLIGHT,         // flight is just a bool
+    TAG_MINOR_STAT_LOSS_XP,        // Stat loss recovers with XP
+    TAG_MINOR_DETERIORATION,       // 2-level deterioration mutation
+    TAG_MINOR_RU_DELAY_STACKING,   // Let Ru delay timers stack again
+    TAG_MINOR_NO_TWISTER,          // Remove ARTP_TWISTER
+    TAG_MINOR_NO_ZOTDEF,           // remove zotdef, along with zot_points and zotdef_wave_name
+    TAG_MINOR_SAVED_PIETY,         // allowed good-god piety to survive through an atheist period
+    TAG_MINOR_GHOST_SINV,          // marshall ghost_demon sinv
+    TAG_MINOR_ID_STATES,           // turn item_type_id_state_type into a bool
+    TAG_MINOR_MON_HD_INFO,         // store player-known monster HD info
+    TAG_MINOR_NO_LEVEL_FLAGS,      // remove a field of env
+    TAG_MINOR_EXORCISE,            // liches, a. liches, & spellforged servitors are no longer ghost_demons
+    TAG_MINOR_BLINK_MUT,           // 1-level blink mutation
+    TAG_MINOR_RUNE_TYPE,           // runes became a base type
 #endif
     NUM_TAG_MINORS,
     TAG_MINOR_VERSION = NUM_TAG_MINORS - 1
 };
 
+// Marshalled as a byte in several places.
+COMPILE_CHECK(TAG_MINOR_VERSION <= 0xff);
+
 #endif
