@@ -2,20 +2,20 @@
 
 #include "target.h"
 
-#include "beam.h"
+#include <cmath>
+#include <utility> // swap
+
 #include "coord.h"
 #include "coordit.h"
+#include "english.h"
 #include "env.h"
 #include "fight.h"
 #include "libutil.h"
 #include "los_def.h"
 #include "losglobal.h"
-#include "player.h"
 #include "spl-damage.h"
 #include "terrain.h"
 
-#include <math.h>
-
 #define notify_fail(x) (why_not = (x), false)
 
 static string _wallmsg(coord_def c)
@@ -59,8 +59,9 @@ bool targetter::has_additional_sites(coord_def loc)
     return false;
 }
 
-targetter_beam::targetter_beam(const actor *act, int range, zap_type zap,
+targetter_beam::targetter_beam(const actor *act, int r, zap_type zap,
                                int pow, int min_ex_rad, int max_ex_rad) :
+                               range(r),
                                min_expl_rad(min_ex_rad),
                                max_expl_rad(max_ex_rad)
 {
@@ -69,7 +70,7 @@ targetter_beam::targetter_beam(const actor *act, int range, zap_type zap,
     ASSERT(max_ex_rad >= 0);
     ASSERT(max_ex_rad >= min_ex_rad);
     agent = act;
-    beam.set_agent(const_cast<actor *>(act));
+    beam.set_agent(act);
     origin = aim = act->pos();
     beam.attitude = ATT_FRIENDLY;
     zappy(zap, pow, beam);
@@ -84,8 +85,7 @@ targetter_beam::targetter_beam(const actor *act, int range, zap_type zap,
     beam.ex_size = min_ex_rad;
     beam.aimed_at_spot = true;
 
-    penetrates_targets = beam.is_beam;
-    range2 = dist_range(range);
+    penetrates_targets = beam.pierce;
 }
 
 bool targetter_beam::set_aim(coord_def a)
@@ -104,13 +104,12 @@ bool targetter_beam::set_aim(coord_def a)
     {
         bolt tempbeam2 = beam;
         tempbeam2.target = origin;
-        for (vector<coord_def>::const_iterator i = path_taken.begin();
-             i != path_taken.end(); ++i)
+        for (auto c : path_taken)
         {
-            if (cell_is_solid(*i) && tempbeam.affects_wall(grd(*i)) != MB_TRUE)
+            if (cell_is_solid(c) && !tempbeam.can_affect_wall(grd(c)))
                 break;
-            tempbeam2.target = *i;
-            if (anyone_there(*i) && !tempbeam.ignores_monster(monster_at(*i)))
+            tempbeam2.target = c;
+            if (anyone_there(c) && !tempbeam.ignores_monster(monster_at(c)))
                 break;
         }
         tempbeam2.use_target_as_pos = true;
@@ -132,7 +131,7 @@ bool targetter_beam::valid_aim(coord_def a)
             return notify_fail("There's something in the way.");
         return notify_fail("You cannot see that place.");
     }
-    if ((origin - a).abs() > range2)
+    if ((origin - a).rdist() > range)
         return notify_fail("Out of range.");
     return true;
 }
@@ -148,28 +147,25 @@ aff_type targetter_beam::is_affected(coord_def loc)
     bool on_path = false;
     coord_def c;
     aff_type current = AFF_YES;
-    for (vector<coord_def>::const_iterator i = path_taken.begin();
-         i != path_taken.end(); ++i)
+    for (auto pc : path_taken)
     {
-        if (cell_is_solid(*i)
-            && beam.affects_wall(grd(*i)) != MB_TRUE
+        if (cell_is_solid(pc)
+            && !beam.can_affect_wall(grd(pc))
             && max_expl_rad > 0)
         {
             break;
         }
 
-        c = *i;
+        c = pc;
         if (c == loc)
         {
             if (max_expl_rad > 0)
                 on_path = true;
-            else if (cell_is_solid(*i))
+            else if (cell_is_solid(pc))
             {
-                maybe_bool res = beam.affects_wall(grd(*i));
-                if (res == MB_TRUE)
+                bool res = beam.can_affect_wall(grd(pc));
+                if (res)
                     return current;
-                else if (res == MB_MAYBE)
-                    return AFF_MAYBE;
                 else
                     return AFF_NO;
 
@@ -177,9 +173,9 @@ aff_type targetter_beam::is_affected(coord_def loc)
             else
                 return current;
         }
-        if (anyone_there(*i)
+        if (anyone_there(pc)
             && !penetrates_targets
-            && !beam.ignores_monster(monster_at(*i)))
+            && !beam.ignores_monster(monster_at(pc)))
         {
             // We assume an exploding spell will always stop here.
             if (max_expl_rad > 0)
@@ -189,13 +185,13 @@ aff_type targetter_beam::is_affected(coord_def loc)
     }
     if (max_expl_rad > 0 && (loc - c).rdist() <= 9)
     {
-        maybe_bool aff_wall = beam.affects_wall(grd(loc));
-        if (!cell_is_solid(loc) || aff_wall != MB_FALSE)
+        bool aff_wall = beam.can_affect_wall(grd(loc));
+        if (!cell_is_solid(loc) || aff_wall)
         {
             coord_def centre(9,9);
             if (exp_map_min(loc - c + centre) < INT_MAX)
             {
-                return (!cell_is_solid(loc) || aff_wall == MB_TRUE)
+                return (!cell_is_solid(loc) || aff_wall)
                        ? AFF_YES : AFF_MAYBE;
             }
             if (exp_map_max(loc - c + centre) < INT_MAX)
@@ -205,8 +201,8 @@ aff_type targetter_beam::is_affected(coord_def loc)
     return on_path ? AFF_TRACER : AFF_NO;
 }
 
-targetter_imb::targetter_imb(const actor *act, int pow, int range) :
-               targetter_beam(act, range, ZAP_ISKENDERUNS_MYSTIC_BLAST, pow, 0, 0)
+targetter_imb::targetter_imb(const actor *act, int pow, int r) :
+               targetter_beam(act, r, ZAP_ISKENDERUNS_MYSTIC_BLAST, pow, 0, 0)
 {
 }
 
@@ -226,13 +222,10 @@ bool targetter_imb::set_aim(coord_def a)
     if (end == origin)
         return true;
 
-    coord_def c;
     bool first = true;
 
-    for (vector<coord_def>::iterator i = path_taken.begin();
-         i != path_taken.end(); i++)
+    for (auto c : path_taken)
     {
-        c = *i;
         cur_path.push_back(c);
         if (!(anyone_there(c)
               && !beam.ignores_monster((monster_at(c))))
@@ -269,18 +262,14 @@ aff_type targetter_imb::is_affected(coord_def loc)
     if (from_path != AFF_NO)
         return from_path;
 
-    for (vector<coord_def>::const_iterator i = splash.begin();
-         i != splash.end(); ++i)
-    {
-        if (*i == loc)
-            return cell_is_solid(*i) ? AFF_NO : AFF_MAYBE;
-    }
-    for (vector<coord_def>::const_iterator i = splash2.begin();
-         i != splash2.end(); ++i)
-    {
-        if (*i == loc)
-            return cell_is_solid(*i) ? AFF_NO : AFF_TRACER;
-    }
+    for (auto c : splash)
+        if (c == loc)
+            return cell_is_solid(c) ? AFF_NO : AFF_MAYBE;
+
+    for (auto c : splash2)
+        if (c == loc)
+            return cell_is_solid(c) ? AFF_NO : AFF_TRACER;
+
     return AFF_NO;
 }
 
@@ -305,8 +294,8 @@ aff_type targetter_view::is_affected(coord_def loc)
 targetter_smite::targetter_smite(const actor* act, int ran,
                                  int exp_min, int exp_max, bool wall_ok,
                                  bool (*affects_pos_func)(const coord_def &)):
-    exp_range_min(exp_min), exp_range_max(exp_max), affects_walls(wall_ok),
-    affects_pos(affects_pos_func)
+    exp_range_min(exp_min), exp_range_max(exp_max), range(ran),
+    affects_walls(wall_ok), affects_pos(affects_pos_func)
 {
     ASSERT(act);
     ASSERT(exp_min >= 0);
@@ -314,7 +303,6 @@ targetter_smite::targetter_smite(const actor* act, int ran,
     ASSERT(exp_min <= exp_max);
     agent = act;
     origin = aim = act->pos();
-    range2 = dist_range(ran);
 }
 
 bool targetter_smite::valid_aim(coord_def a)
@@ -326,7 +314,7 @@ bool targetter_smite::valid_aim(coord_def a)
             return notify_fail("There's something in the way.");
         return notify_fail("You cannot see that place.");
     }
-    if ((origin - a).abs() > range2)
+    if ((origin - a).rdist() > range)
         return notify_fail("Out of range.");
     if (!affects_walls && cell_is_solid(a))
         return notify_fail(_wallmsg(a));
@@ -386,7 +374,7 @@ aff_type targetter_smite::is_affected(coord_def loc)
 }
 
 targetter_fragment::targetter_fragment(const actor* act, int power, int ran) :
-    targetter_smite(act, ran, 1, 1, true, NULL),
+    targetter_smite(act, ran, 1, 1, true, nullptr),
     pow(power)
 {
 }
@@ -399,7 +387,7 @@ bool targetter_fragment::valid_aim(coord_def a)
     bolt tempbeam;
     bool temp;
     if (!setup_fragmentation_beam(tempbeam, pow, agent, a, false,
-                                  true, true, NULL, temp, temp))
+                                  true, true, nullptr, temp, temp))
     {
         return notify_fail("You cannot affect that.");
     }
@@ -415,11 +403,11 @@ bool targetter_fragment::set_aim(coord_def a)
     bool temp;
 
     if (setup_fragmentation_beam(tempbeam, pow, agent, a, false,
-                                 false, true, NULL, temp, temp))
+                                 false, true, nullptr, temp, temp))
     {
         exp_range_min = tempbeam.ex_size;
         setup_fragmentation_beam(tempbeam, pow, agent, a, false,
-                                 true, true, NULL, temp, temp);
+                                 true, true, nullptr, temp, temp);
         exp_range_max = tempbeam.ex_size;
     }
     else
@@ -462,7 +450,7 @@ bool targetter_reach::valid_aim(coord_def a)
     if (!agent->see_cell_no_trans(a))
         return notify_fail("You can't get through.");
 
-    int dist = (origin - a).abs();
+    int dist = (origin - a).rdist();
 
     if (dist > range)
         return notify_fail("You can't reach that far!");
@@ -494,7 +482,7 @@ targetter_cleave::targetter_cleave(const actor* act, coord_def target)
     origin = act->pos();
     aim = target;
     list<actor*> act_targets;
-    get_all_cleave_targets(act, target, act_targets);
+    get_cleave_targets(*act, target, act_targets);
     while (!act_targets.empty())
     {
         targets.insert(act_targets.front()->pos());
@@ -507,16 +495,16 @@ aff_type targetter_cleave::is_affected(coord_def loc)
     return targets.count(loc) ? AFF_YES : AFF_NO;
 }
 
-targetter_cloud::targetter_cloud(const actor* act, int range,
+targetter_cloud::targetter_cloud(const actor* act, int r,
                                  int count_min, int count_max) :
-    cnt_min(count_min), cnt_max(count_max), avoid_clouds(true)
+    range(r), cnt_min(count_min), cnt_max(count_max), avoid_clouds(true)
 {
     ASSERT(cnt_min > 0);
     ASSERT(cnt_max > 0);
     ASSERT(cnt_min <= cnt_max);
-    if (agent = act)
+    agent = act;
+    if (agent)
         origin = aim = act->pos();
-    range2 = dist_range(range);
 }
 
 static bool _cloudable(coord_def loc, bool avoid_clouds)
@@ -528,7 +516,7 @@ static bool _cloudable(coord_def loc, bool avoid_clouds)
 
 bool targetter_cloud::valid_aim(coord_def a)
 {
-    if (agent && (origin - a).abs() > range2)
+    if (agent && (origin - a).rdist() > range)
         return notify_fail("Out of range.");
     if (!map_bounds(a)
         || agent
@@ -558,23 +546,21 @@ bool targetter_cloud::set_aim(coord_def a)
 
     seen.clear();
     queue.clear();
-    queue.push_back(vector<coord_def>());
+    queue.emplace_back();
 
     int placed = 0;
     queue[0].push_back(a);
 
     for (unsigned int d1 = 0; d1 < queue.size() && placed < cnt_max; d1++)
     {
-        unsigned int to_place = queue[d1].size();
-        placed += to_place;
+        placed += queue[d1].size();
 
-        for (unsigned int i = 0; i < to_place; i++)
+        for (coord_def c : queue[d1])
         {
-            coord_def c = queue[d1][i];
             for (adjacent_iterator ai(c); ai; ++ai)
                 if (_cloudable(*ai, avoid_clouds) && !seen.count(*ai))
                 {
-                    unsigned int d2 = d1 + ((*ai - c).abs() == 1 ? 5 : 7);
+                    unsigned int d2 = d1 + 1;
                     if (d2 >= queue.size())
                         queue.resize(d2 + 1);
                     queue[d2].push_back(*ai);
@@ -598,11 +584,12 @@ aff_type targetter_cloud::is_affected(coord_def loc)
     if (!valid_aim(aim))
         return AFF_NO;
 
-    map<coord_def, aff_type>::const_iterator it = seen.find(loc);
-    if (it == seen.end() || it->second <= 0) // AFF_TRACER is used privately
-        return AFF_NO;
-
-    return it->second;
+    if (aff_type *aff = map_find(seen, loc))
+    {
+        if (*aff > 0) // AFF_TRACER is used privately
+            return *aff;
+    }
+    return AFF_NO;
 }
 
 targetter_splash::targetter_splash(const actor* act)
@@ -646,24 +633,23 @@ aff_type targetter_splash::is_affected(coord_def loc)
 }
 
 targetter_los::targetter_los(const actor *act, los_type _los,
-                             int range, int range_max)
+                             int ran, int ran_max):
+    range(ran), range_max(ran_max)
 {
     ASSERT(act);
     agent = act;
     origin = aim = act->pos();
     los = _los;
-    range2 = range * range + 1;
     if (!range_max)
         range_max = range;
     ASSERT(range_max >= range);
-    range_max2 = range_max * range_max + 1;
 }
 
 bool targetter_los::valid_aim(coord_def a)
 {
-    if ((a - origin).abs() > range_max2)
+    if ((a - origin).rdist() > range_max)
         return notify_fail("Out of range.");
-    // If this message ever becomes used, please improve it.  I did not
+    // If this message ever becomes used, please improve it. I did not
     // bother adding complexity just for monsters and "hit allies" prompts
     // which don't need it.
     if (!is_affected(a))
@@ -676,13 +662,13 @@ aff_type targetter_los::is_affected(coord_def loc)
     if (loc == aim)
         return AFF_YES;
 
-    if ((loc - origin).abs() > range_max2)
+    if ((loc - origin).rdist() > range_max)
         return AFF_NO;
 
     if (!cell_see_cell(loc, origin, los))
         return AFF_NO;
 
-    return (loc - origin).abs() > range_max2 ? AFF_MAYBE : AFF_YES;
+    return (loc - origin).rdist() > range_max ? AFF_MAYBE : AFF_YES;
 }
 
 targetter_thunderbolt::targetter_thunderbolt(const actor *act, int r,
@@ -694,7 +680,7 @@ targetter_thunderbolt::targetter_thunderbolt(const actor *act, int r,
     prev = _prev;
     aim = prev.origin() ? origin : prev;
     ASSERT_RANGE(r, 1 + 1, you.current_vision + 1);
-    range2 = sqr(r) + 1;
+    range = r;
 }
 
 bool targetter_thunderbolt::valid_aim(coord_def a)
@@ -706,7 +692,7 @@ bool targetter_thunderbolt::valid_aim(coord_def a)
             return notify_fail("There's something in the way.");
         return notify_fail("You cannot see that place.");
     }
-    if ((origin - a).abs() > range2)
+    if ((origin - a).rdist() > range)
         return notify_fail("Out of range.");
     return true;
 }
@@ -740,14 +726,14 @@ bool targetter_thunderbolt::set_aim(coord_def a)
     // For consistency with beams, we need to
     _make_ray(ray, origin, aim);
     bool hit = true;
-    while ((origin - (p = ray.pos())).abs() <= range2)
+    while ((origin - (p = ray.pos())).rdist() <= range)
     {
         if (!map_bounds(p) || opc_solid_see(p) >= OPC_OPAQUE)
             hit = false;
         if (hit && p != origin && zapped[p] <= 0)
         {
             zapped[p] = AFF_YES;
-            arc_length[origin.range(p)]++;
+            arc_length[origin.distance_from(p)]++;
         }
         ray.advance();
     }
@@ -757,14 +743,14 @@ bool targetter_thunderbolt::set_aim(coord_def a)
 
     _make_ray(ray, origin, prev);
     hit = true;
-    while ((origin - (p = ray.pos())).abs() <= range2)
+    while ((origin - (p = ray.pos())).rdist() <= range)
     {
         if (!map_bounds(p) || opc_solid_see(p) >= OPC_OPAQUE)
             hit = false;
         if (hit && p != origin && zapped[p] <= 0)
         {
             zapped[p] = AFF_MAYBE; // fully affected, we just want to highlight cur
-            arc_length[origin.range(p)]++;
+            arc_length[origin.distance_from(p)]++;
         }
         ray.advance();
     }
@@ -772,19 +758,19 @@ bool targetter_thunderbolt::set_aim(coord_def a)
     coord_def a1 = prev - origin;
     coord_def a2 = aim - origin;
     if (left_of(a2, a1))
-        swapv(a1, a2);
+        swap(a1, a2);
 
     for (int x = -LOS_RADIUS; x <= LOS_RADIUS; ++x)
         for (int y = -LOS_RADIUS; y <= LOS_RADIUS; ++y)
         {
-            if (sqr(x) + sqr(y) > range2)
+            if (max(abs(x), abs(y)) > range)
                 continue;
             coord_def r(x, y);
             if (left_of(a1, r) && left_of(r, a2))
             {
                 (p = r) += origin;
                 if (!zapped.count(p))
-                    arc_length[r.range()]++;
+                    arc_length[r.rdist()]++;
                 if (zapped[p] <= 0 && cell_see_cell(origin, p, LOS_NO_TRANS))
                     zapped[p] = AFF_MAYBE;
             }
@@ -800,7 +786,7 @@ aff_type targetter_thunderbolt::is_affected(coord_def loc)
     if (loc == aim)
         return zapped[loc] ? AFF_YES : AFF_TRACER;
 
-    if ((loc - origin).abs() > range2)
+    if ((loc - origin).rdist() > range)
         return AFF_NO;
 
     return zapped[loc];
@@ -812,7 +798,6 @@ targetter_spray::targetter_spray(const actor* act, int range, zap_type zap)
     agent = act;
     origin = aim = act->pos();
     _range = range;
-    range2 = dist_range(range);
 }
 
 bool targetter_spray::valid_aim(coord_def a)
@@ -823,7 +808,7 @@ bool targetter_spray::valid_aim(coord_def a)
             return notify_fail("There's something in the way.");
         return notify_fail("You cannot see that place.");
     }
-    if ((origin - a).abs() > range2)
+    if ((origin - a).rdist() > _range)
         return notify_fail("Out of range.");
     return true;
 }
@@ -839,28 +824,25 @@ bool targetter_spray::set_aim(coord_def a)
     beams = get_spray_rays(agent, aim, _range, 3);
 
     paths_taken.clear();
-    for (unsigned int i = 0; i < beams.size(); ++i)
-        paths_taken.push_back(beams[i].path_taken);
+    for (const bolt &beam : beams)
+        paths_taken.push_back(beam.path_taken);
 
     return true;
 }
 
 aff_type targetter_spray::is_affected(coord_def loc)
 {
-    coord_def c;
     aff_type affected = AFF_NO;
 
     for (unsigned int n = 0; n < paths_taken.size(); ++n)
     {
         aff_type beam_affect = AFF_YES;
         bool beam_reached = false;
-        for (vector<coord_def>::const_iterator i = paths_taken[n].begin();
-         i != paths_taken[n].end(); ++i)
+        for (auto c : paths_taken[n])
         {
-            c = *i;
             if (c == loc)
             {
-                if (cell_is_solid(*i))
+                if (cell_is_solid(c))
                     beam_affect = AFF_NO;
                 else if (beam_affect != AFF_MAYBE)
                     beam_affect = AFF_YES;
@@ -868,8 +850,8 @@ aff_type targetter_spray::is_affected(coord_def loc)
                 beam_reached = true;
                 break;
             }
-            else if (anyone_there(*i)
-                && !beams[n].ignores_monster(monster_at(*i)))
+            else if (anyone_there(c)
+                     && !beams[n].ignores_monster(monster_at(c)))
             {
                 beam_affect = AFF_MAYBE;
             }
@@ -882,24 +864,23 @@ aff_type targetter_spray::is_affected(coord_def loc)
     return affected;
 }
 
-targetter_jump::targetter_jump(const actor* act, int r2, bool cp,
-                               bool imm) :
-    range2(r2), clear_path(cp), immobile(imm)
+targetter_shadow_step::targetter_shadow_step(const actor* act, int r) :
+    range(r)
 {
     ASSERT(act);
     agent = act;
     origin = act->pos();
-    jump_is_blocked = false;
+    step_is_blocked = false;
 }
 
-bool targetter_jump::valid_aim(coord_def a)
+bool targetter_shadow_step::valid_aim(coord_def a)
 {
-    coord_def c, jump_pos;
+    coord_def c, shadow_step_pos;
     ray_def ray;
 
     if (origin == a)
         return notify_fail("You cannot target yourself.");
-    else if ((origin - a).abs() > range2)
+    else if ((origin - a).rdist() > range)
         return notify_fail("Out of range.");
     else if (!cell_see_cell(origin, a, LOS_NO_TRANS))
     {
@@ -916,10 +897,6 @@ bool targetter_jump::valid_aim(coord_def a)
     {
         switch (no_landing_reason)
         {
-        case BLOCKED_FLYING:
-            return notify_fail("A flying creature is in the way.");
-        case BLOCKED_GIANT:
-            return notify_fail("A giant creature is in the way.");
         case BLOCKED_MOVE:
         case BLOCKED_OCCUPIED:
             return notify_fail("There is no safe place near that"
@@ -937,7 +914,7 @@ bool targetter_jump::valid_aim(coord_def a)
     return true;
 }
 
-bool targetter_jump::valid_landing(coord_def a, bool check_invis)
+bool targetter_shadow_step::valid_landing(coord_def a, bool check_invis)
 {
     actor *act;
     ray_def ray;
@@ -978,36 +955,19 @@ bool targetter_jump::valid_landing(coord_def a, bool check_invis)
         act = actor_at(ray.pos());
         if (ray.pos() == a)
         {
-            if (act && (!check_invis || agent->can_see(act)))
+            if (act && (!check_invis || agent->can_see(*act)))
             {
                 blocked_landing_reason = BLOCKED_OCCUPIED;
                 return false;
             }
             break;
         }
-        const dungeon_feature_type grid = grd(ray.pos());
-        if (clear_path && act && (!check_invis || agent->can_see(act)))
-        {
-            // Can't jump over airborn enemies nor giant enemies not in deep
-            // water or lava.
-            if (act->airborne())
-            {
-                blocked_landing_reason = BLOCKED_FLYING;
-                return false;
-            }
-            else if (act->body_size() == SIZE_GIANT
-                     && grid != DNGN_DEEP_WATER && grid != DNGN_LAVA)
-            {
-                blocked_landing_reason = BLOCKED_GIANT;
-                return false;
-            }
-        }
         ray.advance();
     }
     return true;
 }
 
-aff_type targetter_jump::is_affected(coord_def loc)
+aff_type targetter_shadow_step::is_affected(coord_def loc)
 {
     aff_type aff = AFF_NO;
 
@@ -1018,75 +978,69 @@ aff_type targetter_jump::is_affected(coord_def loc)
     return aff;
 }
 
-// If something unseen either occupies the aim position or blocks the jump path,
-// indicate that with jump_is_blocked, but still return true so long there is at
+// If something unseen either occupies the aim position or blocks the shadow_step path,
+// indicate that with step_is_blocked, but still return true so long there is at
 // least one valid landing position from the player's perspective.
-bool targetter_jump::set_aim(coord_def a)
+bool targetter_shadow_step::set_aim(coord_def a)
 {
-    set<coord_def>::const_iterator site;
-
     if (a == origin)
         return false;
     if (!targetter::set_aim(a))
         return false;
 
-    jump_is_blocked = false;
+    step_is_blocked = false;
 
     // Find our set of landing sites, choose one at random to be the destination
     // and see if it's actually blocked.
     set_additional_sites(aim);
     if (additional_sites.size())
     {
-        int site_ind = random2(additional_sites.size());
-        for (site = additional_sites.begin(); site_ind > 0; site++)
-            site_ind--;
-        landing_site = *site;
+        auto it = random_iterator(additional_sites);
+        landing_site = *it;
         if (!valid_landing(landing_site, false))
-            jump_is_blocked = true;
+            step_is_blocked = true;
         return true;
     }
     return false;
 }
 
 // Determine the set of valid landing sites
-void targetter_jump::set_additional_sites(coord_def a)
+void targetter_shadow_step::set_additional_sites(coord_def a)
 {
      get_additional_sites(a);
      additional_sites = temp_sites;
 }
 
 // Determine the set of valid landing sites for the target, putting the results
-// in the private set variable temp_sites.  This uses valid_aim(), so it looks
+// in the private set variable temp_sites. This uses valid_aim(), so it looks
 // for uninhabited squares that are habitable by the player, but doesn't check
 // against e.g. harmful clouds
-void targetter_jump::get_additional_sites(coord_def a)
+void targetter_shadow_step::get_additional_sites(coord_def a)
 {
     bool agent_adjacent = a.distance_from(agent->pos()) == 1;
     temp_sites.clear();
 
-    if (immobile)
+    const actor *victim = actor_at(a);
+    if (!victim || !victim->as_monster()
+        || mons_is_firewood(victim->as_monster()) || victim->invisible()
+        || !victim->umbraed())
     {
-        const actor *victim = actor_at(a);
-        if (!victim || victim->invisible() || !victim->umbraed())
-        {
-            no_landing_reason = BLOCKED_NO_TARGET;
-            return;
-        }
-        if (!victim->is_stationary()
-            && !victim->cannot_move()
-            && !victim->asleep())
-        {
-            no_landing_reason = BLOCKED_MOBILE;
-            return;
-        }
+        no_landing_reason = BLOCKED_NO_TARGET;
+        return;
+    }
+    if (!victim->is_stationary()
+        && !victim->cannot_move()
+        && !victim->asleep())
+    {
+        no_landing_reason = BLOCKED_MOBILE;
+        return;
     }
 
     no_landing_reason = BLOCKED_NONE;
     for (adjacent_iterator ai(a, false); ai; ++ai)
     {
         // See if site is valid, record a putative reason for why no sites were
-        // found.  A flying or giant monster blocking the landing site gets
-        // priority as an reason, since it's very jump-specific.
+        // found.
         if (!agent_adjacent || agent->pos().distance_from(*ai) > 1)
         {
             if (valid_landing(*ai))
@@ -1094,24 +1048,21 @@ void targetter_jump::get_additional_sites(coord_def a)
                 temp_sites.insert(*ai);
                 no_landing_reason = BLOCKED_NONE;
             }
-            else if (no_landing_reason != BLOCKED_FLYING
-                     && no_landing_reason != BLOCKED_GIANT)
-            {
+            else
                 no_landing_reason = blocked_landing_reason;
-            }
         }
     }
 }
 
 // See if we can find at least one valid landing position for the given monster.
-bool targetter_jump::has_additional_sites(coord_def a)
+bool targetter_shadow_step::has_additional_sites(coord_def a)
 {
     get_additional_sites(a);
     return temp_sites.size();
 }
 
-targetter_explosive_bolt::targetter_explosive_bolt(const actor *act, int pow, int range) :
-                          targetter_beam(act, range, ZAP_EXPLOSIVE_BOLT, pow, 0, 0)
+targetter_explosive_bolt::targetter_explosive_bolt(const actor *act, int pow, int r) :
+                          targetter_beam(act, r, ZAP_EXPLOSIVE_BOLT, pow, 0, 0)
 {
 }
 
@@ -1122,14 +1073,13 @@ bool targetter_explosive_bolt::set_aim(coord_def a)
 
     bolt tempbeam = beam;
     tempbeam.target = origin;
-    for (vector<coord_def>::const_iterator i = path_taken.begin();
-         i != path_taken.end(); ++i)
+    for (auto c : path_taken)
     {
-        if (cell_is_solid(*i))
+        if (cell_is_solid(c))
             break;
 
-        tempbeam.target = *i;
-        if (anyone_there(*i))
+        tempbeam.target = c;
+        if (anyone_there(c))
         {
             tempbeam.use_target_as_pos = true;
             exp_map.init(INT_MAX);
@@ -1144,17 +1094,13 @@ bool targetter_explosive_bolt::set_aim(coord_def a)
 aff_type targetter_explosive_bolt::is_affected(coord_def loc)
 {
     bool on_path = false;
-    coord_def c;
-    for (vector<coord_def>::const_iterator i = path_taken.begin();
-         i != path_taken.end(); ++i)
+    for (auto c : path_taken)
     {
-        if (cell_is_solid(*i))
+        if (cell_is_solid(c))
             break;
-
-        c = *i;
         if (c == loc)
             on_path = true;
-        if (anyone_there(*i)
+        if (anyone_there(c)
             && !beam.ignores_monster(monster_at(c))
             && (loc - c).rdist() <= 9)
         {
@@ -1170,14 +1116,14 @@ aff_type targetter_explosive_bolt::is_affected(coord_def loc)
     return on_path ? AFF_TRACER : AFF_NO;
 }
 
-targetter_cone::targetter_cone(const actor *act, int range)
+targetter_cone::targetter_cone(const actor *act, int r)
 {
     ASSERT(act);
     agent = act;
     origin = act->pos();
     aim = origin;
-    ASSERT_RANGE(range, 1 + 1, you.current_vision + 1);
-    range2 = sqr(range) + 1;
+    ASSERT_RANGE(r, 1 + 1, you.current_vision + 1);
+    range = r;
 }
 
 bool targetter_cone::valid_aim(coord_def a)
@@ -1189,7 +1135,7 @@ bool targetter_cone::valid_aim(coord_def a)
             return notify_fail("There's something in the way.");
         return notify_fail("You cannot see that place.");
     }
-    if ((origin - a).abs() > range2)
+    if ((origin - a).rdist() > range)
         return notify_fail("Out of range.");
     return true;
 }
@@ -1223,12 +1169,12 @@ bool targetter_cone::set_aim(coord_def a)
     coord_def a1 = l - origin;
     coord_def a2 = r - origin;
     if (left_of(a2, a1))
-        swapv(a1, a2);
+        swap(a1, a2);
 
     for (int x = -LOS_RADIUS; x <= LOS_RADIUS; ++x)
         for (int y = -LOS_RADIUS; y <= LOS_RADIUS; ++y)
         {
-            if (sqr(x) + sqr(y) > range2)
+            if (max(abs(x), abs(y)) > range)
                 continue;
             coord_def q(x, y);
             if (left_of_eq(a1, q) && left_of_eq(q, a2))
@@ -1240,7 +1186,7 @@ bool targetter_cone::set_aim(coord_def a)
                     && cell_see_cell(origin, p, LOS_NO_TRANS))
                 {
                     zapped[p] = AFF_YES;
-                    sweep[isqrt((origin - p).abs())][p] = AFF_YES;
+                    sweep[(origin - p).rdist()][p] = AFF_YES;
                 }
             }
         }
@@ -1256,18 +1202,22 @@ aff_type targetter_cone::is_affected(coord_def loc)
     if (loc == aim)
         return zapped[loc] ? AFF_YES : AFF_TRACER;
 
-    if ((loc - origin).abs() > range2)
+    if ((loc - origin).rdist() > range)
         return AFF_NO;
 
     return zapped[loc];
 }
 
-targetter_shotgun::targetter_shotgun(const actor* act, int range)
+targetter_shotgun::targetter_shotgun(const actor* act, size_t beam_count,
+                                     int r)
 {
     ASSERT(act);
     agent = act;
     origin = act->pos();
-    range2 = dist_range(range);
+    num_beams = beam_count;
+    for (size_t i = 0; i < num_beams; i++)
+        rays.emplace_back();
+    range = r;
 }
 
 bool targetter_shotgun::valid_aim(coord_def a)
@@ -1278,7 +1228,7 @@ bool targetter_shotgun::valid_aim(coord_def a)
             return notify_fail("There's something in the way.");
         return notify_fail("You cannot see that place.");
     }
-    if ((origin - a).abs() > range2)
+    if ((origin - a).rdist() > range)
         return notify_fail("Out of range.");
     return true;
 }
@@ -1286,9 +1236,15 @@ bool targetter_shotgun::valid_aim(coord_def a)
 bool targetter_shotgun::set_aim(coord_def a)
 {
     zapped.clear();
-    rays.init(ray_def());
 
-    if (!targetter::set_aim(a))
+    // confused monster targetting might be fuzzed across a wall, so
+    // skip the validation in the parent function and set aim directly.
+    // N.B. We assume this targetter can actually handle an invalid aim
+    // (not all targetters can).
+    if (!agent || agent->is_monster())
+        aim = a;
+    // ... but for UI consistency, players should be restricted to LOS.
+    else if (!targetter::set_aim(a))
         return false;
 
     ray_def orig_ray;
@@ -1296,13 +1252,16 @@ bool targetter_shotgun::set_aim(coord_def a)
     coord_def p;
     bool hit = false;
 
-    const double spread_range = PI / 4.0;
-    for (int i = 0; i < SHOTGUN_BEAMS; i++)
+    const double spread_range = (double)(num_beams - 1) * PI / 40.0;
+    for (size_t i = 0; i < num_beams; i++)
     {
         hit = true;
-        double spread = -(spread_range / 2.0)
-                        + (spread_range * (double)i)
-                                        / (double)(SHOTGUN_BEAMS - 1);
+        double spread = (num_beams == 1)
+                        ? 0.0
+                        : -(spread_range / 2.0)
+                          + (spread_range * (double)i)
+                                          / (double)(num_beams - 1);
+        rays[i] = ray_def();
         rays[i].r.start = orig_ray.r.start;
         rays[i].r.dir.x =
              orig_ray.r.dir.x * cos(spread) + orig_ray.r.dir.y * sin(spread);
@@ -1310,7 +1269,7 @@ bool targetter_shotgun::set_aim(coord_def a)
             -orig_ray.r.dir.x * sin(spread) + orig_ray.r.dir.y * cos(spread);
         ray_def tempray = rays[i];
         p = tempray.pos();
-        while ((origin - (p = tempray.pos())).abs() <= range2)
+        while ((origin - (p = tempray.pos())).rdist() <= range)
         {
             if (!map_bounds(p) || opc_solid_see(p) >= OPC_OPAQUE)
                 hit = false;
@@ -1326,12 +1285,12 @@ bool targetter_shotgun::set_aim(coord_def a)
 
 aff_type targetter_shotgun::is_affected(coord_def loc)
 {
-    if ((loc - origin).abs() > range2)
+    if ((loc - origin).rdist() > range)
         return AFF_NO;
 
-    return (zapped[loc] >= SHOTGUN_BEAMS) ? AFF_YES :
-           (zapped[loc] > 0)              ? AFF_MAYBE
-                                          : AFF_NO;
+    return (zapped[loc] >= num_beams) ? AFF_YES :
+           (zapped[loc] > 0)          ? AFF_MAYBE
+                                      : AFF_NO;
 }
 
 targetter_list::targetter_list(vector<coord_def> target_list, coord_def center)
@@ -1342,13 +1301,8 @@ targetter_list::targetter_list(vector<coord_def> target_list, coord_def center)
 
 aff_type targetter_list::is_affected(coord_def loc)
 {
-    for (unsigned int i = 0; i < targets.size(); ++i)
-    {
-        if (targets[i] == loc)
-            return AFF_YES;
-    }
-
-    return AFF_NO;
+    return find(begin(targets), end(targets), loc) == end(targets) ? AFF_NO
+                                                                   : AFF_YES;
 }
 
 bool targetter_list::valid_aim(coord_def a)
