@@ -184,10 +184,7 @@ bool monster_space_valid(const monster* mons, coord_def target,
     if (is_sanctuary(target) && forbid_sanctuary)
         return false;
 
-    // Don't go into no_ctele_into or n_rtele_into cells.
-    if (testbits(env.pgrid(target), FPROP_NO_CTELE_INTO))
-        return false;
-    if (testbits(env.pgrid(target), FPROP_NO_RTELE_INTO))
+    if (testbits(env.pgrid(target), FPROP_NO_TELE_INTO))
         return false;
 
     return monster_habitable_grid(mons, grd(target));
@@ -257,7 +254,8 @@ void mons_relocated(monster* mons)
 
 void monster_teleport(monster* mons, bool instan, bool silent)
 {
-    bool was_seen = !silent && you.can_see(mons) && !mons_is_lurking(mons);
+    ASSERT(mons); // XXX: change to monster &mons
+    bool was_seen = !silent && you.can_see(*mons) && !mons_is_lurking(mons);
 
     if (!instan)
     {
@@ -383,7 +381,10 @@ void blink_other_close(actor* victim, const coord_def &target)
 // Blink a monster away from the caster.
 bool blink_away(monster* mon, actor* caster, bool from_seen, bool self_cast)
 {
-    if (from_seen && !mon->can_see(caster))
+    ASSERT(mon); // XXX: change to monster &mon
+    ASSERT(caster); // XXX: change to actor &caster
+
+    if (from_seen && !mon->can_see(*caster))
         return false;
     bool jumpy = self_cast && mon->is_jumpy();
     coord_def dest = random_space_weighted(mon, caster, false, false, true);
@@ -406,8 +407,10 @@ bool blink_away(monster* mon, bool self_cast)
 // Blink the monster within range but at distance to its foe.
 void blink_range(monster* mon)
 {
+    ASSERT(mon); // XXX: change to monster &mon
+
     actor* foe = mon->get_foe();
-    if (!foe || !mon->can_see(foe))
+    if (!foe || !mon->can_see(*foe))
         return;
     coord_def dest = random_space_weighted(mon, foe, false, true);
     if (dest.origin())
@@ -422,8 +425,10 @@ void blink_range(monster* mon)
 // Blink the monster close to its foe.
 void blink_close(monster* mon)
 {
+    ASSERT(mon); // XXX: change to monster &mon
+
     actor* foe = mon->get_foe();
-    if (!foe || !mon->can_see(foe))
+    if (!foe || !mon->can_see(*foe))
         return;
     coord_def dest = random_space_weighted(mon, foe, true, true, true);
     if (dest.origin())
@@ -488,7 +493,7 @@ bool random_near_space(const actor* victim,
 
         if (valid_blink_destination(victim, target,
                                     forbid_sanctuary, forbid_unhabitable)
-            && (allow_adjacent || distance2(origin, target) > 2))
+            && (allow_adjacent || grid_distance(origin, target) > 1))
         {
             return true;
         }
