@@ -41,7 +41,6 @@ void packed_cell::clear()
 #if TAG_MAJOR_VERSION == 34
     heat_aura        = 0;
 #endif
-    gold_aura        = 0;
 }
 
 bool packed_cell::operator ==(const packed_cell &other) const
@@ -67,7 +66,6 @@ bool packed_cell::operator ==(const packed_cell &other) const
 #if TAG_MAJOR_VERSION == 34
     if (heat_aura != other.heat_aura) return false;
 #endif
-    if (gold_aura != other.gold_aura) return false;
 
     if (num_dngn_overlay != other.num_dngn_overlay) return false;
     for (int i = 0; i < num_dngn_overlay; ++i)
@@ -150,7 +148,7 @@ static void _pack_shoal_waves(const coord_def &gc, packed_cell *cell)
         return;
     }
 
-    if (feat <= DNGN_LAVA)
+    if (feat_is_solid(feat) || feat == DNGN_LAVA)
         return;
 
     const bool ink_only = (feat == DNGN_DEEP_WATER);
@@ -430,8 +428,8 @@ static void _pack_default_waves(const coord_def &gc, packed_cell *cell)
 static bool _is_seen_wall(coord_def gc)
 {
     const dungeon_feature_type feat = _safe_feat(gc);
-    return feat != DNGN_UNSEEN && feat <= DNGN_MAXWALL
-           && feat != DNGN_TREE;
+    return (feat_is_opaque(feat) || feat_is_wall(feat))
+           && feat != DNGN_TREE && feat != DNGN_UNSEEN;
 }
 
 static void _pack_wall_shadows(const coord_def &gc, packed_cell *cell,
