@@ -282,8 +282,6 @@ void DungeonCellBuffer::pack_background(int x, int y, const packed_cell &cell)
             if (cell.heat_aura)
                 m_buf_feat.add(TILE_HEAT_AURA + cell.heat_aura - 1, x, y);
 #endif
-            if (cell.gold_aura)
-                m_buf_feat.add(TILE_GOLD_AURA + cell.gold_aura - 1, x, y);
             if (cell.is_silenced)
                 m_buf_feat.add(TILE_SILENCED, x, y);
             if (cell.halo == HALO_RANGE)
@@ -299,7 +297,7 @@ void DungeonCellBuffer::pack_background(int x, int y, const packed_cell &cell)
                 m_buf_feat.add(TILE_DISJUNCT + cell.disjunct - 1, x, y);
 
             // Apply the travel exclusion under the foreground if the cell is
-            // visible.  It will be applied later if the cell is unseen.
+            // visible. It will be applied later if the cell is unseen.
             if (bg & TILE_FLAG_EXCL_CTR)
                 m_buf_feat.add(TILE_TRAVEL_EXCLUSION_CENTRE_BG, x, y);
             else if (bg & TILE_FLAG_TRAV_EXCL)
@@ -351,19 +349,6 @@ void DungeonCellBuffer::pack_foreground(int x, int y, const packed_cell &cell)
         m_buf_icons.add(TILEI_SOMETHING_UNDER, x, y);
 
     int status_shift = 0;
-    switch (fg & TILE_FLAG_MIMIC_MASK)
-    {
-    case TILE_FLAG_MIMIC_INEPT:
-        m_buf_icons.add(TILEI_INEPT_MIMIC, x, y);
-        break;
-    case TILE_FLAG_MIMIC:
-        m_buf_icons.add(TILEI_MIMIC, x, y);
-        break;
-    case TILE_FLAG_MIMIC_RAVEN:
-        m_buf_icons.add(TILEI_RAVENOUS_MIMIC, x, y);
-        break;
-    default: ;
-    }
 
     //The berserk icon is in the lower right, so status_shift doesn't need changing.
     if (fg & TILE_FLAG_BERSERK)
