@@ -3,12 +3,14 @@
 #ifdef USE_TILE_LOCAL
 
 #include "tilereg-dgn.h"
-#include "process_desc.h"
+
+#include <algorithm> // any_of
 
 #include "cio.h"
 #include "cloud.h"
 #include "command.h"
 #include "coord.h"
+#include "directn.h"
 #include "dgn-height.h"
 #include "env.h"
 #include "invent.h"
@@ -22,16 +24,16 @@
 #include "mon-util.h"
 #include "options.h"
 #include "output.h"
+#include "process_desc.h"
 #include "prompt.h"
 #include "religion.h"
 #include "spl-cast.h"
-#include "spl-util.h"
+#include "spl-zap.h"
 #include "stash.h"
-
 #include "terrain.h"
+#include "tiledef-dngn.h"
 #include "tiledef-icons.h"
 #include "tiledef-main.h"
-#include "tiledef-dngn.h"
 #include "tilefont.h"
 #include "tilepick.h"
 #include "traps.h"
@@ -146,25 +148,25 @@ void DungeonRegion::pack_buffers()
                                  m_cursor[CURSOR_TUTORIAL].y);
     }
 
-    for (unsigned int i = 0; i < m_overlays.size(); i++)
+    for (const tile_overlay &overlay : m_overlays)
     {
         // overlays must be from the main image and must be in LOS.
-        if (!crawl_view.in_los_bounds_g(m_overlays[i].gc))
+        if (!crawl_view.in_los_bounds_g(overlay.gc))
             continue;
 
-        tileidx_t idx = m_overlays[i].idx;
+        tileidx_t idx = overlay.idx;
         if (idx >= TILE_MAIN_MAX)
             continue;
 
-        const coord_def ep(m_overlays[i].gc.x - m_cx_to_gx,
-                           m_overlays[i].gc.y - m_cy_to_gy);
+        const coord_def ep(overlay.gc.x - m_cx_to_gx,
+                           overlay.gc.y - m_cy_to_gy);
         m_buf_dngn.add_main_tile(idx, ep.x, ep.y);
     }
 }
 
 struct tag_def
 {
-    tag_def() { text = NULL; left = right = 0; }
+    tag_def() { text = nullptr; left = right = 0; }
 
     const char* text;
     char left, right;
@@ -194,17 +196,17 @@ void DungeonRegion::render()
 
     for (int t = TAG_MAX - 1; t >= 0; t--)
     {
-        for (unsigned int i = 0; i < m_tags[t].size(); i++)
+        for (const TextTag &tag : m_tags[t])
         {
-            if (!crawl_view.in_los_bounds_g(m_tags[t][i].gc))
+            if (!crawl_view.in_los_bounds_g(tag.gc))
                 continue;
 
-            const coord_def ep = grid2show(m_tags[t][i].gc);
+            const coord_def ep = grid2show(tag.gc);
 
             if (tag_show(ep).text)
                 continue;
 
-            const char *str = m_tags[t][i].tag.c_str();
+            const char *str = tag.tag.c_str();
 
             int width    = m_tag_font->string_width(str);
             tag_def &def = tag_show(ep);
@@ -313,9 +315,9 @@ void DungeonRegion::draw_minibars()
             const int hp_percent = (you.hp * 100) / you.hp_max;
 
             int hp_colour = GREEN;
-            for (unsigned int i = 0; i < Options.hp_colour.size(); ++i)
-                if (hp_percent <= Options.hp_colour[i].first)
-                    hp_colour = Options.hp_colour[i].second;
+            for (const auto &entry : Options.hp_colour)
+                if (hp_percent <= entry.first)
+                    hp_colour = entry.second;
 
             static const VColour healthy(   0, 255, 0, 255); // green
             static const VColour damaged( 255, 255, 0, 255); // yellow
@@ -456,7 +458,7 @@ static bool _is_appropriate_evokable(const item_def& item,
     if (item.sub_type == WAND_RANDOM_EFFECTS)
         return true;
 
-    spell_type spell = zap_type_to_spell(item.zap());
+    spell_type spell = zap_to_spell(item.zap());
     if (spell == SPELL_TELEPORT_OTHER && target->is_player())
         spell = SPELL_TELEPORT_SELF;
 
@@ -465,38 +467,21 @@ static bool _is_appropriate_evokable(const item_def& item,
 
 static bool _have_appropriate_evokable(const actor* target)
 {
-    // Felids cannot use wands.
-    if (you.species == SP_FELID)
-        return false;
-
-    for (int i = 0; i < ENDOFPACK; i++)
-    {
-        item_def &item(you.inv[i]);
-
-        if (!item.defined())
-            continue;
-
-        if (_is_appropriate_evokable(item, target))
-            return true;
-    }
-
-    return false;
+    return any_of(begin(you.inv), end(you.inv),
+                  [target] (const item_def &item) -> bool
+                  {
+                      return item.defined()
+                          && _is_appropriate_evokable(item, target);
+                  });
 }
 
 static item_def* _get_evokable_item(const actor* target)
 {
     vector<const item_def*> list;
 
-    for (int i = 0; i < ENDOFPACK; i++)
-    {
-        item_def &item(you.inv[i]);
-
-        if (!item.defined())
-            continue;
-
-        if (_is_appropriate_evokable(item, target))
+    for (const auto &item : you.inv)
+        if (item.defined() && _is_appropriate_evokable(item, target))
             list.push_back(&item);
-    }
 
     ASSERT(!list.empty());
 
@@ -512,7 +497,7 @@ static item_def* _get_evokable_item(const actor* target)
     redraw_screen();
 
     if (sel.empty())
-        return NULL;
+        return nullptr;
 
     return const_cast<item_def*>(sel[0].item);
 }
@@ -527,17 +512,13 @@ static bool _evoke_item_on_target(actor* target)
         item = _get_evokable_item(target);
     }
 
-    if (item == NULL)
+    if (item == nullptr)
         return false;
 
-    if (item->base_type == OBJ_WANDS)
+    if (is_known_empty_wand(*item))
     {
-        if (item->plus2 == ZAPCOUNT_EMPTY
-            || item_type_known(*item) && item->plus <= 0)
-        {
-            mpr("That wand is empty.");
-            return false;
-        }
+        mpr("That wand is empty.");
+        return false;
     }
 
     macro_buf_add_cmd(CMD_EVOKE);
@@ -569,7 +550,7 @@ static bool _spell_in_range(spell_type spell, actor* target)
     return range >= grid_distance(you.pos(), target->pos());
 }
 
-static actor* _spell_target = NULL;
+static actor* _spell_target = nullptr;
 
 static bool _spell_selector(spell_type spell)
 {
@@ -599,7 +580,7 @@ static bool _cast_spell_on_target(actor* target)
                                  _spell_selector);
         }
 
-        _spell_target = NULL;
+        _spell_target = nullptr;
 
         if (letter == 0)
             return false;
@@ -634,10 +615,8 @@ static bool _cast_spell_on_target(actor* target)
 
 static bool _have_appropriate_spell(const actor* target)
 {
-    for (size_t i = 0; i < you.spells.size(); i++)
+    for (spell_type spell : you.spells)
     {
-        spell_type spell = you.spells[i];
-
         if (!is_valid_spell(spell))
             continue;
 
@@ -650,7 +629,7 @@ static bool _have_appropriate_spell(const actor* target)
 static bool _can_fire_item()
 {
     return you.species != SP_FELID
-           && you.m_quiver->get_fire_item() != -1;
+           && you.m_quiver.get_fire_item() != -1;
 }
 
 static bool _handle_distant_monster(monster* mon, unsigned char mod)
@@ -681,9 +660,9 @@ static bool _handle_distant_monster(monster* mon, unsigned char mod)
     // Handle weapons of reaching.
     if (!mon->wont_attack() && you.see_cell_no_trans(mon->pos()))
     {
-        const int dist = (you.pos() - mon->pos()).abs();
+        const int dist = (you.pos() - mon->pos()).rdist();
 
-        if (dist > 2 && weapon && weapon_reach(*weapon) >= dist)
+        if (dist > 1 && weapon && weapon_reach(*weapon) >= dist)
         {
             macro_buf_add_cmd(CMD_EVOKE_WIELDED);
             _add_targeting_commands(mon->pos());
@@ -873,7 +852,7 @@ int DungeonRegion::handle_mouse(MouseEvent &event)
                         }
                     }
                     // otherwise wait
-                    return command_to_key(CMD_MOVE_NOWHERE);
+                    return command_to_key(CMD_WAIT);
                 }
                 else
                 {
@@ -934,7 +913,7 @@ int DungeonRegion::handle_mouse(MouseEvent &event)
 int tile_click_cell(const coord_def &gc, unsigned char mod)
 {
     monster* mon = monster_at(gc);
-    if (mon && you.can_see(mon))
+    if (mon && you.can_see(*mon))
     {
         if (_handle_distant_monster(mon, mod))
             return CK_MOUSE_CMD;
@@ -1127,7 +1106,7 @@ static void _add_tip(string &tip, string text)
 bool tile_dungeon_tip(const coord_def &gc, string &tip)
 {
     const int attack_dist = you.weapon() ?
-        weapon_reach(*you.weapon()) : 2;
+        weapon_reach(*you.weapon()) : 1;
 
     vector<command_type> cmd;
     tip = "";
@@ -1147,10 +1126,10 @@ bool tile_dungeon_tip(const coord_def &gc, string &tip)
     else // non-player squares
     {
         const actor* target = actor_at(gc);
-        if (target && you.can_see(target))
+        if (target && you.can_see(*target))
         {
             has_monster = true;
-            if ((gc - you.pos()).abs() <= attack_dist)
+            if ((gc - you.pos()).rdist() <= attack_dist)
             {
                 if (!cell_is_solid(gc))
                 {
@@ -1167,7 +1146,7 @@ bool tile_dungeon_tip(const coord_def &gc, string &tip)
 
             if (you.species != SP_FELID
                 && you.see_cell_no_trans(target->pos())
-                && you.m_quiver->get_fire_item() != -1)
+                && you.m_quiver.get_fire_item() != -1)
             {
                 _add_tip(tip, "[Shift + L-Click] Fire (%)");
                 cmd.push_back(CMD_FIRE);
@@ -1255,7 +1234,7 @@ bool tile_dungeon_tip(const coord_def &gc, string &tip)
             {
                 // otherwise wait
                 _add_tip(tip, "[L-Click] Wait one turn (%)");
-                cmd.push_back(CMD_MOVE_NOWHERE);
+                cmd.push_back(CMD_WAIT);
             }
         }
         else
