@@ -11,13 +11,13 @@ class DollEditRegion : public ControlRegion
 public:
     DollEditRegion(ImageManager *im, FontWrapper *font);
 
-    virtual void render();
-    virtual void clear();
-    virtual void run();
+    virtual void render() override;
+    virtual void clear() override;
+    virtual void run() override;
 
-    virtual int handle_mouse(MouseEvent &event);
+    virtual int handle_mouse(MouseEvent &event) override;
 protected:
-    virtual void on_resize() {}
+    virtual void on_resize() override {}
 
     // Currently edited doll index.
     int m_doll_idx;
