@@ -3,7 +3,6 @@
 #ifdef USE_TILE_LOCAL
 
 #include "tilereg-inv.h"
-#include "process_desc.h"
 
 #include "butcher.h"
 #include "cio.h"
@@ -21,8 +20,9 @@
 #include "mon-util.h"
 #include "options.h"
 #include "output.h"
+#include "process_desc.h"
+#include "rot.h"
 #include "spl-book.h"
-
 #include "tiledef-dngn.h"
 #include "tiledef-icons.h"
 #include "tiledef-icons.h"
@@ -121,9 +121,6 @@ void InventoryRegion::pack_buffers()
             if (item.special)
                 m_buf.add_main_tile(item.special, x, y, 0, 0);
 
-            if (item.flag & TILEI_FLAG_TRIED)
-                m_buf.add_icons_tile(TILEI_TRIED, x, y, 3, TILE_Y / 2);
-
             if (item.flag & TILEI_FLAG_INVALID)
                 m_buf.add_icons_tile(TILEI_MESH, x, y);
         }
@@ -245,7 +242,6 @@ static bool _can_use_item(const item_def &item, bool equipped)
     {
         return you.species == SP_VAMPIRE
                && item.sub_type != CORPSE_SKELETON
-               && !food_is_rotten(item)
                && mons_has_blood(item.mon_type);
     }
 
@@ -357,8 +353,7 @@ bool InventoryRegion::update_tip_text(string& tip)
             }
         }
         if (item.base_type == OBJ_CORPSES
-            && item.sub_type != CORPSE_SKELETON
-            && !food_is_rotten(item))
+            && item.sub_type != CORPSE_SKELETON)
         {
             tip += "\n[Shift + L-Click] ";
             if (can_bottle_blood_from_corpse(item.mon_type))
@@ -375,7 +370,7 @@ bool InventoryRegion::update_tip_text(string& tip)
             }
         }
         else if (item.base_type == OBJ_FOOD
-                 && you.is_undead != US_UNDEAD
+                 && you.undead_state() != US_UNDEAD
                  && you.species != SP_VAMPIRE)
         {
             tip += "\n[Shift + R-Click] Eat (e)";
@@ -508,12 +503,8 @@ bool InventoryRegion::update_tip_text(string& tip)
                 if (item_type_known(item) && item_is_spellbook(item)
                     && can_learn_spell(true))
                 {
-                    if (player_can_memorise_from_spellbook(item)
-                        || has_spells_to_memorise(true))
-                    {
-                        tmp += "Memorise (%)";
-                        cmd.push_back(CMD_MEMORISE_SPELL);
-                    }
+                    tmp += "Memorise (%)";
+                    cmd.push_back(CMD_MEMORISE_SPELL);
                     if (wielded)
                         _handle_wield_tip(tmp, cmd, "\n[Ctrl + L-Click] ", true);
                     break;
@@ -668,20 +659,19 @@ static void _fill_item_info(InventoryTile &desc, const item_info &item)
     desc.tile = tileidx_item(item);
 
     int type = item.base_type;
-    if (type == OBJ_FOOD || type == OBJ_SCROLLS
-        || type == OBJ_POTIONS || type == OBJ_MISSILES)
+    if (is_stackable_item(item))
     {
         // -1 specifies don't display anything
         desc.quantity = (item.quantity == 1) ? -1 : item.quantity;
     }
     else if (type == OBJ_WANDS
              && ((item.flags & ISFLAG_KNOW_PLUSES)
-                 || item.plus2 == ZAPCOUNT_EMPTY))
+                 || item.used_count == ZAPCOUNT_EMPTY))
     {
-        desc.quantity = item.plus;
+        desc.quantity = item.charges;
     }
     else if (type == OBJ_RODS && item.flags & ISFLAG_KNOW_PLUSES)
-        desc.quantity = item.plus / ROD_CHARGE_MULT;
+        desc.quantity = item.charges / ROD_CHARGE_MULT;
     else
         desc.quantity = -1;
 
@@ -696,8 +686,6 @@ static void _fill_item_info(InventoryTile &desc, const item_info &item)
     desc.flag = 0;
     if (item.cursed())
         desc.flag |= TILEI_FLAG_CURSE;
-    if (item.flags & ISFLAG_TRIED)
-        desc.flag |= TILEI_FLAG_TRIED;
     if (item.pos.x != -1)
         desc.flag |= TILEI_FLAG_FLOOR;
 }
