@@ -2,19 +2,19 @@
 #ifndef TILEREG_STAT_H
 #define TILEREG_STAT_H
 
-#include "tilereg-text.h"
 #include "tilebuf.h"
+#include "tilereg-text.h"
 
 class StatRegion : public TextRegion
 {
 public:
     StatRegion(FontWrapper *font);
 
-    virtual int handle_mouse(MouseEvent &event);
-    virtual bool update_tip_text(string &tip);
+    virtual int handle_mouse(MouseEvent &event) override;
+    virtual bool update_tip_text(string &tip) override;
 
-    virtual void render();
-    virtual void clear();
+    virtual void render() override;
+    virtual void clear() override;
 
 protected:
     ShapeBuffer m_shape_buf;
