@@ -2,6 +2,8 @@
 
 #ifdef USE_TILE_LOCAL
 
+#include "tilesdl.h"
+
 #include "ability.h"
 #include "artefact.h"
 #include "cio.h"
@@ -22,12 +24,12 @@
 #include "tiledef-gui.h"
 #include "tiledef-main.h"
 #include "tilefont.h"
-#include "tilereg.h"
 #include "tilereg-abl.h"
 #include "tilereg-cmd.h"
 #include "tilereg-crt.h"
 #include "tilereg-dgn.h"
 #include "tilereg-doll.h"
+#include "tilereg.h"
 #include "tilereg-inv.h"
 #include "tilereg-map.h"
 #include "tilereg-mem.h"
@@ -35,25 +37,24 @@
 #include "tilereg-mon.h"
 #include "tilereg-msg.h"
 #include "tilereg-popup.h"
-#include "tilereg-spl.h"
 #include "tilereg-skl.h"
+#include "tilereg-spl.h"
 #include "tilereg-stat.h"
 #include "tilereg-tab.h"
 #include "tilereg-text.h"
 #include "tilereg-title.h"
-#include "tilesdl.h"
 #include "tileview.h"
 #include "travel.h"
 #include "unwind.h"
 #include "version.h"
-#include "view.h"
 #include "viewgeom.h"
+#include "view.h"
 #include "windowmanager.h"
 
 #ifdef __ANDROID__
 #include <android/log.h>
 #include <GLES/gl.h>
-#include <SDL_android.h>
+//#include <SDL_android.h>
 #endif
 
 #ifdef TARGET_OS_WINDOWS
@@ -165,31 +166,31 @@ void TilesFramework::shutdown()
     delete m_region_crt;
     delete m_region_menu;
 
-    m_region_tile  = NULL;
-    m_region_stat  = NULL;
-    m_region_msg   = NULL;
-    m_region_map   = NULL;
-    m_region_tab   = NULL;
-    m_region_inv   = NULL;
-    m_region_spl   = NULL;
-    m_region_mem   = NULL;
-    m_region_abl   = NULL;
-    m_region_mon   = NULL;
-    m_region_crt   = NULL;
-    m_region_menu  = NULL;
+    m_region_tile  = nullptr;
+    m_region_stat  = nullptr;
+    m_region_msg   = nullptr;
+    m_region_map   = nullptr;
+    m_region_tab   = nullptr;
+    m_region_inv   = nullptr;
+    m_region_spl   = nullptr;
+    m_region_mem   = nullptr;
+    m_region_abl   = nullptr;
+    m_region_mon   = nullptr;
+    m_region_crt   = nullptr;
+    m_region_menu  = nullptr;
 
     for (tab_iterator it = m_tabs.begin(); it != m_tabs.end(); ++it)
         delete it->second;
 
     m_tabs.clear();
 
-    for (unsigned int i = 0; i < LAYER_MAX; i++)
-        m_layers[i].m_regions.clear();
+    for (Layer &layer : m_layers)
+        layer.m_regions.clear();
 
-    for (unsigned int i = 0; i < m_fonts.size(); i++)
+    for (font_info &font : m_fonts)
     {
-        delete m_fonts[i].font;
-        m_fonts[i].font = NULL;
+        delete font.font;
+        font.font = nullptr;
     }
 
     delete m_image;
@@ -347,6 +348,8 @@ void TilesFramework::calculate_default_options()
 
 bool TilesFramework::initialise()
 {
+    if (crawl_state.tiles_disabled)
+        return true;
     _init_consoles();
 
     const char *icon_name =
@@ -371,7 +374,7 @@ bool TilesFramework::initialise()
         return false;
 
     // Initialize the wrapper
-    if (!wm->init(&m_windowsz))
+    if (!wm->init(&m_windowsz, &densityNum, &densityDen))
         return false;
 
     wm->set_window_title(title.c_str());
@@ -386,7 +389,7 @@ bool TilesFramework::initialise()
     m_image = new ImageManager();
 
     // If the window size is less than the view height, the textures will
-    // have to be shrunk.  If this isn't the case, then don't create mipmaps,
+    // have to be shrunk. If this isn't the case, then don't create mipmaps,
     // as this appears to make things blurry on some users machines.
     bool need_mips = (m_windowsz.y < 32 * VIEW_MIN_HEIGHT);
     if (!m_image->load_textures(need_mips))
@@ -497,7 +500,7 @@ int TilesFramework::load_font(const char *font_file, int font_size,
 
     FontWrapper *font = FontWrapper::create();
 
-    if (!font->load_font(font_file, font_size, outline))
+    if (!font->load_font(font_file, font_size, outline, densityNum, densityDen))
     {
         delete font;
         if (default_on_fail)
@@ -612,8 +615,10 @@ int TilesFramework::handle_mouse(MouseEvent &event)
     return 0;
 }
 
-static unsigned int _timer_callback(unsigned int ticks)
+static unsigned int _timer_callback(unsigned int ticks, void *param)
 {
+    UNUSED(param);
+
     // force the event loop to break
     wm->raise_custom_event();
 
@@ -625,7 +630,6 @@ int TilesFramework::getch_ck()
 {
     wm_event event;
     cursor_loc cur_loc;
-    cursor_loc tip_loc;
     cursor_loc last_loc;
 
     int key = 0;
@@ -638,10 +642,9 @@ int TilesFramework::getch_ck()
     const unsigned int ticks_per_screen_redraw = Options.tile_update_rate;
 
     unsigned int res = Options.tile_tooltip_ms;
-    wm->set_timer(res, &_timer_callback);
+    unsigned int timer_id = wm->set_timer(res, &_timer_callback);
 
     m_tooltip.clear();
-    string prev_alt = m_region_msg->alt_text();
     m_region_msg->alt_text().clear();
 
     if (need_redraw())
@@ -658,38 +661,10 @@ int TilesFramework::getch_ck()
         if (wm->wait_event(&event))
         {
             ticks = wm->get_ticks();
-            if (!mouse_target_mode)
+            if (!mouse_target_mode && event.type != WME_CUSTOMEVENT)
             {
-                if (event.type != WME_CUSTOMEVENT)
-                {
-                    tiles.clear_text_tags(TAG_CELL_DESC);
-                    m_region_msg->alt_text().clear();
-                }
-
-                if (tip_loc != m_cur_loc)
-                {
-                    set_need_redraw();
-
-                    // Prevent alt. text getting displayed after a mouseclick,
-                    // only after a mouse movement. (jpeg)
-                    if (m_last_tick_moved != UINT_MAX)
-                    {
-                        m_region_msg->alt_text().clear();
-                        for (unsigned int i = 0;
-                            i < m_layers[m_active_layer].m_regions.size(); ++i)
-                        {
-                            Region *reg = m_layers[m_active_layer].m_regions[i];
-                            if (!reg->inside(m_mouse.x, m_mouse.y))
-                                continue;
-
-                            if (reg->update_alt_text(m_region_msg->alt_text()))
-                                break;
-                        }
-
-                        if (prev_alt != m_region_msg->alt_text())
-                            prev_alt = m_region_msg->alt_text();
-                    }
-                }
+                tiles.clear_text_tags(TAG_CELL_DESC);
+                m_region_msg->alt_text().clear();
             }
 
             switch (event.type)
@@ -747,6 +722,13 @@ int TilesFramework::getch_ck()
                 m_last_tick_moved = UINT_MAX;
                 break;
 
+            case WME_KEYPRESS:
+                key = event.key.keysym.sym;
+                m_region_tile->place_cursor(CURSOR_MOUSE, NO_CURSOR);
+
+                m_last_tick_moved = UINT_MAX;
+                break;
+
             case WME_MOUSEMOTION:
                 {
                     // Record mouse pos for tooltip timer
@@ -762,16 +744,16 @@ int TilesFramework::getch_ck()
                     event.mouse_event.mod  = m_key_mod;
                     int mouse_key = handle_mouse(event.mouse_event);
 
+                    m_region_msg->alt_text().clear();
                     // find mouse location
-                    for (unsigned int i = 0;
-                        i < m_layers[m_active_layer].m_regions.size(); i++)
+                    for (Region *reg : m_layers[m_active_layer].m_regions)
                     {
-                        Region *reg = m_layers[m_active_layer].m_regions[i];
                         if (reg->mouse_pos(m_mouse.x, m_mouse.y,
                                            m_cur_loc.cx, m_cur_loc.cy))
                         {
                             m_cur_loc.reg = reg;
                             m_cur_loc.mode = mouse_control::current_mode();
+                            reg->update_alt_text(m_region_msg->alt_text());
                             break;
                         }
                     }
@@ -789,7 +771,7 @@ int TilesFramework::getch_ck()
                         continue;
 
                     // Stay within this input loop until the mouse moves
-                    // to a semantically different location.  Crawl doesn't
+                    // to a semantically different location. Crawl doesn't
                     // care about small mouse movements.
                     if (!need_redraw() && last_loc == m_cur_loc)
                         continue;
@@ -822,6 +804,13 @@ int TilesFramework::getch_ck()
                 crawl_state.seen_hups++;
                 return ESCAPE;
 
+            case WME_RESIZE:
+                m_windowsz.x = event.resize.w;
+                m_windowsz.y = event.resize.h;
+                resize();
+                set_need_redraw();
+                return CK_REDRAW;
+
             case WME_CUSTOMEVENT:
             default:
                 // This is only used to refresh the tooltip.
@@ -837,14 +826,11 @@ int TilesFramework::getch_ck()
 
             if (timeout)
             {
-                tip_loc = m_cur_loc;
                 tiles.clear_text_tags(TAG_CELL_DESC);
                 if (Options.tile_tooltip_ms > 0 && m_tooltip.empty())
                 {
-                    for (unsigned int i = 0;
-                         i < m_layers[m_active_layer].m_regions.size(); ++i)
+                    for (Region *reg : m_layers[m_active_layer].m_regions)
                     {
-                        Region *reg = m_layers[m_active_layer].m_regions[i];
                         if (!reg->inside(m_mouse.x, m_mouse.y))
                             continue;
                         if (reg->update_tip_text(m_tooltip))
@@ -861,7 +847,6 @@ int TilesFramework::getch_ck()
                     set_need_redraw();
 
                 m_tooltip.clear();
-                tip_loc.reset();
             }
 
             if (need_redraw()
@@ -876,7 +861,7 @@ int TilesFramework::getch_ck()
     // We got some input, so we'll probably have to redraw something.
     set_need_redraw();
 
-    wm->set_timer(0, NULL);
+    wm->remove_timer(timer_id);
 
     return key;
 }
@@ -1079,12 +1064,7 @@ bool TilesFramework::is_using_small_layout()
         return false;
     case MB_MAYBE:
     default:
-#ifdef __ANDROID__
-        Options.tile_use_small_layout = (SDL_ANDROID_GetY16Inches()<40) ?
-            MB_TRUE : MB_FALSE; // about 2.5" high
-#else
         Options.tile_use_small_layout = (m_windowsz.x<=480) ? MB_TRUE : MB_FALSE;
-#endif
         return Options.tile_use_small_layout == MB_TRUE;
     }
 #else
@@ -1130,7 +1110,7 @@ bool TilesFramework::zoom_from_minimap()
     if (!m_region_map || !tiles.is_using_small_layout())
         return false;
     delete m_region_map;
-    m_region_map = NULL;
+    m_region_map = nullptr;
 
     // remove minimap from layers again (was at top of vector)
     m_layers[LAYER_NORMAL].m_regions.erase(m_layers[LAYER_NORMAL].m_regions.begin());
@@ -1171,7 +1151,7 @@ void TilesFramework::place_minimap()
     if (m_region_map->dy * GYM > m_statcol_bottom - m_statcol_top)
     {
         delete m_region_map;
-        m_region_map = NULL;
+        m_region_map = nullptr;
         return;
     }
 
@@ -1289,7 +1269,7 @@ void TilesFramework::layout_statcol()
     if (m_region_map)
     {
         delete m_region_map;
-        m_region_map = NULL;
+        m_region_map = nullptr;
         m_layers[LAYER_NORMAL].m_regions.pop_back();
     }
 
@@ -1334,18 +1314,15 @@ void TilesFramework::layout_statcol()
         m_statcol_bottom = m_region_tab->sy - m_tab_margin;
 
 #if TAG_MAJOR_VERSION == 34
-        // Lava orc temperature bar and zot points.
+        // Lava orc temperature bar
         if (you.species == SP_LAVA_ORC)
             ++crawl_view.hudsz.y;
 #endif
-        if (crawl_state.game_is_zotdef())
-            ++crawl_view.hudsz.y;
         m_region_stat->resize(m_region_stat->mx, crawl_view.hudsz.y);
         m_statcol_top += m_region_stat->dy;
 
-        for (int i = 0, size = Options.tile_layout_priority.size(); i < size; ++i)
+        for (const string &str : Options.tile_layout_priority)
         {
-            string str = Options.tile_layout_priority[i];
             if (str == "inventory")
                 resize_inventory();
             else if (str == "minimap" || str == "map")
@@ -1371,7 +1348,7 @@ void TilesFramework::layout_statcol()
 
 void TilesFramework::clrscr()
 {
-    TextRegion::cursor_region = NULL;
+    TextRegion::cursor_region = nullptr;
 
     if (m_region_stat)
         m_region_stat->clear();
@@ -1452,8 +1429,8 @@ void TilesFramework::redraw()
 
     glmanager->reset_view_for_redraw(m_viewsc.x, m_viewsc.y);
 
-    for (unsigned int i = 0; i < m_layers[m_active_layer].m_regions.size(); ++i)
-        m_layers[m_active_layer].m_regions[i]->render();
+    for (Region *region : m_layers[m_active_layer].m_regions)
+        region->render();
 
     // Draw tooltip
     if (Options.tile_tooltip_ms > 0 && !m_tooltip.empty())
@@ -1545,7 +1522,7 @@ void TilesFramework::add_text_tag(text_tag_type type, const string &tag,
 
 void TilesFramework::add_text_tag(text_tag_type type, const monster_info& mon)
 {
-    // HACK.  Large-tile monsters don't interact well with name tags.
+    // HACK. Large-tile monsters don't interact well with name tags.
     monster_type genus = mons_genus(mon.type);
     if (genus == MONS_PANDEMONIUM_LORD
         || genus == MONS_HELL_LORD
