@@ -6,16 +6,16 @@
 #ifndef TILEWEB_H
 #define TILEWEB_H
 
-#include "externs.h"
-#include "status.h"
-#include "tileweb-text.h"
-#include "tiledoll.h"
-#include "viewgeom.h"
-#include "map_knowledge.h"
-#include <map>
 #include <bitset>
+#include <map>
 #include <sys/un.h>
 
+#include "map_knowledge.h"
+#include "status.h"
+#include "tiledoll.h"
+#include "tileweb-text.h"
+#include "viewgeom.h"
+
 class Menu;
 
 enum WebtilesCRTMode
@@ -78,6 +78,8 @@ struct player_info
     FixedVector<int8_t, NUM_EQUIP> equip;
     int8_t quiver_item;
     string unarmed_attack;
+    uint8_t unarmed_attack_colour;
+    bool quiver_available;
 };
 
 class TilesFramework
@@ -120,7 +122,7 @@ public:
     void draw_doll_edit();
 
     // Webtiles-specific
-    void textcolor(int col);
+    void textcolour(int col);
     void textbackground(int col);
     void put_ucs_string(ucs_t *str);
     void clear_to_end_of_line();
@@ -206,6 +208,7 @@ protected:
     vector<sockaddr_un> m_dest_addrs;
 
     bool m_controlled_from_web;
+    bool m_need_flush;
 
     void _await_connection();
     wint_t _handle_control_message(sockaddr_un addr, string data);
