@@ -3,13 +3,16 @@
 
 struct trap_def
 {
+    /// The position of the trap on the map
     coord_def pos;
+    /// The type of trap
     trap_type type;
+    /// The amount of ammo remaining (also used for a horrible shadow trap hack)
     short     ammo_qty;
+    /// How difficult the trap is to detect
     uint8_t   skill_rnd;
 
     dungeon_feature_type category() const;
-    bool ground_only() const;
     string name(description_level_type desc = DESC_PLAIN) const;
     bool is_known(const actor* act = 0) const;
     bool is_safe(actor* act = 0) const;
