@@ -7,7 +7,6 @@
 #define TRAPS_H
 
 #include "enum.h"
-#include "externs.h"
 
 #define NEWLY_TRAPPED_KEY "newly_trapped"
 
@@ -16,14 +15,12 @@ class monster;
 struct trap_def;
 
 void search_around();
-void disarm_trap(const coord_def& where);
 void free_self_from_net();
 void mons_clear_trapping_net(monster* mon);
 void free_stationary_net(int item_index);
 
-void handle_traps(trap_type trt, int i, bool trap_known);
 int get_trapping_net(const coord_def& where, bool trapped = true);
-const char* held_status(actor *act = NULL);
+const char* held_status(actor *act = nullptr);
 bool monster_caught_in_net(monster* mon, actor *agent);
 bool player_caught_in_net();
 void clear_trapping_net();
@@ -37,12 +34,14 @@ void destroy_trap(const coord_def& pos);
 trap_def* find_trap(const coord_def& where);
 trap_type get_trap_type(const coord_def& where);
 
-bool     is_valid_shaft_level(const level_id &place = level_id::current());
+// known is relevant only during level-gen
+bool is_valid_shaft_level(bool known = false);
 level_id generic_shaft_dest(coord_def pos, bool known);
 void     handle_items_on_shaft(const coord_def& where, bool open_shaft);
 
 int       num_traps_for_place();
 trap_type random_trap_for_place();
+trap_type random_vault_trap();
 
 int count_traps(trap_type ttyp);
 void place_webs(int num);
