@@ -5,18 +5,19 @@
 **/
 
 #include "AppHdr.h"
+
+#include "uncancel.h"
+
 #include "acquire.h"
 #include "decks.h"
 #include "godabil.h"
 #include "libutil.h"
-#include "player.h"
 #include "state.h"
-#include "uncancel.h"
 #include "unwind.h"
 
 void add_uncancel(uncancellable_type kind, int arg)
 {
-    you.uncancel.push_back(pair<uncancellable_type, int>(kind, arg));
+    you.uncancel.emplace_back(kind, arg);
 }
 
 static bool running = false;
@@ -26,7 +27,7 @@ void run_uncancels()
     // Run uncancels iteratively rather than recursively.
     if (running)
         return;
-    unwind_var<bool> run(running, true);
+    unwind_bool run(running, true);
 
     while (!you.uncancel.empty() && !crawl_state.seen_hups)
     {
