@@ -4,7 +4,6 @@
 
 #include "end.h"
 #include "options.h"
-#include "player.h"
 #include "state.h"
 
 
@@ -31,7 +30,7 @@
 // needs to fit into 24 lines!
 #define WEBTILES_MSG_MIN_HEIGHT 6
 
-// Helper for layouts.  Tries to increment lvalue without overflowing it.
+// Helper for layouts. Tries to increment lvalue without overflowing it.
 static void _increment(int& lvalue, int delta, int max_value)
 {
     lvalue = min(lvalue+delta, max_value);
@@ -182,8 +181,8 @@ public:
     { valid = _init(); }
     bool _init()
     {
-        // Don't let the mlist column steal all the width.  Up front,
-        // take some for the view.  If it makes the layout fail, that's fine.
+        // Don't let the mlist column steal all the width. Up front,
+        // take some for the view. If it makes the layout fail, that's fine.
         _increment(viewsz.x, MLIST_MIN_WIDTH/2, Options.view_max_width);
 
         // x: View and mlist share leftover; then hud gutter.
@@ -240,12 +239,12 @@ public:
 
 crawl_view_buffer::crawl_view_buffer()
     : m_size(0, 0)
-    , m_buffer(NULL)
+    , m_buffer(nullptr)
 {
 }
 crawl_view_buffer::crawl_view_buffer(const coord_def &sz)
     : m_size(0, 0)
-    , m_buffer(NULL)
+    , m_buffer(nullptr)
 {
     resize(sz);
 }
@@ -281,7 +280,7 @@ const crawl_view_buffer &crawl_view_buffer::operator = (const crawl_view_buffer
 void crawl_view_buffer::clear()
 {
     delete [] m_buffer;
-    m_buffer = NULL;
+    m_buffer = nullptr;
     m_size = coord_def(0,0);
 }
 
@@ -291,7 +290,7 @@ void crawl_view_buffer::clear()
 
 crawl_view_geometry::crawl_view_geometry()
     : termp(1, 1), termsz(80, 24),
-      viewp(1, 1), viewsz(33, 17),
+      viewp(1, 1), viewsz(VIEW_BASE_WIDTH, 17),
       hudp(40, 1), hudsz(-1, -1),
       msgp(1, viewp.y + viewsz.y), msgsz(80, 7),
       mlistp(hudp.x, hudp.y + hudsz.y),
@@ -360,7 +359,7 @@ void crawl_view_geometry::set_player_at(const coord_def &c, bool centre)
         {
             const coord_def dp = c - last_player_pos;
             const coord_def dc = vgrdc - oldc;
-            if (dc.x == dp.x != (dc.y == dp.y))
+            if ((dc.x == dp.x) != (dc.y == dp.y))
                 vgrdc = oldc + dp;
         }
     }
@@ -377,7 +376,7 @@ void crawl_view_geometry::init_geometry()
 {
     termsz = coord_def(get_number_of_cols(), get_number_of_lines());
     hudsz  = coord_def(HUD_WIDTH,
-                       HUD_HEIGHT + crawl_state.game_is_zotdef()
+                       HUD_HEIGHT
 #if TAG_MAJOR_VERSION == 34
                                   + ((you.species == SP_LAVA_ORC) ? 1 : 0)
 #endif
