@@ -10,34 +10,29 @@
 #include <algorithm>
 
 #include "branch.h"
-#include "cio.h"
 #include "colour.h"
 #include "command.h"
 #include "coord.h"
 #include "coordit.h"
 #include "dgn-overview.h"
+#include "directn.h"
 #include "env.h"
-#include "map_knowledge.h"
-#include "message.h"
-#include "fprop.h"
-#include "exclude.h"
-#include "feature.h"
 #include "files.h"
 #include "format.h"
+#include "fprop.h"
 #include "libutil.h"
 #include "macro.h"
-#include "mon-util.h"
+#include "message.h"
 #include "options.h"
 #include "output.h"
-#include "place.h"
-#include "player.h"
 #include "showsymb.h"
-#include "stash.h"
 #include "state.h"
+#include "stringutil.h"
 #include "terrain.h"
 #include "tileview.h"
 #include "travel.h"
 #include "unicode.h"
+#include "view.h"
 #include "viewchar.h"
 #include "viewgeom.h"
 
@@ -66,7 +61,7 @@ static unsigned _get_travel_colour(const coord_def& p)
 #endif
 
 #ifndef USE_TILE_LOCAL
-static bool _travel_colour_override(const coord_def& p)
+bool travel_colour_override(const coord_def& p)
 {
     if (is_waypoint(p) || is_stair_exclusion(p)
        || travel_point_distance[p.x][p.y] == PD_EXCLUDED)
@@ -122,12 +117,12 @@ static bool _is_explore_horizon(const coord_def& c)
 #ifndef USE_TILE_LOCAL
 static ucs_t _get_sightmap_char(dungeon_feature_type feat)
 {
-    return get_feature_def(feat).symbol;
+    return get_feature_def(feat).symbol();
 }
 
 static ucs_t _get_magicmap_char(dungeon_feature_type feat)
 {
-    return get_feature_def(feat).magic_symbol;
+    return get_feature_def(feat).magic_symbol();
 }
 #endif
 
@@ -162,82 +157,15 @@ bool is_feature(ucs_t feature, const coord_def& where)
         return feat_is_gate(grid) || grid == DNGN_ENTER_SHOP
                || grid == DNGN_UNKNOWN_PORTAL;
     case '<':
-        switch (grid)
-        {
-        case DNGN_ENTER_HELL:
-            return player_in_hell();
-        case DNGN_ESCAPE_HATCH_UP:
-        case DNGN_STONE_STAIRS_UP_I:
-        case DNGN_STONE_STAIRS_UP_II:
-        case DNGN_STONE_STAIRS_UP_III:
-        case DNGN_EXIT_DUNGEON:
-#if TAG_MAJOR_VERSION == 34
-        case DNGN_RETURN_FROM_DWARF:
-        case DNGN_RETURN_FROM_FOREST:
-        case DNGN_RETURN_FROM_BLADE:
-#endif
-        case DNGN_RETURN_FROM_ORC:
-        case DNGN_RETURN_FROM_LAIR:
-        case DNGN_RETURN_FROM_SLIME:
-        case DNGN_RETURN_FROM_VAULTS:
-        case DNGN_RETURN_FROM_CRYPT:
-        case DNGN_RETURN_FROM_TEMPLE:
-        case DNGN_RETURN_FROM_SNAKE:
-        case DNGN_RETURN_FROM_ELF:
-        case DNGN_RETURN_FROM_TOMB:
-        case DNGN_RETURN_FROM_SWAMP:
-        case DNGN_RETURN_FROM_SHOALS:
-        case DNGN_RETURN_FROM_SPIDER:
-#if TAG_MAJOR_VERSION == 34
-        case DNGN_EXIT_PORTAL_VAULT:
-#endif
-            return true;
-        default:
-            return false;
-        }
+        return feat_stair_direction(grid) == CMD_GO_UPSTAIRS
+                && !feat_is_portal_exit(grid)
+                && grid != DNGN_ENTER_SHOP;
     case '>':
-        switch (grid)
-        {
-        case DNGN_ESCAPE_HATCH_DOWN:
-        case DNGN_STONE_STAIRS_DOWN_I:
-        case DNGN_STONE_STAIRS_DOWN_II:
-        case DNGN_STONE_STAIRS_DOWN_III:
-        case DNGN_ABYSSAL_STAIR:
-#if TAG_MAJOR_VERSION == 34
-        case DNGN_ENTER_DWARF:
-        case DNGN_ENTER_FOREST:
-        case DNGN_ENTER_BLADE:
-#endif
-        case DNGN_ENTER_ORC:
-        case DNGN_ENTER_LAIR:
-        case DNGN_ENTER_SLIME:
-        case DNGN_ENTER_VAULTS:
-        case DNGN_ENTER_CRYPT:
-        case DNGN_ENTER_TEMPLE:
-        case DNGN_ENTER_SNAKE:
-        case DNGN_ENTER_ELF:
-        case DNGN_ENTER_TOMB:
-        case DNGN_ENTER_SWAMP:
-        case DNGN_ENTER_SHOALS:
-        case DNGN_ENTER_SPIDER:
-            return true;
-        default:
-            return false;
-        }
+        return feat_stair_direction(grid) == CMD_GO_DOWNSTAIRS
+                && !feat_is_portal_entrance(grid)
+                && grid != DNGN_ENTER_SHOP;
     case '^':
-        switch (grid)
-        {
-        case DNGN_TRAP_MECHANICAL:
-        case DNGN_TRAP_TELEPORT:
-        case DNGN_TRAP_ALARM:
-        case DNGN_TRAP_ZOT:
-        case DNGN_TRAP_SHAFT:
-        case DNGN_TRAP_WEB:
-        case DNGN_PASSAGE_OF_GOLUBRIA:
-            return true;
-        default:
-            return false;
-        }
+        return feat_is_trap(grid);
     default:
         return get_cell_glyph(where).ch == feature;
     }
@@ -253,40 +181,15 @@ static bool _is_feature_fudged(ucs_t glyph, const coord_def& where)
 
     if (glyph == '<')
     {
-        if (grd(where) >= DNGN_EXIT_FIRST_PORTAL && grd(where) <= DNGN_EXIT_LAST_PORTAL)
-            return true;
-        switch (grd(where))
-        {
-        case DNGN_EXIT_HELL:
-#if TAG_MAJOR_VERSION == 34
-        case DNGN_EXIT_PORTAL_VAULT:
-#endif
-        case DNGN_EXIT_ABYSS:
-        case DNGN_EXIT_PANDEMONIUM:
-        case DNGN_RETURN_FROM_DEPTHS:
-        case DNGN_RETURN_FROM_ZOT:
-            return true;
-        default:
-            return false;
-        }
+        return feat_is_portal_exit(grd(where))
+               || grd(where) == DNGN_EXIT_ABYSS
+               || grd(where) == DNGN_EXIT_PANDEMONIUM
+               || grd(where) == DNGN_ENTER_HELL && player_in_hell();
     }
     else if (glyph == '>')
     {
-        if (grd(where) >= DNGN_ENTER_FIRST_PORTAL && grd(where) <= DNGN_ENTER_LAST_PORTAL)
-            return true;
-        switch (grd(where))
-        {
-        case DNGN_ENTER_DIS:
-        case DNGN_ENTER_GEHENNA:
-        case DNGN_ENTER_COCYTUS:
-        case DNGN_ENTER_TARTARUS:
-        case DNGN_TRANSIT_PANDEMONIUM:
-        case DNGN_ENTER_DEPTHS:
-        case DNGN_ENTER_ZOT:
-            return true;
-        default:
-            return false;
-        }
+        return feat_is_portal_entrance(grd(where))
+               || grd(where) == DNGN_TRANSIT_PANDEMONIUM;
     }
 
     return false;
@@ -362,10 +265,8 @@ static int _find_feature(const vector<coord_def>& features,
     int firstx = -1, firsty = -1, firstmatch = -1;
     int matchcount = 0;
 
-    for (unsigned feat = 0; feat < features.size(); ++feat)
+    for (coord_def coord : features)
     {
-        const coord_def& coord = features[feat];
-
         if (_is_feature_fudged(feature, coord))
         {
             ++matchcount;
@@ -436,11 +337,11 @@ static void _draw_level_map(int start_x, int start_y, bool travel_mode,
                 if (show == SH_NOTHING && _is_explore_horizon(c))
                 {
                     const feature_def& fd = get_feature_def(DNGN_EXPLORE_HORIZON);
-                    cell->glyph = fd.symbol;
-                    cell->colour = fd.colour;
+                    cell->glyph = fd.symbol();
+                    cell->colour = fd.colour();
                 }
 
-                if (travel_mode && _travel_colour_override(c))
+                if (travel_mode && travel_colour_override(c))
                     cell->colour = _get_travel_colour(c);
 
                 if (c == you.pos() && !crawl_state.arena_suspended && on_level)
@@ -485,7 +386,7 @@ static void _reset_travel_colours(vector<coord_def> &features, bool on_level)
     features.clear();
 
     if (on_level)
-        find_travel_pos(you.pos(), NULL, NULL, &features);
+        find_travel_pos(you.pos(), nullptr, nullptr, &features);
     else
     {
         travel_pathfind tp;
@@ -556,20 +457,20 @@ class feature_list
 public:
     void init()
     {
-        for (unsigned int i = 0; i < NUM_GROUPS; ++i)
-            data[i].clear();
+        for (vector<cglyph_t> &groupdata : data)
+            groupdata.clear();
         for (rectangle_iterator ri(0); ri; ++ri)
             maybe_add(*ri);
-        for (unsigned int i = 0; i < NUM_GROUPS; ++i)
-            sort(data[i].begin(), data[i].end(), _comp_glyphs);
+        for (vector<cglyph_t> &groupdata : data)
+            sort(begin(groupdata), end(groupdata), _comp_glyphs);
     }
 
     formatted_string format() const
     {
         formatted_string s;
-        for (unsigned int i = 0; i < NUM_GROUPS; ++i)
-            for (unsigned int j = 0; j < data[i].size(); ++j)
-                s.add_glyph(data[i][j]);
+        for (const vector<cglyph_t> &groupdata : data)
+            for (cglyph_t gly : groupdata)
+                s.add_glyph(gly);
         return s;
     }
 };
@@ -601,16 +502,16 @@ static void _draw_title(const coord_def& cpos, const feature_list& feats)
 #endif // WIZARD
 
     cgotoxy(1, 1);
-    textcolor(WHITE);
+    textcolour(WHITE);
 
-    cprintf("%s", chop_string(uppercase_first(place_name(
-                          get_packed_place(), true, true)) + pstr,
-                      columns - helplen).c_str());
+    cprintf("%s", chop_string(
+                    uppercase_first(level_id::current().describe(true, true))
+                      + pstr, columns - helplen).c_str());
 
     cgotoxy(max(1, (columns - titlelen) / 2), 1);
     title.display();
 
-    textcolor(LIGHTGREY);
+    textcolour(LIGHTGREY);
     cgotoxy(max(1, columns - helplen + 1), 1);
     help.display();
 }
@@ -630,6 +531,7 @@ public:
             go_to(original);
     }
 
+    // Not virtual!
     void go_to(const level_id& next)
     {
 #ifdef USE_TILE
@@ -698,10 +600,10 @@ static void _forget_map()
     }
 }
 
-// show_map() now centers the known map along x or y.  This prevents
+// show_map() now centers the known map along x or y. This prevents
 // the player from getting "artificial" location clues by using the
-// map to see how close to the end they are.  They'll need to explore
-// to get that.  This function is still a mess, though. -- bwr
+// map to see how close to the end they are. They'll need to explore
+// to get that. This function is still a mess, though. -- bwr
 bool show_map(level_pos &lpos,
               bool travel_mode, bool allow_esc, bool allow_offlevel)
 {
@@ -762,7 +664,7 @@ bool show_map(level_pos &lpos,
         const int top = 2;
         clrscr();
 #endif
-        textcolor(DARKGREY);
+        textcolour(DARKGREY);
 
         bool on_level = false;
 
@@ -835,7 +737,7 @@ bool show_map(level_pos &lpos,
                 // else if close to bottom of known map, put max_y on bottom.
                 //
                 // The num_lines comparisons are done to keep things neat, by
-                // keeping things at the top of the screen.  By shifting an
+                // keeping things at the top of the screen. By shifting an
                 // additional one in the num_lines > map_lines case, we can
                 // keep the top line clear... which makes things look a whole
                 // lot better for small maps.
@@ -873,7 +775,7 @@ bool show_map(level_pos &lpos,
             {
 #ifdef USE_TILE
                 // Note: Tile versions just center on the current cursor
-                // location.  It silently ignores everything else going
+                // location. It silently ignores everything else going
                 // on in this function.  --Enne
 #ifdef USE_TILE_LOCAL
                 if (first_run)
@@ -942,6 +844,13 @@ bool show_map(level_pos &lpos,
 #endif
             }
 
+            if (key == CK_REDRAW)
+            {
+                viewwindow();
+                display_message_window();
+                continue;
+            }
+
             c_input_reset(false);
 
             switch (cmd)
@@ -1074,6 +983,7 @@ bool show_map(level_pos &lpos,
                 {
                     lpos = dest;
                 }
+                los_changed();
                 continue;
             }
 
@@ -1084,7 +994,7 @@ bool show_map(level_pos &lpos,
 
                 string name;
                 const level_pos pos
-                    = prompt_translevel_target(TPF_DEFAULT_OPTIONS, name).p;
+                    = prompt_translevel_target(TPF_DEFAULT_OPTIONS, name);
 
                 if (pos.id.depth < 1
                     || pos.id.depth > brdepth[pos.id.branch]
@@ -1271,7 +1181,8 @@ bool show_map(level_pos &lpos,
                                                     : RMODE_EXPLORE);
                     _reset_travel_colours(features, on_level);
 
-                    if (!whereto.zero()) {
+                    if (!whereto.zero())
+                    {
                         move_x = whereto.x - lpos.pos.x;
                         move_y = whereto.y - lpos.pos.y;
                     }
@@ -1301,6 +1212,15 @@ bool show_map(level_pos &lpos,
             case CMD_NEXT_CMD:
                 break; // allow mouse clicks to move cursor without leaving map mode
 #endif
+            case CMD_MAP_DESCRIBE:
+                if (map_bounds(lpos.pos) && env.map_knowledge(lpos.pos).known())
+                {
+                    full_describe_square(lpos.pos);
+                    redraw_map = true;
+                }
+                break;
+
+
             default:
                 if (travel_mode)
                 {
@@ -1373,6 +1293,7 @@ bool emphasise(const coord_def& where)
 #ifndef USE_TILE_LOCAL
 // Get glyph for feature list; here because it's so similar
 // to get_map_col.
+// Except that that function doesn't exist...
 static cglyph_t _get_feat_glyph(const coord_def& gc)
 {
     // XXX: it's unclear whether we want to display all features
@@ -1381,14 +1302,14 @@ static cglyph_t _get_feat_glyph(const coord_def& gc)
     const bool terrain_seen = env.map_knowledge(gc).seen();
     const feature_def &fdef = get_feature_def(feat);
     cglyph_t g;
-    g.ch  = terrain_seen ? fdef.symbol : fdef.magic_symbol;
+    g.ch  = terrain_seen ? fdef.symbol() : fdef.magic_symbol();
     unsigned col;
-    if (_travel_colour_override(gc))
+    if (travel_colour_override(gc))
         col = _get_travel_colour(gc);
     else if (emphasise(gc))
-        col = fdef.seen_em_colour;
+        col = fdef.seen_em_colour();
     else
-        col = fdef.seen_colour;
+        col = fdef.seen_colour();
     g.col = real_colour(col);
     return g;
 }
