@@ -136,9 +136,9 @@ function ($, map_knowledge, dungeon_renderer, view_data,
             options.get("tile_mapped_wall_col"),    // MF_MAP_WALL
             options.get("tile_door_col"),           // MF_DOOR
             options.get("tile_item_col"),           // MF_ITEM
-            options.get("tile_friendly_col"),       // MF_MONS_FRIENDLY
-            options.get("tile_peaceful_col"),       // MF_MONS_PEACEFUL
-            options.get("tile_neutral_col"),        // MF_MONS_NEUTRAL
+            options.get("tile_monster_col"),       // MF_MONS_FRIENDLY
+            options.get("tile_monster_col"),       // MF_MONS_PEACEFUL
+            options.get("tile_monster_col"),        // MF_MONS_NEUTRAL
             options.get("tile_monster_col"),        // MF_MONS_HOSTILE
             options.get("tile_plant_col"),          // MF_MONS_NO_EXP
             options.get("tile_upstairs_col"),       // MF_STAIR_UP
