@@ -121,7 +121,7 @@ static void _translate_window_event(const SDL_WindowEvent &sdl_event,
     }
 }
 
-// Suppress the SDL_TEXTINPUT event from this keypress.  XXX: hacks
+// Suppress the SDL_TEXTINPUT event from this keypress. XXX: hacks
 static void _suppress_textinput()
 {
     if (SDL_IsTextInputActive())
@@ -133,7 +133,7 @@ static void _suppress_textinput()
 
 static int _translate_keysym(SDL_Keysym &keysym)
 {
-    // This function returns the key that was hit.  Returning zero implies that
+    // This function returns the key that was hit. Returning zero implies that
     // the keypress (e.g. hitting shift on its own) should be eaten and not
     // handled.
 
@@ -383,22 +383,6 @@ int SDLWrapper::init(coord_def *m_windowsz, int *densityNum, int *densityDen)
     SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 8);
     glDebug("SDL_GL_ALPHA_SIZE 8");
 
-#if !SDL_VERSION_ATLEAST(2,0,0)
-    if (Options.tile_key_repeat_delay > 0)
-    {
-        const int repdelay    = Options.tile_key_repeat_delay;
-        const int interval = SDL_DEFAULT_REPEAT_INTERVAL;
-        if (SDL_EnableKeyRepeat(repdelay, interval) != 0)
-#ifdef __ANDROID__
-            __android_log_print(ANDROID_LOG_INFO, "Crawl",
-                                "Failed to set key repeat mode: %s",
-                                SDL_GetError());
-#else
-            printf("Failed to set key repeat mode: %s\n", SDL_GetError());
-#endif
-    }
-#endif
-
     SDL_SetHint(SDL_HINT_VIDEO_HIGHDPI_DISABLED, "0");
 
 #ifdef USE_GLES
@@ -667,6 +651,8 @@ int SDLWrapper::wait_event(wm_event *event)
         _translate_window_event(sdlevent.window, *event);
         break;
     case SDL_KEYDOWN:
+        if (Options.tile_key_repeat_delay <= 0 && sdlevent.key.repeat != 0)
+            return 0;
         event->type = WME_KEYDOWN;
         event->key.state = sdlevent.key.state;
         event->key.keysym.scancode = sdlevent.key.keysym.scancode;
