@@ -7,52 +7,51 @@
 
 #include "wiz-item.h"
 
-#include <errno.h>
+#include <cerrno>
 
 #include "acquire.h"
 #include "act-iter.h"
-#include "art-enum.h"
 #include "artefact.h"
-#include "coordit.h"
-#include "message.h"
+#include "art-enum.h"
 #include "cio.h"
+#include "coordit.h"
 #include "dbg-util.h"
 #include "decks.h"
-#include "effects.h"
 #include "env.h"
 #include "godpassive.h"
+#include "invent.h"
 #include "itemprop.h"
 #include "items.h"
-#include "invent.h"
+#include "jobs.h"
 #include "libutil.h"
 #include "macro.h"
 #include "makeitem.h"
 #include "mapdef.h"
+#include "message.h"
 #include "misc.h"
 #include "mon-death.h"
 #include "options.h"
 #include "output.h"
-#include "prompt.h"
 #include "player-equip.h"
+#include "prompt.h"
 #include "religion.h"
-#include "skills2.h"
+#include "skills.h"
 #include "spl-book.h"
 #include "spl-util.h"
 #include "stash.h"
 #include "stringutil.h"
 #include "terrain.h"
 #include "unicode.h"
+#include "view.h"
 
 #ifdef WIZARD
 static void _make_all_books()
 {
     for (int i = 0; i < NUM_FIXED_BOOKS; ++i)
     {
-#if TAG_MAJOR_VERSION == 34
-        if (i == BOOK_WAR_CHANTS || i == BOOK_STALKING)
+        if (item_type_removed(OBJ_BOOKS, i))
             continue;
-#endif
-        int thing = items(0, OBJ_BOOKS, i, true, 0, 0, 0, AQ_WIZMODE);
+        int thing = items(false, OBJ_BOOKS, i, 0, 0, AQ_WIZMODE);
         if (thing == NON_ITEM)
             continue;
 
@@ -67,7 +66,7 @@ static void _make_all_books()
         set_ident_flags(book, ISFLAG_KNOW_TYPE);
         set_ident_flags(book, ISFLAG_IDENT_MASK);
 
-        mprf("%s", book.name(DESC_PLAIN).c_str());
+        mpr(book.name(DESC_PLAIN));
     }
 }
 
@@ -99,9 +98,7 @@ void wizard_create_spec_object()
 {
     char           specs[80];
     ucs_t          keyin;
-    monster_type   mon;
     object_class_type class_wanted;
-    int            thing_created;
 
     do
     {
@@ -124,22 +121,24 @@ void wizard_create_spec_object()
         }
     } while (class_wanted == NUM_OBJECT_CLASSES);
 
-    msgwin_reply(make_stringf("%c", keyin));
+    // XXX: hope item_class_by_sym never returns a real class for non-ascii.
+    msgwin_reply(string(1, (char) keyin));
 
     // Allocate an item to play with.
-    thing_created = get_mitm_slot();
+    int thing_created = get_mitm_slot();
     if (thing_created == NON_ITEM)
     {
         mpr("Could not allocate item.");
         return;
     }
+    item_def& item(mitm[thing_created]);
 
     // turn item into appropriate kind:
     if (class_wanted == OBJ_ORBS)
     {
-        mitm[thing_created].base_type = OBJ_ORBS;
-        mitm[thing_created].sub_type  = ORB_ZOT;
-        mitm[thing_created].quantity  = 1;
+        item.base_type = OBJ_ORBS;
+        item.sub_type  = ORB_ZOT;
+        item.quantity  = 1;
     }
     else if (class_wanted == OBJ_GOLD)
     {
@@ -150,13 +149,15 @@ void wizard_create_spec_object()
             return;
         }
 
-        mitm[thing_created].base_type = OBJ_GOLD;
-        mitm[thing_created].sub_type  = 0;
-        mitm[thing_created].quantity  = amount;
+        item.base_type = OBJ_GOLD;
+        item.sub_type  = 0;
+        item.quantity  = amount;
     }
+    // in this case, place_monster_corpse will allocate an item for us, and we
+    // don't use item/thing_created.
     else if (class_wanted == OBJ_CORPSES)
     {
-        mon = debug_prompt_for_monster();
+        monster_type mon = debug_prompt_for_monster();
 
         if (mon == MONS_NO_MONSTER || mon == MONS_PROGRAM_BUG)
         {
@@ -164,17 +165,13 @@ void wizard_create_spec_object()
             return;
         }
 
-        if (mons_weight(mon) == 0)
+        if (!mons_class_can_leave_corpse(mon))
         {
-            if (!yesno("That monster doesn't leave corpses; make one "
-                       "anyway?", true, 'y'))
-            {
-                canned_msg(MSG_OK);
-                return;
-            }
+            mpr("That monster doesn't leave corpses.");
+            return;
         }
 
-        if (mon >= MONS_FIRST_NONBASE_DRACONIAN && mon <= MONS_LAST_DRACONIAN)
+        if (mons_is_draconian_job(mon))
         {
             mpr("You can't make a draconian corpse by its background.");
             mon = MONS_DRACONIAN;
@@ -182,15 +179,14 @@ void wizard_create_spec_object()
 
         monster dummy;
         dummy.type = mon;
+        dummy.position = you.pos();
 
         if (mons_genus(mon) == MONS_HYDRA)
-            dummy.number = prompt_for_int("How many heads? ", false);
+            dummy.num_heads = prompt_for_int("How many heads? ", false);
 
-        if (fill_out_corpse(&dummy, dummy.type,
-                            mitm[thing_created], true) == MONS_NO_MONSTER)
+        if (!place_monster_corpse(dummy, false, true))
         {
             mpr("Failed to create corpse.");
-            mitm[thing_created].clear();
             return;
         }
     }
@@ -218,7 +214,7 @@ void wizard_create_spec_object()
             return;
         }
 
-        if (!get_item_by_name(&mitm[thing_created], specs, class_wanted, true))
+        if (!get_item_by_name(&item, specs, class_wanted, true))
         {
             mpr("No such item.");
 
@@ -228,9 +224,7 @@ void wizard_create_spec_object()
         }
     }
 
-    // Deck colour (which control rarity) already set.
-    if (!is_deck(mitm[thing_created]))
-        item_colour(mitm[thing_created]);
+    item_colour(item);
 
     move_item_to_grid(&thing_created, you.pos());
 
@@ -247,117 +241,8 @@ void wizard_create_spec_object()
     }
 }
 
-static const char* _prop_name[] =
-{
-    "Brand",
-    "AC",
-    "EV",
-    "Str",
-    "Int",
-    "Dex",
-    "rFire",
-    "rCold",
-    "rElec",
-    "rPois",
-    "rNeg",
-    "MR",
-    "SInv",
-    "+Inv",
-    "+Fly",
-#if TAG_MAJOR_VERSION > 34
-    "+Fog",
-#endif
-    "+Blnk",
-    "+Rage",
-    "Noisy",
-    "-Cast",
-    "*Tele",
-    "-Tele",
-    "*Rage",
-#if TAG_MAJOR_VERSION == 34
-    "Hungr",
-#endif
-    "Contm",
-    "Acc",
-    "Dam",
-    "Curse",
-    "Stlth",
-    "MP",
-    "Delay",
-    "HP",
-    "Clar",
-    "BAcc",
-    "BDam",
-    "RMsl",
-#if TAG_MAJOR_VERSION == 34
-    "+Fog",
-#endif
-    "Regen",
-    "SustAb",
-    "noupg",
-};
-
-#define ARTP_VAL_BOOL 0
-#define ARTP_VAL_POS  1
-#define ARTP_VAL_ANY  2
-
-static int8_t _prop_type[] =
-{
-    ARTP_VAL_POS,  //BRAND
-    ARTP_VAL_ANY,  //AC
-    ARTP_VAL_ANY,  //EVASION
-    ARTP_VAL_ANY,  //STRENGTH
-    ARTP_VAL_ANY,  //INTELLIGENCE
-    ARTP_VAL_ANY,  //DEXTERITY
-    ARTP_VAL_ANY,  //FIRE
-    ARTP_VAL_ANY,  //COLD
-    ARTP_VAL_BOOL, //ELECTRICITY
-    ARTP_VAL_BOOL, //POISON
-    ARTP_VAL_BOOL, //NEGATIVE_ENERGY
-    ARTP_VAL_POS,  //MAGIC
-    ARTP_VAL_BOOL, //EYESIGHT
-    ARTP_VAL_BOOL, //INVISIBLE
-    ARTP_VAL_BOOL, //FLIGHT
-#if TAG_MAJOR_VERSION > 34
-    ARTP_VAL_BOOL, //FOG
-#endif
-    ARTP_VAL_BOOL, //BLINK
-    ARTP_VAL_BOOL, //BERSERK
-    ARTP_VAL_POS,  //NOISES
-    ARTP_VAL_BOOL, //PREVENT_SPELLCASTING
-    ARTP_VAL_BOOL, //CAUSE_TELEPORTATION
-    ARTP_VAL_BOOL, //PREVENT_TELEPORTATION
-    ARTP_VAL_POS,  //ANGRY
-#if TAG_MAJOR_VERSION == 34
-    ARTP_VAL_POS,  //METABOLISM
-#endif
-    ARTP_VAL_POS,  //MUTAGENIC
-#if TAG_MAJOR_VERSION == 34
-    ARTP_VAL_ANY,  //ACCURACY
-#endif
-    ARTP_VAL_ANY,  //SLAYING
-    ARTP_VAL_POS,  //CURSED
-    ARTP_VAL_ANY,  //STEALTH
-    ARTP_VAL_ANY,  //MAGICAL_POWER
-    ARTP_VAL_ANY,  //BASE_DELAY
-    ARTP_VAL_ANY,  //HP
-    ARTP_VAL_BOOL, //CLARITY
-    ARTP_VAL_ANY,  //BASE_ACC
-    ARTP_VAL_ANY,  //BASE_DAM
-    ARTP_VAL_BOOL, //RMSL
-#if TAG_MAJOR_VERSION == 34
-    ARTP_VAL_BOOL, //FOG
-#endif
-    ARTP_VAL_ANY,  //REGENERATION
-    ARTP_VAL_BOOL, //SUSTAB
-    ARTP_VAL_BOOL, //NO_UPGRADE
-};
-
 static void _tweak_randart(item_def &item)
 {
-    COMPILE_CHECK(ARRAYSZ(_prop_name) == ARTP_NUM_PROPERTIES);
-    COMPILE_CHECK(ARRAYSZ(_prop_type) == ARTP_NUM_PROPERTIES);
-
     if (item_is_equipped(item))
     {
         mprf(MSGCH_PROMPT, "You can't tweak the randart properties of an equipped item.");
@@ -367,18 +252,20 @@ static void _tweak_randart(item_def &item)
         clear_messages();
 
     artefact_properties_t props;
-    artefact_wpn_properties(item, props);
+    artefact_properties(item, props);
 
     string prompt = "";
 
     vector<unsigned int> choice_to_prop;
     for (unsigned int i = 0, choice_num = 0; i < ARTP_NUM_PROPERTIES; ++i)
     {
-        if (_prop_name[i] == string("UNUSED"))
+#if TAG_MAJOR_VERSION == 34
+        if (i == ARTP_METABOLISM || i == ARTP_ACCURACY || i == ARTP_TWISTER)
             continue;
+#endif
         choice_to_prop.push_back(i);
         if (choice_num % 8 == 0 && choice_num != 0)
-            prompt += "\n";
+            *(prompt.rend()) = '\n'; // Replace the space
 
         char choice;
         char buf[80];
@@ -393,10 +280,11 @@ static void _tweak_randart(item_def &item)
         else
             choice = '-'; // Too many choices!
 
-        if (props[i])
-            snprintf(buf, sizeof(buf), "%c) <w>%-5s</w> ", choice, _prop_name[i]);
-        else
-            snprintf(buf, sizeof(buf), "%c) %-5s ", choice, _prop_name[i]);
+        snprintf(buf, sizeof(buf), "%s) %s%-6s%s ",
+                choice == '<' ? "<<" : string(1, choice).c_str(),
+                 props[i] ? "<w>" : "",
+                 artp_name((artefact_prop_type)choice_to_prop[choice_num]),
+                 props[i] ? "</w>" : "");
 
         prompt += buf;
 
@@ -425,38 +313,39 @@ static void _tweak_randart(item_def &item)
         return;
     }
 
-    unsigned int prop = choice_to_prop[choice];
-    ASSERT(prop < ARRAYSZ(_prop_type));
-
-    int val;
-    switch (_prop_type[prop])
+    const artefact_prop_type prop = (artefact_prop_type)choice_to_prop[choice];
+    switch (artp_potential_value_types(prop))
     {
     case ARTP_VAL_BOOL:
-        mprf(MSGCH_PROMPT, "Toggling %s to %s.", _prop_name[prop],
+        mprf(MSGCH_PROMPT, "Toggling %s to %s.", artp_name(prop),
              props[prop] ? "off" : "on");
         artefact_set_property(item, static_cast<artefact_prop_type>(prop),
                              !props[prop]);
         break;
 
     case ARTP_VAL_POS:
-        mprf(MSGCH_PROMPT, "%s was %d.", _prop_name[prop], props[prop]);
-        val = prompt_for_int("New value? ", true);
+     {
+        mprf(MSGCH_PROMPT, "%s was %d.", artp_name(prop), props[prop]);
+        const int val = prompt_for_int("New value? ", true);
 
         if (val < 0)
         {
             mprf(MSGCH_PROMPT, "Value for %s must be non-negative",
-                 _prop_name[prop]);
+                 artp_name(prop));
             return;
         }
         artefact_set_property(item, static_cast<artefact_prop_type>(prop),
                              val);
         break;
+      }
     case ARTP_VAL_ANY:
-        mprf(MSGCH_PROMPT, "%s was %d.", _prop_name[prop], props[prop]);
-        val = prompt_for_int("New value? ", false);
+      {
+        mprf(MSGCH_PROMPT, "%s was %d.", artp_name(prop), props[prop]);
+        const int val = prompt_for_int("New value? ", false);
         artefact_set_property(item, static_cast<artefact_prop_type>(prop),
                              val);
         break;
+      }
     }
 }
 
@@ -483,9 +372,9 @@ void wizard_tweak_object()
         {
             mprf_nocap("%s", you.inv[item].name(DESC_INVENTORY_EQUIP).c_str());
 
-            mprf(MSGCH_PROMPT, "a - plus  b - plus2  c - %s  "
-                               "d - quantity  e - flags  ESC - exit",
-                               is_art ? "art props" : "special");
+            mprf_nocap(MSGCH_PROMPT, "a - plus  b - plus2  c - %s  "
+                                     "d - quantity  e - flags  ESC - exit",
+                                     is_art ? "art props" : "special");
 
             mprf(MSGCH_PROMPT, "Which field? ");
 
@@ -525,7 +414,10 @@ void wizard_tweak_object()
 
         msgwin_get_line("New value? ", specs, sizeof(specs));
         if (specs[0] == '\0')
+        {
+            canned_msg(MSG_OK);
             return;
+        }
 
         char *end;
         const bool hex = (keyin == 'e');
@@ -540,7 +432,10 @@ void wizard_tweak_object()
         }
 
         if (end == specs)
+        {
+            canned_msg(MSG_OK);
             return;
+        }
 
         if (keyin == 'a')
             you.inv[item].plus = new_val;
@@ -595,7 +490,7 @@ void wizard_value_artefact()
         if (!is_artefact(item))
             mpr("That item is not an artefact!");
         else
-            mprf("%s", debug_art_val_str(item).c_str());
+            mpr(debug_art_val_str(item));
     }
 }
 
@@ -605,7 +500,7 @@ void wizard_create_all_artefacts()
     int octorings = 8;
 
     // Create all unrandarts.
-    for (int i = 0; i < NO_UNRANDARTS; ++i)
+    for (int i = 0; i < NUM_UNRANDARTS; ++i)
     {
         const int              index = i + UNRAND_START;
         const unrandart_entry* entry = get_unrand_entry(index);
@@ -753,41 +648,42 @@ void wizard_uncurse_item()
 
         if (item.cursed())
             do_uncurse_item(item);
-        else if (_item_type_can_be_cursed(item.base_type))
-            do_curse_item(item);
         else
-            mpr("That type of item cannot be cursed.");
+        {
+            if (!_item_type_can_be_cursed(item.base_type))
+            {
+                mpr("That type of item cannot be cursed.");
+                return;
+            }
+            do_curse_item(item);
+        }
+        mprf_nocap("%s", item.name(DESC_INVENTORY_EQUIP).c_str());
     }
 }
 
 void wizard_identify_pack()
 {
     mpr("You feel a rush of knowledge.");
-    for (int i = 0; i < ENDOFPACK; ++i)
-    {
-        item_def& item = you.inv[i];
-        if (item.defined())
-        {
-            set_ident_type(item, ID_KNOWN_TYPE);
-            set_ident_flags(item, ISFLAG_IDENT_MASK);
-        }
-    }
+    identify_inventory();
     you.wield_change  = true;
     you.redraw_quiver = true;
 }
 
+static void _forget_item(item_def &item)
+{
+    set_ident_type(item, false);
+    unset_ident_flags(item, ISFLAG_IDENT_MASK);
+    item.flags &= ~(ISFLAG_SEEN | ISFLAG_HANDLED | ISFLAG_THROWN
+                    | ISFLAG_DROPPED | ISFLAG_NOTED_ID | ISFLAG_NOTED_GET);
+}
+
 void wizard_unidentify_pack()
 {
     mpr("You feel a rush of antiknowledge.");
-    for (int i = 0; i < ENDOFPACK; ++i)
-    {
-        item_def& item = you.inv[i];
+    for (auto &item : you.inv)
         if (item.defined())
-        {
-            set_ident_type(item, ID_UNKNOWN_TYPE);
-            unset_ident_flags(item, ISFLAG_IDENT_MASK);
-        }
-    }
+            _forget_item(item);
+
     you.wield_change  = true;
     you.redraw_quiver = true;
 
@@ -802,49 +698,18 @@ void wizard_unidentify_pack()
                 continue;
 
             item_def &item = mitm[mon->inv[j]];
-
-            if (!item.defined())
-                continue;
-
-            set_ident_type(item, ID_UNKNOWN_TYPE);
-            unset_ident_flags(item, ISFLAG_IDENT_MASK);
+            if (item.defined())
+                _forget_item(item);
         }
     }
 }
 
 void wizard_list_items()
 {
-    bool has_shops = false;
-
-    for (int i = 0; i < MAX_SHOPS; ++i)
-        if (env.shop[i].type != SHOP_UNASSIGNED)
-        {
-            has_shops = true;
-            break;
-        }
-
-    if (has_shops)
-    {
-        mpr("Shop items:");
-
-        for (int i = 0; i < MAX_SHOPS; ++i)
-            if (env.shop[i].type != SHOP_UNASSIGNED)
-            {
-                for (stack_iterator si(coord_def(0, i+5)); si; ++si)
-                    mpr(si->name(DESC_PLAIN, false, false, false).c_str());
-            }
-
-        mpr("");
-    }
-
     mpr("Item stacks (by location and top item):");
-    for (int i = 0; i < MAX_ITEMS; ++i)
+    for (const auto &item : mitm)
     {
-        item_def &item(mitm[i]);
-        if (!item.defined() || item.held_by_monster())
-            continue;
-
-        if (item.link != NON_ITEM)
+        if (item.defined() && !item.held_by_monster() && item.link != NON_ITEM)
         {
             mprf("(%2d,%2d): %s%s", item.pos.x, item.pos.y,
                  item.name(DESC_PLAIN, false, false, false).c_str(),
@@ -916,7 +781,7 @@ static void _debug_acquirement_stats(FILE *ostat)
     int last_percent = 0;
     int acq_calls    = 0;
     int total_quant  = 0;
-    int max_plus     = -127;
+    short max_plus   = -127;
     int total_plus   = 0;
     int num_arts     = 0;
 
@@ -951,8 +816,8 @@ static void _debug_acquirement_stats(FILE *ostat)
         total_quant += item.quantity;
         subtype_quants[item.sub_type] += item.quantity;
 
-        max_plus    = max(max_plus, item.plus + item.plus2);
-        total_plus += item.plus + item.plus2;
+        max_plus    = max(max_plus, item.plus);
+        total_plus += item.plus;
 
         if (is_artefact(item))
         {
@@ -1019,11 +884,11 @@ static void _debug_acquirement_stats(FILE *ostat)
     if (!you_worship(GOD_NO_GOD))
         godname += " of " + god_name(you.religion);
 
-    fprintf(ostat, "%s the %s, Level %d %s %s%s\n\n",
+    fprintf(ostat, "%s %s, Level %d %s %s%s\n\n",
             you.your_name.c_str(), player_title().c_str(),
             you.experience_level,
             species_name(you.species).c_str(),
-            you.class_name.c_str(), godname.c_str());
+            get_job_name(you.char_class), godname.c_str());
 
     // Print player equipment.
     const int e_order[] =
@@ -1046,11 +911,10 @@ static void _debug_acquirement_stats(FILE *ostat)
             // The player has something equipped.
             const int item_idx   = you.equip[e_order[i]];
             const item_def& item = you.inv[item_idx];
-            const bool melded    = !player_wearing_slot(e_order[i]);
 
             fprintf(ostat, "%-7s: %s %s\n", equip_slot_to_name(eqslot),
                     item.name(DESC_PLAIN, true).c_str(),
-                    melded ? "(melded)" : "");
+                    you.melded[i] ? "(melded)" : "");
             naked = false;
         }
     }
@@ -1076,7 +940,7 @@ static void _debug_acquirement_stats(FILE *ostat)
             if (!is_valid_spell(spell))
                 continue;
 
-            if (you_cannot_memorise(spell))
+            if (!you_can_memorise(spell))
                 continue;
 
             // Only use spells available in books you might find lying about
@@ -1086,10 +950,10 @@ static void _debug_acquirement_stats(FILE *ostat)
 
             const bool seen = you.seen_spell[spell];
 
-            const unsigned int disciplines = get_spell_disciplines(spell);
+            const spschools_type disciplines = get_spell_disciplines(spell);
             for (int d = 0; d <= SPTYP_LAST_EXPONENT; ++d)
             {
-                const int disc = 1 << d;
+                const auto disc = spschools_type::exponent(d);
                 if (disc & SPTYP_DIVINATION)
                     continue;
 
@@ -1103,7 +967,7 @@ static void _debug_acquirement_stats(FILE *ostat)
         }
         for (int d = 0; d <= SPTYP_LAST_EXPONENT; ++d)
         {
-            const int disc = 1 << d;
+            const auto disc = spschools_type::exponent(d);
             if (disc & SPTYP_DIVINATION)
                 continue;
 
@@ -1194,13 +1058,13 @@ static void _debug_acquirement_stats(FILE *ostat)
             "cold resistance",
             "poison resistance",
             "see invis",
-            "darkness",
+            "invisibility",
             "strength",
             "dexterity",
             "intelligence",
             "ponderous",
             "flight",
-            "magic reistance",
+            "magic resistance",
             "protection",
             "stealth",
             "resistance",
@@ -1212,7 +1076,9 @@ static void _debug_acquirement_stats(FILE *ostat)
             "reflection",
             "spirit shield",
             "archery",
+#if TAG_MAJOR_VERSION == 34
             "jumping",
+#endif
         };
 
         const int non_art = acq_calls - num_arts;
@@ -1277,12 +1143,12 @@ static void _debug_acquirement_stats(FILE *ostat)
         {
             const int mannum = subtype_quants[BOOK_MANUAL];
             fprintf(ostat, "\nManuals:\n");
-            for (int i = SK_FIRST_SKILL; i <= SK_LAST_SKILL; ++i)
+            for (skill_type sk = SK_FIRST_SKILL; sk <= SK_LAST_SKILL; ++sk)
             {
-                const int k = 200 + i;
+                const int k = 200 + sk;
                 if (subtype_quants[k] > 0)
                 {
-                    fprintf(ostat, "%17s: %5.2f\n", skill_name((skill_type)i),
+                    fprintf(ostat, "%17s: %5.2f\n", skill_name(sk),
                             100.0 * (float) subtype_quants[k] / (float) mannum);
                 }
             }
@@ -1337,18 +1203,28 @@ static void _debug_acquirement_stats(FILE *ostat)
     mpr("Results written into 'items.stat'.");
 }
 
-#define MAX_TRIES 16777216 /* not special anymore */
+/**
+ * Take the median of the provided dataset. Mutates it for efficiency.
+ */
+static int _median(vector<int> &counts)
+{
+    nth_element(counts.begin(), counts.begin() + counts.size()/2, counts.end());
+    return counts[counts.size()/2];
+}
+
+#define MAX_TRIES 27272
 static void _debug_rap_stats(FILE *ostat)
 {
-    int i = prompt_invent_item("Generate randart stats on which item?",
-                                MT_INVLIST, -1);
+    const int inv_index
+        = prompt_invent_item("Generate randart stats on which item?",
+                             MT_INVLIST, -1);
 
-    if (prompt_failed(i))
+    if (prompt_failed(inv_index))
         return;
 
     // A copy of the item, rather than a reference to the inventory item,
     // so we can fiddle with the item at will.
-    item_def item(you.inv[i]);
+    item_def item(you.inv[inv_index]);
 
     // Start off with a non-artefact item.
     item.flags  &= ~ISFLAG_ARTEFACT_MASK;
@@ -1361,81 +1237,28 @@ static void _debug_rap_stats(FILE *ostat)
         return;
     }
 
-    // -1 = always bad, 1 = always good, 0 = depends on value
-    const int good_or_bad[] =
-    {
-         1, //ARTP_BRAND
-         0, //ARTP_AC
-         0, //ARTP_EVASION
-         0, //ARTP_STRENGTH
-         0, //ARTP_INTELLIGENCE
-         0, //ARTP_DEXTERITY
-         0, //ARTP_FIRE
-         0, //ARTP_COLD
-         1, //ARTP_ELECTRICITY
-         1, //ARTP_POISON
-         1, //ARTP_NEGATIVE_ENERGY
-         0, //ARTP_MAGIC
-         1, //ARTP_EYESIGHT
-         1, //ARTP_INVISIBLE
-         1, //ARTP_FLY
-#if TAG_MAJOR_VERSION > 34
-         1, //ARTP_FOG,
-#endif
-         1, //ARTP_BLINK
-         1, //ARTP_BERSERK
-        -1, //ARTP_NOISES
-        -1, //ARTP_PREVENT_SPELLCASTING
-        -1, //ARTP_CAUSE_TELEPORTATION
-        -1, //ARTP_PREVENT_TELEPORTATION
-        -1, //ARTP_ANGRY
-#if TAG_MAJOR_VERSION == 34
-         0, //ARTP_METABOLISM
-#endif
-        -1, //ARTP_MUTAGENIC
-#if TAG_MAJOR_VERSION == 34
-         0, //ARTP_ACCURACY
-#endif
-         0, //ARTP_SLAYING
-        -1, //ARTP_CURSED
-         0, //ARTP_STEALTH
-         0, //ARTP_MAGICAL_POWER
-         0, //ARTP_BASE_DELAY
-         0, //ARTP_HP
-         1, //ARTP_CLARITY
-         0, //ARTP_BASE_ACC
-         0, //ARTP_BASE_DAM
-         1, //ARTP_RMSL
-#if TAG_MAJOR_VERSION == 34
-         1, //ARTP_FOG
-#endif
-         1, //ARTP_REGENERATION
-         1, //ARTP_SUSTAB,
-         0, //ARTP_NO_UPGRADE
-    };
-    COMPILE_CHECK(ARRAYSZ(good_or_bad) == ARTP_NUM_PROPERTIES);
+    FixedVector<int, ARTP_NUM_PROPERTIES> good_props(0);
+    FixedVector<int, ARTP_NUM_PROPERTIES> bad_props(0);
 
-    // No bounds checking to speed things up a bit.
-    int all_props[ARTP_NUM_PROPERTIES];
-    int good_props[ARTP_NUM_PROPERTIES];
-    int bad_props[ARTP_NUM_PROPERTIES];
-    for (i = 0; i < ARTP_NUM_PROPERTIES; ++i)
-    {
-        all_props[i] = 0;
-        good_props[i] = 0;
-        bad_props[i] = 0;
-    }
+    FixedVector<int, ARTP_NUM_PROPERTIES> max_prop_vals(0);
+    FixedVector<int, ARTP_NUM_PROPERTIES> min_prop_vals(0);
+    FixedVector<int, ARTP_NUM_PROPERTIES> total_good_prop_vals(0);
+    FixedVector<int, ARTP_NUM_PROPERTIES> total_bad_prop_vals(0);
 
-    int max_props         = 0, total_props         = 0;
-    int max_good_props    = 0, total_good_props    = 0;
-    int max_bad_props     = 0, total_bad_props     = 0;
+    vector<int> good_prop_counts;
+    vector<int> bad_prop_counts;
+    vector<int> total_prop_counts;
+
+    int max_props         = 0;
+    int max_good_props    = 0;
+    int max_bad_props     = 0;
     int max_balance_props = 0, total_balance_props = 0;
 
     int num_randarts = 0, bad_randarts = 0;
 
     artefact_properties_t proprt;
 
-    for (i = 0; i < MAX_TRIES; ++i)
+    for (int i = 0; i < MAX_TRIES; ++i)
     {
         if (kbhit())
         {
@@ -1446,7 +1269,12 @@ static void _debug_rap_stats(FILE *ostat)
 
         // Generate proprt once and hand it off to randart_is_bad(),
         // so that randart_is_bad() doesn't generate it a second time.
-        artefact_wpn_properties(item, proprt);
+        item.flags  &= ~ISFLAG_ARTEFACT_MASK;
+        item.special = 0;
+        item.props.clear();
+        make_item_randart(item);
+        artefact_properties(item, proprt);
+
         if (randart_is_bad(item, proprt))
         {
             bad_randarts++;
@@ -1454,57 +1282,55 @@ static void _debug_rap_stats(FILE *ostat)
         }
 
         num_randarts++;
-        proprt[ARTP_CURSED] = 0;
 
-        int num_props = 0, num_good_props = 0, num_bad_props = 0;
+        int num_good_props = 0, num_bad_props = 0;
         for (int j = 0; j < ARTP_NUM_PROPERTIES; ++j)
         {
-            const int val = proprt[j];
-            if (val)
+            const artefact_prop_type prop = (artefact_prop_type)j;
+            const int val = proprt[prop];
+            if (!val)
+                continue;
+
+            // assumption: all mixed good/bad props are good iff positive
+            const bool good = !artp_potentially_bad(prop)
+                              || (artp_potentially_good(prop) && val > 0);
+            if (good)
             {
-                num_props++;
-                all_props[j]++;
-                switch (good_or_bad[j])
-                {
-                case -1:
-                    num_bad_props++;
-                    break;
-                case 1:
-                    num_good_props++;
-                    break;
-                case 0:
-                    if (val > 0)
-                    {
-                        good_props[j]++;
-                        num_good_props++;
-                    }
-                    else
-                    {
-                        bad_props[j]++;
-                        num_bad_props++;
-                    }
-                }
+                good_props[prop]++;
+                num_good_props++;
+                total_good_prop_vals[prop] += val;
+                max_prop_vals[prop] = max(max_prop_vals[prop], val);
+            }
+            else
+            {
+                bad_props[prop]++;
+                num_bad_props++;
+                total_bad_prop_vals[prop] += val;
+                min_prop_vals[prop] = min(min_prop_vals[prop], val);
             }
         }
 
-        int balance = num_good_props - num_bad_props;
+        const int num_props = num_good_props + num_bad_props;
+        const int balance   = num_good_props - num_bad_props;
+
+        good_prop_counts.push_back(num_good_props);
+        bad_prop_counts.push_back(num_bad_props);
+        total_prop_counts.push_back(num_props);
 
         max_props         = max(max_props, num_props);
         max_good_props    = max(max_good_props, num_good_props);
         max_bad_props     = max(max_bad_props, num_bad_props);
         max_balance_props = max(max_balance_props, balance);
 
-        total_props         += num_props;
-        total_good_props    += num_good_props;
-        total_bad_props     += num_bad_props;
         total_balance_props += balance;
 
-        if (i % 16767 == 0)
+        if (i % (MAX_TRIES / 100) == 0)
         {
             clear_messages();
             float curr_percent = (float) i * 1000.0
                 / (float) MAX_TRIES;
             mprf("%4.1f%% done.", curr_percent / 10.0);
+            viewwindow();
         }
 
     }
@@ -1512,12 +1338,25 @@ static void _debug_rap_stats(FILE *ostat)
     fprintf(ostat, "Randarts generated: %d valid, %d invalid\n\n",
             num_randarts, bad_randarts);
 
-    fprintf(ostat, "max # of props = %d, avg # = %5.2f\n",
-            max_props, (float) total_props / (float) num_randarts);
-    fprintf(ostat, "max # of good props = %d, avg # = %5.2f\n",
-            max_good_props, (float) total_good_props / (float) num_randarts);
-    fprintf(ostat, "max # of bad props = %d, avg # = %5.2f\n",
-            max_bad_props, (float) total_bad_props / (float) num_randarts);
+    int total_good_props = 0, total_bad_props = 0;
+    for (int i = 0; i < ARTP_NUM_PROPERTIES; ++i)
+    {
+        total_good_props += good_props[i];
+        total_bad_props += bad_props[i];
+    }
+
+    // assumption: all props are good or bad
+    const int total_props = total_good_props + total_bad_props;
+
+    fprintf(ostat, "max # of props = %d, mean = %5.2f, median = %d\n",
+            max_props, (float) total_props / (float) num_randarts,
+            _median(total_prop_counts));
+    fprintf(ostat, "max # of good props = %d, mean = %5.2f, median = %d\n",
+            max_good_props, (float) total_good_props / (float) num_randarts,
+            _median(good_prop_counts));
+    fprintf(ostat, "max # of bad props = %d, mean = %5.2f, median = %d\n",
+            max_bad_props, (float) total_bad_props / (float) num_randarts,
+            _median(bad_prop_counts));
     fprintf(ostat, "max (good - bad) props = %d, avg # = %5.2f\n\n",
             max_balance_props,
             (float) total_balance_props / (float) num_randarts);
@@ -1535,8 +1374,8 @@ static void _debug_rap_stats(FILE *ostat)
         "ARTP_ELECTRICITY",
         "ARTP_POISON",
         "ARTP_NEGATIVE_ENERGY",
-        "ARTP_MAGIC",
-        "ARTP_EYESIGHT",
+        "ARTP_MAGIC_RESISTANCE",
+        "ARTP_SEE_INVISIBLE",
         "ARTP_INVISIBLE",
         "ARTP_FLY",
 #if TAG_MAJOR_VERSION > 34
@@ -1544,7 +1383,7 @@ static void _debug_rap_stats(FILE *ostat)
 #endif
         "ARTP_BLINK",
         "ARTP_BERSERK",
-        "ARTP_NOISES",
+        "ARTP_NOISE",
         "ARTP_PREVENT_SPELLCASTING",
         "ARTP_CAUSE_TELEPORTATION",
         "ARTP_PREVENT_TELEPORTATION",
@@ -1552,12 +1391,12 @@ static void _debug_rap_stats(FILE *ostat)
 #if TAG_MAJOR_VERSION == 34
         "ARTP_METABOLISM",
 #endif
-        "ARTP_MUTAGENIC",
+        "ARTP_CONTAM",
 #if TAG_MAJOR_VERSION == 34
         "ARTP_ACCURACY",
 #endif
         "ARTP_SLAYING",
-        "ARTP_CURSED",
+        "ARTP_CURSE",
         "ARTP_STEALTH",
         "ARTP_MAGICAL_POWER",
         "ARTP_BASE_DELAY",
@@ -1570,23 +1409,44 @@ static void _debug_rap_stats(FILE *ostat)
         "ARTP_FOG",
 #endif
         "ARTP_REGENERATION",
-        "ARTP_SUSTAB",
+        "ARTP_SUSTAT",
         "ARTP_NO_UPGRADE",
+        "ARTP_RCORR",
+        "ARTP_RMUT",
+#if TAG_MAJOR_VERSION == 34
+        "ARTP_TWISTER",
+#endif
+        "ARTP_CORRODE",
+        "ARTP_DRAIN",
+        "ARTP_CONFUSE",
     };
     COMPILE_CHECK(ARRAYSZ(rap_names) == ARTP_NUM_PROPERTIES);
 
-    fprintf(ostat, "                            All    Good   Bad\n");
-    fprintf(ostat, "                           --------------------\n");
+    fprintf(ostat, "                                 All    Good   Bad   Max AvgGood Min AvgBad\n");
+    fprintf(ostat, "                           ------------------------------------------------\n");
+    fprintf(ostat, "%-27s: %6.2f%% %6.2f%% %6.2f%%\n", "Overall", 100.0,
+            (float) total_good_props * 100.0 / (float) total_props,
+            (float) total_bad_props * 100.0 / (float) total_props);
 
-    for (i = 0; i < ARTP_NUM_PROPERTIES; ++i)
+    for (int i = 0; i < ARTP_NUM_PROPERTIES; ++i)
     {
-        if (all_props[i] == 0)
+        const int total_props_of_type = good_props[i] + bad_props[i];
+        if (!total_props_of_type)
             continue;
 
-        fprintf(ostat, "%-25s: %5.2f%% %5.2f%% %5.2f%%\n", rap_names[i],
-                (float) all_props[i] * 100.0 / (float) num_randarts,
+        const float avg_good = good_props[i] ?
+            (float) total_good_prop_vals[i] / (float) good_props[i] :
+            0.0;
+        const float avg_bad = bad_props[i] ?
+            (float) total_bad_prop_vals[i] / (float) bad_props[i] :
+            0.0;
+        fprintf(ostat, "%-27s: %6.2f%% %6.2f%% %6.2f%% %3d %5.2f %5d %5.2f\n",
+                rap_names[i],
+                (float) total_props_of_type * 100.0 / (float) num_randarts,
                 (float) good_props[i] * 100.0 / (float) num_randarts,
-                (float) bad_props[i] * 100.0 / (float) num_randarts);
+                (float) bad_props[i] * 100.0 / (float) num_randarts,
+                max_prop_vals[i], avg_good,
+                min_prop_vals[i], avg_bad);
     }
 
     fprintf(ostat, "\n-----------------------------------------\n\n");
@@ -1610,7 +1470,7 @@ void debug_item_statistics()
     switch (keyin)
     {
     case 'a': _debug_acquirement_stats(ostat); break;
-    case 'b': _debug_rap_stats(ostat);
+    case 'b': _debug_rap_stats(ostat); break;
     default:
         canned_msg(MSG_OK);
         break;
@@ -1652,38 +1512,32 @@ void wizard_draw_card()
 void wizard_identify_all_items()
 {
     wizard_identify_pack();
-    for (int i = 0; i < MAX_ITEMS; ++i)
-    {
-        item_def& item = mitm[i];
+    for (auto &item : mitm)
         if (item.defined())
             set_ident_flags(item, ISFLAG_IDENT_MASK);
-    }
     for (int ii = 0; ii < NUM_OBJECT_CLASSES; ii++)
     {
         object_class_type i = (object_class_type)ii;
         if (!item_type_has_ids(i))
             continue;
         for (int j = 0; j < get_max_subtype(i); j++)
-            set_ident_type(i, j, ID_KNOWN_TYPE);
+            set_ident_type(i, j, true);
     }
 }
 
 void wizard_unidentify_all_items()
 {
     wizard_unidentify_pack();
-    for (int i = 0; i < MAX_ITEMS; ++i)
-    {
-        item_def& item = mitm[i];
+    for (auto &item : mitm)
         if (item.defined())
-            unset_ident_flags(item, ISFLAG_IDENT_MASK);
-    }
+            _forget_item(item);
     for (int ii = 0; ii < NUM_OBJECT_CLASSES; ii++)
     {
         object_class_type i = (object_class_type)ii;
         if (!item_type_has_ids(i))
             continue;
         for (int j = 0; j < get_max_subtype(i); j++)
-            set_ident_type(i, j, ID_UNKNOWN_TYPE);
+            set_ident_type(i, j, false);
     }
 }
 
