@@ -6,10 +6,10 @@
 #ifndef WIZITEM_H
 #define WIZITEM_H
 
-void wizard_create_spec_object(void);
+void wizard_create_spec_object();
 void wizard_create_spec_object_by_name();
-void wizard_tweak_object(void);
-void wizard_make_object_randart(void);
+void wizard_tweak_object();
+void wizard_make_object_randart();
 void wizard_value_artefact();
 void wizard_uncurse_item();
 void wizard_create_all_artefacts();
@@ -19,6 +19,6 @@ void wizard_draw_card();
 void wizard_list_items();
 void wizard_identify_all_items();
 void wizard_unidentify_all_items();
-void debug_item_statistics(void);
+void debug_item_statistics();
 
 #endif
