@@ -4,21 +4,20 @@
 **/
 
 #include "AppHdr.h"
-#include <sstream>
 
 #include "wiz-mon.h"
 
+#include <sstream>
+
 #include "abyss.h"
 #include "act-iter.h"
 #include "areas.h"
-#include "artefact.h"
-#include "cio.h"
 #include "colour.h"
 #include "dbg-util.h"
 #include "delay.h"
 #include "directn.h"
 #include "dungeon.h"
-#include "env.h"
+#include "english.h"
 #include "files.h"
 #include "ghost.h"
 #include "godblessing.h"
@@ -28,28 +27,22 @@
 #include "jobs.h"
 #include "libutil.h"
 #include "macro.h"
-#include "mapdef.h"
 #include "message.h"
-#include "mgen_data.h"
 #include "mon-death.h"
 #include "mon-pathfind.h"
-#include "mon-poly.h"
 #include "mon-place.h"
+#include "mon-poly.h"
 #include "mon-speak.h"
-#include "mon-util.h"
 #include "output.h"
 #include "prompt.h"
 #include "religion.h"
 #include "shout.h"
 #include "spl-miscast.h"
-#include "spl-util.h"
 #include "state.h"
 #include "stringutil.h"
-#include "terrain.h"
 #include "unwind.h"
 #include "view.h"
 #include "viewmap.h"
-#include "wiz-dgn.h"
 
 #ifdef WIZARD
 // Creates a specific monster by mon type number.
@@ -74,93 +67,6 @@ void wizard_create_spec_monster()
     }
 }
 
-static int _make_mimic_item(object_class_type type)
-{
-    int it = items(0, OBJ_RANDOM, OBJ_RANDOM, true, 0, 0);
-
-    if (it == NON_ITEM)
-        return NON_ITEM;
-
-    item_def &item = mitm[it];
-
-    item.base_type = type;
-    item.sub_type  = 0;
-    item.special   = 0;
-    item.colour    = 0;
-    item.flags     = 0;
-    item.quantity  = 1;
-    item.plus      = 0;
-    item.plus2     = 0;
-    item.link      = NON_ITEM;
-
-    int prop;
-    switch (type)
-    {
-    case OBJ_WEAPONS:
-        do
-            item.sub_type = random2(NUM_WEAPONS);
-        while (is_blessed(item));
-
-        prop = random2(100);
-
-        if (prop < 20)
-            make_item_randart(item);
-        else if (prop < 50)
-            set_equip_desc(item, ISFLAG_GLOWING);
-        else if (prop < 80)
-            set_equip_desc(item, ISFLAG_RUNED);
-        break;
-
-    case OBJ_ARMOUR:
-        do
-            item.sub_type = random2(NUM_ARMOURS);
-        while (armour_is_hide(item));
-
-        prop = random2(100);
-
-        if (prop < 20)
-            make_item_randart(item);
-        else if (prop < 40)
-            set_equip_desc(item, ISFLAG_GLOWING);
-        else if (prop < 65)
-            set_equip_desc(item, ISFLAG_RUNED);
-        else if (prop < 90)
-            set_equip_desc(item, ISFLAG_EMBROIDERED_SHINY);
-        break;
-
-    case OBJ_SCROLLS:
-        item.sub_type = random2(NUM_SCROLLS);
-        break;
-
-    case OBJ_POTIONS:
-        do
-            item.sub_type = random2(NUM_POTIONS);
-        while (is_blood_potion(item));
-        break;
-
-    case OBJ_BOOKS:
-        item.sub_type = random2(MAX_NORMAL_BOOK);
-        break;
-
-    case OBJ_STAVES:
-        item.sub_type = random2(NUM_STAVES);
-        break;
-
-    case OBJ_RODS:
-        item.sub_type = random2(NUM_RODS);
-        break;
-
-    case OBJ_GOLD:
-    default:
-        item.quantity = 5 + random2(1000);
-        break;
-    }
-
-    item_colour(item); // also sets special vals for scrolls/potions
-
-    return it;
-}
-
 // Creates a specific monster by name. Uses the same patterns as
 // map definitions.
 void wizard_create_spec_monster_name()
@@ -190,7 +96,7 @@ void wizard_create_spec_monster_name()
 
         if (!newerr.empty())
         {
-            mpr(err.c_str());
+            mpr(err);
             return;
         }
     }
@@ -220,32 +126,6 @@ void wizard_create_spec_monster_name()
         return;
     }
 
-    if (mons_is_feat_mimic(type))
-    {
-        if (wizard_create_feature(place))
-            env.level_map_mask(place) |= MMT_MIMIC;
-        return;
-    }
-
-    if (mons_is_item_mimic(type))
-    {
-        object_class_type item_type = get_item_mimic_type();
-        if (item_type == OBJ_UNASSIGNED)
-        {
-            canned_msg(MSG_OK);
-            return;
-        }
-        int it = _make_mimic_item(item_type);
-        if (it == NON_ITEM)
-        {
-            mprf(MSGCH_DIAGNOSTICS, "Cannot create item.");
-            return;
-        }
-        move_item_to_grid(&it, place);
-        mitm[it].flags |= ISFLAG_MIMIC;
-        return;
-    }
-
     // Wizmode users should be able to conjure up uniques even if they
     // were already created. Yay, you can meet 3 Sigmunds at once! :p
     if (mons_is_unique(type) && you.unique_creatures[type])
@@ -263,11 +143,11 @@ void wizard_create_spec_monster_name()
 
         if (idx >= MAX_MONSTERS || menv[idx].type != MONS_KRAKEN)
         {
-            for (idx = 0; idx < MAX_MONSTERS; idx++)
+            for (auto &mons : menv)
             {
-                if (menv[idx].type == MONS_KRAKEN && menv[idx].alive())
+                if (mons.type == MONS_KRAKEN && mons.alive())
                 {
-                    menv[idx].colour = element_colour(ETC_KRAKEN);
+                    mons.colour = element_colour(ETC_KRAKEN);
                     return;
                 }
             }
@@ -406,7 +286,7 @@ void debug_list_monsters()
             if (count > 1)
             {
                 snprintf(buf, sizeof(buf), "%d %s", count,
-                         pluralise(prev_name).c_str());
+                         pluralise_monster(prev_name).c_str());
             }
             else
                 snprintf(buf, sizeof(buf), "%s", prev_name.c_str());
@@ -436,10 +316,13 @@ void debug_list_monsters()
 
     char buf[80];
     if (count > 1)
-        snprintf(buf, sizeof(buf), "%d %s", count, pluralise(prev_name).c_str());
+    {
+        snprintf(buf, sizeof(buf), "%d %s", count,
+                 pluralise_monster(prev_name).c_str());
+    }
     else
         snprintf(buf, sizeof(buf), "%s", prev_name.c_str());
-    mons.push_back(buf);
+    mons.emplace_back(buf);
 
     mpr_comma_separated_list("Monsters: ", mons);
 
@@ -473,6 +356,7 @@ void wizard_spawn_control()
             const int rate = atoi(specs);
             if (rate || specs[0] == '0')
             {
+                mprf("Setting monster spawn rate to %i.", rate);
                 env.spawn_random_rate = rate;
                 done = true;
             }
@@ -501,6 +385,7 @@ void wizard_spawn_control()
             const int num = min(atoi(specs), max_spawn);
             if (num > 0)
             {
+                mprf("Spawning %i monster%s.", num, num == 1 ? "" : "s");
                 int curr_rate = env.spawn_random_rate;
                 // Each call to spawn_random_monsters() will spawn one with
                 // the rate at 5 or less.
@@ -583,21 +468,21 @@ void debug_stethoscope(int mon)
 
     // Print stats and other info.
     mprf(MSGCH_DIAGNOSTICS,
-         "HD=%d/%d (%u) HP=%d/%d AC=%d(%d) EV=%d MR=%d XP=%d SP=%d "
+         "HD=%d/%d (%u) HP=%d/%d AC=%d(%d) EV=%d(%d) MR=%d XP=%d SP=%d "
          "energy=%d%s%s mid=%u num=%d stealth=%d flags=%04" PRIx64,
          mons.get_hit_dice(),
          mons.get_experience_level(),
          mons.experience,
          mons.hit_points, mons.max_hit_points,
-         mons.ac, mons.armour_class(),
-         mons.ev,
+         mons.base_armour_class(), mons.armour_class(),
+         mons.base_evasion(), mons.evasion(),
          mons.res_magic(),
          exper_value(&mons),
          mons.speed, mons.speed_increment,
          mons.base_monster != MONS_NO_MONSTER ? " base=" : "",
          mons.base_monster != MONS_NO_MONSTER ?
          get_monster_data(mons.base_monster)->name : "",
-         mons.mid, mons.number, mons.stealth(), mons.flags);
+         mons.mid, mons.number, mons.stealth(), mons.flags.flags);
 
     if (mons.damage_total)
     {
@@ -662,24 +547,44 @@ void debug_stethoscope(int mon)
     ostringstream spl;
     const monster_spells &hspell_pass = mons.spells;
     bool found_spell = false;
-    for (int k = 0; k < NUM_MONSTER_SPELL_SLOTS; ++k)
+    for (unsigned int k = 0; k < hspell_pass.size(); ++k)
     {
-        if (hspell_pass[k] != SPELL_NO_SPELL)
+        if (found_spell)
+            spl << ", ";
+
+        found_spell = true;
+
+        spl << k << ": ";
+
+        if (hspell_pass[k].spell >= NUM_SPELLS)
+            spl << "buggy spell";
+        else
+            spl << spell_title(hspell_pass[k].spell);
+
+        spl << "." << (int)hspell_pass[k].freq;
+        for (const auto flag : mon_spell_slot_flags::range())
         {
-            if (found_spell)
-                spl << ", ";
+            if (!(hspell_pass[k].flags & flag))
+                continue;
 
-            found_spell = true;
-
-            spl << k << ": ";
-
-            if (hspell_pass[k] >= NUM_SPELLS)
-                spl << "buggy spell";
-            else
-                spl << spell_title(hspell_pass[k]);
-
-            spl << " (" << static_cast<int>(hspell_pass[k]) << ")";
+            // this is arguably redundant with mons_list::parse_mons_spells
+            // specificially the bit that turns names into flags
+            static const map<mon_spell_slot_flag, string> flagnames = {
+                { MON_SPELL_EMERGENCY,  "E" },
+                { MON_SPELL_NATURAL,    "N" },
+                { MON_SPELL_MAGICAL,    "M" },
+                { MON_SPELL_DEMONIC,    "D" },
+                { MON_SPELL_WIZARD,     "W" },
+                { MON_SPELL_PRIEST,     "P" },
+                { MON_SPELL_BREATH,     "br" },
+                { MON_SPELL_NO_SILENT,  "ns" },
+                { MON_SPELL_INSTANT,    "in" },
+                { MON_SPELL_NOISY,      "noi" },
+            };
+            spl << "." << lookup(flagnames, flag, "bug");
         }
+
+        spl << " (#" << static_cast<int>(hspell_pass[k].spell) << ")";
     }
     if (found_spell)
         mprf(MSGCH_DIAGNOSTICS, "spells: %s", spl.str().c_str());
@@ -721,11 +626,17 @@ void debug_stethoscope(int mon)
 // Detects all monsters on the level, using their exact positions.
 void wizard_detect_creatures()
 {
+    int count = 0;
     for (monster_iterator mi; mi; ++mi)
     {
         env.map_knowledge(mi->pos()).set_monster(monster_info(*mi));
         env.map_knowledge(mi->pos()).set_detected_monster(mi->type);
+#ifdef USE_TILE
+        tiles.update_minimap(mi->pos());
+#endif
+        count++;
     }
+    mprf("Detected %i monster%s.", count, count == 1 ? "" : "s");
 }
 
 // Dismisses all monsters on the level or all monsters that match a user
@@ -736,7 +647,8 @@ void wizard_dismiss_all_monsters(bool force_all)
     if (!force_all)
     {
         mprf(MSGCH_PROMPT, "What monsters to dismiss (ENTER for all, "
-                           "\"harmful\", \"mobile\" or a regex)? ");
+                           "\"harmful\", \"mobile\", \"los\" or a regex, "
+                           "\"keepitem\" to leave items)? ");
         bool validline = !cancellable_get_line_autohist(buf, sizeof buf);
 
         if (!validline)
@@ -746,7 +658,8 @@ void wizard_dismiss_all_monsters(bool force_all)
         }
     }
 
-    dismiss_monsters(buf);
+    int count = dismiss_monsters(buf);
+    mprf("Dismissed %i monster%s.", count, count == 1 ? "" : "s");
     // If it was turned off turn autopickup back on if all monsters went away.
     if (!*buf)
         autotoggle_autopickup(false);
@@ -858,215 +771,30 @@ void wizard_give_monster_item(monster* mon)
     if (prompt_failed(player_slot))
         return;
 
-    for (int i = 0; i < NUM_EQUIP; ++i)
-        if (you.equip[i] == player_slot)
-        {
-            mpr("Can't give equipped items to a monster.");
-            return;
-        }
+    item_def &item = you.inv[player_slot];
 
-    item_def     &item = you.inv[player_slot];
-    mon_inv_type mon_slot = NUM_MONSTER_SLOTS;
-
-    switch (item.base_type)
+    if (item_is_equipped(item))
     {
-    case OBJ_WEAPONS:
-    case OBJ_STAVES:
-    case OBJ_RODS:
-        // Let wizard specify which slot to put weapon into via
-        // inscriptions.
-        if (item.inscription.find("first") != string::npos
-            || item.inscription.find("primary") != string::npos)
-        {
-            mpr("Putting weapon into primary slot by inscription");
-            mon_slot = MSLOT_WEAPON;
-            break;
-        }
-        else if (item.inscription.find("second") != string::npos
-                 || item.inscription.find("alt") != string::npos)
-        {
-            mpr("Putting weapon into alt slot by inscription");
-            mon_slot = MSLOT_ALT_WEAPON;
-            break;
-        }
-
-        // For monsters which can wield two weapons, prefer whichever
-        // slot is empty (if there is an empty slot).
-        if (mons_wields_two_weapons(mon))
-        {
-            if (mon->inv[MSLOT_WEAPON] == NON_ITEM)
-            {
-                mpr("Dual wielding monster, putting into empty primary slot");
-                mon_slot = MSLOT_WEAPON;
-                break;
-            }
-            else if (mon->inv[MSLOT_ALT_WEAPON] == NON_ITEM)
-            {
-                mpr("Dual wielding monster, putting into empty alt slot");
-                mon_slot = MSLOT_ALT_WEAPON;
-                break;
-            }
-        }
-
-        // Try to replace a ranged weapon with a ranged weapon and
-        // a non-ranged weapon with a non-ranged weapon
-        if (mon->inv[MSLOT_WEAPON] != NON_ITEM
-            && (is_range_weapon(mitm[mon->inv[MSLOT_WEAPON]])
-                == is_range_weapon(item)))
-        {
-            mpr("Replacing primary slot with similar weapon");
-            mon_slot = MSLOT_WEAPON;
-            break;
-        }
-        if (mon->inv[MSLOT_ALT_WEAPON] != NON_ITEM
-            && (is_range_weapon(mitm[mon->inv[MSLOT_ALT_WEAPON]])
-                == is_range_weapon(item)))
-        {
-            mpr("Replacing alt slot with similar weapon");
-            mon_slot = MSLOT_ALT_WEAPON;
-            break;
-        }
-
-        // Prefer the empty slot (if any)
-        if (mon->inv[MSLOT_WEAPON] == NON_ITEM)
-        {
-            mpr("Putting weapon into empty primary slot");
-            mon_slot = MSLOT_WEAPON;
-            break;
-        }
-        else if (mon->inv[MSLOT_ALT_WEAPON] == NON_ITEM)
-        {
-            mpr("Putting weapon into empty alt slot");
-            mon_slot = MSLOT_ALT_WEAPON;
-            break;
-        }
-
-        // Default to primary weapon slot
-        mpr("Defaulting to primary slot");
-        mon_slot = MSLOT_WEAPON;
-        break;
-
-    case OBJ_ARMOUR:
-    {
-        // May only return shield or armour slot.
-        equipment_type eq = get_armour_slot(item);
-
-        // Force non-shield, non-body armour to be worn anyway.
-        if (eq == EQ_NONE)
-            eq = EQ_BODY_ARMOUR;
-
-        mon_slot = equip_slot_to_mslot(eq);
-        break;
+        mpr("Can't give equipped items to a monster.");
+        return;
     }
-    case OBJ_MISSILES:
-        mon_slot = MSLOT_MISSILE;
-        break;
-    case OBJ_WANDS:
-        mon_slot = MSLOT_WAND;
-        break;
-    case OBJ_SCROLLS:
-        mon_slot = MSLOT_SCROLL;
-        break;
-    case OBJ_POTIONS:
-        mon_slot = MSLOT_POTION;
-        break;
-    case OBJ_MISCELLANY:
-        mon_slot = MSLOT_MISCELLANY;
-        break;
-    case OBJ_JEWELLERY:
-        mon_slot = MSLOT_JEWELLERY;
-        break;
-    default:
+
+    mon_inv_type mon_slot = item_to_mslot(item);
+
+    if (mon_slot == NUM_MONSTER_SLOTS)
+    {
         mpr("You can't give that type of item to a monster.");
         return;
     }
 
-    if (item_use == MONUSE_STARTING_EQUIPMENT
-        && !mons_is_unique(mon->type))
+    if (mon_slot == MSLOT_WEAPON
+        && item.inscription.find("alt") != string::npos)
     {
-        switch (mon_slot)
-        {
-        case MSLOT_WEAPON:
-        case MSLOT_ALT_WEAPON:
-        case MSLOT_ARMOUR:
-        case MSLOT_JEWELLERY:
-        case MSLOT_MISSILE:
-            break;
-
-        default:
-            mpr("That type of monster can only use weapons and armour.");
-            return;
-        }
+        mon_slot = MSLOT_ALT_WEAPON;
     }
 
-    int index = get_mitm_slot(10);
-    if (index == NON_ITEM)
-    {
-        mpr("Too many items on level, bailing.");
-        return;
-    }
-
-    // Move monster's old item to player's inventory as last step.
-    int  old_eq     = NON_ITEM;
-    bool unequipped = false;
-    if (mon_slot != NUM_MONSTER_SLOTS
-        && mon->inv[mon_slot] != NON_ITEM
-        && !items_stack(item, mitm[mon->inv[mon_slot]]))
-    {
-        old_eq = mon->inv[mon_slot];
-        // Alternative weapons don't get (un)wielded unless the monster
-        // can wield two weapons.
-        if (mon_slot != MSLOT_ALT_WEAPON || mons_wields_two_weapons(mon))
-        {
-            mon->unequip(*(mon->mslot_item(mon_slot)), mon_slot, 1, true);
-            unequipped = true;
-        }
-        mon->inv[mon_slot] = NON_ITEM;
-    }
-
-    mitm[index] = item;
-
-    unwind_var<int> save_speedinc(mon->speed_increment);
-    if (!mon->pickup_item(mitm[index], false, true))
-    {
-        mpr("Monster wouldn't take item.");
-        if (old_eq != NON_ITEM && mon_slot != NUM_MONSTER_SLOTS)
-        {
-            mon->inv[mon_slot] = old_eq;
-            if (unequipped)
-                mon->equip(mitm[old_eq], mon_slot, 1);
-        }
-        unlink_item(index);
-        destroy_item(mitm[index]);
-        return;
-    }
-
-    // Item is gone from player's inventory.
-    dec_inv_item_quantity(player_slot, item.quantity);
-
-    if ((mon->flags & MF_HARD_RESET) && !(item.flags & ISFLAG_SUMMONED))
-    {
-        mprf(MSGCH_WARN, "WARNING: Monster has MF_HARD_RESET and all its "
-             "items will disappear when it does.");
-    }
-    else if ((item.flags & ISFLAG_SUMMONED) && !mon->is_summoned())
-    {
-        mprf(MSGCH_WARN, "WARNING: Item is summoned and will disappear when "
-             "the monster does.");
-    }
-    // Monster's old item moves to player's inventory.
-    if (old_eq != NON_ITEM)
-    {
-        mpr("Fetching monster's old item.");
-        if (mitm[old_eq].flags & ISFLAG_SUMMONED)
-        {
-            mprf(MSGCH_WARN, "WARNING: Item is summoned and shouldn't really "
-                 "be anywhere but in the inventory of a summoned monster.");
-        }
-        mitm[old_eq].pos.reset();
-        mitm[old_eq].link = NON_ITEM;
-        move_item_to_inv(old_eq, mitm[old_eq].quantity);
-    }
+    if (!mon->take_item(player_slot, mon_slot))
+        mpr("Error: monster failed to take item.");
 }
 
 static void _move_player(const coord_def& where)
@@ -1101,7 +829,7 @@ static void _move_monster(const coord_def& where, int idx1)
 
     mgrd(where) = idx2;
 
-    if (mon2 != NULL)
+    if (mon2 != nullptr)
     {
         mon2->moveto(where);
         mon1->check_redraw(where);
@@ -1144,7 +872,7 @@ void wizard_move_player_or_monster(const coord_def& where)
 void wizard_make_monster_summoned(monster* mon)
 {
     int summon_type = 0;
-    if (mon->is_summoned(NULL, &summon_type) || summon_type != 0)
+    if (mon->is_summoned(nullptr, &summon_type) || summon_type != 0)
     {
         mprf(MSGCH_PROMPT, "Monster is already summoned.");
         return;
@@ -1159,13 +887,13 @@ void wizard_make_monster_summoned(monster* mon)
     }
 
     mprf(MSGCH_PROMPT, "[a] clone [b] animated [c] chaos [d] miscast [e] zot");
-    mprf(MSGCH_PROMPT, "[f] wrath [g] aid                [m] misc    [s] spell");
+    mprf(MSGCH_PROMPT, "[f] wrath [g] lantern  [h] aid   [m] misc    [s] spell");
 
     mprf(MSGCH_PROMPT, "Which summon type? ");
 
     char choice = toalower(getchm());
 
-    if (!(choice >= 'a' && choice <= 'g') && choice != 'm' && choice != 's')
+    if (!(choice >= 'a' && choice <= 'h') && choice != 'm' && choice != 's')
     {
         canned_msg(MSG_OK);
         return;
@@ -1181,7 +909,8 @@ void wizard_make_monster_summoned(monster* mon)
         case 'd': type = MON_SUMM_MISCAST; break;
         case 'e': type = MON_SUMM_ZOT; break;
         case 'f': type = MON_SUMM_WRATH; break;
-        case 'g': type = MON_SUMM_AID; break;
+        case 'g': type = MON_SUMM_LANTERN; break;
+        case 'h': type = MON_SUMM_AID; break;
         case 'm': type = 0; break;
 
         case 's':
@@ -1304,17 +1033,14 @@ void debug_pathfind(int idx)
         vector<coord_def> path = mp.backtrack();
         env.travel_trail = path;
 #ifdef USE_TILE_WEB
-        for (unsigned int i = 0; i < env.travel_trail.size(); ++i)
-            tiles.update_minimap(env.travel_trail[i]);
+        for (coord_def pos : env.travel_trail)
+            tiles.update_minimap(pos);
 #endif
         string path_str;
         mpr("Here's the shortest path: ");
-        for (unsigned int i = 0; i < path.size(); ++i)
-        {
-            snprintf(info, INFO_SIZE, "(%d, %d)  ", path[i].x, path[i].y);
-            path_str += info;
-        }
-        mpr(path_str.c_str());
+        for (coord_def pos : path)
+            path_str += make_stringf("(%d, %d)  ", pos.x, pos.y);
+        mpr(path_str);
         mprf("-> path length: %u", (unsigned int)path.size());
 
         mpr("");
@@ -1322,12 +1048,9 @@ void debug_pathfind(int idx)
         path_str = "";
         mpr("");
         mpr("And here are the needed waypoints: ");
-        for (unsigned int i = 0; i < path.size(); ++i)
-        {
-            snprintf(info, INFO_SIZE, "(%d, %d)  ", path[i].x, path[i].y);
-            path_str += info;
-        }
-        mpr(path_str.c_str());
+        for (coord_def pos : path)
+            path_str += make_stringf("(%d, %d)  ", pos.x, pos.y);
+        mpr(path_str);
         mprf("-> #waypoints: %u", (unsigned int)path.size());
     }
 }
@@ -1393,12 +1116,12 @@ void debug_miscast(int target_index)
         return;
     }
 
-    int disciplines = 0;
+    spschools_type disciplines = SPTYP_NONE;
     if (spell != SPELL_NO_SPELL)
     {
         disciplines = get_spell_disciplines(spell);
 
-        if (disciplines == 0)
+        if (!disciplines)
         {
             mprf("Spell '%s' has no disciplines.", spell_title(spell));
             return;
@@ -1411,9 +1134,9 @@ void debug_miscast(int target_index)
         mprf("Miscasting school %s.", spelltype_long_name(school));
 
     if (spell != SPELL_NO_SPELL)
-        mprf(MSGCH_PROMPT, "Enter spell_power,spell_failure: ");
+        mprf(MSGCH_PROMPT, "Enter spell_power,raw_spell_failure: ");
     else
-        mprf(MSGCH_PROMPT, "Enter miscast_level or spell_power,spell_failure: ");
+        mprf(MSGCH_PROMPT, "Enter miscast_level or spell_power,raw_spell_failure: ");
 
     if (cancellable_get_line_autohist(specs, sizeof specs) || !*specs)
     {
@@ -1476,20 +1199,20 @@ void debug_miscast(int target_index)
 
     if (spell != SPELL_NO_SPELL)
     {
-        miscast = new MiscastEffect(target, target_index, spell, pow, fail,
-                                    "", nothing);
+        miscast = new MiscastEffect(target, target, WIZARD_MISCAST, spell, pow,
+                                    fail, "", nothing);
     }
     else
     {
         if (level != -1)
         {
-            miscast = new MiscastEffect(target, target_index, school,
+            miscast = new MiscastEffect(target, target, WIZARD_MISCAST, school,
                                         level, "wizard testing miscast",
                                         nothing);
         }
         else
         {
-            miscast = new MiscastEffect(target, target_index, school,
+            miscast = new MiscastEffect(target, target, WIZARD_MISCAST, school,
                                         pow, fail, "wizard testing miscast",
                                         nothing);
         }
