@@ -7,11 +7,11 @@
 struct zap_info
 {
     zap_type ztype;
-    const char* name;           // NULL means handled specially
+    const char* name;
     int power_cap;
     dam_deducer* damage;
     tohit_deducer* tohit;       // Enchantments have power modifier here
-    int colour;
+    colour_t colour;
     bool is_enchantment;
     beam_type flavour;
     dungeon_char_type glyph;
@@ -61,7 +61,7 @@ static const zap_info zap_data[] =
     ZAP_SLOW,
     "",
     100,
-    NULL,
+    nullptr,
     new tohit_calculator<0, 3, 2>,
     BLACK,
     true,
@@ -73,12 +73,28 @@ static const zap_info zap_data[] =
     0
 },
 
+{
+    ZAP_MEPHITIC,
+    "stinking cloud",
+    100,
+    nullptr,
+    new tohit_calculator<AUTOMATIC_HIT>,
+    GREEN,
+    false,
+    BEAM_MEPHITIC,
+    DCHAR_FIRED_ZAP,
+    true,
+    false,
+    true,
+    0 // Noise comes from explosion
+},
+
 {
     ZAP_HASTE,
     "",
     100,
-    NULL,
-    NULL,
+    nullptr,
+    nullptr,
     BLACK,
     true,
     BEAM_HASTE,
@@ -106,11 +122,11 @@ static const zap_info zap_data[] =
 },
 
 {
-    ZAP_MAJOR_HEALING,
+    ZAP_HEAL_WOUNDS,
     "",
     100,
     new dicedef_calculator<1, 7, 1, 3>,
-    NULL,
+    nullptr,
     BLACK,
     true,
     BEAM_HEALING,
@@ -125,8 +141,8 @@ static const zap_info zap_data[] =
     ZAP_PARALYSE,
     "",
     100,
-    NULL,
-    NULL,
+    nullptr,
+    nullptr,
     BLACK,
     true,
     BEAM_PARALYSIS,
@@ -189,8 +205,8 @@ static const zap_info zap_data[] =
     ZAP_CONFUSE,
     "",
     100,
-    NULL,
-    NULL,
+    nullptr,
+    nullptr,
     BLACK,
     true,
     BEAM_CONFUSION,
@@ -205,8 +221,8 @@ static const zap_info zap_data[] =
     ZAP_TUKIMAS_DANCE,
     "",
     100,
-    NULL,
-    NULL,
+    nullptr,
+    new tohit_calculator<0, 3, 1>,
     BLACK,
     true,
     BEAM_TUKIMAS_DANCE,
@@ -221,8 +237,8 @@ static const zap_info zap_data[] =
     ZAP_INVISIBILITY,
     "",
     100,
-    NULL,
-    NULL,
+    nullptr,
+    nullptr,
     BLACK,
     true,
     BEAM_INVISIBILITY,
@@ -237,8 +253,8 @@ static const zap_info zap_data[] =
     ZAP_DIG,
     "",
     100,
-    NULL,
-    NULL,
+    nullptr,
+    nullptr,
     BLACK,
     true,
     BEAM_DIGGING,
@@ -269,7 +285,7 @@ static const zap_info zap_data[] =
     ZAP_TELEPORT_OTHER,
     "",
     100,
-    NULL,
+    nullptr,
     new tohit_calculator<0, 3, 2>,
     BLACK,
     true,
@@ -301,8 +317,8 @@ static const zap_info zap_data[] =
     ZAP_POLYMORPH,
     "",
     100,
-    NULL,
-    NULL,
+    nullptr,
+    nullptr,
     BLACK,
     true,
     BEAM_POLYMORPH,
@@ -397,8 +413,8 @@ static const zap_info zap_data[] =
     ZAP_ENSLAVEMENT,
     "",
     100,
-    NULL,
-    NULL,
+    nullptr,
+    nullptr,
     BLACK,
     true,
     BEAM_ENSLAVE,
@@ -477,8 +493,8 @@ static const zap_info zap_data[] =
     ZAP_BANISHMENT,
     "",
     100,
-    NULL,
-    NULL,
+    nullptr,
+    nullptr,
     BLACK,
     true,
     BEAM_BANISH,
@@ -661,7 +677,7 @@ static const zap_info zap_data[] =
     BEAM_ACID,
     DCHAR_FIRED_ZAP,
     true,
-    true,
+    false,
     false,
     6
 },
@@ -702,8 +718,8 @@ static const zap_info zap_data[] =
     ZAP_ENSLAVE_SOUL,
     "",
     100,
-    NULL,
-    NULL,
+    nullptr,
+    nullptr,
     BLACK,
     true,
     BEAM_ENSLAVE_SOUL,
@@ -718,7 +734,7 @@ static const zap_info zap_data[] =
     ZAP_AGONY,
     "agony",
     100,
-    NULL,
+    nullptr,
     new tohit_calculator<0, 5, 1>,
     BLACK,
     true,
@@ -771,7 +787,7 @@ static const zap_info zap_data[] =
     WHITE,
     false,
     BEAM_ICE,
-    DCHAR_FIRED_ZAP,
+    DCHAR_FIRED_MISSILE,
     false,
     false,
     false,
@@ -782,7 +798,7 @@ static const zap_info zap_data[] =
     ZAP_CORONA,
     "",
     100,
-    NULL,
+    nullptr,
     new tohit_calculator<0, 3, 2>,
     BLUE,
     true,
@@ -798,8 +814,8 @@ static const zap_info zap_data[] =
     ZAP_HIBERNATION,
     "",
     100, // power is capped to 50 in spl-zap.cc:spell_zap_power.
-    NULL,
-    NULL,
+    nullptr,
+    nullptr,
     BLACK,
     true,
     BEAM_HIBERNATION,
@@ -910,8 +926,8 @@ static const zap_info zap_data[] =
     ZAP_PETRIFY,
     "",
     100,
-    NULL,
-    NULL,
+    nullptr,
+    nullptr,
     BLACK,
     true,
     BEAM_PETRIFY,
@@ -926,8 +942,8 @@ static const zap_info zap_data[] =
     ZAP_PORKALATOR,
     "porkalator",
     100,
-    NULL,
-    NULL,
+    nullptr,
+    nullptr,
     RED,
     true,
     BEAM_PORKALATOR,
@@ -942,8 +958,8 @@ static const zap_info zap_data[] =
     ZAP_SLEEP,
     "",
     100,
-    NULL,
-    NULL,
+    nullptr,
+    nullptr,
     BLACK,
     true,
     BEAM_SLEEP,
@@ -958,7 +974,7 @@ static const zap_info zap_data[] =
     ZAP_IOOD,
     "",
     200,
-    NULL,
+    nullptr,
     new tohit_calculator<AUTOMATIC_HIT>,
     WHITE,
     false,
@@ -971,24 +987,6 @@ static const zap_info zap_data[] =
     0
 },
 
-#if TAG_MAJOR_VERSION == 34
-{
-    ZAP_HOLY_LIGHT,
-    "beam of light",
-    200,
-    new calcdice_calculator<3, 11, 1, 3>,
-    new tohit_calculator<10, 1, 25>,
-    ETC_HOLY,
-    false,
-    BEAM_HOLY_LIGHT,
-    DCHAR_FIRED_ZAP,
-    true,
-    true,
-    false,
-    6
-},
-#endif
-
 {
     ZAP_BREATHE_STICKY_FLAME,
     "splash of liquid fire",
@@ -1025,7 +1023,7 @@ static const zap_info zap_data[] =
     ZAP_INNER_FLAME,
     "",
     100,
-    NULL,
+    nullptr,
     new tohit_calculator<0, 3, 1>,
     BLACK,
     true,
@@ -1058,7 +1056,7 @@ static const zap_info zap_data[] =
     "lance of force",
     100,
     new dicedef_calculator<2, 6, 1, 6>,
-    new tohit_calculator<10, 1, 9>,
+    new tohit_calculator<10, 1, 7>,
     CYAN,
     false,
     BEAM_MMISSILE,
@@ -1121,7 +1119,7 @@ static const zap_info zap_data[] =
     ZAP_EXPLOSIVE_BOLT,
     "explosive bolt",
     200,
-    NULL,
+    nullptr,
     new tohit_calculator<17, 1, 25>,
     RED,
     false,
@@ -1197,4 +1195,20 @@ static const zap_info zap_data[] =
     0
 },
 
+{
+    ZAP_SCATTERSHOT,
+    "burst of metal fragments",
+    200,
+    new calcdice_calculator<9, 8, 3, 8>,
+    new tohit_calculator<7, 1, 15>,
+    CYAN,
+    false,
+    BEAM_FRAG,
+    DCHAR_FIRED_BOLT,
+    true,
+    false,
+    false,
+    6
+},
+
 };
